      program isnasexe
c ======================================================================
c
c     programmer      Kees Kassels
c
c     version  1.0    date    25-03-1994
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNaS Host Program
c
c **********************************************************************
c
c                       KEYWORDS
c
c     host_program
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
c **********************************************************************
c
c                       COMMON BLOCKS
c
      integer nbuffr
      parameter( nbuffr = 8000000 )
      integer ibuffr
      common ibuffr(nbuffr)
c

c     IBUFFR    Integer buffer array in which all array information
c               with respect to the ISNaS software is stored.
c     NBUFFR    Declared length of IBUFFR
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISHMAIN    ISNaS host main subroutine
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c          initialize local buffer
c          call ISNaS host main subroutine
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call ismain( nbuffr )

      end
      function usfuni ( ichois, x, y, z )
c     --- dummy version
c         programmer   Kees Kassels
c         version      1.0  01-03-1995
c         A user supplied function "usfuni" is needed instead of this dummy !!!
c         The function value
c          usfuni
c         must be filled here by the user, corresponding to his/her option
c          ...  func = ichois
c         which has occured in the INITIAL_CONDITIONS part of problem
c         description
      implicit none
      integer ichois
      double precision usfuni, x, y, z
      if ( ichois.eq.1 ) then
         usfuni = x
      else if ( ichois.eq.2 ) then
         usfuni = -2d0*y
      else if ( ichois.eq.3 ) then
         usfuni = x + 3d0 * y
      else if ( ichois.eq.4 ) then
         usfuni = 4d0 * (x+.5d0) * ( 1d0-(x+.5d0))
      else if ( ichois.eq.5 ) then
         usfuni = 8d0 -4d0 * y
      end if
      end
      function usfunc1 ( ichoice, x, y, z, t, soluts, ndegfd )
c     --- User written function subroutine. It gives
c         the user the opportunity to define a
c         coefficient as a function of space, time and previously computed
c         solutions.
      implicit none
      integer ichoice, ndegfd
      double precision usfunc1, x, y, z, t, soluts(ndegfd)
c     --- x        i    x-coordinate
c         y        i    y-coordinate
c         t        i    actual time
c         ichoice  i    choice parameter given by the user input
c         soluts   i    array containing the solution at the previous level
c                       in the node
c         ndegfd   i    Length of array soluts
c         usfunc1  o    computed coefficient
c         temp          temperature
         if ( ichoice.eq.1 ) then

c        --- 1/rho

            usfunc1 = 1d0/5d0
         else if ( ichoice.eq.2 ) then

c        --- 3/rho

            usfunc1 = 3d0/5d0
         else if ( ichoice.eq.3 ) then

c        --- 1/rho + r

            usfunc1 = 1d0/5d0 +  x
         else if ( ichoice.eq.4 ) then

c        --- 3/rho + 4 * z

            usfunc1 =  3d0/5d0 + 4d0*y
         else if ( ichoice.eq.5 ) then

c        --- -3/rho

            usfunc1 =  -3d0/5d0
         else if ( ichoice.eq.6 ) then

c        --- 3/rho

            usfunc1 =  3d0/5d0
         end if
      end
      function usfunb ( ichoice, x, y, z, t )
c     --- User written function subroutine. It gives
c         the user the opportunity to define a
c         boundary condition as a function of space
c         and time.
      implicit none
      double precision usfunb, x, y, z, t
      integer ichoice
c     --- x        i    x-coordinate
c         y        i    y-coordinate
c         t        i    actual time
c         ichoice  i    choice parameter given by the user input
c         usfunb   o    computed boundary condition
         if ( ichoice.eq.1 ) then
             usfunb = x
         else if ( ichoice.eq.2 ) then
             usfunb =  -2d0*y
         else if ( ichoice.eq.3 ) then
            usfunb = 4d0 * (x+.5d0) * ( 1d0-(x+.5d0))
         end if
      end
