      subroutine isalloc( nblocks , neq,
     +                    isiinput, iseddy, isinfbnd,
     +                    isbndcon, ismatrix, isgmat, isrhside,
     +                    isgeom  , iadmin )
c=======================================================================
c
c     programmer   Kees Kassels
c
c     version      l.5   date    21-04-95 RvN virtual point test version
c     version      l.4   date    31-01-95 Erik: error removed
c     version      1.3   date    16-01-95 Erik: added allocation of
c                                preconditioner if saveprec = 1.
c                                New dimensions for ismatrix.
c     version      1.2   date    31-10-94 Kees K array geom added
c     version      1.1   date    17-08-94 RvN
c                                linfbnd and lbndcon adjusted
c     version      1.0   date    30-04-94
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Create space for arrays eddy, infbnd, bndcon, matrix,
c     gmat, and rhside.
c
c***********************************************************************
c
c                  KEYWORDS
c
c     parallel
c     receive
c     start_information
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none

      integer nblocks, neq

      integer iadmin(1:nblocks),     isiinput(1:nblocks),
     +        iseddy(1:nblocks),     isinfbnd(1:nblocks),
     +        isbndcon(1:nblocks),   ismatrix(1:nblocks,1:neq,*),
     +        isgmat(1:nblocks),     isrhside(1:nblocks),
     +        isgeom(1:nblocks)

c     iadmin    i    reserved
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     nblocks   i    Number of blocks
c     neq       i    Number of equations
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'cisblank'
      include 'cisnas'
      include 'cisparal'
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer isingetp

      integer iblock, idummy, iequa, ipiinblk, ipiinput, lbndcon,
     +      leddy, lgeom, lgmat, linfbnd, linfmg, linfmm, linfmt,
     +      lmatrix, lprec, lrhside, lsolutnv, lwork, maxb, maxcon,
     +      maxdim, maxdx, maxpoints, maxvpnts, ndegfd, ndim,
     +      ni, nj, nk, ntimlv, nvirtual, nxmax, saveprec

c
c     iblock    Block sequence number
c     idummy    Integer dummy
c     iequa     General equation sequence number
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     lbndcon   Length of array bndcon
c     leddy     Length of array eddy
c     lgeom     Length of array geom
c     lgmat     Length of array gmat
c     linfbnd   Length of array infbnd
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     lmatrix   Length of array matrix
c     lprec     Length of array prec
c     lrhside   Length of array rhside
c     lsolutnv  Length of temporary solution array
c     lwork     Length of array work
c     maxb      Maximum number of real values for one boundary condition
c     maxcon    Maximum number of non zero diagonals in a matrix
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     maxpoints Maximum number of points in a block
c     maxvpnts  Maximum number of points per block including virtual ones
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     nxmax     Maximum of ni, nj, nk
c     saveprec  Save preconditioner across domain decomposition
c               iterations (=1) or not (=0)
c
c*********************************************************************
c
c                   I/O
c
c*********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISACTIVE    Activate old permanent array
c     ISDCREAT    create space for new permanent double precision array
c     ISERCLOS    removes name of subroutine from error list
c     ISEROPEN    adds name of subroutine to error list
c     ISICREAT    create space for new permanent integer array
c     ISINGETP    get pointer of array
c     ISLSDS02  Compute lengths of temporary arrays
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c      for iblock = 1 (1) nblocks do
c
c         if block(iblock) on this node  then
c
c            for arrays  eddy, infbnd, bndcon, matrix,
c                       gmat, and  rhside  do
c
c               get length of array
c
c               create space for array
c
c=======================================================================

      call iseropen('isalloc')

c     --- determine SAVEPREC

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      saveprec = ibuffr(ipiinblk + 3 - 1)

      do 10 iblock = 1, nblocks

         iblockrf = iblock

         if( iadmin(iblock) .eq. inodenr) then

            call isactive(isiinput(iblock))
            ipiinput = isingetp( isiinput(iblock) )

            ni         = ibuffr( ipiinput + 51 -1 )
            nj         = ibuffr( ipiinput + 52 -1 )
            nk         = ibuffr( ipiinput + 53 -1 )
            ndim       = ibuffr( ipiinput + 57 -1 )
            ndegfd     = ibuffr( ipiinput + 55 -1 )
            nvirtual   = ibuffr( ipiinput + 54 -1 )
            maxb       = ibuffr( ipiinput + 61 -1 )
            maxcon     = ibuffr( ipiinput + 60 -1 )
            maxvpnts   = ibuffr( ipiinput + 69 -1 )
            maxpoints  = ibuffr( ipiinput + 71 -1 )


            nxmax   = max(ni, nj, nk)
            leddy   = maxvpnts

c           --- temporary length of array infbnd

            linfbnd = 2*ndim*(1+2*(ndim-1)) + 2 *
     +          ((2*nxmax+2+4*nvirtual)**(ndim-1)) * (2*ndim)
c           --- temporary
            lbndcon = maxb *
     +          ((2*nxmax+2+4*nvirtual)**(ndim-1)) * (2*ndim)
            lmatrix = maxpoints * ndim * maxcon
            lgmat   = maxpoints * ndim * maxcon
            lrhside = maxpoints * ndegfd
            lgeom   = maxvpnts * (2**ndim) * 2

            call isicreat( isinfbnd(iblock), linfbnd, 'infbnd' )

            call isdcreat( iseddy(iblock),       leddy  , 'eddy' )
            call isdcreat( isbndcon(iblock),     lbndcon, 'bndcon' )
            call isdcreat( ismatrix(iblock,1,1), lmatrix, 'matrix' )
            call isdcreat( isgmat(iblock),       lgmat  , 'gmat' )
            call isdcreat( isrhside(iblock),     lrhside, 'rhside' )
            call isdcreat( isgeom(iblock),       lgeom  , 'geom' )

            if ( saveprec .eq. 0 ) then

                  ismatrix(iblock,1,2) = 0

            else

               lprec = 0
               do iequa = 1, neq

                  call islsds02( iequa, ibuffr(ipiinput),
     +                           linfmm, linfmg, linfmt,
     +                           idummy, lsolutnv,  lwork,
     +                           ni, nj, nk, ndim ,
     +                           nvirtual, ndegfd , ntimlv,
     +                           maxdx, maxdim )

                 lprec = max( lprec, idummy)

               end do

               call isdcreat( ismatrix(iblock,1,2), lprec, 'prec' )

            end if

c           --- all matrices  and preconditioners for a subdomain share
c               the same storage for all equations
c               This may change in the future if for instance the
c               pressure matrix is saved between consecutive
c               time steps.

            do iequa = 2, neq
               ismatrix(iblock,iequa,1) = ismatrix(iblock,1,1)
               ismatrix(iblock,iequa,2) = ismatrix(iblock,1,2)
            end do

         end if

10    continue

      call iserclos('isalloc')

      end
      subroutine isfilb(iequa, iinput, rinput, infbnd, bndcon, coor,
     +   solut, coefs , eddy  , geom  , work  , lwork)
c=======================================================================
c
c     programmer   Kees Kassels
c
c
c     version      l.8   date    21-04-95 RvN virtual point test version
c     *** TEMPORARY and AD HOC SOLUTION ***
c     version      1.7   date    31-01-95  Marcel new calls (isfilb2 and
c                                                 isfilb3)
c     version      1.6   date    25-01-95  Marcel extension parameter list
c     version      1.5   date    03-01-95  Marcel extension transport +
c                                                 turbulence 3D
c     version      1.4   date    27-12-94 3D
c     version      1.3   date    22-11-94 Zijlema extension to k-omega model
c     version      1.2   date    30-09-94 Zijlema extension to turbulence
c
c     version      1.1   date    29-06-94 RvN small correction on
c                                first part of infbnd
c                                (nxmax -> nxmax+1 in positions 1-12)
c     version      1.0   date    13-01-94
c
c   copyright (c) 1993  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     routine which fills arrays infbnd and bndcon
c
c***********************************************************************
c
c                  KEYWORDS
c
c     boundary_conditions
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), infbnd(*), iequa, lwork
      double precision rinput(*), bndcon(*), coor(*), geom(*),
     +   solut(*), coefs(*), eddy(*), work(*)
      
c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     geom      i    reserved
c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     infbnd    o    reserved
c     lwork     i    Length of array work
c     rinput    i    reserved
c     solut     i    reserved
c     work      i    Work array
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer ipiinbc, ipiintur, iprinbc, iprintim, iprintur,
     +   ndim, ni, nj, nk, ntrnsp, nturb, ncoefs, ndegfd,
     +   ntimlv, maxb, nvirtual, mturb, viscorr,
     +   ipinvjac, npointsv
      integer iplowneighb, iphighneighb, ipibipnt, ipibindex,
     +   ipibinfor,
     +   ipinfface, ipindface, ibt1lo, ibt1hi, ibt2lo, ibt2hi, iface
      double precision time
      
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipinvjac  Starting address of array invjac in geom
c     iprinbc   Starting address of array rinbc in array iinput
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     maxb      Maximum number of real values for one boundary condition
c     mturb     Indicates type of turbulence modelling
c     ncoefs    Number of coefficients
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     time      Actual time of time step
c     viscorr   Indicate whether to use viscous corrections or not
c
c**********************************************************************
c
c                   I/O
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILB2   Actualy fills array infbnd and bndcon (2D)
c     ISFILB3   Actualy fills array infbnd and bndcon (3D)
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c      get pointers of the arrays iinbc, rinbc and rintim
c      get information from array iinput
c      call routine isfilbx, which actualy fills arrays infbnd and bndcon
c
c=======================================================================
      
      call iseropen ( 'isfilb' )
      
      ipiinbc   = iinput(2)
      ipiintur  = iinput(8)
      iprinbc   = iinput(27)
      iprintim  = iinput(29)
      iprintur  = iinput(31)
      time      = rinput(iprintim)
      
      ni        = iinput(51)
      nj        = iinput(52)
      nk        = iinput(53)
      nvirtual  = iinput(54)
      ndegfd    = iinput(55)
      ndim      = iinput(57)
      maxb      = iinput(61)
      ntrnsp    = iinput(63)
      ntimlv    = iinput(64)
      ncoefs    = iinput(66)
      
      nturb     = iinput(ipiintur -1 + 1)
      mturb     = iinput(ipiintur -1 + 2)
      viscorr   = iinput(ipiintur -1 + 3)
      
      npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      
      if (ndim.eq.3) then
         npointsv = npointsv*(nk+1+2*nvirtual)
      end if
      
c     --- startaddress for invjac in array geom
      
      ipinvjac = npointsv*(2**ndim) + 1
      
      if (ndim.eq.2) then
         
         call isfilb2( iinput(ipiinbc), rinput(iprinbc),
     +      infbnd, bndcon, coor, maxb, ni, nj, nk,
     +      ndim, ntrnsp, nturb, ntimlv, ncoefs,
     +      ndegfd, iequa, nvirtual, time, coefs,
     +      eddy, solut, mturb, geom(ipinvjac), work,
     +      lwork, viscorr, rinput(iprintur) )
         
      else
         
         call isfilb3(iinput(ipiinbc), rinput(iprinbc),
     +      infbnd, bndcon, coor, maxb, ni, nj, nk,
     +      ndim, ntrnsp, nturb, ntimlv, ncoefs,
     +      ndegfd, iequa, nvirtual, time,
     +      coefs, eddy, solut, mturb, geom(ipinvjac),
     +      work, lwork, viscorr, rinput(iprintur) )
         
      end if
      
c         --- Fill the positions in infbnd that are still zero 
c             with 
      
c     --- Initial values for pointers to parts 1,2 and 3 of infbnd
      
      ipibipnt  = 1
      ipibindex = ipibipnt  + 2*ndim
      ipibinfor = ipibindex + 2*(ndim-1)*2*ndim

      do iface=1,2*ndim
         
c           --- Pointer to face data
         ipinfface = infbnd(ipibipnt-1+iface)
c           --- Pointer to face boundaries
         ipindface = ipibindex+(iface-1)*2*(ndim-1)
         
         if (ndim.eq.2) then
            ibt1lo = infbnd(ipindface-1+1)
            ibt1hi = infbnd(ipindface-1+2)
            ibt2lo = 1
            ibt2hi = 1
c           --- TEMPORARY solution
            if (iface.eq.1 .or. iface.eq.2) then
               iplowneighb = infbnd(ipibipnt-1+3)+nvirtual
               iphighneighb = infbnd(ipibipnt-1+4)+nvirtual
            else if (iface.eq.3 .or. iface.eq.4) then
               iplowneighb = infbnd(ipibipnt-1+1)+nvirtual
               iphighneighb = infbnd(ipibipnt-1+2)+nvirtual
            end if
         else
            ibt1lo = infbnd(ipindface-1+1)
            ibt1hi = infbnd(ipindface-1+2)
            ibt2lo = infbnd(ipindface-1+2+1)
            ibt2hi = infbnd(ipindface-1+2+2)
         end if
         
         call isfilb4( maxb, ndim, ni, nj, nk, nvirtual, iface,
     +      ibt1lo, ibt1hi, ibt2lo, ibt2hi,
     +      infbnd(ipibinfor-1+ipinfface),
     +      bndcon(1+(ipinfface-1)*maxb) )
         
         if (ndim.eq.2) then 
c           --- TEMPORARY solution
            call isfilb5( maxb, ndim, ni, nj, nk, nvirtual, iface,
     +         ibt1lo, ibt1hi, 
     +         infbnd(ipibinfor-1+ipinfface),
     +         infbnd(ipibinfor-1+iplowneighb),
     +         infbnd(ipibinfor-1+iphighneighb),
     +         bndcon(1+(ipinfface-1)*maxb) )
         end if
         
      end do
      
      call iserclos ( 'isfilb' )
      
      end
      subroutine isfilb2(iinbc  , rinbc   , infbnd, bndcon, coor  ,
     +                   maxb   , ni      , nj    , nk    , ndim  ,
     +                   ntrnsp , nturb   , ntimlv, ncoefs, ndegfd,
     +                   iequa  , nvirtual, time  , coefs , eddy  ,
     +                   solut  , mturb   , invjac, work  , lwork ,
     +                   viscorr, rintur                           )
c=======================================================================
c
c     programmer   Marcel Zijlema
c
c     version      1.1   date    21-04-95 RvN virtual point test version
c     version      1.0   date    31-01-95
c
c   copyright (c) 1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     routine which fills arrays infbnd and bndcon (2D)
c
c***********************************************************************
c
c                  KEYWORDS
c
c     2d
c     boundary_conditions
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none

      integer maxb, ni, nj, nk, ndim, nvirtual, ntrnsp, nturb,
     +        ntimlv, ncoefs, ndegfd, viscorr, mturb,
     +        iequa, iinbc(*), infbnd(*), lwork

      double precision coor(*),
     +                 coefs(*), eddy(*), solut(*), invjac(*),
     +                 bndcon(1:2,*),
     +                 rinbc(*), time, work(*), rintur(*)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     iequa     i    General equation sequence number
c     iinbc     i    reserved
c     infbnd    o    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     lwork     i    Length of array work
c     maxb      i    Maximum number of real values for one boundary condition
c     mturb     i    Indicates type of turbulence modelling
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     time      i    Actual time of time step
c     viscorr   i    Indicate whether to use viscous corrections or not
c     work      i    Work array
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer nside, iplrinbc, nxmax

c     iplrinbc  Start address
c     nside     Number of boundary for which we need the
c               information
c     nxmax     Maximum of ni, nj, nk
c
c**********************************************************************
c
c                   I/O
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINBC2   Actualy fills array infbnd and bndcon (2D)
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c=======================================================================

      call iseropen ( 'isfilb2' )

      nxmax      = max(ni,nj,nk)

      infbnd(1 ) = 1
      infbnd(2 ) = (nxmax+1+2*nvirtual)*2 + 1
      infbnd(3 ) = (nxmax+1+2*nvirtual)*4 + 1
      infbnd(4 ) = (nxmax+1+2*nvirtual)*6 + 1
c     --- bounds for face 1
      infbnd(5 ) = 1-nvirtual
      infbnd(6 ) = 2*nj+1+nvirtual
c     --- bounds for face 2
      infbnd(7 ) = 1-nvirtual
      infbnd(8 ) = 2*nj+1+nvirtual
c     --- bounds for face 3
      infbnd(9 ) = 1-nvirtual
      infbnd(10) = 2*ni+1+nvirtual
c     --- bounds for face 4
      infbnd(11) = 1-nvirtual
      infbnd(12) = 2*ni+1+nvirtual

      iplrinbc = 0

      do nside = 1, 2*ndim

        call isinbc2( iinbc, rinbc, nside,
     +                infbnd(2*ndim+(nside-1)*2+1),
     +                infbnd(2*ndim+(nside-1)*2+2),
     +                infbnd(12+infbnd(nside)),
     +                bndcon(1,infbnd(nside)), coor, maxb, ni,
     +                nj, nk, ndim, ntrnsp, nturb, ntimlv, ncoefs,
     +                ndegfd, iequa, nvirtual, time, iplrinbc, coefs,
     +                eddy, solut, mturb, invjac, work,
     +                lwork, viscorr, rintur )

      end do

      call iserclos ( 'isfilb2' )

      end
      subroutine isfilb3(iinbc  , rinbc   , infbnd, bndcon, coor  ,
     +                   maxb   , ni      , nj    , nk    , ndim  ,
     +                   ntrnsp , nturb   , ntimlv, ncoefs, ndegfd,
     +                   iequa  , nvirtual, time  , coefs , eddy  ,
     +                   solut  , mturb   , invjac, work  , lwork ,
     +                   viscorr, rintur                           )
c=======================================================================
c
c     programmer   Marcel Zijlema
c
c     version      1.1   date    21-04-95 RvN virtual point test version
c     version      1.0   date    31-01-95
c
c   copyright (c) 1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     routine which fills arrays infbnd and bndcon (3D)
c
c***********************************************************************
c
c                  KEYWORDS
c
c     3d
c     boundary_conditions
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none

      integer maxb, ni, nj, nk, ndim, nvirtual, ntrnsp, nturb,
     +        ntimlv, ncoefs, ndegfd, viscorr, mturb,
     +        iequa, iinbc(*), infbnd(*), lwork

      double precision coor(*),
     +                 coefs(*), eddy(*), solut(*), invjac(*),
     +                 bndcon(1:3,*),
     +                 rinbc(*), time, work(*), rintur(*)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     iequa     i    General equation sequence number
c     iinbc     i    reserved
c     infbnd    o    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     lwork     i    Length of array work
c     maxb      i    Maximum number of real values for one boundary condition
c     mturb     i    Indicates type of turbulence modelling
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     time      i    Actual time of time step
c     viscorr   i    Indicate whether to use viscous corrections or not
c     work      i    Work array
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer nside, iplrinbc, nxmax

c     iplrinbc  Start address
c     nside     Number of boundary for which we need the
c               information
c     nxmax     Maximum of ni, nj, nk
c
c**********************************************************************
c
c                   I/O
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINBC3   Actualy fills array infbnd and bndcon (3D)
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c=======================================================================

      call iseropen ( 'isfilb3' )

      nxmax      = max(ni,nj,nk)

      infbnd(1 ) = 1
      infbnd(2 ) = 1*(2*nxmax+1+2*nvirtual)**2 + 1
      infbnd(3 ) = 2*(2*nxmax+1+2*nvirtual)**2 + 1
      infbnd(4 ) = 3*(2*nxmax+1+2*nvirtual)**2 + 1
      infbnd(5 ) = 4*(2*nxmax+1+2*nvirtual)**2 + 1
      infbnd(6 ) = 5*(2*nxmax+1+2*nvirtual)**2 + 1
c     --- bounds for face 1
      infbnd(7 ) = 1-nvirtual
      infbnd(8 ) = 2*nj+1+nvirtual
      infbnd(9 ) = 1-nvirtual
      infbnd(10) = 2*nk+1+nvirtual
c     --- bounds for face 2
      infbnd(11) = 1-nvirtual
      infbnd(12) = 2*nj+1+nvirtual
      infbnd(13) = 1-nvirtual
      infbnd(14) = 2*nk+1+nvirtual
c     --- bounds for face 3
      infbnd(15) = 1-nvirtual
      infbnd(16) = 2*ni+1+nvirtual
      infbnd(17) = 1-nvirtual
      infbnd(18) = 2*nk+1+nvirtual
c     --- bounds for face 4
      infbnd(19) = 1-nvirtual
      infbnd(20) = 2*ni+1+nvirtual
      infbnd(21) = 1-nvirtual
      infbnd(22) = 2*nk+1+nvirtual
c     --- bounds for face 5
      infbnd(23) = 1-nvirtual
      infbnd(24) = 2*ni+1+nvirtual
      infbnd(25) = 1-nvirtual
      infbnd(26) = 2*nj+1+nvirtual
c     --- bounds for face 6
      infbnd(27) = 1-nvirtual
      infbnd(28) = 2*ni+1+nvirtual
      infbnd(29) = 1-nvirtual
      infbnd(30) = 2*nj+1+nvirtual

      iplrinbc = 0

      do nside = 1, 2*ndim

         call isinbc3(iinbc, rinbc, nside,
     +                infbnd(2*ndim+(nside-1)*4+1),
     +                infbnd(2*ndim+(nside-1)*4+2),
     +                infbnd(2*ndim+(nside-1)*4+3),
     +                infbnd(2*ndim+(nside-1)*4+4),
     +                infbnd(31-1+infbnd(nside)),
     +                bndcon(1,infbnd(nside)), coor, maxb, ni,
     +                nj, nk, ndim, ntrnsp, nturb, ntimlv, ncoefs,
     +                ndegfd, iequa, nvirtual, time, iplrinbc,
     +                coefs, eddy, solut, mturb, invjac,
     +                work, lwork, viscorr, rintur)

      end do

      call iserclos ( 'isfilb3' )

      end
      subroutine isfilb4 (  maxb, ndim, ni, nj, nk, nvirtual, iface,
     +   ibt1lo, ibt1hi, ibt2lo, ibt2hi, ibinfor, bndcon )
c ======================================================================
c
c
c        programmer     ronald van Nooyen
c        version 1.0    date 21-04-95
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c              Temporary routine to fill virtual points in infbnd 
c              and bndcon
c              with value to indicate outside points except for 
c              edges involved with periodical boundary conditions
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      integer maxdim
      parameter(maxdim=3)
c
      integer maxb, ni, nj, nk, ndim, nvirtual, iface,
     +   ibt1lo, ibt1hi, ibt2lo, ibt2hi
      integer ibinfor(ibt1lo:ibt1hi,ibt2lo:ibt2hi)
      double precision bndcon(1:maxb,ibt1lo:ibt1hi,ibt2lo:ibt2hi)
c
c
c     ibinfor  i/o    part three of infbnd
c     ibt1hi    i    Upper bound on an array in ntdir(2)-direction 
c     ibt1lo    i    Lower bound on an array in ntdir(2)-direction
c     ibt2hi    i    Upper bound on an array in ntdir(3)-direction 
c     ibt2lo    i    Lower bound on an array in ntdir(3)-direction
c     iface     i    Face number
c     ipequa    i    A given equation
c     maxdim         Maximum dimension
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
      integer outside
      parameter (outside = 101)
      integer i, j
      
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      
      do j=ibt2lo,ibt2hi
         do i=ibt1lo,ibt1hi
            if (ibinfor(i,j).eq.0) ibinfor(i,j) = outside
         end do
      end do

      end
      subroutine isfilb5 (  maxb, ndim, ni, nj, nk, nvirtual, iface,
     +   ibt1lo, ibt1hi, ibinfor, lowneighb, highneighb, bndcon )
c ======================================================================
c
c
c        programmer     ronald van Nooyen
c        version 1.0    date 21-04-95
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c              Temporary routine to fill virtual points in infbnd 
c              and bndcon
c              with value to indicate outside points except for 
c              edges involved with periodical boundary conditions
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      integer maxdim
      parameter(maxdim=3)
c
      integer ni, nj, nk, ndim, maxb, nvirtual, iface,
     +   ibt1lo, ibt1hi, lowneighb, highneighb
      integer ibinfor(ibt1lo:ibt1hi)
      double precision bndcon(1:maxb,ibt1lo:ibt1hi)
c
c
c     ibinfor  i/o    part three of infbnd
c     ibt1hi    i    Upper bound on an array in ntdir(2)-direction 
c     ibt1lo    i    Lower bound on an array in ntdir(2)-direction
c     ibt2hi    i    Upper bound on an array in ntdir(3)-direction 
c     ibt2lo    i    Lower bound on an array in ntdir(3)-direction
c     iface     i    Face number
c     ipequa    i    A given equation
c     maxdim         Maximum dimension
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
      integer periodic
      parameter (periodic = 98)
      integer i, ibshift
      
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isfilb5')
      
      if (ndim.eq.2) then
         
         if (ibt1lo.eq.1) then
            call iserrint ( ibt1lo, 1 )
            call iserrint ( ibt1hi, 2 )
            call iserrchr ( 'ibt1lo', 1 )
            call iserrchr ( 'ibt1hi', 2 )
            call iserrsub(12,2,0,2)
         end if
         
         if (lowneighb.eq.periodic) then
            if (highneighb.ne.periodic) then
               call iserrint ( lowneighb, 1 )
               call iserrint ( highneighb, 2 )
               call iserrchr ( 'lowneighb', 1 )
               call iserrchr ( 'highneighb', 2 )
               call iserrsub(12,2,0,2)
            end if
            
            ibshift = (ibt1hi-ibt1lo-4*nvirtual)
            do i=ibt1lo,ibt1lo+2*nvirtual-1
               ibinfor(i) =  ibinfor(i+ibshift)
               bndcon(1,i) = bndcon(1,i+ibshift)
               bndcon(2,i) = bndcon(2,i+ibshift)
            end do
            
            do i=ibt1hi-2*nvirtual+1,ibt1hi
               ibinfor(i) =  ibinfor(i-ibshift)
               bndcon(1,i) = bndcon(1,i-ibshift)
               bndcon(2,i) = bndcon(2,i-ibshift)
            end do

         end if
 
      else

         call iserrsub(5,0,0,0)

      end if
      
      call iserclos('isfilb5')
      
      end
      subroutine ismmbf ( ndim, ni, nj, nk, nvirtual,
     +                    nkvirtual, ndegfd, ntimlv, nconct, maxb,
     +                    maxdx,it1lo, it1hi, it2lo, it2hi,
     +                    mct, mctinv,matrix, solut, rhside, invjac,
     +                    ibinfor, bndcon, bcgeo,
     +                    iface, iconct, idir, matarea, iphase )
c=======================================================================
c
c
c        programmer     Ronald van Nooyen

c     version      l.1   date    21-04-95 RvN virtual point test version
c     *** TEMPORARY and AD HOC SOLUTION ***
c        version 1.0    date 01-11-94 ismcbcf+ismmbf replace ismmbf2
c
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c*********************************************************************
c
c                      DESCRIPTION
c
c   Momentum matrix boundary condition folder
c
c*********************************************************************
c
c                      KEYWORDS
c
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      integer maxdim
      parameter ( maxdim = 3 )
c
c
      integer iconct, idir, it1hi, it1lo, it2hi, it2lo, maxb, nconct,
     +        ndegfd, ndim, ni, nj, nk, nkvirtual, ntimlv, nvirtual,
     +        maxdx


      integer mct(1:ndim,1:ndim,-maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx),
     +        mctinv(0:maxdim,1:ndim,1:nconct),
     +        matarea(1:2,1:maxdim), iphase

      integer ibinfor(it1lo:it1hi,it2lo:it2hi)

      double precision bcgeo(1:ndim,1:ndim,it1lo:it1hi,it2lo:it2hi),
     +                 bndcon(1:maxb,it1lo:it1hi,it2lo:it2hi)

      double precision matrix(1:ni+1, 1:nj+1, 1:nk+1,1:ndim, 1:nconct),
     +                 rhside(1:ni+1, 1:nj+1, 1:nk+1, 1:ndim)

      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,
     +                 1-nkvirtual:nk+1+nkvirtual,
     +                 1:ndegfd, 1:ntimlv),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,
     +                 1-nkvirtual:nk+1+nkvirtual,1:2**ndim)

c     bcgeo     i    Boundary geometry information
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     iconct    i    Loop variable for loop over all connections in the matrx
c     idir      i    General loop variable for 1...ndim loop
c     iface     i    Face number
c     invjac    i    1/sqrt(g) in pnttyp
c     iphase    i    Phase in folding process
c     it1hi     i    Upper bound on first tangential index for destination
c     it1lo     i    Lower bound on first tangential index for destination
c     it2hi     i    Upper bound on second tangential index for destination
c     it2lo     i    Lower bound on second tangential index for destination
c     matarea   i    Index bounds for the active area of the matrix
c     matrix    i    reserved
c     maxb      i    Maximum number of real values for one boundary condition
c     maxdx     i    Maximum relative shift between cell indices
c     mct       i    Integer matrix information
c     mctinv    i    Matrix information
c     nconct    i    Number of connections in a row of the matrix
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nkvirtual  i    ?
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     rhside    i    reserved
c     solut     i    reserved
c
c*********************************************************************
c
c                      COMMON BLOCKS
c
c*********************************************************************
c
c                      LOCAL PARAMETERS
c
c

      integer unut, snnsnt, periodic, antiper, multiblk,
     +        uxuy, unsnt, snnut, perjump, antijump,
     +        oldturb, parabolic
      parameter( unut = 4, snnsnt = 7, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           snnut = 6,
     +           oldturb = 9, parabolic = 12)
c
c     unut         U_n and u_t prescribed
c     snnsnt       Sigma_nn and sigma_nt prescribed
c     periodic     Periodic boundary conditions
c     antiper      Anti-periodic boundary conditions
c     perjump      Periodic boundary conditions with jump in pressure
c     antijump     Anti-periodic boundary conditions with jump in
c                  pressure
c     multiblk     Multi-block boundary conditions
c     uxuy         U_x and u_y prescribed
c     unsnt        U_n and sigma_nt prescribed
c     snnut        Sigma_nn and u_t prescribed
c     oldturb      Old turbulence boundary condtion
c                  (equivalent to unsnt)
c
      integer outside, leavealone, masspar
      parameter (outside = 101, leavealone = 300, masspar = 100)
c
      logical dbhc, Uknown(1:maxdim), Vknown(1:maxdim)

      integer icdp(1:maxdim), icp(1:maxdim), ivp(1:maxdim),
     +        ifcarea(1:2,1:maxdim), ifvarea(1:2,1:maxdim), ivardir,
     +        ievp(1:maxdim,-1:1), ieconct(-1:1),
     +        nsig, ntdir(1:maxdim), ntsig(1:maxdim), lnvar(1:maxdim),
     +        ictp(1:maxdim-1), ivtp(1:maxdim-1), ichois, iface,
     +        ip, jp, kp, icbtype, ivbtype, ibtype, ndist, i, jdim,
     +        itconct(-1:0), ndir, lcell(1:maxdim), lvar(1:maxdim)

      double precision texfactor(-1:0),
     +                 vval(1:maxdim), vfac(1:maxdim,1:maxdim),
     +                 vnfac, exratio, invsqrtg(-1:1)

c

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     dbhc    ?
c     exratio   Ratio of distance between base points for extrapolation
c               and distance between the point to which we extrapolate
c               and the base point closest to it
c                        Examples:
c                             |     *
c                             -     -
c                             |     *
c                        --|--+--|--*--|--+--|--+
c                          B1    B2          E
c                        Linear extrapolation:
c                        f(E) = (exratio+1d0) f(B2) - exratio f(B1)
c                        with exratio = 2
c                             |     *
c                             -     -
c                             |     *
c                        --|--+--|--*--|--+--|--+
c                                B1 B2       E
c                        Linear extrapolation:
c                        f(E) = (exratio+1d0) f(B2) - exratio f(B1)
c                        with exratio = 3
c     i         General loop variable
c     ibtype  ?
c     icbtype  ?
c     icdp      Index offset of cell containing the current variable
c               relative to the cell containing the integration cell centre
c     ichois    General choice parameter
c     icp       Vector of integration cell indices
c     ictp      Vector of ibinfor indices for integation cell centre
c     ieconct  ?
c     ievp    ?
c     ifcarea   Index bounds for cells on this face
c     ifvarea   Index bounds for vertices on this face
c     invsqrtg  ?
c     ip        Local counting variable i-direction
c     itconct  ?
c     ivardir  ?
c     ivbtype  ?
c     ivp       Vector of variable location cell indices
c     ivtp      Vector of ibinfor indices for current variable
c     jdim      Loop variable used for dimension
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     lcell     Location of the integration cell centre within a grid cell
c               in Cell Local Coordinates
c               (i.e. (0,0,0): lower left vertex, (1,1,1): cell centre, etc.)
c     leavealone  ?
c     lnvar   ?
c     lvar      Location of the current variable within a grid cell
c               in Cell Local Coordinates
c               (i.e. (0,0,0): lower left vertex, (1,1,1): cell centre, etc.)
c     masspar  ?
c     maxdim    Highest physical space dimension for which ISNaS can work
c     multiblk  Multi-block boundary conditions
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     ntdir     Basis vector a_ntdir(1) corresponding to the
c               normal direction
c               Basis vector a_ntdir(2) corresponding to the
c               (in 3D: first) tangential direction
c               in 3D: a_ntdir(3) basis vector corresponding to
c               the second tangential direction
c     ntsig     Sign of inner product of true normal or
c               tangential vectors and the corresponding basis
c               vector
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     outside  ?
c     parabolic  ?
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     texfactor  ?
c     uknown    Cartesian velocity in given direction known
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     vfac      Factors for derivation of flux if not known
c               explicitly
c     vnfac     Factors for normal derivation of flux if not known
c               explicitly
c     vknown    Flux in given direction known
c     vval      Flux values
c
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISBNBVI   Boundary information
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      PSEUDO CODE
c
c     Loop over connections
c        Loop over integration cell types
c
c=======================================================================
c
      call iseropen ( 'ismmbf' )

c     --- Get the normal and tangential directions (ntdir,ntsig), the
c         grid cells adjacent to the boundary (ifcarea) and the
c         vertices on the boundary (ifvarea)

      call isbnbvi(ndim, ni, nj, nk, iface, ntdir, ntsig,
     +             ifcarea, ifvarea)

c     --- Get direction and sign of outward normal

      ndir = ntdir(1)
      nsig = ntsig(1)

c     --- Get flux component direction corresponding to current
c         matrix element

      ivardir = mctinv(0,idir,iconct)

c     --- Get local integration cell centre position

      lcell(1) = 1
      lcell(2) = 1

      if (ndim.eq.2) then
c     --- zero for k direction
         lcell(3) = 0
      else
         lcell(3) = 1
      end if

      lcell(idir) = 0

c     --- Get local variable position

      lvar(1)  = 1
      lvar(2)  = 1

      if (ndim.eq.2) then
c     --- zero for k direction
         lvar(3)  = 0
      else
         lvar(3)  = 1
      end if

      lvar(ivardir) = 0

c     --- Get cell index offset relative to integration cell for
c         current variable

      icdp(1) = mctinv(1,idir,iconct)
      icdp(2) = mctinv(2,idir,iconct)
      icdp(3) = mctinv(3,idir,iconct)

c     --- Loop over integration cell centres


      do 10000 kp=matarea(1,3),matarea(2,3)
         do 11000 jp=matarea(1,2),matarea(2,2)
            do 11100 ip=matarea(1,1),matarea(2,1)

               icp(1) = ip
               icp(2) = jp
               icp(3) = kp
               ictp(1) = 2*icp(ntdir(2))-1+lcell(ntdir(2))

               if (ndim.eq.2) then
                  ictp(2) = 1
               else
                  ictp(2) = 2*icp(ntdir(3))-1+lcell(ntdir(3))
               end if

               icbtype = ibinfor(ictp(1),ictp(2))

c              --- Check for Dirichlet half cell

               dbhc = ( idir.eq.ndir .and. (icbtype.eq.uxuy .or.
     +                 icbtype.eq.unut .or. icbtype.eq.parabolic .or.
     +                 icbtype.eq.unsnt .or. icbtype.eq.oldturb ) )

c              --- Check for btypes that need no further action

               ivp(1) = icp(1)+icdp(1)
               ivp(2) = icp(2)+icdp(2)
               ivp(3) = icp(3)+icdp(3)

               ivtp(1) = 2*ivp(ntdir(2))-1+lvar(ntdir(2))

               if (ndim.eq.2) then
                  ivtp(2) = 1
               else
                  ivtp(2) = 2*ivp(ntdir(3))-1+lvar(ntdir(3))
               end if

c              --- Get boundary information

               if ( ivtp(1).lt.it1lo .or. ivtp(1).gt.it1hi .or.
     +            ivtp(2).lt.it2lo .or. ivtp(2).gt.it2hi ) then
c                    --- No boundary information available
                  ivbtype = outside
               else
                  ivbtype = ibinfor(ivtp(1),ivtp(2))
               end if

c              --- Get reduced boundary condition type

               ibtype = ivbtype
               if (ibtype.eq.parabolic) then
                  ibtype = unut
               end if
               if (ibtype.eq.oldturb) then
                  ibtype = unsnt
               end if
               if (ibtype.eq.multiblk .or. ibtype.eq.periodic .or.
     +            ibtype.eq.antiper .or.
     +            ibtype.eq.perjump .or. ibtype.eq.antijump .or.
     +            ibtype.eq.masspar) then
                  ibtype = leavealone
               end if
               ivbtype = ibtype

c              --- Determine distance to domain edge

               if (ivardir.eq.ndir) then
c              --- Flux in normal direction
                  ndist = 2*(ivp(ndir)-ifvarea(1,ndir))
               else
c              --- Flux in tangential direction
                  if (nsig.eq.-1) then
c                 --- Lower edge
                     ndist = 2*(ivp(ndir)-ifvarea(1,ndir))+1
                  else
c                 --- Upper edge
                     ndist = 2*(ivp(ndir)-ifvarea(1,ndir))+1
                  end if
               end if

c              --- Point outside: ndist positive
               ndist = nsig*ndist

c              --- Get boundary flux information (when available)

               ichois = 0
               if (ivbtype.eq.uxuy) ichois  = 1
               if (ivbtype.eq.unut) ichois  = 2
               if (ivbtype.eq.unsnt) ichois = 3
               if (ivbtype.eq.snnut) ichois = 4

               if (ichois.eq.0) then
                  Uknown(1) = .false.
                  Uknown(2) = .false.
                  Uknown(3) = .false.
                  Vknown(1) = .false.
                  Vknown(2) = .false.
                  Vknown(3) = .false.
               else
                  call ismmgbc(ndim, maxb, ntdir, ntsig,
     +                         Uknown, Vknown, Vval, Vfac,
     +                         bcgeo(1,1,ivtp(1),ivtp(2)),
     +                         bndcon(1,ivtp(1),ivtp(2)), ichois )
               end if

               if (dbhc .and. iphase.eq.ndim+1) then
c              --- Dirichlet boundary half cell

                  if ( iconct .eq. mct(ndir,ndir,0,0,0) ) then
c                 --- Matrix element on diagonal

                     matrix(icp(1),icp(2),icp(3),ndir,iconct) = 1d0
                     solut(ivp(1),ivp(2),ivp(3),ndir,ntimlv) =
     +                  Vval(ndir)
                     rhside(icp(1),icp(2),icp(3),ndir) = Vval(ndir)
                  else	
                     matrix(icp(1),icp(2),icp(3),ndir,iconct) = 0d0
                  end if

               else if (ivbtype.eq.leavealone) then
c              --- Leave this matrix element as it is
               else if (ndist.lt.0) then
c              --- Matrix element inside the domain in normal direction
c                 --- No action needed in this direction
c                      If folding is needed in tdir(iface), then
c                      this is done in the first or
c                      second call of ismmbf
c                      for the edge jface
c                      where ndir(jface).eq.tdir(iface)
               else if (ndist.eq.0 .and. iphase.eq.ndim+1) then
c              --- Matrix element on domain edge

                  if (ivbtype.eq.outside) then
c                 --- This element should be folded from another edge
                  else if (ivardir.eq.ndir .and. Vknown(ndir)) then
c                 --- Eliminate known flux on edge

                     if ( ndim .eq. 2 ) then

c                 --- Temporary exception for 3D code

                     rhside(icp(1),icp(2),icp(3),idir) =
     +                   rhside(icp(1),icp(2),icp(3),idir) -
     +                   matrix(icp(1),icp(2),icp(3),idir,iconct)*
     +                   Vval(ndir)
                     solut(ivp(1),ivp(2),ivp(3),ndir,ntimlv)  =
     +                  Vval(ndir)
                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0

                     end if

                  end if

               else if (ndist.gt.0 .and. iphase.le.ndim) then
c              --- Matrix element outside domain in normal direction
c                 --- Eliminate virtual matrix elements where necessary
c                     First determine the cells where the variables
c                     needed for elmination are located

                  do 11110 i=-1,1
                     do 11111 jdim=1,maxdim
                        ievp(jdim,i) = ivp(jdim)
11111                continue
11110             continue

                  if (nsig.eq.-1) then
c                 --- Closest value within domain shares ndir
c                     coordinate with edge vertices
                     ievp(ndir, 0) = ifvarea(1,ndir)
                  else
c                 --- We need to distinguish several cases
                     if (ivardir.eq.ndir) then
c                    --- Closest value within domain shares ndir
c                        coordinate with edge vertices
                        ievp(ndir, 0) = ifvarea(1,ndir)
                     else if (ivardir.ne.ndir) then
c                    --- Closest value within domain is shifted inward
c                        along ndir relative to the edge vertices.
                        ievp(ndir, 0) = ifvarea(1,ndir)-1
                     end if
                  end if

c                 --- ndir location of next closest element

                  ievp(ndir,-1) = ievp(ndir,0)-nsig

c                 --- Get the matrix connection indices corresponding to
c                     the elements involved in the elimination

                  do 11120 i=-1,1
                     ieconct(i) = mct(idir,ivardir,
     +                  ievp(1,i)-icp(1),ievp(2,i)-icp(2),
     +                  ievp(3,i)-icp(3))
11120             continue

c                 --- Perform the elimination

                  if (Vknown(ivardir)) then
c                 --- Known flux

c                    --- Determine the ratio of the distance
c                        between the points on which we base the
c                        extrapolation and the distance between
c                        the point to which we extrapolate and
c                        the closest base point

                     if (ivardir.eq.ndir) then
                        exratio = (1d0*ndist)/2d0
                     else
                        exratio = 1d0*ndist
                     end if

                     matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(0))
     +                  - exratio*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

                     rhside(icp(1),icp(2),icp(3),idir) =
     +                  rhside(icp(1),icp(2),icp(3),idir) -
     +                  (exratio+1d0)*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)*
     +                  Vval(ivardir)

                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0

                  else if ( Uknown(ivardir) .and.
     +                  ( (idir.eq.ndir) .or.
     +                  (ndim.eq.2 .and. nconct.ge.21) .or.
     +                  (ndim.eq.3) .or.
     +                  (icdp(ivardir).eq.0) ) ) then
c                 --- Unknown flux, but known velocity
c                     (must be tangential)

c                     --- On edge: Vt = utfact*ut + Vnfact*Vn
c                         Implementation:
c                            Extrapolate using
c                            closest tdir matrix element in domain
c                            interior and point on domain edge.
c                            The contribution of the point on the
c                            domain edge is distributed over the rhside
c                            and the two adjacent ndir matrix
c                            elements
c                        Note that for a 13 point stencil and a
c                        tangential cell this may call for extrapolation
c                        in the ivardir direction

c                    --- Get ratio

                     exratio = 1d0*ndist

c                    --- First the point in the interior

                     matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(0))
     +                  - exratio*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

c                    --- Then the point on the edge

c                    --- Grid cell index offset for tangential flux
c                        closest to the edge

                     lnvar(1)    = ievp(1,0)-icp(1)
                     lnvar(2)    = ievp(2,0)-icp(2)
                     lnvar(3)    = ievp(3,0)-icp(3)

c                    --- Correct to get ndir location

                     lnvar(ndir) = ifvarea(1,ndir)-icp(ndir)

c                    --- This is the ndir location to the
c                        right of our tdir flux along direction
c                        ivardir

c                    --- Get connection index for the corresponding
c                        matrix element

                     itconct(0) =
     +                  mct(idir,ndir,lnvar(1),lnvar(2),lnvar(3))

c                    --- Grid cell index offset for tangential flux
c                        closest to the edge

                     lnvar(1)    = ievp(1,0)-icp(1)
                     lnvar(2)    = ievp(2,0)-icp(2)
                     lnvar(3)    = ievp(3,0)-icp(3)

c                    --- Correct to get ndir location

                     lnvar(ndir) = ifvarea(1,ndir)-icp(ndir)

c                    --- Correct to get ndir location to the
c                        left of our tdir flux along direction
c                        ivardir

                     lnvar(ivardir) = lnvar(ivardir) - 1

c                    --- Get connection index for the corresponding
c                        matrix element

                     itconct(-1) =
     +                  mct(idir,ndir,lnvar(1),lnvar(2),lnvar(3))

c                    --- Set interpolation weight factors
c                        for interpolation in ivardir

                     texfactor(-1) = 5d-1
                     texfactor(0)  = 5d-1

                     if (itconct(0).lt.1 .or. itconct(0).gt.nconct) then
c                    --- rightmost tangential velocity in tangential
c                        cell lies outside the stencil
c                        use extraplation for ivardir
c                        in stead of interpolation
                        texfactor(-1) = -5d-1
                        texfactor(0)  = 15d-1
                        itconct(0)    = itconct(-1)
                        lnvar(ivardir) = lnvar(ivardir) - 1
                        itconct(-1)    =
     +                     mct(idir,ndir,lnvar(1),lnvar(2),lnvar(3))
                     else if (itconct(-1).lt.1 .or.
     +                     itconct(-1).gt.nconct) then
c                    --- lefttmost tangential velocity in tangential
c                        cell lies outside the stencil
c                        use extraplation for ivardir
c                        in stead of interpolation
                        texfactor(-1) = 15d-1
                        texfactor(0)  = -5d-1
                        itconct(-1)    = itconct(0)
                        lnvar(ivardir) = lnvar(ivardir) + 2
                        itconct(0)     =
     +                     mct(idir,ndir,lnvar(1),lnvar(2),lnvar(3))
                     end if

                     if ( ndim .eq. 2 ) then
                       vnfac = Vfac(ndir,1)
                     else
                       vnfac = Vfac(ndir,ivardir)
c                       vnfac = Vfac(ndir,idir)
                     end if

                     matrix(icp(1),icp(2),icp(3),idir,itconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,itconct(0))
     +                  + texfactor(0)*(exratio+1d0)*vnfac*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

                     matrix(icp(1),icp(2),icp(3),idir,itconct(-1)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,itconct(-1))
     +                  + texfactor(-1)*(exratio+1d0)*vnfac*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

c                    --- Now adjust rhside

                     rhside(icp(1),icp(2),icp(3),idir) =
     +                  rhside(icp(1),icp(2),icp(3),idir) -
     +                  (exratio+1d0)*Vval(ivardir)*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)



                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0


                  else if (ieconct(-1).lt.1 .or. ieconct(-1).gt.nconct)
     +                  then
c                 --- Unknown flux, unknown velocity,
c                     insufficient points in stencil for extrapolation

c                    --- probably 13 point stencil, use
c                        report 91-09 page 34: assume
c                        d U^1 / d ksi^2 = 0
c                        ===== (not d V^1)

c                    --- Get invjac in point outside the domain

                     invsqrtg(1) = invjac(ievp(1,1),ievp(2,1),ievp(3,1),
     +                              2**ndim-2**(ivardir-1))

c                    --- Get invjac in point closest to the edge

                     invsqrtg(0) = invjac(ievp(1,0),ievp(2,0),ievp(3,0),
     +                              2**ndim-2**(ivardir-1))

                     matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(0))+
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)*
     +                  invsqrtg(0)/invsqrtg(1)

                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0

                  else
c                 --- Unknown flux, unknown velocity,
c                     perform extrapolation

                     if (ivardir.eq.ndir) then
                        exratio = 1d0*(ndist/2)
                     else
                        exratio = 1d0*((ndist+1)/2)
                     end if

                     matrix(icp(1),icp(2),icp(3),idir,ieconct(-1)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(-1)) -
     +                  exratio*matrix(icp(1),icp(2),icp(3),idir,iconct)
                     matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) +
     +                  (exratio+1d0)*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0

                  end if
               end if

11100       continue
11000    continue
10000 continue

      call iserclos ( 'ismmbf' )
      end



      subroutine ismmccb2 ( ndim, ni, nj, nk, nvirtual, ndegfd, ntimlv,
     +                      nconct,ncoefs, maxb, maxdx, npointsnv,
     +                      npointsv, dt, iindsc, coor, coefs, matrix,
     +                      solut, rhside, jac, invjac, rho, mu, veloc,
     +                      cellvol,cva,jcta,jctg,chsym, vsum, dvsum,
     +                      bcgeo, bndcon, ibipoint, ibindex, ibinfor,
     +                      npf, rspv, cv, mct, mctinv, istwo, vcellc )
c=======================================================================
c
c
c        programmer     Ronald van Nooyen
c     version      4.3  date    21-04-95 RvN virtual point test version
c     *** TEMPORARY and AD HOC SOLUTION ***
c        version 4.2    date 15-03-95 Guus, correction noconvection
c        version 4.1    date 20-12-94 RvN: wesbeek bounds introduced
c        version 4.0    date 09-11-94 RvN: general code cleanup
c
c     version     3.4  ISNAS     date    23-08-1994 R. van Nooyen
c                                13 point stencil problem at last
c                                isdnxfm call solved
c     version     3.3  ISNAS     date    27-06-1994 R. van Nooyen
c                    Changes:
c                       mct -> mmct, gmct, smct.
c                       Virtual points used to avoid special
c                       treatment of boundaries by interpolation
c                       and finite difference routines.
c                       Bounds:
c                          matrix without virtual points,
c                          solution and coefficients with virtual
c                          points.
c                       Boundary information structure changed
c                       Array bounds transport changed
c     version      3.2   ISNAS    date 07-06-1994 RvN
c                                    parallel version
c     version      3.1   ISNAS    date  11-01-1993 RvN
c                                      Initial cached version;
c                                      Common blocks removed where
c                                      possible;
c                                      Preparations for parallel
c                                      version under way

c     version      3.01  ISNAS    date 03-01-1994 RvN preparations
c     version      3.0   ISNAS    date 08-12-1993 RvN preparations for
c                                 cached and parallel version
c     version      2.4   ISNAS    date 24-11-1993 RvN
c                                 Flux interpolation in stead of
c                                 velocity interpolation in
c                                 isdnxfm for WESBEEK viscosity
c
c     version      2.31  ISNAS    date 22-11-1993 RvN 13 pt stencil for
c                                 folder for abs(viscdn).eq.1
c     version      2.3   ISNAS    date 15-11-1993 RvN WESBEEK viscosity
c     version      2.23  ISNAS    date 05-11-1993 RvN
c                                                 code streamlined
c     version      2.22  ISNAS    date 01-11-1993 RvN
c                                                 edgecode corrected
c     version      2.21  ISNAS    date 01-11-1993 RvN nconct passed to
c                                                 iscsvmd2
c     version      2.1  ISNAS     date 26-10-1993
c                                 Problem with folder corrected
c
c     version      2.0  ISNAS     date    20-10-1993
c                             updated and name changed from ismb20 to
c                             ismmccb2
c
c     version      1.0  ISNAS     date    30-08-93
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c     ISnas Momentum Matrix Construction Control on the Boundary
c     in 2 dimensions
c
c*********************************************************************
c
c                      KEYWORDS
c
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer maxdim, procdim
      parameter (maxdim = 3, procdim = 2)
      integer maxpts
      double precision maxtol
      parameter (maxpts = 16)
      parameter (maxtol = 1d-3)
      integer maxvar, maxspr
      parameter(maxvar=maxdim+1)
      parameter(maxspr=2**maxdim)
c
c     maxdim: highest physical space dimension for which ISNaS can work
      integer npointsnv, npointsv
      integer maxdx
      integer ndim, ncoefs, ndegfd, nconct, iindsc(20), maxb
c
      integer nvirtual
      integer ibipoint(1:2*ndim)
      integer ibindex(1:2,1:ndim-1,1:2*ndim)

      integer ibinfor(*)
      double precision bcgeo(1:ndim,1:ndim,*)
      double precision bndcon(1:maxb,*)
c
      integer ni, nj, nk, ntimlv
      double precision coor(1:npointsv, 1:ndim),
     +                 coefs (1:npointsv, 1:ncoefs),
     +                 matrix(1:npointsnv, 1:ndim, 1:nconct),
     +                 solut (1:npointsv, 1:ndegfd, 1:ntimlv),
     +                 rhside(1:npointsnv, 1:ndegfd),
     +                 jac(1:npointsv,1:2**ndim),
     +                 invjac(1:npointsv,1:2**ndim), dt

      double precision rho(1:npointsv), mu(1:npointsv),
     +                 veloc(1:npointsv,1:ndim),
     +                 cellvol(1:npointsv,1:ndim),
     +                 vsum(1:npointsv,1:ndim),
     +                 dvsum(1:npointsv,1:ndim,1:ndim),
     +                 cva(1:npointsv,1:ndim,1:ndim,1:2**ndim),
     +                 jcta(1:npointsv,1:ndim,1:ndim),
     +                 jctg(1:npointsv,1:(ndim*(ndim+1))/2),
     +                 chsym(1:npointsv,1:ndim,1:(ndim*(ndim+1))/2)
c
      integer npf(0:maxdim,0:maxvar,1:maxspr)
      integer rspv(1:maxdim,1:maxspr,
     +             0:maxdim,0:maxvar,1:maxspr)
      double precision cv(1:maxspr,
     +                    0:maxdim,0:maxvar,1:maxspr)
      integer mct(1:ndim,1:ndim,
     +            -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer mctinv(0:maxdim,1:ndim,1:nconct)
      integer istwo(1:maxdim,1:maxdim), vcellc(1:maxdim+1)
c
c
c     ni       i    number of non virtual cells in i-direction
c     nj       i    number of non virtual cells in j-direction
c     nk       i    number of non virtual cells in k-direction
c     ndim     i    integer indicating the dimension of the space in
c                   which the problem must be solved ( 2 or 3 ).
c     maxb     i    number of values per boundary point in bndcon
c     ndegfd   i    integer indicating the number of solution
c                   arrays that are stored in solut.
c                   so in fact ndegfd is equal to the number
c                   of unknowns in the field (u,v,p,t,k,....).
c
c*********************************************************************
c
c                      COMMON BLOCKS
c
c*********************************************************************
c
c                      LOCAL PARAMETERS
      integer marea(1:2,1:3), iparea(1:2,1:3),
     +        locdx(1:3), dcell(1:3), varea(1:2,1:3), carea(1:2,1:3),
     +        ntdir(1:maxdim), ntsig(1:maxdim), iwbarea(1:2,1:maxdim),
     +        acttyp, nijk(1:maxdim), area(1:2,1:maxdim),
     +        imlo, imhi, jmlo, jmhi, kmlo, kmhi,
     +        ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +        idcvlo, idcvhi, jdcvlo, jdcvhi, kdcvlo, kdcvhi,
     +        it1lo, it1hi, it2lo, it2hi, ipnt, mattype
      logical nzvsum(1:3), nzdvsum(1:3,1:3)
      integer ndir, tdir, nsig, iside, vartyp
c
      double precision vdummy(1:1)
c
      integer ptsused, cellpos(0:maxdim), ivbnd(1:2,1:maxdim)
      integer vtlist(1:maxdim,1:maxpts)
      integer codelist(1:maxpts)
      double precision ptlist(1:maxdim,1:maxpts)
      double precision lptlist(1:maxdim,1:maxpts)
c
c
      integer idir, jdir, pnttyp, kdir, auxdim
c
      integer lmi( 0:maxdim,0:maxvar,1:maxspr)
      integer lsi(1:maxdim,1:maxspr,
     +            0:maxdim,0:maxvar,1:maxspr)
      double precision lci(1:maxspr,
     +                     0:maxdim,0:maxvar,1:maxspr)
c
c     iwbarea   Area of points where coefficients
c               are needed, for the wesbeek discretization
c               this differs from iparea
c
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      PSEUDO CODE
c
c     LOOP over the sides to collect covariant vectors is all points
c          where b.c.'s are provided
c     END LOOP
c
c     LOOP over the sides of the domain
c        LOOP over integration cell flux direction idir
c           Set index bounds for the loop over boundary cells
c           Construct list of integration points in this integration cell
c
c           LOOP over integration points in a cell
c              Determine velocities and coefficients in current integration
c              point for the current group of integration cells
c              Determine geometrical quantities
c              IF (idir==ndir) THEN construct half cell molecules FI
c              IF (idir==tdir) THEN construct tangential cell molecules FI
c           END integration point LOOP
c        END flux direction LOOP
c   END domain side LOOP
c
c   LOOP over the sides of the domain
c      Fold velocities inwards
c   END LOOP
c
c=======================================================================
c
      call iseropen ( 'ismmccb2' )

      if (ndim.ne.procdim) then
         call iserrint(ndim,1)
         call iserrint(procdim,2)
         call iserrsub(10,2,0,0)
      end if


c     --- Get array bounds

      imlo = 1
      imhi = ni+1
      jmlo = 1
      jmhi = nj+1
      kmlo = 1
      kmhi = nk+1

c     --- Get lower and upper bounds on indices for arrays with virtual
c         cells

      ivlo = 1-nvirtual
      ivhi = ni+1+nvirtual
      jvlo = 1-nvirtual
      jvhi = nj+1+nvirtual
      kvlo = 1
      kvhi = 1

      ivbnd(1,1) = ivlo
      ivbnd(2,1) = ivhi
      ivbnd(1,2) = jvlo
      ivbnd(2,2) = jvhi
      ivbnd(1,3) = kvlo
      ivbnd(2,3) = kvhi

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- loop over the sides to fill bcgeo

      do 110 iside = 1, 4

c     --- Get information on a segment
c         (normal and tangential direction,
c         start and end point, type of boundary condition)

         call isbnbvi(ndim,ni,nj,nk,iside,ntdir,ntsig,carea,varea)

         ndir = ntdir(1)
         tdir = ntdir(2)

c        --- Vertex area

         area(1,1) = varea(1,1)
         area(2,1) = varea(2,1)
         area(1,2) = varea(1,2)
         area(2,2) = varea(2,2)

         area(1,3) = 1
         area(2,3) = 1

c        --- bcgeo area

         it1lo = ibindex(1,1,iside)
         it1hi = ibindex(2,1,iside)
         it2lo = 1
         it2hi = 1

c        --- covariant vectors in vertices; including virtual 
c            vertices

         pnttyp    = 1
         area(1,tdir) = varea(1,tdir)-nvirtual+1
         area(2,tdir) = varea(2,tdir)+nvirtual-1

         call isbndcp(ndim,ivlo,ivhi,jvlo,jvhi,kvlo,kvhi,
     +                it1lo,it1hi,it2lo,it2hi,
     +                ndim*ndim, area, pnttyp, ntdir,
     +                cva(1,1,1,pnttyp), bcgeo(1,1,ibipoint(iside)) )

c        --- covariant vectors in side centre; including virtual 
c            vertices

         pnttyp       = 1+tdir
         area(1,tdir) = carea(1,tdir)-nvirtual
         area(2,tdir) = carea(2,tdir)+nvirtual

         call isbndcp(ndim,ivlo,ivhi,jvlo,jvhi,kvlo,kvhi,
     +                it1lo,it1hi,it2lo,it2hi,
     +                ndim*ndim, area, pnttyp, ntdir,
     +                cva(1,1,1,pnttyp), bcgeo(1,1,ibipoint(iside)) )

110   continue

c     --- Loop over domain sides

      do 600 iside = 1, 4

c        --- Get information on boundary

         call isbnbvi(ndim,ni,nj,nk,iside,ntdir,ntsig,carea,varea)

         ndir = ntdir(1)
         tdir = ntdir(2)
         nsig = ntsig(1)

         it1lo = ibindex(1,1,iside)
         it1hi = ibindex(2,1,iside)

c        --- Loop over flux integration cell orientation

         do 500 idir = 1, 2

            if ( idir.eq.ndir ) then

c           --- normal cells
c               matrix builder area

               marea(1,tdir)=carea(1,tdir)
               marea(2,tdir)=carea(2,tdir)
               marea(1,ndir)=varea(1,ndir)
               marea(2,ndir)=varea(2,ndir)
               marea(1,3)    = 1
               marea(2,3)    = 1

            else if ( idir.eq.tdir ) then

c           --- tangential cells
c               matrix builder area

               marea(1,tdir)=varea(1,tdir)+1
               marea(2,tdir)=varea(2,tdir)-1
               marea(1,ndir)=carea(1,ndir)
               marea(2,ndir)=carea(2,ndir)
               marea(1,3)    = 1
               marea(2,3)    = 1

            end if

c           --- Construct list of integration points within the V^idir cell

            cellpos(0) = ndir
            cellpos(1) = 0
            cellpos(2) = 0
            cellpos(3) = 0

            cellpos(ndir) = nsig

            call ismpts(ndim,idir,idir,maxpts,maxtol,cellpos,
     +                  ptsused,vtlist,ptlist,lptlist,codelist,iindsc)

c           --- Loop over integration points

            do 400 ipnt=1,ptsused

               pnttyp = codelist(ipnt)
               locdx(1) = nint(2*lptlist(1,ipnt))
               locdx(2) = nint(2*lptlist(2,ipnt))
               locdx(3) = nint(2*lptlist(3,ipnt))

               dcell(1)  = vtlist(1,ipnt)
               dcell(2)  = vtlist(2,ipnt)
               dcell(3)  = vtlist(3,ipnt)

               do 130 jdir=1,maxdim

                  iparea(1,jdir)  = marea(1,jdir)+dcell(jdir)
                  iparea(2,jdir)  = marea(2,jdir)+dcell(jdir)

                  if (idir.eq.ndir .and. iindsc(3).eq.2) then

c                        *** Only Dirichlet conditions allowed,
c                            so iparea is not used for ndir
c                            Modification needed due to outlying points


                  end if

130            continue

            if (iindsc(3).eq.2) then
c           --- determine coefs area for wesbeek
               do 140 kdir=1,ndim
                  iwbarea(1,kdir) = max(2-nvirtual-
     +               mod((pnttyp-1)/(2**(kdir-1)),2),
     +               marea(1,kdir)+dcell(kdir)-4)
                  iwbarea(2,kdir) = min(nijk(kdir)+nvirtual-
     +               mod((pnttyp-1)/(2**(kdir-1)),2),
     +               marea(2,kdir)+dcell(kdir)+4)
140            continue
            else
               do 150 kdir=1,ndim
                  iwbarea(1,kdir) = marea(1,kdir)+dcell(kdir)
                  iwbarea(2,kdir) = marea(2,kdir)+dcell(kdir)
150            continue
            end if

            if (ndim.eq.2) then
               iwbarea(1,3) = 1
               iwbarea(2,3) = 1
            end if

c           --- Interpolate velocities to points of type pnttyp

            acttyp = 0

            do 160 jdir = 1, ndim
               call isclcndf(solut(1,jdir,1), veloc(1,jdir),
     +                       ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +                       npf, rspv, cv, acttyp, jdir, pnttyp,
     +                       iparea)
160         continue


c           --- Interpolate coefficients to points of type pnttyp

            vartyp = ndim+1

            call isclcndf(coefs(1,1),rho,
     +                    ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +                    npf, rspv, cv, acttyp, vartyp,
     +                    pnttyp, iwbarea)

            call isclcndf(coefs(1,2),mu,
     +                    ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +                    npf, rspv, cv, acttyp, vartyp,
     +                    pnttyp, iwbarea)

c              --- Obtain the necessary geometric quantities
c                  The contravariant basis vectors
c                  (used by the routines that calculate the
c                  metric tensor and the christoffel symbols)

               call isjcta(ndim,
     +                     ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                     cva(1,1,1,pnttyp),jcta,iparea)

c              --- The metric tensor times sqrt(g)

               call isjctg(ndim,
     +                     ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                     ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                     jcta,invjac(1,pnttyp),
     +                     jctg,iparea,istwo)

c              --- The christoffel symbols of the second kind

               call ischsym(ndim,pnttyp,
     +                      ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                      ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                      ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                      coor,jcta,invjac(1,pnttyp),
     +                      chsym,iparea,npf,rspv,cv,istwo)
c
               if (idir.eq.ndir) then

c              --- treat half cells

                  area(1,1) = ivlo
                  area(2,1) = ivhi
                  area(1,2) = jvlo
                  area(2,2) = jvhi
                  area(1,3) = kvlo
                  area(2,3) = kvhi

c                 --- time derivative for
c                     half cells where V^n is not prescribed
c
*                  call ismmtbh2 ( it1lo, it1hi,
*     +                            ni, nj, nvirtual, rho, veloc,
*     +                            matrix, rhside, marea, dt, locdx,
*     +                            idir, ibinfor(ibipoint(iside)),
*     +                            ntdir )

c                 --- Construct half cells for neumann b.c.
c                     part 1: convection terms

                  call issetcon ( vsum, 2*npointsv, 0d0 )
                  do 200 jdir =1,ndim
                     nzvsum(jdir) = .false.
200               continue
                  if ( iindsc(1).eq.1 ) then

c                 --- Standard ISNaS discretization

                     call ismmc2 ( ni, nj, nvirtual, rho,
     +                             cva(1,1,1,pnttyp), chsym,
     +                             invjac(1,pnttyp),
     +                             veloc, cellvol(1,idir),
     +                             cva(1,1,1,vcellc(idir)),
     +                             vsum, idir, iparea, locdx,
     +                             dcell, istwo, iindsc(2),
     +                             iindsc(8) )
                     call ismmcb2 ( ni, nj, nvirtual, it1lo, it1hi,
     +                              veloc, vsum, idir, rho,
     +                              invjac(1,pnttyp),
     +                              chsym, marea, locdx, dcell, istwo,
     +                              ibinfor(ibipoint(iside)),
     +                              ntdir, ntsig, iindsc(2),
     +                              iindsc(8) )

                     do 210 jdir = 1, 2

                        acttyp = 0
                        call isdistrb ( ndim, nijk, nvirtual, ndim,
     +                                  ndim, maxdx, npointsnv,
     +                                  matrix, vsum(1,jdir), mct,
     +                                  dcell, npf, rspv, cv, idir,
     +                                  acttyp, jdir, pnttyp, marea )


c                       --- Compute contribution of convective terms to
c                           right-hand side for Newton linearisation.
c                           Due to the Newton linearization this contribution is
c                           equal to 0.5 * update_matrix * velocity
c
c                           The right-hand side vector is situated in the
c                           centres of the integration cells
c                           The update matrix is positioned in the
c                           integration point. So the multiplication must
c                           be caried out in the integration points
c                           As a consequence both the velocity vactor and
c                           the matrix must be shifted with dcell


                        if (iindsc(8).eq.1 ) then

c                       --- iindsc(8) = 1, Newton lin.

                           call isdaxpy2 ( ndim, ni, nj, nk, nvirtual,
     +                                    dcell(1), dcell(2),
     +                                    dcell(3), dcell(1),
     +                                    dcell(2), dcell(3),
     +                                    rhside(1,idir), 0.5d0,
     +                                    vsum(1,jdir), veloc(1,jdir),
     +                                    marea )

                         end if

210                   continue

                  end if

c                 --- Construct half cells for neumann b.c.
c                     part 2: viscosity terms

                  do 230 jdir =1,ndim
                     nzvsum(jdir) = .false.
                     do 220 kdir=1,ndim
                        nzdvsum(kdir,jdir) = .false.
220                  continue
230               continue

                  call issetcon ( vsum, 2*npointsv, 0d0 )
                  call issetcon ( dvsum, 4*npointsv, 0d0 )

c                  --- Build stencil

                  if (iindsc(3).ne.2)
     +               call ismms2 ( ni, nj, nvirtual, mu, jctg, chsym,
     +                             vsum, dvsum, idir, iparea, locdx,
     +                             nzdvsum, istwo, iindsc(3) )

                  if (iindsc(3).eq.2)
     +               call ismmwbs2(ni, nj,
     +                             ndim, ivlo, ivhi, jvlo, jvhi,
     +                             idir, .true. , nsig,mu,
     +                             cellvol(1,idir), vsum,
     +                             cva, locdx, dcell, iparea, npf,
     +                             rspv, cv)

                  call ismmsb2( ni, nj, nvirtual,
     +                         it1lo, it1hi,
     +                         vsum,dvsum,idir,mu,nzvsum,nzdvsum,
     +                         jac(1,pnttyp), invjac(1,pnttyp), jctg,
     +                         chsym, marea,locdx,dcell,istwo,
     +                         bcgeo(1,1,ibipoint(iside)),
     +                         ibinfor(ibipoint(iside)),
     +                         bndcon(1,ibipoint(iside)), maxb,
     +                         ntdir,ntsig,rhside,iindsc(3))

c                 --- Distribute molecule

                  do 270 kdir =1,ndim

                     acttyp = 0
                     call isdstndf(ndim, nijk, nvirtual, nconct,
     +                             ndim, ndim, maxdx, npointsnv,
     +                             npointsv, matrix, invjac,
     +                             vsum(1, kdir), mct,
     +                             (iindsc(3).ne.2),
     +                             dcell, vcellc, npf, rspv, cv,
     +                             idir, acttyp, kdir, pnttyp, marea )

                     do 260 jdir=1,ndim

                        if (nzdvsum(kdir,jdir))
     +                     call isdstndf(ndim, nijk, nvirtual, nconct,
     +                                   ndim, ndim, maxdx, npointsnv,
     +                                   npointsv, matrix, invjac,
     +                                   dvsum(1, kdir, jdir), mct,
     +                                   (iindsc(3).ne.2),
     +                                   dcell, vcellc, npf, rspv, cv,
     +                                   idir, jdir,
     +                                   kdir, pnttyp, marea )

260                  continue
270               continue

               else if ( idir.eq.tdir ) then

c              --- tangential cells

*                  if ( locdx(1).eq.0 .and. locdx(2).eq.0 ) then
*
*                      call ismmt2 ( nconct, ni, nj, nvirtual, rho,
*     +                              veloc, matrix, rhside, marea,
*     +                              dt, idir )
*
*                  end if
                  if ( iindsc(1).eq.1 ) then

                     do 280 jdir =1,ndim
                        call isfill(ivlo, ivhi,jvlo,jvhi,kvlo,kvhi,
     +                     1,ivbnd,vsum(1,jdir),0d0)
280                  continue

                     call ismmc2 ( ni, nj, nvirtual,
     +                             rho, cva(1,1,1,pnttyp), chsym,
     +                             invjac(1,pnttyp),
     +                             veloc,cellvol(1,idir),
     +                             cva(1,1,1,vcellc(idir)),
     +                             vsum, idir, iparea, locdx,
     +                             dcell, istwo, iindsc(2),
     +                             iindsc(8) )

                     call ismmcb2 ( ni, nj, nvirtual, it1lo, it1hi,
     +                              veloc, vsum, idir, rho,
     +                              invjac(1,pnttyp),
     +                              chsym, marea, locdx, dcell, istwo,
     +                              ibinfor(ibipoint(iside)),
     +                              ntdir, ntsig, iindsc(2),
     +                              iindsc(8) )

                     do 290 jdir =1,ndim

                        acttyp = 0
                        call isdstndf(ndim, nijk, nvirtual, nconct,
     +                                ndim, ndim, maxdx, npointsnv,
     +                                npointsv, matrix, vdummy,
     +                                vsum(1, jdir), mct, .false.,
     +                                dcell, vcellc, npf, rspv, cv,
     +                                idir, acttyp,
     +                                jdir, pnttyp, marea )


                        idcvlo = ivlo - dcell(1)
                        idcvhi = ivhi - dcell(1)
                        jdcvlo = jvlo - dcell(2)
                        jdcvhi = jvhi - dcell(2)
                        kdcvlo = kvlo - dcell(3)
                        kdcvhi = kvhi - dcell(3)


                        if (iindsc(8).eq.1 ) then

c                       --- iindsc(8) = 1, Newton lin.

                           call isvsvvu(ndim,.true.,rhside(1, idir),
     +                                  5d-1,vsum(1, jdir),
     +                                  veloc(1, jdir), imlo, imhi,
     +                                  jmlo, jmhi, kmlo, kmhi,
     +                                  idcvlo, idcvhi, jdcvlo,
     +                                  jdcvhi, kdcvlo, kdcvhi,
     +                                  idcvlo, idcvhi, jdcvlo,
     +                                  jdcvhi, kdcvlo, kdcvhi,
     +                                  marea)

                        end if

290                  continue

                  end if

                  do 310 kdir =1,ndim
                     do 300 jdir=1,ndim
                        nzdvsum(jdir,kdir) = .false.
300                  continue
310               continue

                  call isfill(ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +                        ndim, ivbnd, vsum, 0d0)
                  call isfill(ivlo, ivhi, jvlo, jvhi, kvlo, kvhi,
     +                        ndim*ndim, ivbnd, dvsum, 0d0 )

                  if (iindsc(3).ne.2)
     +               call ismms2 ( ni, nj, nvirtual, mu, jctg, chsym,
     +                             vsum, dvsum, idir, iparea, locdx,
     +                             nzdvsum, istwo, iindsc(3) )

                  if (iindsc(3).eq.2)
     +               call ismmwbs2(ni, nj,
     +                            ndim, ivlo, ivhi, jvlo, jvhi,
     +                            idir, .true. , nsig,
     +                            mu, cellvol(1,idir), vsum,
     +                            cva, locdx, dcell, iparea, npf,
     +                            rspv, cv)

                  call ismmsb2( ni, nj, nvirtual,
     +                         it1lo, it1hi,
     +                         vsum,dvsum,idir,mu,nzvsum,nzdvsum,
     +                         jac(1,pnttyp), invjac(1,pnttyp),
     +                         jctg, chsym,
     +                         marea,locdx,dcell,istwo,
     +                         bcgeo(1,1,ibipoint(iside)),
     +                         ibinfor(ibipoint(iside)),
     +                         bndcon(1,ibipoint(iside)), maxb,
     +                         ntdir,ntsig,rhside,iindsc(3))
c
                  do 340 jdir =1,ndim
c
                     acttyp = 0
                     call isdstndf(ndim, nijk, nvirtual, nconct,
     +                             ndim, ndim, maxdx, npointsnv,
     +                             npointsv, matrix, invjac,
     +                             vsum(1, jdir), mct,(iindsc(3).ne.2),
     +                             dcell, vcellc, npf, rspv, cv, idir,
     +                             acttyp, jdir, pnttyp, marea )
c
                     do 330 kdir=1,ndim
c
                        if (nzdvsum(jdir,kdir)) then
                           if (nconct.eq.13 .and. idir.eq.tdir .and.
     +                        jdir.eq.ndir .and. locdx(tdir).eq.0
     +                        .and. locdx(ndir).eq.nsig .and.
     +                        kdir.eq.ndir) then

c                          --- One sided difference

                              lmi(kdir,jdir,pnttyp) = 4

                              lci(1,kdir,jdir,pnttyp) = -5d-1
                              lci(2,kdir,jdir,pnttyp) = -5d-1
                              lci(3,kdir,jdir,pnttyp) =  5d-1
                              lci(4,kdir,jdir,pnttyp) =  5d-1
     +
     +
                              do 320 auxdim=1,maxdim
                                 lsi(auxdim,1,kdir,jdir,pnttyp) = 0
                                 lsi(auxdim,2,kdir,jdir,pnttyp) = 0
                                 lsi(auxdim,3,kdir,jdir,pnttyp) = 0
                                 lsi(auxdim,4,kdir,jdir,pnttyp) = 0
320                           continue

                              auxdim = tdir
                              lsi(auxdim,1,kdir,jdir,pnttyp) = -1
                              lsi(auxdim,2,kdir,jdir,pnttyp) =  0
                              lsi(auxdim,3,kdir,jdir,pnttyp) = -1
                              lsi(auxdim,4,kdir,jdir,pnttyp) =  0

                              auxdim = ndir
                              lsi(auxdim,1,kdir,jdir,pnttyp) =
     +                                             -(1+nsig)/2
                              lsi(auxdim,2,kdir,jdir,pnttyp) =
     +                                             -(1+nsig)/2
                              lsi(auxdim,3,kdir,jdir,pnttyp) =
     +                                              (1-nsig)/2
                              lsi(auxdim,4,kdir,jdir,pnttyp) =
     +                                              (1-nsig)/2


                              call isdstndf(ndim, nijk, nvirtual,
     +                                      nconct,
     +                                      ndim, ndim, maxdx,
     +                                      npointsnv,
     +                                      npointsv, matrix, invjac,
     +                                      dvsum(1, jdir, kdir), mct,
     +                                      (iindsc(3).ne.2), dcell,
     +                                      vcellc, lmi, lsi, lci, idir,
     +                                      kdir, jdir, pnttyp, marea )
                           else
                              call isdstndf(ndim, nijk, nvirtual,
     +                                      nconct,
     +                                      ndim, ndim, maxdx,
     +                                      npointsnv,
     +                                      npointsv, matrix, invjac,
     +                                      dvsum(1, jdir, kdir), mct,
     +                                      (iindsc(3).ne.2),
     +                                      dcell, vcellc,
     +                                      npf, rspv, cv,
     +                                      idir,
     +                                      kdir, jdir, pnttyp, marea )
                           end if
                        end if
c
330                  continue
340             continue

               end if

400         continue

500      continue

600   continue

c$$$               call isprimt1 ( 'momentum matrix', matrix, ni, nj,
c$$$     +                         nconct, 1 )
      mattype = 1
      call ismcbcf ( ndim, ni, nj, nk, nvirtual, ndegfd, ntimlv,
     +               nconct, maxb, mct, mctinv, ibindex, ibipoint,
     +               matrix, solut, rhside, invjac, ibinfor,
     +               bndcon, bcgeo, mattype )
c$$$               call isprimt1 ( 'momentum matrix', matrix, ni, nj,
c$$$     +                         nconct, 1 )
c$$$      stop
      call iserclos ( 'ismmccb2' )
      end
