      subroutine isbegin ( nbuffr, t0, tact, tend, eps, istation )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 3.2    date 10-02-1999 Kees K: renamed is_start in isbegin
c        version 3.1    date 08-02-1999 Kees K: array eddy and geom
c                                       are now created in subroutine
c                                       isrdint, necessary for postprocessing
c                                       in parallel case
c        version 3.0    date 01-10-1998 Jason: renovated for SPMD
c        version 2.4    date 08-09-1998 Guus: new call to istimi01
c        version 2.3    date 14-05-1998 Duncan: extension for saving
c                                               coefficients dens upwind
c                                               new call to isalloc
c                                               new call to isacth
c        version 2.2    date 04-06-1988 Charles isigeom initialized
c        version 2.1    date 29-05-1998 Charles change of arguments in isalloc
c        version 2.0    date 18-02-1998 Charles change in the call of isalloc
c                                               and isalloc2 removed
c        version 1.2    date 13-10-1997 Charles neq adapted to uncoupled
c        version 1.1    date 12-09-1997 Charles icoupled and ndim in istimi01
c        version 1.0    date 05-06-1997
c
c
c   copyright (c) 1996-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNaS Host start subroutine
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nbuffr, istation
      double precision t0, tact, tend

c     eps       i    Accuracy with respect to the time parameter tout
c     istation  o    Choice whether a stationary solution is needed
c     nbuffr    i    Length of buffer array ibuffr
c     t0        o    Initial time
c     tact      o    As t, the actual time, but not in common block
c     tend      o    End time
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisglob'
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinput, iprinput,
     +        neq, nfrac, i, methodtim,
     +        restartw, cartesian, ndegfd, ndim, nvectr,
     +        iblock, nmachines, icoupled, nsubtst,
     +        inodesv
      double precision thetarr(10),  toutin, touten, toutst,
     +                 rinit, rend, rstep, eps, dt

c     cartesian Cartesian=1, write cartesian values in the vertices to
c               the restart file
c               cartesian=0, dump contravariant solution vector
c     dt        Time step
c     i         General loop variable
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     inodesv   temporary storage of inodenr
c     ipiinput  Starting address of array iinput in ibuffr
c     iprinput  Starting address of array rinput in buffer
c     maxblocki Maximum number of blocks
c     maxneq    Maximum number of equations allowed
c     methodtim Type of time integration
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     neq       Number of equations
c     nfrac     Number of fractional time steps
c     nmachines Number of nodes that are used
c     nsubtst   Number of sub-timesteps
c     nvectr    Maximum number of arrays that may be stored in IBUFFR
c     rend      End time for writing to restart file
c     restartw  If restartw > 0, write restart file
c     rinit     Start time for writing solutions to the restart file
c     rstep     Time step for writing to restart file
c     thetarr   Array of thetas for different theta methods
c     touten    End time for output of results
c     toutin    Initial time for output of results
c     toutst    Time step for output of results
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISALLOC   create space for some arrays
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILG    Fill geometrical quantities
c     ISINCND   initial conditions
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINICLK  initialize the clock
c     ISINIFIL  Open files
c     ISINITCB  Initialize ISNaS machine-independent common blocks
c     ISINITMD  Set machine-dependent quantities
c     ISNDT02   Get dt and test on stationary solution
c     ISRDINT   read user input
c     ISREMFIL  Remove file
c     ISSTARMM  Start memory manager
c     ISTIMI01  Get number of equations and time integration values
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     initialize all pointers to zero
c     initialize common blocks, memory manager, clock, files
c     Read intermediate file
c     Create space for arrays eddy, infbnd, bndcon, matrix,
c        gmat, rhside, geom and iadmin
c     Fill initial conditions
c     Send start information from host to node
c     Actual time integration
c     Close application and files
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize all pointers to zero

      isitopol = 0

      do iblock = 1, maxblocki

         isiinput( iblock ) = 0
         isrinput( iblock ) = 0
         iscoor  ( iblock ) = 0
         iscoefs ( iblock ) = 0
         issolut ( iblock ) = 0
         iseddy  ( iblock ) = 0
         isinfbnd( iblock ) = 0
         isgmat  ( iblock ) = 0
         isrhside( iblock ) = 0
         isgeom  ( iblock ) = 0
         isigeom ( iblock ) = 0
         isdensup( iblock ) = 0
         isaverstr( iblock ) = 0

      end do
      do iblock = 1, maxblocki*maxneq
         isbndcon( iblock ) = 0
      end do

      do iblock = 1, maxblocki*maxneq*2
         ismatrix( iblock ) = 0
      end do

c     --- Initialize machine independent common blocks

      call isinitcb

      call iseropen ( 'isbegin' )

c     --- Set machine dependent parameters in common blocks

      call isinitmd
      if ( nerror.ne.0 ) go to 1000

c     --- Open ISNaS files

      if ( inodenr.eq.ISHOST ) then

c     --- Open input and output files

         call isinifil(6)
         call isinifil(4)

c        --- Remove file isnascomp.out

         call isremfil ( 74 )

         if ( nerror.ne.0 ) go to 1000
      end if

c     --- Open scratch file (all processes)

      call isinifil(3)
      if ( nerror.ne.0 ) go to 1000

c     --- Set timer equal to begin time

      call isiniclk
      if ( nerror.ne.0 ) go to 1000

c     --- Start memory manager

      nvectr = 1000
      call isstarmm ( nbuffr, nvectr )

c     --- Read intermediate file
c         At this moment both the solution arays as well as the coordinate
c         arrays are filled for all blocks on all computers
c         This is superfluous, only the blcoks per computer and possibly
c         neighbouring blocks should be filled to save space
c         Only on the host the complete arrays must be available
c         Perhaps for that reason saving of space is impossible?

      call isrdint ( maxblocki, maxneq, nblocks , isiinput, isrinput,
     +               iscoor, issolut, iscoefs, isitopol, iseddy, isgeom,
     +               iadmin, nmachines, iscotmp, indorien )

      if ( nerror.ne.0 ) go to 1000

      call isactive(isiinput(1))
      call isactive(isrinput(1))
      iprinput = isdpgetp(isrinput(1))
      ipiinput = isingetp(isiinput(1))
      ndegfd   = ibuffr(ipiinput + 55 - 1 )
      ndim     = ibuffr(ipiinput + 57 - 1 )
      icoupled = ibuffr(ipiinput + 76 - 1 )
      if ( icoupled.eq.0 ) then
         neq = ndegfd - ndim + 1
      else
         neq = ndegfd
      end if

c     --- Create space for arrays eddy, infbnd, bndcon, matrix,
c         gmat, rhside, geom and iadmin

      if ( nerror.ne.0 ) go to 1000
      call isalloc ( nblocks, neq, isiinput, isrinput,
     +               isaverstr, isinfbnd, isbndcon,
     +               ismatrix, isgmat, isrhside,
     +               iadmin, isigeom, isdensup )

      if ( nerror.ne.0 ) go to 1000

c     --- Fill initial conditions on host

c         --- Hack:

      inodesv = inodenr
      inodenr = 0
      call isincnd ( isiinput, isrinput, isitopol, iadmin,
     +               iscoor, issolut, iseddy, isgeom)
      inodenr = inodesv
      if ( nerror.ne.0 ) go to 1000

c     --- Fill geometrical quantities

      call isfilg ( nblocks, isiinput, isrinput, iscoor,
     +              isgeom, isigeom, iadmin )

      if ( nerror.ne.0 ) go to 1000

c     --- Get time integration parameters
c         initialize thetarr

      do i = 1, 10
         thetarr(i)   = 0d0
      end do

c     --- get number of equations and time parameters

      call istimi01( ibuffr(ipiinput), buffer(iprinput), neq, t0,
     +               tend, dt, thetarr, toutin, touten, toutst,
     +               rinit, rend, rstep, methodtim, nfrac, restartw,
     +               cartesian, nsubtst )
      if ( nerror.ne.0 ) go to 1000

      t     = t0
      tact  = t0

c     --- get the values dt and istation

      call isndt02 ( ibuffr(ipiinput), buffer(iprinput), t, dt,
     +               toutst, rstep, istation)

      eps = 1d-2 * dt

1000  call iserclos('isbegin')

      end
