      subroutine isbldmat ( iequa, nblocks, isiinput, isrinput, iscoor,
     +                      issolut, iscoefs, iseddy, isinfbnd,
     +                      isbndcon, ismatrix, isgmat, isrhside,
     +                      isgeom, isigeom, iadmin, isitopol,
     +                      isdensup )
c ======================================================================
c
c     programmer    Erik Brakkee
c     version 3.1   date  06-12-1999 Duncan: added itopol to parameterlist
c                                            issysp
c     version 3.0   date  07-10-1998 Guus: renamed to isbldmat because of F90
c                                    compiler of HP
c     version 2.18  date  14-05-1998 Duncan: extension for saving
c                                            coefficients dens upwind
c                                            new call to issysp
c     version 2.17  date  04-06-1998 Charles: isigeom added
c     version 2.16  date  18-05-1998 Charles Bilinear discretization
c     version 2.15  date  20-04-1998 Guus: correction itypsten for mulit-phase
c                                    gas flow
c     version 2.14  date  15-12-1997 Charles: extra arguments (gmat) in issyst
c     version 2.13  date  05-11-1997 Guus: new call to isfilb
c     version 2.12  date  30-10-1997 Guus: removal of firstcall
c     version 2.11  date  13-10-1997 Charles: extra parameter isitopol
c     version 2.10  date  15-09-1997 Guus: ipeddy might be 1
c     version 2.9   date  10-09-1997 Charles: extra parameters icoupled, mshtyp
c     version 2.8   date  10-08-1996 Guus: extension because of compressible
c                                          Navier-Stokes
c     version 2.7   date  01-05-1996 Marcel: parameter list issyst and
c                                            islsds01 changed
c     version 2.6   date  23-04-1996 Guus: New timing subroutines
c     version 2.5   date  10-04-1996 Kees K call isfilb changed
c     version 2.4   date  27-03-1996 Kees K bndcon per equation
c     version 2.3   date  26-02-1996 Hester: parameter list issysp
c                                             changed
c     version 2.2   date  12-02-1996 Kees K.: parameter list islsds01
c                                           changed
c     version 2.1   date  15-12-1995 Marcel: parameter list isfilb
c                                          changed
c     version 2.0   date  18-07-1995 Guus: Replace isbuil01 by body
c     version 1.3   date  02-03-1995 Kees K: skip removed
c     version 1.2   date  22-02-1995 Guus: Extended length of iindsc
c     version 1.1   date  31-01-1995 Erik: name change issds01
c                                          to isbuil01 + new dimensions
c                                          for ismatrix
c     version 1.0   date  05-01-1995
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Build subdomain matrices
c
c **********************************************************************
c
c                       KEYWORDS
c
c     node_domain_decomposition
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, nblocks, isiinput(*), isrinput(*), iscoor(*),
     +        issolut(*), iscoefs(*), iseddy(*), isinfbnd(*),
     +        isbndcon(nblocks,*), ismatrix(nblocks, *), isgmat(*),
     +        isrhside(*), isgeom(*), isigeom(*), iadmin(*),
     +        isitopol, isdensup(*)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isigeom   i    Array of sequence numbers (nblocks) of array igeom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipcoefs, ipcoor, ipeddy, ipiincom, ipgeom,
     +        ipigeom, ipsolut, ipbndcon, ipinfbnd, ipgmat, ipiinput,
     +        ipinfmg, ipinfmm , ipinfmt, ipiinsol, ipmatrix,
     +        iprhside, iprinput, ipwork, isinfmm, isinfmg ,
     +        isinfmt, iswork, linfmm, linfmg, linfmt, itypsten,
     +        lwork, ipiindsc, linsol, rspv(1:3,1:8,0:3,0:4,1:8),
     +        ni,  nj, nk, ndim, nvirtual, ndegfd, ntimlv ,
     +        nturb, maxdim, maxdx, npf(0:3,0:4,1:8),
     +        iseqpres, icoupled, mshtyp, ipdensup, ipitopol
      double precision cv(1:8,0:3,0:4,1:8)
      character*10 matname
      character*39 text

c     cv        array cv(m,action,vartyp,pnttyp)
c               This array of size cv(1:8,0:3,0:4,1:8)
c               defines the weights for the interpolation or
c               finite difference formula.
c               The first index refers to the sequence number of the point,
c               where the sequence number runs from
c               1 to npf(action,vartyp,pnttype).
c               The second index refers to action, the third one to
c               vartyp and the last one to pnttype.
c               Consult the Programmers Guide for a further explanation
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ipbndcon  Starting address of array bndcon in buffer
c     ipcoefs   Starting address of array coefs in buffer
c     ipcoor    Starting address of array coor in buffer
c     ipdensup  Starting address of array densup in buffr
c     ipeddy    Starting address of array eddy in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipgmat    Starting address of array gmat in buffer
c     ipigeom   Starting address of array igeom in ibuffr
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipinfbnd  Starting address of array infbnd in ibuffr
c     ipinfmg   Starting address of array infmg in ibuffr
c     ipinfmm   Starting address of array infmm in ibuffr
c     ipinfmt   Starting address of array infmt in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipmatrix  Starting address of array matrix in buffer
c     iprhside  Starting address of array rhside in buffer
c     iprinput  Starting address of array rinput in buffer
c     ipsolut   Starting address of array solut in buffer
c     ipwork    Starting address of array work in buffer
c     iseqpres  Sequence number of pressure equation
c     isinfmg   Sequence number of array infmg
c     isinfmm   Sequence number of array infmm
c     isinfmt   Sequence number of array infmt
c     iswork    Sequence number of array work
c     itypsten  Indicates the type of stencil
c               Possible values
c               R^2
c               1:  13-point stencil
c               2:  21-point stencil
c               3:  17-point stencil
c               R^3
c               1:  51-point stencil
c               2:  59-point stencil
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     linsol    Length of array iinsol
c     lwork     Length of array work
c     matname   Name of matrix to be build or solved
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     mshtyp    Type of grid arrangement
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npf       npf(action,vartyp,pnttype)
c               This array of size npf(0:3,0:4,1:8) defines the number
c               of points used in a interpolation or finite difference
c               formula, depending on places in grid cell and variable
c               types. The first index refers to action, the second one to
c               vartyp and the last one to pnttype.
c     ntimlv    Number of time levels
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     rspv      rspv(direction,m,action,vartyp,pnttype)
c               This array of size rspv(1:3,1:8,0:3,0:4,1:8)
c               defines a list of relative shifts from point cell index
c               to variable cell index.
c               The first index refers to the co-ordinate direction of
c               the shift, the second gives the sequence number of
c               the point, where the sequence number runs from
c               1 to npf(action,vartyp,pnttype).
c               The third index refers to action, the fourth one to
c               vartyp and the last one to pnttype.
c               With a relative shift we mean the shift in global grid
c               cell numbering of the points where the interpolated
c               values must be computed with respect to the point
c               where the interpolation must be carried out.
c               It is important to notice that the points with
c               pnttype 1 to 4 in R^2 and with pnttype 1 to 8 in R^8
c               all have the same global (i,j,k) index.
c               See the Programmers Guide for details
c     smsg      The array smsg (sub message) is a one dimensional
c     text      String in which a text is stored
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISASKCLK  Print CPU time used
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILB    Fill boundary values
c     ISFILM    Fill integer information about the matrices
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISLSDS01  Compute lengths of temporary arrays
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSETCLK  Set time in common cistime equal to actual CPU time
c     ISSYSM    Build matrix and right-hand side for momentum equation
c     ISSYSP    Build matrix and right-hand side for pressure equation
c     ISSYST    Build matrix and right-hand side for transport equation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c         for iblock = 1 (1) nblocks do
c            if block(iblock) on this node  then
c              build equations for this block
c         end do
c      end if
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isbldmat')

c     --- Initialize tables containing interpolation and finite
c         difference formulas

      call isactive( isiinput(1) )
      ipiinput = isingetp(isiinput(1))
      ndim     = ibuffr(ipiinput-1+57)
      mshtyp   = ibuffr(ipiinput-1+75)
      icoupled = ibuffr(ipiinput-1+76)

      call ispndf(npf,rspv,cv,ndim)

c     --- Choice between coupled or uncoupled resolution

      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim+1
      end if

c     --- build matrices

      do iblock = 1, nblocks

         iblockrf = iblock
         if ( iadmin(iblock).eq.inodenr ) then

            if ( itime.eq.1 ) then

c           --- Start timer for matrix construction

               call issetclk ( 1 )

            end if

c           --- compute lengths of local arrays

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))

            call islsds01(ibuffr(ipiinput), linfmm, linfmg, linfmt,
     +                    lwork, ni, nj, nk, ndim , nvirtual, ndegfd,
     +                    ntimlv, nturb, maxdx, maxdim )

c           --- create local arrays

            isinfmm   = 0
            isinfmg   = 0
            isinfmt   = 0
            iswork    = 0

            call isicreat(isinfmm, linfmm, 'infmm' )
            call isicreat(isinfmg, linfmg, 'infmg' )
            call isicreat(isinfmt, linfmt, 'infmt' )
            call isdcreat(iswork, lwork, 'work' )

c           --- get pointers

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(iscoor(iblock))
            call isactive(issolut(iblock))
            call isactive(iscoefs(iblock))
            if ( iseddy(iblock).gt.0 ) call isactive(iseddy(iblock))

            call isactive(isinfbnd(iblock))
            call isactive(isbndcon(iblock,iequa))
            call isactive(ismatrix(iblock,iequa))
            call isactive(isgmat(iblock))
            call isactive(isrhside(iblock))
            call isactive(isgeom(iblock))
            call isactive(isitopol)
            if ( isigeom(iblock).gt.0 ) call isactive(isigeom(iblock))
            call isactive(isdensup(iblock))

            ipiinput = isingetp(isiinput(iblock))
            ipitopol = isingetp(isitopol)
            iprinput = isdpgetp(isrinput(iblock))
            ipcoor   = isdpgetp(iscoor(iblock)  )
            ipsolut  = isdpgetp(issolut(iblock) )
            ipcoefs  = isdpgetp(iscoefs(iblock) )
            if ( iseddy(iblock).gt.0 ) then
               ipeddy = isdpgetp(iseddy(iblock)  )
            else
               ipeddy = 1
            end if

            ipinfbnd = isingetp(isinfbnd(iblock))
            if ( iequa.eq.iseqpres ) then
               ipbndcon = 1
            else
               ipbndcon = isdpgetp(isbndcon(iblock,iequa))
            end if
            ipmatrix = isdpgetp(ismatrix(iblock,iequa))
            ipgmat   = isdpgetp(isgmat(iblock)  )
            iprhside = isdpgetp(isrhside(iblock))
            ipgeom   = isdpgetp(isgeom(iblock)  )

            if ( isigeom(iblock).gt.0 ) then
               ipigeom  = isingetp(isigeom(iblock)  )
            else
               ipigeom  = 1
            end if

            ipdensup = isdpgetp(isdensup(iblock))

c           --- get pointers local arrays

            ipinfmm  = isingetp( isinfmm )
            ipinfmg  = isingetp( isinfmg )
            ipinfmt  = isingetp( isinfmt )

            ipwork   = isdpgetp( iswork )

c           --- fill integer information about the matrices

            ipiindsc = ibuffr(ipiinput-1+6)
            ipiincom = ibuffr(ipiinput-1+9)
            if ( ibuffr(ipiinput-1+ipiindsc+1).eq.2 .or.
     +           ibuffr(ipiinput-1+ipiindsc+1).eq.4 ) then
               itypsten = 2
            else if ( ibuffr(ipiinput-1+ipiincom).ge.1 .and.
     +                ibuffr(ipiinput-1+ipiindsc-1+3).eq.0 ) then
               itypsten = 3
            else
               itypsten = 1
            end if

            call isfilm ( ibuffr(ipinfmm), ibuffr(ipinfmg),
     +                    ibuffr(ipinfmt), ibuffr(ipiinput), itypsten )

c           --- fill boundary values

            if ( iequa.ne.iseqpres )
     +         call isfilb ( iequa, ibuffr(ipiinput), buffer(iprinput),
     +                       ibuffr(ipinfbnd),buffer(ipbndcon),
     +                       buffer(ipcoor), buffer(ipsolut),
     +                       buffer(ipcoefs), buffer(ipeddy),
     +                       buffer(ipgeom), ndim, iseqpres,
     +                       buffer(ipwork) )

c           --- create system of equations
c               Step 1: discretize inner region and external boundary
c               conditions. Also discretize across internal boundaries

            if ( iequa.lt.iseqpres ) then

c           --- momentum equations

               if ( mshtyp.eq.0 ) then

c              --- staggered

                  call issysm ( ibuffr(ipiinput), buffer(iprinput),
     +                          buffer(ipmatrix), buffer(ipgmat),
     +                          buffer(iprhside), buffer(ipcoefs),
     +                          buffer(ipsolut), buffer(ipcoor),
     +                          ibuffr(ipinfmm), ibuffr(ipinfmg),
     +                          buffer(ipgeom), ibuffr(ipinfbnd),
     +                          buffer(ipbndcon), buffer(ipeddy),
     +                          buffer(ipwork), npf, rspv, cv )
                  matname = 'Momentum'

               else if ( mshtyp.eq.1 ) then

c              --- collocated

                  call issyst ( ibuffr(ipiinput), buffer(iprinput),
     +                          buffer(ipmatrix), buffer(ipgmat),
     +                          buffer(iprhside), buffer(ipcoefs),
     +                          buffer(ipsolut), buffer(ipcoor),
     +                          ibuffr(ipinfmt), ibuffr(ipinfmg),
     +                          buffer(ipgeom), ibuffr(ipigeom),
     +                          ibuffr(ipinfbnd),
     +                          buffer(ipbndcon), buffer(ipwork),
     +                          lwork, iequa, isiinput, issolut,
     +                          isitopol )
                  if ( iequa.eq.1 ) then
                     matname = 'Momentum u'
                  else if ( iequa.eq.2 ) then
                     matname = 'Momentum v'
                  else if ( iequa.eq.3 .and. iseqpres.eq.4 ) then
                     matname = 'Momentum w'
                  end if
               end if

            else if ( iequa.eq.iseqpres ) then

c           --- pressure equations

               call issysp ( ibuffr(ipiinput), buffer(iprinput),
     +                       buffer(ipmatrix), buffer(ipgmat),
     +                       buffer(iprhside), buffer(ipcoefs),
     +                       buffer(ipsolut), buffer(ipcoor),
     +                       ibuffr(ipinfmg), ibuffr(ipinfmt),
     +                       buffer(ipgeom), ibuffr(ipigeom),
     +                       ibuffr(ipinfbnd), buffer(ipwork),
     +                       buffer(ipeddy), buffer(ipdensup),
     +                       ibuffr(ipitopol) )

               matname = 'Pressure'

            else

c           --- transport equations

               call issyst ( ibuffr(ipiinput), buffer(iprinput),
     +                       buffer(ipmatrix), buffer(ipgmat),
     +                       buffer(iprhside), buffer(ipcoefs),
     +                       buffer(ipsolut), buffer(ipcoor),
     +                       ibuffr(ipinfmt), ibuffr(ipinfmg),
     +                       buffer(ipgeom), ibuffr(ipigeom),
     +                       ibuffr(ipinfbnd),
     +                       buffer(ipbndcon), buffer(ipwork),
     +                       lwork, iequa, isiinput, issolut,
     +                       isitopol )

               matname = 'Transport'

            end if

c           --- system of equations has been created, so set PRECBLT = 0
c               in IINSOL

            ipiinsol = ibuffr( ipiinput + 5 - 1 )
            linsol   = 25

            if( iequa.eq.iseqpres ) then
               ipiinsol = ipiinsol + linsol
            else if ( iequa.gt.iseqpres )then
               ipiinsol = ipiinsol + 2*linsol
            end if
            ibuffr(ipiinput + ipiinsol + 16 - 2) = 0

c           --- delete local arrays

            call isdelarr(isinfmm)
            call isdelarr(isinfmg)
            call isdelarr(isinfmt)
            call isdelarr(iswork)

            call isdcrpri

           if ( itime.eq.1 ) then

c          --- report timing results for matrix construction

               text = 'for building the '
               write(text(18:27),100) matname
100            format(a)
               text(28:39) = ' matrix is  '
               call isaskclk ( 1, text )

            end if

         end if

      end do

      call iserclos('isbldmat')

      end
