      subroutine ischsubf ( iinput, ni, nj, nk, ndim, ntrnsp, nturb,
     +                      nblocks, nfaces, maxnsubf, iinbc,
     +                      nsubfaces, subfaces, coor, isitopol )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.2    date   02-02-1998 Guus: also blocks allowed
c        version 1.1    date   28-01-1998 Guus: extension to 3D
c        version 1.0    date   31-10-1997
c
c   copyright (c) 1997-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     change array subfaces for priodic boundary conditions, and
c     take some actions which should be done for the multi block
c     transport routines which are used for the periodic boundary
c     conditions
c **********************************************************************
c
c                       KEYWORDS
c
c     periodic
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  iinput(*), ni, nj, nk, ndim, isitopol
      integer  ntrnsp, nturb, nblocks, nfaces, maxnsubf, iinbc(*),
     +         subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *),
     +         nsubfaces(1:nblocks, 1:nfaces)
      double precision coor(*)

c     coor      i    reserved
c     iinbc     i    reserved
c     iinput    i    reserved
c     isitopol  i    Sequence number of array itopol
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical periodic
      integer iface, nseg, k, ipstart, iequa, btype, vlen, isabsor,
     +        ipabsor, ih, iblock, nbface

c     btype     Type of boundary condition
c     iblock    Block sequence number
c     iequa     General equation sequence number
c     iface     Face number
c     ih        Help variable to store a constant temporarily
c     ipabsor   Starting address of an array 1..nblocks containing the
c               absolute orientation of the subdomains. (1 -> sqrt(g) >0
c               and -1 -> sqrt(g) < 0)
c     ipstart   Starting address
c     isabsor   Pointer to an array 1..nblocks containing the
c               absolute orientation of the subdomains. (1 -> sqrt(g) >0
c               and -1 -> sqrt(g) < 0)
c     k         General loop variable
c     nbface    Neighbouring face number
c     nseg      Number of segments the boundary is divided in
c     periodic  Periodic boundary conditions
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddvlen, isblockor, isingetp

c     ISBLOCKOR Compute the absolute orientation of a subdomain
c     ISDDVLEN  Compute the last dimension of a subdomain virt array and
c               initialize the vstart values of itopol
c     ISDELARR  Delete a permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISIOR     Compute iorient fields of the itopol array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ischsubf' )

      periodic = .false.
      do iface = 1, nfaces

         nseg = iinbc( iinbc(iface) )

         do k = 1, nseg

            ipstart = iinbc(iface)
            if ( ndim.eq.2 ) then
               ih = 2
            else
               ih = 4
            end if
            ipstart = ipstart + (k-1) *(ih +(ntrnsp+nturb+2)*(ndim+1) )

            iequa = 1
            btype = iinbc( ipstart +(iequa-1)*(ndim+1) + ih+1 )
            if ( btype.ne.98 ) then
                iequa = ndim+1
                btype = iinbc( ipstart +(iequa-1)*(ndim+1) + ih+1 )
            end if
            if ( btype.eq.98 ) then
               periodic = .true.
               nbface = iinbc( ipstart +(iequa-1)*(ndim+1) +ih+2)
               iblock = nbface/10+1
               if ( iblock.gt.1 ) nbface=mod(iface,10)
               subfaces(1,iface,k,5)=iblock
               subfaces(1,iface,k,6)=nbface
               subfaces(1,iface,k,7)=
     +            iinbc( ipstart +(iequa-1)*(ndim+1) +ih+3)
            end if

         end do

      end do

      if ( periodic ) then

c     --- vlen is only given a value if there is a multi block
c         boundary indicated by : subfaces(1,iface,k,5).ne.0
c         as  we are changing the value : subfaces(1,iface,k,5)
c         for periodic subfaces in this routine we have to give
c         vlen a value here

         vlen = isddvlen(1, isitopol)
         iinput(58) = vlen

         isabsor = 0
         call isicreat( isabsor,nblocks,'absor')
         ipabsor = isingetp(isabsor)
         if ( nk.eq.0) then
            ibuffr(ipabsor) = isblockor(ni,nj,ndim,coor)
         else
            ibuffr(ipabsor) = 1
         end if
         call isior(nblocks, nfaces, maxnsubf, nsubfaces,
     +              subfaces, ibuffr(ipabsor) )
         call isdelarr(isabsor)

      end if

      call iserclos( 'ischsubf' )

      end
