      subroutine iscofkw2 ( coefs, iinbc, rinbc, solut, coor, eddy, ni,
     +                      nj, ntrnsp, nturb, ndegfd, nvirtual, ntimlv,
     +                      ncoefs, rintur, invjac )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.3    date 19-03-1997 Marcel: correction address rinbc
c        version 1.2    date 16-04-1996 Kees K.: space in iinbc and rinbc
c                                       for pressure bound. cond.
c        version 1.1    date   12-09-1995 Marcel: parameter list of
c                                               iswllqn2 changed
c        version 1.0    date   31-08-1995 Guus: extracted from iscofkwf
c
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fill some coefficients with respect to k-equation using
c     wall functions (2D version)
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ntrnsp, nturb, ndegfd, nvirtual,
     +        ntimlv, ncoefs, iinbc(*)
      double precision coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ncoefs),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 coor(*), rinbc(*), rintur(*),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4)

c     coefs     o    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     iinbc     i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iside, iarea(2,2), ndir, tdir,
     +        ipstart, nseg, j, ipiinbc, ip, jp, nsign, iplrinbc
      double precision ndist, physut, rhow, mu, tke, tauw, uplus

c     iarea     Area of arrays
c     ip        Local counting variable i-direction
c     ipiinbc   Starting address of array iinbc in array iinput
c     iplrinbc  Start address
c     ipstart   Starting address
c     iside     Sequence number of side
c     j         General loop variable
c     jp        Local counting variable j-direction
c     mu        The laminar viscosity
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nseg      Number of segments the boundary is divided in
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     tauw      The wall shear stress
c     tdir      Direction of tangent vector on this side
c     tke       The turbulent kinetic energy
c     uplus     The dimensionless tangential velocity
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISLOGW    Compute the wall shear stress with log-law based
c               wall condition depending on smoothness or roughness
c               of the boundary
c     ISWLLQN2  Determine essential wall quantities
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      iplrinbc = 0

c     --- loop over boundaries

      do 10 iside = 1, 4

c     --- Get information on boundary 'iside'

        if ( iside.le.2 ) then
           ndir = 1
           tdir = 2
           if ( iside.eq.1 ) then
              nsign = -1
              iarea(1,1) = 1
           else
              nsign = 1
              iarea(1,1) = ni
           end if
           iarea(2,1) = iarea(1,1)
        else
           ndir = 2
           tdir = 1
           if ( iside.eq.3 ) then
              nsign = -1
              iarea(1,2) = 1
           else
              nsign = 1
              iarea(1,2) = nj
           end if
           iarea(2,2) = iarea(1,2)
        end if

        ipstart = iinbc(iside)

        nseg = iinbc(ipstart)

c       --- loop over segments

        do 20 j = 1, nseg

          ipiinbc = ipstart +(j-1) *( 2+(ntrnsp+nturb+2)*3 )

          iarea(1,tdir) = iinbc( ipiinbc + 1 )
          iarea(2,tdir) = iinbc( ipiinbc + 2 )

          if ( iinbc( ipiinbc + 3 ).eq.9 ) then

c         --- Fill coefs in wall adjacent cells for turbulent
c             kinetic energy equation using wall functions

c             --- loop over cel

              do 30 jp = iarea(1,2), iarea(2,2)
                do 30 ip = iarea(1,1), iarea(2,1)

c               --- first, compute two essential wall quantities:
c                   distance perpendicular to the wall and physical
c                   tangential velocity

                   call iswllqn2 ( ndist, physut, ni , nj, nvirtual,
     +                             ip, jp, coor, invjac(1,4), solut,
     +                             ndir, tdir, nsign, 1 )

                   rhow = coefs(ip,jp,1)
                   mu = coefs(ip,jp,4) -
     +                  coefs(ip,jp,1)*eddy(ip,jp)/rintur(4)
                   tke = solut(ip,jp,ndegfd-1,1)

                   call islogw ( tauw, uplus, physut, ndist, tke,
     +                           rintur, rhow, mu, rinbc(iplrinbc+1) )

                   coefs(ip,jp,3) = tauw*physut/ndist
                   coefs(ip,jp,2) = rhow*( rintur(3)**(75d-2) )*
     +                                 sqrt( tke )*uplus/ndist

30          continue

          end if

          iplrinbc = iplrinbc +(ntrnsp+nturb+2)*2

20      continue

10    continue

      end
