      subroutine iscomsr2 (ni, nj, ndegfd, nvirtual, solut, eddy,
     +                     invjac, cva, rincom, switch, shiftr,
     +                     mscal, mcav, ichoicedb, istartup, rincav)
c ======================================================================
c
c        programmer     Hester Bijl
c        version 1.7    date 23-02-2000 Duncan: change of loop bounds
c        version 1.6    date 20-03-1999 Duncan: revision of Mach switch
c        version 1.5    date 04-03-1999 Duncan: rincav added; new call
c                                               to iscmrho2
c        version 1.4    date 27-07-1998 Duncan: extension for limiter
c                                               function
c        version 1.3    date 05-03-1998 Duncan: extension for cavity
c        version 1.2    date 12-11-1997 Hester: parameter mscal added
c        version 1.1    date 15-05-1996 Hester: in sqrt abs added.
c        version 1.0    date 07-09-1995
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   ISNaS compressible subroutine for computation of density-biased
c   upwinding in pressure correction equation
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndegfd, nvirtual, mscal, mcav, istartup,
     +    ichoicedb
      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,4),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 rincom(6),eddy(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2),
     +                 switch(ni+1,nj+1,*),
     +                 shiftr(ni+1,nj+1,*),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 rincav(*)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     eddy      i    reserved
c     ichoicedb i    if 1 a gradient based, if 2 a mach based,
c                    if 3 an unconditional density bias is applied
c     invjac    i    1/sqrt(g) in pnttyp
c     istartup  i    If 1 the cavity calculation is in the startup procedure,
c                    if 0 not
c     mcav      i    If 1 a cavity model is used, if 0 not
c     mscal     i    Indicates use of non-dimensionalisation for compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rincav    i    reserved
c     rincom    i    reserved
c     shiftr    i    Shift variable used in compressible density upwinding.
c     solut     i    reserved
c     switch    i    Continuous compressible density upwind switch
c                      switch = 0 for Mach numbers < 1-ls
c                      switch=0.5 for Mach numbers > 1+ls
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k
      double precision gamcm, mach2, cons, cons1, ls,
     +                 locm, signu, signv, speed2, csqrdinv, uabs,
     +                 vabs

c     cons      Constant used in equation of state for compressible comput.
c     cons1     Constant used in equation of state for compressible comput.
c     csqrdinv  Inverse of speed of sound squared
c     gamcm     Gas constant gamma (for compressible comput.)
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     locm      Local Mach number.
c     ls        Length of switch interval for compr. density upwind switch
c                 switch=0   for Mach numbers < 1-ls
c                 switch=0.5 for Mach numbers > 1+ls
c     mach2     Square of the mach number (for compressible comput.)
c     signu     direction of velocity either 1 or -1
c     signv     direction of velocity either 1 or -1
c     speed2    Speed squared
c     uabs      Magnitude of contravariant velocity in i direction
c     vabs      Magnitude of contravariant velocity in j direction
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdstfll, isdstlss

c     ISDSTFLL  Derivative of dimensional equation of state
c     ISDSTLSS  Derivative of dimensionless equation of state
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c      Mach based density bias
c
c          -if nondimensional computation
c              -if speed greater than 0
c                    - compute local Ma nr
c                             |U|     a_alpha U^alpha
c                        M = ------ = ----------------
c                              a             a
c                     a = nondimensional speed of sound
c                    - compute switch function for density shift i direction
c                    - compute switch function for density shift j direction
c              -end if
c          -else if dimensional computation
c              -if speed greater than 0
c                    - compute local Ma nr
c                             |U|     a_alpha U^alpha
c                        M = ------ = ----------------
c                              a             a
c                     a = dimensional speed of sound
c                       = sqrt((gamma-1)|h|)
c                    - compute switch function for density shift i direction
c                    - compute switch function for density shift j direction
c              -end if
c          -end if
c          - compute shift ratio of density equal to shifted density divided
c              by non shifted density i direction
c          - compute shift ratio of density equal to shifted density divided
c              by non shifted density j direction
c            a) For switch == 0 no upwinding, so shift = 1
c            b) For switch == 0.5 full upwinding, so
c
c                          upwinded density
c                  shift= ------------------
c                           central density
c            c) For switch between 0 and 0.5 shift between value of a) and b)
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      gamcm = rincom(1)
      mach2 = rincom(2)**2d0

      cons = 1d0 + ((gamcm - 1d0)*mach2)/2d0

      cons1 = mach2 / cons

      ls=0.995d0
      do k= 1,2
         do j = 1, nj+1
            do i = 1, ni+1

               switch(i,j,k) = 0d0

            end do
         end do
      end do

      if( ichoicedb.eq.2) then

         if (mscal.eq.0) then

c        --- scaled, dimensionless computation

c            first the i direction

           do j = 1, nj+1
              do i = 1, ni+1

c             --- define square of speed of sound at the face

                 if(mcav.eq.1)  then

                    csqrdinv= (isdstlss(solut(i-1,j,3,1), rincom,
     +                                        istartup, rincav)+
     +                         isdstlss(solut(i  ,j,3,1), rincom,
     +                                        istartup, rincav))/2d0
                 else
                    csqrdinv= 2d0*cons1 /
     +                           (abs(solut(i-1,j,4,1))+
     +                                   abs(solut(i,j,4,1)))
                 end if

c                --- define normal component of cartesian velocity squared

                 speed2=   solut(i,j,1,1)**2/
     +                       ( cva(i,j,2,1,3 )**2+cva(i,j,2,2,3 )**2)

                 locm =    sqrt(speed2*csqrdinv)

                 uabs  = abs(solut(i,j,1,1)*invjac(i,j,3) )
                 signu = sign(1d0, solut(i,j,1,1)*invjac(i,j,3))

                 if( uabs.gt.1d-15) then
                    switch(i,j,1) = signu*
     +                           min(.5d0,max(0d0,0.25d0*
     +                           (((locm-1d0)/ls)+1d0)))

                 else

                    switch(i,j,1) = 0d0
                 end if

              end do
           end do

c          --- next the j direction, only for cavitating flow

           if( mcav.eq.1) then

              do j = 1, nj+1
                 do i = 1, ni+1

c                --- define square of inverse of  speed of sound at the face

                    if(mcav.eq.1)  then

                       csqrdinv= (isdstlss(solut(i,j-1,3,1), rincom,
     +                                           istartup, rincav)+
     +                            isdstlss(solut(i,j  ,3,1), rincom,
     +                                           istartup, rincav))
     +                                                          /2d0
                    else
                       csqrdinv= 2d0*cons1 /
     +                              (abs(solut(i,j-1,4,1))+
     +                                      abs(solut(i,j,4,1)))
                    end if

c                   --- define normal component of cartesian velocity squared

                    speed2=   solut(i,j,2,1)**2/
     +                          ( cva(i,j,1,1,2 )**2+
     +                                           cva(i,j,2,1,2 )**2)

                    locm =    sqrt(speed2*csqrdinv)

                    vabs  = abs(solut(i,j,2,1)*invjac(i,j,3) )
                    signv = sign(1d0, solut(i,j,2,1)*invjac(i,j,3))

                    if( vabs.gt.1d-15) then

                       switch(i,j,2) = signv*
     +                           min(.5d0,max(0d0,0.25d0*
     +                           (((locm-1d0)/ls)+1d0)))

                    else

                       switch(i,j,2) = 0d0

                    end if

                 end do

              end do

            end if

         else if (mscal.eq.1) then

c        --- unscaled, dimensional computation

c            first the i direction

           do j = 1, nj+1
              do i = 1, ni+1

c             --- define square of inverse of speed of sound at the face

                 if(mcav.eq.1)  then

                    csqrdinv= (isdstfll(solut(i-1,j,3,1),
     +                                        istartup, rincav)+
     +                         isdstfll(solut(i  ,j,3,1),
     +                                        istartup, rincav))/2d0
                 else
                    csqrdinv= 2d0*cons1 /
     +                           (abs(solut(i-1,j,4,1))+
     +                                   abs(solut(i,j,4,1)))
                 end if

c                --- define normal component of cartesian velocity squared

                 speed2=   solut(i,j,1,1)**2/
     +                       ( cva(i,j,2,1,3 )**2+cva(i,j,2,2,3 )**2)

                 locm =    sqrt(speed2*csqrdinv)

                 uabs  = abs(solut(i,j,1,1)*invjac(i,j,3) )
                 signu = sign(1d0, solut(i,j,1,1)*invjac(i,j,3))

                 if( uabs.gt.1d-15) then
                    switch(i,j,1) = signu*
     +                            min(.5d0,max(0d0,(5d0*locm-0.5d0)))

                 else

                    switch(i,j,1) = 0d0
                 end if

              end do
           end do

c          --- next the j direction, only for cavitating flow

           if( mcav.eq.1) then

              do j = 1, nj+1
                 do i = 1, ni+1

c                --- define square of speed of sound at the face

                    if(mcav.eq.1)  then

                       csqrdinv= (isdstfll(solut(i,j-1,3,1),
     +                                           istartup, rincav)+
     +                            isdstfll(solut(i,j  ,3,1),
     +                                           istartup, rincav))
     +                                                          /2d0
                    else
                       csqrdinv= 0.5d0*(gamcm-1d0)*
     +                              (abs(solut(i,j-1,4,1))+
     +                                      abs(solut(i,j,4,1)))
                    end if

c                   --- define normal component of cartesian velocity squared

                    speed2=   solut(i,j,2,1)**2/
     +                          ( cva(i,j,1,1,2 )**2+
     +                                           cva(i,j,2,1,2 )**2)

                    locm =    sqrt(speed2*csqrdinv)

                    vabs  = abs(solut(i,j,2,1)*invjac(i,j,3) )
                    signv = sign(1d0, solut(i,j,2,1)*invjac(i,j,3))

                    if( vabs.gt.1d-15) then

                       switch(i,j,2) = signv*
     +                           min(.5d0,max(0d0,(5d0*locm-0.5d0)))

                    else

                       switch(i,j,2) = 0d0

                    end if

                 end do

              end do

            end if

         end if

      else if( ichoicedb.eq.3) then

         do j=1,nj+1
            do i=1,ni+1

               switch(i,j,1) =sign(0.5d0,
     +                   solut(i,j,1,1)*invjac(i,j,3))
            end do
         end do

         if( mcav.eq.1) then

            do j=1,nj+1
               do i=1,ni+1

                  switch(i,j,2) =sign(0.5d0,
     +                      solut(i,j,2,1)*invjac(i,j,3))
               end do
            end do

         end if

      end if

c     --- computation of shifting density Sro/ro in points (i-1/2,j)

      i=1
      do j=1,nj+1
        shiftr(i,j,1)=1d0
      end do

      j=1
      do i=1,ni+1
        shiftr(i,j,2)=1d0
      end do

      do i = 2, ni+1
         do j = 1, nj+1

            shiftr(i,j,1) = 2d0*((0.5d0-switch(i,j,1))*eddy(i,j,2)
     +                    +(0.5d0+switch(i,j,1))*eddy(i-1,j,2))/
     +                    (eddy(i,j,2)+eddy(i-1,j,2))

         end do
      end do

      if( mcav.eq.1) then

         do i = 1, ni+1
            do j = 1, nj+1

               shiftr(i,j,2) = 2d0*((0.5d0-switch(i,j,2))*eddy(i,j,2)
     +                    +(0.5d0+switch(i,j,2))*eddy(i,j-1,2))/
     +                    (eddy(i,j,2)+eddy(i,j-1,2))

            end do
         end do
      else

         call issetcon(shiftr,(ni+1)*(nj+1),1d0)

      end if

      end
