      subroutine iscorrh3 ( rhside, ni, nj, nk )

c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date   05-09-95
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c ********************************************************************
c
c                         DESCRIPTION
c
c
c   Correct pressure right-hand side in case of a singular pressure
c   matrix due to given normal velocity at all sides
c ********************************************************************
c
c                      KEYWORDS
c
c
c    3d
c    correction_right_hand_side
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      integer ni, nj, nk
      double precision  rhside(1:ni+1,1:nj+1,1:nk+1)

c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     rhside   i/o   reserved
c ********************************************************************
c
c                      COMMON BLOCKS
c
c    -
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, j, k
      double precision sum, dp

c     dp        Correction of pressure
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     sum       Help variable to compute a sum
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c    Compute sum f(i)
c    correction := ( sum f(i) ) / (ni*nj*nk)
c    Correct f(i):  f(i) := f(i) - correction
c ======================================================================
c
c     --- Compute sum f(i)

      sum = 0d0

      do i = 1, ni
         do j = 1, nj
            do k = 1, nk
               sum = sum  + rhside(i,j,k)
            end do
         end do
      end do

c     --- correction := ( sum f(i) ) / (ni*nj*nk)

      dp = sum/(ni*nj*nk)

c     --- Correct f(i):  f(i) := f(i) - correction

      do i = 1, ni
         do j = 1, nj
            do k = 1, nk
                rhside(i,j,k) = rhside(i,j,k) - dp
            end do
         end do
      end do

      end
