      subroutine isdist2 ( wdis, coor, iinbc, nvirtual, ni, nj, ntrnsp,
     +                     nturb, ndegfd )
c ======================================================================
c
c     programmer   Marcel Zijlema
c     version      1.1    date    16-09-1997  Fred T. use array wdis
c     version      1.0    date    10-07-1996
c
c   copyright (c) 1996-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill array eddy with normal distance to walls in the whole domain
c
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer iinbc(*), ni, nj, nvirtual, ntrnsp, nturb, ndegfd
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual, 1:2),
     +                 wdis(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual)

c     coor      i    reserved
c     iinbc     i    reserved
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     wdis      o    Array containing the wall distance
c **********************************************************************
c
c                       COMMON BLOCKS
c
c     none
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iside, iarea(2,2), ndir, tdir,
     +        ipstart, nseg, j, ipiinbc, ip, jp, nsign,
     +        ipjp(4,2), i, ipl, jpl
      double precision xq(2), tvec(2), xp(2), eucnorm, ndist

c     eucnorm   Euclidian norm of a vector
c     i         General loop variable
c     iarea     Area of arrays
c     ip        Local counting variable i-direction
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipjp      Array containing the ip and jp indices of the 4 vertices of
c               the cell
c               ipjp(i,j): i=1,4; j=1,2
c               j=1: ip     j=2: jp
c     ipl       Local ip value
c     ipstart   Starting address
c     iside     Sequence number of side
c     j         General loop variable
c     jp        Local counting variable j-direction
c     jpl       Local jp value
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nseg      Number of segments the boundary is divided in
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     tdir      Direction of tangent vector on this side
c     tvec      Tangential vector
c     xp        Auxiliary array to calculate boundary point number
c     xq        Array containing the co-ordinates of the point Q at the boundary
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isdist2')

c     --- Initialize eddy

      do ip = 1, ni
        do jp = 1, nj
          wdis(ip,jp) = 1d8
        end do
      end do

c     --- loop over boundaries

      do iside = 1, 4

c     --- Get information on boundary 'iside'

        if ( iside.le.2 ) then
           ndir = 1
           tdir = 2
           if ( iside.eq.1 ) then
              nsign = -1
              iarea(1,1) = 1
           else
              nsign = 1
              iarea(1,1) = ni
           end if
           iarea(2,1) = iarea(1,1)
        else
           ndir = 2
           tdir = 1
           if ( iside.eq.3 ) then
              nsign = -1
              iarea(1,2) = 1
           else
              nsign = 1
              iarea(1,2) = nj
           end if
           iarea(2,2) = iarea(1,2)
        end if

        ipstart = iinbc(iside)

        nseg = iinbc(ipstart)

c       --- loop over segments

        do j = 1, nseg

          ipiinbc = ipstart +(j-1) *( 2+(ntrnsp+nturb+2)*3 )

          iarea(1,tdir) = iinbc( ipiinbc + 1 )
          iarea(2,tdir) = iinbc( ipiinbc + 2 )

          if ( iinbc( ipiinbc + (ndegfd-nturb)*3 + 3 ).eq.48 ) then

            do ip = 1, ni
              do jp = 1, nj

c             --- Compute the co-ordinates of P (centroid)

                do i = 1, 2
                  xp(i) = 25d-2*( coor(ip,jp,i)+coor(ip+1,jp,i)+
     +                            coor(ip+1,jp+1,i)+coor(ip,jp+1,i) )
                end do

                ipl = ip
                jpl = jp
                if ( iarea(1,1).eq.iarea(2,1) ) then
                  ipl = iarea(1,1)
                else if ( iarea(1,2).eq.iarea(2,2) ) then
                  jpl = iarea(1,2)
                end if

c               --- Fill array ipjp as function of ipl, jpl and iside

                do i = 1, 2
                  ipjp(i,1) = ipl
                  ipjp(i,2) = jpl
                end do
                if ( nsign.eq.1 ) then

c               --- nsign = 1, sides 2 and 4

                  ipjp(1,ndir) = ipjp(1,ndir)+1
                  ipjp(2,1)    = ipjp(2,1)+1
                  ipjp(2,2)    = ipjp(2,2)+1

                else

c               --- nsign = -1, sides 1 and 3

                  ipjp(2,tdir) = ipjp(2,tdir)+1

                end if

c               --- Compute the unit tangential vector and
c                   the co-ordinates of Q

                do i = 1, 2
                  tvec(i) = coor(ipjp(2,1),ipjp(2,2),i)-
     +                      coor(ipjp(1,1),ipjp(1,2),i)
                  xq(i) = 5d-1*( coor(ipjp(1,1),ipjp(1,2),i) +
     +                           coor(ipjp(2,1),ipjp(2,2),i) )
                end do
                eucnorm = 1d0/sqrt(tvec(1)**2+tvec(2)**2)
                tvec(1) = tvec(1)*eucnorm
                tvec(2) = tvec(2)*eucnorm
                ndist =  abs( -tvec(2)*( xp(1) - xq(1) ) +
     +                         tvec(1)*( xp(2) - xq(2) ) )

                wdis(ip,jp) = min( ndist, wdis(ip,jp) )

              end do
            end do

          end if

        end do

      end do

      call iserclos('isdist2')

      end
