      subroutine isdstfl2(press, lpress, istartup, isdstfll, rincav)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.0    date 17-11-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   IsNas compressible function isdstdfl2 calculates the
c   dimensional form of the derivative drho/dp out of the dimensional
c   pressure
c   isdstfl2 is the vector form of isdstfll
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible, equation_of_state
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer istartup, lpress
      double precision press(lpress), isdstfll(lpress), rincav(*)

c     isdstfll   o  drhodp
c     istartup   i  If 1 the cavity calculation is in the startup procedure,
c                   if 0 not
c     lpress     i  length of array pressure
c     press      i  pressure
c     rincav     i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i
      double precision  alpha1f, alpha2f, dbigfdksi, dbiggdksi,
     + c1, c2, c3, c4, delrho, ksi, npoly, pres1tr, pres2tr, dksidp

c     alpha1f    Parameter in F(x)
c     alpha2f    Parameter in F(x)
c     c1         General help variable
c     c2         General help variable
c     c3         General help variable
c     c4         General help variable
c     dbigfdksi  First derivative of big F(x)
c     dbiggdksi  First deruvative of big G(x)
c     delrho     Density jump across transition
c     dksidp     Dksi/dp
c     i          General loop variable
c     ksi        The scaled transition pressure
c     npoly      Degree of the interpolation polynomial
c     pres1tr    Pressure at start of transition
c     pres2tr    Pressure at end of transition
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c            if (P < = P1) then
c                 drhodp = drhodp(vapor)
c                     where drhodp is a up to ninth order polynomial
c                      in the help variable ksi.
c            else
c                 if (P > P1 and P< P2) then
c                     drhodp = drhodp(P)
c                 else
c                     drhodp(liquid)
c                 end if
c            end if
c            isdstfll=drhodp
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- input variables

      delrho  = rincav(5)
      pres2tr = rincav(4)
      pres1tr = rincav(3)
      c1      = rincav(2)
      c4      = rincav(1)

c     --- constants to be computed

      c3    = (pres2tr-pres1tr)*(c4-c1)
      c2    = c1*(pres1tr-pres2tr)+delrho-c3/2d0

c     --- Actual derivative of the equation of state

      if( istartup.eq.1) then

c     --- fully wetted flow

         do i=1, lpress
            isdstfll(i) =c4
         end do
      else

c     --- first linear part

         do i=1, lpress
            if(press(i).lt.pres1tr) then
               isdstfll(i)=c1
            else

c           --- second linear part

               if(press(i).gt.pres2tr) then
                  isdstfll(i)=c4
               else

c              --- the transition part

c                  --- function dF(ksi)/dksi & function dG(ksi)/dksi

                   npoly = 9
                   ksi = (press(i)-pres1tr)/(pres2tr-pres1tr)
                   alpha1f = 2*(npoly-1d0)/(npoly+1d0)
                   alpha2f = 2d0**(-1d0*npoly)*(2d0-alpha1f)
                   if (ksi.le.0.5d0) then
                        dbigfdksi= 1/alpha2f*
     +                      (npoly*ksi**(npoly-1d0)-
     +                        (npoly+1d0)*alpha1f*ksi**npoly)
                        dbiggdksi= ((2*ksi)**npoly)/4*((npoly+1d0)*
     +                        (1d0-(npoly-1)/(npoly+2)*2*ksi)-
     +                        (npoly-1)/(npoly+2))
                   else
                        dbigfdksi= ((1d0-ksi)**npoly)*npoly*
     +                        (alpha1f*ksi+1-alpha1f)/
     +                                       (alpha2f*(1-ksi))-
     +                  (1-ksi)**npoly*alpha1f/alpha2f
                        dbiggdksi=1+0.25d0*(2d0-2d0*ksi)**npoly*
     +                   ((npoly+1d0)*((npoly-1d0)/(npoly+2d0)*
     +                       (2d0-2*ksi)-1d0)+
     +                          2d0*(npoly-1d0)/(npoly+2d0))
                   end if
                   dksidp=1d0/(pres2tr-pres1tr)
                   isdstfll(i)=c1+c2*dbigfdksi*dksidp+c3*dbiggdksi*
     +                                                         dksidp
               end if
            end if
         end do
      end if
      end
