      function isdstlss(pressless,rincom, istartup, rincav)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.2    date 03-03-1999  Duncan: Input aangepast
c     version      1.1    date 27-07-1998  Duncan: parameter istartup added
c     version      1.0    date 05-03-1998
c
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c  function isdstlss calculates drho/dp out of
c  the equation of state
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible, equation_of_state
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer istartup
      double precision pressless, isdstlss, rincom(*), rincav(*)

c     isdstlss   o  drhodp
c     istartup   i  If 1 the cavity calculation is in the startup procedure,
c                   if 0 not
c     pressless  i  pressure
c     rincav     i  reserved
c     rincom     i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision  alpha1f, alpha2f, dbigfdksi, dbiggdksi,
     + c1, c2, c3, c4, delrho, ksi, npoly, pres1tr, pres2tr, dksidp,
     +  rhoref, velref, pref, dstadfull, press

c     alpha1f    Parameter in F(x)
c     alpha2f    Parameter in F(x)
c     c1         General help variable
c     c2         General help variable
c     c3         General help variable
c     c4         General help variable
c     dbigfdksi  First derivative of big F(x)
c     dbiggdksi  First deruvative of big G(x)
c     delrho     Density jump across transition
c     dksidp     Dksi/dp
c     dstadfull  Drhodp
c     ksi        The scaled transition pressure
c     npoly      Degree of the interpolation polynomial
c     pref       Reference value for pressure
c     pres1tr    Pressure at start of transition
c     pres2tr    Pressure at end of transition
c     press      Pressure
c     rhoref     Reference value for density
c     velref     Reference value for velocity
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c            P=pressless*rhoref*vref^2 + pref
c            if (P < = P1) then
c                 drhodp = drhodp(vapor)
c
c            else if (P > P1 and P< P2) then
c                      drhodp = drhodp(P)
c                 else
c                      drhodp(liquid)
c                 end if
c            end if
c            isdstlss=drhodp*vref^2
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      pref   =rincom(7)
      rhoref =rincom(8)
      velref =rincom(9)

c     --- Scale the pressure back to dimensionfull

      press=pressless*(rhoref*velref**2d0)+pref

c     --- input variables

      delrho    = rincav(5)
      pres2tr   = rincav(4)
      pres1tr   = rincav(3)
      c1        = rincav(2)
      c4        = rincav(1)

      c3    = (pres2tr-pres1tr)*(c4-c1)
      c2    = c1*(pres1tr-pres2tr)+delrho-c3/2d0

c     --- Actual derivative of the equation of state

      if( istartup.eq.1) then

c     --- fully wetted flow

         dstadfull =c4

      else

c     --- first linear part

         if(press.lt.pres1tr) then
            dstadfull=c1
         else

c        --- second linear part

            if(press.gt.pres2tr) then
               dstadfull=c4
            else

c           --- the transition part

c               --- function dF(ksi)/dksi & function dG(ksi)/dksi

                npoly = 9
                ksi = (press-pres1tr)/(pres2tr-pres1tr)
                alpha1f = 2*(npoly-1d0)/(npoly+1d0)
                alpha2f = 2d0**(-1d0*npoly)*(2d0-alpha1f)
                if (ksi.le.0.5d0) then
                     dbigfdksi= 1/alpha2f*
     +                   (npoly*ksi**(npoly-1d0)-(npoly+1d0)*
     +                            alpha1f*ksi**npoly)
                     dbiggdksi= ((2*ksi)**npoly)/4*((npoly+1d0)*
     +                     (1d0-(npoly-1)/(npoly+2)*2*ksi)-
     +                            (npoly-1)/(npoly+2))
                else
                     dbigfdksi= ((1d0-ksi)**npoly)*npoly*
     +                     (alpha1f*ksi+1-alpha1f)/(alpha2f*(1-ksi))-
     +               (1-ksi)**npoly*alpha1f/alpha2f
                     dbiggdksi=1+0.25d0*(2d0-2d0*ksi)**npoly*
     +                ((npoly+1d0)*((npoly-1d0)/(npoly+2d0)*
     +                      (2d0-2*ksi)-1d0)+
     +                2d0*(npoly-1d0)/(npoly+2d0))
                end if
                dksidp=1d0/(pres2tr-pres1tr)
                dstadfull=c1+c2*dbigfdksi*dksidp+c3*dbiggdksi*dksidp
            end if
         end if
      end if

      isdstlss=dstadfull*velref**2d0

      end
