      subroutine isexso02 ( iblock, iface, itrans, ni, nj, nvirtual,
     +                      ndegfd, itimlv, ntimlv, nblocks, nfaces,
     +                      maxnsubf, solut, nsubfaces, subfaces,
     +                      ibipoint, ibinfor, bndcon, icoupled,
     +                      usebndcn )
c ======================================================================
c
c        programmer   Kees Kassels
c        version 2.1  date 06-08-1998 Guus, usebndcn also for velocity
c        version 2.0  date 14-10-1997 Guus, extra parameters
c        version 1.3  date 04-08-1995 Michiel K: ISEXSO02 is now called
c                                     per face, so that periodic bc's
c                                     can be used.
c        version 1.2  date 28-03-1995 Erik: small error corrected
c        version 1.1  date 24-01-1995 Erik: corrected error in the
c                                     range for ilow and ihigh
c        version 1.0  date 11-11-1994 (copy of isexco02 for fluxes)
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize virtual unknowns in 2D
c
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, itrans, maxnsubf, nblocks, ndegfd, nfaces,
     +        ni, nj, nvirtual, itimlv, ntimlv, iface, ibipoint(*),
     +        ibinfor(*), icoupled
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv), bndcon(2,*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      logical usebndcn

c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iblock    i    Block sequence number
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     iface     i    Face number
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndegfd    i    Number of degrees of freedom
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut    i/o   reserved
c     subfaces  i    reserved
c     usebndcn  i    If true boundary conditions are used in the extrapolation
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer diri
      parameter( diri = 1 )
      integer i, ihigh, ilow, isubface, j, k, ipibinf, ipoint,
     +        ifi, jfi, klow, khigh, btype, ilowloop
      logical tangcell

c     btype     Type of boundary condition
c     diri      Dirichlet boundary condition, i.e. T is given
c     i         General loop variable
c     ifi       Starting value of general loop variable i
c     ihigh     Upper index of subface
c     ilow      Lower index of subface
c     ilowloop  Lower bound for loop
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     isubface  Subface number
c     j         General loop variable
c     jfi       Starting value of general loop variable j
c     k         General loop variable
c     khigh     Upper index of unknowns
c     klow      Lower index of unknowns
c     tangcell  If true a tangential cell has been found with integration point
c               at the boundary
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c        If (subface isubface is external ) then
c
c          Determine index range [ilow,ihigh] along the subface at which
c          to compute virtual coordinates by extrapolation.
c
c          Do i = ilow, ihigh
c
c             extrapolate nvirtual rows of virtual coordinates at
c             index i
c
c          Enddo
c
c        Endif
c
c     Endfor
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- determine index range for unknowns

      if ( icoupled.eq.0 .and. usebndcn ) then

c     --- coupled approach

         if ( itrans.eq.1 ) then

c        --- assume solut, tangential fluxes across the boundary,
c            followed by normal fluxes across the boundary

            klow = 1
            khigh = 2

         else

            klow = itrans
            khigh = itrans

         end if

c        --- handle external subfaces

         ipibinf = ibipoint(iface)+nvirtual-1
         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

               do k = klow, khigh

c              --- determine index range of uninitialized unknowns

                  ilow = subfaces(iblock,iface,isubface,1)
                  ihigh = subfaces(iblock,iface,isubface,2)
                  tangcell = .false.

                  if (  ( k.eq.1 .and. iface.ge.3 ) .or.
     +                  ( k.eq.2 .and. iface.le.2 ) ) then

c                 --- tangential flux component

                     ihigh = ihigh + 1
                     tangcell = .true.

c                    --- if the previous subface was internal then exclude
c                        index ilow

                     if ( isubface.gt.1 ) then
                        if ( subfaces(iblock,iface,isubface-1,5) .ne.
     +                        0 )
     +                     ilow = ilow + 1
                     end if

c                    --- if the next subface is internal then exclude
c                        index ihigh

                     if ( isubface.lt.nsubfaces(iblock,iface) )
     +                  then
                        if ( subfaces(iblock,iface,isubface+1,5) .ne.
     +                        0 )
     +                     ihigh = ihigh - 1
                     end if

                  end if

                  if ( iface.eq.ileft ) then

                     if ( tangcell ) then
                        i = 0
                        do j = ilow, ihigh
                           ipoint = ipibinf+2*j-1
                           btype = ibinfor(ipoint)
                           if ( (btype.eq.4 .or. btype.eq.6) .and.
     +                          bndcon(2,ipoint).eq.0d0 .or.
     +                          btype.eq.12 ) then
                              solut(i,j,k,itimlv) =
     +                                    -solut(i+1,j,k,itimlv)
                           else
                              solut(i,j,k,itimlv) =
     +                                 2d0*solut(i+1,j,k,itimlv) -
     +                                     solut(i+2,j,k,itimlv)
                           end if
                        end do
                        ilowloop = -1
                     else
                        ilowloop = 0
                     end if
                     do i = ilowloop, 1-nvirtual, -1
                        do j = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                 2d0*solut(i+1,j,k,itimlv) -
     +                                     solut(i+2,j,k,itimlv)
                        end do
                     end do

                  else if ( iface.eq.iright ) then

                     if ( k.eq.1 ) then
                        ifi = ni+2
                     else
                        ifi = ni+1
                     end if
                     if ( tangcell ) then
                        i = ifi
                        do j = ilow, ihigh
                           ipoint = ipibinf+2*j-1
                           btype = ibinfor(ipoint)
                           if ( (btype.eq.4 .or. btype.eq.6) .and.
     +                          bndcon(2,ipoint).eq.0d0 .or.
     +                          btype.eq.12 ) then
                               solut(i,j,k,itimlv) =
     +                                    -solut(i-1,j,k,itimlv)
                           else
                               solut(i,j,k,itimlv) =
     +                                 2d0*solut(i-1,j,k,itimlv) -
     +                                     solut(i-2,j,k,itimlv)
                           end if
                        end do
                        ifi = ifi+1
                     end if
                     do i = ifi, ni+1+nvirtual
                        do j = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                 2d0*solut(i-1,j,k,itimlv) -
     +                                     solut(i-2,j,k,itimlv)
                        end do
                     end do

                  else if ( iface.eq.idown ) then

                     if ( tangcell ) then
                        j = 0
                        do i = ilow, ihigh
                           ipoint = ipibinf+2*i-1
                           btype = ibinfor(ipoint)
                           if ( (btype.eq.4 .or. btype.eq.6) .and.
     +                          bndcon(2,ipoint).eq.0d0 .or.
     +                          btype.eq.12 ) then
                              solut(i,j,k,itimlv) =
     +                                    -solut(i,j+1,k,itimlv)
                           else
                              solut(i,j,k,itimlv) =
     +                                 2d0*solut(i,j+1,k,itimlv) -
     +                                     solut(i,j+2,k,itimlv)
                           end if
                        end do
                        ilowloop = -1
                     else
                        ilowloop = 0
                     end if
                     do j = ilowloop, 1-nvirtual, -1
                        do i = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                 2d0*solut(i,j+1,k,itimlv) -
     +                                     solut(i,j+2,k,itimlv)
                        end do
                     end do

                  else

                     if ( k.eq.2 ) then
                        jfi = nj+2
                     else
                        jfi = nj+1
                     end if
                     if ( tangcell ) then
                        j = jfi
                        do i = ilow, ihigh
                           ipoint = ipibinf+2*i-1
                           btype = ibinfor(ipoint)
                           if ( (btype.eq.4 .or. btype.eq.6) .and.
     +                          bndcon(2,ipoint).eq.0d0 .or.
     +                          btype.eq.12 ) then
                              solut(i,j,k,itimlv) =
     +                                    -solut(i,j-1,k,itimlv)
                           else
                              solut(i,j,k,itimlv) =
     +                                 2d0*solut(i,j-1,k,itimlv) -
     +                                     solut(i,j-2,k,itimlv)
                           end if
                        end do
                        jfi = jfi+1
                     end if
                     do j = jfi, nj+1+nvirtual
                        do i = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                 2d0*solut(i,j-1,k,itimlv) -
     +                                     solut(i,j-2,k,itimlv)
                        end do
                     end do

                  end if

               end do

            end if
         end do

      else if ( icoupled.eq.0 ) then

c     --- coupled approach

         if ( itrans.eq.1 ) then

c        --- assume solut, tangential fluxes across the boundary,
c            followed by normal fluxes across the boundary

            klow = 1
            khigh = 2

         else

            klow = itrans
            khigh = itrans

         end if

c        --- handle external subfaces

         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

               do k = klow, khigh

c              --- determine index range of uninitialized unknowns

                  ilow = subfaces(iblock,iface,isubface,1)
                  ihigh = subfaces(iblock,iface,isubface,2)

                  if (  ( k.eq.1 .and. iface.ge.3 ) .or.
     +                  ( k.eq.2 .and. iface.le.2 ) ) then

c                 --- tangential flux component

                     ihigh = ihigh + 1

c                    --- if the previous subface was internal then exclude
c                        index ilow

                     if ( isubface.gt.1 ) then
                        if ( subfaces(iblock,iface,isubface-1,5) .ne.
     +                        0 )
     +                     ilow = ilow + 1
                     end if

c                    --- if the next subface is internal then exclude
c                        index ihigh

                     if ( isubface.lt.nsubfaces(iblock,iface) )
     +                  then
                        if ( subfaces(iblock,iface,isubface+1,5) .ne.
     +                        0 )
     +                     ihigh = ihigh - 1
                     end if

                  end if

                  if ( iface.eq.ileft) then

                     do i = 0, 1-nvirtual, -1
                        do j = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                  2d0*solut(i+1,j,k,itimlv) -
     +                                      solut(i+2,j,k,itimlv)
                        end do
                     end do

                  else if ( iface.eq.iright) then

                     if ( k.eq.1 ) then
                        ifi = ni+2
                     else
                        ifi = ni+1
                     end if
                     do i = ifi, ni+1+nvirtual
                        do j = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                  2d0*solut(i-1,j,k,itimlv) -
     +                                      solut(i-2,j,k,itimlv)
                        end do
                     end do

                  else if ( iface.eq.idown ) then

                     do j = 0, 1-nvirtual, -1
                        do i = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                  2d0*solut(i,j+1,k,itimlv) -
     +                                      solut(i,j+2,k,itimlv)
                        end do
                     end do

                  else

                     if ( k.eq.2 ) then
                        jfi = nj+2
                     else
                        jfi = nj+1
                     end if
                     do j = jfi, nj+1+nvirtual
                        do i = ilow, ihigh
                           solut(i,j,k,itimlv) =
     +                                  2d0*solut(i,j-1,k,itimlv) -
     +                                      solut(i,j-2,k,itimlv)
                        end do
                     end do

                  end if

               end do

            end if
         end do

      else if ( .not. usebndcn ) then

c     --- decoupled approach, without using the boundary conditions
c         handle external subfaces

         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

c           --- determine index range of uninitialized unknowns

               ilow = subfaces(iblock,iface,isubface,1)
               ihigh = subfaces(iblock,iface,isubface,2)

               if ( iface.eq.ileft) then

                  do i = 0, 1-nvirtual, -1
                     do j = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i+1,j,itrans,itimlv) -
     +                                   solut(i+2,j,itrans,itimlv)
                     end do
                  end do

               else if ( iface.eq.iright) then

                  do i = ni+1, ni+1+nvirtual
                     do j = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i-1,j,itrans,itimlv) -
     +                                   solut(i-2,j,itrans,itimlv)
                     end do
                  end do

               else if ( iface.eq.idown ) then

                  do j = 0, 1-nvirtual, -1
                     do i = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i,j+1,itrans,itimlv) -
     +                                   solut(i,j+2,itrans,itimlv)
                     end do
                  end do

               else

                  do j = nj+1, nj+1+nvirtual
                     do i = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i,j-1,itrans,itimlv) -
     +                                   solut(i,j-2,itrans,itimlv)
                     end do
                  end do

               end if

            end if

         end do

      else

c     --- decoupled approach, using the essential boundary conditions
c         handle external subfaces

         ipibinf = ibipoint(iface)+nvirtual-1
         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

c           --- determine index range of uninitialized unknowns

               ilow = subfaces(iblock,iface,isubface,1)
               ihigh = subfaces(iblock,iface,isubface,2)

               if ( iface.eq.ileft) then

                  i = 0
                  do j = ilow, ihigh
                     ipoint = ipibinf+2*j
                     btype = ibinfor(ipoint)
                     if ( btype.eq.diri ) then
                        solut(0,j,itrans,itimlv) =
     +                             2d0*bndcon(1,ipoint) -
     +                                 solut(1,j,itrans,itimlv)
                     else
                        solut(i,j,itrans,itimlv) =
     +                             2d0*solut(i+1,j,itrans,itimlv) -
     +                                 solut(i+2,j,itrans,itimlv)
                     end if
                  end do
                  do i = -1, 1-nvirtual, -1
                     do j = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i+1,j,itrans,itimlv) -
     +                                   solut(i+2,j,itrans,itimlv)
                     end do
                  end do

               else if ( iface.eq.iright) then

                  i = ni+1
                  do j = ilow, ihigh
                     ipoint = ipibinf+2*j
                     btype = ibinfor(ipoint)
                     if ( btype.eq.diri ) then
                        solut(ni+1,j,itrans,itimlv) =
     +                             2d0*bndcon(1,ipoint) -
     +                                 solut(ni,j,itrans,itimlv)
                     else
                        solut(i,j,itrans,itimlv) =
     +                             2d0*solut(i-1,j,itrans,itimlv) -
     +                                 solut(i-2,j,itrans,itimlv)
                     end if
                  end do
                  do i = ni+2, ni+1+nvirtual
                     do j = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i-1,j,itrans,itimlv) -
     +                                   solut(i-2,j,itrans,itimlv)
                     end do
                  end do

               else if ( iface.eq.idown ) then

                  j = 0
                  do i = ilow, ihigh
                     ipoint = ipibinf+2*i
                     btype = ibinfor(ipoint)
                     if ( btype.eq.diri ) then
                        solut(i,j,itrans,itimlv) =
     +                             2d0*bndcon(1,ipoint) -
     +                                 solut(i,1,itrans,itimlv)
                     else
                        solut(i,j,itrans,itimlv) =
     +                             2d0*solut(i,j+1,itrans,itimlv) -
     +                                 solut(i,j+2,itrans,itimlv)
                     end if
                  end do
                  do j = -1, 1-nvirtual, -1
                     do i = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i,j+1,itrans,itimlv) -
     +                                   solut(i,j+2,itrans,itimlv)
                     end do
                  end do

               else

                  j = nj+1
                  do i = ilow, ihigh
                     ipoint = ipibinf+2*i
                     btype = ibinfor(ipoint)
                     if ( btype.eq.diri ) then
                        solut(i,j,itrans,itimlv) =
     +                             2d0*bndcon(1,ipoint) -
     +                                 solut(i,nj,itrans,itimlv)
                     else
                        solut(i,j,itrans,itimlv) =
     +                             2d0*solut(i,j-1,itrans,itimlv) -
     +                                 solut(i,j-2,itrans,itimlv)
                     end if
                  end do
                  do j = nj+2, nj+1+nvirtual
                     do i = ilow, ihigh
                        solut(i,j,itrans,itimlv) =
     +                               2d0*solut(i,j-1,itrans,itimlv) -
     +                                   solut(i,j-2,itrans,itimlv)
                     end do
                  end do

               end if

            end if

         end do

      end if

      end
