      subroutine isexup2b( ni, nj, nvirtual, ishift, jshift, vecy,
     +                       facs, vecx, iarea, alphad,
     +                       iconct, solut,
     +                       vector, ishift1, jshift1, invjac,
     +                       ishift2, jshift2)
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.1    date 27-08-1999 Duncan: move invjac to distribution
c                                       routines
c        version 1.0    date 29-10-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Perform a vector update for ISNaS over a 2-dimensional region
c    The vector to be added may be shifted with respect to the update vector
c    The update looks as follows:
c    y(i,j) = y(i,j) + a * x(i+ishift,j+jshift)
c    Both arrays are defined over the complete region, however y is
c    restricted to the region without virtual points and x uses virtual
c    points.
c    In order to get an optimal code the shift has been applied in the
c    declaration instead of the code, so that the code can use the same
c    indices for the vector to be updated as the vector to be added.
c    Since this is dangerous trick the user must be aware of this application
c    This subroutine is adapted to the new implementation of the convective
c    terms
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ishift, jshift, iarea(1:2,1:2),
     +        alphad, iconct, ishift1 ,jshift1, ishift2 ,jshift2
      double precision vecy(ni+1,nj+1), facs,
     +                 vecx(1-nvirtual-ishift:ni+nvirtual+1-ishift,
     +                      1-nvirtual-jshift:nj+nvirtual+1-jshift),
     +                 invjac(1-nvirtual-ishift-ishift2:
     +                                 ni+nvirtual+1-ishift-ishift2,
     +                        1-nvirtual-jshift-jshift2:
     +                                 nj+nvirtual+1-jshift-jshift2),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual, 2, *),
     +                 vector(1-nvirtual-ishift1:ni+nvirtual+1-ishift1,
     +                        1-nvirtual-jshift1:nj+nvirtual+1-jshift1)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     iconct    i    Loop variable for loop over all connections in the matrx
c     invjac    i    1/sqrt(g) in pnttyp
c     ishift    i    Shift in i-direction
c     ishift1   i    Shift in i-direction
c     ishift2   i    Shift in i-direction
c     jshift    i    Shift in j-direction
c     jshift1   i    Shift in j-direction
c     jshift2   i    Shift in j-direction
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     vector    i    Input flow field
c     vecx      i    Vector to be added
c     vecy     i/o   Vector to be updated
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, ashift, ipshift(2,13), jpshift(2,13)

c     ashift    General shift parameter
c     i         General loop variable
c     ipshift   Shift parameter in stencil
c     j         General loop variable
c     jpshift   Shift parameter in stencil
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( iconct.le.9 ) then
         ashift = 0
      else
         if ( alphad.eq.1 ) then
            ashift = 1
         else
            ashift =-1
         end if
      end if

      ipshift(1,1) = 0
      ipshift(1,2) = -1
      ipshift(1,3) = 0
      ipshift(1,4) = 1
      ipshift(1,5) = -1
      ipshift(1,6) = 1
      ipshift(1,7) = -1
      ipshift(1,8) = 0
      ipshift(1,9) = 1
      ipshift(1,10) = -1
      ipshift(1,11) = 0
      ipshift(1,12) = -1
      ipshift(1,13) = 0

      jpshift(1,1) = 0
      jpshift(1,2) = -1
      jpshift(1,3) = -1
      jpshift(1,4) = -1
      jpshift(1,5) = 0
      jpshift(1,6) = 0
      jpshift(1,7) = 1
      jpshift(1,8) = 1
      jpshift(1,9) = 1
      jpshift(1,10) = 0
      jpshift(1,11) = 0
      jpshift(1,12) = 1
      jpshift(1,13) = 1

      do i = 1, 9
         ipshift(2,i) = ipshift(1,i)
         jpshift(2,i) = jpshift(1,i)
      end do

      ipshift(2,10) = 0
      ipshift(2,11) = 1
      ipshift(2,12) = 0
      ipshift(2,13) = 1

      jpshift(2,10) = -1
      jpshift(2,11) = -1
      jpshift(2,12) = 0
      jpshift(2,13) = 0

      do j = iarea(1,2), iarea(2,2)
        do i = iarea(1,1), iarea(2,1)
           vecy(i,j) =  vecy(i,j)-facs*vecx(i,j)*invjac(i,j)*
     +               solut(i+ipshift(alphad,iconct),
     +                      j+jpshift(alphad,iconct),
     +                     alphad+ashift,1)*vector(i,j)
        end do
      end do

      end
