      subroutine isexvpe3(iperiod, itrans, ni, nj, nk, ndim,
     +                    nvirtual, ndegfd, itimlv, ntimlv, solut,
     +                    iblock, nblocks, nfaces, maxnsubf,
     +                    subfaces)
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.5    date   06-08-1998 Guus: remove cisnas
c        version 1.4    date   27-03-1995 Kees K: itrans+1 for ndim=3
c        version 1.3    date   30-03-1995 Michiel: changed for periodic bc
c        version 1.2    date   28-03-1995 Erik: small error corrected
c        version 1.1    date   24-01-1995 Erik: corrected error in the
c                                         range for ilow and ihigh
c        version 1.0    date   11-11-1994 (copy of isexco02 for fluxes)
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize virtual unknowns in 3D for periodic bc, only valid
c     in one block geometry, but easily adaptable to multi block...
c     The vertices and edges are only filled by isexvpe3 when all
c     connecting boundaries are periodic, otherwise the virtual cells
c     are filled by extrapolation in ISEX03
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndegfd, ni, nj, nk, nvirtual, itimlv, ntimlv, iperiod,
     +        ndim, maxnsubf, nfaces, iblock, itrans, nblocks
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-(ndim-2)*nvirtual:nk+1+(ndim-2)*nvirtual,
     +                       1:ndegfd, 1:ntimlv)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     iperiod   i    Parameter indicating if periodical boundary conditions
c                    are present per boundary
c                    iperiod is a 3 digit number with the sequence 321
c                    Each digit refers to 1 direction
c                    The digits have the following meaning
c                    0:  no periodical boundary conditions
c                    1:  symmetrical periodical boundary conditions
c                    2:  anti-symmetrical periodical boundary conditions
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, l, ihigh, ilow, jhigh, jlow, j, k, idegfd,
     +        lface, rface, ifi, kdegfd, hdegfd, lsubface, rsubface,
     +        lblock, rblock, ilop, jlop, jfi, kfi
      logical period1, period2, period3

c     hdegfd    Highest degree of freedom that is extrapolated
c     i         General loop variable
c     idegfd    Loop variable with respect to a given unknown
c     ifi       Starting value of general loop variable i
c     ihigh     Upper index of subface
c     ilop      Lower index of opposite subface
c     ilow      Lower index of subface
c     j         General loop variable
c     jfi       Starting value of general loop variable j
c     jhigh     Upper j-index for a subface
c     jlop      Lower index of opposite subface
c     jlow      Lower j-index for a subface
c     k         General loop variable
c     kdegfd    Lowest degree of freedom that is extrapolated
c     kfi       Starting value of general loop variable k
c     l         General loop variable
c     lblock    Left block sequence number
c     lface     Left face number
c     lsubface  Left subface number
c     period1   If true there are periodical boundary conditions in the
c               1 direction
c     period2   If true there are periodical boundary conditions in the
c               2 direction
c     period3   If true there are periodical boundary conditions in the
c               3 direction
c     rblock    Right block sequence number
c     rface     Right face number
c     rsubface  Right subface number
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      period1 = .false.
      period2 = .false.
      period3 = .false.
      if ( mod(iperiod,10).eq.1 ) period1 = .true.
      if ( mod(iperiod/10,10).eq.1 ) period2 = .true.
      if ( mod(iperiod/100,10).eq.1 ) period3 = .true.

      do l=1, ndim

c     --- determine if periodic boundary is within block IBLOCK

         if ( l.eq.1 .and. period1 ) then

            lblock   = iblock
            rblock   = iblock
            lface    = 1
            rface    = 2
            lsubface = 1
            rsubface = 1

         else if ( l.eq.2 .and. period2 ) then

            lblock   = iblock
            rblock   = iblock
            lface    = 3
            rface    = 4
            lsubface = 1
            rsubface = 1

         else if ( l.eq.3 .and. period3 ) then

            lblock   = iblock
            rblock   = iblock
            lface    = 5
            rface    = 6
            lsubface = 1
            rsubface = 1

         end if

         if ( l.eq.1 .and. period1 .or.
     +        l.eq.2 .and. period2 .or.
     +        l.eq.3 .and. period3    ) then

c        --- determine index range of uninitialized unknowns

            ilow  = subfaces(lblock,lface,lsubface,1)
            ilop  = subfaces(rblock,rface,rsubface,1)
            ihigh = subfaces(lblock,lface,lsubface,2)
            jlow  = subfaces(lblock,lface,lsubface,3)
            jlop  = subfaces(rblock,rface,rsubface,3)
            jhigh = subfaces(lblock,lface,lsubface,4)

c           --- determine index range for unknowns

            if ( itrans.eq.1 ) then

c           --- assume solut, tangential fluxes across the boundary,
c               followed by normal fluxes across the boundary

               kdegfd = 1
               hdegfd = ndim

            else

               kdegfd = itrans
               hdegfd = itrans

            end if

c           --- handle external subfaces

            do idegfd = kdegfd, hdegfd

               if ( idegfd.lt.4 ) then
               if ( idegfd.ne.1 .and. lface.le.2             .or.
     +              idegfd.ne.2 .and. lface.ge.3 .and. lface .le.
     +              4 .or.
     +              idegfd.ne.3 .and. lface.le.5           ) then

c              --- tangential flux component

                    ihigh = ihigh + 1
                    jhigh = jhigh + 1

               end if
               end if

c              --- We can only handle periodic bc's for opposite boundaries

               if ( lface.eq.1 ) then

                  if ( idegfd.eq.1 ) then
                     ifi = ni + 2
                  else
                     ifi = ni + 1
                  end if
                  do i = 0, 1-nvirtual, -1
                     do j = ilow, ihigh
                        do k = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(ni+i,j-ilow+ilop,k-jlow+jlop,
     +                              idegfd,itimlv)
                           solut(ifi-i,j-ilow+ilop,k-jlow+jlop,
     +                              idegfd,itimlv)=
     +                        solut(ifi-ni-i,j,k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( lface.eq.2  ) then

                  if ( idegfd.eq.1 ) then
                     ifi = ni + 2
                  else
                     ifi = ni + 1
                  end if
                  do i = ifi, ni+1+nvirtual
                     do j = ilow, ihigh
                        do k = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-ni,j-ilow+ilop,k-jlow+jlop,
     +                              idegfd,itimlv)
                           solut(ifi-i,j-ilow+ilop,k-jlow+jlop,
     +                              idegfd,itimlv) =
     +                        solut(ifi-i+ni,j,k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( lface.eq.3 ) then

                  if ( idegfd.eq.2 ) then
                     ifi = nj + 2
                  else
                     ifi = nj + 1
                  end if
                  do j = 0, 1-nvirtual, -1
                     do k = ilow, ihigh
                        do i = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-jlow+jlop,nj+j,k-ilow+ilop,
     +                              idegfd,itimlv)
                           solut(i-jlow+jlop,ifi-j,k-ilow+ilop,
     +                              idegfd,itimlv)=
     +                        solut(i,ifi-nj-j,k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( lface.eq.4 ) then

                  if ( idegfd.eq.2 ) then
                     ifi = nj + 2
                  else
                     ifi = nj + 1
                  end if
                  do j = ifi, ifi+nvirtual
                     do k = ilow, ihigh
                        do i = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-jlow+jlop,j-nj,k-ilow+ilop,
     +                              idegfd,itimlv)
                           solut(i-jlow+jlop,ifi-j,k-ilow+ilop,
     +                              idegfd,itimlv) =
     +                        solut(i,ifi+nj-j,k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( lface.eq.5 ) then

                  if ( idegfd.eq.3 ) then
                     ifi = nk + 2
                  else
                     ifi = nk + 1
                  end if
                  do k = 0, 1-nvirtual, -1
                     do i = ilow, ihigh
                        do j = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-ilow+ilop,j-jlow+jlop,nk+k,
     +                              idegfd,itimlv)
                           solut(i-ilow+ilop,j-jlow+jlop,ifi-k,
     +                              idegfd,itimlv)=
     +                        solut(i,j,ifi-nk-k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( lface.eq.6 ) then

                  if ( idegfd.eq.3 ) then
                     ifi = nk + 2
                  else
                     ifi = nk + 1
                  end if
                  do k = ifi, nk+1+nvirtual
                     do i = ilow, ihigh
                        do j = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-ilow+ilop,j-jlow+jlop,k-nk,
     +                              idegfd,itimlv)
                           solut(i-ilow+ilop,j-jlow+jlop,ifi-k,
     +                              idegfd,itimlv) =
     +                        solut(i,j,ifi+nk-k,idegfd,itimlv)
                        end do
                     end do
                  end do

               end if

c              --- treat edges ( bilinear extrapolation )

               if ( l.eq.1 .and. period1 .and. period2 ) then

                  if ( idegfd.eq.1 ) then
                     ifi = ni + 2
                  else
                     ifi = ni + 1
                  end if
                  if ( idegfd.eq.2 ) then
                     jfi = nj + 2
                  else
                     jfi = nj + 1
                  end if
                  do i = 0, 1-nvirtual,-1
                     do j = 0, 1-nvirtual,-1
                        do k = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(ni+i,nj+j,k-jlow+jlop, idegfd,
     +                              itimlv)
                           solut(ifi-i,jfi-j,k-jlow+jlop, idegfd,
     +                              itimlv) =
     +                        solut(ifi-ni-i,jfi-ni-j,k,idegfd,itimlv)
                           solut(i,jfi-j,k,idegfd,itimlv) =
     +                        solut(ni+i,jfi-ni-j,k-jlow+jlop, idegfd,
     +                              itimlv)
                           solut(ifi-i,j,k-jlow+jlop, idegfd,itimlv)=
     +                        solut(ifi-ni-i,nj+j,k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( l.eq.2 .and. period2 .and. period3 ) then

                  if ( idegfd.eq.2 ) then
                     ifi = nj + 2
                  else
                     ifi = nj + 1
                  end if
                  if ( idegfd.eq.3 ) then
                     jfi = nk + 2
                  else
                     jfi = nk + 1
                  end if
                  do j = 0, 1-nvirtual,-1
                     do k = 0, 1-nvirtual,-1
                        do i = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(i-jlow+jlop,nj+j,nk+k, idegfd,
     +                              itimlv)
                           solut(i-jlow+jlop,ifi-j,jfi-k, idegfd,
     +                              itimlv) =
     +                        solut(i,ifi-nj-j,jfi-nk-k,idegfd,itimlv)
                           solut(i,ifi-j,k,idegfd,itimlv) =
     +                        solut(i-jlow+jlop,ifi-nj-j,nk+k, idegfd,
     +                              itimlv)
                           solut(i-jlow+jlop,j,jfi-k, idegfd,itimlv)=
     +                        solut(i,nj+j,jfi-nk-k,idegfd,itimlv)
                        end do
                     end do
                  end do

               else if ( l.eq.3 .and. period1 .and. period3 ) then

                  if ( idegfd.eq.3 ) then
                     ifi = nk + 2
                  else
                     ifi = nk + 1
                  end if
                  if ( idegfd.eq.1 ) then
                     jfi = ni + 2
                  else
                     jfi = ni + 1
                  end if
                  do i = 0, 1-nvirtual,-1
                     do k = 0, 1-nvirtual,-1
                        do j = jlow, jhigh
                           solut(i,j,k,idegfd,itimlv) =
     +                        solut(ni+i,j-jlow+jlop,nk+k, idegfd,
     +                              itimlv)
                           solut(jfi-i,j-jlow+jlop,ifi-k, idegfd,
     +                              itimlv) =
     +                        solut(jfi-ni-i,j,ifi-nk-k,idegfd,itimlv)
                           solut(i,j,ifi-k,idegfd,itimlv) =
     +                        solut(ni+i,j-jlow+jlop,ifi-nk-k, idegfd,
     +                              itimlv)
                           solut(jfi-i,j-jlow+jlop,k, idegfd,itimlv)=
     +                        solut(jfi-ni-i,j,nk+k,idegfd,itimlv)
                        end do
                     end do
                  end do

               end if

            end do

         end if

         if ( period1 .and. period2 .and. period3 ) then

            do idegfd = kdegfd, hdegfd

               if ( idegfd.eq.1 ) then
                  ifi = ni + 2
               else
                  ifi = ni + 1
               end if
               if ( idegfd.eq.2 ) then
                  jfi = nj + 2
               else
                  jfi = nj + 1
               end if
               if ( idegfd.eq.3 ) then
                  kfi = nk + 2
               else
                  kfi = nk + 1
               end if
               do i = 0, 1-nvirtual,-1
                  do j = 0, 1-nvirtual,-1
                     do k = 0, 1-nvirtual,-1
                        solut(i,j,k,idegfd,itimlv) =
     +                     solut(ni+i,nj+j,nk+k, idegfd, itimlv)
                        solut(ifi-i,jfi-j,kfi-k, idegfd, itimlv) =
     +                     solut(ifi-ni-i,jfi-nj-j,kfi-nk-k,idegfd,
     +                           itimlv)
                        solut(ifi-i,j,k,idegfd,itimlv) =
     +                     solut(ifi-ni-i,nj+j,nk+k, idegfd,itimlv)
                        solut(i,jfi-j,k,idegfd,itimlv) =
     +                     solut(ni+i,jfi-nj-j,nk+k, idegfd,itimlv)
                        solut(i,j,kfi-k,idegfd,itimlv) =
     +                     solut(ni+i,nj+j,kfi-nk-k, idegfd,itimlv)
                        solut(ifi-i,jfi-j,k,idegfd,itimlv) =
     +                     solut(ifi-ni-i,jfi-nj-j,nk+k, idegfd,
     +                           itimlv)
                        solut(ifi-i,j,kfi-k,idegfd,itimlv) =
     +                     solut(ifi-ni-i,nj+j,kfi-nk-k, idegfd,
     +                           itimlv)
                        solut(i,jfi-j,kfi-k,idegfd,itimlv) =
     +                     solut(ni+i,jfi-nj-j,kfi-nk-k, idegfd,
     +                           itimlv)
                     end do
                  end do
               end do

            end do
         end if

      end do

      end
