      subroutine isfilb ( iequa, iinput, rinput, infbnd, bndcon, coor,
     +                    solut, coefs, eddy, geom,  ndim, iseqpres,
     +                    work )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      4.3   date    06-07-1999 Guus: new type of energy equation
c     version      4.2   date    06-08-1998 Guus: new call to isfilb4
c     version      4.1   date    26-11-1997 Guus: new calls to isfilb2/3
c     version      4.0   date    05-10-1997 Guus: adaptation for multi-phase
c                                           gas flow
c     version      3.0   date    10-09-1997 Charles extra parameter iseqpres
c     version      2.4   date    04-06-1996 Marcel extension parameter list
c                                                  isfilb2 and isfilb3
c     version      2.3   date    10-04-1996 Kees K call isfilb3 changed
c     version      2.2   date    27-03-1996 Kees K itimeact added, for
c                                           filling bndcon only the first
c                                           time step
c     version      2.1   date    13-02-1996 Minor change layout
c     version      2.0   date    14-12-1995 Marcel remove isinbc3
c     version      1.9   date    28-08-1995 Guus, complete update for 2D
c     version      1.8   date    21-04-1995 RvN virtual point test version
c     version      1.7   date    31-01-1995 Marcel new calls (isfilb2 and
c                                           isfilb3)
c     version      1.6   date    25-01-1995 Marcel extension parameter list
c     version      1.5   date    03-01-1995 Marcel extension transport +
c                                           turbulence 3D
c
c   copyright (c) 1993-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     routine which fills arrays infbnd and bndcon
c
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), infbnd(*), iequa, ndim, iseqpres
      double precision rinput(*), bndcon(ndim,*), coor(*), geom(*),
     +                 solut(*), coefs(*), eddy(*), work(*)

c     bndcon         o    Array BNDCON is used to indicate the value of the
c                         boundary conditions for each point at the boundary of
c                         each single block for each unknown.
c                         For a description see the programmers guide
c     coefs          i    COEFS(i,k) contains the k-th coefficient in
c                         centroid i. The nodes are ordered in exactly the
c                         same way as the scalar equations in SOLUT.
c                         The coefficients array is extended with virtual
c                         cells in exactly the same way as the locally
c                         copied solution array is.
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     geom           i    The geometrical quantities are only stored in the
c                         one-block solvers. These arrays are treated as
c                         temporary arrays, which means that they are
c                         created in the one-block solver and destroyed if
c                         the next block or step is considered. At present
c                         the storage is restricted to the storage of
c                         sqrt(g) and 1/sqrt(g) in all points of the grid.
c                         Exactly the same cells as for the co-ordinates are
c                         used.
c                         For a description see the programmers guide
c     iequa          i    General equation sequence number
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     infbnd         o    Array infbnd is used to indicate the type of boundary
c                         conditions for each point at the boundary of each
c                         single block for each unknown.
c                         For a description see the programmers guide
c     iseqpres       i    Sequence number of pressure equation
c     ndim           i    Dimension of space (2 or 3)
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c     work           i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinbc, ipiintur, iprinbc, iprintim, iprintur, vartyp,
     +        ni, nj, nk, nturb, ncoefs, ndegfd, pnttyp, menergy,
     +        maxb, nvirtual, mturb, viscorr, mequat, ipiincom,
     +        ipinvjac, npointsv, ipibindex, nxmax, acttyp,
     +        ipibinfor, iplrinbc, ipinfface, ipindface, iprcof,
     +        ibt1lo, ibt1hi, ibt2lo, ibt2hi, iface, itimeact, ipiintim
      double precision time

c     acttyp         Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     ibt1hi         Upper bound of first index in tangential direction
c     ibt1lo         Lower bound of first index in tangential direction
c     ibt2hi         Upper bound of second index in tangential direction
c     ibt2lo         Lower bound of second index in tangential direction
c     iface          Face number
c     ipibindex      Starting address of second part of infbnd in infbnd
c     ipibinfor      Starting address of ibinfor in infbnd
c     ipiinbc        Starting address of array iinbc in array iinput
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiintim       Starting address of array IINTIM in array IINPUT
c     ipiintur       Starting address of array IINTUR in array IINPUT
c     ipindface      Starting address of face in infbnd
c     ipinfface      Starting address of information of face in infbnd
c     ipinvjac       Starting address of array invjac in geom
c     iplrinbc       Start address
c     iprcof         Starting address of array rincof in array rinput
c     iprinbc        Starting address of array rinbc in array iinput
c     iprintim       Starting address of array RINTIM in array RINPUT
c     iprintur       Starting address of array rintur in array RINPUT
c     itimeact       Actual time step
c     maxb           Maximum number of real values for one boundary
c                    condition
c     menergy        Defines the type of energy equation used.
c                    Possible values:
c                    0: none
c                    1: the primary unknown is the enthalpy
c                    2: the primary unknown is the total enthalpy
c                    3: the primary unknown is the internal energy
c     mequat         Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mturb          Indicates type of turbulence modelling
c     ncoefs         Number of coefficients
c     ndegfd         Number of degrees of freedom
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npointsv       Number of points in the mesh with virtual points
c     nturb          Number of extra transport equations for turbulence
c     nvirtual       Number of rows of virtual cells
c     nxmax          Maximum of ni, nj, nk
c     pnttyp         Point where a value or finite difference is
c                    to be calculated within the cell
c                    For a description see the programmers guide
c     time           Actual time of time step
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c     viscorr        Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISFILB2        Actually fills array infbnd and bndcon (2D)
c     ISFILB3        Actually fills array infbnd and bndcon (3D)
c     ISFILB4        Mark points in infbnd to be outside the region
c     ISFILB5        Copy information for virtual parts of infbnd and bndcon
c                    in case of periodical boundary conditions (2D)
c     ISINTCF2       Compute interpolation or finite differences of a
c                    coefficient in an integration point
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      get pointers of the arrays iinbc, rinbc and rintim
c      get information from array iinput
c      call routine isfilbx, which actualy fills arrays infbnd and bndcon
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isfilb' )

      ipiintim  = iinput(4)

      ipiinbc  = iinput(2)
      ipiintur = iinput(8)
      ipiincom = iinput(9)
      iprcof   = iinput(26)
      iprinbc  = iinput(27)
      iprintim = iinput(29)
      iprintur = iinput(31)
      time     = rinput(iprintim)

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      maxb     = iinput(61)
      ncoefs   = iinput(66)

      nturb    = iinput(ipiintur -1 + 1)
      mturb    = iinput(ipiintur -1 + 2)
      viscorr  = iinput(ipiintur -1 + 3)
      mequat   = iinput(ipiincom+2)
      menergy  = iinput(ipiincom-1+10)

      itimeact = iinput(ipiintim -1 + 4)

      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      if (ndim.eq.3) then
         npointsv = npointsv*(nk+1+2*nvirtual)
      end if

c     --- start address for invjac in array geom

      ipinvjac = npointsv*(2**ndim) + 1

      if ( ndim.eq.2 ) then

c     --- ndim = 2, fill infbnd and bndcon

         infbnd(1) = 1
         infbnd(2) = (nj+nvirtual)*2+1 + infbnd(1)
         infbnd(3) = (nj+nvirtual)*2+1 + infbnd(2)
         infbnd(4) = (ni+nvirtual)*2+1 + infbnd(3)

c        --- bounds for face 1

         infbnd(5 ) = 1-nvirtual
         infbnd(6 ) = 2*nj+1+nvirtual

c        --- bounds for face 2

         infbnd(7 ) = 1-nvirtual
         infbnd(8 ) = 2*nj+1+nvirtual

c        --- bounds for face 3

         infbnd(9 ) = 1-nvirtual
         infbnd(10) = 2*ni+1+nvirtual

c        --- bounds for face 4

         infbnd(11) = 1-nvirtual
         infbnd(12) = 2*ni+1+nvirtual

         iplrinbc = 0

         if ( mequat.eq.1 ) then

c        --- two-phase gas flow, interpolate eps from coefs
c            Store in work

            acttyp = 0
            pnttyp = 1
            vartyp = 3
            call isintcf2 ( coefs(5*npointsv+1), work, ni, nj,
     +                      nvirtual, vartyp, acttyp, pnttyp, 0, coor )

         end if
         if  ( menergy.eq.3 ) then

c        --- Compressible flow; the energy equation is formulated in terms of
c            the internal energy e
c            Interpolate rho to the vertices and store in work

            acttyp = 0
            pnttyp = 1
            vartyp = 3
            call isintcf2 ( eddy(npointsv+1), work, ni, nj,
     +                      nvirtual, vartyp, acttyp, pnttyp, 0, coor )
         end if

         do iface = 1, 4

            call isfilb2 ( iinput(ipiinbc), rinput(iprinbc), iface,
     +                     infbnd(12+infbnd(iface)), rinput(iprcof),
     +                     bndcon(1,infbnd(iface)), coor, ni,
     +                     nj, ncoefs, ndegfd,
     +                     iequa, nvirtual, time, iplrinbc, coefs,
     +                     eddy, solut, mturb, geom(ipinvjac),
     +                     viscorr, rinput(iprintur), itimeact,
     +                     iseqpres, mequat, work, iinput(ipiincom) )

         end do

c        --- Copy information to virtual points in case of periodical
c            boundary conditions

         do iface = 1, 4

            call isfilb5 (  ni, nj, nvirtual, iface, infbnd, infbnd(13),
     +                      bndcon )
         end do

      else

c     --- ndim = 3, fill infbnd and bndcon

        nxmax      = max(ni,nj,nk)

        infbnd(1 ) = 1
        infbnd(2 ) = 1*(2*nxmax+1+2*nvirtual)**2 + 1
        infbnd(3 ) = 2*(2*nxmax+1+2*nvirtual)**2 + 1
        infbnd(4 ) = 3*(2*nxmax+1+2*nvirtual)**2 + 1
        infbnd(5 ) = 4*(2*nxmax+1+2*nvirtual)**2 + 1
        infbnd(6 ) = 5*(2*nxmax+1+2*nvirtual)**2 + 1

c       --- bounds for face 1

        infbnd(7 ) = 1-nvirtual
        infbnd(8 ) = 2*nj+1+nvirtual
        infbnd(9 ) = 1-nvirtual
        infbnd(10) = 2*nk+1+nvirtual

c       --- bounds for face 2

        infbnd(11) = 1-nvirtual
        infbnd(12) = 2*nj+1+nvirtual
        infbnd(13) = 1-nvirtual
        infbnd(14) = 2*nk+1+nvirtual

c       --- bounds for face 3

        infbnd(15) = 1-nvirtual
        infbnd(16) = 2*nk+1+nvirtual
        infbnd(17) = 1-nvirtual
        infbnd(18) = 2*ni+1+nvirtual

c       --- bounds for face 4

        infbnd(19) = 1-nvirtual
        infbnd(20) = 2*nk+1+nvirtual
        infbnd(21) = 1-nvirtual
        infbnd(22) = 2*ni+1+nvirtual

c       --- bounds for face 5

        infbnd(23) = 1-nvirtual
        infbnd(24) = 2*ni+1+nvirtual
        infbnd(25) = 1-nvirtual
        infbnd(26) = 2*nj+1+nvirtual

c       --- bounds for face 6

        infbnd(27) = 1-nvirtual
        infbnd(28) = 2*ni+1+nvirtual
        infbnd(29) = 1-nvirtual
        infbnd(30) = 2*nj+1+nvirtual

        iplrinbc = 0

        do iface = 1, 6

           call isfilb3(iinput(ipiinbc), rinput(iprinbc), iface,
     +                  infbnd(2*ndim+(iface-1)*4+1),
     +                  infbnd(2*ndim+(iface-1)*4+2),
     +                  infbnd(2*ndim+(iface-1)*4+3),
     +                  infbnd(2*ndim+(iface-1)*4+4),
     +                  infbnd(31-1+infbnd(iface)),
     +                  bndcon(1,infbnd(iface)), coor, maxb, ni,
     +                  nj, nk, ndim, nturb, ncoefs,
     +                  ndegfd, iequa, nvirtual, time, iplrinbc,
     +                  coefs, eddy, solut, mturb,
     +                  viscorr, rinput(iprintur), rinput(iprcof),
     +                  itimeact, iseqpres)

        end do

c       --- Fill the positions in infbnd that are still zero
c           with
c           Initial values for pointers to parts 1,2 and 3 of infbnd

         ipibindex = 1  + 2*ndim
         ipibinfor = ipibindex + 2*(ndim-1)*2*ndim

         do iface=1,2*ndim

c        --- Pointer to face data

            ipinfface = infbnd(iface)

c           --- Pointer to face boundaries

            ipindface = ipibindex+(iface-1)*2*(ndim-1)

            ibt1lo = infbnd(ipindface-1+1)
            ibt1hi = infbnd(ipindface-1+2)
            ibt2lo = infbnd(ipindface-1+2+1)
            ibt2hi = infbnd(ipindface-1+2+2)

            call isfilb4 ( ibt1lo, ibt1hi, ibt2lo, ibt2hi,
     +                     infbnd(ipibinfor-1+ipinfface) )

         end do

      end if

      call iserclos ( 'isfilb' )

      end
