      subroutine isfilc (iequa, nblocks , isiinput, isrinput, iscoefs ,
     +                   issolut, iscoor, isgeom  , iseddy  , isitopol,
     +                   iadmin, isigeom, isaverstr )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      3.5   date 21-09-1999 Guus: new call to ismblk
c     version      3.4   date 23-11-1998 Jason: remove calls to isini* isterm*
c     version      3.3   date 31-08-1998 Jason: remove check nodeid
c     version      3.2   date 05-01-1998 Guus: extend ncoefs to 10
c     version      3.1   date 20-11-1997 Guus: new call to isfilvcf
c     version      3.0   date 15-09-1997 Fred T : variable igeom added
c     version      2.3   date 11-09-1997 Charles: extra parameters icoupled,
c                                        iseqpres
c     version      2.2   date 10-06-1997 Guus: common cisnas
c     version      2.1   date 29-11-1994 Kees: call of isfilvcf added
c     version      2.0   date 22-11-1994 Erik: extension with multi-block
c                                        exchange of virtual coefficients
c     version      1.0   date 01-11-1994 Kees: complete revision
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill coefficients
c
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isiinput(*), isrinput(*), iscoefs(*), issolut(*),
     +        iscoor(*), isgeom(*), iseddy(*), iadmin(*),
     +        isitopol, isigeom(*), isaverstr(*)
      integer iequa, nblocks

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isaverstr i    Array of sequence numbers (nblocks) of array averstr
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isigeom   i    Array of sequence numbers (nblocks) of array igeom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer dummy, icoef, icoef1, icoef2, ilev, ind, iblock, mequat,
     +        ipiinput, itrans, ndim, iseqpres, icoupled, ipiincom
      logical ddummy

c     ddummy    dummy logical
c     dummy     Dummy
c     iblock    Block sequence number
c     icoef     Sequence number of coefficient
c     icoef1    Sequence number of coefficient
c     icoef2    Sequence number of coefficient
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ilev      Time level of the multi-block transport of information
c     ind       Sequence number of node
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     iseqpres  Sequence number of pressure equation
c     itrans    reserved
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     ndim      Dimension of space (2 or 3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILC01  fill coefficients
c     ISFILVCF  extrapolates the virtual points of the local coefficients
c               arrays of the subdomains for outer boundaries
c     ISINGETP  get the pointer corresponding to an integer array
c     ISMBLK    Performs the local transport of multi-block information
c               within a node
c     ISRCDDHS  Receives the multi-block information from each node
c               and place it at the correct location in the arrays
c               of virtual unknowns.
c     ISRCDDND  receives the virt arrays of virtual unknowns for all
c               local blocks at a node and puts them into the local
c               virt arrays
c     ISSNDDHS  Copy the arrays of virtual unknowns for all
c               blocks to the local copies at the nodes, using PVM
c               if necessary.
c     ISSNDDND  Copy unknowns near or on the boundary of the subdomain
c               solut arrays to the virt arrays of the neighbouring
c               blocks on the host
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      Level 1 pseudo code
c      -------------------
c
c      fill (at least) the inner region of the coefficients
c
c      ensure that the virtual regions of all coefficients arrays
c      of ALL blocks are filled on the host.
c      This means:
c         - transport coefficients of the local blocks if on the
c           host
c         - receive virtual coefficients if on the host
c         - send coefficients if on a node
c
c      Send virtual coefficients back to the nodes.
c
c      Level 2 pseudo code
c      -------------------
c
c      --- fill (at least) the inner region of the coefficients
c
c      for iblock = 1 (1) nblocks do
c
c         if block(iblock) on this node  then
c
c            fill coefficients
c
c         end if
c
c      end do
c
c      --- determine the range of coefficients
c
c      if ( momentum or pressure equation ) then
c
c         range = 1..5
c
c      else
c
c         range = 1..3*ndim
c
c      end if
c
c      --- transport coefficients of the local blocks if on the host
c
c      if ( on the host ) then
c
c         do iblock = 1, nblocks
c
c            if ( iblock solved on the host ) then
c
c               transport coefficients using ismblk(...)
c
c            end if
c
c         end do
c
c      end if
c
c      --- receive coefficients if on the host, otherwise send
c          coefficients
c
c      if ( on the host ) then
c
c         receive coefficients using isrcddhs(...)
c
c      else
c
c         send coefficients using issnddnd(...)
c
c      end if
c
c      --- send coefficients if on the host, otherwise receive
c          coefficients
c
c      if ( on the host ) then
c
c         send coefficients using issnddhs(...)
c
c      else
c
c         receive coefficients using isrcddnd(...)
c
c      end if
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isfilc')

c     --- check if coupled or not

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiincom = ibuffr(ipiinput +  9 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      mequat   = ibuffr(ipiinput-1+ipiincom+2)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim+1
      end if

c     --- fill the inner region of the coefficients arrays

      do iblock = 1, nblocks

         if( iadmin(iblock).eq.inodenr) then

            iblockrf = iblock
            call isfilc01 ( iequa, isiinput(iblock), isrinput(iblock),
     +                      iscoefs(iblock), issolut(iblock),
     +                      iscoor(iblock), isgeom(iblock) ,
     +                      iseddy(iblock), isitopol, iseqpres,
     +                      isigeom(iblock), isaverstr(iblock) )

         end if

      end do
      if ( nerror.gt.0 ) go to 1000

c     --- fill the virtual region by exchanging information
c         first determine the range of coefficients to be
c         exchanged

      if ( iequa.le.iseqpres ) then

c     --- momentum or pressure

         icoef1 = 1
         icoef2 = 5
         if ( mequat.eq.1 ) icoef2 = 10

      else

c     --- transport or turbulence

         icoef1 = 1
         icoef2 = 3*ndim

      end if

c     --- place coefficients of all local blocks into the virtual regions
c         of coefficients arrays of neighbouring blocks if on the host.

      if ( inodenr.eq.ISHOST ) then
         do iblock = 1, nblocks
            iblockrf = iblock
            do icoef = icoef1, icoef2
               if ( iadmin(iblock).eq.inodenr) then

                  itrans = -icoef
                  ilev   = -1

c                 --- ismblk is called with -ilev to indicate that nvirtual
c                     cells must be copied

                  call ismblk(iblock, itrans, ilev, isiinput, iscoefs,
     +                        iadmin, isitopol)

               end if
            end do
         end do
      end if
      if ( nerror.gt.0 ) go to 1000

c     --- exchange coefficients with blocks on other nodes

      if ( inodenr.eq.ISHOST ) then

c     --- receive coefficients if on the host

         do ind = 1, iacnodes

            do icoef = icoef1, icoef2

c           --- receive coefficients

               call isrcddhs( dummy, isiinput, iscoefs, isitopol )

            end do

         end do

      else

c     --- send coefficients on the node

         ind = 0

         do icoef = icoef1, icoef2

c        --- send coefficients

            itrans = -icoef
            ilev   = 1
            call issnddnd(itrans, ilev,   isiinput, iscoefs,
     +                    iadmin, isitopol)

         end do

      end if
      if ( nerror.gt.0 ) go to 1000

c     --- send (host) or receive (node) coefficients

      if ( inodenr.eq.ISHOST ) then

c     --- send coefficients if on the host

         do ind = 1, iacnodes

            do icoef = icoef1, icoef2

c           --- send fluxes

               ddummy = .false.
               itrans = -icoef
               ilev   = 1
               call issnddhs(nblocks, itrans, ilev, ind, isiinput,
     +                       iscoefs, isitopol, iadmin, ddummy)

            end do

         end do

      else

c     --- receive coefficients if on the node

         do icoef = icoef1, icoef2

c        --- receive coefficients

            call isrcddnd( nblocks, isiinput, iscoefs, isitopol,
     +                     iadmin, ddummy)

         end do

      end if
      if ( nerror.gt.0 ) go to 1000

c     --- fill the outer region of the coefficients arrays

      if ( iequa.ge.iseqpres ) mequat = 0
      do iblock = 1, nblocks

         if( iadmin(iblock).eq.inodenr) then

            iblockrf = iblock
            call isfilvcf ( iblock, nblocks, isiinput(iblock),
     +                      iscoefs(iblock), isitopol, mequat )

         end if

      end do

1000  call iserclos('isfilc')

      end
