      subroutine isfilg ( nblocks, isiinput, isrinput, iscoor,
     +                    isgeom, isigeom, iadmin )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.3   date    04-06-1998 Charles: isfilg cleaned
c     version      1.2   date    29-05-1998 Charles: add isdist2 and isdist3
c                                                    from isalloc
c     version      1.1   date    10-06-1996 Guus: replace isfilg01 by text
c     version      1.0   date    01-11-1994 complete revision
c
c   copyright (c) 1994-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill geometrical quantities
c
c **********************************************************************
c
c                       KEYWORDS
c
c     geometry
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isiinput(*), isrinput(*), iscoor(*),
     +        isgeom(*), isigeom(*), iadmin(*)
      integer nblocks

c     iadmin    i    reserved
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isigeom   i    Array of sequence numbers (nblocks) of array igeom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipcoor, ipgeom, ipigeom, ipiinput, ipiintur, ipiindsc,
     +        ipwork, iswork, lwork, maxvpnts,
     +        ndim, ni, nj, nk, npointsv, nvirtual, iblock,
     +        idiscret, viscorr, ndegfd,
     +        iprinput, iprinbc, ipiinbc, mturb,
     +        ntrnsp, nturb, ipwdis

c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipcoor    Starting address of array coor in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipigeom   Starting address of array igeom in ibuffr
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     iprinbc   Starting address of array rinbc in array iinput
c     iprinput  Starting address of array rinput in buffer
c     ipwdis    Starting address of array wdis in array geom
c     ipwork    Starting address of array work in buffer
c     iswork    Sequence number of array work
c     lwork     Length of array work
c     maxvpnts  Maximum number of points per block including virtual ones
c     mturb     Indicates type of turbulence modelling
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     viscorr   Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create real array
c     ISDELARR  Delete a permanent array
c     ISDIST2   Compute the normal distance to walls in the whole domain (2D)
c     ISDIST3   Compute the normal distance to walls in the whole domain (3D)
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILG02  fill geometrical quantities in R2
c     ISFILG03  fill geometrical quantities in R3
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      for iblock = 1 (1) nblocks do
c
c         if block(iblock) on this node  then
c
c            fill geometrical quantities
c
c      end do
c
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isfilg')

      do iblock = 1, nblocks

         if( iadmin(iblock).eq.inodenr) then

            call isactive( isiinput(iblock) )
            ipiinput = isingetp( isiinput(iblock) )

            ni         = ibuffr( ipiinput + 51 - 1 )
            nj         = ibuffr( ipiinput + 52 - 1 )
            nk         = ibuffr( ipiinput + 53 - 1 )
            nvirtual   = ibuffr( ipiinput + 54 - 1 )
            ndegfd     = ibuffr( ipiinput + 55 -1 )
            ndim       = ibuffr( ipiinput + 57 - 1 )
            maxvpnts   = ibuffr( ipiinput + 69 - 1 )
            ipiintur   = ibuffr( ipiinput + 8 - 1 ) + ipiinput - 1
            ipiindsc   = ibuffr( ipiinput + 6 - 1 )
            idiscret   = ibuffr( ipiinput + ipiindsc - 1 + 2 - 1 )
            mturb      = ibuffr( ipiintur + 2 - 1 )
            viscorr    = ibuffr( ipiintur - 1 + 3 )

            lwork    = maxvpnts * ndim**4
            iswork   = 0
            call isdcreat( iswork, lwork, 'work')

            call isactive( iscoor(iblock) )
            call isactive( isgeom(iblock) )
            if ( isigeom(iblock).gt.0 ) then
               call isactive( isigeom(iblock) )
               ipigeom  = isingetp( isigeom(iblock) )
            else
               ipigeom=1
            end if

            ipcoor   = isdpgetp( iscoor(iblock) )
            ipgeom   = isdpgetp( isgeom(iblock) )
            ipwork   = isdpgetp( iswork )

            if ( ndim.eq.2 ) then

               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

               call isfilg02 ( ni, nj, nvirtual, npointsv,
     +                         buffer(ipcoor), buffer(ipgeom),
     +                         ibuffr(ipigeom), buffer(ipwork),
     +                         idiscret )

            else
               call isfilg03 ( ni, nj, nk, nvirtual, buffer(ipcoor),
     +                         buffer(ipgeom), buffer(ipwork) )
            end if

            call isdelarr( iswork )

            if (((mturb.eq.1).or.(mturb.eq.4)).and.viscorr.ne.0) then

               call isactive(isigeom (iblock))
               call isactive(iscoor  (iblock))
               call isactive(isrinput(iblock))
               iprinput = isdpgetp( isrinput(iblock) )
               ipigeom  = isingetp( isigeom (iblock) )
               ipcoor   = isdpgetp( iscoor  (iblock) )
               iprinbc    = ibuffr( ipiinput + 27 -1 )
               ipiinbc    = ibuffr( ipiinput + 2  -1 )
               ntrnsp     = ibuffr( ipiinput + 63 -1 )
               nturb      = ibuffr( ipiinput + 70 -1 )

               if ( ndim.eq. 2 ) then
                  npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
                  ipwdis = npointsv*4*2+1
                  call isdist2(buffer(ipgeom+ipwdis-1), buffer(ipcoor),
     +                 ibuffr( ipiinput + ipiinbc-1), nvirtual,
     +                 ni, nj, ntrnsp, nturb, ndegfd)
               else
                  npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                 (nk+1+2*nvirtual)
                  ipwdis = npointsv*8*2+1
                  call isdist3(buffer(ipgeom+ipwdis-1), buffer(ipcoor),
     +                 ibuffr( ipiinput + ipiinbc-1), nvirtual,
     +                 ni, nj, nk, ntrnsp, nturb,
     +                 buffer(iprinput+iprinbc-1),
     +                 ibuffr(ipigeom),
     +                 ibuffr(ipiintur-1+9))

               end if

            end if

         end if

      end do

      call iserclos('isfilg')

      end
