      subroutine isfilimv ( iequa, coupled, ni, nj, nk, nvirtual,
     +                      nblocks, isitopol, idiscret, ipsmsg,
     +                      iinmtvc )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date   06-08-1998 Guus: remove cisnas
c        version 1.0    date   16-10-1997
c
c
c   copyright (c) 1997-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fills array iinmtvc, this array is necessary for the call
c     of multiblock transport routines in the subroutine which
c     performs the matrix-vector multiplication in the linear
c     solver. The calling subroutine is of low level as the
c     multiblock routines are of a high level, pointers and
c     variables for the call are passed by array iinmatvec.
c     Multiblock routines are called for periodic boundary conditions.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, ni, nj, nk, nvirtual, nblocks, isitopol,
     +        idiscret, iinmtvc(20), ipsmsg
      logical coupled

c     coupled   i    Logical which is true if the momentum equations
c                    are coupled
c     idiscret  i    Type of discretization
c     iequa     i    General equation sequence number
c     iinmtvc   i    reserved
c     ipsmsg    i    Starting address of array smsg in buffer
c     isitopol  i    Sequence number of array itopol
c     nblocks   i    Number of blocks
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, itrans, ilev, ntraq1, ntraq2, ipitopol,
     +        nfaces,  maxnsubf, ipnsubf, ipsubf, i

c     i         General loop variable
c     iblock    Block sequence number
c     ilev      Time level of the multi-block transport of information
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     itrans    reserved
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nfaces    Number of faces
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      iblock = 1
      ilev   = 1
      ntraq2 = 1
      if( iequa.eq.1 .and. coupled ) then
         itrans = 1
         ntraq1 = 2
      else
         itrans = 3
         ntraq1 = 1
      end if
      nblocks = 1

      call isactive( isitopol )
      ipitopol   = isingetp(isitopol)
      nfaces     = ibuffr(ipitopol+2-1)
      maxnsubf   = ibuffr(ipitopol+3-1)
      ipnsubf    = ibuffr(ipitopol+5-1) + ipitopol -1
      ipsubf     = ibuffr(ipitopol+6-1) + ipitopol -1

      iinmtvc(1)  = iblock
      iinmtvc(2)  = itrans
      iinmtvc(3)  = ilev
      iinmtvc(4)  = ni
      iinmtvc(5)  = nj
      iinmtvc(6)  = nk
      iinmtvc(7)  = ntraq1
      iinmtvc(8)  = ntraq2
      iinmtvc(9)  = nvirtual
      iinmtvc(10) = nblocks
      iinmtvc(11) = nfaces
      iinmtvc(12) = maxnsubf
      iinmtvc(13) = idiscret
      iinmtvc(14) = ipnsubf
      iinmtvc(15) = ipsubf
      iinmtvc(16) = ipsmsg
      do i =17,20
         iinmtvc(i) = 0
      end do

      call isdcrpri

      end
