      subroutine isfilm03 ( mmct, gmct, smct, nconct, maxdx )
c ======================================================================
c
c        programmer   Ronald van Nooyen
c        version 2.0    date 12-02-1998 Change of parameter list
c        version 1.1    date 16-09-1994 RvN numbering corrected
c        version 1.0    date 16-08-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The routine isfilm03
c     initialises a code table mct which gives
c     the relation between velocity components and
c     pressure values in
c     given locations and offsets from the diagonal
c     in the momentum, gradient and  divgrad or
c     transport matrix (in this routine for 3=3 only)
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nconct, maxdx
      integer mmct(1:3,1:3,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer gmct(1:3,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer smct(1:1,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)

c     gmct     o    Matrix code table gradient matrix see array infmg
c     maxdx    i    Maximum relative shift between cell indices
c     mmct     o    Matrix code table transport equation see array infmt
c     nconct   i   Number of connections in a row of the matrix
c     smct     o    Column index+row degree of freedom to connection
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer alphat, betat, di, dj, dk

c     alphat    Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     betat     Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     di        Shift in stencil in i-direction
c     dj        Shift in stencil in j-direction
c     dk        Shift in stencil in k-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Initialize all diagonal numbers to -1, so we can allways
c         recognize an attempt to find a code for an unused diagonal

      do dk=-maxdx,maxdx
         do dj=-maxdx,maxdx
            do di=-maxdx,maxdx
               do alphat=1,3
                  do betat=1,3
                     mmct(alphat,betat,di,dj,dk) = -1
                  end do
               end do
               do alphat=1,3
                  do betat=1,1
                     gmct(alphat,betat,di,dj,dk) = -1
                  end do
               end do
               do alphat=1,1
                  do betat=1,1
                     smct(alphat,betat,di,dj,dk) = -1
                  end do
               end do
            end do
         end do
      end do

c     --- Momentum matrix

      do alphat=1,3

c     --- Set the diagonal number for diagonals in the
c         momentum matrix for V^alphat with shift di,dj,dk
c         in a V^alphat cell

         mmct(alphat,alphat,  0,  0,  0)   =   1
         mmct(alphat,alphat, -1, -1,  0)   =   2
         mmct(alphat,alphat,  0, -1,  0)   =   3
         mmct(alphat,alphat,  1, -1,  0)   =   4
         mmct(alphat,alphat, -1,  0,  0)   =   5
         mmct(alphat,alphat,  1,  0,  0)   =   6
         mmct(alphat,alphat, -1,  1,  0)   =   7
         mmct(alphat,alphat,  0,  1,  0)   =   8
         mmct(alphat,alphat,  1,  1,  0)   =   9
         mmct(alphat,alphat,  0, -1, -1)   =  10
         mmct(alphat,alphat, -1,  0, -1)   =  11
         mmct(alphat,alphat,  0,  0, -1)   =  12
         mmct(alphat,alphat,  1,  0, -1)   =  13
         mmct(alphat,alphat,  0,  1, -1)   =  14
         mmct(alphat,alphat,  0, -1,  1)   =  15
         mmct(alphat,alphat, -1,  0,  1)   =  16
         mmct(alphat,alphat,  0,  0,  1)   =  17
         mmct(alphat,alphat,  1,  0,  1)   =  18
         mmct(alphat,alphat,  0,  1,  1)   =  19
         if (alphat.eq.1) then

c        --- Set the diagonal number for diagonals in the
c            momentum matrix for V^betat with shift di,dj,dk
c            in a V^1 cell

            betat = 2
            mmct(alphat, betat, -1,  0, -1)   =  20
            mmct(alphat, betat,  0,  0, -1)   =  21
            mmct(alphat, betat, -1,  1, -1)   =  22
            mmct(alphat, betat,  0,  1, -1)   =  23
            mmct(alphat, betat, -1, -1,  0)   =  24
            mmct(alphat, betat,  0, -1,  0)   =  25
            mmct(alphat, betat, -1,  0,  0)   =  26
            mmct(alphat, betat,  0,  0,  0)   =  27
            mmct(alphat, betat, -1,  1,  0)   =  28
            mmct(alphat, betat,  0,  1,  0)   =  29
            mmct(alphat, betat, -1,  2,  0)   =  30
            mmct(alphat, betat,  0,  2,  0)   =  31
            mmct(alphat, betat, -1,  0,  1)   =  32
            mmct(alphat, betat,  0,  0,  1)   =  33
            mmct(alphat, betat, -1,  1,  1)   =  34
            mmct(alphat, betat,  0,  1,  1)   =  35
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat, -2,  0,  0)   =  52
               mmct(alphat, betat, -2,  1,  0)   =  53
               mmct(alphat, betat,  1,  0,  0)   =  54
               mmct(alphat, betat,  1,  1,  0)   =  55

            end if
            betat = 3
            mmct(alphat, betat, -1,  0, -1)   =  36
            mmct(alphat, betat,  0,  0, -1)   =  37
            mmct(alphat, betat, -1, -1,  0)   =  38
            mmct(alphat, betat,  0, -1,  0)   =  39
            mmct(alphat, betat, -1,  0,  0)   =  40
            mmct(alphat, betat,  0,  0,  0)   =  41
            mmct(alphat, betat, -1,  1,  0)   =  42
            mmct(alphat, betat,  0,  1,  0)   =  43
            mmct(alphat, betat, -1, -1,  1)   =  44
            mmct(alphat, betat,  0, -1,  1)   =  45
            mmct(alphat, betat, -1,  0,  1)   =  46
            mmct(alphat, betat,  0,  0,  1)   =  47
            mmct(alphat, betat, -1,  1,  1)   =  48
            mmct(alphat, betat,  0,  1,  1)   =  49
            mmct(alphat, betat, -1,  0,  2)   =  50
            mmct(alphat, betat,  0,  0,  2)   =  51
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat, -2,  0,  0)   =  56
               mmct(alphat, betat, -2,  0,  1)   =  57
               mmct(alphat, betat,  1,  0,  0)   =  58
               mmct(alphat, betat,  1,  0,  1)   =  59

            end if

         else if (alphat.eq.2) then

c        --- Set the diagonal number for diagonals in the
c            momentum matrix for V^betat with shift di,dj,dk
c            in a V^2 cell

            betat = 3
            mmct(alphat, betat,  0, -1, -1)   =  20
            mmct(alphat, betat,  0,  0, -1)   =  21
            mmct(alphat, betat, -1, -1,  0)   =  22
            mmct(alphat, betat,  0, -1,  0)   =  23
            mmct(alphat, betat,  1, -1,  0)   =  24
            mmct(alphat, betat, -1,  0,  0)   =  25
            mmct(alphat, betat,  0,  0,  0)   =  26
            mmct(alphat, betat,  1,  0,  0)   =  27
            mmct(alphat, betat, -1, -1,  1)   =  28
            mmct(alphat, betat,  0, -1,  1)   =  29
            mmct(alphat, betat,  1, -1,  1)   =  30
            mmct(alphat, betat, -1,  0,  1)   =  31
            mmct(alphat, betat,  0,  0,  1)   =  32
            mmct(alphat, betat,  1,  0,  1)   =  33
            mmct(alphat, betat,  0, -1,  2)   =  34
            mmct(alphat, betat,  0,  0,  2)   =  35
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat,  0, -2,  0)   =  52
               mmct(alphat, betat,  0, -2,  1)   =  53
               mmct(alphat, betat,  0,  1,  0)   =  54
               mmct(alphat, betat,  0,  1,  1)   =  55

            end if
            betat = 1
            mmct(alphat, betat,  0, -1, -1)   =  36
            mmct(alphat, betat,  1, -1, -1)   =  37
            mmct(alphat, betat,  0,  0, -1)   =  38
            mmct(alphat, betat,  1,  0, -1)   =  39
            mmct(alphat, betat, -1, -1,  0)   =  40
            mmct(alphat, betat,  0, -1,  0)   =  41
            mmct(alphat, betat,  1, -1,  0)   =  42
            mmct(alphat, betat,  2, -1,  0)   =  43
            mmct(alphat, betat, -1,  0,  0)   =  44
            mmct(alphat, betat,  0,  0,  0)   =  45
            mmct(alphat, betat,  1,  0,  0)   =  46
            mmct(alphat, betat,  2,  0,  0)   =  47
            mmct(alphat, betat,  0, -1,  1)   =  48
            mmct(alphat, betat,  1, -1,  1)   =  49
            mmct(alphat, betat,  0,  0,  1)   =  50
            mmct(alphat, betat,  1,  0,  1)   =  51
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat,  0, -2,  0)   =  56
               mmct(alphat, betat,  1, -2,  0)   =  57
               mmct(alphat, betat,  0,  1,  0)   =  58
               mmct(alphat, betat,  1,  1,  0)   =  59

            end if

         else if (alphat.eq.3) then

c        --- Set the diagonal number for diagonals in the
c            momentum matrix for V^betat with shift di,dj,dk
c            in a V^3 cell

            betat = 1
            mmct(alphat, betat,  0, -1, -1)   =  20
            mmct(alphat, betat,  1, -1, -1)   =  21
            mmct(alphat, betat, -1,  0, -1)   =  22
            mmct(alphat, betat,  0,  0, -1)   =  23
            mmct(alphat, betat,  1,  0, -1)   =  24
            mmct(alphat, betat,  2,  0, -1)   =  25
            mmct(alphat, betat,  0,  1, -1)   =  26
            mmct(alphat, betat,  1,  1, -1)   =  27
            mmct(alphat, betat,  0, -1,  0)   =  28
            mmct(alphat, betat,  1, -1,  0)   =  29
            mmct(alphat, betat, -1,  0,  0)   =  30
            mmct(alphat, betat,  0,  0,  0)   =  31
            mmct(alphat, betat,  1,  0,  0)   =  32
            mmct(alphat, betat,  2,  0,  0)   =  33
            mmct(alphat, betat,  0,  1,  0)   =  34
            mmct(alphat, betat,  1,  1,  0)   =  35
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat,  0,  0, -2)   =  52
               mmct(alphat, betat,  1,  0, -2)   =  53
               mmct(alphat, betat,  0,  0,  1)   =  54
               mmct(alphat, betat,  1,  0,  1)   =  55

            end if
            betat = 2
            mmct(alphat, betat,  0, -1, -1)   =  36
            mmct(alphat, betat, -1,  0, -1)   =  37
            mmct(alphat, betat,  0,  0, -1)   =  38
            mmct(alphat, betat,  1,  0, -1)   =  39
            mmct(alphat, betat, -1,  1, -1)   =  40
            mmct(alphat, betat,  0,  1, -1)   =  41
            mmct(alphat, betat,  1,  1, -1)   =  42
            mmct(alphat, betat,  0,  2, -1)   =  43
            mmct(alphat, betat,  0, -1,  0)   =  44
            mmct(alphat, betat, -1,  0,  0)   =  45
            mmct(alphat, betat,  0,  0,  0)   =  46
            mmct(alphat, betat,  1,  0,  0)   =  47
            mmct(alphat, betat, -1,  1,  0)   =  48
            mmct(alphat, betat,  0,  1,  0)   =  49
            mmct(alphat, betat,  1,  1,  0)   =  50
            mmct(alphat, betat,  0,  2,  0)   =  51
            if ( nconct.gt.51 ) then

c           --- 59 point molecule

               mmct(alphat, betat,  0,  0, -2)   =  56
               mmct(alphat, betat,  0,  1, -2)   =  57
               mmct(alphat, betat,  0,  0,  1)   =  58
               mmct(alphat, betat,  0,  1,  1)   =  59

            end if
         end if

      end do

c     --- Gradient matrix

      betat = 1
      alphat = 1

      gmct(alphat, betat, -1, -1,  0)   =   1
      gmct(alphat, betat,  0, -1,  0)   =   2
      gmct(alphat, betat, -1,  0,  0)   =   3
      gmct(alphat, betat,  0,  0,  0)   =   4
      gmct(alphat, betat, -1,  1,  0)   =   5
      gmct(alphat, betat,  0,  1,  0)   =   6
      gmct(alphat, betat, -1,  0, -1)   =   7
      gmct(alphat, betat,  0,  0, -1)   =   8
      gmct(alphat, betat, -1,  0,  1)   =   9
      gmct(alphat, betat,  0,  0,  1)   =  10

c     --- diagonal code for di,dj,dk pressure in V^1 integration cell

      alphat = 2

      gmct(alphat, betat,  0, -1, -1)   =   1
      gmct(alphat, betat,  0,  0, -1)   =   2
      gmct(alphat, betat,  0, -1,  0)   =   3
      gmct(alphat, betat,  0,  0,  0)   =   4
      gmct(alphat, betat,  0, -1,  1)   =   5
      gmct(alphat, betat,  0,  0,  1)   =   6
      gmct(alphat, betat, -1, -1,  0)   =   7
      gmct(alphat, betat, -1,  0,  0)   =   8
      gmct(alphat, betat,  1, -1,  0)   =   9
      gmct(alphat, betat,  1,  0,  0)   =  10

c     --- diagonal code for di,dj,dk pressure in V^1 integration cell

      alphat = 3

      gmct(alphat, betat, -1,  0, -1)   =   1
      gmct(alphat, betat, -1,  0,  0)   =   2
      gmct(alphat, betat,  0,  0, -1)   =   3
      gmct(alphat, betat,  0,  0,  0)   =   4
      gmct(alphat, betat,  1,  0, -1)   =   5
      gmct(alphat, betat,  1,  0,  0)   =   6
      gmct(alphat, betat,  0, -1, -1)   =   7
      gmct(alphat, betat,  0, -1,  0)   =   8
      gmct(alphat, betat,  0,  1, -1)   =   9
      gmct(alphat, betat,  0,  1,  0)   =  10

c     --- Divgrad/Transport matrix

      alphat = 1
      betat  = 1

      smct(alphat,alphat,  0,  0,  0)   =   1
      smct(alphat,alphat, -1, -1,  0)   =   2
      smct(alphat,alphat,  0, -1,  0)   =   3
      smct(alphat,alphat,  1, -1,  0)   =   4
      smct(alphat,alphat, -1,  0,  0)   =   5
      smct(alphat,alphat,  1,  0,  0)   =   6
      smct(alphat,alphat, -1,  1,  0)   =   7
      smct(alphat,alphat,  0,  1,  0)   =   8
      smct(alphat,alphat,  1,  1,  0)   =   9
      smct(alphat,alphat,  0, -1, -1)   =  10
      smct(alphat,alphat, -1,  0, -1)   =  11
      smct(alphat,alphat,  0,  0, -1)   =  12
      smct(alphat,alphat,  1,  0, -1)   =  13
      smct(alphat,alphat,  0,  1, -1)   =  14
      smct(alphat,alphat,  0, -1,  1)   =  15
      smct(alphat,alphat, -1,  0,  1)   =  16
      smct(alphat,alphat,  0,  0,  1)   =  17
      smct(alphat,alphat,  1,  0,  1)   =  18
      smct(alphat,alphat,  0,  1,  1)   =  19

      end
