      subroutine isincp2 ( solut, ndegfd, nvirtual, ni, nj, ntimlv,
     +                                                       igauss )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 03-03-1999    Duncan: igauss added to parlist
c        version 1.0    date 19-02-1999
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy time level 1 of array solut to other time levels (2d)
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni, nj, ntimlv, ndegfd, nvirtual, igauss
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

c     igauss     i   if igauss=1 non linear Gauss Seidel is applied else not
c     ndegfd     i   Number of degrees of freedom
c     ni         i   Number of non-virtual cells in the i-direction (one block)
c     nj         i   Number of non-virtual cells in the j-direction (one block)
c     ntimlv     i   Number of time levels
c     nvirtual   i   Number of rows of virtual cells
c     solut     i/o  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c     none
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, frsti, frstj, lasti, lastj,
     +         itimlv, ip, jp

c     frsti     First index in i-direction
c     frstj     First index in j-direction
c     idegfd    Loop variable with respect to a given unknown
c     ip        Local counting variable i-direction
c     itimlv    Loop variable with respect to a given time level
c     jp        Local counting variable j-direction
c     lasti     Last index in i-direction
c     lastj     Last index in j-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For all solution vectors do
c        Copy vector to new time level
c     Set pressure at level ntimlv equal to 0
c     Copy array eddy to new time level
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- copy timelevel 1 to the other timelevels

      frsti = 1-nvirtual
      frstj = 1-nvirtual
      lasti = ni + 1 + nvirtual
      lastj = nj + 1 + nvirtual

      do idegfd = 1, ndegfd
         do itimlv = 2, ntimlv
            do jp = frstj, lastj
               do ip = frsti, lasti
                  solut( ip, jp, idegfd, itimlv ) =
     +                             solut( ip, jp, idegfd, 1 )
               end do
            end do
         end do
      end do

c     --- but set initial pressure correction equal to zero and not
c         the initial pressure, or to a small constant if the nonlinear
c         Gauss-Seidel solver is used.

      if( igauss.eq.1) then

        do jp = frstj, lastj
            do ip = frsti, lasti
                 solut( ip, jp, 3, ntimlv ) = 1d-6
            end do
        end do

      else if( igauss.eq.0) then
        do jp = frstj, lastj
            do ip = frsti, lasti
                 solut( ip, jp, 3, ntimlv ) = 0d0
            end do
        end do
      end if

      end
