      subroutine isincp3 ( solut, ndegfd, nvirtual, ni, nj, nk, ntimlv )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 19-02-1999
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy time level 1 of array solut to other time levels (3d)
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ndegfd, nvirtual, ntimlv
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd,*)

c     ndegfd     i    Number of degrees of freedom
c     ni         i    Number of non-virtual cells in the i-direction (one block)
c     nj         i    Number of non-virtual cells in the j-direction (one block)
c     nk         i    Number of non-virtual cells in the k-direction (one block)
c     ntimlv     i    Number of time levels
c     nvirtual   i    Number of rows of virtual cells
c     solut     i/o   reserved
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, ip, jp, kp, itimlv

c     idegfd       Loop variable with respect to a given unknown
c     ip           Local counting variable i-direction
c     itimlv       Loop variable with respect to a given time level
c     jp           Local counting variable j-direction
c     kp           Local counting variable k-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       copy timelevel 1 to the other timelevels
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- copy timelevel 1 to the other timelevels

      do idegfd = 1, ndegfd
         do itimlv = 2, ntimlv
            do kp = 1-nvirtual, nk + 1 + nvirtual
               do jp = 1-nvirtual, nj + 1 + nvirtual
                  do ip = 1-nvirtual, ni + 1 + nvirtual
                     solut(ip,jp,kp,idegfd,itimlv ) =
     +                                solut(ip,jp,kp,idegfd,1)
                  end do
               end do
            end do
         end do
      end do

c     --- but set initial pressure correction equal to zero and not
c         the initial pressure

      do kp = 1-nvirtual, nk + 1 + nvirtual
         do jp = 1-nvirtual, nj + 1 + nvirtual
            do ip = 1-nvirtual, ni + 1 + nvirtual
               solut(ip,jp,kp,4,ntimlv) = 0d0
            end do
         end do
      end do

      end
