      subroutine islenav3 ( iinbc, ni, nj, nk, ntrnsp, nturb, rinbc,
     +                      icount )
c ======================================================================
c
c     programmer   Charles Moulinec
c     version      1.0    date    29-05-1998
c
c   copyright (c) 1998-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Muenchen".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     give the length of lenaver
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinbc(*), ni, nj, nk, ntrnsp, nturb, icount
      double precision rinbc(*)

c     icount    i    Counts the number of positions necessary for the number
c     iinbc     i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     rinbc     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iside, iarea(2,3), tdir1, tdir2,
     +        ipstart, nseg, j, ipiinbc,
     +        ipl, jpl, kpl, idim, bctyp(3), iprinbc
      double precision bcval(3)

c     bctyp     A given boundary type
c     bcval     A given boundary condition value
c     iarea     Area of arrays
c     idim      Loop variable used for dimension
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipl       Local ip value
c     iprinbc   Starting address of array rinbc in array iinput
c     ipstart   Starting address
c     iside     Sequence number of side
c     j         General loop variable
c     jpl       Local jp value
c     kpl       Local kp value
c     nseg      Number of segments the boundary is divided in
c     tdir1     Direction of first tangent vector on current side
c     tdir2     Direction of second tangent vector on current side
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isdist3')

c     --- loop over boundaries

      iprinbc = 0
      icount = 0

      do iside = 1, 6

c     --- Get information on boundary 'iside'

        if ( iside.le.2 ) then
           tdir1 = 2
           tdir2 = 3
           if ( iside.eq.1 ) then
              iarea(1,1) = 1
           else
              iarea(1,1) = ni
           end if
           iarea(2,1) = iarea(1,1)
        else if (iside.le.4) then
           tdir1 = 3
           tdir2 = 1
           if ( iside.eq.3 ) then
              iarea(1,2) = 1
           else
              iarea(1,2) = nj
           end if
           iarea(2,2) = iarea(1,2)
        else
           tdir1 = 1
           tdir2 = 2
           if (iside.eq.5) then
              iarea(1,3) = 1
           else
              iarea(1,3) = nk
           end if
           iarea(2,3) = iarea(1,3)
        end if

        ipstart = iinbc(iside)

        nseg = iinbc(ipstart)

c       --- loop over segments

        do j = 1, nseg

           ipiinbc = ipstart +(j-1) *( 4+(ntrnsp+nturb+2)*4 )

           iarea(1,tdir1) = iinbc( ipiinbc + 1 )
           iarea(2,tdir1) = iinbc( ipiinbc + 3 )
           iarea(1,tdir2) = iinbc( ipiinbc + 2 )
           iarea(2,tdir2) = iinbc( ipiinbc + 4 )

           if ( iinbc(ipiinbc+5).eq.10 ) then

c          --- All velocities are prescribed, hence a no-slip wall is possible
c              Put velocity values in bcval

              do idim = 1,3
                 bcval(idim) = 1d20
                 bctyp(idim) = iinbc(ipiinbc+5+idim)
                 if (bctyp(idim).eq.-1) then
                    bcval(idim) = rinbc(iprinbc+idim)
                 end if
              end do

              iprinbc = iprinbc+(ntrnsp+nturb+2)*3

              if (bcval(1).eq.0d0 .and. bcval(2).eq.0d0
     +                          .and. bcval(3).eq.0d0 ) then

c             --- no-slip condition

                 do ipl = iarea(1,1), iarea(2,1)
                    do jpl = iarea(1,2), iarea(2,2)
                       do kpl = iarea(1,3), iarea(2,3)

                          icount = icount + 1

                       end do
                    end do
                 end do

              end if

           end if

        end do

      end do

      call iserclos('isdist3')

      end
