      subroutine ismblk02(iblock, itrans, ilev, ni, nj, nk,
     +                    ndegfd, ntimlv, nvirtual,
     +                    nblocks, nfaces, maxnsubf, idiscret,
     +                    isolv, solut,  isiinput, issolut, iadmin,
     +                    nsubfaces, subfaces, smsg)
c ======================================================================
c
c     programmer   Erik Brakkee
c
c     version      2.2 14-10-1997  Kees K.: isolv added
c     version      2.1 12-12-1996  Kees K.: idiscret added
c     version      2.0 24-5-1994  Erik: error removed.
c                               In the previous version the information
c                               was only transported if the neighbouring
c                               block resided on the same node. The current
c                               version transports information also if
c                               this routine is called by the host.
c
c     version      1.0  7-4-1994  Erik:  initial revision
c
c   copyright (c) 1994-1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport certain variables (itrans,ilev) from block iblock
c     to the solut arrays of the neighbouring blocks on the same node.
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, ilev, itrans, maxnsubf, idiscret, isolv,
     +      nblocks, ndegfd, nfaces, ni, nj, nk, ntimlv, nvirtual
      integer  iadmin(1:nblocks)
      integer  isiinput(1:nblocks)
      integer  issolut(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iadmin    i  reserved
c     iblock    i  Block sequence number
c     idiscret  i  Type of discretization
c     ilev      i  Time level of the multi-block transport of information
c     isiinput  i  Array of sequence numbers (nblocks) of array iinput
c     isolv     i  Called by solver for periodic b.c. isolv=1 else isolv=0
c     issolut   i  Array of sequence numbers (nblocks) of array solut
c     itrans    i  reserved
c     maxnsubf  i  Maximum number of subfaces encountered on all subfaces
c     nblocks   i  Number of blocks
c     ndegfd    i  Number of degrees of freedom
c     nfaces    i  Number of faces
c     ni        i  Number of non-virtual cells in the i-direction (one block)
c     nj        i  Number of non-virtual cells in the j-direction (one block)
c     nk        i  Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i  Array containing the number of subfaces of each face,
c                  see array itopol for an extended description
c     ntimlv    i  Number of time levels
c     nvirtual  i  Number of rows of virtual cells
c     smsg     i/o reserved
c     solut     i  reserved
c     subfaces  i reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto, iface, ifto, ipiinput, ipsolut, isfto, isubface,
     +        nito, njto, nkto, idummy

c     ibto       Target block
c     idummy     Integer dummy
c     iface      Face number
c     ifto       Target face
c     ipiinput   Starting address of array iinput in ibuffr
c     ipsolut    Starting address of array solut in buffr
c     isfto      Target subface
c     isubface   Subface number
c     nito       I-dimension of neighbour
c     njto       J-dimension of neighbour
c     nkto       K-dimension of neighbour
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddget, isdpgetp, isingetp, isddput

c     ISDDGET   Get virtual unknowns from a certain subface of a certain
c               block and put them into an intermediate representation
c     ISDDPUT   Copy virtual unknowns to a certain subface of a certain
c               block
c     ISDPGETP  Get the address of a double precision array in buffer
c     ISINGETP  Get the address of an integer array in ibuffr
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- loop across the interfaces with neighbouring blocks

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock, iface)
            ibto = subfaces(iblock, iface, isubface, 5)

c           --- internal subface???

            if ( ibto.ne.0 ) then

c           --- transport unknowns to the solut array of a neighbouring block
c               only if the solut array of this neighbouring block resides
c               on the same node.
c               So there are two possibilities:
c                1. this is the host --> solut arrays of all blocks are
c                   present
c                2. this is not the host but the neighbouring blocks resides
c                   on this node

               if ( isolv .eq. 0) then

c              --- this routine is now called for multi block

                  if ( inodenr.eq.0
     +                 .or.
     +                 iadmin(ibto).eq.inodenr ) then

                     ifto = subfaces(iblock, iface, isubface, 6)
                     isfto = subfaces(iblock, iface, isubface, 7)
                     idummy = isddget(iblock, iface, isubface, itrans,
     +                         ilev,    ni,       nj,           nk,
     +                         ndegfd,  ntimlv, nvirtual,
     +                         nblocks, nfaces,   maxnsubf, idiscret,
     +                         isolv, solut, nsubfaces, subfaces, smsg )

                     ipsolut  = isdpgetp( issolut(ibto))
                     ipiinput = isingetp( isiinput(ibto))

                     nito  = ibuffr(ipiinput + 51 - 1)
                     njto  = ibuffr(ipiinput + 52 - 1)
                     nkto  = ibuffr(ipiinput + 53 - 1)

                     idummy = isddput(ibto, ifto,  isfto, itrans,  ilev,
     +                          nito, njto, nkto, ndegfd,
     +                          ntimlv, nvirtual,
     +                          nblocks, nfaces, maxnsubf, idiscret,
     +                          isolv, buffer(ipsolut), nsubfaces,
     +                          subfaces, smsg)
                  end if

               else

c              --- this routine is now called by solver for periodic b.c.

                  ifto = subfaces(iblock, iface, isubface, 6)
                  isfto = subfaces(iblock, iface, isubface, 7)
                  idummy = isddget(iblock,  iface, isubface, itrans,
     +                         ilev,    ni,       nj,           nk,
     +                         ndegfd,  ntimlv, nvirtual,
     +                         nblocks, nfaces,   maxnsubf, idiscret,
     +                         isolv, solut, nsubfaces, subfaces, smsg )

                  nito  = ni
                  njto  = nj
                  nkto  = nk

                  idummy = isddput(ibto, ifto,  isfto, itrans,  ilev,
     +                          nito, njto, nkto, ndegfd,
     +                          ntimlv, nvirtual,
     +                          nblocks, nfaces, maxnsubf, idiscret,
     +                          isolv, solut, nsubfaces,
     +                          subfaces, smsg)


               end if
            end if
         end do
      end do

      end
