      subroutine ismvg01 (coor, dt, ndim, ni, nj, nvirtual,
     +                     velfrsf, relax )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 02-03-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Adapt coordinates  of freesurface using the kinematic boundary
c   condition dh/dt + u*dh/dx = v
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndim,  nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:ndim),
     +                 dt, velfrsf(1-nvirtual:ni+1+nvirtual,1:ndim)
      double precision  relax

c     coor      i    Array containing the coordinates for one block
c                    In 2d coor(i,j,.) contains the x resp y-coordinate
c                    of point i,j
c                    in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     dt        i    Time step
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     relax     i    Relaxation parameter
c     velfrsf   i    Cartesian velocities at the upper boundary for
c                    the computation of the free surface
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j
      double precision  dhdt, dhdx, oh, normfs

c     dhdt      dh/dt= v-u*dh/dx
c     dhdx      dh/dx discretised
c     i         General loop variable
c     j         General loop variable
c     normfs    L2 norm of residu (u*dh/dx-v)
c     oh        invers of old waterlevel at time t
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     discretise dh/dx
c     discretise dh/dt= v-u*dh/dx
c     compute 1/(old waterlevel)
c     adapt coordinates in  y-direction
c     compute L2 norm of residu (u*dh/dx-v)
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      normfs = 0d0

      do i = ni+2, 2-nvirtual, -1

         dhdx = (coor(i,nj+1,2)-coor(i-1,nj+1,2))
     +          /(coor(i,nj+1,1)-coor(i-1,nj+1,1))

         dhdt = velfrsf(i,2)-velfrsf(i,1)*dhdx

         oh = 1d0/(coor(i,nj+1,2)-coor(i,1,2))

         do j = 1-nvirtual, nj+1+nvirtual
            coor(i,j,2) = dhdt*dt*oh*(coor(i,j,2)-coor(i,1,2))*
     +                     relax
     +                     +coor(i,j,2)
         end do

         normfs = normfs+dhdt**2

      end do

      normfs = sqrt(normfs/(ni-1+2*nvirtual))

      write(6,99)normfs
99    format( ' Norm Freesurface = ', d16.6)

      end
