      subroutine isndd (  ieq, nblocks, isiinput, isrinput, iscoor,
     +                    issolut , iscoefs , iseddy  , isinfbnd,
     +                    isbndcon, ismatrix, isgmat  , isrhside,
     +                    isgeom  , isitopol, isdensup, iadmin )
c ======================================================================
c
c     programmer   Jason Frank
c     version      1.0  22-10-1998
c
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNaS Node domain decomposition routine
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer        ieq , iadmin(*)  , isbndcon(*), iscoor(*)  ,
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*) , isgeom(*), isdensup(*)

c     iadmin    i    reserved
c     ieq       i    Equation number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inodrc, ndegfd, neq, ndim, nblocks
      integer iblock, icoupled, iseqpres
      integer ipiinput, ntimlv, itrans, ilev
      logical done

c     done      indicates the domain decompisition is finished
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ilev      Time level of the multi-block transport of information
c     inodrc    Actual node number received from
c     ipiinput  Starting address of array iinput in ibuffr
c     iseqpres  Sequence number of pressure equation
c     itrans    reserved
c     nblocks   number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     neq       Number of equations
c     ntimlv    Number of time levels
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISRCDDND  receives the virtual unknowns of all local blocks at a node
c               and puts them into the local TRAQ arrays.
c     ISSDD     Sequential domain decomposition routine
c     ISSNDDND  Copy unknowns near or on the boundary of the subdomain
c               TRAQ arrays to the virtual region of the TRAQ arrays of the
c               neighbouring blocks on the host using PVM.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('isndd')

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if
      neq      = ndegfd + ( 1 -  ndim )*( 1 - icoupled )

      done = .false.

      do while (.not. done)

c        --- receive boundary info
	 call isrcddnd( nblocks, isiinput, issolut, isitopol,
     +                  iadmin, done)

         if ( done ) goto 1000

         call issdd ( ieq   , nblocks , isiinput, isrinput,
     +                iscoor  , issolut , ismatrix, isgmat  ,
     +                isrhside, isitopol, iadmin, isinfbnd, isbndcon,
     +                isgeom, isdensup)

c        --- send boundary info

	 call isactive(isiinput(1))
	 ipiinput = isingetp(isiinput(1))
         ntimlv   = ibuffr(ipiinput + 64 -1)
         ilev     = ntimlv

         if ( ieq.lt.iseqpres ) then
            itrans = ieq
         else
            itrans = ieq + ( ndim - 1 ) * ( 1 - icoupled )
         end if

         call issnddnd( itrans, ilev,   isiinput, issolut,
     +                  iadmin, isitopol)

 1000 end do

c     --- stop

      call iserclos('isndd')

      end
