      subroutine isqudtn3( hightens,alphad, betad, pderiv, veloc, cva,
     +                     invjac, jctg, chsym, ni, nj, nk, nvirtual,
     +                     idiscret, ctau1, ctau2, ctau3, iarea        )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date   05-09-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute quadratic products of strain and rotation tensors.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, idiscret, alphad, betad, iarea(2,3)
      double precision hightens(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual,
     +                          1-nvirtual:nk+1+nvirtual),
     +                 pderiv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,
     +                        1:3,1:3),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:3), ctau1, ctau2, ctau3
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,6),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,6)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     betad     i    Direction index, usually the direction perpendicular to the
c                    integration direction of the boundary integral.
c                    For example betad in local point (1,0) is 1 and in (0,1): 2
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     ctau1     i    Parameter ctau1 for nonlinear two-equation model
c     ctau2     i    Parameter ctau2 for nonlinear two-equation model
c     ctau3     i    Parameter ctau3 for nonlinear two-equation model
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     hightens  o    Higher order tensor constains quadratic products of
c                    strain and rotation tensors
c     iarea     i    Area of arrays
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pderiv    i    Partial derivative of flux in point pnttyp
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, deltal, epsl, gammal, i, ii, jj,
     +        istwo(1:3,1:3)
      double precision gac, gbc, contrst, covrst, cntdef1, cntdef2

c     cntdef1   Resulted double contraction of strain or rotation tensor
c     cntdef2   Resulted double contraction of strain or rotation tensor
c     contrst   Contravariant rate of strain tensor
c     covrst    Covariant rate of strain tensor
c     deltal    Loop index
c     epsl      Loop index
c     gac       Covariant metric tensor
c     gammal    Loop index
c     gbc       Covariant metric tensor
c     i         General loop variable
c     ii        Loop variable
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jj        Loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /
c ======================================================================
c
      if ( idiscret.eq.1 ) then

c     --- standard discretization

        do kp = iarea(1,3), iarea(2,3)
          do jp = iarea(1,2), iarea(2,2)
            do ip = iarea(1,1), iarea(2,1)

              hightens(ip,jp,kp) = 0d0

              cntdef1 = 0

              do gammal = 1, 3

                cntdef2 = 0

                do deltal = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do epsl = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,deltal) *
     +                            cva(ip,jp,kp,i,epsl)
                      gbc = gbc + cva(ip,jp,kp,i,gammal)  *
     +                            cva(ip,jp,kp,i,epsl)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                      invjac(ip,jp,kp) *
     +                      ( jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                        pderiv(ip,jp,kp,deltal,epsl) +
     +                        jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                        pderiv(ip,jp,kp,alphad,epsl) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                     ( gac * pderiv(ip,jp,kp,epsl,gammal) +
     +                       gbc * pderiv(ip,jp,kp,epsl,deltal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                  invjac(ip,jp,kp) *
     +                  ( jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                    ( chsym(ip,jp,kp,deltal,istwo(ii,epsl))*
     +                      veloc(ip,jp,kp,ii) -
     +                      chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,kp,deltal) ) +
     +                    jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                    ( chsym(ip,jp,kp,alphad,istwo(ii,epsl))*
     +                      veloc(ip,jp,kp,ii) -
     +                      chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,kp,alphad) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,kp,epsl,istwo(ii,gammal))*
     +                      veloc(ip,jp,kp,ii) -
     +                      chsym(ip,jp,kp,ii,istwo(gammal,ii))*
     +                      veloc(ip,jp,kp,epsl) ) +
     +                    gbc *
     +                    ( chsym(ip,jp,kp,epsl,istwo(ii,deltal))*
     +                      veloc(ip,jp,kp,ii) -
     +                      chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,kp,epsl) ) )

                    end do

                  end do

                  cntdef2 = cntdef2 + contrst*covrst

                end do

                cntdef1 = cntdef1 + cntdef2*invjac(ip,jp,kp)*
     +                    jctg(ip,jp,kp,istwo(gammal,betad))

              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) + cntdef1*ctau1

              cntdef1 = 0

              do deltal = 1, 3
                do epsl = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do jj = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,deltal) *
     +                            cva(ip,jp,kp,i,jj)
                      gbc = gbc + cva(ip,jp,kp,i,epsl)  *
     +                            cva(ip,jp,kp,i,jj)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                        invjac(ip,jp,kp) *
     +                        ( jctg(ip,jp,kp,istwo(epsl,jj)) *
     +                          pderiv(ip,jp,kp,deltal,jj) +
     +                          jctg(ip,jp,kp,istwo(deltal,jj)) *
     +                          pderiv(ip,jp,kp,epsl,jj) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                       ( gac * pderiv(ip,jp,kp,jj,epsl) +
     +                         gbc * pderiv(ip,jp,kp,jj,deltal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                    invjac(ip,jp,kp) *
     +                    ( jctg(ip,jp,kp,istwo(epsl,jj)) *
     +                      ( chsym(ip,jp,kp,deltal,istwo(ii,jj))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(jj,ii))*
     +                        veloc(ip,jp,kp,deltal) ) +
     +                      jctg(ip,jp,kp,istwo(deltal,jj)) *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,jj))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(jj,ii))*
     +                        veloc(ip,jp,kp,epsl) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                    ( gac *
     +                      ( chsym(ip,jp,kp,jj,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,jj) ) +
     +                      gbc *
     +                      ( chsym(ip,jp,kp,jj,istwo(ii,deltal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                        veloc(ip,jp,kp,jj) ) )

                    end do

                  end do

                  cntdef1 = cntdef1 + contrst*covrst

                end do
              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) - cntdef1*ctau1*
     +         invjac(ip,jp,kp)*jctg(ip,jp,kp,istwo(alphad,betad))/3

              cntdef1 = 0

              do gammal = 1, 3

                cntdef2 = 0

                do deltal = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do epsl = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,deltal) *
     +                            cva(ip,jp,kp,i,epsl)
                      gbc = gbc + cva(ip,jp,kp,i,gammal)  *
     +                            cva(ip,jp,kp,i,epsl)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                        invjac(ip,jp,kp) *
     +                        ( -jctg(ip,jp,kp,istwo(betad,epsl)) *
     +                          pderiv(ip,jp,kp,deltal,epsl) +
     +                          jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                          pderiv(ip,jp,kp,betad,epsl) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                       ( gac * pderiv(ip,jp,kp,epsl,gammal) +
     +                         gbc * pderiv(ip,jp,kp,epsl,deltal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                    invjac(ip,jp,kp) *
     +                    ( -jctg(ip,jp,kp,istwo(betad,epsl)) *
     +                      ( chsym(ip,jp,kp,deltal,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,deltal) ) +
     +                      jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                      ( chsym(ip,jp,kp,betad,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,betad) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                    ( gac *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,gammal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(gammal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) +
     +                      gbc *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,deltal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) )

                    end do

                  end do

                  cntdef2 = cntdef2 + contrst*covrst

                end do

                cntdef1 = cntdef1 + cntdef2*invjac(ip,jp,kp)*
     +                    jctg(ip,jp,kp,istwo(gammal,alphad))

              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) + cntdef1*ctau2

              cntdef1 = 0

              do gammal = 1, 3

                cntdef2 = 0

                do deltal = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do epsl = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,deltal) *
     +                            cva(ip,jp,kp,i,epsl)
                      gbc = gbc + cva(ip,jp,kp,i,gammal)  *
     +                            cva(ip,jp,kp,i,epsl)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                        invjac(ip,jp,kp) *
     +                        ( -jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                          pderiv(ip,jp,kp,deltal,epsl) +
     +                          jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                          pderiv(ip,jp,kp,alphad,epsl) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                       ( gac * pderiv(ip,jp,kp,epsl,gammal) +
     +                         gbc * pderiv(ip,jp,kp,epsl,deltal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                    invjac(ip,jp,kp) *
     +                    ( -jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                      ( chsym(ip,jp,kp,deltal,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,deltal) ) +
     +                      jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                      ( chsym(ip,jp,kp,alphad,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,alphad) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                    ( gac *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,gammal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(gammal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) +
     +                      gbc *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,deltal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) )

                    end do

                  end do

                  cntdef2 = cntdef2 + contrst*covrst

                end do

                cntdef1 = cntdef1 + cntdef2*invjac(ip,jp,kp)*
     +                    jctg(ip,jp,kp,istwo(gammal,betad))

              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) + cntdef1*ctau2

              cntdef1 = 0

              do gammal = 1, 3

                cntdef2 = 0

                do deltal = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do epsl = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,gammal) *
     +                            cva(ip,jp,kp,i,epsl)
                      gbc = gbc + cva(ip,jp,kp,i,deltal)  *
     +                            cva(ip,jp,kp,i,epsl)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                        invjac(ip,jp,kp) *
     +                        ( -jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                          pderiv(ip,jp,kp,deltal,epsl) +
     +                          jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                          pderiv(ip,jp,kp,alphad,epsl) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                       ( gac * pderiv(ip,jp,kp,epsl,deltal) -
     +                         gbc * pderiv(ip,jp,kp,epsl,gammal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                    invjac(ip,jp,kp) *
     +                    ( -jctg(ip,jp,kp,istwo(alphad,epsl)) *
     +                      ( chsym(ip,jp,kp,deltal,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,deltal) ) +
     +                      jctg(ip,jp,kp,istwo(deltal,epsl)) *
     +                      ( chsym(ip,jp,kp,alphad,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,alphad) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                    ( gac *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,deltal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) -
     +                      gbc *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,gammal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(gammal,ii))*
     +                        veloc(ip,jp,kp,epsl) ) )

                    end do

                  end do

                  cntdef2 = cntdef2 + contrst*covrst

                end do

                cntdef1 = cntdef1 + cntdef2*invjac(ip,jp,kp)*
     +                    jctg(ip,jp,kp,istwo(gammal,betad))

              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) + cntdef1*ctau3

              cntdef1 = 0

              do deltal = 1, 3
                do epsl = 1, 3

                  contrst = 0d0
                  covrst  = 0d0

                  do jj = 1, 3

                    gac = 0d0
                    gbc = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,deltal) *
     +                            cva(ip,jp,kp,i,jj)
                      gbc = gbc + cva(ip,jp,kp,i,epsl)  *
     +                            cva(ip,jp,kp,i,jj)
                    end do

                    contrst = contrst + invjac(ip,jp,kp) *
     +                        invjac(ip,jp,kp) *
     +                        ( jctg(ip,jp,kp,istwo(epsl,jj)) *
     +                          pderiv(ip,jp,kp,deltal,jj) -
     +                          jctg(ip,jp,kp,istwo(deltal,jj)) *
     +                          pderiv(ip,jp,kp,epsl,jj) )

                    covrst = covrst + invjac(ip,jp,kp) *
     +                       ( gac * pderiv(ip,jp,kp,jj,epsl) -
     +                         gbc * pderiv(ip,jp,kp,jj,deltal) )

                    do ii = 1, 3

                      contrst = contrst + invjac(ip,jp,kp) *
     +                    invjac(ip,jp,kp) *
     +                    ( jctg(ip,jp,kp,istwo(epsl,jj)) *
     +                      ( chsym(ip,jp,kp,deltal,istwo(ii,jj))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(jj,ii))*
     +                        veloc(ip,jp,kp,deltal) ) -
     +                      jctg(ip,jp,kp,istwo(deltal,jj)) *
     +                      ( chsym(ip,jp,kp,epsl,istwo(ii,jj))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(jj,ii))*
     +                        veloc(ip,jp,kp,epsl) ) )

                      covrst = covrst + invjac(ip,jp,kp) *
     +                    ( gac *
     +                      ( chsym(ip,jp,kp,jj,istwo(ii,epsl))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(epsl,ii))*
     +                        veloc(ip,jp,kp,jj) ) -
     +                      gbc *
     +                      ( chsym(ip,jp,kp,jj,istwo(ii,deltal))*
     +                        veloc(ip,jp,kp,ii) -
     +                        chsym(ip,jp,kp,ii,istwo(deltal,ii))*
     +                        veloc(ip,jp,kp,jj) ) )

                    end do

                  end do

                  cntdef1 = cntdef1 + contrst*covrst

                end do
              end do

              hightens(ip,jp,kp) = hightens(ip,jp,kp) - cntdef1*ctau3*
     +         invjac(ip,jp,kp)*jctg(ip,jp,kp,istwo(alphad,betad))/3

            end do
          end do
        end do

      else if ( idiscret.eq.2 ) then

c     --- Not yet implemented

      end if

      end
