      subroutine isshift ( iadmin, isiinput, isrinput, issolut, iseddy,
     +                     iscoefs, turbonly )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.9  12-04-1999 Duncan: removed bug in call dcopy
c     version      2.8  23-03-1999 Duncan: extended for newton linearisation
c     version      2.7  03-03-1999 Duncan: new call to iscmrho2
c     version      2.6  27-07-1998 Duncan: new call to iscmrho2
c                                          changed treatment of new
c                                          timelevel of density,
c                                          shift density.
c     version      2.5  09-06-1998 Charles: nvpoints changed to npointsv
c     version      2.4  27-03-1998 Kees K.: turbonly
c     version      2.3  05-03-1998 Duncan: extension for cavity
c     version      2.2  19-11-1997 Guus: remove call to isdvmafr
c     version      2.1  12-11-1997 Hester: call to iscmrho2 changed
c     version      2.0  07-08-1997 Hester: extra parameter iscoefs
c     version      1.3  15-04-1997 Hester: call to iscmrho2 added, to
c                                  compute eddy at new time level
c     version      1.2  16-07-1996 Marcel: new variables: mturb, viscorr
c     version      1.1  17-10-1994 Michiel Kooper:  implementation of
c                                  second part of the theta-method
c     version      1.0  01-06-1994
c
c   copyright (c) 1994-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy time level n+1 to n (n = 1..ntimlv-1 in the solut
c     arrays with extrapolation of the theta method (Shift time levels)
c     The shift operation is performed for the solut arrays of
c     all local blocks and for all blocks if on the host.
c **********************************************************************
c
c                       KEYWORDS
c
c     shift
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isiinput(*), isrinput(*), issolut(*),
     +        iseddy(*), iscoefs(*)
      logical turbonly

c     iadmin    i   contains information about
c                   which blocks on which computer are stored.
c     iscoefs   i   Array of sequence numbers (nblocks) of array coefs
c     iseddy    i   Array of sequence numbers (nblocks) of array eddy
c     isiinput  i   Array of sequence numbers (nblocks) of array iinput
c     isrinput  i   Array of sequence numbers (nblocks) of array rinput
c     issolut   i   Array of sequence numbers (nblocks) of array solut
c     turbonly  i   Shift only for turbulence equations
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ipsolut, nblocks, ndegfd,
     +        ndim, ni, nj, nk, ntimlv, nvirtual, npointsv,
     +        iprintim, iprinput, ipiintim, ipiintur, methodtim,
     +        mturb, viscorr, ipeddy, ipiincom, iprincom, mcom,
     +        mscal, mcav, ipiincav, istartup, vartyp, iprincav,
     +        newtcom, ipiindsc, mlin
      double precision theta

c     iblock    Block sequence number
c     ipeddy    Starting address of array eddy in buffer
c     ipiincav  Starting address of array IINCAV in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     iprinput  Starting address of array rinput in buffer
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ipsolut   Starting address of array solut in buffer
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     mcav      If 1 a cavity model is used, if 0 not
c     mcom      If zero the flow is incompressible else (1) compressible
c     methodtim Type of time integration
c     mlin      Type of linearization (1=newton, 2=picard)
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     mturb     Indicates type of turbulence modelling
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     newtcom   If one Newton linearisation is used for convective terms
c               if compressible flow.
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     theta     Parameter theta for theta method
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c     viscorr   Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISACTIVE  Activate old permanent array
c     ISCMRHO2  Computation of density, RHO ,from the equation of state, 2D
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISEXTCF2  Extrapolates a coefficient to virtual points
c     ISINGETP  get the pointer corresponding to an integer array
c     ISSHIF02  Shift a subdomain solut array
c     ISTHTA    Application of the second step of the theta method
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c         --- Remark I:
c
c         copying of time levels occurs by
c         passing it to iscopy twice with different dimensioning
c         parameters for the time levels
c
c         Source (0:ntimlv-1)          Target (1:ntimlv)
c
c         time level 1         <--->    time level 2
c         time level 0         <--->    time level 1
c
c         Therefore copying of time level itimlv by iscopy is in fact a
c         copying of time level itimlv+1 to itimlv
c
c         --- Remark II:
c
c         for compressible flow the density is updated after the shift of
c         the pressure and extrapolated to the virtual cells. This treatment
c         is now consistent with the treatment of the density in subroutine
c         iscom01
c         Finally the new timelevel of density eddy(,,2) is copied into
c         the old timelevel eddy(,,1) for use in the momentum equation
c         timederivative term.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- shift arrays of block iblock
c         determine some general parameters

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))

      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)

      do iblock = 1, nblocks

c     --- shift solut

         if ( iadmin(iblock).eq.inodenr .or. inodenr.eq.0 ) then

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(issolut(iblock))
            call isactive(iscoefs(iblock))

            ipiinput = isingetp(isiinput(iblock))
            iprinput = isdpgetp(isrinput(iblock))
            ipsolut  = isdpgetp(issolut(iblock))

            iprintim = ibuffr(ipiinput + 29 - 1)
            ipiintim = ibuffr(ipiinput +  4 - 1)
            ipiintur = ibuffr(ipiinput +  8 - 1)
            ipiincom = ibuffr(ipiinput +  9 - 1)
            iprincom = ibuffr(ipiinput + 34 - 1)
            iprincav = ibuffr(ipiinput + 35 - 1)
            ipiindsc = ibuffr(ipiinput +  6 - 1)

            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            theta    = buffer(iprinput - 1 + iprintim - 1 + 5 )
            methodtim= ibuffr(ipiinput - 1 + ipiintim - 1 + 1 )
            mturb    = ibuffr(ipiinput - 1 + ipiintur - 1 + 2 )
            viscorr  = ibuffr(ipiinput - 1 + ipiintur - 1 + 3 )
            mcom     = ibuffr(ipiinput + ipiincom - 1)
            mscal    = ibuffr(ipiinput + ipiincom + 4 - 1)
            mcav     = ibuffr(ipiinput + 77 - 1)
            mlin     = ibuffr(ipiinput - 1 + ipiindsc - 1 + 8 )

            if(mlin.eq.1.and.mcom.eq.1) then
               newtcom = 1
            else
               newtcom = 0
            end if

            if( mcav.eq.1) then
                ipiincav = ibuffr(ipiinput + 11 - 1)
                istartup = ibuffr(ipiinput -1 + ipiincav -1 + 2)
            else
                istartup = 0
            end if

            if ( mcom.eq.1 ) call isactive(iseddy(iblock))

            if ( mcom.eq.1 ) then
               ipeddy = isdpgetp(iseddy(iblock))
            else
               ipeddy = 1
            end if

            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            if ( ndim.eq.3 ) then
               npointsv = npointsv*(nk+1+2*nvirtual)
            end if

c           --- if (fractional/generalized) theta method is applied
c                 then extrapolation to the NTIMLV + 1 is necessary.

            if ( (methodtim.eq.1 .or. methodtim.eq.3 .or.
     +            methodtim.eq.4) .and.
     +           (theta.gt.1d-5 .and. theta.lt.0.99995d0) ) then

c           --- Theta method

               call isthta ( ndim, ndegfd, npointsv, ntimlv,
     +                       buffer(ipsolut), theta )
            end if

c           --- update the density in eddy(,,2)

            if ( mcom.eq.1 ) then

               call iscmrho2 ( ni, nj, ndegfd, ndim, nvirtual,
     +                         buffer(ipsolut),
     +                         buffer(iprinput-1+iprincom),
     +                         buffer(ipeddy), 1, ni, 1, nj,
     +                         mscal, mcav, 2, istartup,
     +                         buffer(iprinput-1+iprincav),
     +                         ibuffr(ipiinput-1+ipiincom) )

c              --- extrapolate the density in eddy(,,2)
c                                      to the virtual points

               vartyp = 3
               call isextcf2(buffer(ipeddy+npointsv),ni, nj,
     +                 nvirtual, vartyp )

c              --- copy the new timelevel density eddy(,,2) to the
c                   old timelevel eddy(,,1)

               call dcopy(npointsv,buffer(ipeddy+npointsv),1
     +                 ,buffer(ipeddy),1)

c              --- if use is made of the newton linearised momentum
c                  equation with density predictor, than
c                  copy the new timelevel density eddy(,,2) to the
c                       new timelevel solut(,4,1)

               if( newtcom.eq.1) then
                  call dcopy(npointsv,buffer(ipeddy+npointsv),1
     +                 ,buffer(ipsolut+6*npointsv),1)
                  call dcopy(npointsv,buffer(ipeddy+npointsv),1
     +                 ,buffer(ipsolut+7*npointsv),1)
               end if

            end if

c           --- shift eddy(,,3/4), store real old
c                                   rho*u rho*v in eddy 5 and 6

            if ( mcom.eq.1 ) then

                call dcopy(npointsv,buffer(ipeddy+2*npointsv),1
     +                 ,buffer(ipeddy+4*npointsv),1)
                call dcopy(npointsv,buffer(ipeddy+3*npointsv),1
     +                 ,buffer(ipeddy+5*npointsv),1)

            end if

c           --- shift solut

            call isshif02 ( ndim, npointsv, ndegfd, ntimlv,
     +                      buffer(ipsolut), mturb, viscorr, turbonly )

         end if

      end do

      end
