      subroutine issqrg2d ( geom, cva, ni, nj, nvirtual )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 11-06-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute covariant sqrt(g) in all points of the grid
c    two-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual
      double precision geom(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,4,2)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     geom      o    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERREAL       Put real value in error message
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     627   cell has volume zero
c **********************************************************************
c
c                       PSEUDO CODE
c
c     sqrt(g) is defined by
c                1    2       2    1
c     sqrt(g) = a    a     - a    a
c                (1)  (2)     (1)  (2)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- pnttype = 1:

      do j = 2-nvirtual, nj+nvirtual
         do i = 2-nvirtual, ni+nvirtual
            geom(i,j,1,1) = cva(i,j,1,1,1)*cva(i,j,2,2,1)-
     +                      cva(i,j,2,1,1)*cva(i,j,1,2,1)
            if ( abs(geom(i,j,1,1)).ge.epsmac ) then
               geom(i,j,1,2) = 1d0/geom(i,j,1,1)
            else
               call iseropen ( 'issqrg2d' )
               call iserrint ( i, 1 )
               call iserrint ( j, 2 )
               call iserreal ( geom(i,j,1,1), 1 )
               call iserrsub ( 627, 2, 1, 0 )
               call iserclos ( 'issqrg2d' )
            end if
         end do
      end do

c     --- pnttype = 2:

      do j = 2-nvirtual, nj+nvirtual
         do i = 1-nvirtual, ni+nvirtual
            geom(i,j,2,1) = cva(i,j,1,1,2)*cva(i,j,2,2,2)-
     +                      cva(i,j,2,1,2)*cva(i,j,1,2,2)
            geom(i,j,2,2) = 1d0/geom(i,j,2,1)
         end do
      end do

c     --- pnttype = 3:

      do j = 1-nvirtual, nj+nvirtual
         do i = 2-nvirtual, ni+nvirtual
            geom(i,j,3,1) = cva(i,j,1,1,3)*cva(i,j,2,2,3)-
     +                      cva(i,j,2,1,3)*cva(i,j,1,2,3)
            geom(i,j,3,2) = 1d0/geom(i,j,3,1)
         end do
      end do

c     --- pnttype = 4:

      do j = 1-nvirtual, nj+nvirtual
         do i = 1-nvirtual, ni+nvirtual
            geom(i,j,4,1) = cva(i,j,1,1,4)*cva(i,j,2,2,4)-
     +                      cva(i,j,2,1,4)*cva(i,j,1,2,4)
            geom(i,j,4,2) = 1d0/geom(i,j,4,1)
         end do
      end do

      end
