      subroutine issqrg3d ( geom, cva, ni, nj, nk, nvirtual )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 26-06-1997 1/sqrt(g) only if sqrt(g) > 0
c        version 1.0    date 11-06-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute covariant sqrt(g) in all points of the grid
c    three-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision geom(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,8,3)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,8)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     geom      o    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, dx(3), pnttyp, area(2,3)

c     area      Summation index bounds
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     pnttyp    reserved
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     sqrt(g) is defined as described in the mathematical manual
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      do pnttyp = 1, 8
         dx(3) = mod(pnttyp-1,8)/4
         dx(2) = mod(pnttyp-1,4)/2
         dx(1) = mod(pnttyp-1,2)
         area(1,1) = 2-nvirtual-dx(1)
         area(2,1) = ni+nvirtual
         area(1,2) = 2-nvirtual-dx(2)
         area(2,2) = nj+nvirtual
         area(1,3) = 2-nvirtual-dx(3)
         area(2,3) = nk+nvirtual

         do k = area(1,3), area(2,3)
            do j = area(1,2), area(2,2)
               do i = area(1,1), area(2,1)
                  geom(i,j,k,pnttyp,1) = cva(i,j,k,1,1,pnttyp)*
     +                (cva(i,j,k,2,2,pnttyp)*cva(i,j,k,3,3,pnttyp)-
     +                 cva(i,j,k,3,2,pnttyp)*cva(i,j,k,2,3,pnttyp)) +
     +                 cva(i,j,k,2,1,pnttyp)*
     +                (cva(i,j,k,3,2,pnttyp)*cva(i,j,k,1,3,pnttyp)-
     +                 cva(i,j,k,1,2,pnttyp)*cva(i,j,k,3,3,pnttyp)) +
     +                 cva(i,j,k,3,1,pnttyp)*
     +                (cva(i,j,k,1,2,pnttyp)*cva(i,j,k,2,3,pnttyp)-
     +                 cva(i,j,k,2,2,pnttyp)*cva(i,j,k,1,3,pnttyp))

                  if ( geom(i,j,k,pnttyp,1).gt.1d-16) then
                     geom(i,j,k,pnttyp,2) = 1d0/geom(i,j,k,pnttyp,1)
                  end if
               end do
            end do
         end do

      end do

      end
