      subroutine isstcr05 ( ntrnsp, nturb, ntype, iindsc,
     +                      oldnorms, norms, istation, absaccur,
     +                      relaccur, leniindsc )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      3.0  05-07-1997 Guus: remove ncells
c     version      2.3  11-02-1997 Kees K: ntype added
c     version      2.2  11-07-1996 Guus: correction for small velocity
c     version      2.1  17-07-1995 Erik: removed abs(..) in the test
c                                  for convergence because negative
c                                  values of stopcu mean that the
c                                  stopping criterion is not valid
c     version      2.0  28-02-1995 Guus: Extra parameter leniindsc
c     version      1.4  22-02-1995 Guus: New length of iindsc
c     version      1.3  31-01-1995 Michiel: Correction for excluding
c                                  momentum or transport eq.
c     version      1.2  12-01-1995 Michiel: implementation of stop when
c                                  stationary solution has been reached
c     version      1.1  01-18-1994 Marcel: implementation of excluding
c                                  momentum or transport
c                                  equation for print purposes
c     version      1.0  13-07-1994 Erik:  initial revision
c
c
c   copyright (c) 1994-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Perform some final computations on the norms and print the norms.
c     Save the norms in old norms afterwards.
c **********************************************************************
c
c                       KEYWORDS
c
c     stop_criterium
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ntrnsp, nturb, leniindsc, ntype,
     +        istation, iindsc(leniindsc,1+ntrnsp+nturb)
      double precision oldnorms(2+ntrnsp+nturb,2), absaccur
      double precision norms(2+ntrnsp+nturb,2), relaccur

c     absaccur  i   Absolute accuracy with respect to residual
c     iindsc    i   reserved
c     istation  i   Choice whether a stationary solution is needed
c     leniindsc i    Length of first component of array iindsc
c     norms     i   reserved
c     ntrnsp    i   Number of transport equations
c     nturb     i   Number of extra transport equations for turbulence
c     ntype     i  Type of norm used for stopcriterium
c     oldnorms  i/o Array containing computed norms
c     relaccur  i   Relative accuracy
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical noskip
      integer ieq, ifirstl, nstation
      double precision rho, stopcu, rhoh
      save ifirstl

c     ieq       Equation number
c     ifirstl   First call of this routine (=1) or not (=0)
c     noskip    Indicate whether a given equation is skipped or not
c     nstation  Indicates the number of stationary equations
c     rho       Estimate of the asymptotic convergence rate
c     rhoh      Help variable to store corrected value of rho
c     stopcu    Stopping criterion
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Perform some final computations on the norms
c
c     Print the norms
c
c     Copy norms to oldnorms
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data ifirstl /1/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- Initialize nstation

      nstation = 0

c     --- Perform some final computations on the norms

      if( ntype.eq.0 ) then

c     --- L2-norm

         do ieq = 1, 2 + ntrnsp + nturb

            norms(ieq,1) = sqrt( norms(ieq,1) )

            norms(ieq,2) = sqrt( norms(ieq,2) )

         end do

      end if

c     --- Print the norms

      if( ntype.eq.0 ) then

c     --- L2-norm

         write(irefwr,1000)

      else if ( ntype.eq.1 ) then

c     --- Max-norm

         write(irefwr,1010)

      end if

      do ieq = 1, 2 + ntrnsp + nturb

         if ( ieq.eq.1 .or. ieq.eq.2 ) then
           noskip = ( iindsc(1,1).ne.0 )
         else if ( ieq.gt.2 ) then
           noskip = ( iindsc(1,ieq-1).ne.0 )
         end if

         if ( ifirstl.eq.1 ) then

            if ( noskip ) write(irefwr,1001) ieq

         else

            if ( norms(ieq,1).lt.1d-20 .or.
     +           oldnorms(ieq,1).lt.1d-20 ) then

              if ( noskip ) then

                 write(irefwr,1002) ieq

              else

                nstation = nstation + 1

              end if

            else

c           --- Estimate the eigenvalue and store in rho
c               In order to prevent unwanted effects due to transients
c               rho is corrected such that it will never be smaller than 0.5

               rho = norms(ieq,1) / oldnorms(ieq,1)
               rhoh = max(abs(rho),0.5d0)

               if ( istation.eq.1 ) then

                  stopcu = (absaccur + relaccur*norms(ieq,2)) *
     +                     (1d0 - rhoh) / rhoh
                  if ( ieq.eq.1 ) stopcu = max(stopcu,1d-4)

c                 --- Count number of stationary solutions

                  if ( norms(ieq,1).le.stopcu .or.
     +                 .not. noskip ) then

                     nstation = nstation + 1

                  end if

               else

                  stopcu = 1d-2*(1d0 - rhoh)/rhoh*norms(ieq,2)

               end if

               if ( noskip ) write(irefwr,1003) ieq, rho, stopcu,
     +                                     norms(ieq,1)

            end if

         end if

      end do

c     --- Test whether we have reached a stationary solution

      if ( ifirstl.gt.10 .and. nstation.eq.2 + ntrnsp + nturb ) then

c     --- The first 10 time steps are not skipped, since the transient
c         may influence the stopping criterion

         istation = -1

      end if

c     --- Copy norms to oldnorms

      do ieq = 1, 2 + ntrnsp + nturb
         oldnorms(ieq,1) = norms(ieq,1)
         oldnorms(ieq,2) = norms(ieq,2)
      end do

      ifirstl = ifirstl+1

c     --- used formats

1000  format(/,' STOPPING CRITERION FOR A STATIONARY SOLUTION', /,/,
     +       ' Equation       ', 'Lambda1        ',  'Stoppingcrit.  ',
     +       'L2norm   ')

1010  format(/,' STOPPING CRITERION FOR A STATIONARY SOLUTION', /,/,
     +       ' Equation       ', 'Lambda1        ',  'Stoppingcrit.  ',
     +       'Maxnorm   ')

1001  format(I4, '        ',
     +       'not possible to compute, first time step')

1002  format(I4, '        ',
     +       ' norm less then 1d-20 no stopping criterion')

1003  format(1x,ss, I4, '        ',
     +          1pe13.4e2, '  ',
     +          1pe13.4e2, '   ',
     +          1pe13.4e2 )

      end
