      subroutine issysm ( iinput, rinput, matrix, gmat ,rhside, coefs,
     +                    solut , coor  , infmm , infmg, geom , infbnd,
     +                    bndcon, eddy  , work  , npf, rspv, cv )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      3.5   date    22-07-1998 Duncan: new call to ismom2d
c                                                   new call to ismom2dwb
c     version      3.4   date    09-06-1998 Charles: nvpoints changed
c     version      3.3   date    18-05-1998 Charles Bilinear discretization
c     version      3.2   date    26-04-1998 Guus: split wesbeek and classical,3D
c     version      3.1   date    16-02-1998 Guus: split wesbeek and classical
c                                           discretization
c     version      3.0   date    06-11-1997 Guus: remove lwork
c     version      2.5   date    06-02-1996 Kees K.: call of ismom3d changed
c     version      2.4   date    22-11-1995 Kees K.: call of isgmv changed
c     version      2.3   date    30-10-1995 Guus: new call to ismom2d
c     version      2.2   date    03-10-1995 Guus: right-hand side to ismom2d
c     version      2.1   date    01-09-1995 Kees: call ismom3d
c     version      2.0   date    19-07-1995 Guus: new parameter list
c     version      1.3   date    05-07-1995 Hester: cleaned up
c     version      1.2   date    30-05-1995 Hester: array eddy added
c     version      1.1   date    22-02-1995 Guus: incorporation prints
c     version      1.0   date    16-12-1993
c
c
c   copyright (c) 1993-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     create system of equations i.e. matrix and righthandside
c     momentum equation V*
c **********************************************************************
c
c                       KEYWORDS
c
c     momentum_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), rspv(*), npf(*),
     +        infmm(*), infmg(*), infbnd(*)
      double precision rinput(*), bndcon(*), matrix(*), gmat(*),
     +        rhside(*), coefs(*), solut(*), coor(*), geom(*),
     +        work(*), eddy(*), cv(*)

c     bndcon  i    reserved
c     coefs   i    reserved
c     coor    i    reserved
c     cv      i    reserved
c     eddy    i    reserved
c     geom    i    reserved
c     gmat    i    Gradient matrix
c     iinput  i    reserved
c     infbnd  i    reserved
c     infmg   i    Integer information for Pressure-gradient matrix
c     infmm   i    Integer information for Momentum matrix
c     matrix  i    reserved
c     npf     i    reserved
c     rhside  i    reserved
c     rinput  i    reserved
c     rspv    i    reserved
c     solut   i    reserved
c     work    i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiindsc, ipiintim, iprinmt, iprinrhs, idiscret,
     +        itime1, itime2, itimeact, itimest,
     +        ndim, ni, nj, nk, nvirtual, npointsv,
     +        ipvsum, ipdvsum, ipwork, ipinvjac, ipmct,
     +        ipmctg, ipmctinvg

c     idiscret  Type of discretization
c     ipdvsum   Starting address of array DVSUM in array WORK
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipinvjac  Starting address of array invjac in geom
c     ipmct     Pointer to mct in infmat
c     ipmctg    Pointer to mct in infmat
c     ipmctinvg Pointer to mctinv in infmat
c     iprinmt   Indicates whether the matrix must be printed or not
c               Possible values
c               0:  the matrix is not printed
c               1:  the standard matrix is printed
c               2:  the gradient matrix is printed
c               3:  the standard and the gradient matrix are printed
c               4:  the pressure matrix is printed
c               5:  the standard and the pressure matrix are printed
c               6:  the gradient and the pressure matrix are printed
c               7:  the standard, thegradient and the pressure matrix
c                   are printed
c     iprinrhs  Indicates whether the right-hand side must be printed or not
c               Possible values
c               0:  the right-hand side is not printed
c               1:  the standard right-hand side is printed
c               2:  the pressure right-hand side is printed
c               3:  the standard and the pressure  right-hand side are
c                   printed
c     ipvsum    Starting address of array VSUM in array WORK
c     ipwork    Starting address of array work in buffer
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISMOM2D   Compute the 2D momentum matrix
c     ISMOM2DWB Compute the 2D momentum matrix (WesBeek discretization)
c     ISMOM3D   Compute the 3D momentum matrix
c     ISMOM3DWB Compute the 3D momentum matrix (WesBeek discretization)
c     ISPRIMT1  Print 2D matrix
c     ISPRIMT2  Print 3D matrix
c     ISPRIRH1  Print 2D right-hand side
c     ISPRIRH2  Print 3D right-hand side
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('issysm')

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndim     = iinput(57)
      ipiintim = iinput(4)
      ipiindsc = iinput(6)
      idiscret = iinput(ipiindsc+1)
      iprinmt  = iinput(ipiindsc+10)
      iprinrhs = iinput(ipiindsc+11)
      itime1   = iinput(ipiindsc+12)
      itime2   = iinput(ipiindsc+13)
      itimest  = iinput(ipiindsc+14)
      itimeact = iinput(ipiintim+3)

      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if (ndim.eq.3)  npointsv = npointsv*(nk+1+2*nvirtual)

c     --- fill matrix and rhside(..., 1:ndim )

      if ( ndim.eq.2 ) then

         ni       = iinput(51)
         nj       = iinput(52)
         nvirtual = iinput(54)
         npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         ipvsum  = 1
         ipdvsum = ipvsum+2*npointsv
         ipwork  = ipdvsum+4*npointsv
         ipinvjac = npointsv*4 + 1
         ipmct    = infmm(11)
         if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c        --- WesBeek discretization

            call ismom2dwb ( iinput, matrix, rhside, solut, coefs,
     +                       work(ipwork), rinput, npointsv, infmm,
     +                       work(ipvsum), work(ipdvsum),
     +                       geom(ipinvjac), coor, infmm(ipmct), bndcon,
     +                       infbnd(1), infbnd(13), geom,
     +                       eddy, gmat, infmg )
         else

c        --- Classical discretization

            call ismom2d ( iinput, matrix, rhside, solut, coefs,
     +                     work(ipwork), rinput, npointsv, infmm,
     +                     work(ipvsum), work(ipdvsum), geom(ipinvjac),
     +                     coor, infmm(ipmct), bndcon, infbnd(1),
     +                     infbnd(13), geom, eddy, gmat,
     +                     infmg )
         end if

      else

         ni       = iinput(51)
         nj       = iinput(52)
         nk       = iinput(53)
         nvirtual = iinput(54)
         npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                 (nk+1+2*nvirtual)
         ipvsum  = 1
         ipinvjac = npointsv*(2**ndim) + 1
         ipmct    = infmm(11)
         ipmctg   = infmg(11)
         ipmctinvg   = infmg(12)
         if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c        --- WesBeek discretization

            ipwork = ipvsum+9*npointsv
            call ismom3dwb ( iinput, matrix, rhside, solut, coefs,
     +                       work(ipwork), npf, rspv, cv, rinput,
     +                       npointsv, infmm, work(ipvsum),
     +                       geom, geom(ipinvjac), coor,
     +                       infmm(ipmct), bndcon, infbnd(1),
     +                       infbnd(2*ndim+1),
     +                       infbnd(2*ndim*(1+2*(ndim-1))+1), gmat,
     +                       infmg(ipmctinvg) )

         else

c        --- Classical discretization

            ipdvsum = ipvsum+3*npointsv
            ipwork  = ipdvsum+9*npointsv
            call ismom3d ( iinput, matrix, rhside, solut, coefs,
     +                     work(ipwork), npf, rspv, cv, rinput,
     +                     npointsv, infmm, work(ipvsum), work(ipdvsum),
     +                     geom, geom(ipinvjac), coor,
     +                     infmm(ipmct), bndcon, infbnd(1),
     +                     infbnd(2*ndim+1),
     +                     infbnd(2*ndim*(1+2*(ndim-1))+1), gmat,
     +                     infmg(ipmctg), infmg(ipmctinvg) )

         end if

      end if

      if ( mod(iprinmt,2).eq.1 ) then

c     --- Print momentum matrix

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprimt1 ( 'momentum matrix', matrix, ni, nj,
     +                         infmm(6), 1 )
            else
               call isprimt2 ( 'momentum matrix', matrix, ni, nj, nk,
     +                         infmm(6), 1 )
            end if
         end if
      end if

      if ( mod(iprinmt,4).ge.2 ) then

c     --- Print g matrix

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprimt1 ( 'gradient matrix', gmat, ni, nj,
     +                         infmg(6), 3 )
            else
               call isprimt2 ( 'gradient matrix', gmat, ni, nj, nk,
     +                         infmg(6), 2 )
            end if
         end if

      end if
      if ( mod(iprinrhs,2).eq.1 ) then

c     --- Print momentum right-hand side

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then

               call isprirh1 ( 'momentum right-hand side', rhside,
     +                          ni, nj, 2 )
            else

               call isprirh2 ( 'momentum right-hand side', rhside,
     +                          ni, nj, nk, 3 )
            end if
         end if

      end if

      call iserclos('issysm')

      end
