      subroutine iswllfn2 ( iequa, iside, i1, i2, bndcon, rincof,
     +                      coefs, coor, eddy ,solut ,rintur,
     +                      ni, nj, ndegfd, nvirtual, ncoefs,
     +                      invjac, rough, wtemp, ibinfor, btype,
     +                      iseqpres )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date   10-09-1997 Charles: extra parameter iseqpres
c        version 2.0    date   04-06-1996 Marcel: wall law for temperature
c        version 1.3    date   13-09-1995 Marcel: linear interpolation of
c                                               BNDCON for momentum
c        version 1.2    date   12-09-1995 Marcel: parameter list iswllqn2
c                                               changed and correction
c                                               tauw
c        version 1.1    date   11-09-1995 Guus: correction tsign
c        version 1.0    date   28-08-1995 2D version of iswllfnc
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Use wall functions to fill BNDCON for the momentum and
c     dissipation rate equations
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iside, nvirtual, ni, nj, iequa, ncoefs, i1, i2,
     +        ibinfor(1-nvirtual:*), btype, ndegfd, iseqpres
      double precision coor(*), rintur(*), rincof(*), invjac(*),
     +                 rough, wtemp
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ncoefs),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 bndcon(2,1-nvirtual:*)

c     bndcon    o    reserved
c     btype     i    Type of boundary condition
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     i1        i    Lower index number along first direction of a face or edge
c     i2        i    Upper index number along first direction of a face or edge
c     ibinfor   o    Third part of infbnd
c     iequa     i    General equation sequence number
c     invjac    i    1/sqrt(g) in pnttyp
c     iseqpres  i    Sequence number of pressure equation
c     iside     i    Sequence number of side
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rincof    i    reserved
c     rintur    i    reserved
c     rough     i    The average roughness height
c     solut     i    reserved
c     wtemp     i    Wall temperature
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, inci, incj, i, ndir, tdir, nsign, tsign, ieq
      double precision ndist, physut, tauw, uplus, c, rhow, mu, tke,
     +                 cp, const, yplus, diffus

c     c         General help variable
c     const     General const
c     cp        Specific heat coefficient at constant pressure
c     diffus    Diffusivity (thermal)
c     i         General loop variable
c     ieq       Equation number
c     inci      Offset between vertices in i-direction
c     incj      Offset between vertices in j-direction
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     mu        The laminar viscosity
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     tauw      The wall shear stress
c     tdir      Direction of tangent vector on this side
c     tke       The turbulent kinetic energy
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     uplus     The dimensionless tangential velocity
c     yplus     The dimensionless normal distance to the wall
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISLOGW    Compute the wall shear stress with log-law based
c               wall condition depending on smoothness or roughness
c               of the boundary
c     ISWLLQN2  Determine essential wall quantities
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( iside.le.2 ) then
         ndir = 1
         tdir = 2
         jp = i1-1
         inci  = 0
         incj  = 1
         if ( iside.eq.1 ) then
            nsign = -1
            ip = 1
         else
            nsign = 1
            ip = ni
         end if
         tsign = nsign
      else
         ndir = 2
         tdir = 1
         ip = i1-1
         inci  = 1
         incj  = 0
         if ( iside.eq.3 ) then
            nsign = -1
            jp = 1
         else
            nsign = 1
            jp = nj
         end if
         tsign = -nsign
      end if

c     --- if ( iequa.ne.1 ) c = rintur(3)**(75d-2)

c         --- Decoupling allowed

      if ( iequa.ge.iseqpres ) c = rintur(3)**(75d-2)

      if ( btype.eq.34 ) ieq = iequa

      do i = i1, i2-1

         ip = min(ip+inci,ni)
         jp = min(jp+incj,nj)

c        --- First, compute two essential wall quantities: distance
c            perpendicular to the wall and physical tangential velocity

         call iswllqn2 ( ndist, physut, ni, nj, nvirtual, ip, jp,
     +                   coor, invjac, solut, ndir, tdir, nsign, tsign )

c        --- if ( iequa.eq.1 ) then

         if ( iequa.lt.iseqpres ) then

c        --- momentum equations; fill BNDCON using wall functions

            rhow = coefs(ip,jp,1)
            mu = coefs(ip,jp,2)-coefs(ip,jp,1)*eddy(ip,jp)
            tke = solut(ip,jp,ndegfd-1)

            call islogw ( tauw, uplus, physut, ndist,
     +                    tke, rintur, rhow, mu, rough )

            bndcon(2,2*i)   = -tauw
            bndcon(2,2*i-1) = 5d-1*( bndcon(2,2*i) + bndcon(2,2*i-2) )

         else if ( iequa.eq.ieq ) then

c        --- temperature equation; fill BNDCON using wall functions

            rhow = rincof(1)
            mu = rincof(2)
            diffus = coefs(ip,jp,4)-coefs(ip,jp,1)*eddy(ip,jp)/rintur(9)
            cp = coefs(ip,jp,1)/rhow
            tke = solut(ip,jp,ndegfd-1)
            yplus = rhow*( rintur(3)**(25d-2) )*sqrt(tke)*ndist/mu
            const = rintur(9)
            if ( yplus.le.11.3d0 ) const = cp*mu/diffus

            call islogw ( tauw, uplus, physut, ndist,
     +                    tke, rintur, rhow, mu, 0d0 )

            bndcon(1,2*i-1) = 0d0
            bndcon(1,2*i)   = bndcon(1,2*i-1)
            bndcon(2,2*i-1) = 0d0
            if ( tauw.ne.0d0 )
     +        bndcon(2,2*i-1) = cp*(wtemp-solut(ip,jp,ieq+1))*tauw/
     +                          (const*physut)
            bndcon(2,2*i)   = bndcon(2,2*i-1)

            btype = 2

         else

c        --- dissipation rate equation; fill BNDCON using wall functions

            tke = solut(ip,jp,ndegfd-1)
            bndcon(1,2*i-1)  = c*tke**(15d-1)/(rintur(1)*ndist)
            bndcon(1,2*i)  = bndcon(1,2*i-1)

         end if
         ibinfor(2*i-1) = btype
         ibinfor(2*i) = btype

      end do

c     --- if ( iequa.eq.1 ) then

      if ( iequa.lt.iseqpres ) then
        bndcon(2,2*i1-1) = 15d-1*bndcon(2,2*i1) - 5d-1*bndcon(2,2*i1+2)
      end if

      ibinfor(2*i2-1) = btype

c     --- if ( iequa.eq.1 ) then

      if ( iequa.lt.iseqpres ) then
         bndcon(2,2*i2-1) = 15d-1*bndcon(2,2*i2-2) -
     +                       5d-1*bndcon(2,2*i2-4)
      else
         bndcon(1,2*i2-1) = bndcon(1,2*i2-2)
         bndcon(2,2*i2-1) = bndcon(2,2*i2-2)
      end if

      end
