      subroutine isaxpy2d ( ni, nj, nvirtual, ishift, jshift, vecy,
     +                      intn, k, facs, vecx, iarea, ichoisdis )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 22-03-1996 Marcel: First order upwind scheme
c        version 1.0    date 23-05-1995
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Perform a vector update for ISNaS over a 2-dimensional region
c    The vector to be added may be shifted with respect to the update vector
c    The update looks as follows:
c    y(i,j) = y(i,j) + a * x(i+ishift,j+jshift)
c    Both arrays are defined over the complete region, however y is
c    restricted to the region without virtual points and x uses virtual
c    points.
c    In order to get an optimal code the shift has been applied in the
c    declaration instead of the code, so that the code can use the same
c    indices for the vector to be updated as the vector to be added.
c    Since this is dangerous trick the user must be aware of this application
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ishift, jshift, iarea(1:2,1:2), k,
     +        intn, ichoisdis
      double precision vecy(ni+1,nj+1), facs,
     +                 vecx(1-nvirtual-ishift:ni+nvirtual+1-ishift,
     +                      1-nvirtual-jshift:nj+nvirtual+1-jshift)

c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     intn      i    Integration point number
c     ishift    i    Shift in i-direction
c     jshift    i    Shift in j-direction
c     k         i    General loop variable
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecx      i    Vector to be added
c     vecy     i/o   Vector to be updated
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( ichoisdis.eq.0 ) then

c     --- central scheme

        do 110 j = iarea(1,2), iarea(2,2)
          do 100 i = iarea(1,1), iarea(2,1)
            vecy(i,j) = vecy(i,j)+facs*vecx(i,j)
100       continue
110     continue

      else

c     --- first order or higher order upwind scheme

        if ( intn.ne.1 ) then

c       --- integration points 2, 3, 4, 5

          if ( k.eq.1 ) then

            do 210 j = iarea(1,2), iarea(2,2)
              do 200 i = iarea(1,1), iarea(2,1)
                vecy(i,j) = vecy(i,j)+
     +                      5d-1*(1d0+sign(1d0,vecx(i,j)))*vecx(i,j)
200           continue
210         continue

          else if ( k.eq.2 ) then

            do 310 j = iarea(1,2), iarea(2,2)
              do 300 i = iarea(1,1), iarea(2,1)
                vecy(i,j) = vecy(i,j)+
     +                      5d-1*(1d0-sign(1d0,vecx(i,j)))*vecx(i,j)
300           continue
310         continue

          end if

        else

c       --- integration point 1

          do 410 j = iarea(1,2), iarea(2,2)
            do 400 i = iarea(1,1), iarea(2,1)
              vecy(i,j) = vecy(i,j)+facs*vecx(i,j)
400         continue
410       continue

        end if

      end if

      end
