      subroutine isaxyz3d ( ni, nj, nk, nvirtual, ishift, jshift,
     +                      kshift, izshift, jzshift, kzshift, vecy,
     +                      facs, vecz, vecx, iarea )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 02-06-95 (copy of isaxyz2d
c                                       of Guus Segal )
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Perform a vector*vector update for ISNaS over a 3-dimensional region
c    The vectors to be added may be shifted with respect to the update vector
c
c    The update looks as follows:
c
c    y(i,j,k) = y(i,j,k) + a * x(i+ishift,j+jshift,k+kshift) *
c                              z(i+izshift,j+jzshift,k+kshift)
c
c    All arrays are defined over the complete region, however y is
c    restricted to the region without virtual points and x and z use virtual
c    points.
c    In order to get an optimal code the shift has been applied in the
c    declaration instead of the code, so that the code can use the same
c    indices for the vector to be updated as the vector to be added.
c    Since this is dangerous trick the user must be aware of this application
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk,  nvirtual, ishift, jshift, kshift,
     +        izshift, jzshift, kzshift, iarea(1:2,1:3)
      double precision vecy(ni+1,nj+1,nk+1), facs,
     +                 vecx(1-nvirtual-ishift:ni+nvirtual+1-ishift,
     +                      1-nvirtual-jshift:nj+nvirtual+1-jshift,
     +                      1-nvirtual-kshift:nk+nvirtual+1-kshift),
     +                 vecz(1-nvirtual-izshift:ni+nvirtual+1-izshift,
     +                      1-nvirtual-jzshift:nj+nvirtual+1-jzshift,
     +                      1-nvirtual-kzshift:nk+nvirtual+1-kzshift)

c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     ishift    i    Shift in i-direction
c     izshift   i    Shift in i-direction with respect to vecz
c     jshift    i    Shift in j-direction
c     jzshift   i    Shift in j-direction with respect to vecz
c     kshift    i    Shift in k-direction
c     kzshift   i    Shift in k-direction with respect to vecz
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecx      i    Vector to be added
c     vecy     i/o   Vector to be updated
c     vecz      i    Second vector to be used in multiplication
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, j, k

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      do 120 k = iarea(1,3), iarea(2,3)
         do 110 j = iarea(1,2), iarea(2,2)
            do 100 i = iarea(1,1), iarea(2,1)
               vecy(i,j,k) = vecy(i,j,k)+facs*vecx(i,j,k)*vecz(i,j,k)
100         continue
110      continue
120   continue

      end
