      subroutine isbcs3t ( iface, alphad, locdx, ni, nj, nk,
     +                     ibinfor, ilow1, ihigh1, ilow2,
     +                     ihigh2, bndcon, prems, rhside )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 25-05-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Fill prescribed velocities in the term precm, which is used to build the
c   stress matrix. 3D WesBeek version
c   This subroutine is restricted to a tangential cell with cell center
c   at haf a cell distance of the boundary
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    discretization
c    momentum_equation
c    stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, alphad, iface, locdx(3),
     +        ilow1, ihigh1, ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2)
      double precision rhside(ni+1,nj+1,nk+1,3),
     +                 prems(ni+1,nj+1,nk+1,3,6),
     +                 bndcon(3,ilow1:ihigh1,ilow2:ihigh2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     prems    i/o   Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy
      parameter( unut = 4, uxuy = 10 )
      integer i, j, k, l, tdir(2), iarea(3,2), mxp(3), btype,
     +        valpha(3), ishiftlc(3), iextrash(2), ibounsh(2), idir,
     +        ih1, ih2, m, pnttyput, ishiftgl(3)
      double precision sum

c     btype     Type of boundary condition
c     i         General loop variable
c     iarea     Area of arrays
c     ibounsh   Array containing the shift in global coordinates with
c               respect to the boundary cells
c               For faces 1 and 2 ibounsh refers to j (1) and k (2)
c               For faces 3 and 4 ibounsh refers to k (1) and i (2)
c               For faces 5 and 6 ibounsh refers to i (1) and j (2)
c               Together with iextrash the position in the boundary cell
c               may be determined
c     idir      General loop variable for 1...ndim loop
c     iextrash  Array containing the extra shift in global coordinates with
c               respect to the boundary cells
c               Together with ibounsh the position in the boundary cell
c               may be determined as follows
c               For faces 1 and 2:  (2*(j+ibounsh(1))-1+iextrash(1),
c                                    2*(k+ibounsh(2))-1+iextrash(2))
c               For faces 3 and 4:  (2*(k+ibounsh(1))-1+iextrash(1),
c                                    2*(i+ibounsh(2))-1+iextrash(2))
c               For faces 5 and 6:  (2*(i+ibounsh(1))-1+iextrash(1),
c                                    2*(j+ibounsh(2))-1+iextrash(2))
c     ih1       Help variable to store a constant temporarily
c     ih2       Help variable to store a constant temporarily
c     ishiftgl  Array containing the shift in global coordinates
c     ishiftlc  Array containing the shift in local coordinates
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     m         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     pnttyput  Point type of point where the shift goes to
c     sum       Help variable to compute a sum
c     tdir      Direction of tangent vector on this side
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISGLBASH3 Compute the shift in global numbering given the local shift
c               and the cell center type
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Array prems(i,j,k,beta,gamma) contains the contribution of the velocity
c    to the stress tensor for a specific integration point with respect to the
c    V^alphad cell with center (i,j,k) (global coordinates)
c    beta (1,2,3) refers to the components of the Cartesian vector
c    Each integration point is surrounded by 6 points that are used to
c    express the gradient in terms of the velocity vector
c    Gamma refers to these six points
c    With respect to alphad = 1, the local shift of a gamma point compared
c    to an integration point is (-1,0,0), (1,0,0), (0,-1,0), ... , (0,0,1)
c    For alphad = 2 and 3 a cyclic permutation is used hence, for alphad
c    the sequence is (0,-1,0), (0,1,0), (0,0,-1), ...
c
c    If the cell center is on the boundary and the velocity is prescribed,
c    then the unknown itself is eliminated
c    hence in this subroutine we only consider points on the boundary that
c    correspond to cells with their center not on the boundary
c    There are two such cells:
c    Cells corresponding to a normal component at one cell distance of the
c    boundary and cells corresponding to a tangential component at half a
c    cell distance of the boundary
c
c    For a tangential cell we have much more points on the boundary
c    For example for face 1 (i=1), which implies alphad=2 or 3
c    we have an integration point on the boundary when locdx(1) = -1
c    For this integration point we have four neighbouring points on the
c    boundary and since an one-sided discretization is used, also the
c    point self is considered as a neighbouring point on the boundary
c    Points corresponding to locdx(1) = 0, are at a half cell distance of the
c    boundary, which implies that only one neighbouring point in the
c    alphad(=1) direction is positioned on the boundary
c    The same arguments can be used for the other faces
c
c    In this subroutine only the tangential cell is considered
c
c    For face 1 we have the following situation:
c
c       i=1, alphad=2,3
c       If alphad = 2 then: i=1, j=2(1)nj, k=1(1)nk
c       If locdx(1) = -1 then point is on the boundary
c       If locdx(1) = 0 then point is at half a cell distance of the boundary
c       If point on boundary then it has 6 neigbours, one of which is the
c       point itself.
c       This point is the point original indicated by the -alphad direction.
c       The point in the alphad direction is not on the boundary.
c       All points on the boundary with a given value of u must be
c       substituted in prems and rhside
c       If point at half a cell distance of the boundary then only the
c       point in the - alphad direction is on the boundary
c
c       If alphad = 3 then: i=1, j=1(1)nj, k=2(1)nk
c       Rest as alphad = 2
c
c       For the boundary points we have to perform the following substitution
c       rhside(i,j,k,alphad) = rhside(i,j,k,alphad) +
c                              prems(i,j,k,beta,gamma) * u_beta(boundary_point)
c
c       gamma refers to the neighbour point
c       Mark that prems is computed with respect to the specific integration
c       point
c    For face 2 we have i=ni, -alphad := alphad, -1 := 1, rest see face 1
c    For face 3 we have j=1, alpahd=3,1 k:=i, j:=k
c    Rest all cyclic
c
c    Pseudo code
c
c    Set under and upper indices for loop variables
c    Define tangential directions along the boundary (necessary for arrays
c    bndcon and ibinfor)
c    For all points on the boundary do
c       Find the type of boundary condition in the point
c       If ( velocity is prescribed in point ) then
c          update right-hand side
c          set corresponding value in prems equal to zero
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      valpha(1) = alphad
      valpha(2) = mod(alphad  ,3) + 1
      valpha(3) = mod(alphad+1,3) + 1

c     --- Set under and upper indices for loop variables

      iarea(1,1) = 1
      iarea(1,2) = ni
      iarea(2,1) = 1
      iarea(2,2) = nj
      iarea(3,1) = 1
      iarea(3,2) = nk

c     --- Define tangential directions along the boundary

      if ( iface.le.2 ) then

c     --- Faces 1 and 2

         tdir(1) = 2
         tdir(2) = 3
         if ( iface.eq.1 ) then
            iarea(1,1) = 1
            iarea(1,2) = 1
            idir = -1
         else
            iarea(1,1) = ni
            iarea(1,2) = ni
            idir = 1
         end if

      else if ( iface.le.4 ) then

c     --- Faces 3 and 4

         tdir(1) = 3
         tdir(2) = 1
         if ( iface.eq.3 ) then
            iarea(2,1) = 1
            iarea(2,2) = 1
            idir = -1
         else
            iarea(2,1) = nj
            iarea(2,2) = nj
            idir = 1
         end if

      else

c     --- Faces 5 and 6

         tdir(1) = 1
         tdir(2) = 2
         if ( iface.eq.5 ) then
            iarea(3,1) = 1
            iarea(3,2) = 1
            idir = -1
         else
            iarea(3,1) = nk
            iarea(3,2) = nk
            idir = 1
         end if

      end if
      if ( alphad.eq.1 ) then
         iarea(1,1) = 2
      else if ( alphad.eq.2 ) then
         iarea(2,1) = 2
      else
         iarea(3,1) = 2
      end if

c     --- First we consider the case of the integration point on the boundary

      if ( locdx((iface+1)/2).eq.idir ) then

c     --- Integration point on boundary
c         Loop over 6 surrounding points

         do m = 1, 6

            if ( valpha((m+1)/2).eq.(iface+1)/2  .and.
     +            ( idir.eq.-1 .and. mod(m,2).eq.0 .or.
     +              idir.eq. 1 .and. mod(m,2).ne.0 ) )
     +         then

c           --- This is the point m that is not on the boundary, hence no
c               action has to be performed

            else

c           --- point m on the boundary
c               Determine the position in array bndcon
c               The shift of the integration point is stored in locdx
c               The extra shift due to the corresponding point depends on m

               do l = 1, 3
                  ishiftlc(l) = locdx(l)
               end do
               if ( valpha((m+1)/2).ne.(iface+1)/2 ) then
                 ishiftlc(valpha((m+1)/2)) = ishiftlc(valpha((m+1)/2))
     +                                         +  (-1 )** m
               end if

c              --- Compute shifts with respect to boundary point

               call isglbash3 ( iface, alphad, ishiftlc, ishiftgl,
     +                          pnttyput, ibounsh, iextrash )

c              --- For all points on the boundary do

               do i = iarea(1,1), iarea(1,2)
                  do j = iarea(2,1), iarea(2,2)
                     do k = iarea(3,1), iarea(3,2)

c                    --- Find the type of boundary condition in the point

                        mxp(1) = i
                        mxp(2) = j
                        mxp(3) = k
                        ih1 = 2*(mxp(tdir(1))+ibounsh(1))-1+iextrash(1)
                        ih2 = 2*(mxp(tdir(2))+ibounsh(2))-1+iextrash(2)
                        btype = ibinfor(ih1,ih2)
                        if ( btype.eq.unut .or. btype.eq.uxuy ) then

c                       --- velocity is prescribed in point
c                           update right-hand side
c                           set corresponding value in prems equal to zero

                           sum = 0d0
                           do l =1 ,3
                              sum = sum+prems(i,j,k,l,m)*
     +                              bndcon(l,ih1,ih2)
                              prems(i,j,k,l,m) = 0d0
                           end do
                           rhside(i,j,k,alphad) =
     +                         rhside(i,j,k,alphad)-sum
                        end if
                     end do
                  end do
               end do

            end if

         end do

      else if ( locdx((iface+1)/2).eq.0 ) then

c     --- Integration point at half a cell distance of the boundary
c         Only the point in the (iface+1)/2  direction is on the boundary
c         point on the boundary
c         Determine the position in array bndcon
c         The shift of the integration point is stored in locdx
c         The extra shift due to the corresponding point depends on m
c         The sum of the shifts is stored in ishiftlc

         do m = 1, 6

            if ( valpha((m+1)/2).eq.(iface+1)/2  .and.
     +            ( idir.eq.-1 .and. mod(m,2).ne.0 .or.
     +              idir.eq. 1 .and. mod(m,2).eq.0 ) )
     +       then

c           --- point m at the boundary

               do l = 1, 3
                  ishiftlc(l) = locdx(l)
               end do
               ishiftlc(valpha((m+1)/2)) = ishiftlc(valpha((m+1)/2))
     +                                      + (-1) ** m

c              --- +                                      1 - 2*mod(m,2)

c                           --- Compute shifts with respect to boundary point

               call isglbash3 ( iface, alphad, ishiftlc, ishiftgl,
     +                           pnttyput, ibounsh, iextrash )

c              --- For all points on the boundary do

               do i = iarea(1,1), iarea(1,2)
                  do j = iarea(2,1), iarea(2,2)
                     do k = iarea(3,1), iarea(3,2)

c                    --- Find the type of boundary condition in the point

                        mxp(1) = i
                        mxp(2) = j
                        mxp(3) = k
                        ih1 = 2*(mxp(tdir(1))+ibounsh(1))-1+iextrash(1)
                        ih2 = 2*(mxp(tdir(2))+ibounsh(2))-1+iextrash(2)
                        btype = ibinfor(ih1,ih2)
                        if ( btype.eq.unut .or. btype.eq.uxuy ) then

c                       --- velocity is prescribed in point
c                           update right-hand side
c                           set corresponding value in prems equal to zero

                           sum = 0d0
                           do l =1 ,3
                              sum = sum+prems(i,j,k,l,m)*
     +                               bndcon(l,ih1,ih2)
                              prems(i,j,k,l,m) = 0d0
                           end do
                           rhside(i,j,k,alphad) =
     +                         rhside(i,j,k,alphad)-sum
                        end if
                     end do
                  end do
               end do
             end if
          end do

      end if
      end
