      subroutine isbmomc2 ( ni, nj, nvirtual, veloc, vsum, alphad,
     +                      densrho, invjac, chsym, locdx, dcell,
     +                      ibinfor, ibipoint, iindsc, adir, cva )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 18-05-1998 Charles Bilinear discretization
c        version 1.1    date 17-10-1996 Kees K. wesbeek discretization
c        version 1.0    date 29-05-1995 (Subroutine extracted from ismmcb2)
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c    determines the coefficients of the velocity components in a given
c    integration point within a boundary flux cell for the convection term in R2
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iindsc(*), locdx(2),
     +        dcell(2), iparea(2,2), alphad, ibipoint(*),
     +        ibinfor(*)
      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iindsc    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut,  periodic, antiper, multiblk,
     +        uxuy, unsnt,  perjump, antijump,
     +        oldturb, parabolic, skipval, masspar
      parameter( unut = 4,  periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           oldturb = 9, parabolic = 12)
      parameter (skipval = 300, masspar = 100)
      logical ungiven, noskip
      integer i, mip, mjp, xp(2), cbc, ip, jp, mxp(2), icbtype, btype,
     +        istwo(2,2), iside, tdir, ndir, nsign,
     +        mlin, ipibinf, idiscret, sign, betad, gammad, ish, jsh
      double precision rmult1, rmult2, fact, facs, u(2)

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     cbc       Boundary point number corresponding to the
c               integration cell centre
c     facs      Multiplication factor
c     fact      Multiplication factor
c     gammad    Direction index
c     i         General loop variable
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     idiscret  Type of discretization
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ish       Shift in i-direction
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mlin      Type of linearization (1=newton, 2=picard)
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     noskip    Indicate whether a given equation is skipped or not
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     rmult1    Multiplication factor
c     rmult2    Multiplication factor
c     sign      sign for multiplication (1 or -1)
c     skipval   This value indicates that effect of boundary condition
c               must be skipped
c     tdir      Direction of tangent vector on this side
c     u         Cartesian velocities
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       Relevant formula for an internal cell
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho      a  1 |i+1/2,j     rho      a  2 |i,j+1/2
c   ----- V  V  |         + -----    V  V  |          +
c   sqrt(g)        |i-1/2,j    sqrt(g)        |i,j-1/2
c
c
c    rho    |  a  |   c  b |
c   ----- |     |  V  V  |
c   sqrt(g) | b c |        |i,j
c
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho      a |i+1/2,j
c   ----- V  |         + -----    V  |          + -----    V  |         +
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g)     |i-1/2,j
c
c
c  2 rho    |  a  |   b |
c   ----- |     |  V  |
c   sqrt(g) | 1 b |     |i,j
c
c                                              2
c  and of course a similar expression for the V  component
c
c  Application of the Picard linearization (instead of the Newton lin.) gives
c  for the b-th momentum equation:
c
c     a  b     _a  b          b  b    _b  b
c    V  V   =  V  V   and    V  V  =  V  V
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              |-----+------|            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c   The cell centre is indicated by *, the integration points by +
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      idiscret = iindsc(2)
      mlin     = iindsc(8)

c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj
            tdir = 2
            ndir = 1
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               nsign = -1

            else

c           --- iside = 2:  right-hand side

               nsign = 1
               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni
            tdir = 1
            ndir = 2
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               nsign = -1

            else

c           --- iside = 4:  upper side

               nsign = 1
               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         noskip = locdx(ndir).ne.nsign .and. alphad.eq.ndir
         if ( noskip ) then

c        --- Determine multiplication factors due
c            to linearisation type

            if ( mlin.eq.1 ) then
               rmult1 = 2d0
               rmult2 = 1d0
            else
               rmult1 = 1d0
               rmult2 = 0.5d0
            end if

c           --- idiscret=2, Wesbeek discretization

            if ( alphad.eq.1 ) then
               ish = 0
               jsh = 0
               betad = 1
               if ( iside.eq.1) then
                  sign = -1
               else
                  sign = 1
               end if
               if ( locdx(1).ne.0 ) then
                  betad = 1
                  sign = locdx(1)
                   if ( sign.lt.0 ) ish = -sign
               else if ( locdx(2).ne.0 ) then
                  betad = 2
                  sign = locdx(2)
                  if ( sign.gt.0 ) jsh = -sign
               end if
            else
               ish = 0
               jsh = 0
               betad = 2
               if ( iside.eq.3) then
                  sign = -1
               else
                  sign = 1
               end if
               if ( locdx(1).ne.0 ) then
                  betad = 1
                  sign = locdx(1)
                   if ( sign.gt.0 ) ish = -sign
               else if ( locdx(2).ne.0 ) then
                  betad = 2
                  sign = locdx(2)
                  if ( sign.lt.0 ) jsh = -sign
               end if
            end if

c           --- loop over cells

            do 200 mjp = iparea(1,2), iparea(2,2)
               do 150 mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get boundary type for integration cell centre

                  btype = ibinfor(ipibinf+2*mxp(tdir))

c                 --- Check if un is given in half cell

                  ungiven = btype.eq.uxuy .or. btype.eq.unut .or.
     +                      btype.eq.parabolic .or. btype.eq.unsnt .or.
     +                      btype.eq.oldturb

c                 --- Get reduced boundary condition type

                  if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +                 btype.eq.antiper .or.
     +                 btype.eq.perjump .or. btype.eq.antijump .or.
     +                 btype.eq.masspar ) btype = skipval
                  icbtype = btype

c                 --- Get integration point index

                  xp(1) = mxp(1) + dcell(1)
                  xp(2) = mxp(2) + dcell(2)

                  ip = xp(1)
                  jp = xp(2)

c                 --- Get integration point position for ibinfor

                  cbc = 2*mxp(tdir)+locdx(tdir)

c                 --- Get boundary information

                  btype = ibinfor(ipibinf+cbc)

c                 --- Get reduced boundary condition type

                  if ( btype.eq.parabolic ) then
                     btype = unut
                  else if ( btype.eq.oldturb ) then
                     btype = unsnt
                  else if ( btype.eq.multiblk .or. btype.eq.periodic
     +                      .or. btype.eq.antiper .or.
     +                      btype.eq.perjump .or. btype.eq.antijump .or.
     +                      btype.eq.masspar ) then
                     btype = skipval
                  end if

                  if ( ungiven ) then

c                 --- Normal velocity given, no contribution to stencil

                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0

                  else if ( icbtype.ne.skipval ) then

c                 --- Centre of cell is not adjacent to another block,
c                     construct basic stencil for half cell

                     vsum(ip,jp,1) = 0
                     vsum(ip,jp,2) = 0

                     if ( locdx(ndir).eq.0 .and. locdx(tdir).eq.0 ) then

c                    --- Cell centre (0,0)
c                        First standard term

                        if ( idiscret.eq.1) then

c                       --- Standard isnas discretization

                           vsum(ip,jp,ndir) = vsum(ip,jp,ndir) +
     +                          nsign*rmult1*densrho(ip,jp)*
     +                          invjac(ip,jp)*veloc(ip,jp,ndir)

c                          --- Next term with Christoffel symbols

                           do 100 i = 1, 2

                             vsum(ip,jp,i) = vsum(ip,jp,i) +
     +                             invjac(ip,jp)*rmult2*densrho(ip,jp)*
     +                             (chsym(ip,jp,alphad,istwo(i,1))*
     +                             veloc(ip,jp,1)+
     +                             chsym(ip,jp,alphad,istwo(i,2))*
     +                             veloc(ip,jp,2))

100                        continue

                        else if (idiscret.eq.2.or.idiscret.eq.4) then

c                       --- Wesbeek discretization

                           if ( mlin.eq.1 ) then

c                          --- Newton linearization

                              gammad = 3-betad
                              facs = sign*densrho(ip,jp)*invjac(ip,jp)
                              u(1) = cva(ip,jp,1,1)*veloc(ip,jp,1)+
     +                               cva(ip,jp,1,2)*veloc(ip,jp,2)
                              u(2) = cva(ip,jp,2,1)*veloc(ip,jp,1)+
     +                               cva(ip,jp,2,2)*veloc(ip,jp,2)
                              vsum(ip,jp,betad) = facs*
     +                                 (adir(ip+ish,jp+jsh,1)*(u(1)+
     +                        cva(ip,jp,1,betad)*veloc(ip,jp,betad))+
     +                               adir(ip+ish,jp+jsh,2)*(u(2)+
     +                        cva(ip,jp,2,betad)*veloc(ip,jp,betad)))
                              vsum(ip,jp,gammad) = facs*
     +                                             veloc(ip,jp,betad)*
     +                      (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                       adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           else

c                          --- Picard linearization

                              gammad = 3-betad
                              facs = sign*densrho(ip,jp)*invjac(ip,jp)
                            vsum(ip,jp,betad) = facs*veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,betad)+
     +                        adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,betad))
                           vsum(ip,jp,gammad) = facs*veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                        adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           end if
                        end if

                     else if ( locdx(ndir).eq.-nsign ) then

c                    --- Point (0,1) (See picture)

                        if ( idiscret.eq.1) then

c                       --- Standard isnas discretization

                           vsum(ip,jp,ndir) = vsum(ip,jp,ndir)+
     +                          locdx(ndir)*rmult1*densrho(ip,jp)*
     +                          invjac(ip,jp)*veloc(ip,jp,ndir)
     +

                        else if (idiscret.eq.2.or.idiscret.eq.4) then

c                       --- Wesbeek discretization

                           if ( mlin.eq.1 ) then

c                          --- Newton linearization

                             gammad = 3-betad
                              facs = sign*densrho(ip,jp)*invjac(ip,jp)
                              u(1) = cva(ip,jp,1,1)*veloc(ip,jp,1)+
     +                             cva(ip,jp,1,2)*veloc(ip,jp,2)
                              u(2) = cva(ip,jp,2,1)*veloc(ip,jp,1)+
     +                             cva(ip,jp,2,2)*veloc(ip,jp,2)
                              vsum(ip,jp,betad) = facs*
     +                               (adir(ip+ish,jp+jsh,1)*(u(1)+
     +                        cva(ip,jp,1,betad)*veloc(ip,jp,betad))+
     +                             adir(ip+ish,jp+jsh,2)*(u(2)+
     +                        cva(ip,jp,2,betad)*veloc(ip,jp,betad)))
                              vsum(ip,jp,gammad) = facs*
     +                                           veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                       adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           else

c                          --- Picard linearization

                              gammad = 3-betad
                              facs = sign*densrho(ip,jp)*invjac(ip,jp)
                            vsum(ip,jp,betad) = facs*veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,betad)+
     +                        adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,betad))
                           vsum(ip,jp,gammad) = facs*veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                        adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           end if
                        end if

                     else if ( locdx(ndir).eq.0 .and.
     +                                     locdx(tdir).ne.0) then

c                    --- Point (1,0) or (-1,0)

                        if ( idiscret.eq.1) then

c                       --- Standard isnas discretization

                           if ( mlin.eq.1 ) then

c                          --- mlin=1, Newton lin. No contribution in
c                              tangential direction for Picard.

                              fact = 0.5d0*locdx(tdir)*densrho(ip,jp)*
     +                             invjac(ip,jp)*veloc(ip,jp,ndir)

                              vsum(ip,jp,tdir) = vsum(ip,jp,tdir)+fact

                           end if

                           vsum(ip,jp,ndir) = vsum(ip,jp,ndir)+
     +                        0.5d0*locdx(tdir)*densrho(ip,jp)*
     +                         invjac(ip,jp)*veloc(ip,jp,tdir)
                        else if (idiscret.eq.2.or.idiscret.eq.4) then

c                       --- Wesbeek discretization

                           if ( mlin.eq.1 ) then

c                          --- Newton linearization

                              gammad = 3-betad
                              facs =  (5d-1) *
     +                             sign*densrho(ip,jp)*invjac(ip,jp)
                              u(1) = cva(ip,jp,1,1)*veloc(ip,jp,1)+
     +                             cva(ip,jp,1,2)*veloc(ip,jp,2)
                              u(2) = cva(ip,jp,2,1)*veloc(ip,jp,1)+
     +                             cva(ip,jp,2,2)*veloc(ip,jp,2)
                              vsum(ip,jp,betad) = facs*
     +                               (adir(ip+ish,jp+jsh,1)*(u(1)+
     +                        cva(ip,jp,1,betad)*veloc(ip,jp,betad))+
     +                             adir(ip+ish,jp+jsh,2)*(u(2)+
     +                        cva(ip,jp,2,betad)*veloc(ip,jp,betad)))
                              vsum(ip,jp,gammad) = facs*
     +                                           veloc(ip,jp,betad)*
     +                       (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                        adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           else

c                          --- Picard linearization

                              gammad = 3-betad
                              facs = 5d-1 *
     +                            sign*densrho(ip,jp)*invjac(ip,jp)
                            vsum(ip,jp,betad) = facs*veloc(ip,jp,betad)*
     +                    (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,betad)+
     +                     adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,betad))
                          vsum(ip,jp,gammad) = facs*veloc(ip,jp,betad)*
     +                    (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                     adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))

                           end if
                        end if

                     end if

                  end if

150            continue

200         continue

         else if ( locdx(ndir).eq.nsign .and. alphad.eq.ndir ) then

c        --- Point outside region
c            loop over cells

            do 300 mjp = iparea(1,2), iparea(2,2)
               do 250 mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get integration cell position for ibinfor

                  btype = ibinfor(ipibinf+2*mxp(tdir))
                  if (btype.eq.multiblk .or. btype.eq.periodic .or.
     +               btype.eq.antiper .or.
     +               btype.eq.perjump .or. btype.eq.antijump .or.
     +               btype.eq.masspar) then
                     btype = skipval
                  end if

c                 --- Get integration point index

                  xp(tdir) = mxp(tdir) + dcell(tdir)
                  xp(ndir) = mxp(ndir) + dcell(ndir)

                  ip = xp(1)
                  jp = xp(2)
                  if ( btype.ne.skipval ) then
                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0
                  end if

250            continue
300         continue

         end if

      end do

      end
