      subroutine isbmomp2 ( ni, nj, nvirtual, alphad,
     +                      jctg, chsym, ibinfor,
     +                      ibipoint, bndcon, rhside)
c ======================================================================
c
c        programmer  Hester Bijl
c        version 2.0    date 18-02-1998 Guus remove dcell and locdx
c        version 1.0    date 10-12-1997
c
c   copyright (c) 1997-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute stress contribution to right-hand side for the Euler
c   equations for given pressure boundary condition, i.e.normal stress.
c   Similar to routine isbmoms2, but only snn pieces kept here. If
c   boundary condition handling is changed here, also change isbmoms2.
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, ibinfor(*), ibipoint(*)
      double precision bndcon(2,*),
     +   jctg(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,3),
     +   chsym(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,3),
     +   rhside(ni+1,nj+1,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter(snnsnt = 7,  snnut = 6)
      integer btype, ip, jp, xp(2), ipibinf, tsign,
     +        ict1, ndir, ilow, jlow, iupp, jupp,
     +        tdir, nsign, iside, istwo(2,2)
      double precision sigmann

c     btype     Type of boundary condition
c     ict1      Pointer in ibinfor with respect to boundary point
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     iupp      Upper i-index of subface
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jupp      Upper j-index of subface
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     sigmann   Value of sigma^nn
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c  1          a1 |i+1/2,j              a2 |i,j+1/2   1        |  a  |   cb |
c  - sqrt(g) T   |         + sqrt(g)  T   |        + -sqrt(g) |     |  T   |
c  2             |i-1/2,j                 |i,j       2        | b c |      |i,j
c
c
c   The boundary condition involving the pressure is sigmann,
c   so here, we will deal only with:
c
c   Sigma nn, Sigma nt given:       Replace all terms sigma nn
c                                   at the boundary by their given value.
c
c      Formulae:
c
c          nn    nn  nn
c     sigma   = g   S
c
c
c          nt        (n)                sqrt(g^nn)  nt    g_nt      nn
c     sigma   = sgn(a   .n) sgn(a   .t) ---------- S    - ---- sigma
c                   -    -      -(t) -  sqrt(g_tt)        g_tt
c
c
c     sqrt(g^nn)/sqrt(g_tt) = 1/sqrt(g)
c     g_nt/g_tt sigma^nn = -g^nt/g^nn g^nn S^nn = -g^nt S^nn
c
c   Sigma nn, ut given:             Replace all terms sigma nn
c                                   at the boundary by the given value.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1

c        --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jlow = 1
            jupp = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            ilow = 1
            iupp = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               jlow = nj+1
               jupp = nj+1

            end if

         end if
         if ( alphad.eq.ndir ) then

c        --- loop over cells

            do jp = jlow, jupp
               do ip = ilow, iupp

c              --- Get integration cell index

                  xp(1) = ip
                  xp(2) = jp

c                 --- Get boundary information

                  ict1 = ipibinf+2*xp(tdir)
                  btype = ibinfor(ict1)

                     if ( btype.eq.snnsnt ) then

c                    --- Both sigma^nn and sigma^nt given in integration point
c                        Get boundary condition information

                        sigmann = jctg(ip,jp,istwo(ndir,ndir))*
     +                            bndcon(1,ict1)

                        rhside(ip,jp,ndir) = rhside(ip,jp,ndir) +
     +                     nsign*sigmann + 0.5d0*
     +                     chsym(ip,jp,ndir,istwo(ndir,ndir))*
     +                     sigmann

                     else if ( btype.eq.snnut ) then

c                    --- sigma^nn provided and ut provided

                        sigmann = jctg(ip,jp,istwo(ndir,ndir))*
     +                            bndcon(1,ict1)

                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad)+
     +                     nsign*sigmann
                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad)+
     +                     0.5d0*
     +                     chsym(ip,jp,ndir,istwo(ndir,ndir))*sigmann

                     end if

               end do
            end do
         end if
      end do

      end
