      subroutine isbmoms1 ( ni, nj, nvirtual, iside, vsum, dvsum,
     +                      alphad, mu, jctg, chsym, dcell, ibinfor,
     +                      ibipoint, bndcon, rhside, matrix, invjac,
     +                      mct, nconct, reduced, mequat )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 2.0    date 08-10-1997 Hester: extra parameter mequat
c        version 1.5    date 30-01-1997 Hester: extra parameter nconct
c        version 1.4    date 04-01-1996 Marcel, correction outflow condition
c        version 1.3    date 15-11-1995 Guus, coorection for non-zero sigma^nt
c        version 1.2    date 28-09-1995 Kees V. use of subroutine isorient2
c        version 1.1    date 29-08-1995 Correction ni,nj adjacent sides
c        version 1.0    date 18-04-1995 Extracted from subroutine ismmsb2
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISnas Momentum Matrix Stress at the Boundary in 2 dimensions
c     Tangential cells only
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     matrix
c     stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iside, dcell(2), alphad, mequat,
     +        ibinfor(*), ibipoint(*), mct(2,2,-2:2,-2:2,-2:2), nconct
      double precision
     +   invjac(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,4),
     +   mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual),
     +   vsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2),
     +   jctg(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,3),
     +   chsym(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,3),
     +   rhside(ni+1,nj+1,2), matrix(ni+1,nj+1,2,*),
     +   bndcon(2,*)
      logical reduced

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     dcell     i    reserved
c     dvsum    i/o   Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     invjac    i    1/sqrt(g) in pnttyp
c     iside     i    Sequence number of side
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mu        i    The laminar viscosity
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     reduced   i    If true the reduced stress tensor corresponding
c                    to a constant viscosity and incompressible flow is used
c     rhside   i/o   reserved
c     vsum     i/o   Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, unsnt, di1, dj1, di2, dj2, iparea(2,2),
     +        walllaw, istwo(2,2)
      parameter( snnsnt = 7, unsnt = 5, walllaw = 9 )
      integer i, btype, mip, mjp, mxp(2), ict1, ip, jp, xp(2), ndir,
     +        tdir, nsign, tsign, iconct, ipibinf, ipibinsd
      double precision vfac, facs
      logical noskip

c     btype     Type of boundary condition
c     di1       First shift in i-direction
c     di2       Second shift in i-direction
c     dj1       First shift in j-direction
c     dj2       Second shift in j-direction
c     facs      Multiplication factor
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     ict1      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipibinsd  Starting address of array infor for adjacent side minus one
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     noskip    Indicate whether a given equation is skipped or not
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     unsnt     U_n and sigma_nt prescribed
c     vfac      Factors for derivation of flux if not known
c               explicitly
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j              a2 |i,j+1/2            |  a  |   cb |
c   sqrt(g)  T   |         + sqrt(g)  T   |        +  sqrt(g) |     |  T   |
c                |i-1/2,j                 |i,j-1/2            | b c |      |i,j
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c
c   If the incompressibility condition is substituted then:
c
c    12         11 2      22 1
c  -T   = mu ( g  U    + g  U   )
c                  ,1        ,2
c
c
c   In the reduced case we have T^11 = mu (U^1,1)
c                               T^12 = mu (U^1,2)
c                    a                  a        b     |  a   |  c
c    The derivative U   is equal to  d U  / d Ksi    + |      | U
c                    ,b                                |b   c |
c
c
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsign=(-1,1)
c
c
c
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c   The following boundary conditions have the following effects:
c
c
c   Dirichlet boundary conditions:  do not change the matrix elements
c                                   Use the folding subroutine for the
c                                   virtual velocities
c
c   Sigma nn, ut given:             no sigmann term at the boundary
c                                   Folding routine for ut
c
c   Sigma nt, un given:
c      The only point at the boundary is the point (0,-1)
c      un does not need any precautions
c      sqrt(g) sigma^nt = S^nt - sqrt(g) g_21/g_11 sigma^nn
c      So Snt to right-hand side and the effect of sigma^nn to the matrix
c      General form:
c
c            (n)               nt                 nt
c      -sgn(a   .n) sqrtg sigma   = -sgn(t.a   ) S   +
c           -    -                       - -(t)
c
c                                          (n)          g_nt      nn
c                                   + sgn(a   .n) sqrtg ---- sigma
c                                         -    -        g_tt
c
c      g_nt / g_tt = -g^nt / g^nn
c      The sigma^nn term requires some one-sided differentation in the point
c      (0,-1) since not all derivatives of velocity components can be
c      evaluated.
c
c      We have sigma^nn = 2 mu ( G^nt U^n,t + g^nn U^n,n )
c      Since  n     n      n  |   n   |   a
c            U  = dU /d ksi + |       | U
c             ,n              | n   a |
c
c      we first compute the factor before U^n,n and compute the
c      Christoffel symbol part that is stored in vsum
c
c      Next dU^n / d ksi^n is approximated by  U^n          - U^n
c                                                 | b-nsign       | b
c
c      The factor is removed from dvsum and added to the appropriate vsum
c      values using the correct signs.
c      This is possible since it concerns the same integration points,
c      however in different cells
c   Sigma nt, Sigma nn given:
c      The only point at the boundary is the point (0,-1)
c      sigma^nt in this point may be computed directly and must be added to
c      the right-hand side
c      The contribution to the matrix is zero
c      Formulae:
c
c            (n)               nt                 nt
c      -sgn(a   .n) sqrtg sigma   = -sgn(t.a   ) S   +
c           -    -                       - -(t)
c
c                                          (n)          g^nt      nn
c                                   - sgn(a   .n) sqrtg ---- sigma
c                                         -    -        g^nn
c
c
c                nn    nn  nn
c      with sigma   = g   S
c
c    The comments refer to a cell at the lower boundary
c
c    This subroutine uses all tangential unknowns along the side,
c    except when the first and last unknown (corresponding to adjacent sides),
c    do not correspond to periodical or multiblock boundary conditions
c    In that case the boundary unknown is skipped
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      iparea(1,1) = 1
      iparea(1,2) = 1
      iparea(2,1) = 1
      iparea(2,2) = 1
      ipibinf = ibipoint(iside)+nvirtual-1

c     --- Compute tdir and ndir

      call isorient2 ( iside, tdir, ndir, tsign, nsign )

      if ( iside.le.2 ) then

c     --- iside = 1, 2:  left- and right-hand side

         iparea(1,2) = 1
         iparea(2,2) = nj+1
         if ( iside.eq.1 ) then

c        --- iside = 1:  left-hand side

            di1 = 0
            dj1 = 0
            di2 = 0
            dj2 = -1
            ipibinsd = ibipoint(3)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,2) = 2
            ipibinsd = ibipoint(4)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,2) = nj

         else

c        --- iside = 2:  right-hand side

            iparea(1,1) = ni
            iparea(2,1) = ni
            di1 = 1
            dj1 = 0
            di2 = 1
            dj2 = -1
            ipibinsd = ibipoint(3)+nvirtual-1
            btype = ibinfor(ipibinsd+2*ni)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,2) = 2
            ipibinsd = ibipoint(4)+nvirtual-1
            btype = ibinfor(ipibinsd+2*ni)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,2) = nj

         end if

      else

c     --- iside = 3, 4:  lower- and upper side

         iparea(1,1) = 1
         iparea(2,1) = ni+1
         if ( iside.eq.3 ) then

c        --- iside = 3:  lower side

            di1 = -1
            dj1 = 1
            di2 = 0
            dj2 = 1
            ipibinsd = ibipoint(1)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,1) = 2
            ipibinsd = ibipoint(2)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,1) = ni

         else

c        --- iside = 4:  upper side

            iparea(1,2) = nj
            iparea(2,2) = nj
            di1 = -1
            dj1 = 0
            di2 = 0
            dj2 = 0
            ipibinsd = ibipoint(1)+nvirtual-1
            btype = ibinfor(ipibinsd+2*nj)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,1) = 2
            ipibinsd = ibipoint(2)+nvirtual-1
            btype = ibinfor(ipibinsd+2*nj)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,1) = ni

         end if

      end if

c     --- loop over cells

      do mjp = iparea(1,2), iparea(2,2)
         do mip = iparea(1,1), iparea(2,1)

c        --- Get integration cell index

            mxp(1) = mip
            mxp(2) = mjp

c           --- Get integration point index

            xp(1) = mxp(1) + dcell(1)
            xp(2) = mxp(2) + dcell(2)

            ip = xp(1)
            jp = xp(2)

c           --- Get integration point position for ibinfor

            ict1 = ipibinf+2*xp(tdir)-1
            btype = ibinfor(ict1)
            if ( btype.eq.walllaw ) btype = unsnt

            if ( btype.eq.unsnt .or. btype.eq.89 ) then

c           --- un and Snt given
c               Get boundary condition information
c               vfac contains the multiplication factor for sigma^nn
c               sigma nt almost given in this point
c               Replace nsign sqrt(g) sigma^nt  in
c               (0,nsign/2) for horizontal sides
c               (nsign/2,0) for vertical sides
c               Put contribution of S^nt into right-hand side

               rhside(mip,mjp,alphad) = rhside(mip,mjp,alphad) +
     +                                  tsign * bndcon(2,ict1)

               vsum(ip,jp,tdir) = 0d0
               vsum(ip,jp,ndir) = 0d0
               dvsum(ip,jp,tdir,tdir) = 0d0
               dvsum(ip,jp,tdir,ndir) = 0d0
               if ( reduced ) then
                  facs = 1d0
               else
                  facs = 2d0
               end if
               vfac = -nsign*jctg(ip,jp,istwo(ndir,tdir))/
     +                jctg(ip,jp,istwo(ndir,ndir))*facs*mu(ip,jp)

c              --- Compute contribution to sigma^nn term
c                  Multiply by vfac

               dvsum(ip,jp,ndir,ndir) =
     +            vfac*jctg(ip,jp,istwo(ndir,ndir))
               dvsum(ip,jp,ndir,tdir) =
     +            vfac*jctg(ip,jp,istwo(ndir,tdir))

               if ( nconct.eq.17 .and. mequat.eq.0 ) then

c              --- compressible stress tensor sigma^nn also has
c                  -2/3 mu div(u)

                 dvsum(ip,jp,1,1)=dvsum(ip,jp,1,1)+nsign*vfac/3d0
                 dvsum(ip,jp,2,2)=dvsum(ip,jp,2,2)+nsign*vfac/3d0

               end if

               do i = 1, 2

                  vsum(ip,jp,1) = vsum(ip,jp,1)+
     +               dvsum(ip,jp,ndir,i)*
     +               chsym(ip,jp,ndir,istwo(i,1))
                  vsum(ip,jp,2) = vsum(ip,jp,2)+
     +               dvsum(ip,jp,ndir,i)*
     +               chsym(ip,jp,ndir,istwo(i,2))

               end do

c              --- Correction for one-sided difference

               vsum(ip,jp,ndir) = vsum(ip,jp,ndir) +
     +                            nsign*dvsum(ip,jp,ndir,ndir)
               vfac = 0.5d0*nsign*dvsum(ip,jp,ndir,ndir)
               iconct = mct(alphad,ndir,di1,dj1,0)
               matrix(mip,mjp,alphad,iconct) =
     +                     matrix(mip,mjp,alphad,iconct) -
     +                     vfac*invjac(mip+di1,mjp+dj1,4-ndir)
               iconct = mct(alphad,ndir,di2,dj2,0)
               matrix(mip,mjp,alphad,iconct) =
     +                     matrix(mip,mjp,alphad,iconct) -
     +                     vfac*invjac(mip+di2,mjp+dj2,4-ndir)
               dvsum(ip,jp,ndir,ndir) = 0d0

            else if ( btype.eq.snnsnt ) then

c           --- Known stress
c               Point (0,-1)
c               Contribution of matrix is zero

               dvsum(ip,jp,1,1) = 0d0
               dvsum(ip,jp,1,2) = 0d0
               dvsum(ip,jp,2,1) = 0d0
               dvsum(ip,jp,2,2) = 0d0
               vsum(ip,jp,1) = 0d0
               vsum(ip,jp,2) = 0d0

               rhside(mip,mjp,alphad) = rhside(mip,mjp,alphad) +
     +                                  tsign*bndcon(2,ict1) + nsign*
     +                                  jctg(ip,jp,istwo(ndir,tdir))*
     +                                  bndcon(1,ict1)

            end if

         end do
      end do
      end
