      subroutine isbms23d ( ni ,nj ,nk ,nvirtual, ilow1, ihigh1,
     +                      ilow2, ihigh2 , vsum, dvsum , alphad,
     +                      mu, nzvsum, nzdvsum, locjac, jctg, chsym,
     +                      locdx, dcell, ibinfor, rhside, iface )
c ======================================================================
c
c        programmer  Kees Kassels
c        version 1.0    date 29-06-95 (extracted from ismmsb3 of
c                                      Marcel Zijlema)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c*********************************************************************
c
c                      DESCRIPTION
c
c     ISnas Momentum Matrix Stress at the Boundary in 3 dimensions
c
c*********************************************************************
c
c                      KEYWORDS
c
c     3d
c     boundary_conditions
c     matrix
c     stress
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1, ilow2,
     +        ihigh2, locdx(3), dcell(3), alphad,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface
      double precision locjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)
      logical nzvsum(3),nzdvsum(3,3)

      double precision
     +   mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +      1-nvirtual:nk+1+nvirtual),
     +   vsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +        1-nvirtual:nk+1+nvirtual,3),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +         1-nvirtual:nk+1+nvirtual,3,3),
     +   jctg(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +        1-nvirtual:nk+1+nvirtual,6),
     +   chsym(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +         1-nvirtual:nk+1+nvirtual,3,6),
     +   rhside(ni+1,nj+1,nk+1,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     dcell     i    reserved
c     dvsum     o    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     istwo     i    Translation table to store and retrieve the
c                    elements of a symmetric ndim-dimensional
c                    rank 2 tensor from a vector with length
c                    (ndim*(ndim+1))/2
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     locjac    i    Array containing the Jacobian for all integration
c                    points of one type
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     nzdvsum   o    Indicates whether or not a partial derivative in a
c                    certain direction occurs in the stencil.
c                    For example the derivative dU2/dksi1 in the point (0,1)
c                    or the V1-stencil of the 13-point molecule does not exist
c                    nzdvsum should therefore be false in that point
c     nzvsum    o    Indicates whether or not a velocity in a
c                    certain direction has a contribution in the stencil.
c     rhside    i    reserved
c     vsum      o    Flux contributions to the matrix
c
c*********************************************************************
c
c                      COMMON BLOCKS
c
c*********************************************************************
c
c                      LOCAL PARAMETERS

      integer istwo(3,3), iparea(2,3)

      integer unut, snnsnt, periodic, antiper, multiblk,
     +        uxuy, unsnt, snnut, perjump, antijump,
     +        oldturb, parabolic, skip, masspar
      parameter ( skip = 300, masspar = 100)
      parameter( unut = 4, snnsnt = 7, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5, snnut = 6,
     +           oldturb = 9, parabolic= 12)

      logical ungiven
      integer i, j, btype, mip, mjp, mkp, mxp(3),
     +        ict1, ict2, icbtype, ip, jp, kp, xp(3), ndir,
     +        tdir(2),  nsig, iptensor1, iptensor2, iptensor3,
     +        gammal, deltal
      double precision vval(3)

      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     btype     Type of boundary condition
c     deltal    Loop index
c     gammal    Loop index
c     i         General loop variable
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iptensor1 Tensor index
c     iptensor2 Tensor index
c     iptensor3 Tensor index
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mkp       Integration cell indices
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     skip      If true boundary condition must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     vval      Flux values
c     xp        Auxiliary array to calculate boundary point number
c
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      METHOD
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j,k              a2 |i,j+1/2,k
c   sqrt(g)  T   |           + sqrt(g)  T   |          +
c                |i-1/2,j,k                 |i,j-1/2,k
c
c
c              a3 |i,j,k+1/2              |  a  |   cb |
c     sqrt(g) T   |           +   sqrt(g) |     |  T   |
c                 |i,j,k-1/2              | b c |      |i,j,k
c
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c
c
c                    a                  a        b     |  a   |  c
c    The derivative U   is equal to  d U  / d Ksi    + |      | U
c                    ,b                                |b   c |
c
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(3,1,2)
c       ntsig=(-1,1,1)
c
c   The picture shows the intersection with the plane ksi2 = constant (0)
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c   The cell centre is indicated by *, the possible integration points by +
c
c   The stress term for this cell has the following shape:
c
c  1          a1 |i+1/2,j,k    1         a2 |i,j+1/2,k            a3 |i,j,k+1/2
c  - sqrt(g) T   |          +  -sqrt(g) T   |          + sqrt(g) T   |
c  2             |i-1/2,j,k    2            |i,j-1/2,k               |i,j,k
c
c
c      1         |  a  |  bc |
c    + - sqrt(g) |     | T   |
c      2         | b c |     |i,j,k
c
c
c   The following boundary conditions have the following effects:
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:       Replace all terms sigma nn and sigma nt
c                                   at the boundary by their given value
c
c   Sigma nn, ut1 and ut2 given:    Replace all terms sigma nn
c                                   at the boundary by the given value
c                                   Use the folding subroutine to eliminate
c                                   virtual tangential velocities
c
c
c=======================================================================
c
      call iseropen ( 'isbms23d' )

         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         iparea(1,3) = 1
         iparea(2,3) = 1

         if ( iface.le.2 ) then

c        --- iface = 1, 2:  back and front plane

            iparea(2,2) = nj
            iparea(2,3) = nk
            ndir    = 1
            tdir(1) = 2
            tdir(2) = 3
            nsig    = -1



            if ( iface.eq.2 ) then

c           --- iface = 2:  front

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1
               nsig    =  1

            end if

         else if ( iface.gt.2  .and. iface .lt. 5) then

c        --- iface = 3, 4:  left- and right plane

            iparea(2,1) = ni
            iparea(2,3) = nk
            ndir = 2
            tdir(1) = 3
            tdir(2) = 1
            nsig    = -1

            if ( iface.eq.4 ) then

c           --- iface = 4:  right side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1
               nsig    =  1


            end if

         else if ( iface.gt.4  ) then

c        --- iface = 5, 6:  bottom and top

            iparea(2,1) = ni
            iparea(2,2) = nj
            ndir = 3
            tdir(1) = 1
            tdir(2) = 2
            nsig    = -1

            if ( iface.eq.6 ) then

c           --- iface = 6:  upper side

               iparea(1,3) = nk+1
               iparea(2,3) = nk+1
               nsig    =  1

            end if

         end if

c     --- loop over cells

      do mkp = iparea(1,3), iparea(2,3)
        do mjp = iparea(1,2), iparea(2,2)
          do mip = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

            mxp(1) = mip
            mxp(2) = mjp
            mxp(3) = mkp

c           --- Get integration cell position for ibinfor

            if ( alphad.eq.tdir(1) ) then
               ict1 = 2*mxp(tdir(1))-1
            else
               ict1 = 2*mxp(tdir(1))
            end if
            if ( alphad.eq.tdir(2) ) then
               ict2 = 2*mxp(tdir(2))-1
            else
               ict2 = 2*mxp(tdir(2))
            end if
            icbtype = ibinfor(ict1,ict2)

c           --- Check for Dirichlet condition in half cell

            ungiven = ( alphad.eq.ndir .and. (icbtype.eq.uxuy .or.
     +                 icbtype.eq.unut .or. icbtype.eq.parabolic .or.
     +                 icbtype.eq.unsnt .or. icbtype.eq.oldturb ) )

c           --- Get reduced boundary condition type

            btype = icbtype
            if ( btype.eq.parabolic ) btype = unut
            if ( btype.eq.oldturb ) btype = unsnt
            if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +           btype.eq.antiper  .or. btype.eq.perjump  .or.
     +           btype.eq.antijump .or. btype.eq.masspar ) then
              btype = skip
            end if
            icbtype = btype

c           --- Get integration point index

            xp(tdir(1)) = mxp(tdir(1)) + dcell(tdir(1))
            xp(tdir(2)) = mxp(tdir(2)) + dcell(tdir(2))
            xp(ndir )   = mxp(ndir )   + dcell(ndir )

            ip = xp(1)
            jp = xp(2)
            kp = xp(3)

c           --- Get integration point position for ibinfor

            if (alphad.eq.ndir) then
               ict1 = 2*mxp(tdir(1))+locdx(tdir(1))
               ict2 = 2*mxp(tdir(2))+locdx(tdir(2))
            else if (alphad.eq.tdir(1)) then
               ict1 = 2*mxp(tdir(1))-1+locdx(tdir(1))
               ict2 = 2*mxp(tdir(2))+locdx(tdir(2))
            else if (alphad.eq.tdir(2)) then
               ict1 = 2*mxp(tdir(1))+locdx(tdir(1))
               ict2 = 2*mxp(tdir(2))-1+locdx(tdir(2))
            end if

c           --- Get boundary information



               btype = ibinfor(ict1,ict2)


c           --- Get reduced boundary condition type

            if ( btype.eq.parabolic ) btype = unut
            if ( btype.eq.oldturb ) btype = unsnt
            if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +           btype.eq.antiper .or. btype.eq.perjump .or.
     +           btype.eq.antijump .or. btype.eq.masspar ) then
              btype = skip
            end if

c           --- Local contribution of covariant derivatives
c               to christoffel symbols

            if ( ungiven ) then

c           --- Dirichlet half cell
c           --- Normal velocity given, no contribution to stencil

              do gammal = 1, 3
                vsum(ip,jp,kp,gammal) = 0d0
                do deltal = 1, 3
                  dvsum(ip,jp,kp,gammal,deltal) = 0d0
                end do
              end do


            else if ( alphad.eq.ndir .and. icbtype.ne.skip ) then

c           --- velocity integration cell (half cell for V^ndir)
c               Construct basic stencil for half cell

               do gammal = 1, 3
                 vsum(ip,jp,kp,gammal) = 0d0
                 do deltal = 1, 3
                   dvsum(ip,jp,kp,gammal,deltal) = 0d0
                 end do
               end do

               if ( locdx(tdir(1)).eq.0 .and.
     +              locdx(tdir(2)).eq.0 .and.
     +              locdx(ndir ).eq.0      ) then

c              --- Point (0,0,0)
c              The terms involving sigma^(t1 t1), sigma^(t1 t2) and
c              sigma^(t2 t2) are common for both boundary conditions

               iptensor1 = istwo(tdir(1),tdir(1))
               iptensor2 = istwo(tdir(1),tdir(2))
               iptensor3 = istwo(tdir(2),tdir(2))

               do i = 1, 3
                 nzdvsum(tdir(1),i) = .true.
                 dvsum(ip,jp,kp,tdir(1),i) =
     +                 -0.5d0*2d0*mu(ip,jp,kp)*
     +                 chsym(ip,jp,kp,ndir,iptensor1)*
     +                 jctg(ip,jp,kp,istwo(tdir(1),i))
     +                 -0.5d0*2d0*mu(ip,jp,kp)*
     +                 chsym(ip,jp,kp,ndir,iptensor2)*
     +                 jctg(ip,jp,kp,istwo(tdir(2),i))
               end do

               do i = 1, 3
                 nzdvsum(tdir(2),i) = .true.
                 dvsum(ip,jp,kp,tdir(2),i) =
     +                 -0.5d0*2d0*mu(ip,jp,kp)*
     +                 chsym(ip,jp,kp,ndir,iptensor2)*
     +                 jctg(ip,jp,kp,istwo(tdir(1),i))
     +                 -0.5d0*2d0*mu(ip,jp,kp)*
     +                 chsym(ip,jp,kp,ndir,iptensor3)*
     +                 jctg(ip,jp,kp,istwo(tdir(2),i))
               end do

               else if ( locdx(tdir(1)).eq.0 .and.
     +                   locdx(tdir(2)).eq.0 .and.
     +                   locdx(ndir).eq.-nsig ) then

c              --- Point (0,0,1)
c                  Term sqrt(g)*sigma^33

                 do i= 1, 3
                   nzdvsum(ndir,i) = .true.
                   dvsum(ip,jp,kp,ndir,i) = nsig*2d0*mu(ip,jp,kp)*
     +                                      jctg(ip,jp,kp,istwo(i,ndir))
                 end do

               else if ( locdx(tdir(1)).ne.0 .and.
     +                   locdx(tdir(2)).eq.0 .and.
     +                   locdx( ndir).eq.0      ) then

c              --- Points (-1,0,0) and (1,0,0)
c                  Term 0.5*sqrt(g)*sigma^31

                 nzdvsum(tdir(1),tdir(1)) = .true.
                 nzdvsum(tdir(1),tdir(2)) = .true.
                 nzdvsum(tdir(1),ndir ) = .true.
                 dvsum(ip,jp,kp,tdir(1),tdir(1)) = -locdx(tdir(1))*
     +                             0.5d0* mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(ndir,tdir(1)))
                 dvsum(ip,jp,kp,tdir(1),tdir(2)) = -locdx(tdir(1))*
     +                             0.5d0*mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(ndir,tdir(2)))
                 dvsum(ip,jp,kp,tdir(1),ndir) = -locdx(tdir(1))*
     +                             0.5d0*mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(ndir,ndir))

                 nzdvsum(ndir,tdir(1)) = .true.
                 nzdvsum(ndir,tdir(2)) = .true.
                 nzdvsum(ndir,ndir ) = .true.
                 dvsum(ip,jp,kp,ndir,tdir(1)) = -locdx(tdir(1))*
     +                             0.5d0*mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(tdir(1),tdir(1)))
                 dvsum(ip,jp,kp,ndir,tdir(2)) = -locdx(tdir(1))*
     +                             0.5d0* mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(tdir(1),tdir(2)))
                 dvsum(ip,jp,kp,ndir,ndir) = -locdx(tdir(1))*
     +                             0.5d0*mu(ip,jp,kp)*
     +                             jctg(ip,jp,kp,istwo(tdir(1),ndir))

               else if ( locdx(tdir(1)).eq.0 .and.
     +                   locdx(tdir(2)).ne.0 .and.
     +                   locdx( ndir).eq.0      ) then

c              --- Points (0,-1,0) and (0,1,0)
c                  Term 0.5*sqrt(g)*sigma^32

                 nzdvsum(tdir(2),tdir(1)) = .true.
                 nzdvsum(tdir(2),tdir(2)) = .true.
                 nzdvsum(tdir(2),ndir ) = .true.
                 dvsum(ip,jp,kp,tdir(2),tdir(1)) = -locdx(tdir(2))*
     +                               0.5d0*mu(ip,jp,kp)*
     +                               jctg(ip,jp,kp,istwo(ndir,tdir(1)))
                 dvsum(ip,jp,kp,tdir(2),tdir(2)) = -locdx(tdir(2))*
     +                               0.5d0*mu(ip,jp,kp)*
     +                               jctg(ip,jp,kp,istwo(ndir,tdir(2)))
                 dvsum(ip,jp,kp,tdir(2),ndir) = -locdx(tdir(2))*0.5d0*
     +                               mu(ip,jp,kp)*
     +                               jctg(ip,jp,kp,istwo(ndir,ndir))

                 nzdvsum(ndir,tdir(1)) = .true.
                 nzdvsum(ndir,tdir(2)) = .true.
                 nzdvsum(ndir,ndir ) = .true.
                 dvsum(ip,jp,kp,ndir,tdir(1)) = -locdx(tdir(2))*0.5d0*
     +                            0.5d0*mu(ip,jp,kp)*
     +                            jctg(ip,jp,kp,istwo(tdir(2),tdir(1)))
                 dvsum(ip,jp,kp,ndir,tdir(2)) = -locdx(tdir(2))*
     +                            0.5d0*mu(ip,jp,kp)*
     +                            jctg(ip,jp,kp,istwo(tdir(2),tdir(2)))
                 dvsum(ip,jp,kp,ndir,ndir) = -locdx(tdir(2))*
     +                            0.5d0*mu(ip,jp,kp)*
     +                            jctg(ip,jp,kp,istwo(tdir(2),ndir))

               end if

c              --- Modify basic stencil for half cell to take
c                  into account the boundary conditions

               if ( btype.eq.snnsnt ) then

c              --- Both sigma^nn and sigma^nt given in integration point
c                  Get boundary condition information
c
c              --- At this moment we assume that only outflow is
c                  prescribed, i.e.
c
c                   nn    nt1    nt2
c                  S   = S    = S    = 0

                  vval(ndir) = 0d0
                  vval(tdir(1)) = 0d0
                  vval(tdir(2)) = 0d0

                 if ( locdx(tdir(1)).eq.0 .and.
     +                locdx(tdir(2)).eq.0 .and.
     +                locdx(ndir ).eq.0      ) then

c                --- Point (0,0,0), substitute terms in right-hand side

                   rhside(ip,jp,kp,ndir) = rhside(ip,jp,kp,ndir) +
     +              nsig*locjac(ip,jp,kp)*vval(ndir) +
     +              0.5d0*locjac(ip,jp,kp)*
     +              chsym(ip,jp,kp,ndir,istwo(ndir,ndir))*vval(ndir) +
     +              0.5d0*locjac(ip,jp,kp)*2d0*
     +              chsym(ip,jp,kp,ndir,istwo(ndir,tdir(1)))*
     +              vval(tdir(1)) + 0.5d0*locjac(ip,jp,kp)*2d0*
     +              chsym(ip,jp,kp,ndir,istwo(ndir,tdir(2)))*
     +              vval(tdir(2))

                 else if ( locdx(tdir(1)).ne.0 .and.
     +                     locdx(tdir(2)).eq.0 .and.
     +                     locdx(ndir ).eq.0      ) then

c                --- Points (1,0,0) and (-1,0,0)
c                    substitute terms in right-hand side

                   dvsum(ip,jp,kp,tdir(1),tdir(1)) = 0d0
                   dvsum(ip,jp,kp,tdir(1),tdir(2)) = 0d0
                   dvsum(ip,jp,kp,tdir(1),ndir ) = 0d0
                   dvsum(ip,jp,kp,ndir,tdir(1)) = 0d0
                   dvsum(ip,jp,kp,ndir,tdir(2)) = 0d0
                   dvsum(ip,jp,kp,ndir,ndir ) = 0d0

                   rhside(mip,mjp,mkp,ndir) = rhside(mip,mjp,mkp,ndir) +
     +               0.5d0*locdx(tdir(1))*locjac(ip,jp,kp)*vval(tdir(1))

                 else if ( locdx(tdir(1)).eq.0 .and.
     +                     locdx(tdir(2)).ne.0 .and.
     +                     locdx(ndir ).eq.0      ) then

c                --- Points (0,1,0) and (0,-1,0)
c                    substitute terms in right-hand side

                   dvsum(ip,jp,kp,tdir(2),tdir(1)) = 0d0
                   dvsum(ip,jp,kp,tdir(2),tdir(2)) = 0d0
                   dvsum(ip,jp,kp,tdir(2),ndir ) = 0d0
                   dvsum(ip,jp,kp,ndir,tdir(1)) = 0d0
                   dvsum(ip,jp,kp,ndir,tdir(2)) = 0d0
                   dvsum(ip,jp,kp,ndir,ndir ) = 0d0

                   rhside(mip,mjp,mkp,ndir) = rhside(mip,mjp,mkp,ndir) +
     +                0.5d0*locdx(tdir(2))*locjac(ip,jp,kp)*
     +                vval(tdir(2))

                 end if
c
               else if ( btype.eq.snnut ) then

c              --- sigma^nn provided and ut provided

                 if ( locdx(tdir(1)).eq.0 .and.
     +                locdx(tdir(1)).eq.0 .and.
     +                locdx(ndir ).eq.0      ) then

c                --- Point (0,0,0)

c                --- At this moment we assume that only outstream
c                    is prescribed, i.e.
c                                                       nn
c                                        u.t1 = u.t2 = S   = 0
c                                        - -    - -

                   vval(ndir) = 0d0

                   rhside(ip,jp,kp,alphad) = rhside(ip,jp,kp,alphad) +
     +                            nsig*locjac(ip,jp,kp)*vval(ndir) +
     +                            0.5d0*locjac(ip,jp,kp)*
     +                            chsym(ip,jp,kp,ndir,istwo(ndir,ndir))*
     +                            vval(ndir)

c                                             1         | n |
c                  --- Matrix contribution: - - sqrt(g) |   | sigma^(t n)
c                                             2         |t n|
c
c                      with t = tdir(1) or tdir(2)

                   do gammal = 1, 3
                     nzdvsum(ndir,gammal) = .true.
                     nzdvsum(tdir(1),gammal) = .true.
                     nzdvsum(tdir(2),gammal) = .true.

                     dvsum(ip,jp,kp,ndir,gammal) =
     +                    dvsum(ip,jp,kp,ndir,gammal) -
     +                    0.5d0*mu(ip,jp,kp)*
     +                    chsym(ip,jp,kp,ndir,istwo(ndir,tdir(1)))*
     +                    jctg(ip,jp,kp,istwo(tdir(1),gammal)) -
     +                    0.5d0*mu(ip,jp,kp)*
     +                    chsym(ip,jp,kp,ndir,istwo(ndir,tdir(2)))*
     +                    jctg(ip,jp,kp,istwo(tdir(2),gammal))

                     dvsum(ip,jp,kp,tdir(1),gammal) =
     +                    dvsum(ip,jp,kp,tdir(1),gammal) -
     +                    0.5d0*mu(ip,jp,kp)*
     +                    chsym(ip,jp,kp,ndir,istwo(ndir,tdir(1)))*
     +                    jctg(ip,jp,kp,istwo(ndir,gammal))

                     dvsum(ip,jp,kp,tdir(2),gammal) =
     +                    dvsum(ip,jp,kp,tdir(2),gammal) -
     +                    0.5d0*mu(ip,jp,kp)*
     +                    chsym(ip,jp,kp,ndir,istwo(ndir,tdir(2)))*
     +                    jctg(ip,jp,kp,istwo(ndir,gammal))

                   end do

                 end if

               end if

c              --- Add Christoffel symbols for dvsum terms

               do j = 1, 3
                 nzvsum(j) = .true.
                 do i = 1, 3
                   vsum(ip,jp,kp,1) = vsum(ip,jp,kp,1)+
     +                                dvsum(ip,jp,kp,j,i)*
     +                                chsym(ip,jp,kp,j,istwo(i,1))
                   vsum(ip,jp,kp,2) = vsum(ip,jp,kp,2)+
     +                                dvsum(ip,jp,kp,j,i)*
     +                                chsym(ip,jp,kp,j,istwo(i,2))
                   vsum(ip,jp,kp,3) = vsum(ip,jp,kp,3)+
     +                                dvsum(ip,jp,kp,j,i)*
     +                                chsym(ip,jp,kp,j,istwo(i,3))
                 end do
               end do

            end if

          end do
        end do
      end do

      call iserclos ( 'isbms23d' )

      end


