      subroutine isbtra3d ( iside, ni, nj, nk, nvirtual, dtsum,
     +                      ilow1, ihigh1, ilow2, ihigh2, matrix,
     +                      ibinfor, bndcon, rhside, jcta, invjac,
     +                      idiscret, mct )
c ======================================================================
c
c        programmer  Marcel Zijlema
c        version 1.0    date 02-05-1996
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Correct dtsum, matrix and right-hand side for Robbins type boundary
c   conditions for the transport equation in R3
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     boundary_conditions
c     matrix
c     diffusion
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual,
     +        iside, ilow1, ihigh1,ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), idiscret,
     +        mct(-2:2,-2:2,-2:2)
      double precision bndcon(3,ilow1:ihigh1,ilow2:ihigh2),
     +                 matrix(ni+1,nj+1,nk+1,*),
     +                 rhside(ni+1,nj+1,nk+1),
     +                 dtsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)

c     bndcon    i    reserved
c     dtsum     i    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     idiscret  i    Type of discretization
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     invjac    i    1/sqrt(g) in pnttyp
c     iside     i    Sequence number of side
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer robb
      parameter ( robb = 2 )
      integer iarea(2,3), ip, jp, kp, mip, mjp, mkp, mxp(3), ishift,
     +        jshift, kshift, tdir(2), ndir, nsign, ofstnr,
     +        ict1, ict2
      double precision gnn, ljcta

c     gnn       Contravariant metric tensor
c     iarea     Area of arrays
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     kp        Local counting variable k-direction
c     kshift    Shift in k-direction
c     ljcta     Length of a contravariant base vector times Jacobian
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mkp       Integration cell indices
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     ofstnr    Offset diagonal number of matrix element that
c               corresponds to the current point of the interpolation
c               or extrapolation formula
c     robb      Robbins boundary condition, i.e. aT + (k grad T).n given
c                                                       T
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   Consider a 2D boundary cell at the lower boundary:
c
c
c
c                -------|-------
c                |             |
c                |             |
c                |             |
c                -      +      -
c                |             |
c                |             |
c                |             |
c           ------------|------------------
c
c
c   For the internal elements we have a contribution for all four
c   integration points.
c
c   For the classical discretization the contribution to the lower
c   boundary is:
c
c              ab        |
c   - sqrt(g) K     T,   |              (a=1)
c                     b  | (0,0,-1)
c
c   Because of the Robbins boundary condition this term is replaced by:
c
c       / brhs - sigma T,
c   boundary
c
c   where brhs is stored in bndcon(2,.) and sigma in bndcon(1,.)
c
c   The integral is equal to
c
c   g_tt (brhs - sigma T) = sqrt(g) sqrt(g^nn) (brhs - sigma T)
c
c   Hence the dtsum contribution for the boundary point should be made zero,
c   and the contribution to matrix and right-hand side are given above
c
c   The term T should be distributed over the points (0,0,0) and (0,0,-1),
c   with a factor 1/2, since (0,0,-1/2) does not contain a transport unknown
c
c   However, since (0,0,-1) is a virtual point, this point is folded
c   inwardly in the following way:
c
c   T(0,0,-1) = 2T(0,0,0) - T(0,0,1)
c
c   Hence the final distribution becomes:
c
c   T(0,0,-1/2) = 3/2 T(0,00,) - 1/2T(0,0,1)
c
c   In the Wesbeek we follow the same approach
c **********************************************************************
c
c                      PSEUDO CODE
c
c   Compute constants depending on the side sequence number
c   For all mid side points at the boundary iside do
c     if ( boundary condition is Robbns condition) then
c        set dtsum equal to zero
c        update matrix and right-hand side
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      iarea(1,1) = 1
      iarea(2,1) = ni
      iarea(1,2) = 1
      iarea(2,2) = nj
      iarea(1,3) = 1
      iarea(2,3) = nk
      ishift = 0
      jshift = 0
      kshift = 0
      if ( iside.eq.1 ) then
         iarea(2,1) = 1
      else if ( iside.eq.2 ) then
         iarea(1,1) = ni
         ishift = 1
      else if ( iside.eq.3 ) then
         iarea(2,2) = 1
      else if ( iside.eq.4 ) then
         iarea(1,2) = nj
         jshift = 1
      else if ( iside.eq.5 ) then
         iarea(2,3) = 1
      else
         iarea(1,3) = nk
         kshift = 1
      end if

c     --- Compute nsign, tdir and ndir

      if ( iside.le.2 ) then

c     --- iside = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3
         if ( iside.eq.1 ) then

c        --- iside = 1:  back

            nsign = -1

         else

c        --- iside = 2:  front

            nsign = 1

         end if

      else if ( iside.gt.2  .and. iside.lt.5) then

c     --- iside = 3, 4:  left and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1
         if ( iside.eq.3 ) then

c        --- iside = 3:  left

            nsign = -1

         else

c        --- iside = 4:  right

            nsign = 1

         end if

      else

c     --- iside = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

         if ( iside.eq.5 ) then

c        --- iside = 5:  bottom

            nsign = -1

         else

c        --- iside = 6:  top

            nsign = 1

         end if

      end if
      ofstnr = mct(nsign*ishift,nsign*jshift,nsign*kshift)

      if ( idiscret.eq.1 ) then

c     --- Standard ISNaS discretization
c         loop over boundary cells along side iside

        do mkp = iarea(1,3), iarea(2,3)
          do mjp = iarea(1,2), iarea(2,2)
            do mip = iarea(1,1), iarea(2,1)

c           --- Get integration cell index

               mxp(1) = mip
               mxp(2) = mjp
               mxp(3) = mkp
               ict1   = 2*mxp(tdir(1))
               ict2   = 2*mxp(tdir(2))

c              --- Get integration cell position for ibinfor

               if ( ibinfor(ict1,ict2).eq.robb ) then

c              --- Robbins boundary conditions found
c                  First remove contribution to dtsum

                  ip = mip+ishift
                  jp = mjp+jshift
                  kp = mkp+kshift

                  dtsum(ip,jp,kp,1) = 0d0
                  dtsum(ip,jp,kp,2) = 0d0
                  dtsum(ip,jp,kp,3) = 0d0

                  gnn = invjac(ip,jp,kp)**2*(jcta(ip,jp,kp,1,ndir)**2 +
     +                                       jcta(ip,jp,kp,2,ndir)**2 +
     +                                       jcta(ip,jp,kp,3,ndir)**2 )

                  rhside(mip,mjp,mkp) = rhside(mip,mjp,mkp)+sqrt(gnn)*
     +                           bndcon(2,ict1,ict2) / invjac(ip,jp,kp)

                  matrix(mip,mjp,mkp,1) = matrix(mip,mjp,mkp,1)+1.5d0 *
     +                                sqrt(gnn) * bndcon(1,ict1,ict2) /
     +                                invjac(ip,jp,kp)
                  matrix(mip,mjp,mkp,ofstnr) =
     +                                matrix(mip,mjp,mkp,ofstnr) -
     +                                0.5d0 * sqrt(gnn) *
     +                                bndcon(1,ict1,ict2) /
     +                                invjac(ip,jp,kp)
               end if

            end do
          end do
        end do

      else

c     --- WesBeek discretization
c         loop over boundary cells along side iside

        do mkp = iarea(1,3), iarea(2,3)
          do mjp = iarea(1,2), iarea(2,2)
            do mip = iarea(1,1), iarea(2,1)

c           --- Get integration cell index

               mxp(1) = mip
               mxp(2) = mjp
               mxp(3) = mkp
               ict1   = 2*mxp(tdir(1))
               ict2   = 2*mxp(tdir(2))

c              --- Get integration cell position for ibinfor

               if ( ibinfor(ict1,ict2).eq.robb ) then

c              --- Robbins boundary conditions found
c                  First remove contribution to dtsum

                  ip = mip+ishift
                  jp = mjp+jshift
                  kp = mkp+kshift
                  dtsum(ip,jp,kp,1) = 0d0
                  dtsum(ip,jp,kp,2) = 0d0
                  dtsum(ip,jp,kp,3) = 0d0

                  ljcta = sqrt(jcta(ip,jp,kp,1,ndir)**2 +
     +                         jcta(ip,jp,kp,2,ndir)**2 +
     +                         jcta(ip,jp,kp,3,ndir)**2) * nsign

                  rhside(mip,mjp,mkp) = rhside(mip,mjp,mkp) + ljcta *
     +                                  bndcon(2,ict1,ict2)

                  matrix(mip,mjp,mkp,1) = matrix(mip,mjp,mkp,1)+1.5d0*
     +                                ljcta * bndcon(1,ict1,ict2)
                  matrix(mip,mjp,mkp,ofstnr) =
     +                                matrix(mip,mjp,mkp,ofstnr) -
     +                                0.5d0 * ljcta *
     +                                bndcon(1,ict1,ict2)
               end if

            end do
          end do
        end do

      end if

      end
