      subroutine isbwbs2 ( ni, nj, nvirtual, dvsum, alphad, mu,locdx,
     +                     dcell, ibinfor, ibipoint, bndcon, rhside,
     +                     cva, bcgeo, adir, adiri, coor, axisym )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 6.0    date 09-12-1999 Guus: extra parameters coor and axisym
c                                       (axi-synmmetric extension)
c        version 5.2    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 1.1    date 25-11-1998 pnttyp added
c        version 1.0    date 21-10-1996
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute stress contribution to matrix and right-hand side for "normal"
c   half cells, Wesbeek discretization.
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     matrix
c     stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2), dcell(2), alphad, ibinfor(*),
     +        ibipoint(*), axisym
      double precision bndcon(2,*), bcgeo(8,*)
      double precision
     +      mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2),
     +    adir(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   adiri(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +     cva(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2,4),
     +   rhside(ni+1,nj+1,2),
     +   coor(1-nvirtual:1+ni+nvirtual,
     +        1-nvirtual:1+nj+nvirtual,1:2)

c     adir           i    Contravariant base vector in integration cell centre
c     adiri          i    Contravariant base vector in integration point
c     alphad         i    Direction index of integration.
c                         Hence if a V1-cell is used, alphad = 1 and so on
c     axisym         i    In axi-symmetric case axisym=1 else axisym=0
c     bcgeo          i    Array containing information about the geometrical
c                         quantities along the boundaries
c                         The contents of bcgeo depend on the dimension.
c                         In R2 the following definition is used:
c                         bcgeo(igeo,icount,k), igeo=1(1)8,icount=1(1)..,k=1,2
c                         with: icount defines the cell index, where the cells
c                                  are numbered along the boundaries
c                                  First all cells of boundary 1,
c                                  then of boundary 2 etc.
c                                  k = 1, corresponds to vertex of cell and
c                                  k = 2 to midside point of cell
c                                  igeo defines the type of quantity.
c                                  Possible values:
c                                  1: sqrt(g) g^11
c                                  2: sqrt(g) g^12
c                                  3: sqrt(g) g^22
c                                  4: sqrt(g) a^(1)_1
c                                  5: sqrt(g) a^(1)_2
c                                  6: sqrt(g) a^(2)_1
c                                  7: sqrt(g) a^(2)_2
c                                  8: sqrt(g)
c                         In R3 the following definition is used:
c                         bcgeo(igeo,i,j), igeo=1(1)12, i and j vary over all
c                                  i and j indices in the same way as ibinfor
c                                  which means that both vertices and mid points
c                                  are used
c                                  igeo defines the type of quantity.
c                                  Possible values:
c                                  1: sqrt(g) g^11  7: g_11 13: sqrt(g) a^(1)_1
c                                  2: sqrt(g) g^12  8: g_12 14: sqrt(g) a^(1)_2
c                                  3: sqrt(g) g^13  9: g_13 15: sqrt(g) a^(1)_3
c                                  4: sqrt(g) g^22 10: g_22 16: sqrt(g) a^(2)_1
c                                  5: sqrt(g) g^23 11: g_23 17: sqrt(g) a^(2)_2
c                                  6: sqrt(g) g^33 12: g_33 18: sqrt(g) a^(2)_3
c     bndcon         i    Array BNDCON is used to indicate the value of the
c                         boundary conditions for each point at the boundary of
c                         each single block for each unknown.
c                         For a description see the programmers guide
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     cva            i    Array containing the covariant basis vectors
c                         in 2D we have
c                                                      l
c                         cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                      (alpha)     point type p
c                         in 3D it is cva(i,j,k,l,alpha,p) referring to
c                         cell i,j,k
c     dcell          i    Array of length ndim containing the distance in global
c                         co-ordinates of the integration point to the centre of
c                         the integration cell.
c     dvsum          i    Flux derivative contributions to the matrix
c     ibinfor        i    Third part of infbnd
c     ibipoint       i    ipoint(side)
c                                      position of start of ibinfor information
c                                      for SIDE in ibinfor
c     locdx          i    Position of the integration point with respect to
c                         the centre of the integration cell
c                         The cell sizes are assumed to be 2 times 2, hence the
c                         distance of a midside point to a cell centre is
c                         always 1
c     mu             i    The laminar viscosity
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rhside        i/o   right-hand side vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, snnsnt, uxuy, unsnt, snnut,
     +        walllaw, parabolic, skipval
      parameter ( skipval = 300 )
      parameter( unut = 4, snnsnt = 7, uxuy = 10, unsnt = 5, snnut = 6,
     +           walllaw = 9, parabolic= 12)
      logical ungiven, noskip
      integer j, btype, mip, mjp, mxp(2), ipibinf, tsign, pnttyp,
     +        ict1, icbtype, ip, jp, xp(2), ndir, sign, signa(2),
     +        tdir, nsign, iside, istwo(2,2), iparea(2,2), gammad
      double precision an1, an2, gtt, gnn, fact, rmean,
     +                 t1, t2, n1, n2

c     an1            Component 1 of contravariant base vector in normal
c                    direction
c     an2            Component 2 of contravariant base vector in normal
c                    direction
c     btype          Type of boundary condition
c     fact           Multiplication factor
c     gammad         Direction index
c     gnn            Contravariant metric tensor
c     gtt            Contravariant metric tensor in tt direction
c     icbtype        Type of boundary condition with respect to centre of
c                    integration cell
c     ict1           Pointer in ibinfor with respect to boundary point
c     ip             Local counting variable i-direction
c     iparea         Area of integration points where a contribution to
c                    the matrix is to be calculated
c     ipibinf        Starting address of array infor for present side
c                    minus one
c     iside          Sequence number of side
c     istwo          Translation table to store and retrieve the
c                    elements of a symmetric ndim-dimensional
c                    rank 2 tensor from a vector with length
c                    (ndim*(ndim+1))/2
c     j              General loop variable
c     jp             Local counting variable j-direction
c     mip            Integration cell indices
c     mjp            Integration cell indices
c     mxp            Auxiliary array to calculate boundary point number
c     n1             Component 1 of normal vector
c     n2             Component 2 of normal vector
c     ndir           Direction of normal vector on this side
c     noskip         Indicate whether a given equation is skipped or not
c     nsign          Inner product of outward normal in computational
c                    domain with the base vector perpendicular to the face
c     parabolic      Parabolic boundary conditions
c     pnttyp         Point where a value or finite difference is
c                    to be calculated within the cell
c                    For a description see the programmers guide
c     rmean          Mean value of r coordinate
c     sign           sign for multiplication (1 or -1)
c     signa          transformation sign from covariant to contravariant
c                    base vector
c     skipval        This value indicates that effect of boundary condition
c                    must be skipped
c     snnsnt         Sigma_nn and sigma_nt prescribed
c     snnut          Sigma_nn and u_t prescribed
c     t1             Component 1 of tangent vector
c     t2             Component 2 of tangent vector
c     tdir           Direction of tangent vector on this side
c     tsign          Inner product of tangential vector defined counter
c                    clockwise with respect to the outward normal and the
c                    base vector along the side
c     ungiven        If true un is given
c     unsnt          U_n and sigma_nt prescribed
c     unut           U_n and u_t prescribed
c     uxuy           U_x and u_y prescribed
c     walllaw        Law of the wall boundary condition
c     xp             Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2      Computes ndir, nsign, tdir, tsign for a 2-dimensional
c                    problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the wesbeek discretization are given by:
c
c
c    (a)                (1)  b |i+1/2,j     (2)          b |i,j+1/2
c   a      . { sqrt(g) a    T  |         + a   sqrt(g)  T  |        }
c    (0,0)              b      |i-1/2,j     b              |i,j-1/2
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c    (a)       1          (1)  b |i+1/2,j     (2)          b |i,j+1/2
c   a      . { - sqrt(g) a    T  |         + a   sqrt(g)  T  |        }
c    (0,0)     2          b      |i-1/2,j     b              |i,j-1/2
c
c
c   The following boundary conditions have the following effects:
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:       Replace all terms sigma nn and sigma nt
c                                   at the boundary by their given value
c                                   and multipli the contributions
c                                   at the half sides with factor 0.5 .
c
c      Formulae:
c
c
c     (2)          b |      ||  (2)        ||
c    a   sqrt(g)  T  |    = || a    sqrt(g)|| {sigma^nn n + sigma^nt t }
c     b              |i,j   ||             ||
c
c     with sigma^nn and sigma^nt given.
c
c
c   Sigma nn, ut given:             Replace all terms sigma nn
c                                   at the boundary by the given value
c                                   and multipli the contributions
c                                   at the half sides with factor 0.5 .
c                                   Use the folding subroutine to eliminate
c                                   virtual tangential velocities
c
c      Formulae:
c
c
c     (2)          b |      ||  (2)        ||                   b
c    a   sqrt(g)  T  |    = || a    sqrt(g)|| {sigma^nn n + n (T.t)t  }
c     b              |i,j   ||             ||                b - - -
c
c     with  sigma^nn given, t expressed in a
c                           -               (t)
c
c                               (n)
c     and  a     expressed in a
c           (t)
c
c       1                 (2)           1                 (1)
c     a     =   sqrt(g)  a            a     = - sqrt(g)  a
c      (1)                (2)          (2)                (2)
c
c       2                 (2)           2                 (1)
c     a     = - sqrt(g)  a            a     =   sqrt(g)  a
c      (1)                (2)          (2)                (1)
c
c
c
c    The comments refer to a cell at the lower boundary
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      rmean = 1d0
      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1

c        --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         noskip = locdx(ndir).ne.nsign .and. alphad.eq.ndir
         if ( noskip ) then

c        --- loop over cells

            do mjp = iparea(1,2), iparea(2,2)
               do mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get integration cell position for ibinfor

                  icbtype = ibinfor(ipibinf+2*mxp(tdir))

c                 --- Check for Dirichlet condition in half cell

                  ungiven = icbtype.eq.uxuy .or. icbtype.eq.unut .or.
     +                      icbtype.eq.parabolic .or.
     +                      icbtype.eq.unsnt .or. icbtype.eq.88 .or.
     +                      icbtype.eq.walllaw

c                 --- Get reduced boundary condition type

                  btype = icbtype
                  if (btype.eq.parabolic) then
                     btype = unut
                  end if
                  if (btype.eq.walllaw) then
                     btype = unsnt
                  end if
                  if ( btype.ge.89 .and. btype.le.99 ) then
                     btype = skipval
                  end if
                  icbtype = btype

c                 --- Get integration point index

                  xp(tdir) = mxp(tdir) + dcell(tdir)
                  xp(ndir) = mxp(ndir) + dcell(ndir)

                  ip = xp(1)
                  jp = xp(2)
                  if ( axisym.eq.1 ) then
                     if ( iside.le.2 ) then
                        rmean = 0.5d0*(coor(ip,jp+1,1)+coor(ip,jp,1))
                     else
                        rmean = 0.5d0*(coor(ip+1,jp,1)+coor(ip,jp,1))
                     end if
                  end if

c                 --- Get boundary information

                  ict1 = ipibinf+2*mxp(tdir)+locdx(tdir)
                  btype = ibinfor(ict1)

c                 --- Get reduced boundary condition type

                  if ( btype.eq.parabolic .or. btype.eq.88 )
     +               btype = unut
                  if ( btype.eq.walllaw ) btype = unsnt
                  if ( btype.ge.89 .and. btype.le.99 )
     +                 btype = skipval

c                 --- Local contribution of covariant derivatives
c                     to christoffel symbols

                  if ( ungiven ) then

c                 --- Dirichlet half cell

                     dvsum(mip,mjp,1,1) = 0d0
                     dvsum(mip,mjp,2,1) = 0d0
                     dvsum(mip,mjp,1,2) = 0d0
                     dvsum(mip,mjp,2,2) = 0d0

                  else if ( icbtype.ne.skipval ) then

c                 --- velocity integration cell (half cell for V^ndir)
c                     Construct the basic stencil in so far necessary
c                     In this situation we have always sigma^nn given

                     if ( locdx(tdir).eq.0 .and.locdx(ndir).eq.0 ) then

c                    --- Point (0,0)

                        dvsum(mip,mjp,1,1) = 0d0
                        dvsum(mip,mjp,2,1) = 0d0
                        dvsum(mip,mjp,1,2) = 0d0
                        dvsum(mip,mjp,2,2) = 0d0

                     else if ( locdx(tdir).ne.0 .and.
     +                         locdx(ndir).eq.0 ) then

c                    --- Points (-1,0) and (1,0)

                        dvsum(mip,mjp,1,1) = 0.5d0 * dvsum(mip,mjp,1,1)
                        dvsum(mip,mjp,2,1) = 0.5d0 * dvsum(mip,mjp,2,1)
                        dvsum(mip,mjp,1,2) = 0.5d0 * dvsum(mip,mjp,1,2)
                        dvsum(mip,mjp,2,2) = 0.5d0 * dvsum(mip,mjp,2,2)

                     end if

c                    --- Modify basic stencil for half cell to take
c                        into account the boundary conditions

                     if ( btype.eq.snnsnt ) then

c                    --- Both sigma^nn and sigma^nt given in integration point
c                        Get boundary condition information

                        if ( locdx(tdir).eq.0 .and. locdx(ndir).eq.0 )
     +                     then

c                       --- Point (0,0), substitute terms in right-hand side

                           dvsum(mip,mjp,1,1) = 0d0
                           dvsum(mip,mjp,2,1) = 0d0
                           dvsum(mip,mjp,1,2) = 0d0
                           dvsum(mip,mjp,2,2) = 0d0

                           if( iside.lt.3 ) then
                              pnttyp = 3
                           else
                              pnttyp = 2
                           end if

                           gtt = cva(ip,jp,1,tdir,pnttyp) *
     +                           cva(ip,jp,1,tdir,pnttyp) +
     +                           cva(ip,jp,2,tdir,pnttyp) *
     +                           cva(ip,jp,2,tdir,pnttyp)
                           t1 = cva(ip,jp,1,tdir,pnttyp)/sqrt(gtt)
                           t2 = cva(ip,jp,2,tdir,pnttyp)/sqrt(gtt)
                           fact =  (adir(mip,mjp,1)*t1 +
     +                               adir(mip,mjp,2)*t2) *
     +                              sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                              adiri(ip,jp,2)*adiri(ip,jp,2))

                           rhside(mip,mjp,alphad) =
     +                            rhside(mip,mjp,alphad) +
     +                                 fact * tsign * bndcon(2,ict1)

                           an1 = bcgeo((2+2*ndir),ict1)/
     +                           bcgeo(8,ict1)
                           an2 = bcgeo((3+2*ndir),ict1)/
     +                           bcgeo(8,ict1)
                           gnn = bcgeo(istwo(ndir,ndir),ict1)/
     +                           bcgeo(8,ict1)
                           n1 = an1/sqrt(gnn)
                           n2 = an2/sqrt(gnn)

                           fact =  (adir(mip,mjp,1)*n1 +
     +                               adir(mip,mjp,2)*n2) *
     +                           sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                              adiri(ip,jp,2)*adiri(ip,jp,2))

                           rhside(mip,mjp,alphad) =
     +                            rhside(mip,mjp,alphad) +
     +                            fact * nsign * bndcon(1,ict1)*rmean

                        end if
                     else if ( btype.eq.snnut ) then

c                    --- sigma^nn provided and ut provided

                        if (locdx(tdir).eq.0 .and. locdx(ndir).eq.0)
     +                     then

c                       --- Point (0,0)

                           dvsum(ip,jp,1,1) = 0d0
                           dvsum(ip,jp,1,2) = 0d0
                           dvsum(ip,jp,2,1) = 0d0
                           dvsum(ip,jp,2,2) = 0d0

c                          --- fill dvsum

                           sign = -locdx(ndir)

                           if( ndir.eq.1) then
                              signa(1) = 1
                              signa(2) = -1
                           else
                              signa(1) = -1
                              signa(2) = 1
                           end if

                           do gammad = 1, 2
                              do j = 1, 2

                                 fact = (adir(mip,mjp,1)*
     +                                     signa(1)*adiri(ip,jp,2) +
     +                                     adir(mip,mjp,2)*
     +                                     signa(2)*adiri(ip,jp,1) ) *
     +                                    (1d0/(adiri(ip,jp,1)*
     +                                          adiri(ip,jp,1)+
     +                                          adiri(ip,jp,2)*
     +                                          adiri(ip,jp,2)) )

                                 dvsum(mip,mjp,gammad,j) = fact*
     +                                           sign*mu(mip,mjp)*
     +                                  signa(j)*adiri(ip,jp,3-j)*
     +                                           adiri(ip,jp,gammad)
                              end do
                           end do

                           an1 = bcgeo((2+2*ndir),ict1)/
     +                           bcgeo(8,ict1)
                           an2 = bcgeo((3+2*ndir),ict1)/
     +                           bcgeo(8,ict1)
                           gnn = bcgeo(istwo(ndir,ndir),ict1)/
     +                           bcgeo(8,ict1)
                           n1 = an1/sqrt(gnn)
                           n2 = an2/sqrt(gnn)

                           fact = (adir(mip,mjp,1)*n1 +
     +                               adir(mip,mjp,2)*n2) *
     +                           sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                              adiri(ip,jp,2)*adiri(ip,jp,2))


                           rhside(mip,mjp,alphad) =
     +                            rhside(mip,mjp,alphad) +
     +                            fact * nsign * bndcon(1,ict1)*rmean

                        end if

                     end if

                  end if
               end do
            end do

         else if ( locdx(ndir).eq.nsign .and. alphad.eq.ndir ) then

c        --- Point outside region
c            loop over cells

            do mjp = iparea(1,2), iparea(2,2)
               do mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get integration cell position for ibinfor

                  btype = ibinfor(ipibinf+2*mxp(tdir))
                  if ( btype.ge.89 .and. btype.le.99 ) then
                     btype = skipval
                  end if

c                 --- Get integration point index

                  xp(tdir) = mxp(tdir) + dcell(tdir)
                  xp(ndir) = mxp(ndir) + dcell(ndir)

                  ip = xp(1)
                  jp = xp(2)
                  if ( btype.ne.skipval ) then
                     dvsum(mip,mjp,1,1) = 0d0
                     dvsum(mip,mjp,2,1) = 0d0
                     dvsum(mip,mjp,1,2) = 0d0
                     dvsum(mip,mjp,2,2) = 0d0
                  end if

               end do
            end do

         end if

      end do

      end
