      subroutine iscdu12d( uold , mnew, pold, switch, shiftr,
     +                   shiftrfo, densup, ni, nj, nvirtual, rinput,
     +                   iinput, eddy, nblocks, nfaces, maxnsubf,
     +                   nsubfaces, subfaces)
c ======================================================================
c
c     programmer     Duncan van der Heul
c     version      1.5    date   02-12-1999  Duncan: revision of supersonic
c                                                    schemes
c     version      1.4    date   01-04-1999  Duncan: Change of density bias
c                                                      gradient_based
c     version      1.3    date   25-03-1999  Duncan: inclusion supersonic mix
c                                                    scheme
c     version      1.2    date   18-03-1999  Duncan: inclusion of Supersonic
c                                                    scheme
c     version      1.1    date   04-03-1999  Duncan: total revision; new
c                                                    call to isdstfl/ls
c                                                    fixed bugs
c     version      1.0    date   22-07-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Subroutine ISCDUP2D calculates the contributions to the pressure
c     matrix due to the application of the density bias in the continuity
c     equation.
c     Currently 6 versions are available:
c     1) higher order upwind density bias, with deferred correction
c                                           ~ n   n+1        ~ n   n+1
c        n+1  n                           (rho   m )   -   (rho   m )
c     rho -rho              n+1 n+1           fo    x          ho    x
c     --------- +   (sigma   m )    =    -------          --------
c        dt                 fo    x            n+1             n+1
c     where fo denotes first order interpolation
c           ho denotes limted higher order upwind interpolation
c     2) first order upwind with Mach dependant switch
c        n+1  n
c     rho -rho                    n+1 n+1       n+1
c     --------- +   ((1-s)*sigma   m )  + (s m )   =  0
c        dt                       fo    x         x
c     3) first order upwind unconditional
c        n+1  n
c     rho -rho              n+1 n+1
c     --------- +   (sigma   m )   =  0
c        dt                 fo    x
c     4) supersonic scheme, convection velocity at previous time level
c        n+1  n
c     rho -rho                 n+1 n         n+1
c     --------- +   ((1-s)rho   u )  + (s m )   =  0
c        dt                    fo    x         x
c     5) supersonic scheme CN, cranck-nicholsom version of 4 with
c         combination of convection velocity at previous and new time level
c         here the first term is used as formulated in 3)
c        n+1  n
c     rho -rho                 n+1 n           n+1 n+1
c     --------- +   ((1-s)rho   u )  + (s rho   u)   =  0
c        dt                    fo    x         fo   x
c        where now the switch is blocked at 0.5d0 in subroutine
c        iscomsup
c     6) supersonic scheme CN 2, cranck-nicholson version of 4 with
c         combination of convection velocity at previous and new time level
c         here the first term is used as formulated in 1)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), nblocks, nfaces, maxnsubf,
     +                 nsubfaces(*), subfaces(*)
      double precision shiftr(ni+1,nj+1), switch(ni+1,nj+1),
     +                 uold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 densup(ni+1,nj+1,3), rinput(*),
     +                 mnew(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 pold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*),
     +                 shiftrfo(ni+1,nj+1)

c     densup   i/o   coefficients of density upwind in pressure matrix
c     eddy      i    reserved
c     iinput    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     mnew      i    Momentum flux at the present time level
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     pold      i    Pressure at the old time level
c     rinput    i    reserved
c     shiftr    i    Shift variable used in compressible density upwinding.
c     shiftrfo  i    Shift variable used in compressible density upwinding,
c                    first order.
c     subfaces  i    reserved
c     switch    i    Continuous compressible density upwind switch
c                      switch = 0 for Mach numbers < 1-ls
c                      switch=0.5 for Mach numbers > 1+ls
c     uold      i    Velocity at previous time step.
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, mscal, ichoicedb, iprincom, istartup, mcav,
     +                      iprincav
      logical periodic

c     i         General loop variable
c     ichoicedb if 1 a gradient based, if 2 a mach based,
c               if 3 an unconditional density bias is applied
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     k         General loop variable
c     mcav      If 1 a cavity model is used, if 0 not
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     periodic  Periodic boundary conditions
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdstfll, isdstlss

c     ISCHECKPERI Check for periodic boundary conditions
c     ISDSTFLL    Derivative of dimensional equation of state
c     ISDSTLSS    Derivative of dimensionless equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      ichoicedb =  iinput(iinput(9)-1 +7)
      mscal     =  iinput(iinput(9)+4)
      mcav      =  iinput(77)
      if( mcav.eq.1)
     +      istartup=iinput(iinput(11)-1+2)
      iprincom  =  iinput(34)
      iprincav  =  iinput(35)

c     --- reset densup

      do k=1,3
         do j=1,nj+1
            do i=1,ni+1
               densup(i,j,1)=0d0
               densup(i,j,2)=0d0
               densup(i,j,3)=0d0
            end do
         end do
      end do

      if(ichoicedb.eq.1) then

         do j=1, nj+1

           if( mscal.eq.0) then

              do i=2, ni-1
                 if( abs(mnew(i,j)).gt.1d-12.and.
     +                 abs(switch(i,j)-1d0).gt.1d-8) then
                    densup(i,j,1) = -1d0*abs(mnew(i,j))*
     +              1d0/(eddy(i-1,j,2)+eddy(i,j,2))**2*(
     +              isdstlss((pold(i-1,j)+pold(i,j))/2d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j)-pold(i-1,j))+1d0*
     +               (eddy(i-1,j,2)+eddy(i,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i-1,j,2)+eddy(i,j,2))
                 end if
              end do

            else if( mscal.eq.1) then

              do i=2, ni-1

                 if( abs(mnew(i,j)).gt.1d-12) then
                    densup(i,j,1) = -1d0*abs(mnew(i,j))*
     +              1d0/(eddy(i-1,j,2)+eddy(i,j,2))**2*(
     +              isdstfll((pold(i-1,j)+pold(i,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j)-pold(i-1,j))+1d0*
     +               (eddy(i-1,j,2)+eddy(i,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i-1,j,2)+eddy(i,j,2))
                 end if

              end do

            end if

         end do

         do j=1, nj+1

           densup(1,j,2)   = 0d0

           if( mscal.eq. 0) then

              do i=2, ni-1

                 if( abs(mnew(  i,j)).gt.1d-12) then

                    densup(i,j,2) = -1d0*abs(mnew(i  ,j))*
     +              1d0/(eddy(i-1,j,2)+eddy(i  ,j,2))**2*(
     +              isdstlss((pold(i-1,j)+pold(i  ,j))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i  ,j)-pold(i-1,j))-1d0*
     +               (eddy(i-1,j,2)+eddy(i  ,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i-1,j,2)+eddy(i,j,2))

                 end if

                 if( abs(mnew(i+1,j)).gt.1d-12) then

                    densup(i,j,2) =  densup(i,j,2)+
     +                               1d0*abs(mnew(i+1,j))*
     +              1d0/(eddy(i  ,j,2)+eddy(i+1,j,2))**2*(
     +              isdstlss((pold(i  ,j)+pold(i+1,j))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i+1,j)-pold(i  ,j))+1d0*
     +               (eddy(i  ,j,2)+eddy(i+1,j,2)))
     +                              +1d0*mnew(i+1,j)*
     +              (shiftrfo(i+1,j)-shiftr(i+1,j))/
     +               (eddy(i+1,j,2)+eddy(i,j,2))
                 end if

              end do

           else if( mscal.eq.1) then

              do i=2, ni-1

                 if( abs(mnew(i,j)).gt.1d-12) then

                    densup(i,j,2) = -1d0*abs(mnew(i  ,j))*
     +              1d0/(eddy(i-1,j,2)+eddy(i  ,j,2))**2*(
     +              isdstfll((pold(i-1,j)+pold(i  ,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i  ,j)-pold(i-1,j))-1d0*
     +               (eddy(i-1,j,2)+eddy(i  ,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i-1,j,2)+eddy(i,j,2))
                 end if

                 if( abs(mnew(i+1,j)).gt.1d-12) then
                    densup(i,j,2) =  densup(i,j,2)+
     +                               1d0*abs(mnew(i+1,j))*
     +              1d0/(eddy(i  ,j,2)+eddy(i+1,j,2))**2*(
     +              isdstfll((pold(i  ,j)+pold(i+1,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i+1,j)-pold(i  ,j))+1d0*
     +               (eddy(i  ,j,2)+eddy(i+1,j,2)))
     +                              +1d0*mnew(i+1,j)*
     +              (shiftrfo(i+1,j)-shiftr(i+1,j))/
     +               (eddy(i+1,j,2)+eddy(i,j,2))
                 end if

              end do

           end if

         end do

         do j=1, nj+1

           if(mscal.eq.0) then

              do i=2, ni-1

                 if( abs(mnew(i+1,j)).gt.1d-12) then

                    densup(i,j,3) = 1d0*abs(mnew(i+1,j))*
     +              1d0/(eddy(i  ,j,2)+eddy(i+1,j,2))**2*(
     +              isdstlss((pold(i  ,j)+pold(i+1,j))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i+1,j)-pold(i  ,j))-1d0*
     +               (eddy(i  ,j,2)+eddy(i+1,j,2)))
     +                              +1d0*mnew(i+1,j)*
     +              (shiftrfo(i+1,j)-shiftr(i+1,j))/
     +               (eddy(i+1,j,2)+eddy(i,j,2))
                 end if

              end do

           else if( mscal.eq.1) then

              do i=2, ni-1

                 if( abs(mnew(i+1,j)).gt.1d-12) then

                    densup(i,j,3) =  1d0*abs(mnew(i+1,j))*
     +              1d0/(eddy(i  ,j,2)+eddy(i+1,j,2))**2*(
     +              isdstfll((pold(i  ,j)+pold(i+1,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i+1,j)-pold(i  ,j))-1d0*
     +               (eddy(i  ,j,2)+eddy(i+1,j,2)))
     +                              +1d0*mnew(i+1,j)*
     +              (shiftrfo(i+1,j)-shiftr(i+1,j))/
     +               (eddy(i+1,j,2)+eddy(i,j,2))
                 end if

              end do

           end if

         end do

      else if( ichoicedb.ge.2.and.ichoicedb.le.3) then

         do j=1, nj

           densup(1,j,1)   = 0d0

           do i=2, ni-1

             densup(i,j,1) = -1d0*(
     +                 uold(i,j)*(.5d0*(1d0-shiftr(i,j))+switch(i,j)) )

           end do

           densup(ni,j,1)  = 0d0

         end do

         do j=1, nj

           densup(1,j,2)   = 0d0

           do i=2, ni-1

             densup(i,j,2) = 1d0*(
     +                 uold(i+1,j)*(.5d0*(1d0-shiftr(i+1,j))+
     +                 switch(i+1,j))
     +                -uold(i,j)*(.5d0*(1d0-shiftr(i,j))-
     +                 switch(i,j)))
           end do

           densup(ni,j,2)  = 0d0

         end do

         do j=1, nj

           densup(1,j,3)   = 0d0

           do i=2, ni-1

             densup(i,j,3) = 1d0*(uold(i+1,j)*
     +                    (.5d0*(1d0-shiftr(i+1,j))
     +                 -switch(i+1,j)) )

           end do

           densup(ni,j,3)   = 0d0

         end do

      else if( ichoicedb.eq.4) then

         do j=1, nj

           do i=1, ni

             densup(i,j,1) = -switch(i,j)*
     +                       uold(i,j)*(0.5d0*(1d0+(uold(i,j)+1d-12)/
     +                                       (abs(uold(i,j))+1d-12)))

           end do

         end do

         do j=1, nj

           do i=1, ni

             densup(i,j,2) = -switch(i,j)*
     +                  uold(i  ,j)*(0.5d0*(1d0-(uold(i  ,j)+1d-12)/
     +                                      (abs(uold(i,j))+1d-12)))
     +                 +     switch(i+1,j)*
     +                  uold(i+1,j)*(0.5d0*(1d0+(uold(i+1,j)+1d-12)/
     +                                    (abs(uold(i+1,j))+1d-12)))
           end do

         end do

         do j=1, nj

           do i=1, ni

             densup(i,j,3) = switch(i+1,j)*
     +                  uold(i+1,j)*(0.5d0*(1d0-(uold(i+1,j)+1d-12)/
     +                                    (abs(uold(i+1,j))+1d-12)))

           end do

         end do

      else if( ichoicedb.eq.5 .or. ichoicedb.eq.6  ) then

         do j=1, nj+1

           periodic = .FALSE.

           call ischeckperi(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, 1, j, periodic)

           if( periodic ) then

              densup(1,j,1) = (1d0-switch(1,j))*( -1d0*(
     +           uold(1,j)*(.5d0*(1d0-shiftr(1,j))+sign(0.5d0,
     +                   uold(1,j))) ))+
     +
     +                       switch(1,j)*(
     +                      -uold(1,j)*(0.5d0*(1d0+(uold(1,j)+1d-12)/
     +                                       (abs(uold(1,j))+1d-12))))

           else

              densup(1,j,1) = 0d0

           end if

           do i=2, ni

             densup(i,j,1) = (1d0-switch(i,j))*( -1d0*(
     +           uold(i,j)*(.5d0*(1d0-shiftr(i,j))+sign(0.5d0,
     +                   uold(i,j))) ))+
     +
     +                       switch(i,j)*(
     +                      -uold(i,j)*(0.5d0*(1d0+(uold(i,j)+1d-12)/
     +                                       (abs(uold(i,j))+1d-12))))

           end do

         end do

         do j=1, nj+1

           densup(1,j,2) = (1d0-switch(1+1,j))*(
     +              uold(1+1,j)*(.5d0*(1d0-shiftr(1+1,j))+
     +              sign(0.5d0, uold(1+1,j))))+
     +                    switch(i+1,j)*(
     +              +uold(1+1,j)*(0.5d0*(1d0+(uold(1+1,j)+1d-12)/
     +                                (abs(uold(1+1,j))+1d-12))))
           periodic = .FALSE.

           call ischeckperi(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, 1, j, periodic)

           if( periodic ) then

              densup(1,j,2) = densup(1,j,2) +
     +                       (1d0-switch(1  ,j))*(
     +                -uold(1,j)*(.5d0*(1d0-shiftr(1,j))-
     +                 sign(0.5d0, uold(1  ,j))))+
     +                       switch(1  ,j)*(
     +                 -uold(1  ,j)*(0.5d0*(1d0-(uold(1  ,j)+1d-12)/
     +                                   (abs(uold(1  ,j))+1d-12))))

           end if

           do i=2, ni-1

             densup(i,j,2) = (1d0-switch(i+1,j))*(
     +                 uold(i+1,j)*(.5d0*(1d0-shiftr(i+1,j))+
     +                 sign(0.5d0, uold(i+1,j))))+
     +                       (1d0-switch(i  ,j))*(
     +                -uold(i,j)*(.5d0*(1d0-shiftr(i,j))-
     +                 sign(0.5d0, uold(i  ,j))))+
     +
     +                       switch(i  ,j)*(
     +                 -uold(i  ,j)*(0.5d0*(1d0-(uold(i  ,j)+1d-12)/
     +                                   (abs(uold(i  ,j))+1d-12))))+
     +                       switch(i+1,j)*(
     +                 +uold(i+1,j)*(0.5d0*(1d0+(uold(i+1,j)+1d-12)/
     +                                   (abs(uold(i+1,j))+1d-12))))
           end do

           periodic = .FALSE.

           call ischeckperi(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, ni, j, periodic)

           densup(ni,j,2) = (1d0-switch(ni  ,j))*(
     +             -uold(ni,j)*(.5d0*(1d0-shiftr(ni,j))-
     +              sign(0.5d0, uold(ni  ,j))))+
     +                    switch(ni  ,j)*(
     +              -uold(ni  ,j)*(0.5d0*(1d0-(uold(ni  ,j)+1d-12)/
     +                                (abs(uold(ni  ,j))+1d-12))))

           if( periodic ) then

              densup(ni,j,2) = densup(ni,j,2) +
     +                       (1d0-switch(ni+1,j))*(
     +                 uold(ni+1,j)*(.5d0*(1d0-shiftr(ni+1,j))+
     +                 sign(0.5d0, uold(ni+1,j))))+
     +                       switch(ni+1,j)*(
     +                 +uold(ni+1,j)*(0.5d0*(1d0+(uold(ni+1,j)+1d-12)/
     +                                   (abs(uold(ni+1,j))+1d-12))))

           end if

         end do

         do j=1, nj+1

           do i=1, ni-1

             densup(i,j,3) = (1d0-switch(i+1,j))*(
     +                 uold(i+1,j)*(.5d0*(1d0-shiftr(i+1,j))-
     +                  sign(0.5d0,uold(i+1,j)))          )+
     +
     +                       switch(i+1,j)*(
     +                  uold(i+1,j)*(0.5d0*(1d0-(uold(i+1,j)+1d-12)/
     +                                    (abs(uold(i+1,j))+1d-12))))

           end do

           periodic = .FALSE.

           call ischeckperi(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, ni, j, periodic)

           if( periodic ) then

              densup(ni,j,3) = (1d0-switch(ni+1,j))*(
     +                 uold(ni+1,j)*(.5d0*(1d0-shiftr(ni+1,j))-
     +                  sign(0.5d0,uold(ni+1,j)))          )+
     +
     +                       switch(ni+1,j)*(
     +                  uold(ni+1,j)*(0.5d0*(1d0-(uold(ni+1,j)+1d-12)/
     +                                    (abs(uold(ni+1,j))+1d-12))))

           else

              densup(ni,j,3) = 0d0

           end if

         end do

      end if

      end
