      subroutine iscmtdu ( ni, nj, nvirtual, axisym, iparea, solut,
     +                      matrix, coor, gamcm )
c ======================================================================
c
c        programmer     Hester Bijl
c        version 2.4    date 22-12-1999 Guus: correction axisymmetric
c        version 2.3    date 04-03-1999 Kees K: axisymmetric
c        version 2.2    date 22-02-1999 Kees K: variable gamcm added
c        version 2.1    date 26-10-1995 Guus: Add contribution
c                                       immediately to matrix
c        version 2.0    date 10-10-1995 Guus: complete revision
c        version 1.1    date 05-07-1995 Hester: cleaned up
c        version 1.0    date 30-05-1995
c
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Computes compressible contribution to linear part of the transport
c    equation: -.6d0*div(V) and adds to matrix
c **********************************************************************
c
c                       KEYWORDS
c
c    transport_matrix
c    compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, iparea(2,2), nvirtual, axisym
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2),
     +                 matrix(ni+1,nj+1), gamcm,
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:2)

c     axisym    i    In axisymmetric case axisym=1 else axisym=0
c     coor      i    reserved
c     gamcm     i    Gas constant gamma (for compressible comput.)
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
       integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     The compressible equations require an extra first order term of the
c     form -(2-gamma) div(V) with respect to the equation for the density rho
c
c     This extra term is computed in this subroutine.
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- zot = -(2-gamma) * div( V ),
c         (in non-conservative transport equation : .6 h*div(V);
c          since in ISNaS the conservative form, dh/dt + div(uh),
c          is used h*div(u) has to be subtracted)

      if ( axisym.eq.1) then

c     --- axisymmetric case, contribution multiplied by r

         do j = iparea(1,2),iparea(2,2)
            do i = iparea(1,1),iparea(2,1)
               matrix(i,j) = 0.5d0* ( 2d0 - gamcm ) *
     +                        ( -solut(i+1,j,1)*
     +                           (coor(i+1,j,1) + coor(i+1,j+1,1))
     +                          +solut(i,j,1)*
     +                           (coor(i,j,1) + coor(i,j+1,1))
     +                          -solut(i,j+1,2)*
     +                           (coor(i,j+1,1) + coor(i+1,j+1,1))
     +                          +solut(i,j,2)*
     +                           (coor(i,j,1) + coor(i+1,j,1)) )
     +                        +  matrix(i,j)
            end do
         end do

      else

         do j = iparea(1,2),iparea(2,2)
            do i = iparea(1,1),iparea(2,1)
               matrix(i,j) = ( 2d0 - gamcm ) *
     +                        ( -solut(i+1,j,1)+solut(i,j,1)
     +                          -solut(i,j+1,2)+solut(i,j,2) ) +
     +                          matrix(i,j)
            end do
         end do

      end if

      end
