      subroutine iscom ( nblocks, isiinput, isrinput, issolut,
     +                   iseddy, iadmin, isitopol)
c ======================================================================
c
c     programmer   Hester Bijl
c     version      1.5   23-11-1998 Jason: removed calls to isini* isterm*
c     version      1.4   31-08-1998 Jason: remove check nodeid
c     version      1.3   29-07-1997 Guus   : Skip enthalpy if necessary
c     version      1.2   29-04-1996 Hester : Multi block implemented.
c     version      1.1   04-10-1995 Kees K.: warning for itrans
c     version      1.0   30-05-1995
c
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   ISnas COMpressible routine
c   Calls routine where, for compressible flow, the density will be computed
c   and the velocity updated, then the solution is extrapolated.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), iseddy(*), isiinput(*),
     +        isrinput(*), issolut(*), isitopol, nblocks

c     iadmin   i  reserved
c     iseddy   i  Array of sequence numbers (nblocks) of array eddy
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isitopol i  Sequence number of array itopol
c     isrinput i  Array of sequence numbers (nblocks) of array rinput
c     issolut  i  Array of sequence numbers (nblocks) of array solut
c     nblocks  i  Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, itrans, itimlv, ilev, ntimlv,
     +        ndim, dummy, ind, noenthal, ipiincom, icoupled
      logical usebndcn, ddummy

c     ddummy    dummy logical
c     dummy     Dummy
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ilev      Time level of the multi-block transport of information
c     ind       Sequence number of node
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     itimlv    Loop variable with respect to a given time level
c     itrans    reserved
c     ndim      Dimension of space (2 or 3)
c     noenthal  If 1 the enthalpy is not taken into account, if 0 it is
c     ntimlv    Number of time levels
c     usebndcn  If true boundary conditions are used in the extrapolation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer  isingetp

c     ISACTIVE  Activate old permanent array
c     ISCOM01   Calls routines which compute density and update velocity.
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILVSO  Extrapolate solution at outer boundaries
c     ISINGETP  get the pointer corresponding to an integer array
c     ISMBLK    Performs the local transport of multi-block information
c               within a node
c     ISRCDDHS  Receives the multi-block information from each node
c               and places it at the correct location in the TRAQ arrays
c     ISRCDDND  receives the virtual unknowns of all local blocks at a node
c               and puts them into the local TRAQ arrays.
c     ISSNDDHS  Copy the virtual unknowns of all blocks to the local copies
c               stored by the node programs using PVM.
c     ISSNDDND  Copy unknowns near or on the boundary of the subdomain
c               TRAQ arrays to the virtual region of the TRAQ arrays of the
c               neighbouring blocks on the host using PVM.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('iscom')

c     --- compute some general parameters

      call isactive(isiinput(1))
      call isactive(isrinput(1))

      ipiinput     = isingetp(isiinput(1))

      icoupled = ibuffr( ipiinput -1 + 76 )
      ipiincom = ibuffr(ipiinput +  9 - 1)
      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      noenthal = ibuffr(ipiinput + ipiincom - 1 + 4-1)

      do iblock = 1, nblocks

         if ( iadmin(iblock).eq.inodenr .or. inodenr.eq.ISHOST ) then

            call iscom01 ( isiinput(iblock), isrinput(iblock),
     +                     issolut(iblock), iseddy(iblock) )

          end if

      end do

c     --- put computed compr entities of the local blocks
c         in the solut arrays of the neighbouring blocks if on the host

      if ( inodenr.eq.ISHOST ) then
         do iblock = 1, nblocks
            if ( iadmin(iblock).eq.inodenr) then

               itrans = 1
               ilev   = ntimlv

               call ismblk(iblock, itrans, ilev, isiinput, issolut,
     +                     iadmin, isitopol)

               itrans = ndim+1
               ilev   = ntimlv-1
               call ismblk(iblock, itrans, ilev, isiinput, issolut,
     +                     iadmin, isitopol)

            end if
         end do
      end if

c     --- extrapolate enthalpy to virtual points

      if ( noenthal.eq.0 ) then

c     --- The enthalpy equation is available
c         Mark that it is assumed to be unknown 4

         do iblock = 1, nblocks

            if ( iadmin(iblock).eq.inodenr ) then

               itrans = 4

c              --- itrans is used only for 2D at this moment

               itimlv = 2
               usebndcn = .false.
               call isfilvso ( iblock, itrans, itimlv, nblocks,
     +                         isiinput(iblock), issolut(iblock),
     +                         isitopol, isitopol, isitopol,
     +                         icoupled, usebndcn )
            end if

         end do

      end if

c     --- receive (host) or send (node) fluxes

      if ( inodenr.eq.ISHOST ) then

c     --- receive fluxes if on the host

         do ind = 1, iacnodes

c           --- receive fluxes

            call isrcddhs( dummy, isiinput, issolut, isitopol )

         end do

      else

c     --- send fluxes on the node

         ind = 0

c        --- send fluxes

         itrans = 1
         ilev   = ntimlv
         call issnddnd ( itrans, ilev,   isiinput, issolut,
     +                   iadmin, isitopol)

      end if

c     --- send (host) or receive (node) fluxes

      if ( inodenr.eq.ISHOST ) then

c     --- send fluxes if on the host

         do ind = 1, iacnodes

c           --- send fluxes

            ddummy = .false.
            itrans = 1
            ilev   = ntimlv
            call issnddhs(nblocks, itrans, ilev, ind, isiinput,
     +                    issolut, isitopol, iadmin, ddummy)

         end do

      else

c     --- receive fluxes if on the node
c        --- receive fluxes

         call isrcddnd( nblocks, isiinput, issolut, isitopol,
     +                  iadmin, ddummy)

      end if

      call iserclos('iscom')

      end
