      subroutine isconpc1 ( issolut, isiinput, isrinput, finished )
c ======================================================================
c
c     programmer     Duncan van der Heul
c     version      1.0    date 25-05-1998
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Subroutine calculates the 2-norm of the iterative pressure correction
c     the iterative pressure correction is formulated in a way that when the
c     solution converges the iterative pressure correction vanishes.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer issolut(*), isiinput(*), isrinput(*)
      logical finished

c     finished  o    if ready=1 convergence is achieved to the required
c                    accuracy, if ready =0 iteration has to be continued
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipsolut, iblock, ndegfd, nblocks, ndim,
     +        ipiinput, ni, nj, nk, nvirtual, npointsv, i, iprintim,
     +        iprinput
      double precision dpres1, dpres2, toll, dpinfnorm

c     dpinfnorm infinite norm of the iterative pressure correction
c     dpres1    2-norm of the iterative pressure correction old
c     dpres2    2-norm of the iterative pressure correction new
c     i         General loop variable
c     iblock    Block sequence number
c     ipiinput  Starting address of array iinput in ibuffr
c     iprinput  Starting address of array rinput in buffer
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ipsolut   Starting address of array solut in array buffer
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c     toll      Tolerance for iterative solution of pressure correction
c               equation
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     finished =  false
c     dpres1 =  0
c     dpres2 =  dpres1
c     for all blocks do
c               ||  (k)||
c       dpres1= ||dp   ||
c               ||     ||2
c
c       dpres2  =   max(dpres1,dpres2)
c       dpres1  =   dpres2
c     end
c     if dpres1 < toll iteration has converged=>
c     finished =  TRUE
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call isactive(isiinput(1))
      call isactive(isrinput(1))
      ipiinput= isingetp(isiinput(1))
      iprinput= isdpgetp(isrinput(1))
      iprintim= ibuffr(ipiinput+ 29 -1)
      ndegfd  = ibuffr(ipiinput+ 55 -1)
      ndim    = ibuffr(ipiinput+ 57 -1)
      nblocks = ibuffr(ipiinput+ 73 -1)
      toll    = buffer(iprinput - 1 + iprintim - 1 + 19)

c     --- reset convergence check parameters

      finished=.FALSE.
      dpres1= 0d0
      dpres2= dpres1

c     --- calculate the maximum over all blocks of the two norm of the
c           pressure correction

      do iblock=1,nblocks

         call isactive(issolut(iblock))
         call isactive(isiinput(iblock))
         ipsolut   = isdpgetp(issolut(iblock))
         ipiinput  = isingetp(isiinput(iblock))
         nvirtual  = ibuffr(ipiinput + 54 -1)
         ni        = ibuffr(ipiinput + 51 -1)
         nj        = ibuffr(ipiinput + 52 -1)

         if( ndim.eq.2) then

            nk        = 1
            npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)* nk

         else if( ndim.eq.3) then

            nk        = ibuffr(ipiinput + 53 -1)
            npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                                        (nk+1+2*nvirtual)

         end if

         dpinfnorm=0d0

         do i=1,npointsv

           dpinfnorm=max(dpinfnorm,abs(buffer( ipsolut+ npointsv *
     +        (ndegfd+ndim)+i-1)))

         end do

         dpres2=max( dpres1, dpinfnorm)
         dpres1  = dpres2

      end do

      write(*,*) ' inf norm pressure correction = ', dpres1

      if (dpres1.lt.toll) then

        finished = .TRUE.

      end if

      end
