      subroutine isconti ( adiri,  cva, ni, nj, nvirtual, locdx,
     +                     alphad )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 30-10-1996 extension for locdx = 0
c        version 1.0    date 11-07-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute contravariant base vectors times jacobian
c    two-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, locdx(2)
      double precision adiri(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4)

c     adiri     i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, pnttyp

c     i         General loop variable
c     j         General loop variable
c     pnttyp    reserved
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The contravariant base vectors may be computed from the covariant ones
c     in the following way:
c
c      (1)                  2     1
c     a    =  1/sqrt(g) ( a    ,-a   )
c                          (2)    (2)
c
c
c      (2)                  2     1
c     a    =  1/sqrt(g) (-a    , a   )
c                          (1)    (1)
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- initialize adiri

            do j = 1-nvirtual, nj+1+nvirtual
               do i = 1-nvirtual, ni+1+nvirtual
                  adiri(i,j,1) = 0d0
                  adiri(i,j,2) = 0d0
               end do
            end do
      if ( alphad.eq.1 ) then

c     --- alphad = 1 cell

         if ( locdx(1).ne.0 ) then

            pnttyp = 4

            do j = 1, nj +1
               do i = 0, ni+1
                  adiri(i,j,1) =  cva(i,j,2,2,pnttyp)
                  adiri(i,j,2) = -cva(i,j,1,2,pnttyp)
               end do
            end do

         else if ( locdx(2).ne.0 ) then

            pnttyp = 2

            do j = 1, nj +1
               do i = 0, ni+1
                  adiri(i,j,1) = -5d-1 * ( cva(i,j,2,1,pnttyp) +
     +                                     cva(i-1,j,2,1,pnttyp) )
                  adiri(i,j,2) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                    cva(i-1,j,1,1,pnttyp) )
               end do
            end do

         else

            pnttyp = 4

            do j = 1, nj +1
               do i = 0, ni+1
                  adiri(i,j,1) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                    cva(i-1,j,2,2,pnttyp) )
                  adiri(i,j,2) = -5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i-1,j,1,2,pnttyp) )
               end do
            end do

         end if

      else

c     --- alphad = 2 cell

          if ( locdx(2).ne.0 ) then

             pnttyp = 4

             do j = 0, nj+1
                do i = 1, ni +1
                   adiri(i,j,1) =-cva(i,j,2,1,pnttyp)
                   adiri(i,j,2) = cva(i,j,1,1,pnttyp)
                end do
             end do

          else if ( locdx(1).ne.0 ) then

             pnttyp = 3

             do j = 0, nj+1
                do i = 1, ni +1
                   adiri(i,j,1) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                     cva(i,j-1,2,2,pnttyp) )
                   adiri(i,j,2) =-5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i,j-1,1,2,pnttyp) )
                end do
             end do

         else

             pnttyp = 4

             do j = 0, nj+1
                do i = 1, ni +1
                   adiri(i,j,1) = -5d-1 * ( cva(i,j,2,1,pnttyp) +
     +                                     cva(i,j-1,2,1,pnttyp) )
                   adiri(i,j,2) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                     cva(i,j-1,1,1,pnttyp) )
                end do
             end do

         end if

      end if

      end
