      subroutine iscpgmo2d ( ni, nj, nvirtual, rhside, jac,
     +                       alphad, cva, cvc, ctc, solut,
     +                       gmat, gmctinv, invjac, ibipoint, ibinfor,
     +                       idiscret, adir,
     +                       bicof, ibicof )
c ======================================================================
c
c     programmer     Charles Moulinec
c     version 1.3    date   04-06-1998 Charles: bilinear interpolation added
c     version 1.2    date   15-12-1997 Charles: use of the jacobian
c     version 1.1    date   29-10-1997 Charles: small correction
c     version 1.0    date   15-07-1997
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute contribution of pressure gradient for momentum equation
c     over collocated grids
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni ,nj, nvirtual, alphad, gmctinv(0:3,2,6),
     +ibipoint(*), ibinfor(*), idiscret
      integer ibicof(1-nvirtual:ni+1+nvirtual,
     +               1-nvirtual:nj+1+nvirtual,2,2)
      double precision bicof(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2)
      double precision gmat(ni+1,nj+1,2,6),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,4),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 cvc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 ctc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4)
      double precision rhside(ni+1,nj+1),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bicof     i    Array containing bilinear coefficients
c     ctc       i    Work array containing the contravariant WESBEEK vectors
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     cvc       i    Work array containing the covariant WESBEEK vectors
c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ibicof    i    Integer array containing bilinear coefficients
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ishift, jshift, l, idisgra
      integer i, j, ip, jp, betad, gammad, dx(2,2)
      double precision det, sol2d(2,2)
      double precision uloc, vloc

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     det       Determinant
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     gammad    Direction index
c     i         General loop variable
c     idisgra   Choice of the discretization of the gradient for collocated
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     l         General loop variable
c     sol2d     Result of the bilinear interpolation at the volume face
c     uloc      Local u-coordinate for the interpolation
c     vloc      Local v-coordinate for the interpolation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCONT2D  Compute contravariant base vectors in centroids of
c               alphad cells two-dimensional case
c     ISGMAT2D  Compute gradient matrix in R^2
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( idiscret.eq.2 ) then
         idisgra=1
      else if ( idiscret.eq.4 ) then
         idisgra=2
      end if

      if (idisgra.eq.0) then

         call iscont2d ( adir, jac, cva, ni, nj, nvirtual, alphad )

         call isgmat2d ( alphad, ni, nj, nvirtual, gmat, cva,
     +                   invjac, ibipoint, ibinfor, idiscret, jac,
     +                   adir )

         do l = 1, 6

            ishift = gmctinv(1,alphad,l)
            jshift = gmctinv(2,alphad,l)

            do jp = 1, nj
               do ip = 1, ni
                  if (jp.eq.1.or.ip.eq.1) then
                     gmat(ip,jp,alphad,l)=0.d0
                  end if

                  if (jp.eq.nj.or.ip.eq.ni) then
                     gmat(ip+2-alphad,jp+alphad-1,alphad,l)=0.d0
                  end if

                  rhside(ip,jp) = rhside(ip,jp) - 0.5d0 *
     +          ( gmat(ip+2-alphad,jp+alphad-1,alphad,l) *
     +            solut(ip+ishift+2-alphad,jp+jshift+alphad-1) +
     +            gmat(ip,jp,alphad,l) *
     +            solut(ip+ishift,jp+jshift) )

               end do
            end do

         end do

      end if

      if (idisgra.eq.1) then

         dx(1,1)      =  0
         dx(1,2)      =  0
         dx(1,alphad) =  1

         do betad = 1, 2
            do gammad = 1, 2
               do jp = 0, nj+1
                  do ip = 0, ni+1
                     cvc(ip,jp,gammad,betad) =
     +                 5d-1*cva(ip+dx(1,1),jp+dx(1,2),gammad,betad,4)+
     +                 cva(ip,jp,gammad,betad,4) +
     +                 5d-1*cva(ip-dx(1,1),jp-dx(1,2),gammad,betad,4)
                  end do
               end do
            end do
         end do

c        --- Determine contravariant WESBEEK vectors

         do jp=0, nj+1
            do ip=0, ni+1

c           --- First, calculate determinant ...

               det = cvc(ip,jp,1,1)*cvc(ip,jp,2,2) -
     +               cvc(ip,jp,2,1)*cvc(ip,jp,1,2)

c              --- then contravariant WESBEEK vectors

               ctc(ip,jp,1,1) =  cvc(ip,jp,2,2)/det
               ctc(ip,jp,2,1) = -cvc(ip,jp,1,2)/det
               ctc(ip,jp,1,2) = -cvc(ip,jp,2,1)/det
               ctc(ip,jp,2,2) =  cvc(ip,jp,1,1)/det

            end do
         end do

         do jp = 2, nj-1
            do ip = 2, ni-1
               rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +                 ( ctc(ip,jp,alphad,1) *
     +                 ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +                   ctc(ip,jp,alphad,2) *
     +                 ( solut(ip,jp+1) - solut(ip,jp-1) ) )
            end do
         end do

         jp = 1
         do ip = 2, ni-1
            solut(ip,jp-1)=2.*solut(ip,jp)-solut(ip,jp+1)
            rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +              ( ctc(ip,jp,alphad,1) *
     +              ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +                ctc(ip,jp,alphad,2) *
     +              ( solut(ip,jp+1) - solut(ip,jp-1) ) )
         end do

         jp = nj
         do ip = 2, ni-1
            solut(ip,jp+1)=2.*solut(ip,jp)-solut(ip,jp-1)
            rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +              ( ctc(ip,jp,alphad,1) *
     +              ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +                ctc(ip,jp,alphad,2) *
     +              ( solut(ip,jp+1) - solut(ip,jp-1) ) )
         end do

         ip = 1
         do jp = 2, nj-1
            solut(ip-1,jp)=2.*solut(ip,jp)-solut(ip+1,jp)
            rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +              ( ctc(ip,jp,alphad,1) *
     +              ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +                ctc(ip,jp,alphad,2) *
     +              ( solut(ip,jp+1) - solut(ip,jp-1) ) )
         end do

         ip = ni
         do jp = 2, nj-1
            solut(ip+1,jp)=2.*solut(ip,jp)-solut(ip-1,jp)
            rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +              ( ctc(ip,jp,alphad,1) *
     +              ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +                ctc(ip,jp,alphad,2) *
     +              ( solut(ip,jp+1) - solut(ip,jp-1) ) )
         end do

         ip = 1
         jp = 1
         solut(ip-1,jp)=2.*solut(ip,jp)-solut(ip+1,jp)
         solut(ip,jp-1)=2.*solut(ip,jp)-solut(ip,jp+1)
         rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +           ( ctc(ip,jp,alphad,1) *
     +           ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +             ctc(ip,jp,alphad,2) *
     +           ( solut(ip,jp+1) - solut(ip,jp-1) ) )

         ip = ni
         jp = 1
         solut(ip+1,jp)=2.*solut(ip,jp)-solut(ip-1,jp)
         solut(ip,jp-1)=2.*solut(ip,jp)-solut(ip,jp+1)
         rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +           ( ctc(ip,jp,alphad,1) *
     +           ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +             ctc(ip,jp,alphad,2) *
     +           ( solut(ip,jp+1) - solut(ip,jp-1) ) )

         ip = 1
         jp = nj
         solut(ip-1,jp)=2.*solut(ip,jp)-solut(ip+1,jp)
         solut(ip,jp+1)=2.*solut(ip,jp)-solut(ip,jp-1)
         rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +           ( ctc(ip,jp,alphad,1) *
     +           ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +             ctc(ip,jp,alphad,2) *
     +           ( solut(ip,jp+1) - solut(ip,jp-1) ) )

         ip = ni
         jp = nj
         solut(ip+1,jp)=2.*solut(ip,jp)-solut(ip-1,jp)
         solut(ip,jp+1)=2.*solut(ip,jp)-solut(ip,jp-1)
         rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +           ( ctc(ip,jp,alphad,1) *
     +           ( solut(ip+1,jp) - solut(ip-1,jp) ) +
     +             ctc(ip,jp,alphad,2) *
     +           ( solut(ip,jp+1) - solut(ip,jp-1) ) )

      end if

      if (idisgra.eq.2) then

         dx(1,1)      =  0
         dx(1,2)      =  0
         dx(1,alphad) =  1

         do betad = 1, 2
            do gammad = 1, 2
               do jp=0, nj+1
                  do ip=0, ni+1
                     cvc(ip,jp,gammad,betad) =cva(ip,jp,gammad,betad,4)
                  end do
               end do
            end do
         end do

c        --- Determine contravariant WESBEEK vectors

         do jp=0, nj+1
            do ip=0, ni+1

c           --- First, calculate determinant ...

               det = cvc(ip,jp,1,1)*cvc(ip,jp,2,2) -
     +               cvc(ip,jp,2,1)*cvc(ip,jp,1,2)

c              --- then contravariant WESBEEK vectors

               ctc(ip,jp,1,1) =  cvc(ip,jp,2,2)/det
               ctc(ip,jp,2,1) = -cvc(ip,jp,1,2)/det
               ctc(ip,jp,1,2) = -cvc(ip,jp,2,1)/det
               ctc(ip,jp,2,2) =  cvc(ip,jp,1,1)/det

            end do
         end do

         do jp = 1, nj
            do ip = 1, ni

               uloc = bicof(ip+1,jp,1,1)
               vloc = bicof(ip+1,jp,2,1)
               i = ibicof(ip+1,jp,1,1)
               j = ibicof(ip+1,jp,2,1)

               sol2d(1,1) =      uloc *     vloc  * solut(i  ,j  )+
     +                           uloc *(1d0-vloc) * solut(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * solut(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * solut(i-1,j-1)

               uloc = bicof(ip,jp,1,1)
               vloc = bicof(ip,jp,2,1)
               i = ibicof(ip,jp,1,1)
               j = ibicof(ip,jp,2,1)

               sol2d(1,2) =      uloc *     vloc  * solut(i  ,j  )+
     +                           uloc *(1d0-vloc) * solut(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * solut(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * solut(i-1,j-1)

               uloc = bicof(ip,jp+1,1,2)
               vloc = bicof(ip,jp+1,2,2)
               i = ibicof(ip,jp+1,1,2)
               j = ibicof(ip,jp+1,2,2)

               sol2d(2,1) =      uloc *     vloc  * solut(i  ,j  )+
     +                           uloc *(1d0-vloc) * solut(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * solut(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * solut(i-1,j-1)

               uloc = bicof(ip,jp,1,2)
               vloc = bicof(ip,jp,2,2)
               i = ibicof(ip,jp,1,2)
               j = ibicof(ip,jp,2,2)

               sol2d(2,2) =      uloc *     vloc  * solut(i  ,j  )+
     +                           uloc *(1d0-vloc) * solut(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * solut(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * solut(i-1,j-1)

               rhside(ip,jp) = rhside(ip,jp) - jac(ip,jp,4) *
     +                 ( ctc(ip,jp,alphad,1) *
     +                 ( sol2d(1,1) - sol2d(1,2) ) +
     +                   ctc(ip,jp,alphad,2) *
     +                 ( sol2d(2,1) - sol2d(2,2) ) )

        end do
      end do

      end if

      end
