      subroutine isdswb2d ( alphad, ni, nj, nvirtual, locdx, iarea,
     +                       prems, matrix, cva, invjac, mct, invvar,
     +                       mcom, eddy )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 12-02-1999 compressible:extra array invvar
c        version 1.0    date 19-05-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Distribution of the intermediate matrix 'prems' into
c     the  matrix, Wesbeek discretization.
c     In the compressible case the unknown is rho*V, that means
c     that in the points where the contribution for V^a is evaluated
c     this contribution has to be devided by rho
c **********************************************************************
c
c                       KEYWORDS
c
c    stress
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, mcom, locdx(2), iarea(2,2),
     +        mct(1:2,1:2,-2:+2,-2:+2,-2:+2), alphad
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4),
     +                 matrix(1:ni+1,1:nj+1,2,*),
     +                 prems(1:ni+1,1:nj+1,2,4),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 invvar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     eddy      i    reserved
c     iarea     i    Area of arrays
c     invjac    i    1/sqrt(g) in pnttyp
c     invvar    i    Array with 1/variable
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     matrix    i    reserved
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prems     i    Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ip, jp, betad,  deltad, pnttype,
     +        iconct, ish, ish4, jsh, jsh4,  upnt, ispnt,
     +        onepnt, fourpnt, locu(2), loc1(2), loc4(2), loc4t(2),
     +        locupnt(1:4,1:2,1:2), locspnt(1:4,1:2),
     +        acttyp, vartyp, pntvar, npointsv
      double precision  fact1pnt, fact4pnt, dummy(1)
      data locupnt / -1, +1,  0,  0,
     +                 0,  0, -1, +1,
     +                 0,  0, -1, +1 ,
     +                -1, +1,  0,  0 /
      data locspnt / -1, +1, -1, +1,
     +                +1, +1, -1, -1 /

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     deltad    Direction index
c     dummy     Dummy
c     fact1pnt  Factor for one point contribution
c     fact4pnt  Factor for four point contribution
c     fourpnt   Variable indicating what kind of point the four points are
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     ip        Local counting variable i-direction
c     ish       Shift in i-direction
c     ish4      Shift for the four point distribution in i-direction
c     ispnt     Counting variable for surrounding points
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     jsh4      Shift for the four point distribution in j-direction
c     loc1      Help variable to change from local shift to shift in stencil
c     loc4      Help variable to change from local shift to shift in stencil
c     loc4t     Help variable to change from local shift to shift in stencil
c     locspnt   Shift of four surrounding V-points in units of half a cell
c     locu      Shift of u-point w.r.t. cell centre
c     locupnt   Shift of four surrounding u-points in units of half a cell
c     npointsv  Number of points in the mesh with virtual points
c     onepnt    Variable indicating what kind of point the one points is
c     pnttype   Array containing the point types for the various directions
c     pntvar    Point type for var
c     upnt      Counting variable for u points
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      The Wesbeek discrtization is carried out by a loop over
c      the cells. For each cell we do a loop over the integration
c      points.
c
c      In a integration point we need the gradient of the
c      velocity vector. Using the path-integral method the
c      gradient of the velocity vector is expressed in
c      terms of the cartesian velocities in the 4
c      surrounding points.
c
c      The coefficients for these 4 cartesian velocities are
c      stored in array prems(i,j,l,gamma)(l=1:2;gamma=1:4),
c      where l refers to the vector and gamma to the 4 points.
c
c      In this subroutine these contributions are distributed
c                alpha
c      over the V      components and stored at the right positions
c      in the matrix.
c
c      The shifts of the 4 surrounding points with respect to
c      the integration point are stored in array
c      locupnt(1:4,1:2,1:2)
c
c      locupnt(i,j,alphad) refers to the i-th point, shift in
c      j direction for an alphad cell. For example in an
c      alphad=1 cell point 1 is at the left of the integration
c      point and has a local shift of (-1,0). Hence
c      locupnt(1,.,1)=(-1,0).
c
c      The shift of the integration point with respect to the
c      integration cell center is stored in locdx.
c      Combination of locupnt and locdx gives the shift of the
c      new points with respect to the integration cell center.
c                                         alpha
c      Each cartesian velocity component u      may be expressed
c      into the contravariant fluxes by
c
c
c                alpha  1       alpha  2
c               a      V   +   a      V
c       alpha    (1)            (2)
c      u      = -----------------------
c
c                      sqrt(g)
c
c      In each point where the velocity must be evaluated either
c           1        2
c      the V or the V  unknown is present. The other one must be
c      interpolated from 4 surrounding points.
c      If the composite shift with respect to the integration cell
c      center has even components then the V alphad component
c      is available and the other one must be interpolated,
c      otherwise vice versa.
c      The local shifts for this interpolation is stored in
c      array locspnt(1:4,1:2) whre thr first component refers to
c      the point and the second one to the component of the shift.
c
c      Once the shifts with respect to the integration cell center
c      are known the global i,j position of the point can be
c      computed by transforming the local shift into the global
c      shift. This can be done by the following formulae
c
c      contribution to alphad in alphad cell
c         shift(alphad) = shift(alphad)/2
c         shift(betad)  = shift(betad)/2
c      contribution to betad in alphad cell
c         shift(alphad) = (shift(alphad)-1)/2
c         shift(betad)  = (shift(betad)+1)/2
c
c      These global shifts are used in array mct. Array mct gives
c      withe these shifts the diagonal number in the matrix.
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      betad  = 3 - alphad

      do upnt = 1, 4

c     --- location u punt

         do i =1, 2
             locu(i) =  locdx(i) + locupnt(upnt,i,alphad)
         end do

c        --- check the u-point, is it an alphad-point or a betad-point
c            the onepnt contribution can be distributed directly
c            the fourpnt contribution must be distributed over 4
c            surrounding points

         if ( mod( locu( alphad),2).eq.0) then
            onepnt  = alphad
            fourpnt = betad
            ish  = locu(1)/2
            jsh  = locu(2)/2
         else
            onepnt  = betad
            fourpnt = alphad
            loc1(alphad) = ( locu(alphad) -1 )/ 2
            loc1(betad)  = ( locu(betad)  +1 )/ 2
            ish  = loc1(1)
            jsh  = loc1(2)
         end if

c        --- contribution for one point (onepnt)

         if ( onepnt.eq.1) then
            pnttype = 3
         else
            pnttype = 2
         end if

         if ( mcom.eq. 1) then

c        --- compressible interpolate invvar from eddy

            acttyp = 0
            vartyp = 3
            pntvar = pnttype
            call isintcf2 ( eddy, invvar, ni, nj, nvirtual,
     +                       vartyp, acttyp, pntvar, 0, dummy )
         else

c        --- incompressible invvar = 1

            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            call issetcon ( invvar, npointsv, 1d0 )
         end if

         iconct = mct ( alphad, onepnt, ish, jsh, 0)

         do deltad = 1, 2

            do jp = iarea(1,2), iarea(2,2)
               do ip = iarea(1,1), iarea(2,1)

                  fact1pnt =
     +               cva(ip+ish,jp+jsh,deltad,onepnt,pnttype) *
     +                         invjac(ip+ish,jp+jsh,pnttype)  /
     +                         invvar(ip+ish,jp+jsh)
                  matrix(ip,jp,alphad,iconct) =
     +                 matrix(ip,jp,alphad,iconct) +
     +                     prems(ip,jp,deltad,upnt) * fact1pnt
               end do
            end do
         end do

c        --- contribution for four points ( fourpnt)

         do ispnt = 1,4

c        --- location 4 surrounding points

            do i =1, 2
               loc4(i) =  locu(i) + locspnt(ispnt,i)
            end do

            if ( fourpnt.eq.alphad) then
               ish4 = loc4(1)/2
               jsh4 = loc4(2)/2
            else
               loc4t(alphad) = ( loc4(alphad)-1)/2
               loc4t(betad)  = ( loc4(betad)+1)/2
               ish4 = loc4t(1)
               jsh4 = loc4t(2)
            end if

            if ( mcom.eq. 1) then

c           --- compressible interpolate invvar from eddy

               acttyp = 0
               vartyp = 3
               pntvar = 1+onepnt
               call isintcf2 ( eddy, invvar, ni, nj, nvirtual,
     +                       vartyp, acttyp, pntvar, 0, dummy )
            else

c           --- incompressible invvar = 1

               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
               call issetcon ( invvar, npointsv, 1d0 )
            end if

            iconct = mct ( alphad, fourpnt, ish4, jsh4, 0)

            do deltad = 1, 2

               do jp = iarea(1,2), iarea(2,2)
                  do ip = iarea(1,1), iarea(2,1)

                     fact4pnt =
     +                   cva(ip+ish,jp+jsh,deltad,fourpnt,pnttype) *
     +                         invjac(ip+ish,jp+jsh,pnttype) * 25d-2/
     +                         invvar(ip+ish4,jp+jsh4)
                     matrix(ip,jp,alphad,iconct) =
     +                     matrix(ip,jp,alphad,iconct) +
     +                        prems(ip,jp,deltad,upnt) * fact4pnt
                  end do
               end do
            end do
         end do

      end do

      end
