      subroutine isexbo21(iblock , itrans, itimlv, nblocks, isiinput,
     +                    issolut, isitopol, ibinfor, ibipoint, bcgeo,
     +                    bndcon)
c ======================================================================
c
c        programmer     Kees Vuik
c
c        version 1.2    date 29-10-1998 Duncan: iseropen/iserclose added
c        version 1.1    date 06-08-1998 Guus: new call to isexco02
c        version 1.0    date 16-10-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine fills the virtual points of the local solution arrays
c     for outer boundaries. Some boundary conditions are taken into account
c **********************************************************************
c
c                       KEYWORDS
c
c   extrapolation
c   virtual
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  iblock, itrans, itimlv, nblocks, issolut,
     +         isiinput, isitopol, ibipoint(*), ibinfor(*)
      double precision bcgeo(8,*), bndcon(2,*)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iblock    i    Block sequence number
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     itrans    i    reserved
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipsolut, ipiinput, ipitopol, ipnsubf, iperiod,
     +        ipsubf, maxnsubf, ndim, nfaces, ni, nj,
     +        nvirtual, ndegfd, ntimlv, iface

c     iface     Face number
c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsolut   Starting address of array solut in buffer
c     ipsubf    Starting address of the subfaces array
c     itimlv    Loop variable with respect to a given time level
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISEXBO22  Fills virtual points at the sides for theta is 0
c     ISEXVC02  Compute virtual unknowns in corner
c               points by extrapolation (2d case)
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      fill parameters from array itopol
c
c      fill parameters from array iinput
c      --- actual extrapolation
c      call isexso
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('isexbo21')

c     --- compute pointers and some parameters for the itopol array

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- fill the  virtual unknowns by extrapolation

      call isactive(isiinput)
      call isactive(issolut)
      ipiinput = isingetp(isiinput)
      ipsolut  = isdpgetp(issolut)

      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      iperiod  = ibuffr(ipiinput + 56 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      if ( nerror.ne.0 ) go to 1000

      if (ndim.eq.2) then

c     --- handle external (sub)faces

         do iface = 1, nfaces

            if ( iface.eq.1 .and. mod(iperiod   ,10).eq.0   .or.
     +           iface.eq.2 .and. mod(iperiod   ,10).eq.0   .or.
     +           iface.eq.3 .and. mod(iperiod/10,10).eq.0   .or.
     +           iface.eq.4 .and. mod(iperiod/10,10).eq.0 ) then

            call isexbo22(iblock , iface, itrans, ni, nj, nvirtual  ,
     +                    ndegfd , itimlv , ntimlv, nblocks, nfaces ,
     +                    maxnsubf,buffer(ipsolut), ibuffr(ipnsubf) ,
     +                    ibuffr(ipsubf), ibinfor, ibipoint, bcgeo,
     +                    bndcon)
            if ( nerror.ne.0 ) go to 1000

            else

               write(irefwr,*) 'Periodic boundary conditions are not',
     +                    ' implemented in the case theta = 0',
     +                    ' error message from isexbo21'
               go to 1000

            end if

         end do

c        --- handle corner points

         call isexvc02(iblock , itrans, ni, nj, nvirtual, ndegfd,
     +                 itimlv , ntimlv, nblocks, nfaces , maxnsubf,
     +                 buffer(ipsolut), ibuffr(ipnsubf),
     +                 ibuffr(ipsubf) )
         if ( nerror.ne.0 ) go to 1000

      else
         write(irefwr,*) 'For ndim is', ndim, ' the method does not',
     +              ' work in the case theta = 0'
         go to 1000
      end if

1000  call isdcrpri

      call iserclos('isexbo21')

      end
