      subroutine isexm2d ( iinput, matrix, rhside, solut, coefs, work,
     +                     rinput, npointsv, infmm,
     +                     vsum, dvsum, invjac, coor, mct, bndcon,
     +                     ibipoint, ibinfor, jac, eddy, iblock ,
     +                     itrans, itimlv, nblocks, isiinput, issolut,
     +                     isitopol, gmat, infmg )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.12   date 29-09-1999 Guus: new call to ismomrh2
c        version 1.11   date 29-10-1998 Duncan: new call to isextim2
c        version 1.10   date 08-10-1997 Guus: adaptation to multi-phase gas flow
c        version 1.9    date 21-10-1997 Guus: correction dt
c        version 1.8    date 03-07-1997 Guus: new call to isintcf2
c        version 1.7    date 12-03-1997 Guus: possibility reduced stress
c        version 1.6    date 30-01-1997 Hester: 2nd extension for compr. N-S
c        version 1.5    date 09-10-1996 Guus: extension for compressible N-S
c        version 1.4    date 04-10-1996 Kees K: call changed of ismomcv2
c        version 1.3    date 03-10-1996 Kees K: changes for wesbeek
c                       discretization. Place and call of isgmat2d
c                       and pointers.
c        version 1.2    date 07-12-1995 Marcel: local parameter acttyp
c        version 1.1    date 30-10-1995 Guus: new calls to subroutines
c        version 1.0    date 06-03-1995 subroutine based on ideas of ismcci
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum matrix in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmm(*), infmg(*),
     +        mct(2,2,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        iblock , itrans, itimlv, nblocks, isiinput, issolut,
     +        isitopol
      double precision rhside(*), work(*), matrix(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*),  invjac(npointsv,*),
     +                 dvsum(npointsv,2,2), bndcon(2,*), jac(*),
     +                 eddy(npointsv), gmat(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     dvsum          Flux derivative contributions to the matrix
c     eddy      i    reserved
c     gmat      i    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iblock    i    Block sequence number
c     iinput    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmm     i    Integer information for Momentum matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     nblocks   i    Number of blocks
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     vsum           Flux contributions to the matrix
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprho, nconct, alphad, iprintim, pnttyp, acttyp,
     +        vartyp, ni, nj, nvirtual, ipmu, mequat,
     +        iarea(2,2), locdx(2), dcell(2),
     +        iparea(2,2), ipcva, ipjcta, ipjctg,
     +        ipchsym , ipveloc, iptemp, ipgeo,
     +        i, idir, iside, ipiindsc, mcom, ipiincom,
     +        ndegfd, ntimlv, nintpol(2), dij(2,4,2),
     +        ipadir, idiscret, ipgradu, ipprems
      double precision dt
      logical reuse, tangcell, reduced

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     reserved
c     dij       Array containing the distances of the interpolation
c               points to the cell centra in global co-ordinates
c               dij(alpha,i,j) denotes the distance of the i-th
c               interpolation point, component j and with respect to
c               the V-alpha unknown (distribution)
c     dt        Time step
c     i         General loop variable
c     iarea     Area of arrays
c     idir      General loop variable for 1...ndim loop
c     idiscret  Type of discretization
c     ipadir    Starting address of array ADIR in array WORK
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipgeo     Starting address of array GEO in array WORK
c     ipgradu   Starting address of array GRADU in array WORK
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ipmu      Starting address of array MU in array WORK
c     ipprems   Starting address of array PREMS in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     ipveloc   Starting address of array VELOC in array WORK
c     iside     Sequence number of side
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     mcom      If zero the flow is incompressible else (1) compressible
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nintpol   Number of interpolation points in the distribution
c     nj        Number of non-virtual cells in the j-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     reduced   If true the reduced stress tensor corresponding
c               to a constant viscosity and incompressible flow is used
c     reuse     If true the computed coefficients may be reused again
c     tangcell  If true a tangential cell has been found with integration point
c               at the boundary
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBGEO2D  Fill geometrical quantities along boundary in R2
c     ISBMOMC2  Fill normal half cells for convection in R2
c     ISBMOMS1  Fill tangential cells for stress in R2
c     ISBMOMS2  Fill normal half cells for stress in R2
c     ISCHRYS2  Fill chrystoffel symbols in one integration point in R2
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISEXBO21  Fills virtual points at outer boundaries for theta is 0
c     ISEXBOU2  Fills the boundary conditions for theta = 0
c     ISEXMAC2  ?
c     ISEXMAT2  ?
c     ISEXTIM2  Calculates the solution at the new time level
c               for theta=0
c     ISGMAT2D  Compute gradient matrix in R^2
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISJCTG2D  Fill Jacobian times contravariant metric tensors
c               in one integration point in R2
c     ISMAKMP2  Make a list of integration points for the momentum equations
c               in R2
c     ISMOMCV2  Fill convective terms for one integration point in R2
c     ISMOMRH2  Build the right-hand side source term in R2
c     ISMOMST2  Fill stress terms for one integration point in R2
c     ISMTVC23  Gradient matrix times pressure in 2D
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Make list of integration points
c    Interpolate coefficients to integration points
c    For idim := 1 (1) 2 do
c       Build time-derivative part of the matrix and rhside
c       Make list of integration points
c       For all integration points do
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          distribute convective terms
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct boundary cells for stress terms
c          distribute stress terms
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ipiindsc = iinput(6)
      iprintim = iinput(29)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)
      mequat   = iinput(ipiincom+2)
      nconct   = infmm(6)
      dt       = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      ipcva    = 1
      ipjcta   = ipcva+16*npointsv
      ipjctg   = ipjcta+4*npointsv
      ipchsym  = ipjctg+3*npointsv
      iprho    = ipchsym+6*npointsv
      ipmu     = iprho+npointsv
      ipveloc  = ipmu+npointsv

      ipadir   = ipjctg
      ipgradu  = ipveloc+2*npointsv
      ipprems  = ipgradu+8*npointsv
      ipgeo    = ipprems+16*npointsv
      idiscret = iinput(ipiindsc+1)
      if ( idiscret.eq.3 ) then
         idiscret = 1
         reduced = .true.
      else
         reduced = .false.
      end if

      if ( iinput(ipiindsc-1+8).ne.2 ) then
         if ( inodenr.eq.0 ) then
            write(irefwr,*) '*************************************'
            write(irefwr,*) 'Warning: for theta is 0 the nonlinear',
     +                 ' solver should be the Picard',
     +                 ' method'
            write(irefwr,*) '*************************************'
         end if
         iinput(ipiindsc-1+8) = 2
      end if

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*nconct*2, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*2, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- The part below is only for testing *************************

c         --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill array bcgeo and put into array work

      call isbgeo2d ( work(ipgeo), ni, nj, nvirtual, work(ipcva),
     +                ibipoint, invjac, jac )

c     --- Finally the boundary conditions are used and the virtual points
c         are filled

      ntimlv = 2
      call isexbou2 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

      call isexbo21 ( iblock, itrans, itimlv, nblocks,
     +                isiinput, issolut, isitopol,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

c     --- The part  above is only for testing *************************

      do alphad = 1, 2

c     --- Build time-derivative part of the matrix and rhside
c         Interpolate rho to integration point (Cell centre)

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
         else
            iarea(2,2) = nj+1
         end if
         pnttyp = 4-alphad
         vartyp = 3

c        --- Loop over integration points

         do i = 1, 5

c        --- Compute local co-ordinates, distances to cell center in global
c            co-ordinates, point type, number of interpolation points
c            and interpolatoin formula

            call ismakmp2 ( i, alphad, locdx, pnttyp, dcell,
     +                      nintpol, dij )

            reuse = locdx(1).eq.1 .or. locdx(2).eq.1

            if ( .not. reuse ) then

c           --- Compute inverse of Jacobian, contravariant basis vectors
c               and christoffel symbols

               call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                         work(ipjcta), ni, nj, nvirtual )

c              --- The metric tensor times sqrt(g)

               call isjctg2d ( invjac(1,pnttyp), work(ipjcta),
     +                         work(ipjctg), ni, nj, nvirtual )

c              --- The christoffel symbols of the second kind

               call ischrys2 ( work(ipcva), work(ipjcta), work(ipchsym),
     +                         invjac(1,pnttyp), ni, nj, nvirtual,
     +                         pnttyp )

            end if

c           --- List of loop bounds for all integration points
c               iparea gives the bounds for the cell indices of the integration
c               points
c               These are the indices of the centres of the integration cells
c               but shifted over a distance dcell, since the actual
c               integration points may be in another cell

            iparea(1,1) = iarea(1,1)+dcell(1)
            iparea(2,1) = iarea(2,1)+dcell(1)
            iparea(1,2) = iarea(1,2)+dcell(2)
            iparea(2,2) = iarea(2,2)+dcell(2)

            if ( iinput(ipiindsc).eq.1 .and. iinput(ipiindsc+1).eq.1 )
     +         then

c           --- iindsc(1)=1, use convection term
c               First clear array vsum

               call issetcon ( vsum, 2*npointsv, 0d0 )

c              --- Fill some coefficients

               if ( i.ne.1 .and. .not. reuse ) then

c              --- Fill rho except for first integration point

                  vartyp = 3
                  acttyp = 0
                  call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
               end if

               if ( .not. reuse ) then

c              --- Fill velocity componentwise

                  acttyp = 0
                  do idir = 1, 2
                     vartyp = idir
                     iptemp = ipveloc+(idir-1)*npointsv
                     call isintcf2 ( solut(1,idir), work(iptemp), ni,
     +                               nj, nvirtual, vartyp, acttyp,
     +                               pnttyp, 0, coor )
                  end do

               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

c                  we use rho in ismomcv2, so here it is filled, later on this
c                  may be done in a more efficient way

               if ( mcom.eq.0 ) then

c              --- incompressible case, interpolate rho from coefs

                  acttyp = 0
                  call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )

               else

c              --- compressible case, interpolate rho from eddy

                  acttyp = 0
                  call isintcf2 ( eddy, work(iprho), ni, nj, nvirtual,
     +                            vartyp, acttyp, pnttyp, 0, coor )

               end if

               call ismomcv2 ( ni, nj, nvirtual, work(iprho),
     +                         work(ipchsym), invjac(1,pnttyp),
     +                         work(ipveloc), vsum, alphad, locdx,
     +                         iinput(ipiindsc), iparea, work(ipadir),
     +                         work(ipcva+4*npointsv*(pnttyp-1)) )

c              --- Correct contribution of convective terms for half cells
c                  this goto is used since the method is not applicable to
c                  all the parts: convection etc vartype
c                  is used as dummy

               if ( .false. ) then

c              --- this goto is used since the method is not applicable to
c                  all the parts: tangential boundary cells

                  call isbmomc2 ( ni, nj, nvirtual, work(ipveloc), vsum,
     +                            alphad, work(iprho), invjac(1,pnttyp),
     +                            work(ipchsym), locdx, dcell, ibinfor,
     +                            ibipoint, iinput(ipiindsc),
     +                            work(ipadir),
     +                            work(ipcva+4*npointsv*(pnttyp-1))  )

               end if

c              --- Distribute computed values over the positions
c                  corresponding to the unknowns

               call isexmac2 ( ni, nj, nvirtual,
     +                            rhside, vsum,
     +                            dcell, alphad, iarea,
     +                            nintpol, dij, mct, solut, ndegfd )

            end if

            if ( iinput(ipiindsc+2).eq.0 ) then

c           --- Stress terms according to standard ISNaS
c               Clear arrays vsum and dvsum

               call issetcon ( vsum, 2*npointsv, 0d0 )
               call issetcon ( dvsum, 4*npointsv, 0d0 )
               vartyp = 3
               acttyp = 0
               call isintcf2 ( coefs(1,2), work(ipmu), ni, nj,
     +                         nvirtual, vartyp, acttyp, pnttyp, 0,
     +                         coor )

               call ismomst2 ( ni, nj, nvirtual, work(ipmu),
     +                         work(ipjctg), work(ipchsym), vsum,
     +                         dvsum, alphad, locdx, iparea, nconct,
     +                         jac, reduced, mequat )

c              --- this goto is used since the method is not applicable to
c                  all the parts:  tangential boundary cells vartype
c                  is used as dummy

               if ( .false. ) then

c              --- this goto is used since the method is not applicable to
c                  all the parts: tangential boundary cells

c                        --- Correction for tangential boundary cells

                  if ( i.gt.1 ) then

c                 --- Only the integration point at the boundary is important

                     tangcell = .false.
                     if ( alphad.eq.1 ) then
                        if ( locdx(2).eq.-1 ) then
                           iside = 3
                           tangcell = .true.
                        else if ( locdx(2).eq.1 ) then
                           iside = 4
                           tangcell = .true.
                        end if
                     else
                        if ( locdx(1).eq.-1 ) then
                           iside = 1
                           tangcell = .true.
                        else if ( locdx(1).eq.1 ) then
                           iside = 2
                           tangcell = .true.
                        end if
                     end if

c                    --- Get information on boundary

                     if ( tangcell )
     +               call isbmoms1 ( ni, nj, nvirtual, iside, vsum,
     +                               dvsum, alphad, work(ipmu),
     +                               work(ipjctg), work(ipchsym), dcell,
     +                               ibinfor, ibipoint, bndcon, rhside,
     +                               matrix, invjac, mct, nconct,
     +                               reduced, mequat )
                  end if

c                 --- Correct contribution of stress terms for half cells

                  call isbmoms2 ( ni, nj, nvirtual, vsum, dvsum, alphad,
     +                            work(ipmu), work(ipjctg),
     +                            work(ipchsym), locdx, dcell, ibinfor,
     +                            ibipoint, bndcon, rhside, nconct,
     +                            jac, reduced, mequat )

               end if

c              --- Distribution of matrix

               call isexmat2 ( i, ni, nj, nvirtual, vsum, dvsum,
     +                         invjac, mct, dcell, alphad, iarea,
     +                         nintpol, dij, rhside, solut, ndegfd,
     +                         nconct )
            end if

         end do

c        --- Compute the g-matrix

         call isgmat2d ( alphad, ni, nj, nvirtual, gmat, work(ipcva),
     +                   invjac, ibipoint, ibinfor, idiscret, jac,
     +                   work(ipadir) )

      end do

c     --- Update the right-hand side vector with -(dt/rho)*Gp^n
c         In two-phase flow: -eps Gp^n**

      call ismtvc23 ( gmat, solut(1,3), rhside, ni, nj, nvirtual,
     +                infmg(infmg(12)), coefs(1,6), mequat )

c     --- Update the right-hand side vector with the source terms

      if ( mcom.eq.0 ) then

c     --- incompressible case, use rho from coefs

         call ismomrh2 ( coefs(1,3), coefs(1,1), coor, work, gmat,
     +                   infmg(infmg(12)), rhside,
     +                   work(1+2*npointsv), ni, nj, nvirtual,
     +                   ibipoint, ibinfor, work(ipcva) )

      else

c     --- compressible case, use rho from eddy

         call ismomrh2 ( coefs(1,3), eddy, coor, work, gmat,
     +                   infmg(infmg(12)), rhside,
     +                   work(1+2*npointsv), ni, nj, nvirtual,
     +                   ibipoint, ibinfor, work(ipcva) )

      end if

c     --- Update the solution vector on the new time

      if ( mcom.eq.0 ) then

c     --- incompressible case, use rho from coefs

         call isextim2 ( solut, rhside, coefs(1,1), dt, ni, nj,
     +                   nvirtual, ndegfd, mcom, 1)

      else

c     --- compressible case, use rho from eddy

         call isextim2 ( solut, rhside, eddy, dt, ni, nj,
     +                   nvirtual, ndegfd, mcom, 1)

      end if

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill array bcgeo and put into array work

      call isbgeo2d ( work(ipgeo), ni, nj, nvirtual, work(ipcva),
     +                ibipoint, invjac, jac )

c     --- Finally the boundary conditions are used and the virtual points
c         are filled

      ntimlv = 2
      call isexbou2 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

      call isexbo21 ( iblock, itrans, itimlv, nblocks,
     +                isiinput, issolut, isitopol,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

      end
