      subroutine isgmatr2 ( ni, nj, nvirtual, mcom, gmat, densrho,
     +                      rhoinv, rinput, iinput, ibipoint, ibinfor,
     +                      massfrac )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 10-02-1999 KeesK: in compressible case the
c                                       value of densrho = 1d0
c        version 2.0    date 03-09-1997 Hester: extra parameter massfrac
c        version 1.0    date 06-10-1995
c
c   copyright (c) 1995-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build gradient matrix in R2 multiplied by -2 dt / rho
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, mcom, iinput(*), ibipoint(*), ibinfor(*)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 rhoinv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2),
     +                 gmat(ni+1,nj+1,2,6),
     +                 rinput(*),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual)

c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     gmat      o    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     massfrac  i    Array containing the mass fraction
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhoinv    i    Array containing the inverse of rho in the integration
c                    points, multiplied by theta dt
c     rinput    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter ( snnsnt = 7, snnut = 6 )
      integer i, j, k, iparea(2,2), tdir, ndir, iside,
     +        ipibinf, mxp(2), btype, mequat, npointsv
      double precision dt

c     btype     Type of boundary condition
c     dt        Time step
c     i         General loop variable
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     j         General loop variable
c     k         General loop variable
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     npointsv  Number of points in the mesh with virtual points
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Evaluate -2 dt / rho in the integration points
c    Correct this factor for half cells (multiply by 1/2)
c    Fill array gmat in the same as in ISGMAT02
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      dt = rinput(iinput(29)-1+4)
      mequat = iinput(iinput(9)+2)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      if ( mcom.eq.1 ) then
         call issetcon(densrho, npointsv, 1d0 )
      end if

c     --- Evaluate 1/rho in integration points

      do j = 1, nj
         do i = 1, ni+1
            rhoinv(i,j,1) = -2d0*dt / (densrho(i-1,j)+densrho(i,j))
         end do
      end do

      do j = 1, nj+1
         do i = 1, ni
            rhoinv(i,j,2) = -2d0*dt / (densrho(i,j-1)+densrho(i,j))
         end do
      end do

      if ( mequat.eq.1 ) then

c     --- Multiplication by mass fraction

         do j = 1, nj
            do i = 1, ni+1
               rhoinv(i,j,1)=rhoinv(i,j,1)*0.5d0*
     +                       (massfrac(i-1,j)+massfrac(i,j))
            end do
         end do

         do j = 1, nj+1
            do i = 1, ni
               rhoinv(i,j,2)=rhoinv(i,j,2)*0.5d0*
     +                       (massfrac(i,j-1)+massfrac(i,j))
            end do
         end do

      end if

c     --- Correct rhoinv by a factor 2 for boundary points corresponding to
c         half cells

      do iside = 1, 4
         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            tdir = 2
            ndir = 1
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            tdir = 1
            ndir = 2
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if

         do j = iparea(1,2), iparea(2,2)
            do i = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

               mxp(1) = i
               mxp(2) = j

c              --- Get integration cell position for ibinfor

               btype = ibinfor(ipibinf+2*mxp(tdir))
               if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c              --- stress given, multiply rhoinv by 2

                  rhoinv(i,j,ndir) = 2d0*rhoinv(i,j,ndir)

               end if

            end do
         end do

      end do

      do k = 1, 6
         do j = 1, nj
            do i = 1, ni+1
               gmat(i,j,1,k) = gmat(i,j,1,k)*rhoinv(i,j,1)
            end do
         end do
         do j = 1, nj+1
            do i = 1, ni
               gmat(i,j,2,k) = gmat(i,j,2,k)*rhoinv(i,j,2)
            end do
         end do
      end do

      end
