      subroutine isgmatr3 ( ni, nj, nk, nvirtual, gmat, densrho,
     +                      rhoinv, rinput, iinput, ibipoint, ibindex,
     +                      ibinfor )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 13-03-1996 call to isgmtr23 i.o. isgmatr23
c        version 1.0    date 02-02-1996
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c    Build gradient matrix in R3 multiplied by - dt / rho
c    dt is not yet used in this subroutine
c
c **********************************************************************
c
c                      KEYWORDS
c
c    3d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iinput(*), ibipoint(*),
     +        ibindex(2,2,*), ibinfor(*)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual),
     +                 rhoinv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3),
     +                 gmat(ni+1,nj+1,nk+1,3,10),
     +                 rinput(*)

c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     gmat      i    Gradient matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhoinv    i    Array containing the inverse of rho in the integration
c                    points, multiplied by theta dt
c     rinput    i    reserved
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ilow1, ihigh1, ilow2, ihigh2, iface
      integer i, j, k, m
      double precision dt

c     dt        Time step
c     i         General loop variable
c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh2    Second upper index of an array
c     ilow1     First lower index of an array
c     ilow2     Second lower index of an array
c     j         General loop variable
c     k         General loop variable
c     m         General loop variable
c
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c    Evaluate -2 dt / rho in the integration points
c    Correct this factor for half cells (multiply by 1/2)
c    Fill array gmat in the same as in ISGMAT02
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c

      dt = rinput(iinput(29)-1+4)

c     --- Evaluate 1/rho in integration points

      do k = 1, nk
         do j = 1, nj
            do i = 1, ni+1
               rhoinv(i,j,k,1) = -2d0*dt /
     +                           (densrho(i-1,j,k)+densrho(i,j,k))
            end do
         end do
      end do

      do k = 1, nk
         do j = 1, nj+1
            do i = 1, ni
               rhoinv(i,j,k,2) = -2d0*dt /
     +                           (densrho(i,j-1,k)+densrho(i,j,k))
            end do
         end do
      end do

      do k = 1, nk+1
         do j = 1, nj
            do i = 1, ni
               rhoinv(i,j,k,3) = -2d0*dt /
     +                           (densrho(i,j,k-1)+densrho(i,j,k))
            end do
         end do
      end do

c     --- Correct rhoinv by a factor 2 for boundary points corresponding to
c         half cells

      do iface = 1, 6
         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)
         call isgmtr23 ( iface, ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                   ilow2, ihigh2, ibinfor(ibipoint(iface)),
     +                   rhoinv )
      end do

      do m = 1, 10
         do k = 1, nk
            do j = 1, nj
               do i = 1, ni+1
                  gmat(i,j,k,1,m) = gmat(i,j,k,1,m)*rhoinv(i,j,k,1)
               end do
            end do
         end do
         do k = 1, nk
            do j = 1, nj+1
               do i = 1, ni
                  gmat(i,j,k,2,m) = gmat(i,j,k,2,m)*rhoinv(i,j,k,2)
               end do
            end do
         end do
         do k = 1, nk+1
            do j = 1, nj
               do i = 1, ni
                  gmat(i,j,k,3,m) = gmat(i,j,k,3,m)*rhoinv(i,j,k,3)
               end do
            end do
         end do
      end do

      end
