      subroutine isgmatw02 ( ni, nj, nk, nvirtual, gmat, xcentr, jac,
     +                       alphad, adir, iface, ibinfor, ilow1,
     +                       ihigh1, ilow2, ihigh2 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 01-04-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute boundary part of gradient matrix in R^3 for one face
c    WesBeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad, iface, ilow1, ihigh1,
     +        ilow2, ihigh2, ibinfor(ilow1:ihigh1,ilow2:ihigh2)
      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 xcentr(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     gmat      o    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     xcentr    i    Coordinates of centroids of cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, l, iarea(3,2), iperm(5), ishift(3), jshift1(3),
     +        jshift2(3), jshift3(3), jshift4(3), kshift1(3), btype,
     +        kshift2(3), kshift3(3), kshift4(3), tdir(2), mxp(3),
     +        igmat1, igmat2, igmat3, igmat4, igmat5, igmat6, igmat7,
     +        igmat8, igmat9, igmat10, ia1, ia2, ia3
      double precision fact, facs, cmat(3,3), det

c     btype     Type of boundary condition
c     cmat      c_matrix
c     det       Determinant
c     facs      Multiplication factor
c     fact      Multiplication factor
c     i         General loop variable
c     ia1       alphad (cyclic)
c     ia2       alphad+1 (cyclic)
c     ia3       alphad+2 (cyclic)
c     iarea     Area of arrays
c     igmat1    Position in Gradient-matrix
c     igmat10   Position in Gradient-matrix
c     igmat2    Position in Gradient-matrix
c     igmat3    Position in Gradient-matrix
c     igmat4    Position in Gradient-matrix
c     igmat5    Position in Gradient-matrix
c     igmat6    Position in Gradient-matrix
c     igmat7    Position in Gradient-matrix
c     igmat8    Position in Gradient-matrix
c     igmat9    Position in Gradient-matrix
c     iperm     Array of permutations of the numbers 1, 2 and 3
c               Contents: 1, 2, 3, 1, 2
c     ishift    Shift in i-direction
c     j         General loop variable
c     jshift1   Shift in j-direction
c     jshift2   Shift in j-direction
c     jshift3   Shift in j-direction
c     jshift4   Shift in j-direction
c     k         General loop variable
c     kshift1   Shift in k-direction
c     kshift2   Shift in k-direction
c     kshift3   Shift in k-direction
c     kshift4   Shift in k-direction
c     l         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The following discretization is used in the WesBeek case
c
c   V^1-cell:
c
c      a^(1) . sqrt(g) grad p (0,0,0), with
c
c      grad p (0,0,0) = c^1 (p(1,0,0)-p(-1,0,0)) +
c                       c^2 (p(-1,2,0)-p(-1,-2,0) + p(1,2,0)-p(1,-2,0))+
c                       c^3 (p(-1,0,2)-p(-1,0,-2) + p(1,0,2)-p(1,0,-2))
c
c      c_1 = x(1,0,0)-x(-1,0,0)
c      c_2 = x(-1,2,0)-x(-1,-2,0) + x(1,2,0)-x(1,-2,0)
c      c_3 = x(-1,0,2)-x(-1,0,-2) + x(1,0,2)-x(1,0,-2)
c
c      Cmat = ( c_1, c_2, c_3 ) ^T
c
c      c^1 = Cmat^(-1) (1,0,0)^T
c      c^2 = Cmat^(-1) (0,1,0)^T
c      c^3 = Cmat^(-1) (0,0,1)^T
c
c
c      The other cells arise by cyclic permutation
c
c      At the boundaries we use one-sided approximations in case of tangential
c      cells, provided there are no periodical or multiblock boundary conditions
c
c   To make the program alpha-independent we define the following shift arrays:
c   ishift, jshift1..4, kshift1..4
c   The matrix c is then defined as:
c      c_1 = x(i,j,k)-x(i+ishift(1),j+ishift(2),k+ishift(3))
c      c_2 = x(i+jshift1(1),j+jshift1(2),k+jshift1(3))-
c            x(i+jshift2(1),j+jshift2(2),k+jshift2(3))+
c            x(i+jshift3(1),j+jshift3(2),k+jshift3(3))-
c            x(i+jshift4(1),j+jshift4(2),k+jshift4(3))
c      c_3 = x(i+kshift1(1),j+kshift1(2),k+kshift1(3))-
c            x(i+kshift2(1),j+kshift2(2),k+kshift2(3))+
c            x(i+kshift3(1),j+kshift3(2),k+kshift3(3))-
c            x(i+kshift4(1),j+kshift4(2),k+kshift4(3))
c
c   The shift arrays are defined as follows:  (use cyclic permutation)
c
c   ishift(alpha) = -1
c   jshift1(alpha+1) =  1,  jshift1(alpha) = -1
c   jshift2(alpha+1) = -1,  jshift2(alpha) = -1
c   jshift3(alpha+1) =  1
c   jshift4(alpha+1) = -1
c   kshift1(alpha+2) =  1,  kshift1(alpha) = -1
c   kshift2(alpha+2) = -1,  kshift2(alpha) = -1
c   kshift3(alpha+2) =  1
c   kshift4(alpha+2) = -1
c
c   With respect to the half cells we use one-sided differences in order to
c   prevent points outside the region
c   In case alpha = 1, it concerns the faces 3, 4, 5 and 6 (Rest cyclic)
c
c   With respect to face 1 all points with
c   jshift(1) = -1 or kshift(1) = -1
c   must be replaced by shift zero
c   face 2:  xshift(1) = 1
c   face 3:  xshift(2) =-1
c   face 4:  xshift(2) = 1
c   face 5:  xshift(3) =-1
c   face 6:  xshift(3) = 1
c
c   The corresponding entries in gmat must be adapted
c   jshift1 corresponds to gmat(5) and must be transported to gmat(3)
c   jshift2 corresponds to gmat(1) and must be transported to gmat(3)
c   jshift3 corresponds to gmat(6) and must be transported to gmat(4)
c   jshift4 corresponds to gmat(2) and must be transported to gmat(4)
c   kshift1 corresponds to gmat(9) and must be transported to gmat(3)
c   kshift2 corresponds to gmat(7) and must be transported to gmat(3)
c   kshift3 corresponds to gmat(10) and must be transported to gmat(4)
c   kshift4 corresponds to gmat(8) and must be transported to gmat(4)
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data iperm/1,2,3,1,2/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Clear shift arrays

      do i = 1, 3
         ishift(i) = 0
         jshift1(i) = 0
         jshift2(i) = 0
         jshift3(i) = 0
         jshift4(i) = 0
         kshift1(i) = 0
         kshift2(i) = 0
         kshift3(i) = 0
         kshift4(i) = 0
      end do

c     --- Fill upper bounds of loops

      iarea(1,1) = 1
      iarea(2,1) = 1
      iarea(3,1) = 1
      iarea(1,2) = ni
      iarea(2,2) = nj
      iarea(3,2) = nk
      iarea(alphad,2) = iarea(alphad,2)+1

c     --- Set entries in gmat

      igmat1 = 1
      igmat2 = 2
      igmat3 = 3
      igmat4 = 4
      igmat5 = 5
      igmat6 = 6
      igmat7 = 7
      igmat8 = 8
      igmat9 = 9
      igmat10= 10

c     --- Set permutations

      ia1 = alphad
      ia2 = iperm(alphad+1)
      ia3 = iperm(alphad+2)

c     --- Fill shift arrays as functions of alphad

      ishift(ia1) = -1
      jshift1(ia2) =  1
      jshift1(ia1) = -1
      jshift2(ia2) = -1
      jshift2(ia1) = -1
      jshift3(ia2) =  1
      jshift4(ia2) = -1
      kshift1(ia3) =  1
      kshift1(ia1) = -1
      kshift2(ia3) = -1
      kshift2(ia1) = -1
      kshift3(ia3) =  1
      kshift4(ia3) = -1

c     --- Detect which entries point outside the region as function of the face
c         Compute corresponding gmat entries

      if ( iface.eq.1 ) then

c     --- Face 1, i = 1

         iarea(1,2) = 1
         tdir(1) = 2
         tdir(2) = 3
         if ( alphad.eq.2 ) then

c        --- alpha = 2; kshift2 and kshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift2(1) = 0
            kshift4(1) = 0
            igmat7 = 3
            igmat8 = 4

         else

c        --- alpha = 3; jshift2 and jshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift2(1) = 0
            jshift4(1) = 0
            igmat1 = 3
            igmat2 = 4

         end if

      else if ( iface.eq.2 ) then

c     --- Face 2, i = ni

         iarea(1,1) = ni
         tdir(1) = 2
         tdir(2) = 3
         if ( alphad.eq.2 ) then

c        --- alpha = 2; kshift1 and kshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift1(1) = 0
            kshift3(1) = 0
            igmat9 = 3
            igmat10= 4

         else

c        --- alpha = 3; jshift1 and jshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift1(1) = 0
            jshift3(1) = 0
            igmat5 = 3
            igmat6 = 4

         end if

      else if ( iface.eq.3 ) then

c     --- Face 3, j = 1

         iarea(2,2) = 1
         tdir(1) = 3
         tdir(2) = 1
         if ( alphad.eq.3 ) then

c        --- alpha = 3; kshift2 and kshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift2(2) = 0
            kshift4(2) = 0
            igmat7 = 3
            igmat8 = 4

         else

c        --- alpha = 1; jshift2 and jshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift2(2) = 0
            jshift4(2) = 0
            igmat1 = 3
            igmat2 = 4

         end if

      else if ( iface.eq.4 ) then

c     --- Face 4, j = nj

         iarea(2,1) = nj
         tdir(1) = 3
         tdir(2) = 1
         if ( alphad.eq.3 ) then

c        --- alpha = 3; kshift1 and kshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift1(2) = 0
            kshift3(2) = 0
            igmat9 = 3
            igmat10= 4

         else

c        --- alpha = 1; jshift1 and jshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift1(2) = 0
            jshift3(2) = 0
            igmat5 = 3
            igmat6 = 4

         end if

      else if ( iface.eq.5 ) then

c     --- Face 5, k = 1

         iarea(3,2) = 1
         tdir(1) = 1
         tdir(2) = 2
         if ( alphad.eq.1 ) then

c        --- alpha = 1; kshift2 and kshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift2(3) = 0
            kshift4(3) = 0
            igmat7 = 3
            igmat8 = 4

         else

c        --- alpha = 2; jshift2 and jshift4 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift2(3) = 0
            jshift4(3) = 0
            igmat1 = 3
            igmat2 = 4

         end if

      else

c     --- Face 6, k = nk

         iarea(3,1) = nk
         tdir(1) = 1
         tdir(2) = 2
         if ( alphad.eq.1 ) then

c        --- alpha = 1; kshift1 and kshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            kshift1(3) = 0
            kshift3(3) = 0
            igmat9 = 3
            igmat10= 4

         else

c        --- alpha = 2; jshift1 and jshift3 point outside the region,
c            Set the shifts back to zero and store corresponding gmat entries

            jshift1(3) = 0
            jshift3(3) = 0
            igmat5 = 3
            igmat6 = 4

         end if

      end if
      do i = iarea(1,1), iarea(1,2)
         do j = iarea(2,1), iarea(2,2)
            do k = iarea(3,1), iarea(3,2)

               mxp(1) = i
               mxp(2) = j
               mxp(3) = k
               fact = jac(i,j,k)
               btype = ibinfor(2*mxp(tdir(1)),2*mxp(tdir(2)))
               if ( btype.lt.90 ) then

c              --- Tangential boundary cell, without periodical or
c                  multiblock boundary conditions
c                  First clear array gmat

                  do l = 1, 10
                     gmat(i,j,k,alphad,l) = 0
                  end do

c                 --- Fill array C with the coordinate differences

                  do l = 1, 3
                     cmat(ia1,l) = xcentr(i,j,k,l)-
     +                   xcentr(i+ishift(1),j+ishift(2),k+ishift(3),l)
                     cmat(ia2,l) =
     +                 xcentr(i+jshift1(1),j+jshift1(2),k+jshift1(3),l)-
     +                 xcentr(i+jshift2(1),j+jshift2(2),k+jshift2(3),l)+
     +                 xcentr(i+jshift3(1),j+jshift3(2),k+jshift3(3),l)-
     +                 xcentr(i+jshift4(1),j+jshift4(2),k+jshift4(3),l)
                     cmat(ia3,l) =
     +                 xcentr(i+kshift1(1),j+kshift1(2),k+kshift1(3),l)-
     +                 xcentr(i+kshift2(1),j+kshift2(2),k+kshift2(3),l)+
     +                 xcentr(i+kshift3(1),j+kshift3(2),k+kshift3(3),l)-
     +                 xcentr(i+kshift4(1),j+kshift4(2),k+kshift4(3),l)
                  end do
                  det =
     +               cmat(1,1)*(cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +              +cmat(1,2)*(cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +              +cmat(1,3)*(cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1))

                  fact = jac(i,j,k)/det

c                 --- Compute a^1 . c^k

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(ia2,2)*cmat(ia3,3)-
     +                    cmat(ia2,3)*cmat(ia3,2))+
     +                    adir(i,j,k,2)*
     +                   (cmat(ia2,3)*cmat(ia3,1)-
     +                    cmat(ia2,1)*cmat(ia3,3))+
     +                    adir(i,j,k,3)*
     +                   (cmat(ia2,1)*cmat(ia3,2)-
     +                    cmat(ia2,2)*cmat(ia3,1)))
                  gmat(i,j,k,alphad,igmat3) = -facs
                  gmat(i,j,k,alphad,igmat4) =  facs

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(ia3,2)*cmat(ia1,3)-
     +                    cmat(ia3,3)*cmat(ia1,2))+
     +                    adir(i,j,k,2)*
     +                   (cmat(ia3,3)*cmat(ia1,1)-
     +                    cmat(ia3,1)*cmat(ia1,3))+
     +                    adir(i,j,k,3)*
     +                   (cmat(ia3,1)*cmat(ia1,2)-
     +                    cmat(ia3,2)*cmat(ia1,1)))
                  gmat(i,j,k,alphad,igmat1) = gmat(i,j,k,alphad,igmat1)-
     +                                        facs
                  gmat(i,j,k,alphad,igmat2) = gmat(i,j,k,alphad,igmat2)-
     +                                        facs
                  gmat(i,j,k,alphad,igmat5) = gmat(i,j,k,alphad,igmat5)+
     +                                        facs
                  gmat(i,j,k,alphad,igmat6) = gmat(i,j,k,alphad,igmat6)+
     +                                        facs

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(ia1,2)*cmat(ia2,3)-
     +                    cmat(ia1,3)*cmat(ia2,2))+
     +                    adir(i,j,k,2)*
     +                   (cmat(ia1,3)*cmat(ia2,1)-
     +                    cmat(ia1,1)*cmat(ia2,3))+
     +                    adir(i,j,k,3)*
     +                   (cmat(ia1,1)*cmat(ia2,2)-
     +                    cmat(ia1,2)*cmat(ia2,1)))
                  gmat(i,j,k,alphad,igmat7) = gmat(i,j,k,alphad,igmat7)-
     +                                        facs
                  gmat(i,j,k,alphad,igmat8) = gmat(i,j,k,alphad,igmat8)-
     +                                        facs
                  gmat(i,j,k,alphad,igmat9) = gmat(i,j,k,alphad,igmat9)+
     +                                        facs
                  gmat(i,j,k,alphad,igmat10)=gmat(i,j,k,alphad,igmat10)+
     +                                        facs

               end if

            end do
         end do
      end do

      end
