      subroutine isgmwb12 ( ni, nj, nvirtual, gmat, ibipoint, ibinfor )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 1.0    date 06-11-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Correct the gradient matrix for boundary cells in 2D
c    Wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*)
      double precision gmat(ni+1,nj+1,2,6)

c     gmat     i/o   Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter ( snnsnt = 7, snnut = 6 )
      integer ip, jp, iparea(2,2), k, mxp(2), tdir, ndir,
     +        iside, ipibinf,  btype
      logical skip

c     btype     Type of boundary condition
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     jp        Local counting variable j-direction
c     k         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     skip      If true boundary condition must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            tdir = 2
            ndir = 1
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            tdir = 1
            ndir = 2

            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if

c        --- First the normal cells are treated

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

               mxp(1) = ip
               mxp(2) = jp

c              --- Get integration cell position for ibinfor

               btype = ibinfor(ipibinf+2*mxp(tdir))

c              --- clear row in matrix if not periodical or multiblock

               skip = btype.ge.88 .and. btype.le.100
               if ( btype.ne.snnsnt .and. btype.ne.snnut
     +              .and. btype.gt.0 .and. .not. skip ) then
                  do k = 1, 6
                     gmat(ip,jp,ndir,k) = 0d0
                  end do
               end if

               if ( btype.eq.0 ) then

c              --- No boundary condition given, fold inwards

                  if ( mod(iside,2).eq.1 ) then

c                 --- sides 1 and 3

                     do k = 1, 5, 2
                        gmat(ip,jp,ndir,k+1) = gmat(ip,jp,ndir,k+1) +
     +                     gmat(ip,jp,ndir,k)
                        gmat(ip,jp,ndir,k) = 0d0
                     end do

                  else

c                 --- sides 2 and 4

                     do k = 2, 6, 2
                        gmat(ip,jp,ndir,k-1) = gmat(ip,jp,ndir,k-1) +
     +                     gmat(ip,jp,ndir,k)
                        gmat(ip,jp,ndir,k) = 0d0
                     end do

                  end if

               end if
            end do
         end do

      end do
      end
