      subroutine isintcfa ( coef  , coefint, ni, nj, nvirtual, vartyp,
     +                      acttyp, pnttyp, mshtyp, coor )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 02-02-1999 copy of isintcf2 for
c                                       axisymmetric case
c
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   See for description subroutine isintcf2, in some cases now in
c   this routine the interpolated value is multiplied by r for the
c   axisymetric case
c **********************************************************************
c
c                       KEYWORDS
c
c    axisymmetric
c    coefficients
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, vartyp, acttyp, pnttyp, mshtyp
      double precision coef(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual)
      double precision coefint(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2)

c     acttyp    i    Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     coef      i    A specific coefficient
c     coefint   o    Interpolated coefficient
c     coor      i    reserved
c     mshtyp    i    Type of grid arrangement
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j
      double precision dfcijm, dfcijp, dfaccen

c     dfaccen   Interpolation coefficient
c     dfcijm    Distance between the centroid(i-1,j) and the face center(i,j)
c                        for calculation in the i-direction,
c                        between the centroid(i,j-1) and the face center(i,j)
c                        for calculation in the j-direction
c     dfcijp    Distance between the centroid(i,j) and the face center(i,j)
c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( vartyp.eq.3 ) then

c     --- Interpolate values given in the centroid

         if ( pnttyp.eq.2 ) then

c        --- Interpolation or finite differences to V2-point

           if ( acttyp.eq.0 ) then
             if ( mshtyp.eq.0 ) then
                do j = 2-nvirtual, nj+nvirtual
                  do i = 2-nvirtual, ni+nvirtual
                     coefint(i,j) = 0.5d0*(coef(i,j)+coef(i,j-1)) *
     +                               0.5d0*(coor(i,j,1)+coor(i+1,j,1))
                  end do
                end do
             else if ( mshtyp.eq.1 ) then
                do j = 2-nvirtual, nj+nvirtual
                  do i = 2-nvirtual, ni+nvirtual
                    dfcijm = 25d-2*
     +                     sqrt((coor(i,j,1)-coor(i,j-1,1)+
     +                           coor(i+1,j,1)-coor(i+1,j-1,1))**2+
     +                          (coor(i,j,2)-coor(i,j-1,2)+
     +                           coor(i+1,j,2)-coor(i+1,j-1,2))**2)
                    dfcijp = 25d-2*
     +                     sqrt((coor(i,j+1,1)-coor(i,j,1)+
     +                           coor(i+1,j+1,1)-coor(i+1,j,1))**2+
     +                          (coor(i,j+1,2)-coor(i,j,2)+
     +                           coor(i+1,j+1,2)-coor(i+1,j,2))**2)
                    dfaccen = dfcijm/(dfcijp+dfcijm)
                    coefint(i,j) = dfaccen*coef(i,j)+
     +                       (1d0-dfaccen)*coef(i,j-1)
                  end do
                end do
             end if
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i+1,j)+coef(i+1,j-1)-
     +                                 coef(i-1,j)-coef(i-1,j-1))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i,j)-coef(i,j-1)
               end do
             end do
           end if

         else if ( pnttyp.eq.3 ) then

c        --- Interpolation or finite differences to V1-point

           if ( acttyp.eq.0 ) then
             if ( mshtyp.eq.0 ) then
                do j = 2-nvirtual, nj+nvirtual
                  do i = 2-nvirtual, ni+nvirtual
                     coefint(i,j) = 0.5d0*(coef(i,j)+coef(i-1,j)) *
     +                               0.5d0*(coor(i,j,1)+coor(i,j+1,1))
                  end do
                end do
             else if ( mshtyp.eq.1 ) then
               do j = 2-nvirtual, nj+nvirtual
                 do i = 2-nvirtual, ni+nvirtual
                   dfcijm = 25d-2*
     +                    sqrt((coor(i,j,1)-coor(i-1,j,1)+
     +                          coor(i,j+1,1)-coor(i-1,j+1,1))**2+
     +                         (coor(i,j,2)-coor(i-1,j,2)+
     +                          coor(i,j+1,2)-coor(i-1,j+1,2))**2)
                   dfcijp = 25d-2*
     +                    sqrt((coor(i+1,j,1)-coor(i,j,1)+
     +                          coor(i+1,j+1,1)-coor(i,j+1,1))**2+
     +                         (coor(i+1,j,2)-coor(i,j,2)+
     +                          coor(i+1,j+1,2)-coor(i,j+1,2))**2)
                   dfaccen = dfcijm/(dfcijp+dfcijm)
                   coefint(i,j) = dfaccen*coef(i,j)+
     +                      (1d0-dfaccen)*coef(i-1,j)
                 end do
               end do
             end if
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i,j)-coef(i-1,j)
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i,j+1)+coef(i-1,j+1)-
     +                                 coef(i,j-1)-coef(i-1,j-1))
               end do
             end do
           end if

         else if ( pnttyp.eq.4 ) then

c        --- No interpolation required, just copying
c            Finite differences to centroid

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = coef(i,j) *
     +                            25d-2 * (coor(i,j,1)+coor(i+1,j,1)+
     +                                   coor(i,j+1,1)+coor(i+1,j+1,1))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i+1,j)-coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j+1)-coef(i,j-1))
               end do
             end do
           end if

         else if ( pnttyp.eq.1 ) then

c        --- Interpolation or finite differences to vertex

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.25d0*(coef(i,j)+coef(i-1,j)+
     +                                   coef(i,j-1)+coef(i-1,j-1)) *
     +                           coor(i,j,1)

               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j)+coef(i,j-1)-
     +                                coef(i-1,j)-coef(i-1,j-1))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j)+coef(i-1,j)-
     +                                coef(i,j-1)-coef(i-1,j-1))
               end do
             end do
           end if

         end if

      else if ( vartyp.eq.1 ) then

c     --- Interpolate values given in V1-point

         if ( pnttyp.eq.4 ) then

c        --- Interpolation or finite differences to centroid

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.5d0*(coef(i,j)+coef(i+1,j))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i+1,j)-coef(i,j)
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i+1,j+1)+coef(i,j+1)-
     +                                 coef(i+1,j-1)-coef(i,j-1))
               end do
             end do
           end if

         else if ( pnttyp.eq.1 ) then

c        --- Interpolation or finite differences to vertex

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.5d0*(coef(i,j)+coef(i,j-1))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i+1,j)+coef(i+1,j-1)-
     +                                 coef(i-1,j)-coef(i-1,j-1))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i,j)-coef(i,j-1)
               end do
             end do
           end if

         else if ( pnttyp.eq.3 ) then

c        --- No interpolation required, just copying
c            Finite differences to V1-point

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = coef(i,j)
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i+1,j)-coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j+1)-coef(i,j-1))
               end do
             end do
           end if

         else

c        --- Interpolation or central differences to V2-point

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.25d0*(coef(i,j)+coef(i+1,j)+
     +                                   coef(i,j-1)+coef(i+1,j-1))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j) = 5d-1*(coef(i+1,j)-coef(i,j)+
     +                                 coef(i+1,j-1)-coef(i,j-1))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j) = 5d-1*(coef(i+1,j)+coef(i,j)-
     +                                 coef(i+1,j-1)-coef(i,j-1))
               end do
             end do
           end if

         end if

      else if ( vartyp.eq.2 ) then

c     --- Interpolate values given in V2-point

         if ( pnttyp.eq.4 ) then

c        --- Interpolation or finite differences to centroid

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.5d0*(coef(i,j)+coef(i,j+1))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i+1,j+1)+coef(i+1,j)-
     +                                 coef(i-1,j+1)-coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i,j+1)-coef(i,j)
               end do
             end do
           end if

         else if ( pnttyp.eq.1 ) then

c        --- Interpolation or central differences to vertex

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.5d0*(coef(i,j)+coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = coef(i,j)-coef(i-1,j)
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 25d-2*(coef(i,j+1)+coef(i-1,j+1)-
     +                                 coef(i,j-1)-coef(i-1,j-1))
               end do
             end do
           end if

         else if ( pnttyp.eq.2 ) then

c        --- No interpolation required, just copying
c            Finite differences to V2-point

           if (acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = coef(i,j)
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i+1,j)-coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j+1)-coef(i,j-1))
               end do
             end do
           end if

         else

c        --- Interpolation to V1-point

           if ( acttyp.eq.0 ) then
             do j = 2-nvirtual, nj+nvirtual
               do i = 2-nvirtual, ni+nvirtual
                  coefint(i,j) = 0.25d0*(coef(i,j)+coef(i-1,j)+
     +                                   coef(i,j+1)+coef(i-1,j+1))
               end do
             end do
           else if ( acttyp.eq.1 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j+1)+coef(i,j)-
     +                                coef(i-1,j+1)-coef(i-1,j))
               end do
             end do
           else if ( acttyp.eq.2 ) then
             do j = 2-nvirtual, nj+nvirtual-1
               do i = 2-nvirtual, ni+nvirtual-1
                 coefint(i,j) = 5d-1*(coef(i,j+1)+coef(i-1,j+1)-
     +                                coef(i,j)-coef(i-1,j))
               end do
             end do
           end if

         end if

      end if

      end
