      subroutine ismomf32 ( ktdir, ndir, ni, nj, nk, iparea, ilow1,
     +                      ihigh1, ilow2, ihigh2, mct, matrix,
     +                      rhside, ibinfor, bcgeo, bndcon, nsign,
     +                      tdir )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 28-11-1995 Kees K. bug in tdirsh
c        version 1.1    date 28-11-1995 Kees K. points outside
c        version 1.0    date 29-06-1995 subroutine based on ideas of ismmbf
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c    Folding part for 3d case.
c    Normal unknowns of tangential cell are folded
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ilow1, ihigh1,ilow2, ihigh2, ktdir, tdir(2),
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), nsign,
     +        mct(3,3,-2:2,-2:2,-2:2), iparea(2,3), ni, nj, nk
      double precision bcgeo(21,ilow1:ihigh1,ilow2:ihigh2),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 matrix(ni+1,nj+1,nk+1,3,*),
     +                 bndcon(3,ilow1:ihigh1,ilow2:ihigh2)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     ktdir     i    Sequence number of tangential direction (1 or 2)
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ndir      i    Direction of normal vector on this side
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     rhside   i/o   reserved
c     tdir      i    Direction of tangent vector on this side
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer unut, periodic, antiper, multiblk,
     +        uxuy, unsnt, perjump, antijump,
     +        oldturb, parabolic, masspar, outside
      parameter ( masspar = 100 )
      parameter ( outside = 101 )
      parameter( unut = 4, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           oldturb = 9, parabolic= 12)

      logical skip, ungiven
      integer btype, alphad,
     +        iconcto, iconctc, iconcti, k,
     +        ip, jp, xp(3), ndir, betad, istwo(3,3),
     +        inttab(2), tdirsh(3), dx(3),
     +        notktdir, kp, ict1, ict2
      double precision fact, un

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     fact      Multiplication factor
c     iconctc   Connection in matrix for central point
c     iconcti   Connection in matrix for internal point
c     iconcto   Connection in matrix for outside point
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     inttab    Table with integration points relative to cell centre
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     k         General loop variable
c     kp        Local counting variable k-direction
c     masspar   Boundary condition for massive parallel computations
c     multiblk  Multi-block boundary conditions
c     notktdir  Tangential direction orthogonal to ktdir
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     outside   Type of boundary condition for points outside the ibinfor
c               array
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     skip      If true boundary condition must be skipped
c     tdirsh    Shift of integration point relative to cell centre in
c               tangential directions
c     un        Normal velocity (physical)
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    METHOD
c
c    The folding process will be carried out per face.
c    Some rows corresponding to unknowns in the neighbourhood of the edges
c    and vertices require special attention
c
c    With respect to the boundary conditions the following combinations
c    are important:
c
c      n           (n)
c     V   = sgn(n.a   ) sqrtg sqrt(g^nn) u.n                       (1)
c               - -                      - -
c
c     If u^t is given:
c
c         we assume that both tangential velocities are zero,
c
c         u.t  = u.t  = 0
c         - -1   - -2
c
c         Hence, we have
c
c          t1      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det             (2)
c                        2 2     1      1 2     2
c
c          t2      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det
c                        1 1     2      1 2     1
c
c                                            2
c         with det = g_t t  g_t t  - (g_t t )
c                       1 1    2 2       1 2
c
c    u_n and u_t given
c    u_n given, u_t not
c    u_t given, u_n not
c    u_t nor u_n given
c    multiblock
c    pericodical boundary conditions
c
c    For illustration purposes we limit ourselves to the lower face and
c    in case of an edge to the combination of lower face and left-hand side face
c    The vertex is illustrated at the corner (0,0,0)
c    With respect to the tangential cells we limit ourselves to the cells
c    i=2 (1) ni, j=2(1)nj hence all tangential velocities not at other
c    boundaries
c
c    At the lower boundary (face 5, ndir=3, nsig=-1), we have two tangential
c    cells: a V1 (tdir=1) and a V2 (tdir=2) cell.
c    The only unknowns to be eliminated are those that are outside the
c    region. Unknowns at the boundary are kept.
c    For those cells the following unknowns are outside the region:
c
c                                V1            V2
c    normal unknowns (V3):    1,0,-3         0,1,-3
c                            -1,0,-3        0,-1,-3
c
c    tangential unknown V1:   0,-2,-2      -1,-1,-2
c                             0, 0,-2       1,-1,-2
c                             0, 2,-2      -1, 1,-2
c                            -2, 0,-2       1, 1,-2
c                             2, 0,-2
c    tangential unknown V2:  -1,-1,-2      -2, 0,-2
c                             1,-1,-2       0, 0,-2
c                            -1, 1,-2       2, 0,-2
c                             1, 1,-2       0,-2,-2
c                                           0, 2,-2
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n:
c            U^n(.,.,-3) = 2U^n(.,.,-1) - U^n(.,.,1)                   (3)
c    c. U^n and U^t given at the boundary (.,-1):
c            U^t(.,.,-2) = 2U^t(.,.,-1) - U^t(.,.,0)                   (4)
c    d. U^t not given:
c            U^t(.,.,-2) = 2U^t(.,.,0) - U^t(.,.,2)                    (5)
c    e. U^t given and U^n not:
c            At the point (0,0) use a combination of (2) and (4)
c            At the other points use (5)
c
c
c    Remark: in order to compute the matrix entry in a row it is necessary
c            to use array mct
c            Infortunatley mct does not use the shifts as indicated by
c            our local tables.
c            Let shift be the vector of local shifts with respect to
c            the cell centre
c            Let dx be the shift vector with respect to mct
c            Suppose we are in an alphad cell and are concerned with a betad
c            connection.
c            Then the relation between dx and shift is given by:
c
c            dx = (shift - e_alphad + e_betad) /2
c **********************************************************************
c
c                    DATA STATEMENTS
c
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /
      data inttab / -1, 1 /
c ======================================================================
c
      if ( ktdir.eq.1 ) then
         notktdir = 2
      else
         notktdir = 1
      end if
      do k = 1, 2

         alphad = tdir(ktdir)
         betad  = tdir(notktdir)
         tdirsh(alphad) = inttab(k)
         tdirsh(betad) = 0

c        --- dx(ndir) = 2*nsign

         dx(ndir) = nsign + ( nsign+1)/2
         dx(betad) = 0
         dx(alphad) = (inttab(k)-1)/2
         iconcto = mct(alphad,ndir,dx(1),dx(2),dx(3))
         dx(ndir) = dx(ndir) - nsign
         iconctc = mct(alphad,ndir,dx(1),dx(2),dx(3))
         dx(ndir) = dx(ndir) - nsign
         iconcti = mct(alphad,ndir,dx(1),dx(2),dx(3))

c        --- loop over cells to find points outside the
c            region for which boundary conditions are given

         do kp = iparea(1,3), iparea(2,3)
            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)

                  xp(1) = ip
                  xp(2) = jp
                  xp(3) = kp
                     if ( alphad.eq.tdir(1) ) then
                        ict1 = 2*xp(tdir(1))-1+tdirsh(tdir(1))
                        ict2 = 2*xp(tdir(2))+tdirsh(tdir(2))
                     else
                        ict1 = 2*xp(tdir(1))+tdirsh(tdir(1))
                        ict2 = 2*xp(tdir(2))-1+tdirsh(tdir(2))
                     end if
                  btype = ibinfor(ict1,ict2)
                  if ( btype.eq.outside ) then
                     fact = matrix(ip,jp,kp,alphad,iconcto)
                     matrix(ip,jp,kp,alphad,iconcto) = 0d0
                     matrix(ip,jp,kp,alphad,iconctc) =
     +                  matrix(ip,jp,kp,alphad,iconctc)+2d0*fact
                     matrix(ip,jp,kp,alphad,iconcti) =
     +                  matrix(ip,jp,kp,alphad,iconcti)-fact
                  end if

               end do
            end do
         end do

c        --- loop over cells

         do kp = iparea(1,3), iparea(2,3)
            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)

                  xp(1) = ip
                  xp(2) = jp
                  xp(3) = kp
                     if ( alphad.eq.tdir(1) ) then
                        ict1 = 2*xp(tdir(1))-1+tdirsh(tdir(1))
                        ict2 = 2*xp(tdir(2))+tdirsh(tdir(2))
                     else
                        ict1 = 2*xp(tdir(1))+tdirsh(tdir(1))
                        ict2 = 2*xp(tdir(2))-1+tdirsh(tdir(2))
                     end if
                  btype = ibinfor(ict1,ict2)

                  skip = btype.eq.multiblk .or. btype.eq.periodic.or.
     +                   btype.eq.antiper .or. btype.eq.perjump .or.
     +                   btype.eq.antijump .or.btype.eq.masspar
                  if ( .not. skip ) then
                     if ( btype.eq.parabolic ) then
                        btype = unut
                     else if ( btype.eq.oldturb ) then
                        btype = unsnt
                     end if
                     ungiven = btype.eq.parabolic .or. btype.eq.oldturb
     +                         .or. btype.eq.unut .or. btype.eq.uxuy
     +                         .or. btype.eq.unsnt
                     fact = matrix(ip,jp,kp,alphad,iconcto)
                     matrix(ip,jp,kp,alphad,iconcto) = 0d0
                     if ( .not. ungiven ) then

c                    --- U_n has not been given use formula (3)

                        matrix(ip,jp,kp,alphad,iconctc) =
     +                     matrix(ip,jp,kp,alphad,iconctc)+2d0*fact
                        matrix(ip,jp,kp,alphad,iconcti) =
     +                     matrix(ip,jp,kp,alphad,iconcti)-fact

                     else

c                    --- un has been given

                        if ( btype.eq.uxuy ) then

c                       --- ux and uy given

                           un = bcgeo(10+3*ndir,ict1,ict2)*
     +                          bndcon(1,ict1,ict2)+
     +                          bcgeo(11+3*ndir,ict1,ict2)*
     +                          bndcon(2,ict1,ict2)+
     +                          bcgeo(12+3*ndir,ict1,ict2)*
     +                          bndcon(3,ict1,ict2)

                        else

c                       --- un given

                           un = bndcon(1,ict1,ict2)*nsign*
     +                          bcgeo(istwo(ndir,ndir),ict1,ict2)

                        end if

                        matrix(ip,jp,kp,alphad,iconctc) =
     +                     matrix(ip,jp,kp,alphad,iconctc)-fact
                        rhside(ip,jp,kp,alphad) =
     +                     rhside(ip,jp,kp,alphad) - 2d0*fact*un

                     end if

                  end if

               end do

            end do

         end do

      end do

      end
