      subroutine ismomf33 ( ndir, ni, nj, nk, iparea, ilow1,
     +                      ihigh1, ilow2, ihigh2, mct, matrix,
     +                      ibinfor, bcgeo,  nsign, tdir )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 18-08-1995 subroutine based on ideas of ismmbf
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c    Folding part for 3d case.
c    Tangential unknowns of normal cells are folded
c
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ilow1, ihigh1,ilow2, ihigh2, tdir(2),
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), nsign,
     +        mct(3,3,-2:2,-2:2,-2:2), iparea(2,3), ni, nj, nk
      double precision bcgeo(21,ilow1:ihigh1,ilow2:ihigh2),
     +                 matrix(ni+1,nj+1,nk+1,3,*)


c     bcgeo     i    reserved
c     ibinfor   i    Third part of infbnd
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ndir      i    Direction of normal vector on this side
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     tdir      i    Direction of tangent vector on this side
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer unut, periodic, antiper, multiblk,
     +        uxuy, unsnt, snnut, perjump, antijump,
     +        oldturb, parabolic, masspar
      parameter ( masspar = 100 )
      parameter( unut = 4, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5, snnut = 6,
     +           oldturb = 9, parabolic= 12)

      logical skip, utgiven, ungiven
      integer btype, alphad, gammad,
     +        iconcto,  iconcti, iconctr, iconctl, j, k,
     +        ip, jp, xp(3), ndir, betad, istwo(3,3),
     +        inttab(2,8),  dx(3),
     +        kp, ict1, ict2, ltdir, iconct1, iconct2
      double precision fact, det, gvt1

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     det       Determinant
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     fact      Multiplication factor
c     gammad    Direction index
c     gvt1      Factor preceding the V^(t1) component multiplied by determinant
c     iconct1   Connection in matrix for first interpolation point
c     iconct2   Connection in matrix for second interpolation point
c     iconcti   Connection in matrix for internal point
c     iconctl   Connection in matrix for "left" point
c     iconcto   Connection in matrix for outside point
c     iconctr   Connection in matrix for "right" point
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     inttab    Table with integration points relative to cell centre
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     k         General loop variable
c     kp        Local counting variable k-direction
c     ltdir     Local tangential direction
c     masspar   Boundary condition for massive parallel computations
c     multiblk  Multi-block boundary conditions
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     skip      If true boundary condition must be skipped
c     snnut     Sigma_nn and u_t prescribed
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     utgiven   If true U_tis  given in point
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    METHOD
c
c    The folding process will be carried out per face.
c    Some rows corresponding to unknowns in the neighbourhood of the edges
c    and vertices require special attention
c
c    With respect to the boundary conditions the following combinations
c    are important:
c
c      n           (n)
c     V   = sgn(n.a   ) sqrtg sqrt(g^nn) u.n                       (1)
c               - -                      - -
c
c     If u^t is given:
c
c         we assume that both tangential velocities are zero,
c
c         u.t  = u.t  = 0
c         - -1   - -2
c
c         Hence, we have
c
c          t1      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det             (2)
c                        2 2     1      1 2     2
c
c          t2      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det
c                        1 1     2      1 2     1
c
c                                            2
c         with det = g_t t  g_t t  - (g_t t )
c                       1 1    2 2       1 2
c
c    u_n and u_t given
c    u_n given, u_t not
c    u_t given, u_n not
c    u_t nor u_n given
c    multiblock
c    pericodical boundary conditions
c
c    For illustration purposes we limit ourselves to the lower face and
c    in case of an edge to the combination of lower face and left-hand side face
c    The vertex is illustrated at the corner (0,0,0)
c    With respect to the tangential cells we limit ourselves to the cells
c    i=2 (1) ni, j=2(1)nj hence all tangential velocities not at other
c    boundaries
c
c    At the lower boundary (face 5, ndir=3, nsig=-1), we have two tangential
c    cells: a V1 (tdir=1) and a V2 (tdir=2) cell.
c    The only unknowns to be eliminated are those that are outside the
c    region. Unknowns at the boundary are kept.
c    For those cells the following unknowns are outside the region:
c
c                                V1            V2
c    normal unknowns (V3):    1,0,-3         0,1,-3
c                            -1,0,-3        0,-1,-3
c
c    tangential unknown V1:   0,-2,-2      -1,-1,-2
c                             0, 0,-2       1,-1,-2
c                             0, 2,-2      -1, 1,-2
c                            -2, 0,-2       1, 1,-2
c                             2, 0,-2
c    tangential unknown V2:  -1,-1,-2      -2, 0,-2
c                             1,-1,-2       0, 0,-2
c                            -1, 1,-2       2, 0,-2
c                             1, 1,-2       0,-2,-2
c                                           0, 2,-2
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n:
c            U^n(.,.,-3) = 2U^n(.,.,-1) - U^n(.,.,1)                   (3)
c    c. U^n and U^t given at the boundary (.,-1):
c            U^t(.,.,-2) = 2U^t(.,.,-1) - U^t(.,.,0)                   (4)
c    d. U^t not given:
c            U^t(.,.,-2) = 2U^t(.,.,0) - U^t(.,.,2)                    (5)
c    e. U^t given and U^n not:
c            At the point (0,0) use a combination of (2) and (4)
c            At the other points use (5)
c
c
c    At the lower boundary (face 5, ndir=3, nsig=-1), we have one normal
c    cell: a V3 (ndir=3) .
c    The only unknowns to be eliminated are those that are outside the
c    region. Unknowns at the boundary are kept.
c    For those cells the following unknowns are outside the region:
c
c                                 V3
c    5 normal unknowns  V3 :    0,-2,-2
c                               0, 0,-2
c                               0, 2,-2
c                              -2, 0,-2
c                               2, 0,-2
c
c    8 tangential unknowns V1: -3, 0,-1
c                               3, 0,-1
c                              -1, 0,-1
c                               1, 0,-1
c                              -1,-2,-1
c                               1,-2,-1
c                              -1, 2,-1
c                               1, 2,-1
c
c    8 tangential unknowns V2:  0,-3,-1
c                               0, 3,-1
c                               0,-1,-1
c                               0, 1,-1
c                              -2,-1,-1
c                              -2, 1,-1
c                               2,-1,-1
c                               2, 1,-1
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n:
c            U^n given: diagonal element 1, off-diagonal elements 0
c                       Value in right-hand side.
c            In all other situations we do:
c            U^n(.,.,-2) = 2U^n(.,.,0) - U^n(.,.,2)
c    d. U^t not given:
c            U^t(.,.,-1) = U^t(.,.,1)
c    e. U^t given:
c            use a combination of (2) and (4)
c            except for the points (-3,.,.),(3,.,.) and (.,-3,.),(.,3,.)
c            which must be folded first to other points outside
c            the region.
c
c
c    Remark: in order to compute the matrix entry in a row it is necessary
c            to use array mct
c            Infortunatley mct does not use the shifts as indicated by
c            our local tables.
c            Let shift be the vector of local shifts with respect to
c            the cell centre
c            Let dx be the shift vector with respect to mct
c            Suppose we are in an alphad cell and are concerned with a betad
c            connection.
c            Then the relation between dx and shift is given by:
c
c            dx = (shift - e_alphad + e_betad) /2
c **********************************************************************
c
c                    DATA STATEMENTS
c
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /
      data inttab / -3,  0,
     +               3,  0,
     +              -1,  0,
     +               1,  0,
     +              -1, -2,
     +               1, -2,
     +              -1,  2,
     +               1,  2 /
c ======================================================================
c
c     --- loop over two tangential directions

      do j = 1, 2

         ltdir = tdir(j)
         alphad = ndir
         betad  = ltdir
         if ( j.eq.1) then
            gammad = tdir(2)
         else
            gammad = tdir(1)
         end if

         if ( nsign.eq.-1 ) then
            dx(ndir) = nsign
         else
            dx(ndir) = 0
         end if

c        --- loop over eight points

         do k = 1, 8

            if ( tdir(1).eq.ltdir ) then
               dx(tdir(1)) = (inttab(1,k)+1)/2
               dx(tdir(2)) = inttab(2,k)/2
            else
               dx(tdir(1)) = inttab(2,k)/2
               dx(tdir(2)) = (inttab(1,k)+1)/2
            end if

            iconcto = mct(alphad,betad,dx(1),dx(2),dx(3))
            if ( nsign.eq.-1 ) then
               dx(ndir) = 0
            else
               dx(ndir) = -1
            end if
            iconcti = mct(alphad,betad,dx(1),dx(2),dx(3))

c           --- for the first two points there is no iconctl, iconctr

            if( k.lt.3 ) then
               if ( nsign.eq.-1 ) then
                  dx(ndir) = nsign
               else
                  dx(ndir) = 0
               end if
               if ( tdir(1).eq.ltdir ) then
                  if ( inttab(1,k).gt.0 ) then
                     dx(tdir(1)) = dx(tdir(1)) -1
                     iconct1 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                     dx(tdir(1)) = dx(tdir(1)) -1
                     iconct2 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                  else
                     dx(tdir(1)) = dx(tdir(1)) +1
                     iconct1 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                     dx(tdir(1)) = dx(tdir(1)) +1
                     iconct2 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                  end if
               else
                  if ( inttab(1,k).gt.0 ) then
                     dx(tdir(2)) = dx(tdir(2)) -1
                     iconct1 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                     dx(tdir(2)) = dx(tdir(2)) -1
                     iconct2 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                  else
                     dx(tdir(2)) = dx(tdir(2)) +1
                     iconct1 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                     dx(tdir(2)) = dx(tdir(2)) +1
                     iconct2 =  mct(alphad,alphad,dx(1),dx(2),dx(3))
                  end if
               end if
            else
               dx(ndir) = 0
               if ( tdir(1).eq.ltdir ) then
                  if( dx(tdir(1)).ne.0 ) then
                     dx(tdir(1)) = -1
                  else
                     dx(tdir(1)) =  1
                  end if
                  iconctl = mct(alphad,alphad,dx(1),dx(2),dx(3))
                  dx(tdir(1)) = 0
                  iconctr = mct(alphad,alphad,dx(1),dx(2),dx(3))
               else
                  if( dx(tdir(2)).ne.0 ) then
                     dx(tdir(2)) = -1
                  else
                     dx(tdir(2)) =  1
                  end if
                  iconctl = mct(alphad,alphad,dx(1),dx(2),dx(3))
                  dx(tdir(2)) = 0
                  iconctr = mct(alphad,alphad,dx(1),dx(2),dx(3))
               end if
            end if

c           --- loop over cells

            do kp = iparea(1,3), iparea(2,3)
               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     xp(1) = ip
                     xp(2) = jp
                     xp(3) = kp
                     ict1 = 2*xp(tdir(1))
                     ict2 = 2*xp(tdir(2))
                     btype = ibinfor(ict1,ict2)

                     ungiven = btype.eq.parabolic .or. btype.eq.oldturb
     +                        .or. btype.eq.unut .or. btype.eq.uxuy .or.
     +                         btype.eq.unsnt

                     if ( tdir(1).eq.ltdir ) then
                        ict1 = 2*xp(tdir(1))+inttab(1,k)
                        ict2 = 2*xp(tdir(2))+inttab(2,k)
                     else
                        ict1 = 2*xp(tdir(1))+inttab(2,k)
                        ict2 = 2*xp(tdir(2))+inttab(1,k)
                     end if
                     btype = ibinfor(ict1,ict2)

                     if ( .not. ungiven ) then

                        skip = btype.eq.multiblk .or. btype.eq.
     +                  periodic.or. btype.eq.antiper .or. btype.eq.
     +                  perjump .or. btype.eq.antijump .or.btype.eq.
     +                  masspar
                        if ( .not. skip ) then
                           if ( btype.eq.parabolic ) then
                              btype = unut
                           else if ( btype.eq.oldturb ) then
                              btype = unsnt
                           end if
                           utgiven = btype.eq.snnut .or. btype.eq.unut
     +                               .or. btype.eq.uxuy
                           fact = matrix(ip,jp,kp,alphad,iconcto)
                           matrix(ip,jp,kp,alphad,iconcto) = 0d0

                           if ( .not. utgiven ) then

c                          --- U_t has not been given

                              matrix(ip,jp,kp,alphad,iconcti) =
     +                        matrix(ip,jp,kp,alphad,iconcti)+fact

                           else

                              if ( k.lt.3 ) then
                                 matrix(ip,jp,kp,alphad,iconct1) =
     +                              matrix(ip,jp,kp,alphad,iconct1) +
     +                              2d0 * fact
                                 matrix(ip,jp,kp,alphad,iconct2) =
     +                              matrix(ip,jp,kp,alphad,iconct2) -
     +                              fact
                              else

c                             --- U_t has been given and U_n not
c                                 use formulas (2) and (4)

                                 det = bcgeo(6+istwo(alphad,alphad),
     +                                 ict1,ict2)*
     +                                 bcgeo(6+istwo(betad,betad),
     +                                 ict1,ict2)-
     +                                 bcgeo(6+istwo(betad,alphad),
     +                                 ict1,ict2)**2
                                gvt1 = bcgeo(6+istwo(betad,betad),
     +                                 ict1,ict2)*
     +                                 bcgeo(6+istwo(ndir,gammad),
     +                                 ict1,ict2)-
     +                                 bcgeo(6+istwo(betad,gammad),
     +                                 ict1,ict2)*
     +                                 bcgeo(6+istwo(ndir,betad),
     +                                 ict1,ict2)
                                matrix(ip,jp,kp,alphad,iconcti) =
     +                             matrix(ip,jp,kp,alphad,iconcti)-fact
                                fact = fact*gvt1/det
                                matrix(ip,jp,kp,alphad,iconctl) =
     +                             matrix(ip,jp,kp,alphad,iconctl)-fact
                                matrix(ip,jp,kp,alphad,iconctr) =
     +                             matrix(ip,jp,kp,alphad,iconctr)-fact

                              end if

                           end if

                        end if

                     end if

                  end do
               end do
            end do
         end do
      end do

      end
