      subroutine ismomtd3 ( ni, nj, nk, nvirtual, densrho, veloc,
     +                      matrix, rhside, deltat, iparea )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 23-05-95 (as ismomt2d)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     adds the time derivative to the idagonal of the matrix and
c     the right-hand side
c     R^3 only
c ********************************************************************
c
c                      KEYWORDS
c
c     3d
c     matrix
c     right_hand_side
c     time_derivative
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iparea(2,3)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 matrix(ni+1,nj+1,nk+1),
     +                 rhside(ni+1,nj+1,nk+1), deltat


c     deltat    i    Product of theta and dt
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp, kp
      double precision deltatin

c     deltatin  Inverse of deltat, i.e. 1/deltat
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c      It is assumed that the matrix and right-hand side have been initialized
c      before.
c      Furthermore this subroutine may only be called for centroids of
c      velocity cells.
c
c        The element for the time-derivative in the matrix diagonal is
c
c                   alphad
c          rho     V      / deltat
c             (0,0) (0,0)
c
c
c        The element for the time-derivative in the right-hand side is
c
c                  _alphad
c          rho     V      / deltat
c             (0,0) (0,0)
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over cells
c
      deltatin = 1d0/deltat
      do 120 kp = iparea(1,3), iparea(2,3)
         do 110 jp = iparea(1,2), iparea(2,2)
            do 100 ip = iparea(1,1), iparea(2,1)

               matrix(ip,jp,kp) = densrho(ip,jp,kp) * deltatin

100         continue
110      continue
120   continue

      do 220 kp = iparea(1,3), iparea(2,3)
         do 210 jp = iparea(1,2), iparea(2,2)
            do 200 ip = iparea(1,1), iparea(2,1)

               rhside(ip,jp,kp) =  densrho(ip,jp,kp) *
     +                          veloc(ip,jp,kp) * deltatin

200         continue
210      continue
220   continue

      end
