      subroutine ismomur(matrix, rhside, eddy, ni, nj,
     +                                       nconct, nvirtual)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.1    date 13-04-1998  Duncan: fixed bug for mscal=0
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c Subroutine underrelaxates the estimate of the new momentum
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nconct, nvirtual
      double precision matrix(ni+1,nj+1,2,nconct),
     +                 rhside(ni+1,nj+1,2),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2)

c     eddy      i    reserved
c     matrix   i/o   reserved
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i,j
      double precision cons1ur, alphain, alpha

c     alpha     Relaxation parameter alpha
c     alphain   1/alpha
c     cons1ur   (1-alpha)/alpha
c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c                                        (k+1)*
c   solution of the momentum equations Am  =rhs            (k)
c   is underrelaxated with regard to the previous iterate m
c   in the following way:
c        (k+1)*        (k+1)*
c    A  m       = -A  m        +rhs
c     ii  i         ij j,j=/i
c
c        (k+1)*    (k)         (k+1)*            (k)
c       m       = m    + (-A  m        +rhs    -m   )
c         i        i        ij j,j=/i             i
c                         __________________
c                               A
c                                 ii
c        (k+1)*    (k)                 (k+1)*            (k)
c       m       = m    + alpha * (-A  m        +rhs    -m   )
c         i        i                ij j,j=/i             i
c                                 __________________
c                                       A
c                                         ii
c
c    this gives:
c        1         (k+1)*    (k+1)*                         (k)
c   _________* A  m    + A  m        = rhs + (1-alpha)* A  m
c     alpha     ii  i     ij j,j=/i          _________   ii  i
c                                              alpha
c  this will help the pressure correction from coming excessively large
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- alpha  = 0.5d0

      alpha  = 0.5d0
      alphain=1d0/alpha
      cons1ur  =(1d0-alpha)*alphain

      do i=1,ni+1
         do j=1,nj
            rhside(i,j,1)   = rhside(i,j,1) +
     +                  eddy(i,j,1)*matrix(i,j,1,1)*cons1ur
        end do
      end do

      do i=1,ni
         do j=1,nj+1
            rhside(i,j,2)   = rhside(i,j,2) +
     +                  eddy(i,j,2)*matrix(i,j,2,1)*cons1ur
        end do
      end do

      do i=1,ni+1
         do j=1,nj
            matrix(i,j,1,1) = matrix(i,j,1,1)*alphain
        end do
      end do

      do i=1,ni
         do j=1,nj+1
            matrix(i,j,2,1) = matrix(i,j,2,1)*alphain
        end do
      end do

      end
