      subroutine ismst3wb ( ni, nj, nk, nvirtual, mu, dvsum, alphad,
     +                      locdx, iarea, adir, adiri )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 13-05-1998
c
c   copyright (c) 1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Components of the stress tensor in the integration points.
c     Three-dimensional 59-point Wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    stress
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, locdx(3), alphad, iarea(2,3)
      double precision mu(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 dvsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,3),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 adiri(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3)

c     adir      i    Contravariant base vector in integration cell center
c     adiri     i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dvsum     i    Flux derivative contributions to the matrix
c     iarea     i    Area of arrays
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, betad, gammad, j, ish, jsh, ksh
      double precision sign

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     gammad    Direction index
c     ip        Local counting variable i-direction
c     ish       Shift in i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     kp        Local counting variable k-direction
c     ksh       Shift in k-direction
c     sign      sign for multiplication (1 or -1)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       Relevant formula
c
c
c    In case of the Wesbeek discretization we have the following expressions
c    for the stress terms (alpha cell):
c
c
c         (a)               (1)  b |i+1/2,j,k            (2)  b |i,j+1/2,k
c        a    . (  sqrt(g) a    T  |         +   sqrt(g) a   T  |
c                            b     |i-1/2,j,k              b    |i,j-1/2,k
c          (i,j)
c
c
c
c                           (3)  b |i,j,k+1/2
c                  sqrt(g) a    T  |
c                            b     |i,j,k-1/2
c
c
c      in dvsum(i,j,k,gamma,1:3) the vector for the inproduct
c
c
c         (a)               (1)     gamma
c        a    .    sqrt(g) a       T       is stored
c                            gamma
c          (i,j)
c
c
c                                               (1)      (a)
c        so : dvsum(i,j,k,gamma,j) = mu sqrt(g) a        a
c                                                 gamma   j
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      if ( locdx(1).ne.0 .or. locdx(2).ne.0 .or. locdx(3).ne.0) then

c     --- Wesbeek discretization
c         The centroid has no contribution at all

         if ( abs(locdx(1)).ne.0 ) betad = 1
         if ( abs(locdx(2)).ne.0 ) betad = 2
         if ( abs(locdx(3)).ne.0 ) betad = 3
         sign = -locdx(betad)

         if ( alphad.eq.1 ) then
            ish = (locdx(1)-1)/2
            jsh = (locdx(2)+1)/2
            ksh = (locdx(3)+1)/2
         else if ( alphad.eq.2 ) then
            ish = (locdx(1)+1)/2
            jsh = (locdx(2)-1)/2
            ksh = (locdx(3)+1)/2
         else
            ish = (locdx(1)+1)/2
            jsh = (locdx(2)+1)/2
            ksh = (locdx(3)-1)/2
         end if

         do  gammad = 1, 3
            do  j = 1, 3
               do  kp = iarea(1,3), iarea(2,3)
                  do  jp = iarea(1,2), iarea(2,2)
                     do  ip = iarea(1,1), iarea(2,1)
                        dvsum(ip,jp,kp,gammad,j) =
     +                     sign * mu(ip,jp,kp) * adir(ip,jp,kp,j) *
     +                     adiri(ip+ish,jp+jsh,kp+ksh,gammad)
                     end do
                  end do
               end do
            end do
         end do

      end if

      end
