      subroutine ispres2d ( ni, nj, nvirtual, matrix, work, coefs, gmat,
     +                      rinput, iinput, geom, solut, infbnd, coor,
     +                      rhside, eddy, idiscret, bicof, ibicof,
     +                      densup, axisym, itopol )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.13   date 06-12-1999 Duncan: added itopol to parameter list
c                                       new call to iscdu22d en iscdu12d
c        version 2.12   date 30-06-1999 Guus: extension with computation of
c                                       drhodp/drhodh
c        version 2.11   date 03-04-1999 Duncan: new call to iscdu.12d and 22d
c                                               new call to iscomlr2
c        version 2.10   date 18-03-1999 Duncan: Supersonic Cranck-Nicholson
c        version 2.9    date 18-03-1999 Duncan: extension for Supersonic scheme
c        version 2.8    date 22-03-1999 Kees K:new call to ispmat2d
c        version 2.7    date 19-11-1998 Duncan:new call to ispmat2d
c        version 2.6    date 02-02-1999 call of isgmatr2 with mcom
c        version 2.5    date 02-02-1999 Kees K axisymmetric
c        version 2.4    date 22-07-1998 Duncan extension to iterate timestep
c                                       new call to isprhs2d
c        version 2.3    date 14-05-1998 Duncan: extension for saving
c                                           coefficients dens upwind
c                                           new call to ispmat2d
c        version 2.2    date 04-06-1998 Charles bilinear interpolation
c        version 2.1    date 05-03-1998 Duncan extension for cavity
c        version 2.0    date 11-12-1997 Hester:new call ispmat2d
c        version 1.9    date 01-12-1997 Hester:new call isprhsd2, ispmat2d
c        version 1.8    date 12-11-1997 Hester:new call iscomsr2
c        version 1.7    date 05-11-1997 Charles: new call to isprhs2d
c        version 1.6    date 30-10-1997 Hester: new call to isprhs2d
c        version 1.5    date 29-10-1997 Charles: change in collocated part
c        version 1.4    date 13-10-1997 Charles: collocated allowed
c        version 1.3    date 07-07-1996 Guus: remove change sign and
c                                             computation of gmat
c        version 1.2    date 24-06-1996 Kees K.: changing sign depending
c                                       on blockorientation added
c        version 1.1    date 26-02-1996 Hester: parameter list of ispmat2d
c                                          and isprhs2d changed, call to
c                                          iscomsr2 added.
c        version 1.0    date 06-10-1995
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build pressure matrix and right-hand side in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), infbnd(*), idiscret
      integer ibicof(*), axisym, itopol(*)
      double precision bicof(*)
      double precision geom((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 solut((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 work(*), matrix(*),
     +                 coefs((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 rinput(*), coor(*),
     +                 rhside(*), gmat(*), eddy(*), densup(*)

c     axisym         i    In axi-symmetric case axisym=1 else axisym=0
c     bicof          i    Array containing bilinear coefficients
c     coefs          i    COEFS(i,k) contains the k-th coefficient in
c                         centroid i. The nodes are ordered in exactly the
c                         same way as the scalar equations in SOLUT.
c                         The coefficients array is extended with virtual
c                         cells in exactly the same way as the locally
c                         copied solution array is.
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     densup         i    coefficients of density upwind in pressure matrix
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     geom           i    The geometrical quantities are only stored in the
c                         one-block solvers. These arrays are treated as
c                         temporary arrays, which means that they are
c                         created in the one-block solver and destroyed if
c                         the next block or step is considered. At present
c                         the storage is restricted to the storage of
c                         sqrt(g) and 1/sqrt(g) in all points of the grid.
c                         Exactly the same cells as for the co-ordinates are
c                         used.
c                         For a description see the programmers guide
c     gmat           i    Gradient matrix
c     ibicof         i    Integer array containing bilinear coefficients
c     idiscret       i    Type of discretization
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     infbnd         i    Array infbnd is used to indicate the type of boundary
c                         conditions for each point at the boundary of each
c                         single block for each unknown.
c                         For a description see the programmers guide
c     itopol         i    The itopol array stores the connections between
c                         the blocks. The basic idea behind the itopol
c                         data-structure is that the blocks communicate
c                         through common sub-faces.
c     matrix         o    Contains the matrix to be used in the solution process
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rhside         o    right-hand side vector
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c     work                Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iph, ipinvjac, ipvstar, ndegfd, npointsv, ntrnsp, nturb,
     +        ipsw, ipsr, mcomupw, iprincom, ipu, mshtyp, ipveloc,
     +        ipvtmp1, ipvtmp2, iptmp1, ipvstr1, ipvstr2, mprcor,
     +        ipjcta1,ipjcta2, ipcva, mscal, iphn, ipv, nk,
     +        ipp, mcav, ipun, npoints, istartup, ichoicedb,
     +        ipiincom, mcom, ipdrhodp, ippn, iprincav, ipsrwo1,
     +        ipsrwo2, iprtild

c     ichoicedb      if 1 a gradient based, if 2 a mach based,
c                    if 3 an unconditional density bias is applied
c     ipcva          Starting address of array CVA in array WORK
c     ipdrhodp       Starting address of array drhodp in array WORK
c     iph            Starting address of array h in solut
c     iphn           Starting address of array h in solut
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipinvjac       Starting address of array invjac in geom
c     ipjcta1        Starting address of array JCTA1 in array WORK
c     ipjcta2        Starting address of array JCTA2 in array WORK
c     ipp            Starting address of array p in solut
c     ippn           Starting address of array p in solut
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprtild        Starting address of array RTILD in array WORK
c     ipsr           Place of shift variable in array work
c     ipsrwo1        Starting address of array SRWO1 in array WORK
c     ipsrwo2        Starting address of array SRWO2 in array WORK
c     ipsw           Place of switch variable in array work
c     iptmp1         Starting address of interpolation array in work array
c     ipu            Starting address in array solut
c     ipun           Starting address in array solut
c     ipv            Starting address in array solut
c     ipveloc        Starting address of array VELOC in array WORK
c     ipvstar        Starting address of array vstar in solut
c     ipvstr1        Starting address of array vstar1 in solut
c     ipvstr2        Starting address of array vstar2 in solut
c     ipvtmp1        Starting address of array vstartmp1 in solut
c     ipvtmp2        Starting address of array vstartmp2 in solut
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mcomupw        If zero the rho equation for compressible flow is
c                    solved by a central scheme,
c                    otherwise by a upwind scheme
c     mprcor         Indicates choice of compressible pressure correction
c                    method:
c                    0:   Original formulation
c                    1:   Pressure corr. equation better for contact
c                         discontinuities, eq. obtained from
c                         energy+momentum eq.
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     mshtyp         Type of grid arrangement
c     ndegfd         Number of degrees of freedom
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npoints        Number of points in the mesh
c     npointsv       Number of points in the mesh with virtual points
c     ntrnsp         Number of transport equations
c     nturb          Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY          Copy double precision array into other one
c                    (BLAS subroutine)
c     ISCDU12D       Calculate the contribution to the pressure matrix of
c                    density upwind bias i-direction
c     ISCDU22D       Calculate the contribution to the pressure matrix of
c                    density upwind bias j-direction
c     ISCOMLR2       Upwind for compressible density, gradient based
c     ISCOMSR2       Upwind for compressible density
c     ISCOMSUP       Upwind for compressible density, supersonic scheme
c     ISCOVA2D       Fill covariant base vectors in all integration points in R2
c     ISDRHODH       Compute drho/dh in cell centres
c     ISDRHODP       Compute drho/dp in cell centres
c     ISGMATR2       Build gradient matrix in R2 multiplied by -2 dt / rho
c     ISJCTA2D       Fill Jacobian times contravariant base vectors
c                    in one integration point in R2
c     ISPMAT2D       Build pressure matrix in R2
c     ISPRHS2D       Build pressure right-hand-side vector in R2
c     ISSETCON       Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    The pressure correction method in combination with the theta method
c    (theta > 0) leads to the following system of equations for the pressure:
c
c                 -1                  *
c     theta dt D M   G (delta p) = D V
c
c     with:
c
c     D:        the divergence matrix
c     G:        the gradient matrix
c     M:        a diagonal matrix with rho at the main diagonal
c     V^*:      the computed velocity in the prediction step
c     delta p:  the pressure correction to be computed
c     theta dt: theta * time step
c
c
c     In case of a compressible flow the matrix is updated by adding the
c     following term to the diagonal:
c
c     1/dt drho/dp = gamma M^2/(dt*(1+(1-gamma)/2)M^2)*h^n) sqrt(g)
c
c
c     and the right-hand side by
c
c     -1/dt drho/dh delta h = (h^n+1-h^n)/((h^n)^2 dt) *
c        ((gamma M^2)/(1+(1-gamma)/2)M^2) p^n + alpha ) sqrt(g)
c
c     with alpha = 1 + p_v ((1+(1-gamma)/2)M^2)^(-gamma/(gamma-1))-1)
c
c
c                    PSEUDO CODE
c
c
c    Compute gmat
c    Construct the pressure matrix
c    Construct the pressure right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Fill covariant basis vectors in cva for all integration points

      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)

      mshtyp   = iinput( 75 )
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      npoints  = (ni+1)*(nj+1)
      ntrnsp   = iinput( 63 )
      nturb    = iinput( 70 )
      ndegfd   = 3+ntrnsp+nturb
      ipinvjac = 5
      ipu = 1
      ipv = 2
      ipp = 3
      iph = 4
      ipun = ndegfd + ipu
      ippn = ndegfd + ipp
      iphn = ndegfd + iph
      ipvstar = 1+ndegfd
      nk = 0

      ipcva = 1
      if ( mshtyp.eq.0 ) then
         call issetcon ( work, 18*npointsv, 0d0 )
      else if ( mshtyp.eq.1 ) then
         call issetcon ( work, 34*npointsv, 0d0 )
      end if

      call iscova2d ( coor, work, ni, nj, nvirtual )

c     --- compute pointers for work array

      ipsw      =             16*npointsv+1
      ipsr      =    ipsw    + 2*npointsv
      iprtild   =    ipsr    + 2*npointsv
      ipsrwo1   =    iprtild + 2*npointsv
      ipsrwo2   =    ipsrwo1 + 2*npointsv
      iptmp1    =    ipsrwo2 + 2*npointsv
      ipjcta1   =    iptmp1  + 2*npointsv
      ipjcta2   =    ipjcta1 + 4*npointsv
      ipvtmp1   =    ipjcta2 + 4*npointsv
      ipvtmp2   =    ipvtmp1 +   npointsv
      ipvstr1   =    ipvtmp2 +   npointsv
      ipvstr2   =    ipvstr1 +   npointsv
      ipdrhodp   =    ipvstr2 +   npointsv
      ipveloc   =    ipdrhodp +   npointsv

      iprincom  = iinput(34)
      iprincav  = iinput(35)

c     --- compute some general parameters

      mcav      = iinput(77)
      istartup  = iinput(iinput(11) -1 + 2)
      mcomupw   = iinput(ipiincom+1)
      ichoicedb = iinput(ipiincom-1+7)
      mscal     = iinput(ipiincom+5-1)
      mprcor    = iinput(ipiincom+6-1)

c     --- If density upwind in compressible computation, compute
c         density shift and switch function, necessary for press.corr.form.

      if ( mcomupw.ne.0 ) then

         if( ichoicedb.eq.1) then

c        --- limited higher order upwind scheme

            call iscomlr2 ( ni, nj, ndegfd, nvirtual, solut(1,ipu),
     +                   eddy, work(ipsw), work(ipsr), work(ipsrwo1))

         else if( ichoicedb.ge.2.and.ichoicedb.le.3) then

c        --- first order upwind scheme, unconditional or mach based

            call iscomsr2 ( ni, nj, ndegfd, nvirtual, solut(1,ipu),
     +                      eddy, geom(1,ipinvjac), work,
     +                      rinput(iprincom), work(ipsw), work(ipsr),
     +                              mscal, mcav, ichoicedb,
     +                                    istartup, rinput(iprincav))
         else if ( ichoicedb.ge.4 ) then

c        --- supersonic scheme, either explicit or cranck-nicholson

            call iscomsup ( ni, nj, ndegfd, nvirtual, solut(1,ipu),
     +                      eddy, geom(1,ipinvjac), work,
     +                      rinput(iprincom), work(ipsw), work(ipsr),
     +                      mscal, mcav, ichoicedb, istartup,
     +                      rinput(iprincav), work(ipsrwo1),
     +                      work(ipsrwo2), work(iprtild), itopol(1),
     +                      itopol(2), itopol(3), itopol(itopol(5)),
     +                      itopol(itopol(6)) )
         end if
      end if

c     --- Multiply gmat by -theta dt / rho

      call isgmatr2 ( ni, nj, nvirtual, mcom, gmat, coefs, work,
     +                rinput, iinput, infbnd(1), infbnd(13),
     +                coefs(1,6) )

c     --- Calculate the matrix contribution due to density upwind
c         for cavitating flow in two directions, for compressible
c         flow just in the i-direction

      if ( mcomupw.ne.0 ) then

        call iscdu12d( solut(1,ipu), solut(1,ipun),
     +                   solut(1,ipp),work(ipsw),
     +                   work(ipsr), work(ipsrwo1),
     +                   densup, ni, nj, nvirtual,
     +                   rinput, iinput, eddy, itopol(1), itopol(2),
     +                   itopol(3), itopol(itopol(5)),
     +                   itopol(itopol(6)))
        if( mcav.eq.1) then

          call iscdu22d( solut(1,ipv), solut(1,ipun+1),
     +                   solut(1,ipp),work(ipsw+npoints),
     +                   work(ipsr+npoints), work(ipsrwo1+npoints),
     +                   densup(1+3*npoints), ni, nj, nvirtual,
     +                   rinput, iinput, eddy, itopol(1), itopol(2),
     +                   itopol(3), itopol(itopol(5)),
     +                   itopol(itopol(6)))
        else

          call issetcon(densup(1+3*npoints),3*npoints, 0d0)
          call issetcon(work(ipsr+npoints),npoints, 1d0)

        end if

      end if

c     --- Construct the pressure matrix

      if ( mcom.ne.0 .and. mprcor.eq.0 ) then

c     --- Compressible flow, fill the vector drhodp with drho/dp

         call isdrhodp ( 2, ni, nj, nk, nvirtual, ndegfd, rinput,
     +                   iinput, solut(1,iph), solut(1,ipp),
     +                   work(ipdrhodp), solut(1,ippn), eddy, solut )

      end if
      call ispmat2d ( ni, nj, nvirtual, matrix, gmat, rinput,
     +                iinput, geom(1,4), solut(1,ipu),
     +                solut(1,ipv), solut(1,iphn), work(ipsr),
     +                work(iptmp1), coefs(1,6), densup,
     +                work(ipdrhodp), coor, axisym, work(ipsw) )

c     --- Construct the pressure right-hand side

      call isjcta2d ( work ( ipcva+4*npointsv ),
     +                work( ipjcta1 ), ni, nj, nvirtual )
      call isjcta2d ( work ( ipcva+8*npointsv ),
     +                work( ipjcta2 ), ni, nj, nvirtual )

      call dcopy ( 2*npointsv, solut(1,ipvstar), 1,
     +             work(ipveloc), 1 )

      if ( mcav.eq.0 .and. mcom.ne.0 .and. mprcor.eq.0 ) then

c     --- Compressible flow, fill the vector drhodp with drho/dh

         call isdrhodh ( 2, ni, nj, nk, nvirtual, solut, eddy, rinput,
     +                   iinput, ndegfd, work(ipdrhodp) )

      end if
      call isprhs2d ( ni, nj, nvirtual, work(ipveloc),
     +                work(ipvstr1), work(ipvstr2),
     +                work(ipvtmp1), work(ipvtmp2),
     +                rhside, solut, coor, work(ipjcta1), work(ipjcta2),
     +                geom(1,4), rinput, iinput, ndegfd, solut(1,ipu),
     +                work(ipsw), work(ipsr), mshtyp, coefs(1,6),
     +                work(iptmp1), idiscret, bicof, ibicof, eddy,
     +                axisym, work(ipdrhodp) )

      end
