      subroutine ispres3d ( ni, nj, nk, ndim, nvirtual, matrix, work,
     +                      coefs, gmat, rinput,  iinput, geom, solut,
     +                      infbnd, coor, rhside, mctg, mctinvg )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 08-04-1998 Remove computation of gmat
c        version 1.0    date 05-02-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build pressure matrix and right-hand side in R3
c
c **********************************************************************
c
c                       KEYWORDS
c
c    3d
c    matrix
c    pressure_equation
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ndim, nvirtual, infbnd(*), mctg(*),
     +        mctinvg(*), iinput(*)
      double precision geom((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                      (nk+1+2*nvirtual),*),
     +                 solut((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                      (nk+1+2*nvirtual),*),
     +                 work(*), coefs(*), rinput(*), coor(*),
     +                 rhside(*), gmat(*), matrix(*)

c     coefs     i    reserved
c     coor      i    reserved
c     geom      i    reserved
c     gmat      i    Gradient matrix
c     iinput    i    reserved
c     infbnd    i    reserved
c     matrix    i    reserved
c     mctg      i    Integer matrix information
c     mctinvg   i    Matrix information
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    i    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipvstar, ndegfd, ntrnsp, nturb
      integer ipinvjac, npointsv,
     +        rspv(1:3,1:8,0:3,0:4,1:8),
     +                 npf(0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     cv        reserved
c     ipinvjac  Starting address of array invjac in geom
c     ipvstar   Starting address of array vstar in solut
c     ndegfd    Number of degrees of freedom
c     npf       reserved
c     npointsv  Number of points in the mesh with virtual points
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     rspv      reserved
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISGMATR3  Build gradient matrix in R3 multiplied by -2 dt / rho
c     ISPMAT3D  Compute pressure matrix
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISPRHS3D  Compute right-hand side pressure matrix
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    The pressure correction method in combination with the theta method
c    (theta > 0) leads to the following system of equations for the pressure:
c
c                 -1                  *
c     theta dt D M   G (delta p) = D V
c
c     with:
c
c     D:        the divergence matrix
c     G:        the gradient matrix
c     M:        a diagonal matrix with rho at the main diagonal
c     V^*:      the computed velocity in the prediction step
c     delta p:  the pressure correction to be computed
c     theta dt: theta * time step
c
c
c     In case of a compressible flow the matrix is updated by adding the
c     following term to the diagonal:
c
c     1/dt drho/dp = gamma M^2/(dt*(1+(1-gamma)/2)M^2)*h^n) sqrt(g)
c
c
c     and the right-hand side by
c
c     -1/dt drho/dh delta h = (h^n+1-h^n)/((h^n)^2 dt) *
c        ((gamma M^2)/(1+(1-gamma)/2)M^2) p^n + alpha ) sqrt(g)
c
c     with alpha = 1 + p_v ((1+(1-gamma)/2)M^2)^(-gamma/(gamma-1))-1)
c
c
c                    PSEUDO CODE
c
c
c    Compute gmat
c    Construct the pressure matrix
c    Construct the pressure right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)
      ntrnsp   = iinput( 63 )
      nturb    = iinput( 70 )
      ndegfd   = 4+ntrnsp+nturb
      ipinvjac = 9
      ipvstar = 1+ndegfd

c     --- Fill covariant basis vectors in cva for all integration points

      call ispndf   ( npf, rspv, cv, ndim )
      call issetcon ( work , 72*npointsv, 0d0 )
      call iscova3d ( coor, work , ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

c     --- Multiply gmat by -theta dt / rho

      call isgmatr3 ( ni, nj, nk, nvirtual, gmat, coefs,
     +                work, rinput, iinput, infbnd(1),
     +                infbnd(2*ndim+1),
     +                infbnd(2*ndim*(1+2*(ndim-1))+1) )

c     --- Construct the pressure matrix

      call ispmat3d( gmat, matrix, ni, nj, nk, ndim )

c     --- Construct the pressure right-hand side

      call isprhs3d( ni, nj, nk, nvirtual, solut(1,ipvstar), rhside )

      end
