      subroutine isprhs3d ( ni, nj, nk, nvirtual, vstar, rhside )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 06-02-96
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Build pressure right-hand-side vector in R3
c
c ********************************************************************
c
c                      KEYWORDS
c
c    3d
c    matrix
c    pressure_equation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual

      double precision vstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3),
     +                 rhside(ni+1,nj+1,nk+1)


c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     vstar     i    Array containing the predicted velocity V^*
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, j, k

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISSETCON  Make array equal to a constant
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    METHOD
c
c
c    The pressure correction right-hand side has the shape:
c
c        *
c     D V
c
c     with:
c
c     D:        the divergence matrix
c     V*:       the predicted velocity
c
c     The divergence per cell looks like
c
c
c                          1
c                   -------|-------
c                   |             |
c                   |             |
c                -1 -             - 1
c                   |             |
c                   |             |
c                   -------|-------
c                         -1
c
c
c                    PSEUDO CODE
c
c
c     Clear pressure right-hand side
c     Construct the pressure right-hand side
c     If ( compressible flow ) then
c        Update right-hand side
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- Clear pressure right-hand side

      call issetcon ( rhside, (ni+1)*(nj+1)*(nk+1), 0d0 )

c     --- Construct the pressure right-hand side


      do k = 1, nk
         do j = 1, nj
            do i = 1, ni
               rhside(i,j,k) = -vstar(i+1,j,k,1)+vstar(i,j,k,1)
     +                         -vstar(i,j+1,k,2)+vstar(i,j,k,2)
     +                         -vstar(i,j,k+1,3)+vstar(i,j,k,3)
            end do
         end do
      end do


      end
