      subroutine istran2d ( iinput, matrix, rhside, solut, coefs, work,
     +                      invjac, rinput, npointsv, infmt, tsum,
     +                      dtsum, coor, mct, bndcon, ibipoint, ibinfor,
     +                      jac, iequa, ndegfd, nwork, gmat, infmg,
     +                      isiinput, issolut, isitopol, itimeact,
     +                      bicof, ibicof )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.6    date 21-12-1999 Guus: correction for internal energy
c        version 2.5    date 21-09-1999 Guus: rename iconst
c        version 2.4    date 23-03-1999 Duncan: difference normal transport
c                                               /enthalpy added
c        version 2.3    date 26-02-1999 Kees K: axisymmetric
c        version 2.2    date 22-02-1999 Kees K: variable gamcm added
c        version 2.1    date 04-06-1998 Charles: bilinear interpolation
c        version 2.0    date 15-12-1997 Charles: changes in convection
c        version 1.9    date 11-12-1997 Guus: correct definition of ipiindsc
c                                       in case of transport equation and
c                                       decoupled momentum equations
c        version 1.8    date 10-12-1997 Guus: make definition of iequa
c        version 1.7    date 05-11-1997 Charles: correction in collocated part
c        version 1.6    date 15-09-1997 Charles: adaptation as momentum
c                                                equation for collocated
c                                                grids
c        version 1.5    date 03-07-1997 Guus: new call to isintcf2
c        version 1.4    date 17-02-1997 Marcel: parameter list of
c                                               isurhdc2 changed
c        version 1.3    date 17-10-1996 Guus: new definition of iindsc
c        version 1.2    date 11-12-1995 Marcel: modified tranformation
c                                               coefficients
c        version 1.1    date 07-12-1995 Marcel: local parameter acttyp
c        version 1.0    date 26-10-1995
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build transport matrix and right-hand side in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    transport_matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmt(*), infmg(*), iequa, ndegfd,
     +        mct(-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        isiinput(*), isitopol, issolut(*), itimeact
      integer ibicof(*)
      double precision bicof(*)
      double precision rhside(*), work(*), matrix(*), coor(*),
     +                 solut(npointsv,ndegfd,*), coefs(npointsv,*),
     +                 rinput(*), tsum(npointsv), invjac(npointsv,*),
     +                 dtsum(npointsv,2), bndcon(2,*),
     +                 jac(npointsv,*), gmat(*)

c     bicof     i    Array containing bilinear coefficients
c     bndcon    i    reserved
c     coefs    i/o   reserved
c     coor      i    reserved
c     dtsum     i    Flux derivative contributions to the matrix
c     gmat      i    Gradient matrix
c     ibicof    i    Integer array containing bilinear coefficients
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmt     i    Integer information for Pressure and Transport matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     itimeact  i    Actual time step
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     ndegfd    i    Number of degrees of freedom
c     npointsv  i    Number of points in the mesh with virtual points
c     nwork     i    Second dimension of array work
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     tsum      i    Flux contributions to the matrix
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nconct, alphad, iprintim, pnttyp, vartyp, ni, nj, ndim,
     +        nvirtual, iarea(2,2), locdx(2), iprindsc, nwork,
     +        dcell(2), iparea(2,2), ipcva, ipjcta, acttyp, ncoefs,
     +        i, ipiindsc, mcom, ipcvc, ipctc, ipveloc,
     +        ipiincom, iprincom, leniindsc, ipcvac, betad, idegfd,
     +        ipcstar, ipfstar, ipzot, ipkdif1, ipkdif2, ipkdif3,
     +        ichoisdis, idiscret, mshtyp, icoupled, iseqpres,
     +        icstar, iconstant, isource, ik11, ik12, ik22, itimlv,
     +        iblock, itrans, nblocks, axisym, ipradius1, ipradius2,
     +        newtcom, mlin, menergy
      double precision deltat, gamcm
      logical reuse, usebndcn

c     acttyp         Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     alphad         Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     axisym         In axisymmetric case axisym=1 else axisym=0
c     betad          Direction index, usually the direction perpendicular to the
c                    integration direction of the boundary integral.
c                    For example betad in local point (1,0) is 1 and in (0,1): 2
c     dcell          reserved
c     deltat         Product of theta and dt
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     iarea          Area of arrays
c     iblock         Block sequence number
c     ichoisdis      Choice parameter with respect to the convection scheme
c                    for discretization of convective term of transport
c                    equation(s) including turbulence.
c                    Meaning of ichoisdis:
c                      ichoisdis = 0: 2nd order central scheme
c                      ichoisdis = 1: 1st order standard upwind scheme
c                      ichoisdis = 2: Hybrid central/upwind scheme
c                      ichoisdis = 3: Linear kappa-scheme
c                      ichoisdis = 4: Sweby Phi-limiter
c                      ichoisdis = 5: R-kappa limiter
c                      ichoisdis = 6: USR-kappa limiter
c                      ichoisdis = 7: PL-kappa limiter
c                      ichoisdis = 8: USPL-kappa limiter
c                      ichoisdis = 9: Symmetric PL-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     iconstant      Sequence number of the coefficient that corresponds to
c                    const
c     icoupled       If 0 the momentum equations are coupled, if 1 they are not
c     icstar         Sequence number of the coefficient that corresponds to
c                    cstar
c     idegfd         Loop variable with respect to a given unknown
c     idiscret       Type of discretization
c     ik11           Sequence number of the coefficient that corresponds to k11
c     ik12           Sequence number of the coefficient that corresponds to k12
c     ik22           Sequence number of the coefficient that corresponds to k22
c     iparea         Area of integration points where a contribution to
c                    the matrix is to be calculated
c     ipcstar        Starting address of array CSTAR in array WORK
c     ipctc          Starting address of array CTC in array WORK
c     ipcva          Starting address of array CVA in array WORK
c     ipcvac         Starting address of array CVA with respect to centre point
c                    in array WORK
c     ipcvc          Starting address of array CVC in array WORK
c     ipfstar        Starting address of array FSTAR in array WORK
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiindsc       Starting address of array IINDSC in array IINPUT
c     ipjcta         Starting address of array JCTA in array WORK
c     ipkdif1        Starting address of array KDIF1 in array WORK
c     ipkdif2        Starting address of array KDIF2 in array WORK
c     ipkdif3        Starting address of array KDIF3 in array WORK
c     ipradius1      Starting address of array radius part one in array WORK
c     ipradius2      Starting address of array radius part two in array WORK
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprindsc       Starting address of array RINDSC in array RINPUT
c     iprintim       Starting address of array RINTIM in array RINPUT
c     ipveloc        Starting address of array VELOC in array WORK
c     ipzot          Starting address of array ZOT in array WORK
c     iseqpres       Sequence number of pressure equation
c     isource        Sequence number of the coefficient that corresponds to
c                    source
c     itimlv         Loop variable with respect to a given time level
c     itrans         reserved
c     leniindsc      Length of first component of array iindsc
c     locdx          Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mcom           If zero the flow is incompressible else (1) compressible
c     menergy        Defines the type of energy equation used.
c                    Possible values:
c                    0: none
c                    1: the primary unknown is the enthalpy
c                    2: the primary unknown is the total enthalpy
c                    3: the primary unknown is the internal energy
c     mlin           Type of linearization (1=newton, 2=picard)
c     mshtyp         Type of grid arrangement
c     nblocks        Number of blocks
c     ncoefs         Number of coefficients
c     nconct         Number of connections in a row of the matrix
c     ndim           Dimension of space (2 or 3)
c     newtcom        If one Newton linearisation is used for convective terms
c                    if compressible flow.
c     ni             Number of non-virtual cells in the i-direction (one block)
c     nj             Number of non-virtual cells in the j-direction (one block)
c     nvirtual       Number of rows of virtual cells
c     pnttyp         reserved
c     reuse          If true the computed coefficients may be reused again
c     usebndcn       If true boundary conditions are used in the extrapolation
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISBTRAD2  Correct dtsum, matrix and right-hand side for Robbins type
c               boundary conditions for the transport equation in R2
c     ISCMTDU   Computes compressible contribution to linear part of the
c               transport equation in R2
c     ISCONC2D  Perform the contravariant components form the Cartesian
c               at the cell faces
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISCRMO2D  Create the rhs of the momentum equation for collocated
c     ISDISCT2  Distribute convective terms for transport equations over the
c               matrix in R2
c     ISDISDT2  Distribute diffusion term for tranport equation in R2
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISFILVSO    Extrapolate solution at outer boundaries
c     ISINTCF2  Compute interpolation or finite differences of a
c               coefficient in an integration point
c     ISINTCFA  Compute interpolation of a coefficient in an integration point
c               in the axisymmetric case
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISSETCON  Make array equal to a constant
c     ISTRAC2D  Compute contribution of convection term in R2 for the
c               transport equations
c     ISTRAD2D  Compute contribution of diffusion term in R2 for the
c               transport equations
c     ISTRAF2D  All parts of the matrix outside the region are folded to
c               internal domain for the transport matrix in R2
c     ISTRATD2  adds the time derivative to the diagonal of the matrix and
c               the right-hand side for the 2D transport equation
c     ISTRFM2D  Transform the cartesian diffusion coefficients
c               into contravariant coefficients in R2
c     ISURHDC2  Update right-hand side vector with higher order upwind scheme
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Fill covariant basis vectors in cva for all integration points
c    Transform the cartesian diffusion coefficients into
c    contravariant coefficients
c    Interpolate C^*, f^* and D to integration point (Cell centre)
c    Build time-derivative part and constant part of the matrix and rhside
c    For all the integration points, except the centr do
c       Interpolate coefficients to integration points
c       Compute local co-ordinates, distances to cell center in global
c       co-ordinates, point type
c       Interpolate coefficients to integration points
c       Compute contribution of convection term in integration point
c       Distribute contribution of convection term in integration point
c       Correct right-hand side in case of higher order upwind scheme
c       (defect correction)
c       Compute contribution of diffusion term in integration point
c       Compute effect of boundary conditions
c       Distribute contribution of diffusion term in integration point
c    Fold all points in the matrix outside the region inwardly
c
c    Remark:  Array cvac is the last part of array cva
c             In fact the last 4 arrays of cva are reused as cvac
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      leniindsc= iinput(106)
      iprintim = iinput(29)
      ndim     = iinput(57)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ncoefs   = iinput(66)
      nblocks  = iinput(73)
      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)
      menergy  = iinput(ipiincom+9)
      mshtyp   = iinput(75)
      icoupled = iinput(76)

      if ( icoupled.eq.1 ) then

c     --- Decoupled solution of momentum equations

         iseqpres = ndim+1
         if ( iequa.lt.iseqpres ) then

c        --- momentum equation
c            In this part we assume that the sixth coeeficient is 0,
c            which is of course only valid if no mass fraction is used.
c            It is just a trick to make the constant zero
c            This must be corrected in the future

            icstar   = 1
            iconstant   = 6
            isource  = iequa + 2
            ik11     = 2
            ik12     = 6
            ik22     = 2
            ipiindsc = iinput(6)+leniindsc*(iequa-1)
            iprindsc = iinput(33)+10*(iequa-1)

         else

c        --- transport equation

            icstar   = 1
            iconstant   = 2
            isource  = 3
            ik11     = 4
            ik12     = 5
            ik22     = 6
            ipiindsc = iinput(6)+leniindsc*(iequa-2)
            iprindsc = iinput(33)+10*(iequa-2)

         end if
         idegfd   = iequa

      else

c     --- Coupled solution of momentum equations

         iseqpres = 2
         ipiindsc = iinput(6)+leniindsc*(iequa-2)
         iprindsc = iinput(33)+10*(iequa-2)
         icstar   = 1
         iconstant   = 2
         isource  = 3
         ik11     = 4
         ik12     = 5
         ik22     = 6
         idegfd   = iequa+1

      end if

      axisym   = iinput(ipiindsc-1+10)
      mlin     = iinput(iinput(6)-1+ 8)
      if( mcom.eq.1.and.mlin.eq.1) then
         newtcom  = 1
      else
         newtcom  = 0
      end if

      nconct   = infmt(6)
      deltat   = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      ipcva    = 1
      ipcvac   = 1+12*npointsv
      ipjcta   = ipcva+16*npointsv
      ipcstar  = ipjcta+4*npointsv
      ipzot    = ipcstar+npointsv
      ipfstar  = ipzot+npointsv
      ipkdif1  = ipfstar+npointsv
      ipkdif2  = ipkdif1+npointsv
      ipkdif3  = ipkdif2+npointsv
      ipcvc    = ipkdif3+npointsv
      ipctc    = ipcvc+4*npointsv
      ipveloc  = ipctc+4*npointsv
      ipradius1 = ipveloc+4*npointsv
      ipradius2 = ipradius1+npointsv

      if ( ipradius1+2*npointsv.gt.nwork ) then
        call iseropen( 'istran2d' )
        call iserrchr( 'work',1 )
        call iserrint( ipradius1+2*npointsv, 1 )
        call iserrint( nwork, 2 )
        call iserrsub ( 17, 2, 0, 1 )
        call iserclos( 'istran2d' )
      end if

c     --- Fill radius with 1d0 if not axisymmetric

      call issetcon ( work(ipradius1), 2*npointsv, 1d0 )

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*nconct, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1), 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill contravariant basis vectors in cell centre

      pnttyp = 4
      call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                work(ipjcta), ni, nj, nvirtual )

c     --- Transform the cartesian diffusion coefficients into
c         contravariant coefficients

      idiscret = iinput(ipiindsc+1)
      if ( idiscret.eq.1 ) then

c     --- classical discretization

        call istrfm2d ( coefs, invjac(1,pnttyp), work(ipjcta),
     +                  ncoefs, ni, nj, nvirtual )

      end if

c     --- Build time-derivative part and constant part of the matrix and rhside
c         Interpolate C^*, f^* and D to integration point (Cell centre)

      iarea(1,1) = 1
      iarea(1,2) = 1
      iarea(2,1) = ni
      iarea(2,2) = nj
      pnttyp = 4
      vartyp = 3
      acttyp = 0

      if ( iequa.lt.iseqpres ) then
         call istratd2 ( ni, nj, nvirtual, axisym, coefs(1,icstar),
     +            solut(1,idegfd,2), matrix, rhside, deltat, iarea,
     +            jac(1,pnttyp), coefs(1,isource), coefs(1,iconstant),
     +            coor )
      else
         call istratd2 ( ni, nj, nvirtual, axisym, coefs(1,icstar),
     +            solut(1,idegfd,1), matrix, rhside, deltat, iarea,
     +            jac(1,pnttyp), coefs(1,isource), coefs(1,iconstant),
     +            coor )
      end if

      if ( mcom.eq.1 .and. newtcom.ne.1 .and. menergy.le.1 ) then

c     --- compressible case, add -0.6 div(u) to matrix diagonal in case
c         of enthalpy equation becuase the non-conservative form is used

        iprincom = iinput(34)-1
        gamcm = rinput(iprincom+1)
        call iscmtdu ( ni, nj, nvirtual, axisym, iarea, solut,
     +                 matrix, coor, gamcm )

      end if

c     --- Loop over integration points

      do 300 i = 2, 5

c     --- Compute local co-ordinates, distances to cell center in global
c         co-ordinates, point type

         dcell(1) = 0
         dcell(2) = 0
         if ( i.eq.2 ) then
            locdx(1) = -1
            locdx(2) =  0
            alphad   = 1
         else if ( i.eq.3 ) then
            locdx(1) = 1
            locdx(2) = 0
            alphad   = 1
            dcell(1) = 1
         else if ( i.eq.4 ) then
            locdx(1) =  0
            locdx(2) = -1
            alphad   = 2
         else
            locdx(1) = 0
            locdx(2) = 1
            alphad   = 2
            dcell(2) = 1
         end if
         pnttyp = 4-alphad
         reuse = locdx(1).eq.1 .or. locdx(2).eq.1

         if ( .not. reuse ) then

c        --- Compute cstar and kdif in integration point

            vartyp = 3
            acttyp = 0

            if ( iinput(ipiindsc).eq.1 ) then

c           --- Convection

               if ( axisym.eq.1 ) then

                  call isintcfa ( coefs(1,icstar), work(ipcstar), ni,
     +                             nj, nvirtual, vartyp, acttyp,
     +                             pnttyp, mshtyp, coor )
               else
                  call isintcf2 ( coefs(1,icstar), work(ipcstar), ni,
     +                             nj, nvirtual, vartyp, acttyp,
     +                             pnttyp, mshtyp, coor )
               end if

            end if

            if ( iinput(ipiindsc-1+3).eq.0 ) then

c           --- Diffusion

               call isintcf2 ( coefs(1,ik11), work(ipkdif1), ni, nj,
     +                         nvirtual, vartyp, acttyp, pnttyp,
     +                         mshtyp, coor )
               call isintcf2 ( coefs(1,ik12), work(ipkdif2), ni, nj,
     +                         nvirtual, vartyp, acttyp, pnttyp,
     +                         mshtyp, coor )
               call isintcf2 ( coefs(1,ik22), work(ipkdif3), ni, nj,
     +                         nvirtual, vartyp, acttyp, pnttyp,
     +                         mshtyp, coor )

               if( axisym.eq.1 ) then

c              --- if not axisymmetric the diffusion terms are
c                  multplied by 1d0, now in the axisymmetric case
c                  work is filled with the real radius

                  call isintcfa ( work(ipradius2), work(ipradius1), ni,
     +                             nj, nvirtual, vartyp, acttyp,
     +                             pnttyp, mshtyp, coor )
               end if

            end if

         end if

c        --- List of loop bounds for all integration points
c            iparea gives the bounds for the cell indices of the integration
c            points
c            These are the indices of the centres of the integration cells
c            but shifted over a distance dcell, since the actual
c            integration points may be in another cell

         iparea(1,1) = iarea(1,1)+dcell(1)
         iparea(2,1) = iarea(2,1)+dcell(1)
         iparea(1,2) = iarea(1,2)+dcell(2)
         iparea(2,2) = iarea(2,2)+dcell(2)

         if ( iinput(ipiindsc).eq.1 ) then

c        --- convection term
c            First clear array tsum

            call issetcon ( tsum, npointsv, 0d0 )

c           --- Next fill tsum with contribution of convective terms to
c               integration points

            if ( mshtyp.eq.0 ) then

c           --- Staggered grid, copy velocity to work(ipveloc)

               call dcopy ( npointsv, solut(1,alphad,2), 1,
     +                      work(ipveloc), 1 )

            else if ( mshtyp.eq.1 ) then

               if ( itimeact.eq.1) then

c              --- Extrapolation in the outer domain in function of
c                  the boundary conditions at the first time step

                  iblock = 1
                  itimlv = 1
                  usebndcn = .false.
                  do itrans = 1,2
                     call isfilvso ( iblock , itrans, itimlv, nblocks,
     +                               isiinput, issolut, isitopol,
     +                               isitopol, isitopol, icoupled,
     +                               usebndcn )
                  end do

               end if

c              --- Colocated grid: interpolation of the Cartesian
c                  components at the faces
c                  If momentum equation, use of the previous velocity
c                  else use of the current one.

               if ( iequa.gt.iseqpres ) then
                  itimlv = 2
               else if ( iequa.lt.iseqpres ) then
                  itimlv = 1
               end if

               pnttyp = 4 - alphad
               call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                         work(ipjcta), ni, nj, nvirtual )

               call issetcon ( work(ipveloc ), 2*npointsv, 0d0 )

               do betad = 1, 2

                  call isintcf2 ( solut( 1, betad, itimlv ),
     +                            work( ipveloc+(betad-1)*npointsv ),
     +                            ni, nj, nvirtual, vartyp, acttyp,
     +                            pnttyp, mshtyp, coor )

               end do

               call isconc2d ( work( ipveloc+2*npointsv ),
     +                        work( ipveloc ), work( ipveloc+npointsv ),
     +                        work( ipjcta ), ni, nj, nvirtual, alphad )

               call dcopy ( npointsv,work( ipveloc+2*npointsv ), 1,
     +                      work( ipveloc ), 1 )
            end if

c           --- Next fill tsum with contribution of convective terms to
c               integration points

            pnttyp = 4 - alphad
            call istrac2d ( ni, nj, nvirtual, work(ipcstar),
     +                      work(ipveloc) , tsum, iparea,
     +                      locdx(alphad) )

c           --- Distribute computed values over the positions
c               corresponding to the unknowns

            ichoisdis = iinput(ipiindsc-1+5)
            call isdisct2 ( alphad, ni, nj, nvirtual, matrix, tsum,
     +                      work(ipveloc), mct, iarea, dcell,
     +                      work(ipcstar), work(ipkdif1),
     +                      jac(1,pnttyp), ichoisdis,
     +                      iinput(ipiindsc-1+3) )

            if ( ichoisdis.gt.2 ) then

c           --- Higher order upwind scheme
c               Since defect correction is used this implies updating the
c               right-hand side

              call isurhdc2 (alphad, ni, nj, nvirtual, rhside, tsum,
     +                       work(ipveloc), solut(1,idegfd,1),
     +                       iarea, dcell, ichoisdis,
     +                       rinput(iprindsc-1+1), rinput(iprindsc-1+2),
     +                       rinput(iprindsc-1+3), rinput(iprindsc-1+4))

            end if

         end if

         if ( iinput(ipiindsc-1+3).eq.0 ) then

c        --- Diffusion term
c            First clear array dtsum

            call issetcon ( dtsum, 2*npointsv, 0d0 )
            ichoisdis = iinput(ipiindsc-1+6)

            if ( idiscret.gt.1 ) then

c           --- Wesbeek discretization, fill contravariant basis vectors

               call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                         work(ipjcta), ni, nj, nvirtual )

            end if

            call istrad2d ( ni, nj, nvirtual, work(ipkdif1),
     +                      jac(1,pnttyp), dtsum, iparea, locdx(alphad),
     +                      alphad, coefs(1,2+2*alphad*(1-icoupled+
     +                      icoupled*max(0,iequa-iseqpres)/
     +                      abs(iequa-iseqpres))),
     +                      work(ipcvac),
     +                      work(ipjcta), work(ipcvc), work(ipctc),
     +                      idiscret, work(ipradius1) )

c           --- Compute effect of boundary conditions

            call isbtrad2 ( i-1, ni, nj, nvirtual, dtsum, work(ipcva),
     +                      matrix, ibinfor, ibipoint, bndcon, rhside,
     +                      work(ipjcta), invjac(1,pnttyp), idiscret,
     +                      mct )

c           --- Distribution of matrix

            call isdisdt2 ( alphad, ni, nj, nvirtual, matrix, dtsum,
     +                      mct, iarea, dcell, ichoisdis )

         end if

300   continue

      if ( mshtyp.eq.1 .and. iequa.lt.iseqpres ) then

c     --- Compute the RHS (explicit part of the stress-term and
c         pressure gradient) in the colocated case

         alphad = iequa

         call iscrmo2d ( iinput, npointsv, ndegfd, alphad, bndcon,
     +                   rhside, work, coor, solut, coefs, jac,
     +                   isiinput, issolut, isitopol,
     +                   ibinfor, ibipoint, iseqpres, gmat,
     +                   infmg(infmg(12)), invjac, idiscret,
     +                   bicof, ibicof )

      end if

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      call istraf2d ( ni, nj, nvirtual, mct, matrix, rhside,
     +                ibinfor, ibipoint, bndcon )

      end
