      subroutine istrat3d ( ni, nj, nk, nvirtual, cstar, solut, matrix,
     +                      rhside, deltat, iparea, jac, fstar, zot )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date 01-05-1996
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     adds the time derivative to the diagonal of the matrix and
c     the right-hand side
c     R^3 only
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     matrix
c     right_hand_side
c     time_derivative
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iparea(2,3)
      double precision cstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 fstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 zot(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 matrix(ni+1,nj+1,nk+1), rhside(ni+1,nj+1,nk+1),
     +                 deltat,
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual)

c     cstar     i    The capacity in a given integration point
c     deltat    i    Product of theta and dt
c     fstar     i    The source term in a given integration point
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     solut     i    reserved
c     zot       i    The zeroth order term in a given integration point
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp
      double precision deltatin

c     deltatin  Inverse of deltat, i.e. 1/deltat
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      It is assumed that the matrix and right-hand side have been
c      initialized before.
c      Furthermore this subroutine may only be called for centroids of
c      transport cells.
c
c        The element for the time-derivative in the matrix diagonal is
c
c          cstar        T        / deltat
c               (0,0,0)  (0,0,0)
c
c
c        The element for the time-derivative in the right-hand side is
c
c          cstar        T        / deltat
c               (0,0,0)  (0,0,0)
c
c        The element for the source part in the right-hand side is
c
c              *
c             f        sqrt(g)
c              (0,0,0)        (0,0,0)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over cells

      deltatin = 1d0/deltat

      do kp = iparea(1,3), iparea(2,3)
        do jp = iparea(1,2), iparea(2,2)
          do ip = iparea(1,1), iparea(2,1)
            matrix(ip,jp,kp) = jac(ip,jp,kp)*( cstar(ip,jp,kp)*deltatin+
     +                                         zot(ip,jp,kp) )
          end do
        end do
      end do

      do kp = iparea(1,3), iparea(2,3)
        do jp = iparea(1,2), iparea(2,2)
          do ip = iparea(1,1), iparea(2,1)
            rhside(ip,jp,kp) = jac(ip,jp,kp)*( cstar(ip,jp,kp)*
     +                                         solut(ip,jp,kp)*deltatin+
     +                                         fstar(ip,jp,kp) )
          end do
        end do
      end do

      end
