      subroutine istrfm2d(coefs, invjac, jcta, ncoefs, ni, nj, nvirtual)
c=======================================================================
c
c
c        programmer     Marcel Zijlema
c        version 1.0    date   11-12-95
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c***********************************************************************
c
c                      DESCRIPTION
c
c     Transform the cartesian diffusion coefficients
c     into contravariant coefficients in R2
c
c***********************************************************************
c
c                      KEYWORDS
c
c     2d
c     coefficients
c
c***********************************************************************

c                      INPUT / OUTPUT    PARAMETERS

      implicit none

      integer ncoefs, ni, nj, nvirtual

      double precision coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ncoefs),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)

c     coefs     o    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ncoefs    i    Number of coefficients
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c
c***********************************************************************
c
c                      COMMON BLOCKS
c
c
c***********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer alphad, betad, gammal, deltal,
     +        ip, jp, dummy1, dummy2, j

      double precision cvcof(1:3)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cvcof     The contravariant diffusion coefficient (help variable)
c     deltal    Loop index
c     dummy1    Dummy
c     dummy2    Dummy
c     gammal    Loop index
c     ip        Local counting variable i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c
c***********************************************************************
c
c                      SUBROUTINES CALLED
c
c***********************************************************************
c
c                      I/O
c
c
c***********************************************************************
c
c                      ERROR MESSAGES
c
c
c***********************************************************************
c
c                      PSEUDO CODE
c
c     The following transformation rule has been used:
c
c                 pq    (p)  (q)  rs
c                K   = a    a    k
c                       r    s
c
c            rs                                        pq
c     where k   is the second order cartesian tensor, K   the second order
c
c                               (p)
c     contravariant tensor and a    is the r-the component of the contravariant
c     base vector.              r
c
c     A relationship between the 2D and 1D index sets for a given
c
c                          ab
c     second order tensor T   (both cartesian and contravariant), which is
c     written as T(i), is given in the following table:
c
c                      a | b | i
c                      ---------
c                      1 | 1 | 1
c                      1 | 2 | 2
c                      2 | 2 | 3
c
c     A vectorizable variant of this one-to-one map is given by
c
c     i = a + b - 1
c
c=======================================================================

      do jp = 2 - nvirtual, nj + nvirtual
        do ip = 2 - nvirtual, ni + nvirtual

          do alphad = 1, 2
            do betad = 1, 2

              dummy1 = alphad + betad - 1

              cvcof(dummy1) = 0d0

              do gammal = 1, 2
                do deltal = 1, 2

                  dummy2 = gammal + deltal - 1

                  cvcof(dummy1) = cvcof(dummy1) + invjac(ip,jp) *
     +                            invjac(ip,jp) *
     +                            jcta(ip,jp,gammal,alphad) *
     +                            jcta(ip,jp,deltal,betad) *
     +                            coefs(ip,jp,dummy2+3)

                end do
              end do

            end do
          end do

          do j = 4, 6
            coefs(ip,jp,j) = cvcof(j-3)
          end do

        end do
      end do

      end
