      subroutine isxcentr3 ( coor, xcentr, ni, nj, nk, nvirtual )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 31-03-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute coordinates of centroids of elements and store in xcentr
c    three-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)
      double precision xcentr(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     coor      i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     xcentr    o    Coordinates of centroids of cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, l

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The coordinates of the centroid are defined by the mean
c    value of the coordinates of the vertices
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Loop over three components

       do l = 1, 3

c      --- Clear outside rows of virtual cells

          do i = 1-nvirtual, ni+1+nvirtual, ni+1+2*nvirtual
             do j = 1-nvirtual, nj+1+nvirtual
                do k = 1-nvirtual, nk+1+nvirtual
                   xcentr(i,j,k,l) = 0d0
                end do
             end do
          end do
          do i = 1-nvirtual, ni+1+nvirtual
             do j = 1-nvirtual, nj+1+nvirtual, nj+1+2*nvirtual
                do k = 1-nvirtual, nk+1+nvirtual
                   xcentr(i,j,k,l) = 0d0
                end do
             end do
          end do
          do i = 1-nvirtual, ni+1+nvirtual
             do j = 1-nvirtual, nj+1+nvirtual
                do k = 1-nvirtual, nk+1+nvirtual, nk+1+2*nvirtual
                   xcentr(i,j,k,l) = 0d0
                end do
             end do
          end do

c         --- Compute centroid

          do i = 2-nvirtual, ni+nvirtual
             do j = 2-nvirtual, nj+nvirtual
                do k = 2-nvirtual, nk+nvirtual
                   xcentr(i,j,k,l) = 0.125d0*(
     +                   coor(i,j,k,l)+coor(i+1,j,k,l)+
     +                   coor(i,j+1,k,l)+coor(i+1,j+1,k,l)+
     +                   coor(i,j,k+1,l)+coor(i+1,j,k+1,l)+
     +                   coor(i,j+1,k+1,l)+coor(i+1,j+1,k+1,l))
                end do
             end do
          end do

       end do
       end
