      subroutine isextco3 ( iblock, ni, nj, nk, nvirtual,
     +                      nblocks, nfaces, maxnsubf,
     +                      coor, nsubfaces, subfaces )
c ======================================================================
c
c        programmer   Michiel Kooper
c        version 2.1    date 07-04-1997 Kees K. only external subfaces
c        version 2.0    date 27-03-1995 Guus renamed from isexco03
c        version 1.1    date 19-08-1994 RvN (line 262: ni replaced by nj)
c        version 1.0    date 28-07-1994
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize those virtual coordinates of the local coordinate array
c     of a certain subdomain that were not already initialized by istrco
c     by extrapolation.
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, nfaces, ni, nj,
     +      nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual, * )
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iface, isubface, iv, j, k, l
      double precision w1, w2, w3

c     i         General loop variable
c     iface     Face number
c     isubface  Subface number
c     iv        Indicates the number of the virtual row (0..nvirtual-1)
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     w1        Weight factor for extrapolation in direction 1 at the
c               faces/edges/vertices
c     w2        Weight factor for extrapolation in direction 2 at the
c               faces/edges/vertices
c     w3        Weight factor for extrapolation in direction 3 at the
c               faces/edges/vertices
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     None.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- handle external subfaces

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

               if ( iface.eq.1) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        i = 1 - iv
                        do j = 1, nj+1
                           do k= 1, nk+1
                              coor(i-1,j,k,l) = 2d0*coor(i  ,j,k,l) -
     +                                              coor(i+1,j,k,l)
                           end do
                        end do
                     end do
                  end do

               else if ( iface.eq.2 ) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        i = ni+1+iv
                        do j = 1, nj+1
                           do k= 1, nk+1
                              coor(i+1,j,k,l) = 2d0*coor(i  ,j,k,l) -
     +                                              coor(i-1,j,k,l)
                           end do
                        end do
                     end do
                  end do

               else if ( iface.eq.3 ) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        j = 1-iv
                        do i = 1, ni+1
                           do k = 1, nk+1
                              coor(i,j-1,k,l) = 2d0*coor(i,j  ,k,l) -
     +                                              coor(i,j+1,k,l)
                           end do
                        end do
                     end do
                   end do

               else if ( iface.eq.4 ) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        j = nj+1+iv
                        do i = 1, ni+1
                           do k = 1, nk+1
                              coor(i,j+1,k,l) = 2d0*coor(i,j  ,k,l) -
     +                                              coor(I,j-1,k,l)
                           end do
                        end do
                     end do
                  end do

               else if ( iface.eq.5 ) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        k = 1-iv
                        do i = 1, ni+1
                           do j = 1, nj+1
                              coor(i,j,k-1,l) = 2d0*coor(i,j,k  ,l) -
     +                                              coor(I,j,k+1,l)
                           end do
                        end do
                     end do
                  end do

               else if ( iface.eq.6 ) then

                  do l = 1, 3
                     do iv = 0, nvirtual-1
                        k = nk+1+iv
                        do i = 1, ni+1
                           do j = 1, nj+1
                              coor(i,j,k+1,l) = 2d0*coor(i,j,k  ,l) -
     +                                              coor(I,j,k-1,l)
                           end do
                        end do
                     end do
                  end do

               end if
            end if
         end do
      end do

c     --- handle edges

      do l = 1, 3

         w1 = 5d-1
         w2 = 5d-1
         w3 = 5d-1

c        --- edge face 1 / face 5

         do i = 0, 1-nvirtual, -1
            do k = 0, 1-nvirtual, -1
               do j = 1, nj+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 2 / face 5

         do i = ni+2, ni+1+nvirtual
            do k = 0, 1-nvirtual, -1
               do j = 1, nj+1
                  coor(i,j,k,l) =
     +              ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +              ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 3 / face 5

         do k = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               do i = 1, ni+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 4 / face 5

         do j = nj+2, nj+1+nvirtual
            do k = 0, 1-nvirtual, -1
               do i = 1, ni+1
                  coor(i,j,k,l) =
     +              ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +              ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 1 / face 6

         do i = 0, 1-nvirtual, -1
            do k = nk+2, nk+1+nvirtual
               do j = 1, nj+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 2 / face 6

         do i = ni+2, ni+1+nvirtual
            do k = nk+2, nk+1+nvirtual
               do j = 1, nj+1
                  coor(i,j,k,l) =
     +              ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +              ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 3 / face 6

         do k = nk+2, nk+1+nvirtual
            do j = 0, 1-nvirtual, -1
               do i = 1, ni+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 4 / face 6

         do k = nk+2, nk+1+nvirtual
            do j = nj+2, nj+1+nvirtual
               do i=1, ni+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- edge face 1 / face 3

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               do k = 1, nk+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2
               end do
            end do
         end do

c        --- edge face 2 / face 3

         do i = ni+2, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               do k = 1, nk+1
                  coor(i,j,k,l) =
     +              ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +              ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2
               end do
            end do
         end do

c        --- edge face 1 / face 4

         do i = 0, 1-nvirtual, -1
            do j = nj+2, nj+1+nvirtual
               do k = 1, nk+1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2
               end do
            end do
         end do

c        --- edge face 2 / face 4

         do i = ni+2, ni+1+nvirtual
            do j = nj+2, nj+1+nvirtual
               do k = 1, nk+1
                  coor(i,j,k,l) =
     +              ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +              ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2
               end do
            end do
         end do

      end do

c     --- handle vertices

      do l = 1, 3

         w1 = 1d0/3d0
         w2 = 1d0/3d0
         w3 = 1d0/3d0

c        --- vertex 1

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               do k = 0, 1-nvirtual, -1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 2

         do i = ni+2, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               do k = 0, 1-nvirtual, -1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 3

         do i = 0, 1-nvirtual, -1
            do j = nj+2, nj+1+nvirtual
               do k = 0, 1-nvirtual, -1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 4

         do i = ni+2, ni+1+nvirtual
            do j = nj+2, nj+1+nvirtual
               do k = 0, 1-nvirtual, -1
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k+1,l) - coor(i,j,k+2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 5

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               do k = nk+2, nk+1+nvirtual
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 6

         do i = ni+2, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               do k = nk+2, nk+1+nvirtual
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j+1,k,l) - coor(i,j+2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 7

         do i = 0, 1-nvirtual, -1
            do j = nj+2, nj+1+nvirtual
               do k = nk+2, nk+1+nvirtual
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i+1,j,k,l) - coor(i+2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

c        --- vertex 8

         do i = ni+2, ni+1+nvirtual
            do j = nj+2, nj+1+nvirtual
               do k = nk+2, nk+1+nvirtual
                  coor(i,j,k,l) =
     +               ( 2d0*coor(i-1,j,k,l) - coor(i-2,j,k,l) ) * w1 +
     +               ( 2d0*coor(i,j-1,k,l) - coor(i,j-2,k,l) ) * w2 +
     +               ( 2d0*coor(i,j,k-1,l) - coor(i,j,k-2,l) ) * w3
               end do
            end do
         end do

      end do
      end
