      subroutine isposor ( coor, work, ni, nj, iorien )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 08-12-1998 Kees K. check if the Jacobian
c                       has the same sign in all cells
c        version 1.0    date 25-06-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the block orientation and if necessary change directions
c
c **********************************************************************
c
c                       KEYWORDS
c
c   orientation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, iorien
      double precision coor((ni+1)*(nj+1),2), work((ni+1)*(nj+1),*)

c     coor     i/o   reserved
c     iorien    o    If 1 the sign of the Jacobian is positive and the
c                    directions are kept, otherwise -1 the directions are
c                    interchanged
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, ncount
      double precision a1(2), a2(2), sqrtg, signjac

c     a1        Covariant xi-1 base vector
c     a2        Covariant xi-2 base vector
c     i         General loop variable
c     j         General loop variable
c     ncount    Counter
c     signjac   Sign of Jacobian
c     sqrtg     The Jacobian in the center of a cell
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     720   The Jacobian does not have the same sign through the whole grid
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Compute sqrt(g) for first cell
c     If sqrt(g) < 0 then
c        iorien = -1
c        change sequence in coor
c     else
c        iorien = 1
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen( 'isposor' )
      ncount = 0
      do i = 1,ni
         do j = 1, nj
            a1(1) = 0.5d0*(coor(i+1,1)-coor(i,1)+
     +                      coor(ni+1+i+1,1)-coor(ni+1+i,1))
            a1(2) = 0.5d0*(coor(i+1,2)-coor(i,2)+
     +                      coor(ni+1+i+1,2)-coor(ni+1+i,2))
            a2(1) = 0.5d0*(coor(ni+1+i,1)-coor(i,1)+
     +                      coor(ni+1+i+1,1)-coor(ni+1+i,1))
            a2(2) = 0.5d0*(coor(ni+1+i,2)-coor(i,2)+
     +                      coor(ni+1+i+1,2)-coor(ni+1+i,2))
            sqrtg = a1(1)*a2(2) - a1(2)*a2(1)
            if ( ncount.gt.0 ) then
               if ( int(sign(1d0,sqrtg)).ne. int(signjac) ) then
                  call iserrsub ( 720, 0, 0, 0 )
                  call isstop
               end if
            end if
            signjac = sign(1d0,sqrtg)
            ncount = ncount + 1
         end do
      end do

      if ( signjac.gt.0d0 ) then

c     --- Standard orentation, no need to change

         iorien = 1

      else

c     --- Reversed orentation, interchange ksi1 and ksi2

         iorien = -1
         do i = 1, ni+1
            do j = 1, nj+1
               work((i-1)*(nj+1)+j,1) = coor((j-1)*(ni+1)+i,1)
               work((i-1)*(nj+1)+j,2) = coor((j-1)*(ni+1)+i,2)
            end do
         end do
         do i = 1, (ni+1)*(nj+1)
            coor(i,1) = work(i,1)
            coor(i,2) = work(i,2)
         end do

      end if

      call iserclos ( 'isposor' )
      end
