      subroutine isrdtop ( lensfread, nblocks, nfaces, maxnsubf,
     +                     nsubfaces, subfaces )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.2    date 04-03-1997  Kees K.: change for 3D
c        version 1.1    date 13-09-1995  Guus: allow for unformatted files
c        version 1.0    date 23-05-1994
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Read the block topology from the grid input file.
c
c **********************************************************************
c
c                       KEYWORDS
c
c   read_mesh
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer lensfread, maxnsubf, nblocks, nfaces
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     lensfread i    Number of integers to be read for each subface
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer faces(6), i, iblock, iface, isubface, j, inum, nbblock,
     +        nbface, oldfc, oldnb
      double precision dnumbr(10)

c     dnumbr    Work array to store a series numbers that have been read
c     faces     Array needed for adjusting the ordering of the
c               faces to the ISNaS ordering
c     i         General loop variable
c     iblock    Block sequence number
c     iface     Face number
c     inum      Indicates the number of numbers that have been read
c     isubface  Subface number
c     j         General loop variable
c     nbblock   Sequence number of a neighbouring block
c     nbface    Neighbouring face number
c     oldfc     Old face number
c     oldnb     Old neigbouring block number
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       5   Three-dimensional version not yet available
c    1606   Possible problems in the matrix building routines
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Initialize nsubfaces and subfaces array to zeroes
c
c     For iblock = 1, nblocks
c       For iface = 1, nfaces
c         Read nsubfaces
c         For isubfaces = 1, nsubfaces
c           Read lensfread integers from itopol
c         End
c       End
c     End
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- compute faces array

      faces(1) = ileft
      faces(2) = iright
      faces(3) = idown
      faces(4) = iup
      faces(5) = 5
      faces(6) = 6

c     --- if ( nfaces.gt.4 .and. nblocks.gt.1 ) then
c           call iseropen ( 'isrdtop' )
c           call iserrsub ( 5, 0, 0, 0 )
c           call isstop
c         end if

c         --- initialize nsubfaces and subfaces to zeroes

      do iblock = 1, nblocks
         do iface = 1, nfaces
            nsubfaces(iblock,iface) = 0
            do isubface = 1, maxnsubf
               do i = 1, lensfread
                  subfaces(iblock,iface,isubface,i) = 0
               end do
            end do
         end do
      end do

c     --- read itopol from file

      do iblock = 1, nblocks
         do j = 1, nfaces
            iface = faces(j)
            if ( iref10.gt.0 ) then
               call isreadrl ( dnumbr, 1, inum )
               nsubfaces(iblock,iface) = nint(dnumbr(1))
               do isubface = 1, nsubfaces(iblock, iface)
                  call isreadrl ( dnumbr, lensfread, inum )
                  do i = 1, lensfread
                     subfaces(iblock,iface,isubface,i) =
     +                        nint(dnumbr(i))
                  end do
                  if ( subfaces(iblock,iface,isubface,5).ne.0 )
     +               subfaces(iblock,iface,isubface,6) =
     +                 faces(subfaces(iblock,iface,isubface,6))
               end do
            else
               read ( irefre ) nsubfaces(iblock,iface)
               do isubface = 1, nsubfaces(iblock, iface)
                  read ( irefre )
     +               (subfaces(iblock,iface,isubface,i),
     +                        i = 1, lensfread)
                  if ( subfaces(iblock,iface,isubface,5).ne.0 )
     +               subfaces(iblock,iface,isubface,6) =
     +                 faces(subfaces(iblock,iface,isubface,6))
               end do
            end if
         end do
      end do

c     --- check the topology for two different neighbouring blocks on
c         that are adjacent the same side

      do iblock = 1, nblocks
         do iface = 1, nfaces
            oldnb = subfaces(iblock,iface,1,5)
            oldfc = subfaces(iblock,iface,1,6)
            do isubface = 2, nsubfaces(iblock,iface)
               nbblock = subfaces(iblock,iface,isubface,5)
               nbface  = subfaces(iblock,iface,isubface,6)
               if ( nbblock.eq.0 .or.
     +              oldnb.eq.0 .or.
     +              ( nbblock.eq.oldnb .and.
     +                nbface .eq.oldfc )) then

c              --- this situation is always handled correctly

               else

c              --- possible problems in the discretization

                  call iserrint(iface, 1)
                  call iserrint(iblock, 2)
                  call iserrint(isubface-1, 3)
                  call iserrint(isubface, 4)
                  call iserrint(oldnb, 5)
                  call iserrint(oldfc, 6)
                  call iserrint(nbblock, 7)
                  call iserrint(nbface, 8)
                  call iswarsub(1606, 8, 0, 0)

               end if
               oldnb = nbblock
               oldfc = nbface
            end do
         end do
      end do

      end
