      subroutine isreadcf ( iincof, rincof, ncoefs, ntrans, dnumbr,
     +                      ndim, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.0    date 19-05-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the coefficients from the intermediate
c    file and store in the arrays iincof and rincof for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    coefficients
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ncoefs, ntrans, iincof(ncoefs,*), icoupled, ndim
      double precision dnumbr(*), rincof(ncoefs,*)

c     dnumbr         Work array to store a series numbers that have been read
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     iincof    o    reserved
c     ncoefs    i    Number of coefficients
c     ndim      i    Dimension of space (2 or 3)
c     ntrans    i    Number of transport equations plus number of turbulence
c                    equations
c     rincof    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, icoef, inum, j

c     i         General loop variable
c     icoef     Sequence number of coefficient
c     inum      Indicates the number of numbers that have been read
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     For i := 1(1) ntrans+1
c        iincof(j), j = 1, ncoefs
c        rincof(j), j = 1, ncoefs
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     503   iincof(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For i := 1 (1) ntrans+1
c        Read iincof
c        Check iincof
c        Read rincof
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadcf' )

      do 200 i = 1, ntrans+1+icoupled*(ndim-1)

c     --- Read iincof

         call isreadrl ( dnumbr, ncoefs, inum )
         do 100 j = 1, ncoefs
            icoef = nint(dnumbr(j))
            if ( icoef.ne.-1 .and. icoef.lt.-1 ) then
               call iserrint ( j, 1 )
               call iserrint ( i, 2 )
               call iserrint ( icoef, 3 )
               call iserrsub ( 503, 3, 0, 0 )
            end if
            iincof(j,i) = icoef
100      continue

c        --- Read rincof

         call isreadrl ( rincof(1,i), ncoefs, inum )

200   continue

      call iserclos ( 'isreadcf' )
      end
