      subroutine isreadrl ( dnumbr, maxnum, inum )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   16-05-94 (Subroutine extracted from the SEPRAN
c                                        subroutine READSR version 1.1)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   read a series of numbers
c   The numbers may be alternated with texts, which are skipped
c   The reading of numbers is stopped if
c
c    an end of file is found
c    maxnum numbers have been read
c
c   End of record is considered as a separator
c   After reading the numbers, icolum points to the next position to be read
c
c ********************************************************************
c
c                      KEYWORDS
c
c   read
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer maxnum, inum
      double precision dnumbr(maxnum)

c     dnumbr  o  Array to be filled with the numbers
c     inum    o  Number of numbers that have been actually read
c     maxnum  i  Maximum number of numbers to be read
c ********************************************************************
c
c                      COMMON BLOCKS

      include 'IScommon/cseprd'
      include 'IScommon/cisnas'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      logical ready
      integer i, kcolsv, ioutpsv

c    i        General loop variable
c    ioutpsv  Help parameter to save the old value of ioutpt
c    kcolsv   Help parameter to save the old value of kcolum
c    ready    Indicates if a while loop must be terminated (true) or not (false)
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c     500   End of file found
c ********************************************************************
c
c                    PSEUDO CODE
c
c   Clear array dnumbr
c   ready  := false
c   inum   := 0
c   ispecl := 0
c   istop  := 2
c
c   while ( not ready ) do
c      Read next item (ISASKNXT)
c      if ( ityprd = 2 or 3 ) then
c         inum := inum+1
c      else if ( ityprd = 0 ) then
c         end of file found: error
c      if ( inum = maxnum ) then
c         ready := true
c ======================================================================
c
c     --- Make some initializations

      call iseropen ( 'isreadrl' )

c     --- Clear array dnumbr

      do 100 i = 1, maxnum
         dnumbr(i) = 0
100   continue

      ready  = .false.
      inum   = 0
      ispecl = 0
      istop  = 2
      kcolsv = kcolum
      ioutpsv = ioutpt
      ioutpt = -1

200   if ( .not. ready ) then

c     --- While loop
c         Read next item

         call isasknxt

         if ( ityprd.eq.2 .or. ityprd.eq.3 ) then

c        --- ityprd = 2 or 3, number found

            inum = inum + 1
            dnumbr(inum) = realrd

         else if ( ityprd.eq.0 ) then

c        --- ityprd = 0, error end of file found

            call iserrint ( irefre, 1 )
            call iserrsub ( 500, 1, 0, 0 )
            call isstop

         end if

         if ( inum.ge.maxnum ) then

c        --- inum = maxnum, stop reading

            ready = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )
      ioutpt = ioutpsv

      call iserclos ( 'isreadrl' )

      end
