      subroutine isaccma ( nmbsol, rhsd, x, work, iequa,
     +                     isiinput, isrinput, iscoor  , issolut ,
     +                     iscoefs , iseddy  , isinfbnd, isbndcon,
     +                     ismatrix, isgmat  , isrhside, isgeom  ,
     +                     isitopol, isdensup, iadmin             )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.0  date 12-03-1999 Kees V. revision of ishgmr05
c     version      1.8  date 23-11-1998 Jason: removed iaction
c     version      1.7  date 15-05-1998 Duncan: new call to ishdd
c     version      1.6  date 13-10-1997 Charles: uncoupled allowed
c     version      1.5  date 06-08-1997 Guus: return in case of error
c     version      1.4  date 14-04-1997 Kees K call of ishdd changed
c     version      1.3  date 06-04-1995 Kees K isitopol not an array
c     version      1.2  date 06-02-1995 Michiel K:array raction removed
c     version      1.1  date 03-11-1994 Kees K coefs and geom added
c     version      1.0  date 20-05-1995 Erik: initial revision
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Performs a matrix vector product for the multiblock algorithm with
c     accurate solution of the subdomains.
c **********************************************************************
c
c                       KEYWORDS
c
c     domain_decompostion
c     linear_solver
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nmbsol, iequa, iadmin(*), isbndcon(*), iscoor(*),
     +                 iseddy(*), isgmat(*), isiinput(*), isinfbnd(*),
     +                 isitopol, ismatrix(*), isrhside(*), isrinput(*),
     +                 issolut(*), iscoefs(*), isgeom(*), isdensup(*)
      double precision x(nmbsol), work(nmbsol), rhsd(nmbsol)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nmbsol    i    Length of the multi-block solution vector
c     rhsd      i    Vector containing the right-hand side vector of the
c                    system of equations.
c     work      i    Work array
c     x         i    The original vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ilev, ipiinput, itrans, nblocks, ntimlv, ndim,
     +        icoupled, iseqpres

c     i         General loop variable
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ilev      Time level of the multi-block transport of information
c     ipiinput  Starting address of array iinput in ibuffr
c     iseqpres  Sequence number of pressure equation
c     itrans    reserved
c     nblocks   Number of blocks
c     ndim      Dimension of space (2 or 3)
c     ntimlv    Number of time levels
c **********************************************************************
c
c                       I/O
c
c      None
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISACTIVE  Activate old permanent array
c     ISDDREL   Copy the virtual unknowns of all blocks to a single
c               vector
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISHDD     Perform a single domain decomposition iteration on the
c               host.
c     ISINGETP  Get the pointer corresponding to an integer array
c     ISRELDD   Copy an array of relevant variables to the subdomain
c               solut arrays
      integer   isingetp
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Put the initial guess x into the subdomain solut arrays
c
c     Perform a single domain decomposition iteration
c
c     Put the virtual unknowns after the iteration into the vector work
c
c     This vector contains the new iterate: work = x + (rhsd-A*x)
c
c     Compute A*x = x+rhsd-work
c
c     References:
c
c     E. Brakkee
c     Domain decomposition for the incompressible Navier-Stokes equations
c     Thesis, Delft University of Technology, 1996
c
c     E. Brakkee and C. Vuik and P. Wesseling
c     Domain decomposition for the incompressible Navier-Stokes
c     equations: solving subdomain problems accurately and inaccurately
c     Int. J. for Num. Meth. Fluids, 26, pp.1217-1237, 1998
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen ( 'isaccma' )

      call isactive(isiinput(1))

      ipiinput = isingetp(isiinput(1))
      ndim     = ibuffr(ipiinput + 57 - 1)
      nblocks  = ibuffr(ipiinput + 73 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)

      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if
      ilev     = ntimlv

c     --- compute itrans, ilev

      if ( iequa.lt.iseqpres ) then
         itrans = 1
      else
         itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
      end if

c     --- copy x to subdomain solut arrays

      call isreldd(itrans, ilev, isiinput, issolut, isitopol, x)
      if ( nerror.ne.0 ) go to 1000

c     --- perform a single domain decomposition iteration

      call ishdd( ndim, nblocks , iequa, isiinput, isrinput,
     +            iscoor, issolut, iscoefs, iseddy, isinfbnd,
     +            isbndcon, ismatrix, isgmat, isrhside, isgeom,
     +            isitopol, isdensup, iadmin )
      if ( nerror.ne.0 ) go to 1000

c     --- copy subdomain solut arrays to work

      call isddrel(itrans, ilev, isiinput, issolut, isitopol, work)

c     --- work:= x+rhsd-work

      do i = 1, nmbsol
         work(i) = x(i) - work(i) + rhsd(i)
      end do

1000  call iserclos ( 'isaccma' )
      end
