      function isblockor(ni, nj, ndim, coor)
c=====================================================================
c
c        programmer   Erik Brakkee
c        version 1.0    date 21-07-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c   Compute the absolute orientation of a block in 2-D.
c   The return value is 1 if the normal anti-clockwise directions
c   are used (sqrt(g) > 0 everywhere) and
c   it is -1 if clockwise directions are used (sqrt(g) < 0 everywhere)
c
c***********************************************************************
c
c                  KEYWORDS
c
c   topology
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none

      integer isblockor, ndim, ni, nj

      double precision coor(1:ni+1, 1:nj+1, 1:ndim)

c
c     coor      i    reserved, local coordinate array of block iblock
c                    without virtual points (exists only in input phase)
c     isblockor o    computed absolute orientation.
c                    The return value is 1 if the normal anti-clockwise
c                    directions are used (sqrt(g) > 0 everywhere) and
c                    it is -1 if clockwise directions are used (sqrt(g) < 0
c                    everywhere)
c     ndim      i    dimension of the problem
c     ni        i    i-dimension of a block
c     nj        i    j-dimension of a block
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisnas'
c***********************************************************************
c
c     LOCAL PARAMETERS
c

      integer i, j, k

      double precision a1(2), a2(2), sqrtg

c
c     a1        Covariant base vector a_1
c     a2        Covariant base vector a_2
c     i         Grid i-index
c     j         Grid j-index
c     k         Loop variable
c     sqrtg     jacobian of the coordinate transformation
c
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c     CALLED SUBROUTINES
c
c     None.
c
c**********************************************************************
c
c                   ERROR MESSAGES :
c
c        -
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c
c=======================================================================
c

c     --- select an arbitrary cell of the grid

      i = 1
      j = 1

c     --- compute base vectors

      do k = 1, 2

         a1(k) = 5d-1*( coor(i+1,j+1,k) - coor(i,j+1,k) +
     +                  coor(i+1,j,k)   - coor(i,j,k) )
         a2(k) = 5d-1*( coor(i+1,j+1,k) - coor(i+1,j,k) +
     +                  coor(i,j+1,k)   - coor(i,j,k) )
      enddo

c     --- compute jacobian of the coordinate transformation

      sqrtg = a1(1)*a2(2) - a2(1)*a1(2)

c     --- compute absolute orientation

      if ( sqrtg .gt. 0d0 ) then

         isblockor = 1

      else

         isblockor = -1

      endif

      end








