      subroutine isglom01 ( ni, nj, ndegfd, ndim, ntimlv, nvirtual,
     +                      matrix, solut, outvec, idiscret )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.0  18-05-1998 Charles Bilinear discretization
c                                  Remove nk
c     version      1.1  13-01-1997 Kees K:  wesbeek discretization
c     version      1.0  04-01-1995 Erik:  initial revision
c
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute momentum matrix-vector product, using a matrix with
c     virtual coefficients and a vector with virtual unknowns.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndegfd, ndim, ni, nj, ntimlv, nvirtual, idiscret
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)
      double precision matrix(1:ni+1, 1:nj+1, 1:ndim, *)
      double precision outvec(1:ni+1, 1:nj+1, 1:ndim)

c     idiscret   i  Type of discretization
c     matrix     i  reserved
c     ndegfd     i  Number of degrees of freedom
c     ndim       i  Dimension of space (2 or 3)
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     ntimlv     i  Number of time levels
c     nvirtual   i  Number of rows of virtual cells
c     outvec     o  Output vector
c     solut      i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, idim

c     i         General loop variable
c     idim      Loop variable used for dimension
c     j         General loop variable
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      idim = 1
      j    = nj+1
      do i = 1, ni+1
         outvec(i,j,idim) = 0d0
      end do

      idim = 2
      i = ni+1
      do j = 1, nj+1
         outvec(i,j,idim) = 0d0
      end do

c     --- handle V1 component

      idim = 1
      do j = 1, nj
         do i = 1, ni+1

             if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c            --- Wesbeek discretization

            outvec(i,j,idim) =
     +         matrix(i,j,idim,1)  * solut(i,j,idim,ntimlv) +
     +         matrix(i,j,idim,2)  * solut(i-1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,3)  * solut(i,   j-1, idim,ntimlv) +
     +         matrix(i,j,idim,4)  * solut(i+1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,5)  * solut(i-1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,6)  * solut(i+1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,7)  * solut(i-1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,8)  * solut(i,   j+1, idim,ntimlv) +
     +         matrix(i,j,idim,9)  * solut(i+1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,10) * solut(i-1, j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,11) * solut(i,   j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,12) * solut(i-1, j+1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,13) * solut(i,   j+1, 3-idim,ntimlv)
            outvec(i,j,idim) = outvec(i,j,idim) +
     +         matrix(i,j,idim,14) * solut(i-1, j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,15) * solut(i,   j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,16) * solut(i-1, j+2, 3-idim,ntimlv) +
     +         matrix(i,j,idim,17) * solut(i,   j+2, 3-idim,ntimlv) +
     +         matrix(i,j,idim,18) * solut(i-2,   j, 3-idim,ntimlv) +
     +         matrix(i,j,idim,19) * solut(i+1,   j, 3-idim,ntimlv) +
     +         matrix(i,j,idim,20) * solut(i-2, j+1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,21) * solut(i+1, j+1, 3-idim,ntimlv)

             else

            outvec(i,j,idim) =
     +         matrix(i,j,idim,1)  * solut(i,j,idim,ntimlv) +
     +         matrix(i,j,idim,2)  * solut(i-1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,3)  * solut(i,   j-1, idim,ntimlv) +
     +         matrix(i,j,idim,4)  * solut(i+1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,5)  * solut(i-1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,6)  * solut(i+1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,7)  * solut(i-1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,8)  * solut(i,   j+1, idim,ntimlv) +
     +         matrix(i,j,idim,9)  * solut(i+1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,10) * solut(i-1, j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,11) * solut(i,   j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,12) * solut(i-1, j+1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,13) * solut(i,   j+1, 3-idim,ntimlv)

            end if

         end do
      end do

c     --- handle V2 component

      idim = 2
      do j = 1, nj+1
         do i = 1, ni

             if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c            --- Wesbeek discretization

            outvec(i,j,idim) =
     +         matrix(i,j,idim,1)  * solut(i,j,idim,ntimlv) +
     +         matrix(i,j,idim,2)  * solut(i-1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,3)  * solut(i,   j-1, idim,ntimlv) +
     +         matrix(i,j,idim,4)  * solut(i+1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,5)  * solut(i-1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,6)  * solut(i+1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,7)  * solut(i-1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,8)  * solut(i,   j+1, idim,ntimlv) +
     +         matrix(i,j,idim,9)  * solut(i+1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,10) * solut(i,   j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,11) * solut(i+1, j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,12) * solut(i,   j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,13) * solut(i+1, j,   3-idim,ntimlv)
            outvec(i,j,idim) = outvec(i,j,idim) +
     +         matrix(i,j,idim,14) * solut(i-1, j-1,   3-idim,ntimlv) +
     +         matrix(i,j,idim,15) * solut(i+2, j-1,   3-idim,ntimlv) +
     +         matrix(i,j,idim,16) * solut(i-1, j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,17) * solut(i+2, j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,18) * solut(i,   j-2,   3-idim,ntimlv) +
     +         matrix(i,j,idim,19) * solut(i+1, j-2,   3-idim,ntimlv) +
     +         matrix(i,j,idim,20) * solut(i,   j+1,   3-idim,ntimlv) +
     +         matrix(i,j,idim,21) * solut(i+1, j+1,   3-idim,ntimlv)

            else

            outvec(i,j,idim) =
     +         matrix(i,j,idim,1)  * solut(i,j,idim,ntimlv) +
     +         matrix(i,j,idim,2)  * solut(i-1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,3)  * solut(i,   j-1, idim,ntimlv) +
     +         matrix(i,j,idim,4)  * solut(i+1, j-1, idim,ntimlv) +
     +         matrix(i,j,idim,5)  * solut(i-1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,6)  * solut(i+1, j,   idim,ntimlv) +
     +         matrix(i,j,idim,7)  * solut(i-1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,8)  * solut(i,   j+1, idim,ntimlv) +
     +         matrix(i,j,idim,9)  * solut(i+1, j+1, idim,ntimlv) +
     +         matrix(i,j,idim,10) * solut(i,   j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,11) * solut(i+1, j-1, 3-idim,ntimlv) +
     +         matrix(i,j,idim,12) * solut(i,   j,   3-idim,ntimlv) +
     +         matrix(i,j,idim,13) * solut(i+1, j,   3-idim,ntimlv)

            end if

         end do
      end do

      end
