      subroutine isgloma ( nblocks, iequa, isiinput, issolut, ismatrix,
     +                     isitopol, iadmin, invec, outvec )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.3  18-05-1998  Guus: new call to isglom01
c     version      1.2  25-04-1997  Kees K.: 3D
c     version      1.1  13-01-1997  Kees K.: wesbeek discretization
c     version      1.0  04-01-1995  Erik: initial revision
c
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computation of the global matrix vector product.
c     IMPORTANT!!!
c        The vector outvec in the subroutine parameterlist may not be
c        governed by memory management routines since this routine also
c        uses memory management.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, iequa
      integer isitopol
      integer iadmin(1:nblocks), isiinput(1:nblocks),
     +        ismatrix(1:nblocks), issolut(1:nblocks)
      double precision invec(*), outvec(*)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     invec     i    Input vector
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c     outvec    o    Output vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ipmatrix, ipsolut, istartv
      integer ndegfd, ndim, ni, nj, nk, ntimlv, nvirtual,
     +        ipiindsc, idiscret

c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipmatrix  Starting address of array matrix in buffer
c     ipsolut   Starting address of array solut in buffer
c     istartv   First free position in a vector
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISACTIVE  Activate old permanent array
c     ISDDSOL   copy 1-D vector to inner regions of all subdomain
c               solution vectors
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISGLOM01  momentum matrix-vector product for a solut array with
c               virtual points
c     ISGLOM02  pressure or transport matrix-vector product for a solut
c               array with virtual points
c     ISGLOM31  momentum matrix-vector product for a solut array with
c               virtual points 3D
c     ISGLOM32  pressure or transport matrix-vector product for a solut
c               array with virtual points 3D
c     ISINGETP  get the pointer corresponding to an integer array
      integer isdpgetp, isingetp
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c     istart = 1
c
c     For iblock:= 1(1)nblocks
c
c        matrix_vector_product(matrix(iblock), solut(iblock), y(istart))
c
c        istart = istart + (ni+1)*(nj+1)*(nk+1)
c
c **********************************************************************
c
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call isddsol( nblocks, iequa, isiinput, issolut,
     +              isitopol, iadmin, invec )

      istartv = 1

      do iblock = 1, nblocks
      
      if ( iadmin(iblock).eq.inodenr ) then 

         call isactive(isiinput(iblock))
         call isactive(issolut(iblock))
         call isactive(ismatrix(iblock))

         ipiinput = isingetp(isiinput(iblock))
         ipsolut  = isdpgetp(issolut(iblock))
         ipmatrix = isdpgetp(ismatrix(iblock))

         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         ndim     = ibuffr(ipiinput + 57 - 1)
         ndegfd   = ibuffr(ipiinput + 55 - 1)
         ntimlv   = ibuffr(ipiinput + 64 - 1)
         nvirtual = ibuffr(ipiinput + 54 - 1)

         ipiindsc     = ibuffr(ipiinput-1+6)
         idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

         if ( iequa.eq.1 ) then

c        --- momentum equations

            if ( ndim.eq.2 ) then

               call isglom01 ( ni, nj, ndegfd, ndim, ntimlv,
     +                         nvirtual,buffer(ipmatrix),
     +                         buffer(ipsolut), outvec(istartv),
     +                         idiscret )
            else
               call isglom31 ( ni, nj, nk, ndegfd, ndim, ntimlv,
     +                         nvirtual,buffer(ipmatrix),
     +                         buffer(ipsolut), outvec(istartv) )
            end if

         else

c        --- pressure or transport

            if ( ndim.eq.2 ) then

               call isglom02 ( ni, nj, nk, ndegfd, ntimlv, nvirtual,
     +                        iequa,
     +                        buffer(ipmatrix), buffer(ipsolut),
     +                        outvec(istartv) )
            else
               call isglom32 ( ni, nj, nk, ndegfd, ntimlv, nvirtual,
     +                        iequa,
     +                        buffer(ipmatrix), buffer(ipsolut),
     +                        outvec(istartv) )
            end if

         end if

         if ( iequa.eq.1 ) then

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)*ndim

         else

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)

         end if
         
         end if

      end do

      end
