      subroutine ismbex02 ( itrans, ilev, ndegfd, ntimlv, nvirtual,
     +                      nblocks, nfaces, maxnsubf, idiscret,
     +                      isolv, isiinput, issolut, iadmin,
     +                      nsubfaces, subfaces, smsg, maxmsgln,
     +                      mean )
c ======================================================================
c
c     programmer   Jason Frank
c     version      1.0  28-10-1999  Jason:  revision of ismblk02
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport certain variables (itrans,ilev) to neighboring blocks
c     (on another processor if necessary.)
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ilev, itrans, maxnsubf, idiscret, isolv,
     +      nblocks, ndegfd, nfaces, ntimlv, nvirtual, maxmsgln
      integer iadmin(1:nblocks)
      integer isiinput(1:nblocks)
      integer issolut(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      logical mean

c     iadmin    i  reserved
c     idiscret  i  Type of discretization
c     ilev      i  Time level of the multi-block transport of information
c     isiinput  i  Array of sequence numbers (nblocks) of array iinput
c     isolv     i  Called by solver for periodic b.c. isolv=1 else isolv=0
c     issolut   i  Array of sequence numbers (nblocks) of array solut
c     itrans    i  reserved
c     maxmsgln  i  Maximum submessage length
c     maxnsubf  i  Maximum number of subfaces encountered on all subfaces
c     mean      i  If true, normal fluxes will be averaged on the interface
c     nblocks   i  Number of blocks
c     ndegfd    i  Number of degrees of freedom
c     nfaces    i  Number of faces
c     nsubfaces i  Array containing the number of subfaces of each face,
c                  see array itopol for an extended description
c     ntimlv    i  Number of time levels
c     nvirtual  i  Number of rows of virtual cells
c     smsg     i/o reserved
c     subfaces  i reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ibto, iface, ifto, ipiinpfr, ipiinpto,
     +        ipsolfr, ipsolto, isenreq, isfto, isubface,
     +        lenmsg, ni, nito, nj, njto, nk, nkto

c     iblock     Source block
c     ibto       Target block
c     iface      Face number
c     ifto       Target face
c     ipiinpfr   Starting address of source array iinput in ibuffr
c     ipiinpto   Starting address of target array iinput in ibuffr
c     ipsolfr    Starting address of source array solut in buffr
c     ipsolto    Starting address of target array solut in buffr
c     isenreq    MPI send request
c     isfto      Target subface
c     isubface   Subface number
c     lenmsg     Actual length of sent message
c     ni         I-dimension of source
c     nito       I-dimension of target
c     nj         J-dimension of source
c     njto       J-dimension of target
c     nk         K-dimension of source
c     nkto       K-dimension of target
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddget, isdpgetp, isingetp, isddput, isddptbc,
     +        isddputmn

c     ISACTIVE  Activate a permanent array
c     ISDDGET   Get virtual unknowns from a certain subface of a certain
c               block and put them into an intermediate representation
c     ISDDPTBC  Copy normal fluxes to a certain subface of a certain
c               block
c     ISDDPUT   Copy virtual unknowns to a certain subface of a certain
c               block
c     ISDDPUTMN Copy normal fluxes to a certain subface of a certain
c               block, using the mean
c     ISDPGETP  Get the address of a double precision array in buffer
c     ISINGETP  Get the address of an integer array in ibuffr
c     ISRECMPI  Receive a message from another process using MPI
c     ISSENMPI  Send a message to another process using MPI
c     ISWAITALL Wait for sent message to be received (MPI)
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Loop over blocks
c        Loop over faces
c           Loop over subfaces
c              if this subface interfaces with another block then
c                 determine "from" and "to" block geometries
c                 if "from" block is local then
c                    get data to be transferred
c                    if "to" block is not local then
c                       send data to the responsible process
c                    else
c                       put data in proper array
c                 else if "to" block is local then
c                    receive data from the responsible process
c                    put data in proper array
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      do iblock = 1, nblocks

         do iface = 1, nfaces

            do isubface = 1, nsubfaces (iblock, iface)

               ibto = subfaces(iblock, iface, isubface, 5)

               if ( ibto.ne.0 ) then

c              --- iblock and ibto share a common interface

                  ifto = subfaces(iblock, iface, isubface, 6)
                  isfto = subfaces(iblock, iface, isubface, 7)

c                 --- "from" block dimensions

                  if (isiinput(iblock).ne.0) then

                     call isactive(issolut(iblock))
                     call isactive(isiinput(iblock))
                     ipsolfr  = isdpgetp( issolut(iblock))
                     ipiinpfr = isingetp( isiinput(iblock))
                     ni  = ibuffr(ipiinpfr + 51 - 1)
                     nj  = ibuffr(ipiinpfr + 52 - 1)
                     nk  = ibuffr(ipiinpfr + 53 - 1)

                  end if

c                 --- "to" block dimensions

                  if (isiinput(ibto).ne.0) then

                     call isactive(issolut(ibto))
                     call isactive(isiinput(ibto))
                     ipsolto  = isdpgetp( issolut(ibto))
                     ipiinpto = isingetp( isiinput(ibto))
                     nito  = ibuffr(ipiinpto + 51 - 1)
                     njto  = ibuffr(ipiinpto + 52 - 1)
                     nkto  = ibuffr(ipiinpto + 53 - 1)

                  end if

                  if (isolv.ne.0) then

c                 --- periodic boundary conditions

                     ibto = iblock
                     ifto = iface
                     isfto = isubface

                     nito = ni
                     njto = nj
                     nkto = nk
                     ipsolto = ipsolfr

                  end if

                  if ( iadmin(iblock).eq.inodenr ) then

c                 --- get data from local array

                     lenmsg = isddget(iblock, iface, isubface, itrans,
     +                         ilev, ni, nj, nk, ndegfd, ntimlv,
     +                         nvirtual, nblocks, nfaces,  maxnsubf,
     +                         idiscret, isolv, buffer(ipsolfr),
     +                         nsubfaces, subfaces, smsg )

                     if ( iadmin(ibto).ne.inodenr) then

c                    --- send data with MPI

                        call issenmpi ( smsg, lenmsg, ISDOUBLE,
     +                           iadmin(ibto), ICVCDATA, isenreq )
                        call iswaitall ( 1, isenreq )

                        if ( mean ) then

                           call isrecmpi ( smsg, lenmsg, ISDOUBLE,
     +                           iadmin(ibto), ICVCDATA )

                           lenmsg = isddptbc(iblock,iface, isubface,
     +                          ilev, ni, nj, nk, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolfr),
     +                          nsubfaces, subfaces, smsg)

                        end if

                     else

c                    --- put data directly in local array

                        if ( .not. mean ) then

                           lenmsg = isddput(ibto, ifto,  isfto, itrans,
     +                          ilev, nito, njto, nkto, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolto),
     +                          nsubfaces, subfaces, smsg)
                        else

                           lenmsg = isddputmn(ibto,ifto, isfto,
     +                          ilev, nito, njto, nkto, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolto),
     +                          nsubfaces, subfaces, smsg)

                           lenmsg = isddptbc(iblock,iface, isubface,
     +                          ilev, ni, nj, nk, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolfr),
     +                          nsubfaces, subfaces, smsg)

                        end if

                     end if

                  else if ( iadmin(ibto).eq.inodenr ) then

c                 --- receive data with MPI

                        lenmsg = maxmsgln
                        call isrecmpi ( smsg, lenmsg, ISDOUBLE,
     +                           iadmin(iblock), ICVCDATA )

c                       --- put data in local array

                        if (.not. mean ) then

                           lenmsg = isddput(ibto, ifto,  isfto, itrans,
     +                          ilev, nito, njto, nkto, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolto),
     +                          nsubfaces, subfaces, smsg)

                        else

                           lenmsg = isddputmn(ibto,ifto, isfto,
     +                          ilev, nito, njto, nkto, ndegfd, ntimlv,
     +                          nvirtual, nblocks, nfaces, maxnsubf,
     +                          idiscret, isolv, buffer(ipsolto),
     +                          nsubfaces, subfaces, smsg)

                           call issenmpi ( smsg, lenmsg, ISDOUBLE,
     +                             iadmin(iblock), ICVCDATA, isenreq )
                           call iswaitall (1, isenreq)

                        end if

                  end if

               end if

            end do

         end do

      end do

      end
