      subroutine isrestore(iequa, matonly, iinput, infmm, infmt,
     +                     matrix, matvirt, rhside, rhsvirt,
     +                     work, lmatvir )
c ======================================================================
c
c     programmer Erik Brakkee
c
c     version    1.5   date 24-08-1998   Kees K. work array added
c     version    1.4   date 09-12-1997   Kees V. efficient memory use
c                                        for the virtual matrix elements
c     version    1.3   date 13-10-1997   Charles: uncoupled allowed
c     version    1.2   date 08-01-1997   Kees K.: idiscret added
c     version    1.1   date 28-03-1995   Erik: copying of rhs only if matonly
c                                 is false.
c     version    1.0   date 22-02-1995   Erik: initial revision adapted from
c                                 isuprhs
c
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Restore the matrix and right-hand side.
c     The virtual coefficients of the matrix are copied back into the
c     matrix. The positions of the right-hand side that are perturbed
c     by linear combinations of virtual unknowns are restored to their
c     original values.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, iinput(*), infmm(*), infmt(*), lmatvir
      double precision matrix(*)
      double precision matvirt(*)
      double precision rhside(*)
      double precision rhsvirt(*)
      double precision work(*)
      logical matonly

c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     infmm     i    Integer information for Momentum matrix
c     infmt     i    Integer information for Pressure and Transport matrix
c     lmatvir   i    Length of the array of virtual matrix elements
c     matonly   i    If true, only copy matrix elements and leave
c                    right-hand side alone.
c     matrix   i/o   reserved
c     matvirt   o    Storage for the virtual coefficients of a matrix
c     rhside   i/o   reserved
c     rhsvirt   o    Storage for the elements of the right-hand side
c                    affected by virtual unknowns of neighbouring blocks
c     work     i/o   Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nk, ndim, nvirtual, nconct, ldegfd, npointsnv,
     +        npointsv, ipiindsc, idiscret, iseqpres, icoupled

c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     iseqpres  Sequence number of pressure equation
c     ldegfd    Number of degrees of freedom in the matrix
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsnv Number of points in the mesh without virtual points
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISRESTO01 Restore matrix and right-hand side
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isrestore')

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      ndim     = iinput(57)
      nvirtual = iinput(54)
      icoupled = iinput(76)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      ipiindsc = iinput(6)
      idiscret = iinput(ipiindsc+1)

      npointsnv = (ni+1)*(nj+1)*(nk+1)
      npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if (ndim.eq.3) npointsv = npointsv*(nk+1+2*nvirtual)

      if ( iequa.lt.iseqpres ) then

c     --- momentum matrix

         nconct = infmm(6)
         if ( icoupled.eq.0 ) then
            ldegfd = ndim
         else
            ldegfd = 1
         end if

         call isresto01( iequa, ndim, ni, nj, nk,
     +                   npointsnv, idiscret, ldegfd, nconct,
     +                   matonly,matrix, matvirt,
     +                   rhside, rhsvirt, infmm(infmm(12)), work,
     +                   icoupled, lmatvir )

      else

c     --- pressure or transport matrix

         nconct = infmt(6)
         ldegfd = 1

         call isresto01( iequa, ndim, ni, nj, nk,
     +                   npointsnv, idiscret, ldegfd, nconct,
     +                   matonly,matrix, matvirt,
     +                   rhside, rhsvirt, infmt(infmt(12)), work,
     +                   icoupled, lmatvir )

      end if

      call iserclos('isrestore')

      return
      end
