      subroutine issnnd02(itrans, ilev, nblocks, nfaces, maxnsubf,
     +                    idiscret, isiinput, istraq, iadmin,
     +                    nsubfaces, subfaces, smsg)
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.5  07-10-1999  Jason:  added send-requests
c     version      1.4  23-11-1998  Jason:  issenmpi version
c     version      1.3  21-10-1998  Kees K: isolv added for isddget
c     version      1.2  12-12-1996  Kees K: idiscret added
c     version      1.1  22-11-1994  Erik: extended for coefs
c                                       changed solut into traq
c     version      1.0  03-05-1994
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Send certain virtual unknowns (itrans,ilev) from the local blocks
c     to the host using issend.
c     On entry, the message
c     to the host has already been opened. At return, it is still
c     open and can be appended to.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), ilev, isiinput(*), istraq(*), itrans,
     +        maxnsubf, nblocks, nfaces, idiscret
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iadmin    i    reserved
c     idiscret  i    Type of discretization
c     ilev      i    Time level of the multi-block transport of information
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     smsg      i    reserved
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iarray(5), iblock, iface, ipiinput,
     +      iptraq, isubface, lengmes, lensmsg, ntraq1,
     +      ni, nj, nk, ntraq2, nvirtual, isolv,
     +      ireq (nblocks*nfaces*maxnsubf), isndcnt

c     iarray    Integer array for issend and isreceiv
c     iblock    Block sequence number
c     iface     Face number
c     ipiinput  Starting address of array iinput in ibuffr
c     iptraq    Starting address of array traq in buffer
c     ireq      array of send-requests
c     isndcnt   send-request counter
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     isubface  Subface number
c     lengmes   Length of the message
c     lensmsg   Upper bound for the length of a submessage array
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddget, isdpgetp, isingetp

c     ISDDGET        Get virtual unknowns from a certain subface of a certain
c                    block and put them into an intermediate representation
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISINGETP       get the pointer corresponding to an integer array
c     ISSENMPI       Send data to node or host
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     DO for all internal subfaces of the local blocks
c        Get virtual unknowns from this segment
c        Send virtual unknowns to the host
c     ENDDO
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      smsg(1) = 0
      iarray(1) = 0
      isndcnt = 0

      do iblock = 1, nblocks
      if ( iadmin(iblock).eq.inodenr ) then

         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         if ( itrans.lt.0 ) then

c        --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

            ntraq1    = ibuffr(ipiinput + 66 - 1)
            ntraq2    = 1

         else

c        --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

            ntraq1    = ibuffr(ipiinput + 55 - 1)
            ntraq2    = ibuffr(ipiinput + 64 - 1)

         end if
         nvirtual = ibuffr(Ipiinput + 54 - 1)

         iptraq  = isdpgetp(istraq(iblock))

         do iface = 1, nfaces

            do isubface = 1, nsubfaces(iblock, iface)

               if ( subfaces(iblock,iface,isubface,5).ne.0 ) then

                  iarray(1) = iblock
                  iarray(2) = iface
                  iarray(3) = isubface
                  iarray(4) = itrans
                  iarray(5) = ilev

c                 --- send informative part of submessage

                  lengmes = 5
                  isndcnt = isndcnt + 1
                  call issenmpi (iarray, lengmes, ISINT, ISHOST,
     +               ICVCINFO, ireq(isndcnt) )

c                 --- send data part

                  isolv = 0
                  lensmsg = isddget(iblock, iface, isubface,
     +                        itrans, ilev, ni, nj, nk,
     +                        ntraq1, ntraq2, nvirtual,
     +                        nblocks, nfaces, maxnsubf, idiscret,
     +                        isolv, buffer(iptraq), nsubfaces,
     +                        subfaces, smsg)

                  lengmes = lensmsg
                  isndcnt = isndcnt + 1
                  call issenmpi (smsg, lengmes, ISDOUBLE, ISHOST,
     +               ICVCDATA, ireq(isndcnt))

               end if

            end do

         end do

      end if
      end do

c     --- send sentinel

      iarray(1) = 0
      iarray(2) = 0
      iarray(3) = 0
      iarray(4) = 0
      iarray(5) = 0
      lengmes = 5
      isndcnt = isndcnt + 1
      call issenmpi ( iarray, lengmes, ISINT, ISHOST, ICVCINFO,
     +               ireq(isndcnt) )
      call iswaitall ( isndcnt, ireq )

      end
