      subroutine LU_all3 (mat3, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   10-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   calculates LU decomposition of the tridiagonal blocks on all levels.
c   Both LUx LUy and LUz (for alternating line jacobi)
c   mklvlinfo must have been called previously.
c **********************************************************************
c
c                       KEYWORDS
c
c      multi_grid
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision mat3 (*), rbuf (*)
      integer ibuf(*)

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel

c     hileveli  Ibuf (hileveli) contains the highest level of
c               coarsening for this block. (<= MAXLEVEL) (module
c               mgpreset)
c     ilevel    Levelcounter (multi grid levels)
c     luxri     Rbuf (ibuf (LUxri + ilevel)) is the first element
c               of the LUx decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     luyri     Rbuf (ibuf (LUyri + ilevel)) is the first element
c               of the LUy decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     luzri     Rbuf (ibuf (LUzri + ilevel)) is the first element
c               of the LUz decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     matri     Rbuf (ibuf (matri + ilevel)) is the first element
c               of the RAP operator at level ilevel (ilevel >= 1)
c               (module mgpreset)
c     sizexi    Ibuf(sizexi + ilevel)) contains number of points
c               in x direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizeyi    Ibuf(sizeyi + ilevel)) contains number of points
c               in y direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizezi    Ibuf(sizezi + ilevel)) contains number of points
c               in z direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     slacki    Ibuf (slacki) is the first element of the 6 element
c               slack array xslacklo, xslackhi, yslacklo, yslackhi,
c               zslacklo, zslackhi, (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     LUX3      Block LU decomposition for line Jacobi in x dir (MG3)
c     LUY3      Block LU decomposition for line Jacobi in y dir (MG3)
c     LUZ3      Block LU decomposition for line Jacobi in z dir (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Both LUx LUy and LUz on one level
c         This is independent of coarsening

        call LUx3 (mat3, rbuf ( ibuf (LUxri)), ibuf (sizexi),
     +       ibuf (sizeyi), ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
        call LUy3 (mat3, rbuf ( ibuf (LUyri)), ibuf (sizexi),
     +       ibuf (sizeyi), ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
        call LUz3 (mat3, rbuf ( ibuf (LUzri)), ibuf (sizexi),
     +       ibuf (sizeyi), ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

        do ilevel = 1, ibuf (hileveli)
        call LUx3 (rbuf ( ibuf (matri + ilevel)),
     +            rbuf ( ibuf (LUxri + ilevel)),
     +            ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +            ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
        call LUy3 (rbuf ( ibuf (matri + ilevel)),
     +            rbuf ( ibuf (LUyri + ilevel)),
     +            ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +            ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
        call LUz3 (rbuf ( ibuf (matri + ilevel)),
     +            rbuf ( ibuf (LUzri + ilevel)),
     +            ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +            ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
        end do
      end
