      subroutine RAPxy3 ( finemat, coarsmat, scratch, nxf, nyf, nzf,
     +   izf, isp,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   10-03-1997
c
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c    RAPxy3 makes the coarsening in both x & y direction for one
c    z-layer of the block.
c **********************************************************************
c
c                       KEYWORDS
c
c    multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxf, nyf, nzf, izf, isp
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6, 27),
     +coarsmat (nxf/2+1, nyf/2+1, 0:8),
     +scratch (nxf/2+1, nyf, 0:8)

c     coarsmat  o    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     isp       i    Counter to stencil plane
c     izf       i    Counter to number of grid z points on fine grid
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     scratch   i    Scratch array
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, nxc
      integer pts (0:26)
      double precision, allocatable :: zeros (:,:)

c     flevel0   Flag to denote whether we are on level 0
c               (module mgpreset)
c     ip        Local counting variable i-direction
c     molstruc  To signify the molecule structure on the finest level:
c               pt27 (default), pt19 or pt7 (module mgpreset)
c     nxc       Number of grid x points on coarse grid
c     pt19      Named constant (1). Molecule has 19 points on level 0
c               (module mgpreset)
c     pt27      Named constant (0). Molecule has 27 points on level 0
c               (module mgpreset)
c     pts       Array of indices of stencil points 3D
c     zeros     Array of zeros
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     RAPX1P3   Forms coarse grid operator semic in x-direction (1
c               plane only)  (MG3)
c     RAPY1P3   Forms coarse grid operator semic in y-direction (1
c               plane only)  (MG3)
c     VECMAP    Maps stencil indices onto array pts
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nxc = nxf/2 + 1

c--- we map the stencil points on to a vector for ease of

c     --- addressing them

      call vecmap (pts)
      if ((molstruc.eq.pt27) .or. ((molstruc.eq.pt19) .and.
     + (isp.eq.1)) .or. (.not. flevel0)) then

c--- isp is stencil plane: 0 = lower, 1 = central, 2 = higher
         do ip = 0, 8, 3

c--- make *FOR ONE STENCIL PLANE* only the x-coarsening
            call RAPx1P3 (finemat (1, 1, izf, pts (9*isp+ip)),
     +             finemat (1, 1, izf, pts (9*isp+ip+1)),
     +             finemat (1, 1, izf, pts (9*isp+ip+2)),
     +             scratch (1, 1, ip),
     +             scratch (1, 1, ip + 1),
     +             scratch (1, 1, ip + 2), nxf, nyf,
     +             sl1, sl2, sl3, sl4)
         end do
      else
         allocate (zeros (1-sl1:nxf+sl2,1-sl3:nyf+sl4))
         zeros = 0d0

         do ip = 0, 8, 6
            call RAPx1P3 (zeros,
     +             finemat (1, 1, izf, pts (9*isp+ip+1)),
     +             zeros,
     +             scratch (1, 1, ip),
     +             scratch (1, 1, ip + 1),
     +             scratch (1, 1, ip + 2), nxf, nyf,
     +             sl1, sl2, sl3, sl4)
         end do

         deallocate (zeros)
         ip = 3

         call RAPx1P3 (finemat (1, 1, izf, pts (9*isp+ip)),
     +          finemat (1, 1, izf, pts (9*isp+ip+1)),
     +          finemat (1, 1, izf, pts (9*isp+ip+2)),
     +          scratch (1, 1, ip),
     +          scratch (1, 1, ip + 1),
     +          scratch (1, 1, ip + 2), nxf, nyf,
     +          sl1, sl2, sl3, sl4)

      end if

c--- make for this stencil plane the y-coarsening
      do ip = 0,2
      call RAPy1P3 (scratch (1, 1, ip),
     +              scratch (1, 1, ip+3),
     +              scratch (1, 1, ip+6),
     +              coarsmat (1, 1, ip),
     +              coarsmat (1, 1, ip+3),
     +              coarsmat (1, 1, ip+6),
     +              nxc, nyf)
      end do
      end
