      subroutine isl2t ( laminar, solut, ndegfd, nvirtual,
     +                   ni , nj, ntimlv)
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.0    date    16-02-1998
c
c   copyright (c) 1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Read initial condition from laminar solution to turbulent
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni, nj, ntimlv, ndegfd, nvirtual
      double precision laminar(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1:(ndegfd-2), 1:ntimlv),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

c     laminar   i    Laminar solution array
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, itimlv, ip, jp

c     idegfd    Loop variable with respect to a given unknown
c     ip        Local counting variable i-direction
c     itimlv    Loop variable with respect to a given time level
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       PSEUDO CODE
c
c      copy solution from laminar solution to tubulent start solution
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isl2t')

c     --- copy solution

      do itimlv = 1, 2
         do idegfd = 1, ndegfd-2
            do jp = 1-nvirtual, nj+1+nvirtual
               do ip = 1-nvirtual, ni+1+nvirtual
                  solut( ip, jp, idegfd, itimlv  ) =
     +                         laminar( ip, jp, idegfd, itimlv  )
               end do
            end do
         end do
      end do

      do itimlv = 1, 2
         do idegfd = ndegfd-1, ndegfd
            do jp = 1-nvirtual, nj+1+nvirtual
               do ip = 1-nvirtual, ni+1+nvirtual
                  solut( ip, jp, idegfd, itimlv  ) = 1d-4
               end do
            end do
         end do
      end do

      call iserclos('isl2t')

      end
