      subroutine isord(isitopol, isorder)
c=====================================================================
c
c     programmer   Erik Brakkee
c     version      1.0  16-05-94  Erik:  initial revision
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     This routine computes an ordering of blocks suitable for
c     computation of the streamfunction.
c     The ordering is stored in output array order(1..2*nblocks)
c
c     order(1..nblocks)
c     -----------------
c
c     The first part of this array 1..nblocks indicates the ordering
c     of the blocks:
c        order(1), order(2), ..., order(nblocks)
c
c     The ordering is such that either:
c       1. block order(i) has some neighbour order(j) with j < i
c     or
c       2. block order(i) does not have any neighbours in order(j) 1 <= j <i
c          and no single block order(k) with k > i has a neighbour in
c          order(j) with j < i. In this case we speak of a so-called
c          isolated structure
c
c
c     order(nblocks+1..2*nblocks)
c     ---------------------------
c
c     The second part is the reverse of the first:
c       order(i) = j   iff   order(nblocks+j) = i
c     So this part contains in fact the new numbers of the blocks after
c     reordering.
c
c
c
c
c***********************************************************************
c
c                  KEYWORDS
c
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
c
      implicit none
      integer isitopol, isorder

c     isitopol  i    Sequence number of array itopol
c     isorder   o    Sequence number of array order
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisblank'
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer ipconsidr, ipitopol, ipnsubf, iporder, ipsubf,
     +        ipused,
     +        isconsidr, isused, lenorder,
     +        maxnsubf, nblocks, nfaces

c     ipconsidr Starting address of the considered array (see routine isord02)
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     iporder   Starting address of the order array
c     ipsubf    Starting address of the subfaces array
c     ipused    Starting address of the used array (see routine isord02)
c     isconsidr Pointer to the considered array (see routine isord02)
c     isused    Pointer to the used array (see routine isord02)
c     lenorder  Length of the order array
c     maxnsubf  Maximum number of subfaces encoutered on all subfaces
c     nblocks   Number of blocks
c     nfaces    Number of faces
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISORD02   Does all the work for this subroutine
c**********************************************************************
c
c                   ERROR MESSAGES :
c
c     None
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c
c=======================================================================
c

c     --- compute dimensioning parameters

      call isactive(isitopol)
      ipitopol     = isingetp(isitopol)
      nblocks      = ibuffr(ipitopol)
      nfaces       = ibuffr(ipitopol+1)
      maxnsubf = ibuffr(ipitopol+2)

c     --- allocate the order, used and considered arrays (see routine
c         isord02)

      isorder = 0
      isused  = 0
      isconsidr = 0

      lenorder = 2*nblocks
      call isicreat(isorder, lenorder, 'order' )
      call isicreat(isused, nblocks, 'used' )
      call isicreat(isconsidr, nblocks, 'considr' )

c     --- compute pointers

      ipitopol     = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
      iporder      = isingetp(isorder)
      ipused       = isingetp(isused)
      ipconsidr = isingetp(isconsidr)

c     --- call isord02

      call isord02(nblocks, nfaces, maxnsubf,
     +             ibuffr(ipnsubf), ibuffr(ipsubf),
     +             ibuffr(iporder), ibuffr(ipused),
     +             ibuffr(ipconsidr))

c     --- delete local arrays

      call isdelarr(isused)
      call isdelarr(isconsidr)
      call isdcrpri


      end











