      subroutine isord02 ( nblocks, nfaces, maxnsubf,
     +                     nsubfaces, subfaces,
     +                     order, used, considerd )
c=====================================================================
c
c     programmer   Erik Brakkee
c
c
c     version      1.0  16-05-94
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     This routine computes an ordering of blocks suitable for
c     computation of the streamfunction.
c     The ordering is stored in output array order(1..2*nblocks)
c
c     order(1..nblocks)
c     -----------------
c
c     The first part of this array 1..nblocks indicates the ordering
c     of the blocks:
c        order(1), order(2), ..., order(nblocks)
c
c     The ordering is such that either:
c       1. block order(i) has some neighbour order(j) with j < i
c     or
c       2. block order(i) does not have any neighbours in order(j) 1 <= j <i
c          and no single block order(k) with k > i has a neighbour in
c          order(j) with j < i. In this case we speak of a so-called
c          isolated structure
c
c
c     order(nblocks+1..2*nblocks)
c     ---------------------------
c
c     The second part is the reverse of the first:
c       order(i) = j   iff   order(nblocks+j) = i
c     So this part contains in fact the new numbers of the blocks after
c     reordering.
c
c***********************************************************************
c
c                  KEYWORDS
c
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxnsubf, nblocks, nfaces
      integer considerd(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer order(1:nblocks)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      integer used(1:nblocks)

c     considerd o    Considerd(i) = 1 if the neighbours of block i
c                    are already present in order(1..iorder) (see
c                    local variables)
c     maxnsubf  i    Maximum number of subfaces encoutered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     order     o    reserved
c     subfaces  i    reserved
c     used      o    Used(i) = 1 if block i is already present in
c                    order(1..iorder) (see local variables)
c***********************************************************************
c
c                   COMMON BLOCKS
c
c     None
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer iblock, iface, iorder, isubface, nbblock

c     iblock    Block sequence number
c     iface     Face number
c     iorder    Order(1..iorder) have already been filled
c     isubface  Subface number
c     nbblock   Sequence number of a neighbouring block
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c      None
c
c**********************************************************************
c
c                   ERROR MESSAGES :
c
c     None
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c
c=======================================================================
c
c     --- initialization

      iorder = 1
      order(1) = 1
      do iblock = 1, nblocks
         used(iblock) = 0
         considerd(iblock) = 0
      enddo
      used(1) = 1

c     --- main loop

        do while ( iorder .lt. nblocks )

c       ---  find a used block which has not been considered already

           iblock = 1
           do while ( iblock .le. nblocks .and. .not.
     +                ( used(iblock) .eq.  1 .and.
     +                  considerd(iblock) .eq. 0 ) )
              iblock = iblock + 1
           enddo

           if ( iblock .gt. nblocks) then

c          ---  isolated structure detected, find an unused block

              iblock = 1
              do while ( used(iblock) .eq. 1 )
                 iblock = iblock + 1
              enddo
              iorder = iorder + 1
              order(iorder) = iblock
              used(iblock) = 1

           else

c          ---  add all neighbours of block iblock to the list

              do iface = 1, nfaces
                 do isubface = 1, nsubfaces(iblock,iface)
                    nbblock = subfaces(iblock,iface,isubface,5)
                    if ( nbblock .ne. 0 ) then
                       if ( used(nbblock) .eq. 0 ) then
                          iorder = iorder + 1
                          order(iorder) = nbblock
                          used(nbblock) = 1
                       endif
                    endif
                 enddo
              enddo

              considerd(iblock) = 1

           endif
        enddo

c       --- compute the second part of the order array

      do iblock = 1, nblocks

         order(nblocks + order(iblock))  = iblock

      enddo

      end











