      subroutine isout01 ( iblock, ni, nj, nvirtual,  nblocks, nfaces,
     +                     maxnsubf, nsubfaces, subfaces, coor, scalar,
     +                     output, itypepost )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      3.0  02-04-1997 Guus: extra parameter itypepost
c     version      2.0  06-03-1996 Guus: remove ndim
c     version      1.0  16-05-1994
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   This subroutine performs the interpolation of a scalar quantity
c   defined in the centroids of the 2D cells, to the vertices
c   Bilinear interpolation  is used.
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, nfaces,
     +        ni, nj, nvirtual, itypepost
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1:2)
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision output(1:ni+1,1:nj+1)
      double precision scalar(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     itypepost i    Indicates the type of postprocessing required
c                    0: standard Newton    1: linear in computational domain
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Cartesian components of the flow field in the vertices
c     scalar    i    Quantities in the centroids
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iface, isubface, j
      double precision uloc, vloc

c     i         General loop variable
c     iface     Face number
c     isubface  Subface number
c     j         General loop variable
c     uloc      Local u-coordinate for the interpolation
c     vloc      Local v-coordinate for the interpolation
c **********************************************************************
c
c                       I/O
c
c      -
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISNTON    Compute local coordinates for the interpolation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( itypepost.eq.0 ) then

c     --- Standard situation, Newton is applied
c         First, consider all cells as if they were internal

         do j = 1, nj+1
            do i = 1, ni+1
               call isnton(ni,nj,nvirtual,coor,i,j,i,j,uloc,vloc)
               output(i,j) =      uloc *     vloc  * scalar(i  ,j  )+
     +                            uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                       (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                       (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)
            end do
         end do

c        --- Overwrite the external parts with extrapolated values from the
c            inner region
c            The left boundary

         iface = ileft
         i     = 2
         do isubface = 1, nsubfaces(iblock, iface)

            if ( subfaces(iblock,iface,isubface,5).eq.0 ) then

               do j = subfaces(iblock,iface,isubface,1),
     +                subfaces(iblock,iface,isubface,2) + 1

                  call isnton(ni,nj,nvirtual,coor,i,j,i-1,j,uloc,vloc)
                  output(i-1,j) =    uloc *     vloc  * scalar(i  ,j  )+
     +                               uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                          (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                          (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)

               end do

            end if

         end do

c        --- The right boundary

         iface = iright
         i     = ni
         do isubface = 1, nsubfaces(iblock, iface)

            if ( subfaces(iblock,iface,isubface,5).eq.0 ) then

               do j = subfaces(iblock,iface,isubface,1),
     +                subfaces(iblock,iface,isubface,2) + 1

                  call isnton(ni,nj,nvirtual,coor,i,j,i+1,j,uloc,vloc)
                  output(i+1,j) =    uloc *     vloc  * scalar(i  ,j  )+
     +                               uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                          (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                          (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)
               end do

            end if

         end do

c        --- The under boundary

         iface = idown
         j     = 2
         do isubface = 1, nsubfaces(iblock,iface)

            if ( subfaces(iblock, iface, isubface,5).eq.0 ) then

               do i = subfaces(iblock,iface,isubface,1),
     +                subfaces(iblock,iface,isubface,2) + 1

                  call isnton(ni,nj,nvirtual,coor,i,j,i,j-1,uloc,vloc)
                  output(i,j-1) =    uloc *     vloc  * scalar(i  ,j  )+
     +                               uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                          (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                          (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)

               end do

            end if

         end do

c        --- The upper boundary

         iface = iup
         j     = nj

         do isubface = 1, nsubfaces(iblock,iface)

            if ( subfaces(iblock, iface, isubface,5).eq.0 ) then

               do i = subfaces(iblock,iface,isubface,1),
     +                subfaces(iblock,iface,isubface,2) + 1

                  call isnton(ni,nj,nvirtual,coor,i,j,i,j+1,uloc,vloc)
                  output(i,j+1) =    uloc *     vloc  * scalar(i  ,j  )+
     +                               uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                          (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                          (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)

               end do

            end if

         end do

c        --- bottom-left corner

         if ( subfaces(iblock, ileft, 1, 5).eq.0  .or. 1.eq.1 .or.
     +        subfaces(iblock, idown, 1, 5).eq.0 ) then

            i = 2
            j = 2
            call isnton(ni, nj, nvirtual, coor,
     +                  i, j, i-1, j-1, uloc, vloc)
            output(i-1,j-1) =        uloc *      vloc * scalar(i  ,j  )+
     +                          (1d0-uloc)*      vloc * scalar(i-1,j  )+
     +                          (1d0-uloc)* (1d0-vloc)* scalar(i-1,j-1)+
     +                               uloc * (1d0-vloc)* scalar(i  ,j-1)

         end if

c        --- bottom-right corner

         if (subfaces(iblock, iright, 1, 5).eq.0 .or. 1.eq.1 .or.
     +       subfaces(iblock, idown, nsubfaces(iblock,idown),5).eq.0 )
     +       then

            i = ni
            j = 2
            call isnton(ni, nj, nvirtual, coor,
     +                  i, j, i+1, j-1, uloc, vloc)
            output(i+1,j-1) =        uloc *      vloc * scalar(i  ,j  )+
     +                          (1d0-uloc)*      vloc * scalar(i-1,j  )+
     +                          (1d0-uloc)* (1d0-vloc)* scalar(i-1,j-1)+
     +                               uloc * (1d0-vloc)* scalar(i  ,j-1)

         end if

c        --- top-left corner

         if ( subfaces(iblock,ileft,nsubfaces(iblock,ileft),5).eq.0
     +        .or. 1.eq. 1 .or.
     +        subfaces(iblock, iup, 1, 5 ).eq.0 ) then

            i = 2
            j = nj
            call isnton(ni, nj, nvirtual, coor,
     +                  i, j, i-1, j+1, uloc, vloc)
            output(i-1,j+1) =        uloc *      vloc * scalar(i  ,j  )+
     +                          (1d0-uloc)*      vloc * scalar(i-1,j  )+
     +                          (1d0-uloc)* (1d0-vloc)* scalar(i-1,j-1)+
     +                               uloc * (1d0-vloc)* scalar(i  ,j-1)

         end if

c        --- top-right corner

         if ( subfaces(iblock,iright,nsubfaces(iblock,iright),5).eq.0
     +        .or. 1.eq.1 .or.
     +        subfaces(iblock, iup, nsubfaces(iblock,iup), 5).eq.0 )
     +      then

            i = ni
            j = nj
            call isnton(ni, nj, nvirtual, coor,
     +                  i, j, i+1, j+1, uloc, vloc)
            output(i+1,j+1)   =      uloc *      vloc * scalar(i  ,j  )+
     +                          (1d0-uloc)*      vloc * scalar(i-1,j  )+
     +                          (1d0-uloc)* (1d0-vloc)* scalar(i-1,j-1)+
     +                               uloc * (1d0-vloc)* scalar(i  ,j-1)

         end if

      else

c     --- Special situation, Linear interpolation in computational
c         domain is applied

         do j = 1, nj+1
            do i = 1, ni+1
               output(i,j) = 0.25d0* ( scalar(i,j) + scalar(i,j-1) +
     +                                 scalar(i-1,j) + scalar(i-1,j-1) )
            end do
         end do

      end if

      end
