      subroutine isout013 ( ni, nj, nk, nvirtual, coor, scalar, output )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.1  08-11-1995 extension for edges and corners
c     version      1.0  30-12-1994
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                  DESCRIPTION
c
c   This subroutine performs the interpolation of a scalar quantity
c   defined in the centroids of the 3D cells, to the vertices.
c   Single block only.
c   For the values on the boundary the values in the vertices are
c   extrapolated from the inner area, the virtual values are not
c   used.
c
c   Bilinear interpolation  is used.
c
c **********************************************************************
c
c           INPUT / OUTPUT PARAMETERS

      implicit none
      integer ni, nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)
      double precision output(1:ni+1,1:nj+1,1:nk+1)
      double precision scalar(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)

c     coor      i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Cartesian components of the flow field in the vertices
c     scalar    i    Quantities in the centroids

c **********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisnas'
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, j, k
      double precision uci, vci, wci

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     uci       Local coordinate for the interpolation in the i-direction
c     vci       Local coordinate for the interpolation in the j-direction
c     wci       Local coordinate for the interpolation in the k-direction
c
c **********************************************************************
c
c                         I/O
c
c
c **********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISNT3D    Compute local coordinates for the interpolation
c **********************************************************************
c
c                     ERROR MESSAGES
c
c
c **********************************************************************
c
c                     PSEUDO CODE
c
c ======================================================================
c
c     --- inner area

      do k = 2, nk
         do j = 2, nj
            do i = 2, ni
               call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                     i, j, k, uci, vci, wci)
               output(i,j,k) = (1-uci)*(1-vci)*(1-wci) *
     +                          scalar(i-1,j-1,k-1)      +
     +                          uci*(1-vci)*(1-wci)    *
     +                          scalar(i  ,j-1,k-1)      +
     +                          (1-uci)*vci*(1-wci)    *
     +                          scalar(i-1,j  ,k-1)      +
     +                          uci*vci*(1-wci)        *
     +                          scalar(i  ,j  ,k-1)      +
     +                          (1-uci)*(1-vci)*wci    *
     +                          scalar(i-1,j-1,k  )      +
     +                          uci*(1-vci)*wci        *
     +                          scalar(i  ,j-1,k  )      +
     +                          (1-uci)*vci*wci        *
     +                          scalar(i-1,j  ,k  )      +
     +                          uci*vci*wci            *
     +                          scalar(i  ,j  ,k  )

            end do
         end do
      end do

c     --- plane 1

      i = 2
      do k = 2, nk
         do j = 2, nj
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i-1, j, k, uci, vci, wci)
            output(i-1,j,k) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- plane 2

      i = ni
      do k = 2, nk
         do j = 2, nj
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i+1, j, k, uci, vci, wci)
            output(i+1,j,k) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- plane 3

      j = 2
      do k = 2, nk
         do i = 2, ni
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i, j-1, k, uci, vci, wci)
            output(i,j-1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- plane 4

      j = nj
      do k = 2, nk
         do i = 2, ni
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i, j+1, k, uci, vci, wci)
            output(i,j+1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- plane 5

      k = 2
      do j = 2, nj
         do i = 2, ni
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i, j, k-1, uci, vci, wci)
            output(i,j,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- plane 6

      k = nk
      do j = 2, nj
         do i = 2, ni
            call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +                  i, j, k+1, uci, vci, wci)
            output(i,j,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                        scalar(i-1,j-1,k-1)      +
     +                        uci*(1-vci)*(1-wci)    *
     +                        scalar(i  ,j-1,k-1)      +
     +                        (1-uci)*vci*(1-wci)    *
     +                        scalar(i-1,j  ,k-1)      +
     +                        uci*vci*(1-wci)        *
     +                        scalar(i  ,j  ,k-1)      +
     +                        (1-uci)*(1-vci)*wci    *
     +                        scalar(i-1,j-1,k  )      +
     +                        uci*(1-vci)*wci        *
     +                        scalar(i  ,j-1,k  )      +
     +                        (1-uci)*vci*wci        *
     +                        scalar(i-1,j  ,k  )      +
     +                        uci*vci*wci            *
     +                        scalar(i  ,j  ,k  )

         end do
      end do

c     --- i edge 1,  ( edge direction is i )

      j = 2
      k = 2
      do i = 2, ni
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i, j-1, k-1, uci, vci, wci)
         output(i,j-1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- i edge 2

      j = nj
      k = 2
      do i = 2, ni
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i, j+1, k-1, uci, vci, wci)
         output(i,j+1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- i edge 3

      j = 2
      k = nk
      do i = 2, ni
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i, j-1, k+1, uci, vci, wci)
         output(i,j-1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- i edge 4

      j = nj
      k = nk
      do i = 2, ni
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i, j+1, k+1, uci, vci, wci)
         output(i,j+1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- j edge 1,  ( edge direction is j )

      i = 2
      k = 2
      do j = 2, nj
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i-1, j, k-1, uci, vci, wci)
         output(i-1,j,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- j edge 2,  ( edge direction is j )

      i = ni
      k = 2
      do j = 2, nj
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i+1, j, k-1, uci, vci, wci)
         output(i+1,j,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- j edge 3,  ( edge direction is j )

      i = 2
      k = nk
      do j = 2, nj
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i-1, j, k+1, uci, vci, wci)
         output(i-1,j,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- j edge 4,  ( edge direction is j )

      i = ni
      k = nk
      do j = 2, nj
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i+1, j, k+1, uci, vci, wci)
         output(i+1,j,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- k edge 1

      i = 2
      j = 2
      do k = 2, nk
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i-1, j-1, k, uci, vci, wci)
         output(i-1,j-1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- k edge 2

      i = ni
      j = 2
      do k = 2, nk
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i+1, j-1, k, uci, vci, wci)
         output(i+1,j-1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- k edge 3

      i = 2
      j = nj
      do k = 2, nk
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i-1, j+1, k, uci, vci, wci)
         output(i-1,j+1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- k edge 4

      i = ni
      j = nj
      do k = 2, nk
         call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +               i+1, j+1, k, uci, vci, wci)
         output(i+1,j+1,k) = (1-uci)*(1-vci)*(1-wci) *
     +                     scalar(i-1,j-1,k-1)      +
     +                     uci*(1-vci)*(1-wci)    *
     +                     scalar(i  ,j-1,k-1)      +
     +                     (1-uci)*vci*(1-wci)    *
     +                     scalar(i-1,j  ,k-1)      +
     +                     uci*vci*(1-wci)        *
     +                     scalar(i  ,j  ,k-1)      +
     +                     (1-uci)*(1-vci)*wci    *
     +                     scalar(i-1,j-1,k  )      +
     +                     uci*(1-vci)*wci        *
     +                     scalar(i  ,j-1,k  )      +
     +                     (1-uci)*vci*wci        *
     +                     scalar(i-1,j  ,k  )      +
     +                     uci*vci*wci            *
     +                     scalar(i  ,j  ,k  )

      end do

c     --- corner 1

      i = 2
      j = 2
      k = 2
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i-1, j-1, k-1, uci, vci, wci)
      output(i-1,j-1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k  )

c     --- corner 2

      i = ni
      j = 2
      k = 2
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i+1, j-1, k-1, uci, vci, wci)
      output(i+1,j-1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k  )

c     --- corner 3

      i = 2
      j = nj
      k = 2
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i-1, j+1, k-1, uci, vci, wci)
      output(i-1,j+1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k   )

c     --- corner 4

      i = ni
      j = nj
      k = 2
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i+1, j+1, k-1, uci, vci, wci)
      output(i+1,j+1,k-1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k   )

c     --- corner 5

      i = 2
      j = 2
      k = nk
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i-1, j-1, k+1, uci, vci, wci)
      output(i-1,j-1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k  )

c     --- corner 6

      i = ni
      j = 2
      k = nk
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i+1, j-1, k+1, uci, vci, wci)
      output(i+1,j-1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k  )

c     --- corner 7

      i = 2
      j = nj
      k = nk
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i-1, j+1, k+1, uci, vci, wci)
      output(i-1,j+1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k   )

c     --- corner 8

      i = ni
      j = nj
      k = nk
      call isnt3d(ni, nj, nk, nvirtual, coor, i, j, k,
     +            i+1, j+1, k+1, uci, vci, wci)
      output(i+1,j+1,k+1) = (1-uci)*(1-vci)*(1-wci) *
     +                  scalar(i-1,j-1,k-1)      +
     +                  uci*(1-vci)*(1-wci)    *
     +                  scalar(i  ,j-1,k-1)      +
     +                  (1-uci)*vci*(1-wci)    *
     +                  scalar(i-1,j  ,k-1)      +
     +                  uci*vci*(1-wci)        *
     +                  scalar(i  ,j  ,k-1)      +
     +                  (1-uci)*(1-vci)*wci    *
     +                  scalar(i-1,j-1,k  )      +
     +                  uci*(1-vci)*wci        *
     +                  scalar(i  ,j-1,k  )      +
     +                  (1-uci)*vci*wci        *
     +                  scalar(i-1,j  ,k  )      +
     +                  uci*vci*wci            *
     +                  scalar(i  ,j  ,k   )

      end
