      subroutine isout261 ( outvec, keps, mturb, ni, nj, eddy,
     +                      rintur, nvirtual, ichoiceo )
c ======================================================================
c
c     programmer  Marcel Zijlema
c
c     version     1.1   17-07-1996 Marcel: parameter list changed
c     version     1.0   09-06-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute several quantities for output purpose in case of turbulence models
c   2D case
c **********************************************************************
c
c                       KEYWORDS
c
c   2d
c   output
c   turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mturb, ni, nj, nvirtual, ichoiceo
      double precision outvec(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 keps(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*), rintur(*)

c     eddy      i    reserved
c     ichoiceo  i    Choice parameter for turbulent output subroutine
c                    1:  Compute turbulent viscosity
c                    2:  Compute turbulent intensity
c                    3:  Compute turbulent length scale
c     keps      i    Array containing the part of the solution corresponding
c                    to k and epsilon or k and omega
c                    keps(i,j,k,1) contains k
c                    keps(i,j,k,2) contains eps or omega
c     mturb     i    Indicates type of turbulence modelling
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     outvec    o    Output vector
c     rintur    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp

c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Initialize outvec array
c        (in this array the quantity to be computed is stored)
c     if ( ichoiceo=1 ) then
c        compute turbulent viscosity
c     else if ( ichoiceo=2 ) then
c        compute turbulent intensity
c     else ( ichoiceo=3 ) then
c        compute turbulent length scale
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Initialize outvec array

      do jp = 1-nvirtual, nj+1+nvirtual
         do ip = 1-nvirtual, ni+1+nvirtual
            outvec(ip,jp) = 0d0
         end do
      end do
      if ( ichoiceo.eq.1 ) then

c     --- compute turbulent viscosity

         do jp = 1, nj
            do ip = 1, ni
               outvec(ip,jp) = eddy(ip,jp)
            end do
         end do

      else if ( ichoiceo.eq.2 ) then

c     --- compute turbulent intensity

         do jp = 1, nj
            do ip = 1, ni
               outvec(ip,jp) = sqrt(keps(ip,jp,1))
            end do
         end do

      else

c     --- compute length scale

        if ( mturb.eq.3 ) then

c       --- Wilcox k-omega model

           do jp = 1, nj
              do ip = 1, ni
                 outvec(ip,jp) = sqrt(keps(ip,jp,1)) / keps(ip,jp,2)
              end do
           end do

        else

           do jp = 1, nj
              do ip = 1, ni
                 outvec(ip,jp) = rintur(3) * keps(ip,jp,1) *
     +                         sqrt(keps(ip,jp,1)) / keps(ip,jp,2)
              end do
           end do

        end if

      end if

      end
