      subroutine isoutstrm ( time, isiinput, isrinput, iscoor, issolut,
     +                       isitopol, indorien, isoutput, isvel,
     +                       isorder, isstream, iseq1 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 16-03-1999 delete work array
c        version 1.0    date 27-01-1999
c
c
c   copyright (c) 1999-1999 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write the before last time level in the subdomain solution arrays
c     to the output file. This subroutine writes the stream function only
c **********************************************************************
c
c                       KEYWORDS
c
c   output
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isoutput, indorien(*), isvel(*), isorder,
     +        isstream, iseq1
      double precision time

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseq1     o    Sequence number of output quantity
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isorder   i    Sequence number of array order
c     isoutput  i    Pointer to a local output array containing quantities in
c                    the vertices
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     isstream  i    Pointer to the stream array containing for each subface
c                    the value of the stream function at the beginning and
c                    ending of that subface
c     isvel     i    Pointer to a local array for storing Cartesian velocity
c                    components at the vertices
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, ipcoor, ipiinput, ipitopol, ipoutput,
     +        itype1, ipsolut, nblocks, ndegfd, mshtyp, ipjcta,
     +        ndim, ni, nj, nk, ntimlv, nvirtual, npointsv, vartyp,
     +        ipwork1, idim, acttyp, pnttyp, betad, ipcva, ipiinbc,
     +        ipnsubf, iporder, ipstream, ipsubf, ipvel, ipiindsc,
     +        iswork, maxnsubf, nfaces, ipwork, axisym
      character * 30 name

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     axisym    In axisymmetric case axisym=1 else axisym=0
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     iblock    Block sequence number
c     idim      Loop variable used for dimension
c     ipcoor    Starting address of array coor in buffer
c     ipcva     Starting address of array CVA in array WORK
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipjcta    Starting address of array JCTA in array WORK
c     ipnsubf   Starting address of the nsubfaces array
c     iporder   Starting address of the order array
c     ipoutput  Starting address of a local output array containing quantities
c               in the vertices
c     ipsolut   Starting address of array solut in buffer
c     ipstream  Starting address of the stream array containing for each
c               subface the value of the stream function at the
c               beginning and ending of that subface
c     ipsubf    Starting address of the subfaces array
c     ipvel     Starting address of a local array for storing Cartesian velocity
c               components at the vertices
c     ipwork    Starting address of array work in buffer
c     ipwork1   Starting address of array work1 in buffer
c     iswork    Sequence number of array work
c     itype1    Indicates the type of output vector
c               1: scalar,  2: vector,  3: tensor
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     mshtyp    Type of grid arrangement
c     name      Name of file
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       I/O
c
c      This solution and some related quantities are written to the file
c      isnascomp.out
c      Depending on the parameters in the file isnas.env this file is
c      formatted or unformatted
c      The records in the file isnascomp.out are filled in the following way:
c      -  ndim, nblocks
c      For each array to be written:
c         -  Name of vector
c         -  time, iseq1, itype1    (itype1=1:scalar, 2:vector)
c         For each block:
c            - iblock, nx, ny, nz
c            For all components
c               - values of vector in the sequence (1:nx+1,1:ny+1,1:nz+1)
c     The blocks are written in the natural sequence, except for the stream
c     function, which uses a special block ordering
c     The sequence of the vectors to be written is:
c        velocity
c        pressure
c        stream function
c        transport quantities
c        turbulent quantities
c        extra quantities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISCONC2D  Perform the contravariant components form the Cartesian
c               at the cell faces
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISCPPER2  Compute mean values on common sides of periodical boundaries
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISOUT03   determine the stream function for a subdomain
c     ISOUT03A  determine the stream function for a subdomain axisymmetric
c     ISPOSOR1  Change directions in solution array
c     ISPRINS2  Print one block of scalars in 2D
c     ISSETCON  Make array equal to a constant
c     ISTMSPAC  Create space for temporary array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        Compute the stream function
c     ENDDO
c     Delete local arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isoutstrm')

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol)
      nfaces   = ibuffr(ipitopol + 1)
      maxnsubf = ibuffr(ipitopol + 2)

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      ipiinbc    = ibuffr(ipiinput +  2 - 1)
      ipiindsc   = ibuffr(ipiinput +  6 - 1)
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      ntimlv     = ibuffr(ipiinput + 64 - 1)
      mshtyp     = ibuffr(ipiinput + 75 - 1 )
      axisym     = ibuffr(ipiinput-1 + ipiindsc -1+10)
      iswork = 0

      nvirtual   = ibuffr(ipiinput + 54 - 1)

c     --- write stream function

      iseq1 = 3
      itype1 = 1
      name = 'stream_function'
      if ( iref74.gt.0 ) then
         write(iref74,110 ) name, time, iseq1, itype1
110      format(a/f12.5,2i3,12x,a)
      else
         write(-iref74) name
         write(-iref74) time, iseq1, itype1
      end if
      call isactive(isorder)
      do i = 1, nblocks

c     --- compute block number

         iporder = isingetp(isorder)
         iblock = ibuffr(iporder + i - 1)

c        --- determine subdomain parameters

         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         if ( ndim.eq.2 ) then
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         else
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                 (nk+1+2*nvirtual)
         end if
         if ( indorien(iblock).eq.-1 )
     +      call istmspac((ni+1)*(nj+1)*intlen,ipwork1)
         call isactive(isvel(iblock))
         call isactive(iscoor(iblock))
         call isdcreat ( iswork, 22*npointsv, 'work' )

         ipcoor   = isdpgetp(iscoor(iblock))
         ipwork   = isdpgetp(iswork)
         ipstream = isdpgetp(isstream)
         ipitopol = isingetp(isitopol)
         ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
         ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
         ipsolut  = isdpgetp(issolut(iblock)) +
     +                 (ntimlv-2)*npointsv*ndegfd
         ipvel    = isdpgetp(isvel(iblock))
         ipoutput = isdpgetp(isoutput)

c        --- Write first record of block

         if ( iref74.gt.0 ) then
            write(iref74,120 ) iblock, ni, nj, nk
120         format(4i8,12x,a)
         else
            write(-iref74) iblock, ni, nj, nk
         end if

c        --- compute and write stream function

         ipvel = ipsolut
         if ( mshtyp.eq.1 ) then
            ipcva    = ipwork
            ipjcta   = ipcva+16*npointsv
            call issetcon ( buffer(ipcva), 20*npointsv, 0d0 )
            call iscova2d ( buffer(ipcoor), buffer(ipcva),
     +                      ni, nj, nvirtual )
            vartyp = 3
            acttyp = 0
            do idim = 1, 2
              pnttyp = 4 - idim
               do betad = 1, 2
                  call isintcf2 ( buffer(ipsolut+(betad-1)*npointsv),
     +                 buffer( ipwork+(20+betad-1)*npointsv ),
     +                 ni, nj, nvirtual, vartyp, acttyp,
     +                 pnttyp, mshtyp, buffer(ipcoor) )
               end do
               call isjcta2d ( buffer(ipcva+4*npointsv*(pnttyp-1)),
     +                         buffer(ipjcta), ni, nj, nvirtual )
               call isconc2d ( buffer(ipvel+(idim-1)*npointsv),
     +                         buffer(ipwork+20*npointsv),
     +                         buffer(ipwork+21*npointsv),
     +                         buffer(ipjcta), ni, nj,
     +                         nvirtual, idim )
            end do
         end if

         if ( axisym.eq.1 ) then

c        --- Axisymmetric case

            call isout03a ( iblock, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf,
     +                      buffer(ipstream), ibuffr(iporder),
     +                      ibuffr(ipnsubf), ibuffr(ipsubf),
     +                      buffer(ipvel), buffer(ipoutput),
     +                      buffer(ipcoor) )
         else

c        --- Cartesian case

            call isout03 ( iblock, ni, nj, nvirtual,
     +                     nblocks, nfaces, maxnsubf,
     +                     buffer(ipstream), ibuffr(iporder),
     +                     ibuffr(ipnsubf), ibuffr(ipsubf),
     +                     buffer(ipvel), buffer(ipoutput) )

         end if

c        --- Compute mean value for periodical boundaries

         call iscpper2 ( ibuffr(ipiinput),
     +                   ibuffr(ipiinput-1+ipiinbc),
     +                   buffer(ipoutput),  ni, nj, iblock )

         if ( indorien(iblock).eq.-1 ) then
            call isposor1 ( buffer(ipoutput),
     +                      buffer(ipwork1), ni, nj )
            call isprins2 ( nj, ni, buffer(ipoutput) )
         else
            call isprins2 ( ni, nj, buffer(ipoutput) )
         end if

      end do
      if ( iswork.gt.0 ) call isdelarr ( iswork )

      call iserclos('isoutstrm')

      end
