      subroutine isrestf (time, isiinput, isrinput, isitopol, issolut)
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 24-04-95
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c ********************************************************************
c
c                      KEYWORDS
c
c     restart
c     backing_storage
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer isiinput(*), isrinput(*), issolut(*), isitopol
      double precision time

c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     time      i    Actual time of time step
c
c ********************************************************************
c
c                    COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ifirstl

      integer isingetp, isdpgetp, isinleng, isdpleng, isinisgl

      logical notfound
      integer i, iblock, ios, ipiinput, ipitopol,
     +        iprinput, ipsolut, ipwork, irec, irecses,
     +        irectime, irectimef, irsol, iswork, itimenext,
     +        iunder, lengthc, lentopol, lwork, indref,
     +        nblocks

      double precision  timeold

      save ifirstl, irecses, irectimef
      data ifirstl /1/

c     i         General loop variable
c     iblock    Block sequence number
c     ifirstl   First call of this routine (=1) or not (=0)
c     ios       Iostatus variable
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     iprinput  Starting address of array rinput in buffr
c     ipsolut   Starting address of array solut in buffr
c     ipwork    Starting address of array work in buffr
c     irec      Record number
c     irecses   Record number of session record
c     irectime  Record number of time record
c     irectimef Record number of first time record of the session
c     irsol     Record number where the first part of the solution is stored
c     iswork    Sequence number of array work
c     itimenext Record number of next time record
c     iunder    First element in inparray to be read
c     lengthc   Length parameter for issend and isreceiv
c     lentopol  Length of the itopol array
c     lwork     Length of array work
c     nblocks   Number of blocks
c     notfound  Logical help variable
c     timeold   Help variable
c
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c
c     ISACTIVE  Activate old permanent array
c     ISCONTRST Print contents of restartfile
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISDPLENG  Return with the length of the array in double precisions
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIRST  Startup of writing the restart file
c     ISINISGL  Recompute pointer for buffer to pointer for ibuffr
c     ISINLENG  Return with the length of the array in integers
c     ISWRITFL  Write to direct access file
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c     19      Error in writing record
c     20      Error in reading record
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c     if ( first ) then
c        initialize irecses, irectime, itimefirst
c        call isinirst
c     end if
c
c     irecses    = record number isession
c     irectime   = record number time step to be written
c     itimefirst = record number first time step of this session
c     ifree2     = irectime   (common cisnas )
c
c     itime = ifree2
c     irsol = itime + 1
c
c     WRITE RECORD ITIME ( time=time, itime_next=-1, isol=irsol )
c     ifree2 = ifree2 +1
c     indref = ifree2
c
c     do i=1, nblocks
c        WRITE SOLUTION(indref, iunder, iupper, ireffile, inparray )
c     end do
c     # ifree2 is changed with writing solution so :
c     WRITE RECORD 1  for ifree2
c     WRITE RECORD IRECSES with issesion = ifree2
c
c     if( itime > itimefirst) then
c
c        # last but one timestep is not the last anymore so:
c        # WRITE RECORD ITIME_OLD( time=time_old,itime_next=itime,isol=isol_old)
c
c        itimeold = itimefirst
c        notfound = .true.
c        while ( notfound ) do
c           READ( iref2, rec=itimeold, iostat=ios) timeold, itimnold, isolold
c           if( itimnold .ne.  -1 ) then
c              itimeold= itimnold
c           else
c              notfound = .false.
c              WRITE( iref2, rec=itimeold, iostat=ios) timeold, itime, isolold
c           end if
c           goto 100
c        end do
c
c     end if
c
c
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      iswork = 0
      lwork = nrec2
      call isicreat(iswork, lwork, 'work' )
      ipwork  = isingetp( iswork )

      call isactive( isiinput(1) )
      ipiinput = isingetp( isiinput(1) )

      if ( ifirstl .eq. 1) then

         call isactive( isrinput(1) )
         iprinput = isdpgetp( isrinput(1) )
         call isactive( isitopol )
         ipitopol = isingetp( isitopol )
         lentopol  = isinleng( isitopol )

         call isinirst( ibuffr(ipiinput), buffer(iprinput),
     +                  ibuffr(ipitopol), lentopol, ibuffr(ipwork),
     +                  lwork, irecses, irectime, irectimef)
         ifirstl = 0
      end if

c     --- write record itime

      itimenext = -1
      irsol     = ifree2 +1
      irectime  = ifree2
      write(iref2, rec=irectime, iostat=ios ) time, itimenext, irsol
      if ( ios .ne. 0 ) then
c     --- error in writing record
         call iseropen ( 'isinibw' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 19, 2, 0, 0 )
         call iserclos ( 'isinibw' )
         stop
      end if
      ifree2 = ifree2 +1

c     --- write solution to restart file

      nblocks = ibuffr(ipiinput -1 + 73)
      do iblock = 1, nblocks
         call isactive( issolut(iblock) )
         ipsolut  = isdpgetp( issolut(iblock) )
         lengthc  = isdpleng( issolut(iblock) )
         ipsolut  = isinisgl(ipsolut)
         lengthc  = lengthc * intlen
         iunder   = 1
         indref   = ifree2
         call iswritfl(indref, iunder, lengthc, iref2, ibuffr(ipsolut) )
      end do


c     --- write ifree2 to record 1

      irec = 1
      read(iref2, rec=irec, iostat=ios )
     +     (ibuffr(ipwork-1+i),i=1,lwork)
      if ( ios .ne. 0 ) then
c     --- error in reading record
         call iseropen ( 'isrestf' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 20, 2, 0, 0 )
         call iserclos ( 'isrestf' )
         stop
      end if
      ibuffr(ipwork-1+2) = ifree2
      write(iref2, rec=irec, iostat=ios )
     +      (ibuffr(ipwork-1+i),i=1,lwork)
      if ( ios .ne. 0 ) then
c     --- error in writing record
         call iseropen ( 'isrestf' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 19, 2, 0, 0 )
         call iserclos ( 'isrestf' )
         stop
      end if

c     --- write ifree2 to record irecses

      irec = irecses
      read(iref2, rec=irec, iostat=ios )
     +     (ibuffr(ipwork-1+i),i=1,lwork)
      if ( ios .ne. 0 ) then
c     --- error in reading record
         call iseropen ( 'isrestf' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 20, 2, 0, 0 )
         call iserclos ( 'isrestf' )
         stop
      end if
      ibuffr(ipwork-1+1) = ifree2
      write(iref2, rec=irec, iostat=ios )
     +      (ibuffr(ipwork-1+i),i=1,lwork)
      if ( ios .ne. 0 ) then
c     --- error in writing record
         call iseropen ( 'isrestf' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 19, 2, 0, 0 )
         call iserclos ( 'isrestf' )
         stop
      end if


c     --- last but one time record is not the last anymore, so
c         write record itime_last_but_one with itimenext = record
c         number of current time step

c     --- start with first timerecord

      irec = irectimef
      notfound = .true.
100   if ( notfound ) then
         read( iref2, rec=irec, iostat=ios) timeold, itimenext, irsol

         if ( ios .ne. 0 ) then
c        --- error in reading record
            call iseropen ( 'isrestf' )
            call iserrint ( iref2, 1 )
            call iserrint ( ios, 2 )
            call iserrsub ( 20, 2, 0, 0 )
            call iserclos ( 'isrestf' )
            stop
         end if

         if( itimenext .ne.  -1 ) then

c        --- next timerecord

            irec= itimenext
         else

c        --- found, but must not be the last timerecord if there is only
c            one time step stored

            notfound = .false.
            if ( irec .ne. irectime ) then
               write( iref2, rec=irec, iostat=ios)timeold,irectime,irsol

               if ( ios .ne. 0 ) then
c              --- error in writing record
                  call iseropen ( 'isrestf' )
                  call iserrint ( iref2, 1 )
                  call iserrint ( ios, 2 )
                  call iserrsub ( 19, 2, 0, 0 )
                  call iserclos ( 'isrestf' )
                  stop
               end if
            end if

         end if
         goto 100
      end if

      call iscontrst( ibuffr(ipwork), lwork)

      call isdelarr(iswork)

      end
