      subroutine isexch01( itrans, ilev, ndegfd, ntimlv, nvirtual,
     +                    nblocks, nfaces, maxnsubf, idiscret,
     +                    maxlensb, isiinput, issolut, iadmin,
     +                    nsubfaces, subfaces, smsg )
c ======================================================================
c
c     programmer   Jason Frank
c     version      1.1 07-10-1999  Jason:  added send-requests
c     version      1.0 18-01-1999  Jason:  modified ismblk02 for parallel
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport certain variables (itrans,ilev) from block iblock
c     to the solut arrays of the neighbouring blocks on another node.
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c     parallel
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ilev, itrans, maxnsubf, idiscret, nblocks,
     +      ndegfd, nfaces, ni, nj, nk, ntimlv, nvirtual,
     +      maxlensb
      integer  iadmin(1:nblocks)
      integer  isiinput(1:nblocks)
      integer  issolut(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iadmin    i  reserved
c     idiscret  i  Type of discretization
c     ilev      i  Time level of the multi-block transport of information
c     isiinput  i  Array of sequence numbers (nblocks) of array iinput
c     issolut   i  Array of sequence numbers (nblocks) of array solut
c     itrans    i  reserved
c     maxlensb  i  Maximum submessage length
c     maxnsubf  i  Maximum number of subfaces encountered on all subfaces
c     nblocks   i  Number of blocks
c     ndegfd    i  Number of degrees of freedom
c     nfaces    i  Number of faces
c     ni        i  Number of non-virtual cells in the i-direction (one block)
c     nj        i  Number of non-virtual cells in the j-direction (one block)
c     nk        i  Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i  Array containing the number of subfaces of each face,
c                  see array itopol for an extended description
c     ntimlv    i  Number of time levels
c     nvirtual  i  Number of rows of virtual cells
c     smsg     i/o reserved
c     subfaces  i reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ibto, iface, ifto, ipiinput, ipsolut, isfto,
     +        isubface, nito, njto, nkto, lengmpi, imsgtag, isolv,
     +        ireq(nblocks*nfaces*maxnsubf), isndcnt

c     iblock     Block sequence number
c     ibto       Target block
c     iface      Face number
c     ifto       Target face
c     imsgtag    MPI message tag
c     ipiinput   Starting address of array iinput in ibuffr
c     ipsolut    Starting address of array solut in buffer
c     ireq       array of send requests
c     isfto      Target subface
c     isndcnt    send-request counter
c     isolv      Called by solver for periodic b.c. isolv=1 else isolv=0
c     isubface   Subface number
c     lengmpi    length of MPI message
c     nito       I-dimension of neighbour
c     njto       J-dimension of neighbour
c     nkto       K-dimension of neighbour
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddget, isdpgetp, isingetp, isddput

c     ISACTIVE       Activate old permanent array
c     ISDDGET        Get virtual unknowns from a certain subface of a certain
c                    block and put them into an intermediate representation
c     ISDDPUT        Copy virtual unknowns to a certain subface of a certain
c                    block
c     ISDPGETP       Get the address of a double precision array in buffer
c     ISINGETP       Get the address of an integer array in ibuffr
c     ISRECMPI       receive a message from another process
c     ISSENMPI       send a message to another process
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- this function always called for multiblock solution

      isolv = 0
      isndcnt = 0

c     --- loop across blocks

      do iblock = 1, nblocks

c     --- loop across the interfaces with neighbouring blocks

         do iface = 1, nfaces
            do isubface = 1, nsubfaces(iblock, iface)
               ibto = subfaces(iblock, iface, isubface, 5)

c              --- internal subface???

               if ( ibto.ne.0 ) then

c              --- choose a unique message tag

                  imsgtag = 100 + isubface*(nfaces*nblocks)
     +                       + iface*nblocks + iblock

                  if ( iadmin (iblock).eq.inodenr .and.
     +                  iadmin (ibto).ne.inodenr ) then

c                 --- iblock is on this node, neighbor on another

                     call isactive(issolut(iblock))
                     ipsolut  = isdpgetp(issolut(iblock))
                     call isactive(isiinput(iblock))
                     ipiinput = isingetp(isiinput(iblock))

                     ni       = ibuffr(ipiinput + 51 - 1)
                     nj       = ibuffr(ipiinput + 52 - 1)
                     nk       = ibuffr(ipiinput + 53 - 1)
                     ifto = subfaces(iblock, iface, isubface, 6)
                     isfto = subfaces(iblock, iface, isubface, 7)

                     lengmpi = isddget(iblock, iface, isubface, itrans,
     +                         ilev, ni, nj, nk, ndegfd, ntimlv,
     +                         nvirtual, nblocks, nfaces, maxnsubf,
     +                         idiscret, isolv, buffer(ipsolut),
     +                         nsubfaces, subfaces, smsg )

                     isndcnt = isndcnt + 1
                     call issenmpi (smsg, lengmpi, ISDOUBLE,
     +                              iadmin(ibto), imsgtag,
     +                              ireq(isndcnt))

                  else if ( iadmin (ibto).eq.inodenr .and.
     +                      iadmin (iblock).ne.inodenr ) then

c                 --- ibto is on this node, iblock on another

                     lengmpi = maxlensb
                     call isrecmpi (smsg, lengmpi, ISDOUBLE,
     +                              iadmin(iblock), imsgtag)

                     call isactive (issolut(ibto))
                     ipsolut  = isdpgetp( issolut(ibto))
                     call isactive (isiinput(ibto))
                     ipiinput = isingetp( isiinput(ibto))

                     nito  = ibuffr(ipiinput + 51 - 1)
                     njto  = ibuffr(ipiinput + 52 - 1)
                     nkto  = ibuffr(ipiinput + 53 - 1)
                     ifto = subfaces(iblock, iface, isubface, 6)
                     isfto = subfaces(iblock, iface, isubface, 7)

                     lengmpi = isddput(ibto, ifto,  isfto, itrans, ilev,
     +                          nito, njto, nkto, ndegfd,
     +                          ntimlv, nvirtual,
     +                          nblocks, nfaces, maxnsubf, idiscret,
     +                          isolv, buffer(ipsolut), nsubfaces,
     +                          subfaces, smsg)
                  end if
               end if
            end do
         end do
      end do

c     --- wait for messages to be sent, if any

      call iswaitall ( isndcnt, ireq )

      end
