      subroutine isreflhs(nblocks, isiinput, issolut, iadmin )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.3 date 23-11-1998 Jason: isrecmpi version
c     version      1.2 date 31-08-1998 Jason: remove check nodeid
c     version      1.1 date 09-06-1998 Charles: nvpoints changed to npointsv
c     version      1.0 date 19-04-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Receive field from node at host
c
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     receive
c     field
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  isiinput(*), issolut(*) , iadmin(*), nblocks

c     iadmin    i    reserved
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, iblockd, ifirst, inodrc, ipiinput,
     +      ipsolut, itype, length, ndim, ni, nj, nk, nodenr,
     +      nvirtual, npointsv

c     iblock    Block sequence number
c     iblockd   Dummy argument
c     ifirst    reserved
c     inodrc    Actual node number received from
c     ipiinput  Starting address of array iinput in ibuffr
c     ipsolut   Starting address of array solut in buffer
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     length    Length of the array in words
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nodenr    Sequence number of node to which message must be sent.
c               If the message must be sent to the host, nodenr should be 0.
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp, isdpleng

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISDPLENG  Return with the length of the array in double precisions
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISRECMPI  Receive data using MPI
c     ISSOURCE  Determine source of first context-matching communication
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     for all nodes do
c
c        first receive node number inodrc
c
c           for all blocks on inodrc receive field
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isreflhs')

      do nodenr = 1, iacnodes

c     --- for all nodes not on the host
c         get the first message from a node

	 call issource(inodrc, ICFIELD)

         do iblock = 1, nblocks
            if ( iadmin(iblock).eq.inodrc ) then

c           --- for all blocks on node inodrc

               call isactive(isiinput(iblock))
               call isactive(issolut(iblock))

               ipiinput = isingetp(isiinput(iblock))
               ipsolut = isdpgetp( issolut(iblock))

               ni       = ibuffr(ipiinput + 51 - 1)
               nj       = ibuffr(ipiinput + 52 - 1)
               nk       = ibuffr(ipiinput + 53 - 1)
               ndim     = ibuffr(ipiinput + 57 - 1)
               nvirtual = ibuffr(ipiinput + 54 - 1)
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
               if ( ndim.eq.3 ) then
                  npointsv = npointsv*(nk+1+2*nvirtual)
               end if

               ifirst = 2
               itype  = 6
               length = isdpleng( issolut(iblock) )

c              --- Only a single time level needs to be send.
c                  However, pressure is stored at time level ntimlv-1 instead of
c                  ntimlv
c                             length = isdpleng( issolut(iblock) ) /ntimlv
c                             ipsolut = ipsolut + (ntimlv-1)*npointsv

	       call isrecmpi (buffer(ipsolut), length, ISDOUBLE, inodrc,
     +            ICFIELD)

            end if
         end do

      end do

      call iserclos('isreflhs')

      end
