      subroutine isrdmult ( nmain, codemn, nmcode, iretur, iinblk,
     +                      rinblk, maxtrans, maxturb, icheckeq )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.8    date 15-07-1999 Guus: extra keywords
c        version 1.7    date 21-12-1998 Guus: update coments
c        version 1.6    date 18-11-1996 Guus: extra keywords
c        version 1.5    date 06-04-1995 Kees K. tabout declared as array
c        version 1.4    date 08-03-1995 Kees K : bug fixed for maxt
c        version 1.3    date 27-02-1995 Erik: added truncation_strategy
c                                       keyword.
c        version 1.2    date 16-01-1995 Erik: added save_preconditioner
c                                       keyword
c        version 1.1    date 04-01-1995 Erik: added subdomain_solution
c                                       keyword
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read multi block part of input files and store result in
c    arrays iinblk and rinblk
c **********************************************************************
c
c                       KEYWORDS
c
c       multiblock
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, maxtrans, maxturb,
     +        iinblk(10,maxtrans+3+maxturb),
     +        icheckeq(maxtrans+2+maxturb)
      double precision rinblk(10,maxtrans+2+maxturb)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     icheckeq       Work array to indicate if an equation has been considered
c                    (icheckeq(i)=1 or not icheckeq(i)=0)
c     iinblk    o    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rinblk    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, newmain, lenchar, nncode, nncod2, nout
      parameter ( nmainnw=50, newmain=8, nncode=16, nncod2=8,
     +            lenchar=14, nout=1 )
      integer nmcode1(newmain), nmcoda(nmainnw), itype(nncode),
     +        nmcode2(nncod2), ncodmx(nncode), nstrng,
     +        ncode(nncode), i, iequa, ih, iseq, iseqprev, keyprev,
     +        maxt, nmain1, nmmaintot, numsub
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(1)
      character *14 codema(nmainnw), code1(newmain)
      character *16 code2(nncod2)
      character *19 code(nncode)
      character *1 strout(nout)
      logical ready

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     iequa     General equation sequence number
c     ih        Help variable to store a constant temporarily
c     iseq      Sequence number read after transport or turbulence equation
c     iseqprev  Previous value of iseq
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     keyprev   Sequence number of previous keyword found with respect to code
c     lenchar   Number of characters in main keywords
c     maxt      Maximum number of equations allowed for a specific type
c               With respect to transport: maxtrans-10
c               With respect to turbulence: 10
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     newmain   Number of main keywords stored in array code1
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmmaintot Total number of keywords in codema
c     nncod2    Number of subsubkeywords stored in array code2
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     numsub    Number of subkeywords that may be detected by isintx02
c               In the first step numsub=0 for any other step numsub=nncode
c     ready     Indicates if a while loop is finished or not
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword linear_solver
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    TYPE_OF_ALGorithm = itype
c    MOMentum_equations
c    TRANSport_equation i or all
c    PRESsure_equation
c    TURBulence_equation i or all
c    ALL_equations
c
c    The following values for itype are available:
c
c        itype = SEQuential
c        itype = PARallel
c
c    These keywords itself may be followed by subkeywords that indicate
c     how the linear equations must be solved.
c    The following subkeywords are recognized:
c
c    METHOD = imethod
c    MAXITer = n
c    KRYLOV_OUter = n
c    KRYLOV_INner = n
c    RELACcuracy = rel
c    RELAXation = r
c    REUSE_SEArch = n
c    REUSE_VALues = y/n
c    ORTHOGONALIZATION = iortho
c
c    imethod indicates the type of algorithm to be used.
c    Possible values:
c
c          ACCelerated
c          STAndard
c
c    iortho indicates the type of orthogonalizations to be used.
c    Possible values:
c
c          modified
c          reorthogonalized
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     530   integer number read out of range
c     531   Internal error in subroutine; character array too small
c     567   Equation has been found more than once
c     586   incorrect keyword found after keyword
c     587   two keywords found that are mutually exclusive
c     592   mutually exclusive keywords of a set of three were found
c     713   more search directions than Krylov vectors
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the local main keywords
c    and the global main keywords and the subsubkeywords
c    ready := false
c    number_of_subkeywords := 0
c    while ( not ready ) do
c       iseq := 0
c       Read next subkeywords until a new main keyword is found
c       if ( main keyword = transport or turbulence ) then
c          Read sequence number iseq
c       if ( number_of_subkeywords=0 ) then
c          number_of_subkeywords := actual number
c       else
c          interpret subkeywords as function of keyword_previous
c       if ( keyword found = main keyword ) then
c          ready := true
c       keyword_previous := keyword found
c       iseq_previous := iseq
c
c    Remark:
c
c      The first array iinblk/rinblk(.,1) corresponds to general part
c      The array iinblk/rinblk(.,2) corresponds to the momentum equations
c      The array iinblk/rinblk(.,3) corresponds to the pressure equations
c      The arrays iinblk/rinblk(.,4 to maxtrans+3) correspond to the transport
c      equations 1 to maxtrans
c      The arrays iinblk/rinblk(maxtrans+4 to maxtrans+3+maxturb) correspond to
c      the turbulence equations 1 to maxturb
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'MOMENTUM_EQ', 'PRESSURE_EQ', 'TRANSPORT_EQ',
     +             'TURBULENCE_EQ', 'ALL_EQUATIONS', 'TYPE_OF_ALGORI',
     +             'SUBDOMAIN_SOLU', 'SAVE_PRECONDIT' /
      data nmcode1 / 3, 4, 4, 12, 4, 11, 9, 9 /
      data code  / 'METHOD', 'MAXITER', 'KRYLOV_OUTER', 'KRYLOV_INNER',
     +             'RELACCURACY', 'RELAXATION', 'SEQUENTIAL',
     +             'PARALLEL', 'ACCURATE', 'INACCURATE', 'ILU',
     +             'AMOUNT_OF_OUTPUT', 'TRUNCATION_STRATEGY',
     +             'REUSE_SEARCH', 'REUSE_VALUES', 'ORTHOGONALIZATION' /
      data ncode  / 6, 5,  9,  9,  5,    5,  3, 3, 3, 5,
     +              3, 4, 10,  9,  9,    5 /
      data ncodmx / 6, 7, 12, 12, 11,   10, 10, 8, 8, 10,
     +              3,16, 19, 12, 12,   17 /
      data itype  / 5, 0,  0,  0,  1,    1,  2, 2, 2, 2,
     +              2, 0,  5,  0,  5,    5 /
      data tabmin / 0d0, 1d0, 0d0, 0d0,1d-15, 1d-15, 0d0, 0d0, 0d0, 0d0,
     +              0d0,-1d0, 0d0, 0d0, 0d0,  0d0 /
      data tabmax / 0d0, 1d3, 1d2, 1d2, 1d-1, 1d0,   0d0, 0d0, 0d0, 0d0,
     +              0d0, 2d0, 0d0, 1d2, 0d0,  0d0 /
      data code2 / 'ACCELERATED', 'STANDARD',
     +             'NONE', 'JACKSON_ROBINSON', 'NO', 'YES',
     +             'MODIFIED', 'REORTHOGONALIZED' /
      data nmcode2 / 3, 3, 4, 7, 2, 3, 3, 5 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  METHOD
c               2:  MAXITER
c               3:  KRYLOV_OUTER
c               4:  KRYLOV_INNER
c               5:  RELACCURACY
c               6:  RELAXATION
c               7:  SEQUENTIAL
c               8:  PARALLEL
c               9:  ACCURATE
c              10:  INACCURATE
c              11:  ILU
c              12:  AMOUNT_OF_OUTPUT
c              13:  TRUNCATION_STRATEGY
c              14:  REUSE_SEARCH
c              15:  REUSE_VALUES
c              16:  ORTHOGONALIZATION
c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  MOMENTUM_EQUATIONS
c               2:  PRESSURE_EQUATION
c               3:  TRANSPORT_EQUATION
c               4:  TURBULENCE_EQUATION
c               5:  ALL_EQUATIONS
c               6:  TYPE_OF_ALGORITHM
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  ACCELERATED
c               2:  STANDARD
c               3:  NONE
c               4:  JACKSON_ROBINSON
c               5:  NO
c               6:  YES
c               7:  MODIFIED
c               8:  REORTHOGONALIZED
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdmult' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords and the subsubkeywords

      nmain1 = newmain+nmain
      nmmaintot = nmain1+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( newmain, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, newmain
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
      end do
      do i = 1, nmain
         codema(i+newmain)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+newmain) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain1)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain1) = nmcode2(i)
      end do

c     --- Clear array icheckeq

      do i = 1, maxtrans+2+maxturb
         icheckeq(i) = 0
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(1) = nmain1+1
      tabmax(1) = tabmin(1)+1
      tabmin(13) = tabmax(1)+1
      tabmax(13) = tabmin(13)+1
      tabmin(15) = tabmax(13)+1
      tabmax(15) = tabmin(15)+1
      tabmin(16) = tabmax(15)+1
      tabmax(16) = tabmin(16)+1
      ready = .false.
      numsub = 0

200   if ( .not. ready ) then

c     --- while loop ready

         iseq = 0

c        --- Read next subkeywords until a new main keyword is found

         do i = 1, numsub
            table(i) = -1d20
         end do
         table(14) = 0d0
         nstrng = 1
         call isintx02 ( code, ncode, numsub, itype, table, codema,
     +                   nmcoda, nmain1, iretur, tabmin, tabmax,
     +                   tabout, nout, strout, nstrng, ncodmx )
         if ( iretur.eq.3 .or. iretur.eq.4 ) then

c        --- transport or turbulence found
c            Read sequence number iseq

            call isasknxt
            if ( ityprd.eq.2 ) then

c           --- number found

               iseq = nint(realrd)
               if ( iretur.eq.3 ) then
                  ih = maxtrans
               else
                  ih = maxturb
               end if
               if ( iseq.lt.1 .or. iseq.gt.ih ) then

c              --- iseq out of range

                  call iserrchr ( codema(iretur), 1 )
                  call iserrint ( iseq, 1 )
                  call iserrint ( 1, 2 )
                  call iserrint ( ih, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  call iserrcrd
                  iseq = 1

               end if

            else if ( ityprd.eq.4 .and. string(1:3).eq.'ALL' ) then

c           --- text all found set iseq equal to -iretur

               iseq = -iretur

            else

c           --- Error not an integer number found

               call iserrchr ( codema(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

         end if

         if ( numsub.eq.0 ) then

c        --- number_of_subkeywords = 0

            numsub = nncode

         else

c        --- interpret subkeywords as function of keyword_previous

            if ( keyprev.lt.6 ) then

c           --- Information for the equations

               if ( keyprev.eq.1 .or. keyprev.eq.2 ) then
                  iequa = keyprev
               else if ( keyprev.eq.3 ) then
                  if ( iseqprev.gt.0 ) then
                     iequa = 2+iseqprev
                  else
                     iequa = 3
                  end if
               else if ( keyprev.eq.4 ) then
                  if ( iseqprev.gt.0 ) then
                     iequa = iseqprev+maxtrans+2
                  else
                     iequa = 3+maxtrans
                  end if
               else
                  iequa = 1
               end if

c              --- Test if equation has already been treated before

               if ( icheckeq(iequa).ne.0 ) then
                  call iserrint ( iseq, 1 )
                  call iswarsub ( 567, 1, 0, 0 )
                  call iserrcrd
               end if

c              --- Fill icheckeq

               do i = iequa, iequa+maxt-1
                  icheckeq(i) = 1
               end do

               if ( table(1).gt.0 ) then

c              --- method has been found

                  ih =  nint(table(1))-1
                  do i = iequa+1, iequa+maxt
                     iinblk(1,i) = ih
                  end do

               end if

               if ( table(2).gt.0 ) then

c              --- maxiter has been found

                  ih =  nint(table(2))
                  do i = iequa+1, iequa+maxt
                     iinblk(2,i) = ih
                  end do

               end if

               if ( table(3).ge.0 ) then

c              --- Krylov_outer has been found

                  ih =  nint(table(3))
                  do i = iequa+1, iequa+maxt
                     iinblk(3,i) = ih
                  end do

               end if

               if ( table(4).ge.0 ) then

c              --- Krylov_inner has been found

                  ih =  nint(table(4))
                  do i = iequa+1, iequa+maxt
                     iinblk(4,i) = ih
                  end do

               end if

               if ( table(5).ge.0 ) then

c              --- relaccuracy has been found

                  do i = iequa, iequa+maxt-1
                     rinblk(1,i) = table(5)
                  end do

               end if

               if ( table(6).ge.0 ) then

c              --- relaxation has been found

                  do i = iequa, iequa+maxt-1
                     rinblk(2,i) = table(6)
                  end do

               end if

               do i = 7, 11
                  if ( table(i).gt.0d0 ) then
                     call iserrchr ( code(i), 1 )
                     call iserrchr ( code1(keyprev), 2 )
                     call iserrsub ( 586, 0, 0, 2 )
                  end if
               end do

               if ( table(12).ge.0 ) then

c              --- amount of output has been found

                  ih =  nint(table(12))
                  do i = iequa+1, iequa+maxt
                     iinblk(5,i) = ih
                  end do

               end if

               if ( table(13).ge.0 ) then

c              --- truncation_strategy has been found

                  ih =  nint(table(13))-1
                  do i = iequa+1, iequa+maxt
                     iinblk(6,i) = ih
                  end do

               end if
               ih =  nint(table(14))
               if ( ih.gt.iinblk(3,iequa+1) ) then

c              --- reuse_search > krylov_outer

                  call iserrint ( ih, 1 )
                  call iserrint ( iinblk(3,iequa+1), 2 )
                  call iserrint ( iequa+1, 3 )
                  call iswarsub ( 713, 3, 0, 0 )
                  ih = iinblk(3,iequa+1)

               end if
               do i = iequa+1, iequa+maxt
                  iinblk(7,i) = ih
               end do

               if ( table(15).gt.0 ) then

c              --- reuse_values has been found

                  ih =  nint(table(15))-1
                  do i = iequa+1, iequa+maxt
                     iinblk(8,i) = ih
                  end do

               end if

               if ( table(16).gt.0 ) then

c              --- orthogonalization has been found

                  ih =  nint(table(16))-1
                  do i = iequa+1, iequa+maxt
                     iinblk(9,i) = ih
                  end do

               end if

            else

               if ( keyprev.eq.6 ) then

c              --- type of algorithm found

                  do i = 1,6
                     if ( table(i).gt.0d0 ) then
                        call iserrchr ( code(i), 1 )
                        call iserrchr ( code1(keyprev), 2 )
                        call iserrsub ( 586, 0, 0, 2 )
                     end if
                  end do
                  do i = 9, nncode
                     if ( table(i).gt.0d0 ) then
                        call iserrchr ( code(i), 1 )
                        call iserrchr ( code1(keyprev), 2 )
                        call iserrsub ( 586, 0, 0, 2 )
                     end if
                  end do

                  if ( table(7).gt.0 ) iinblk(1,1) = 0
                  if ( table(8).gt.0 ) iinblk(1,1) = 1
                  if ( table(7).gt.0d0 .and. table(8).gt.0d0 ) then

c                 --- both sequential and parallel present

                     call iserrchr ( code(7), 1 )
                     call iserrchr ( code(8), 2 )
                     call iserrsub ( 587, 0, 0, 2 )
                  end if

               else if ( keyprev.eq.7 ) then

c              --- subdomain_solution found

                  do i = 1, 8
                     if ( table(i).gt.0d0 ) then
                        call iserrchr ( code(i), 1 )
                        call iserrchr ( code1(keyprev), 2 )
                        call iserrsub ( 586, 0, 0, 2 )
                     end if
                  end do
                  do i = 12, nncode
                     if ( table(i).gt.0d0 ) then
                        call iserrchr ( code(i), 1 )
                        call iserrchr ( code1(keyprev), 2 )
                        call iserrsub ( 586, 0, 0, 2 )
                     end if
                  end do

                  if ( table(9) .gt.0 ) iinblk(2,1) = 0
                  if ( table(10).gt.0 ) iinblk(2,1) = 1
                  if ( table(11).gt.0 ) iinblk(2,1) = 2

                  table(9)  = MAX(table(9), 0d0)
                  table(10) = MAX(table(10), 0d0)
                  table(11) = MAX(table(11), 0d0)
                  if ( table(9) + table(10) + table(11).gt.1.5 ) then

                    call iserrchr ( code(9),  1 )
                    call iserrchr ( code(10), 2 )
                    call iserrchr ( code(11), 3 )
                    call iserrsub ( 592, 0, 0, 3 )

                  end if

               else if ( keyprev.eq.8 ) then

c              --- save_preconditioner found, no subkeywords allowed

                  do i = 1, nncode
                     if ( table(i).gt.0d0 ) then
                        call iserrchr ( code(i), 1 )
                        call iserrchr ( code1(keyprev), 2 )
                        call iserrsub ( 586, 0, 0, 2 )
                     end if
                  end do

                  iinblk(3,1) = 1

               end if

            end if

         end if

         if ( iretur.gt.newmain ) then

c        --- main keyword found

            ready = .true.
            iretur = iretur-newmain

         else if ( iretur.le.-1 ) then

c        --- end of file found

            ready = .true.

         end if
         if ( iseq.eq.-3 ) then
            maxt = maxtrans
         else if ( iseq.eq.-4 ) then
            maxt = maxturb
         else if ( iretur.eq.5 ) then
            maxt = maxtrans+maxturb+2
         else
            maxt = 1
         end if
         keyprev = iretur
         iseqprev = iseq
         go to 200

      end if

      call iserclos ( 'isrdmult' )
      end
