      subroutine isrdturb ( nmain, codemn, nmcode, iretur, iintur,
     +                      rintur, idimens )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.8    date 17-09-1997 Guus: New keywords for LES
c        version 1.7    date 14-07-1997 Guus: extension of nmainnw
c        version 1.6    date 10-09-1996 Marcel extension closure constants
c        version 1.5    date 23-08-1996 Marcel extended anisotropic models
c        version 1.4    date 23-05-1996 Marcel extended k-eps model
c        version 1.3    date 22-05-1996 Marcel Z. Kato-Launder corrrection
c        version 1.2    date 06-04-1995 Kees K. tabout declared as array
c        version 1.1    date 27-10-1994 Marcel Zijlema: extension with
c                                                     Wilcox k-omega model
c        version 1.0    date 06-09-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read turbulence part of input files and store result in
c    arrays iintur and rintur
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, iintur(100), idimens(100)
      double precision rintur(100)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     idimens  i/o   Integer dimension parameters for the intermediate file
c     iintur    o    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rintur    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, lenchar, nncode, nncod1, nncod2, nncod3, nncod4
      parameter ( nmainnw=50, nncode=30, nncod1=2, nncod2=7, nncod3=5,
     +            nncod4=1, lenchar=14 )
      integer nmcoda(nmainnw), itype(nncode), nmcode1(nncod1),
     +        nmcode2(nncod2), nmcode3(nncod3), ncodmx(nncode), nout,
     +        nstrng, ncode(nncode), i, nmmaintot, ih, nextra,
     +        nmcode4(nncod4)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(1)
      character *14 codema(nmainnw), code1(nncod1), code2(nncod2)
      character *19 code(nncode), code3(nncod3), code4(nncod4)
      character *1 strout(1)

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     code3     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     code4     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     ih        Help variable to store a constant temporarily
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     lenchar   Number of characters in main keywords
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     nextra    Number of extra positions already used in array
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmcode3   Array containing the number of significant characters for
c               each keyword in code3
c     nmcode4   Array containing the number of significant characters for
c               each keyword in code4
c     nmmaintot Total number of keywords in codema
c     nncod1    Number of subsubkeywords stored in array code1
c     nncod2    Number of subsubkeywords stored in array code2
c     nncod3    Number of subsubkeywords stored in array code3
c     nncod4    Number of subsubkeywords stored in array code4
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding to
c               the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword TURBULENCE
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    MODEL = m
c    VISCOUS_CORRECTIONS = v
c    ANISOTROPY = a
c    KAPPA = k
c    E = e
c    C_MU = c
c    SIGMA_K = s
c    SIGMA_EPS = s
c    CONE_EPS = c
c    CTWO_EPS = c
c    C_D = c
c    SIGMA_T = s
c    A = a
c    E_ROUGH = e
c    ETA_Zero = e
c    BETARNG = b
c    C_S = c
c    ALPHA = a
c    BETASTAR = b
c    BETA = b
c    SIGMA = s
c    SIGMASTAR = s
c    MOD_PRODUCTION = k
c    CTHREE_EPS = c
c    CTAU_ONE = c
c    CTAU_TWO = c
c    CTAU_THREE = c
c    LES_MODEL = c
c    MAX_WINDOWS = m
c    TIME_INTERVAL = t
c
c    The keyword MODEL must be followed by the type of model required.
c    The following models may be used:
c
c    none         no turbulence model is used
c    algebraic    not yet implemented
c    K_EPS        standard k-eps model
c    RNG_K_EPS    RNG k-eps model
c    K_OMEGA      Wilcox k-omega model
c    LES          Large eddy model (defualt if LES_MODEL is used)
c    EXT_K_EPS    extended k-eps model of Chen and Kim
c
c    The keyword LES_MODEL must be followed by the type of LES model required.
c    The following models may be used:
c
c    SMAGORINSKY  The Smagorinsky model is used
c
c    The parameter v in VISCOUS_CORRECTIONS may take one of the forms:
c
c      YES
c      NO
c
c    The parameter a in ANISOTROPY may take one of the forms:
c
c    none                no anisotropic model is used
c    SPEZIALE            model derived from continuum mechanical arguments
c    RUBINSTEIN_BARTON   model based on the RNG theory
c    NISIZIMA_YOSHIZAWA  model based on the DIA theory
c    MYONG_KASAGI        anisotropic model
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     531   Internal error in subroutine; character array too small
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the global main keywords
c    and the subsubkeywords
c    Read subkeywords until a new main keyword is found
c    interpret subkeywords
c    Fill nturb in idimens
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code  / 'MODEL', 'VISCOUS_CORRECTIONS', 'ANISOTROPY',
     +             'KAPPA', 'E', 'C_MU', 'SIGMA_K', 'SIGMA_EPS',
     +             'CONE_EPS', 'CTWO_EPS', 'C_D', 'SIGMA_T', 'A',
     +             'E_ROUGH', 'ETA_ZERO', 'BETARNG', 'C_S', 'ALPHA',
     +             'BETA_STAR', 'BETA', 'SIGMA', 'SIGMA_STAR',
     +             'MOD_PRODUCTION', 'CTHREE_EPS', 'CTAU_ONE',
     +             'CTAU_TWO', 'CTAU_THREE', 'LES_MODEL',
     +             'MAX_WINDOWS', 'TIME_INTERVAL' /
      data ncode  / 5,11, 5, 5, 1,   4, 7, 9, 8, 8,    3, 7, 1, 7, 5,
     +              7, 3, 5, 6, 4,   5, 7, 8,10, 8,    8,10, 6, 7,10 /
      data ncodmx / 5,19,10, 5, 1,   4, 7, 9, 8, 8,    3, 7, 1, 7, 8,
     +              7, 3, 5, 9, 4,   5, 10,14,10,8,    8,10, 9,11,13 /
      data itype  / 5, 5, 5, 1, 1,   1, 1, 1, 1, 1,    1, 1, 1, 1, 1,
     +              1, 1, 1, 1, 1,   1, 1, 1, 1, 1,    1, 1, 5, 0, 1 /
      data tabmin /   0d0,     0d0,    0d0, 0.40d0,  7.45d0,
     +             0.01d0,   0.7d0,  0.7d0,  1.0d0,   1.5d0,
     +             0.04d0,   0.5d0,   20d0,   26d0,     2d0,
     +             0.01d0,   0.1d0,  0.1d0, 0.04d0,  0.01d0,
     +             0.50d0,   0.5d0,    0d0,    0d0,  -5.0d0,
     +             -5.0d0,  -5.0d0,    0d0,    1d0,     0d0 /
      data tabmax /   0d0,     0d0,    0d0, 0.42d0,     1d1,
     +             0.36d0,   1.4d0,  1.4d0, 1.55d0,   2.0d0,
     +             0.15d0,   4.0d0,   30d0,   37d0,    16d0,
     +            0.015d0,  0.25d0,  1.0d0, 0.15d0,   0.1d0,
     +             2.00d0,  2.00d0,  1.0d0,  0.4d0,   5.0d0,
     +             5.00d0,  5.00d0,    0d0,    5d2,     1d5 /
      data code1 / 'YES', 'NO' /
      data code2 / 'NONE', 'ALGEBRAIC', 'K_EPS',
     +             'RNG_K_EPS', 'K_OMEGA', 'LES',
     +             'EXT_K_EPS' /
      data code3 / 'NONE', 'SPEZIALE', 'RUBINSTEIN_BARTON',
     +             'NISIZIMA_YOSHIZAWA', 'MYONG_KASAGI' /
      data code4 / 'SMAGORINSKY' /
      data nmcode1 / 3, 2 /
      data nmcode2 / 2, 3, 5, 9, 7, 3, 9 /
      data nmcode3 / 2, 3, 3, 3, 3 /
      data nmcode4 / 4 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  MODEL
c               2:  VISCOUS_CORRECTIONS
c               3:  ANISOTROPY
c               4:  KAPPA
c               5:  E
c               6:  C_MU
c               7:  SIGMA_K
c               8:  SIGMA_EPS
c               9:  CONE_EPS
c              10:  CTWO_EPS
c              11:  C_D
c              12:  SIGMA_T
c              13:  A
c              14:  E_ROUGH
c              15:  ETA_Zero
c              16:  BETARNG
c              17:  C_S
c              18:  ALPHA
c              19:  BETA_STAR
c              20:  BETA
c              21:  SIGMA
c              22:  SIGMA_STAR
c              23:  MOD_PRODCUCTION
c              24:  CTHREE_EPS
c              25:  CTAU_ONE
c              26:  CTAU_TWO
c              27:  CTAU_THREE
c              28:  LES_MODEL
c              29:  MAX_WINDOWS
c              30:  TIME_INTERVAL
c     code1     Table containing the subsubkeywords corresponding to VISCOUS_CO
c               Contents:
c               1:  YES
c               2:  NO
c     code2     Table containing the subsubkeywords corresponding MODEL
c               Contents:
c               1:  NOne
c               2:  ALGebraic
c               3:  K_EPS
c               4:  RNG_K_EPS
c               5:  K_OMEGA
c               6:  LES
c               7:  EXT_K_EPS
c     code3     Table containing the subsubkeywords corresponding to ANISOTROPY_
c               Contents:
c               1:  NOne
c               2:  SPEziale
c               3:  RUBinstein_barton
c               4:  NISizima_yoshizawa
c               5:  MYOng_kasagi
c     code4     Table containing the subsubkeywords corresponding to LES_MODEL
c               Contents:
c               1:  SMAGorinsky
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdturb' )

c     --- Make table of new main keywords consisting of the global main keywords
c         and the subsubkeywords

      nmmaintot = nmain+nncod1+nncod2+nncod3
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( nncod1, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, nmain
         codema(i)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i) = nmcode(i)
      end do
      do i = 1, nncod1
         codema(i+nmain)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i+nmain) = nmcode1(i)
      end do
      nextra = nncod1
      do i = 1, nncod2
         codema(i+nmain+nextra)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain+nextra) = nmcode2(i)
      end do
      nextra = nextra + nncod2
      do i = 1, nncod3
         codema(i+nmain+nextra)(1:lenchar) = code3(i)(1:lenchar)
         nmcoda(i+nmain+nextra) = nmcode3(i)
      end do
      nextra = nextra + nncod3
      do i = 1, nncod4
         codema(i+nmain+nextra)(1:lenchar) = code4(i)(1:lenchar)
         nmcoda(i+nmain+nextra) = nmcode4(i)
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(1) = nmain+nncod1+1
      tabmax(1) = tabmin(1)+6
      tabmin(2) = nmain+1
      tabmax(2) = tabmin(2)+1
      tabmin(3) = nmain+nncod1+nncod2+1
      tabmax(3) = tabmin(3)+4
      tabmin(28) = nmain+nncod1+nncod2+nncod3+1
      tabmax(28) = tabmin(28)+4

c     --- Read subkeywords until a main keyword is found

      do i = 1, nncode
         table(i) = -1d20
      end do
      table(1) = -1d0
      table(2) = -1d0
      table(3) = -1d0
      table(28) = -1d0
      table(29) = 20d0
      table(30) = 1d0

      nout = 1
      nstrng = 1
      call isintx02 ( code, ncode, nncode, itype, table, codema,
     +                nmcoda, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

c     --- interpret subkeywords

      ih = nint(table(1))
      if ( ih.eq.1 ) then
         iintur(2) = 0
      else if ( ih.ge.3 ) then
         iintur(1) = 2
         if ( ih.eq.6 ) iintur(1) = 0
         iintur(2) = ih-2
         if ( ih.eq.4 ) then

c        --- RNG model found
c            in this case it is necessary to use other default values

             rintur(3) = 0.085d0
             rintur(4) = 0.7179d0
             rintur(5) = 0.7179d0
             rintur(6) = 1.42d0
             rintur(7) = 1.68d0

         else if ( ih.eq.7 ) then

c        --- extended k-eps model found
c            in this case it is necessary to use other default values

             rintur(4) = 0.75d0
             rintur(5) = 1.15d0
             rintur(6) = 1.35d0
             rintur(7) = 1.9d0

         end if
      end if

      ih = nint(table(2))
      if ( ih.eq.1 ) then
        iintur(3) = 1
      else if ( ih.eq.2 ) then
        iintur(3) = 0
      end if

      ih = nint(table(3))
      if ( ih.gt.0 ) then

        iintur(4) = ih-1
        if ( ih.eq.2 ) then

c       --- Speziale model found

          rintur(22) = 0.1512d0
          rintur(23) = 0.1512d0
          rintur(24) = 0d0

        else if ( ih.eq.3 ) then

c       --- Rubinstein and Barton model found

          rintur(22) =  0.68d0
          rintur(23) =  0.14d0
          rintur(24) = -0.56d0

        else if ( ih.eq.4 ) then

c       --- Nisizima and Yoshizawa model found

          rintur(22) = -0.7881d0
          rintur(23) =  0.1769d0
          rintur(24) =  1.0675d0

        else if ( ih.eq.5 ) then

c       --- Myong and Kasagi model found

          rintur(22) = 0.2750d0
          rintur(23) = 0.2375d0
          rintur(24) = 0.0500d0

        end if

      end if
      ih = nint(table(28))
      if ( ih.eq.1 ) then

c     --- Smagorinski found

         if ( iintur(2).ne.0 .and. iintur(2).ne.4 ) then
         end if
         iintur(1) = 0
         iintur(2) = 4
         iintur(5) = 1
         iintur(6) = nint(table(29))
         rintur(25) = table(30)
      end if

      do i = 4, 27
         if ( table(i).gt.-1d19 ) rintur(i-3) = table(i)
      end do

c     --- Fill nturb in idimens

      idimens(6) = iintur(1)

      call iserclos ( 'isrdturb' )
      end
