      subroutine isreadfrs ( iinfrsur, rinfrsur, dnumbr, igeotime )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 18-03-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the freesurface model from the
c    intermediate file and store in arrays iinfrsur and rinfrsur for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    free_surface
c    compressible
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinfrsur(*), igeotime
      double precision dnumbr(*), rinfrsur(*)

c     dnumbr         Work array to store a series numbers that have been read
c     igeotime    i    If 1 a free surface model is used, if 0 not
c     iinfrsur    o    Contains integer input with respect to the free surface
c                      computation
c     rinfrsur    o    Contains real input with respect to the free surface
c                      computation
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum

c     inum      Indicates the number of numbers that have been read
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 reals with respect to iinfrsur
c     100 reals with respect to rinfrsur
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     512   rinfrsur(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadfrs' )

c     --- Read iinfrsur

      call isreadrl ( dnumbr, 100, inum )
      iinfrsur(1) = nint(dnumbr(1))
      if ( igeotime.gt.0 ) then

c     --- Free surface flow, test parameters

         if ( igeotime.lt.0 .or. igeotime.gt.1 ) then
            call iserrint ( igeotime, 1 )
            call iserrchr ( 'igeotime', 1 )
            call iserrsub ( 512, 1, 0, 1 )
         end if

      end if

c     --- Read rinfrsur

      call isreadrl ( rinfrsur, 10, inum )

      call isreadrl ( dnumbr, 90, inum )

      call iserclos ( 'isreadfrs' )
      end
