      subroutine iswrtif5 ( idimens, maxtrans, maxturb, iinblk,
     +                      rinblk, ndim, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.3    date 18-11-1996 Guus: extra keywords
c        version 1.2    date 27-02-1995 Erik: added 'truncation_strategy'
c        version 1.1    date 16-01-1995 Erik: added 'save_preconditioner'
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write fifth part of the intermediate file
c    It concerns the blocks 10 to 10
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100), maxturb, maxtrans,
     +        iinblk(10,maxtrans+3+maxturb), ndim, icoupled
      double precision rinblk(10,maxtrans+2+maxturb)

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iinblk    i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     rinblk    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, itrans, ntrnsp, nturb

c     i         General loop variable
c     itrans    reserved
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 10: Parameters with respect to the multi block process
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get ntrnsp and nturb
c     write general parameters
c     write parameters for momentum equation
c     write parameters for pressure equation
c     write parameters for transport equations
c     write parameters for turbulence equations
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write information for momentum equation

      write(irefwr,100) (iinblk(i,1),i=1,10)
100   format ( /'**** BLOCK 10: Parameters with respect to the',
     +          ' multi-block process'//
     +          ' * iinblk:'//
     +          '** general information'//
     +          'method             = ',i5/
     +          'subdomain_solution = ', i5/
     +          'save precondit.    = ', i5/
     +          7i2,2x,' # Positions  4-10 not yet defined'/ )

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,110)
         write(irefwr,120) (iinblk(i,2),i=1,10)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,130)
         write(irefwr,120) (iinblk(i,2),i=1,10)

c        --- Next v-momentum

         write(irefwr,140)
         write(irefwr,120) (iinblk(i,2),i=1,10)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,150)
            write(irefwr,120) (iinblk(i,2),i=1,10)

         end if

      end if

110   format ( /'** Momentum equations'// )
120   format ( /'algorithm           =', i5/
     +          'maxiter             =', i5/
     +          'Krylov_outer        =', i5/
     +          'Krylov_inner        =', i5/
     +          'Amount_of_output    =', i5/
     +          'Truncation strategy =', i5/
     +          'Reuse_search        =', i5/
     +          'Reuse_values        =', i5/
     +          2i2,10x,' # Positions  9-10 not yet defined'/ )
130   format ( /'** u-momentum equations'// )
140   format ( /'** v-momentum equations'// )
150   format ( /'** w-momentum equations'// )

c     --- write information for pressure equation

      write(irefwr,180)
180   format ( /'** Pressure equations'// )
      write(irefwr,120) (iinblk(i,3),i=1,10)

c     --- write information about the transport equations

      do itrans = 1, ntrnsp
         write(irefwr,200) itrans
200      format ( /'** transport equation',i2// )
         write(irefwr,120) (iinblk(i,3+itrans),i=1,10)
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,300) itrans
300      format ( /'** turbulence equation',i2// )
         write(irefwr,120) (iinblk(i,3+itrans+maxtrans),i=1,10)
      end do

c     --- Array rinblk:

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,400)
         write(irefwr,410) (rinblk(i,1),i=1,10)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,420)
         write(irefwr,410) (rinblk(i,1),i=1,10)

c        --- Next v-momentum

         write(irefwr,430)
         write(irefwr,410) (rinblk(i,1),i=1,10)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,440)
            write(irefwr,410) (rinblk(i,1),i=1,10)

         end if

      end if

400   format ( /'* rinblk :'//'** Momentum equations'// )
410   format ( /'relative accuracy    =', d15.7/
     +          'relaxation parameter =', d15.7/
     +          8f4.0,6x,' # Positions  8-10 not yet defined'/ )
420   format ( /'* rinblk :'//'** u-momentum equations'// )
430   format ( //'** v-momentum equations'// )
440   format ( //'** w-momentum equations'// )

      write(irefwr,180)
      write(irefwr,410) (rinblk(i,2),i=1,10)

c     --- write information about the transport equations

      do itrans = 1, ntrnsp
         write(irefwr,200) itrans
         write(irefwr,410) (rinblk(i,2+itrans),i=1,10)
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,300) itrans
         write(irefwr,410) (rinblk(i,2+itrans+maxtrans),i=1,10)
      end do

      end
