      subroutine iswrtif7 ( nblocks, ndim, idimens, iinbc, rinbc,
     +                      maxtrans, maxturb, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 26-11-1997 Print also the pressure
c                                       Extra parameter icoupled
c        version 1.5    date 17-09-1997 Change due to decoupled approach
c        version 1.4    date 08-09-1997 New block number
c        version 1.3    date 03-04-1996 Kees K. boundary conditions
c                                       for the pressure equation
c        version 1.2    date 20-03-1996 Layout
c        version 1.1    date 08-02-1995 Kees K : 3D
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write eigth part of the intermediate file
c    It concerns the blocks 15 to 15
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, ndim, idimens(100), iinbc(nblocks,*), maxtrans,
     +        maxturb, icoupled
      double precision rinbc(nblocks,*)

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iinbc     i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     rinbc     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, iface, ipos, ipos1, iseg, itrans, iturb,
     +        nseg, ntrnsp, nturb

c     i         General loop variable
c     iblock    Block sequence number
c     iface     Face number
c     ipos      Position in array
c     ipos1     Position in array
c     iseg      Counting variable: sequence number of segment
c     itrans    reserved
c     iturb     Turbulence sequence number
c     nseg      Number of segments the boundary is divided in
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c     Block 13: Parameters with respect to the boundary conditions
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Get ntrnsp and nturb
c    write array iinbc
c    write array rinbc
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write array iinbc

      write(irefwr,100)
100   format ( /'**** BLOCK 15: Parameters with respect to the',
     +          ' boundary conditions'//)

      do iblock = 1, nblocks
         write(irefwr,110) iblock
110      format ( /'* iinbc:'//'# Block ', i3/ )
         do iface = 1, 2*ndim
            ipos = iinbc(iblock,iface)
            nseg = iinbc(iblock,ipos)
            write(irefwr,120) iface, nseg
120         format ( /'# Boundary ', i3//i3,8x,'# Number of segments' )
            do iseg = 1, nseg
               if ( ndim.eq.2 ) then

c              --- ndim = 2

                  if ( icoupled.eq.0 ) then

c                 --- Coupled approach

                      write(irefwr,130) (iinbc(iblock,ipos+i),i=1,5)
130                   format ( 2i5,8x,'# i1, i2 '/
     +                         3i5,3x,'# momentum' )
                      ipos = ipos + 5

                  else

c                 --- Decoupled approach

                      write(irefwr,140) (iinbc(iblock,ipos+i),i=1,5)
140                   format ( 2i5,8x,'# i1, i2 '/
     +                         3i5,3x,'# u_momentum' )
                      ipos = ipos + 5
                      write(irefwr,150) (iinbc(iblock,ipos+i),i=1,3)
150                   format ( 3i5,3x,'# v_momentum ' )

                  end if
                  write(irefwr,160)
160               format ( '    0    0    0',3x,'# pressure' )
                  ipos = ipos + 3
                  ipos1 = ipos
                  do itrans = 1, ntrnsp
                     write(irefwr,170) (iinbc(iblock,ipos1+i),i=1,3),
     +                                  itrans
170                  format ( 3i5, 3x, '# transport ', i2 )
                     ipos1 = ipos1 + 3
                  end do
                  ipos = ipos + 3*maxtrans
                  ipos1 = ipos
                  do iturb = 1, nturb
                     write(irefwr,180) (iinbc(iblock,ipos1+i),i=1,3),
     +                                  iturb
180                  format ( 3i5, 3x, '# turbulence ', i2 )
                     ipos1 = ipos1 + 3
                  end do
                  ipos = ipos + 3*maxturb

               else

c              --- ndim = 3

                  write(irefwr,200) (iinbc(iblock,ipos+i),i=1,4)
200               format ( 2i5,8x,'# begin index '/
     +                     2i5,8x,'# end   index ' )
                  if ( icoupled.eq.0 ) then

c                 --- Coupled approach

                     write(irefwr,210) (iinbc(iblock,ipos+i),i=5,8)
210                  format ( 4i5,3x,'# momentum' )

                  else

c                 --- Decoupled approach

                     write(irefwr,220) (iinbc(iblock,ipos+i),i=5,16)
220                  format ( 4i5,3x,'# u_momentum'/
     +                        4i5,3x,'# v_momentum'/
     +                        4i5,3x,'# w_momentum')

                  end if
                  write(irefwr,230)
230               format ( '    0    0    0    0',3x,'# pressure' )
                  ipos = ipos + 16
                  ipos1 = ipos
                  do itrans = 1, ntrnsp
                     write(irefwr,240) (iinbc(iblock,ipos1+i),i=1,4),
     +                                  itrans
240                  format ( 4i5, 3x, '# transport ', i2 )
                     ipos1 = ipos1 + 4
                  end do
                  ipos = ipos + 4*maxtrans
                  ipos1 = ipos
                  do iturb = 1, nturb
                     write(irefwr,260) (iinbc(iblock,ipos1+i),i=1,4),
     +                                  iturb
260                  format ( 4i5, 3x, '# turbulence ', i2 )
                     ipos1 = ipos1 + 4
                  end do
                  ipos = ipos + 4*maxturb
               end if
            end do
         end do

c        --- write array rinbc

         write(irefwr,310)
310      format ( /'* rinbc :'//  )
         ipos1 = 0
         do iface = 1, 2*ndim
            ipos = iinbc(iblock,iface)
            nseg = iinbc(iblock,ipos)
            write(irefwr,320) iface
320         format ( /'# Boundary ', i3/ )
            do iseg = 1, nseg
               if ( icoupled.eq.0 ) then

c              --- Coupled approach

                  if ( ndim.eq.2 ) then
                     write(irefwr,330) (rinbc(iblock,ipos1+i),i=1,2)
330                  format ( 2d15.7, 5x, '# momentum' )
                  else
                     write(irefwr,340) (rinbc(iblock,ipos1+i),i=1,3)
340                  format ( 3d15.7, 5x, '# momentum' )
                  end if

               else

c              --- Decoupled approach

                  if ( ndim.eq.2 ) then
                     write(irefwr,350) (rinbc(iblock,ipos1+i),i=1,4)
350                  format ( 2d15.7, 5x, '# u_momentum' /
     +                        2d15.7, 5x, '# v_momentum' )
                  else
                     write(irefwr,360) (rinbc(iblock,ipos1+i),i=1,9)
360                  format ( 3d15.7, 5x, '# u_momentum' /
     +                        3d15.7, 5x, '# v_momentum' /
     +                        3d15.7, 5x, '# w_momentum' )
                  end if

               end if
               if ( ndim.eq.2 ) then
                  write(irefwr,370) 0d0, 0d0
370               format ( 2d15.7, 5x, '# pressure' )
               else
                  write(irefwr,380) 0d0, 0d0, 0d0
380               format ( 3d15.7, 5x, '# pressure' )
               end if

               ipos1 = ipos1+ndim*ndim
               do itrans = 1, ntrnsp
                  if ( ndim.eq.2) then
                     write(irefwr,400)
     +              (rinbc(iblock,ipos1+(itrans-1)*2+i),i=1,2),itrans
400                  format ( 2d15.7, 5x, '# transport ', i2 )
                  else
                     write(irefwr,410)
     +              (rinbc(iblock,ipos1+(itrans-1)*3+i),i=1,3),itrans
410                  format ( 3d15.7, 5x, '# transport ', i2 )
                  end if
               end do
               ipos1 = ipos1 + ndim*maxtrans
               do iturb = 1, nturb
                  if ( ndim.eq.2) then
                     write(irefwr,500)
     +               (rinbc(iblock,ipos1+(iturb-1)*2+i),i=1,2), iturb
500                  format ( 2d15.7, 5x, '# turbulence ', i2 )
                  else
                     write(irefwr,510)
     +               (rinbc(iblock,ipos1+(iturb-1)*3+i),i=1,3), iturb
510                  format ( 3d15.7, 5x, '# turbulence ', i2 )
                  end if
               end do
               ipos1 = ipos1 + ndim*maxturb
            end do
         end do
      end do

      end
