      subroutine isdcreat ( index, length, namearray )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date   08-03-1996 Put name of array in message
c        version 2.0    date   05-10-1994 Extra parameter namearray)
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         ini071 version 1.4
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     compute starting position of a new or already existing double precision
c     array in buffer array buffer
c     the array space is reserved without reading from file 3
c
c     If the array did not exist, or the old array took too few positions to
c     keep the new one, then space in array IBUFFR is reserved
c     if no space is available in array IBUFFR, ISDCREAT tries to
c     create space in the following ways:
c
c       -  by writing low priority arrays to backing storage
c
c       -  if this is not sufficient by compressing
c          array ibuffr after writing of all arrays with priority
c          smaller than 10 to backing storage
c     The name of the calling subroutine is extracted from common blocks
c     CISMESSC and CISMESSG, so it is supposed that the subroutine ISEROPEN has
c     been used
c **********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer index, length
      character *(*) namearray

c     index    i/o   Integer variable containing the sequence number of the
c                    array to be processed
c     length    i    Length of the array in words
c     namearray i    Name of the array to be created
c **********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cisnamtab'
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, ibfree, iextra, iinfor, indfor, ipstar, lenchar,
     +        lengol, nbufef, nbuffr, nvectr

c     i         General loop variable
c     ibfree    Next free position in array IBUFFR
c     iextra    Extra space needed at both sides of the array for control
c               integers
c     iinfor    Number of arrays that has been referenced
c     indfor    Help parameter to store the record number from which the
c               array has been stored in file 3
c     ipstar    Computed starting position in array IBUFFR
c     lenchar   Number of characters in main keywords
c     lengol    Length of the preceding array corresponding to index
c     nbufef    Last position in IBUFFR before array inback
c     nbuffr    Length of buffer array ibuffr
c     nvectr    Maximum number of arrays that may be stored in IBUFFR
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISINI049  Create space in IBUFFR by writing low priority arrays to
c               file 3, or to compress IBUFFR
c     ISINICHK  Check IBUFFR
c     ISPRINBF  Print and check length of array IBUFFR
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                    I/O
c
c     If necessary arrays are written to and/or read from file 3 (iref3)
c **********************************************************************
c
c                    ERROR MESSAGES
c
c      16   index > maxarrays
c    1401   length of array less than one
c    1402   iinfor too large
c **********************************************************************
c
c                    PSEUDO CODE
c
c     If ( idebug > 0 ) then
c        Set iextra = extra space needed
c        check all control integers
c     if ( length <= 0 ) then
c        print error
c     If ( Array existed before ) then
c        If ( array in IBUFFR and old length <= new length ) then
c           Compute IPSTAR
c        else ( not enough space at old positions )
c           If ( enough space in IBUFFR after IBFREE ) then
c              IPSTAR = IBFREE
c              Raise IBFREE
c           else
c              Create space by writing low priority arrays to file 3
c              or compressing IBUFFR
c     else (array is new )
c         If ( enough space in IBUFFR after IBFREE ) then
c             IPSTAR = IBFREE
c             Raise IBFREE
c         else
c             Create space by writing low priority arrays to file 3
c             or compressing IBUFFR
c         Raise IINFOR
c         INDEX = IINFOR
c     Fill array INFOR
c     Set ibuffr(nbuffr+index-1)) negative if necessary
c     If ( idebug > 0 ) then
c        place control integers for this array
c     Fill name in names table
c ======================================================================
c
      nvectr = ibuffr(2)
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call isinichk ( ibuffr, ibuffr(11) )
      end if
      if ( length.le.0 ) then

c     --- length <= 0, error

         call iserrint ( length, 1 )
         call iserrsub ( 1401, 1, 0, 0 )
         call isstop

      end if

      if ( index.gt.0 .and. index.le.iinfor ) then

c     --- 0 < index <= iinfor  array has been filled before

         indfor = ibuffr(11+3*(index-1))
         lengol = ibuffr(12+3*(index-1))
         if ( indfor.gt.0 .and. length*intlen.le.lengol ) then

c        --- indfor > 0  and length*intlen>lengol   old positions are used

            ipstar = indfor
            if ( idebug.ge.2 ) then

               do 100 i = lengol+1, length*intlen
                  ibuffr(ipstar+i) = iinfin
100            continue

             end if

         else

c        --- length*intlen > lengol or indfor < 0
c            new array is bigger than preceding one,
c            or array is stored in backing storage
c            old positions are deleted, new positions are used

c            check free positions

            if ( idebug.ge.2 .and. indfor.gt.0 ) then
               do 110 i = indfor-1, indfor+lengol
                  ibuffr(i) = iinfin
110            continue
            end if

            if ( ibfree+length*intlen+iextra*2-1.le.3*nbuffr/4 ) then

c           --- enough free space available   ( standard situation )

               ipstar = ibfree
               ibfree = ibfree+length*intlen+iextra*2
               if ( intlen.eq.2 .and. mod ( ibfree, 2 ).eq.0 )
     +             ibfree = ibfree+1
            else

c           --- try to create free space by writing low priority arrays
c               to backing storage   ( file 3 ) or compressing ibuffr

               call isini049 ( length*intlen, ipstar, ibuffr )

            end if

         end if

      else

c     --- new array

         if ( ibfree+length*intlen+iextra*2-1.le.3*nbuffr/4 ) then

c        --- enough free space available   ( standard situation )

            ipstar = ibfree
            ibfree = ibfree+length*intlen+iextra*2

c           --- For a 32-bits computer, ibfree must always be odd

            if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 ) ibfree=ibfree+1

         else

c        --- try to create free space by writing low priority arrays
c            to backing storage ( file 3 ) or compressing ibuffr

               call isini049 ( length*intlen, ipstar, ibuffr )

         end if
         iinfor = iinfor+1
         index = iinfor
         if ( iinfor.ge.nvectr ) then

c        --- iinfor too large
c            Check if some array has been deleted, if not error

             do 130 i = 1, nvectr
                if ( ibuffr(12+3*(i-1)).eq.0 ) then

c               --- infor(2,i) = 0, means that array i is free

                   index = i
                   iinfor = nvectr
                   go to 150

                end if
130          continue

            call iseropen ( 'isdcreat' )
            call iserrint ( iinfor, 1 )
            call iserrint ( nvectr, 2 )
            call iserrsub ( 1402, 2, 0, 0 )
            call isstop
         end if

      end if

150   ibuffr(11+3*(index-1)) = ipstar
      ibuffr(12+3*(index-1)) = length*intlen
      ibuffr(13+3*(index-1)) = 10
      ibfree = max(ibfree,ipstar+length*intlen+iextra*2)
      nbufef = ibuffr(1)-6*ibuffr(2)
      if ( ibuffr(nbufef+index).gt.0 )
     +     ibuffr(nbufef+index) = -ibuffr(nbufef+index)
      call isprinbf ( ibuffr, ibfree-1 )

      if ( idebug.gt.0 ) then

c     --- idebug > 0, set control integers and control the buffer array

         ibuffr(ipstar-1) = index
         ibuffr(ipstar+length*intlen) = index

      end if
      if ( ioutpt.gt.2 ) then
         write(irefwr,200) namearray, index, ipstar,
     +                     namsubs(levelsub)//':isdcreat'
200      format ( ' Create array ',a,' with sequence number', i4,
     +            ' and starting address', i9/' in subroutine ', a )
      end if
      ibuffr(3) = iinfor
      ibuffr(4) = ibfree

c     --- Fill names table

      if ( index.gt.maxarrays ) then
         call iserrint ( index, 1 )
         call iserrint ( maxarrays, 2 )
         call iserrsub ( 16, 2, 0, 0 )
      else
         lenchar = min ( 9, len(namearray) )
         namarrays(index) = namearray(1:lenchar)
      end if

      end
