      subroutine isendspc ( ipstar, length, nbufol )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         ini079 version 1.7
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     Reserve temporarily LENGTH words at the end of array IBUFFR
c     The starting position is equal to IPSTAR and NBUFFR is set equal
c     to IPSTAR-1
c     NBUFOL gets the old value of NBUFFR, so that at the end NBUFFR can
c     be reset. If the last positions of IBUFFR are already occupied, the
c     corresponding arrays are written to file 3 and in the case these
c     arrays have priority 10 or larger, they are read in new positions
c     in IBUFFR
c     The name of the calling subroutine is extracted from common blocks
c     CISMESSC and CISMESSG, so it is supposed that the subroutine ISEROPEN has
c     been used
c **********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS

      implicit none
      integer ipstar, length, nbufol

c     ipstar  o  Computed starting position in array IBUFFR
c     length  i  Length of the array in words
c     nbufol  o  Value of NBUFFR at input
c                This value may be used to reset NBUFFR in a later stage
c **********************************************************************
c
c                    COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
c **********************************************************************
c
c                    LOCAL PARAMETERS

      integer index, ibfrsv, indtmp, lenloc, ipold, ipnew, iextra,
     +        nbuffr, iinfor, ibfree

c     ibfree    Next free position in array IBUFFR
c     ibfrsv    Help variable to save or to compute ibfree
c     iextra    Extra space needed at both sides of the array for control
c               integers
c     iinfor    Number of arrays that has been referenced
c     index     Integer variable containing the sequence number of the
c               array to be processed
c     indtmp    Reference number of a temporarily created array in INFOR
c     ipnew     Starting address of new array in IBUFFR
c     ipold     Starting address of old array in IBUFFR
c     lenloc    Length of local array
c     nbuffr    Length of buffer array ibuffr
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISDELARR  delete temporary array
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISICREAT  Create permanent space in array IBUFFR
c     ISINCOPY  Copy integer array to other integer array possibly itself
c     ISINICHK  Check IBUFFR
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                    I/O
c
c     none
c **********************************************************************
c
c                    ERROR MESSAGES
c
c    1401   length of array less than one
c    1405   Length of array IBUFFR too small
c    1406   Too much subsequent calls of isendspc
c **********************************************************************
c
c                    PSEUDO CODE
c
c     if ( idebug>0 ) then
c        Check IBUFFR
c     if ( length<=0 or length>nbuffr ) then
c        Error
c     ipstar := nbuffr-length
c     nbuffr := ipstar-1
c     if ( ibfree >= ipstar ) then
c        move all arrays with end address > ipstar to first part of IBUFFR
c        decrease ibfree accordingly
c     if ( idebug>0 ) then
c        Set control integers
c ======================================================================
c
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call isinichk ( ibuffr, ibuffr(11) )
      end if
      if ( length.le.0 ) then

c     --- length <= 0, error

         call iserrint ( length, 1 )
         call iserrsub ( 1401, 1, 0, 0 )
         call isstop

      end if

      nbufol = nbuffr
      if ( length.gt.nbuffr ) then
         call iserrint ( length, 1 )
         call iserrint ( nbuffr, 2 )
         call iserrsub ( 1405, 2, 0, 0 )
      end if
      ipstar = nbuffr - length + 1 - iextra
      if ( intlen.eq.2 .and. mod(ipstar,2).eq.0 ) ipstar = ipstar - 1
      nbuffr = ipstar - 1 - iextra
      ibfree = min ( ibfree, nbuffr+1 )

      if ( ibfree.ge.ipstar ) then

c     --- ibfree >= ipstar
c         Move all arrays with starting address > nbuffr to positions in
c         the first part
c         Decrease IBFREE accordingly

         ibfrsv = ibfree
         do 100 index = 1, iinfor
            if ( ibuffr(11+3*(index-1)).gt.0 .and.
     +           ibuffr(11+3*(index-1)) + ibuffr(12+3*(index-1))+iextra
     +           .ge.nbuffr ) then

c           --- array "INDEX" has end address larger than NBUFFR
c               Move to new positions
c               First a new permanent array is created

               indtmp = 0
               ibfrsv = min ( ibfrsv, ibuffr(11+3*(index-1)) )
               lenloc = ibuffr(12+3*(index-1))
               call isicreat ( indtmp, lenloc , 'indtmp' )
               ipnew  = ibuffr(11+3*(indtmp-1))
               ipold  = ibuffr(11+3*(index-1))
               ibfrsv = max ( ibfrsv, ipnew+lenloc+2*iextra )
               if ( intlen.eq.2 .and. mod(ibfrsv,2).eq.0 )
     +            ibfrsv = ibfrsv - 1

c              --- Check if this array is in-core
c                  If so move contents of old array to this new one

               if ( ipold.gt.0 ) then

c              --- old array is still in-core, which means that there was
c                  space enough
c                  Copy array to new positions

                  call isincopy ( ibuffr, ibuffr, ipold, ipnew, lenloc,
     +                            0 )
                  ibuffr(11+3*(index-1)) = ipnew

c                 --- Delete array indtmp, but use old positions

                  ibuffr(11+3*(indtmp-1)) = ipold
                  if ( idebug.gt.0 ) then
                     ibuffr(ipold-1) = indtmp
                     ibuffr(ipold+lenloc) = indtmp
                     ibuffr(ipnew-1) = index
                     ibuffr(ipnew+lenloc) = index
                  end if

               end if
               call isdelarr ( indtmp )

            end if

100      continue

         ibfree = ibfrsv

      end if

      if ( idebug.gt.0 ) then

c     --- set control integers

         narbuf = narbuf + 1
         if ( narbuf.gt.25 ) then

c        --- error 1816: maximum number of calls of isendspc overwritten

            call iserrsub ( 1406, 0, 0, 0 )
            call isstop

         end if
         iarbuf(narbuf,1) = ipstar-1
         iarbuf(narbuf,2) = ipstar+length
         ibuffr(ipstar-1) = 1600+narbuf
         ibuffr(ipstar+length) = 1600+narbuf

      end if

      if ( ioutpt.gt.2 ) then
         write(irefwr,200) narbuf, ipstar,
     +       namsubs(levelsub)//':isendspc'
200      format ( ' Create temporary array ', i2,' at end of ibuffr,',
     +            ' with starting address', i9/' in subroutine ', a )
      end if
      ibuffr(4) = ibfree
      ibuffr(7) = nbuffr

      end
