      function isingetp ( index )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date   08-03-1996 Put name of array in message
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         iniget version 1.0
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     Return with pointer in IBUFFR corresponding to index
c     It is checked if pointer is available
c **********************************************************************
c
c                      KEYWORDS
c
c    memory_management
c    pointers
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer isingetp, index

c     index   i    Integer variable containing the sequence number of the
c                  array to be processed
c     isingetp o   Gives pointer in array IBUFFR corresponding to index
c **********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisnamtab'
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ipstar, iprio, nvectr

c     iprio   Priority number
c     ipstar    Computed starting position in array IBUFFR
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Put character value in error message file
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c **********************************************************************
c
c                    I/O
c
c    none
c **********************************************************************
c
c                    ERROR MESSAGES
c
c    1412   INDEX out of range
c    1413   array has not been activated properly
c **********************************************************************
c
c                    PSEUDO CODE
c
c   Check if index between 1 and nvectr
c   Check if index has been activated
c   Set pointer
c ======================================================================
c
      nvectr = ibuffr(2)
      ipstar = 11+3*nvectr
      if ( index.lt.1 .or. index.gt.nvectr ) then

c     --- index is not a valid index

         call iseropen ( 'isingetp' )
         call iserrint ( index, 1 )
         call iserrint ( abs(index), 2 )
         call iserrint ( nvectr, 3 )
         call iserrsub ( 1412, 3, 0, 0 )
         call iserclos ( 'isingetp' )

      else

c     --- index in the correct range, check if array has been activated

         ipstar = ibuffr(11+3*(index-1))
         iprio  = ibuffr(13+3*(index-1))
         if ( ipstar.lt.0 .or. iprio.lt.10 ) then

c        --- Array has not been activated properly

            call iseropen ( 'isingetp' )
            call iserrint ( index, 1 )
            call iserrint ( ipstar, 2 )
            call iserrint ( iprio, 3 )
            call iserrchr ( namarrays(index), 1)
            call iserrsub ( 1413, 3, 0, 1 )
            call iserclos ( 'isingetp' )

         end if

      end if

      isingetp = ipstar

      end
