      subroutine isini003 ( ibuffr, iworkgen, infor )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini003 version 1.4)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   write all arrays with priority < 10 to backing storage file 3
c   move other arrays to start of ibuffr from position ibufst
c ********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c     backing_storage
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ibuffr(*), iworkgen(*), infor(3,*)

c     ibuffr  i    reserved
c     infor   i    Information concerning the storage of arrays in IBUFFR
c                  is stored in the following way:
c                  Pos. (1,i)  If > 0: array i is stored in IBUFFR from position
c                              stored in infor(1,i)
c                              If < 0: array i is stored in file 3 from record
c                              number -infor(1,i)
c                  Pos. (2,i)  Length of the array in words
c                  Pos. (3,i)  Priority number. Is used to decide which arrays
c                              are put to file 3. 0 <= prio <= 10
c     iworkgen     General integer work array.
c                  See pseudo code for the contents
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer iextra, numarr, ipfirs, i, inext, ipnext, loclen, iprio,
     +        nbuffr, iinfor, ibfree, nvectr, nbufef

c     i       General loop variable
c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     iinfor  Number of arrays that has been referenced
c     inext   Sequence number of next array in IBUFFR with respect to INFOR
c     ipfirs  Starting address of present array in IBUFFR
c     ipnext  Starting address of array inext in IBUFFR
c     iprio   Priority number
c     loclen  Length of present array to be considered
c     nbufef  Last position in IBUFFR before array inback
c     nbuffr  Length of buffer array ibuffr
c     numarr  Number of arrays that are stored in IBUFFR
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISINCOPY  Copy integer array to other integer array possibly itself
c     ISINI100  write one array from IBUFFR to backing storage
c     ISINI102  Sort the sequence of the starting addresses in IBUFFR
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c   write all arrays with priority < 10 to file 3
c   Sort starting addresses of IBUFFR
c   Compress IBUFFR
c
c   Contents of array iworkgen:
c   iworkgen     work array of length 4*nvectr, in which the entries in infor
c                are sorted for increasing starting address
c                iworkgen(1) contains the index of the array with the smallest
c                            starting address,
c                iworkgen(2) with the next starting address etc.
c ======================================================================
c
c     --- write arrays with priority<10 to file 3

      nvectr = ibuffr(2)
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
      end if
      nbufef = ibuffr(1)-6*ibuffr(2)
      do 100 i = 1,iinfor
         iprio = infor(3,i)
         if ( iprio.lt.10 .and. infor(1,i).gt.0 )
     +      call isini100 ( i, ibuffr, infor, ibuffr(nbufef+1),
     +                      ibuffr(nbufef+nvectr+1) )
100   continue

c     --- make ibuffr compact
c         first sort starting positions

      call isini102 ( iworkgen, numarr, ibuffr, infor,
     +                iworkgen(nvectr+1), iworkgen(2*nvectr+1),
     +                iworkgen(3*nvectr+1) )
      ipfirs = 1501 + iextra
      do 200 i = 1, numarr
         inext  = iworkgen(i)
         ipnext = infor(1,inext) - iextra
         loclen = infor(2,inext) + 2*iextra
         if ( ipnext.gt.ipfirs ) then

c        --- move array inext to starting address ipfirs

            call isincopy ( ibuffr, ibuffr, ipnext, ipfirs-iextra,
     +                    loclen, 1 )
            infor(1,inext) = ipfirs
            if ( idebug.gt.0 ) ibuffr(ipfirs-1) = inext

         end if
         ipfirs = ipfirs + loclen
         if ( intlen.eq.2 .and. mod(ipfirs,2).eq.0 ) ipfirs=ipfirs+1

200   continue

      ibfree = ipfirs
      if ( idebug.ge.2 ) then
         do 210 i = ibfree, nbuffr
            ibuffr(i) = iinfin
210      continue
      end if
      ibuffr(4) = ibfree

      end
