      subroutine isini049 ( length, ipstar, ibuffr )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini049 version 1.3)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c  create space in array ibuffr by writing low priority
c  arrays to backing storage or by compressing
c
c       Steps to be carried out
c
c       -  Check if there is enough contiguous space in IBUFFR
c       -  if not create space by writing low priority arrays to file
c       -  if this is not sufficient by compressing
c          array ibuffr after writing of all arrays with priority
c          smaller than 10 to backing storage
c ********************************************************************
c
c                      KEYWORDS
c
c    memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer length, ipstar, ibuffr(*)

c     ibuffr i/o reserved
c     ipstar  o  Computed starting position in array IBUFFR
c     length  i  Length of the array in words
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer iextra, nbuffr, ibfree, nvectr, nbufef

c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     nbufef  Last position in IBUFFR before array inback
c     nbuffr  Length of buffer array ibuffr
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISINI003  Clear array IBUFFR by writing all low priority arrays to file 3
c               and compressing array IBUFFR
c     ISINI101  Try to create free space in IBUFFR by writing sufficient low
c               priority arrays to file 3
c     ISPRINBF  Print and check length of array IBUFFR
c ********************************************************************
c
c                    I/O
c
c     If necessary arrays are written to and/or read from file 3 (iref3)
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c     Try to make space by writing as much low priority arrays to file 3 as
c     necessary to get length free positions in IBUFFR
c
c     If ( not sufficient space available ) then
c
c        Create space by writing all arrays with priority < 10 to file 3 and
c        compressing IBUFFR
c        Check length of IBUFFR
c ======================================================================
c
c     --- try to create free space by writing low priority arrays
c         to backing storage  (file 3)
c
      nvectr = ibuffr(2)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      nbufef = ibuffr(1)-6*ibuffr(2)
      call isini101 ( length, ipstar, ibuffr(nbufef+2*nvectr+1), nbuffr,
     +                ibuffr, ibuffr(11), ibuffr(nbufef+1),
     +                ibuffr(nbufef+nvectr+1) )

      if ( ipstar.eq.0 ) then

c     --- ipstar = 0  write all parts in
c         ibuffr with priority smaller than 10 to file 3
c         compress array ibuffr
c         Remark:  In this case all pointers must be recomputed!!!

         call isini003 ( ibuffr, ibuffr(nbufef+2*nvectr+1), ibuffr(11) )
         if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 ) ibfree = ibfree+1
         if ( idebug.gt.0 ) then
            iextra = intlen
         else
            iextra = 0
         end if
         call isprinbf ( ibuffr, ibfree+length+iextra*2-1 )
         ipstar = ibfree
         ibfree = ibfree+length+iextra*2-1
         if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 ) ibfree = ibfree+1

      end if
      ibuffr(4) = ibfree

      end
