      subroutine isini101 ( length, ipstar, iworkgen, ilast, ibuffr,
     +                      infor, inback, lenbac )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini101 version 1.8)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   create space by either looking for contiguous space in array ibuffr
c   or by writing arrays to file 3
c   at least length succeeding positions are needed
c ********************************************************************
c
c                      KEYWORDS
c
c    memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer length, ipstar, ilast, iworkgen(*), ibuffr(*), infor(3,*),
     +        inback(*), lenbac(*)


c     ibuffr i/o   reserved
c     ilast   i    Last position in ibuffr that may be used
c     inback  i    Integer array of length nvectr in which of each array that
c                  is referenced in array INFOR is given
c                  whether the last version is also stored on backing storage
c                  inback(i) > 0, or not inback(i) <= 0.
c                  When inback(i) > 0, the value of inback(i) gives the record
c                  number from which array i is stored,
c                  if inback(i) = 0, the array has never been stored on file 3,
c                  and if inback(i) < 0, then |inback(i)| gives the record
c                  number from which a preceding version of array has been
c                  stored
c     infor   i    Information concerning the storage of arrays in IBUFFR
c                  is stored in the following way:
c                  Pos. (1,i)  If > 0: array i is stored in IBUFFR from position
c                              stored in infor(1,i)
c                              If < 0: array i is stored in file 3 from record
c                              number -infor(1,i)
c                  Pos. (2,i)  Length of the array in words
c                  Pos. (3,i)  Priority number. Is used to decide which arrays
c                              are put to file 3. 0 <= prio <= 10
c     ipstar  o    Computed starting position in array IBUFFR
c     iworkgen     General integer work array.
c                  See pseudo code for the contents
c     lenbac  i    Integer array of length nvectr corresponding to inback
c                  For each ARRAY i with INBACK(i)#0, LENBAC(i) contains the
c                  the number of records that are used by the last version of
c                  array i that has been written to file 3
c     length  i    Length of the array in words
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer iextra, numarr, ipfirs, ispace, k, inext, iplast, ipnext,
     +        loclen, kprio, mwrite, iprio, j, iprint, nbuffr, ibfree,
     +        nvectr
      save iprint
      data iprint /0/

c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     inext   Sequence number of next array in IBUFFR with respect to INFOR
c     ipfirs  Starting address of present array in IBUFFR
c     iplast  Ending address of array in IBUFFR
c     ipnext  Starting address of array inext in IBUFFR
c     iprint  Indicates if the warning with respect to backing storage has
c             been printed (1) or not
c     iprio   Priority number
c     ispace  Defines the available contiguous space in IBUFFR
c     j       General loop variable
c     k       General loop variable
c     kprio   Loop variable for priorities
c     loclen  Length of present array to be considered
c     mwrite  First sequence number of arrays to be written to file 3
c     nbuffr  Length of buffer array ibuffr
c     numarr  Number of arrays that are stored in IBUFFR
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISINCOPY  Copy integer array to other integer array possibly itself
c     ISINI100  write one array from IBUFFR to backing storage
c     ISINI102  Sort the sequence of the starting addresses in IBUFFR
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c    ipstar = 0
c    If ( idebug > 0 ) then
c       Set iextra = extra space needed
c    Sort array infor with respect to increasing starting addresses
c    Put sorted sequence in iwork
c    Check the contiguous space between these entries
c    If ( contiguous space found ) then
c       set ipstar
c    else if ( space at the end of IBUFFR is sufficient ) then
c       set ipstar at end
c    else if ( total free space enough ) then
c       compress IBUFFR as much as necessary
c       set ipstar
c    else
c       ready := false
c       for iprio := 1 (1) 9, while (not ready) do
c          write arrays with priority < iprio to backing storage until
c          contiguous space is found
c          if ( contiguous space is found ) then
c             set ipstar
c             ready = true
c
c   Contents of array iworkgen:
c   iworkgen     work array of length 4*nvectr, in which the entries in infor
c                are sorted for increasing starting address
c                iworkgen(1) contains the index of the array with the smallest
c                            starting address,
c                iworkgen(2) with the next starting address etc.
c ======================================================================
c
c     --- check if there are free succeeding positions in
c         ibuffr(1),...,ibuffr(ibfree)
c         sort the sequence in infor and store in array iwork

      nvectr = ibuffr(2)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      ipstar = 0
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
      end if
      call isini102 ( iworkgen, numarr, ibuffr, infor,
     +                iworkgen(nvectr+1), iworkgen(2*nvectr+1),
     +                iworkgen(3*nvectr+1) )

c     --- Check contiguous free space

      ipfirs = 3*nvectr+11 + iextra
      ispace = 0
      do 100 k = 1, numarr
         inext  = iworkgen(k)
         iplast = min(nbuffr+1,infor(1,inext)) - iextra - 1
         ispace = ispace + max(0,iplast-ipfirs)
         if ( iplast-ipfirs.gt.length+iextra ) then

c        --- sufficient contiguous space found

            ipstar = ipfirs
            go to 1000

         else

c        --- not enough space, raise ipfirs

            ipfirs = infor(1,inext)+infor(2,inext)+iextra*2
            if ( intlen.eq.2 .and. mod(ipfirs,2).eq.0 ) ipfirs=ipfirs+1

         end if

100   continue
      ispace = ispace + ilast - iextra - ipfirs + 1

c     --- no free contiguous space found
c         Check if there is space at the end of IBUFFR

      if ( ibfree+length+iextra*2-1.le.ilast ) then

c     --- enough free space available

         ipstar = ibfree
         ibfree = ibfree+length+iextra*2
         if ( intlen.eq.2 .and. mod (ibfree,2).eq.0 ) ibfree = ibfree+1

      else if ( ispace.gt.3*length ) then

c     ---  enough space available, compress IBUFFR from the start,
c          until enough contiguous free space is found

         ipfirs = 3*nvectr+11 + iextra
         do 200 k = 1, numarr
            inext  = iworkgen(k)
            ipnext = infor(1,inext) - iextra
            if ( ipnext-ipfirs-1.gt.length+iextra ) then
               ipstar = ipfirs
               if ( idebug.eq.2 ) then
                  do 110 j = ipstar, ipnext-1
                     ibuffr(j) = iinfin
110               continue
               end if
               go to 1000
            end if
            loclen = infor(2,inext) + 2*iextra
            if ( ipnext.gt.ipfirs ) then

c           --- move array inext to starting address ipfirs

               call isincopy ( ibuffr, ibuffr, ipnext, ipfirs-iextra,
     +                         loclen, 1 )
               infor(1,inext) = ipfirs

            end if
            ipfirs = ipfirs + loclen
            if ( intlen.eq.2 .and. mod(ipfirs,2).eq.0 ) ipfirs=ipfirs+1

200      continue

c        --- end has been reached

         ipstar = ipfirs
         ibfree = ipstar + length + iextra*2
         if ( intlen.eq.2 .and. mod (ibfree,2).eq.0 ) ibfree = ibfree+1

      else

c     --- no free contiguous space found; try to find succeeding space by
c         writing low priority arrays to file 3
c         Decrease numarr in order to circumvent interference with ini079

         k = numarr
         do 205 j = k, 1, -1
           if ( infor(1,iworkgen(j))+infor(2,iworkgen(j))+iextra .ge.
     +          nbuffr ) then
              numarr = j - 1
           else
              goto 210
           end if
205      continue

c        --- Loop over the priorities

210      if ( iprint.eq.0 .and. ioutpt.ge.0 ) then
            write(irefwr,220)
220         format(///' ********** Warning ************'//
     +                ' There is not enough space in IBUFFR'/
     +                ' The swapping mechanism is activated'/
     +                ' This may lead to a loss of performance'/
     +                ' It is advised to enlarge array IBUFFR if',
     +                ' possible'/// )
            iprint = 1
         end if
         do 300 kprio = 1, 9
            ipfirs = 3*nvectr+11+iextra
            mwrite = 0

c           --- Loop over all arrays in IBUFFR

            do 250 k = 1, numarr
               inext  = iworkgen(k)
               iplast = min(nbuffr+1,infor(1,inext)) - iextra - 1
               iprio  = infor(3,inext)
               if ( iprio.le.kprio ) then

c              --- only arrays with priority <= kprio are considered

                  if ( mwrite.eq.0 ) mwrite = k
                  if ( k.eq.numarr ) then
                     iplast = ilast-iextra-1
                  else
                     iplast = min(nbuffr+1,infor(1,iworkgen(k+1)))-
     +                        1-iextra
                  end if
                  if ( iplast-ipfirs.ge.length+iextra) then

c                 --- Enough contiguous space found
c                     write arrays mwrite to k   to file 3

                     do 230 j = mwrite, k
                        call isini100 ( iworkgen(j), ibuffr, infor,
     +                                  inback, lenbac )
230                  continue

c                    --- Recompute ibfree

                     ibfree = nvectr+1 + iextra
                     do 240 j = 1, numarr
                        ibfree = max(ibfree,
     +                               infor(1,iworkgen(j))+
     +                               infor(2,iworkgen(j))+2*iextra)
240                  continue

                     ipstar = ipfirs
                     ibfree = max ( ibfree, ipstar+length+iextra*2 )
                     if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 )
     +                    ibfree=ibfree+1
                     go to 1000

                  end if

               else

c              --- priority of present array not enough
c                  The space starts after the present one

                  ipfirs = iplast+infor(2,inext)+1+2*iextra
                  if ( intlen.eq.2 .and. mod(ipfirs,2).eq.0)
     +                 ipfirs = ipfirs+1
                  mwrite = 0

               end if

250         continue

300      continue

      end if
1000  continue
      ibuffr(4) = ibfree
      end
