      subroutine isreadmd ( irefenv )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date   30-01-1996 Correction do-loop dumtx1
c        version 1.0    date   21-06-1994 Subroutine extracted from the SEPRAN
c                                         subroutine READMD version 1.9
c
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c  Read default values from sepran environment file (isnas.env) and store
c  these quantities in some common blocks which
c  The ISNaS environment file may be in the local directory of the
c  user or in the ISNaS main directory (SPHOME)
c
c  First it is checked if a local environment file exists named SPRENV,
c  if not the global environment file is used
c ********************************************************************
c
c                      KEYWORDS
c
c   defaults
c   environment_file
c   read
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer irefenv

c     irefenv  i  Reference number with respect to the environment file
c                 This reference number is only used locally, the file
c                 is opened and closed in this subroutine. Afterwards the
c                 reference number may be reused.
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisnasch'
      include 'IScommon/cfile3'
      include 'IScommon/cseprd'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer lenspr, lenhom, irefrt, ios, idummy, i, ivers
      logical check
      character * 50 namefl

c     check     Indication if a file has been opened (true) or not (false)
c     i         General loop variable
c     idummy    Integer dummy
c     ios       Iostatus variable
c     irefrt    Help variable to store irefre temporarily
c     ivers     Version number of environment file
c     lenhom    Number of characters in the name of the home directory
c     lenspr    Number of characters in the name of the environment file
c     namefl    Help variable to store a name temporarily
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISREADM0  Read a name from the environment file
c     ISREADM1  Read an integer from the environment file
c     ISREADM2  Read a real from the environment file
c     ISREADM3  Read whether a file referred to in the environment file is
c               formatted (>0) or unformatted (<0)
c     ISREADM4  Read the version number from the environment file
c ********************************************************************
c
c                    I/O
c
c      ISREADMD reads the environment file, which must be available,
c      either locally or globally
c      The environment file contains the following quantities (the sequence
c      is fixed)
c
c      SPHOME
c      version number
c      stoper
c      macnam
c      machin
c      spver
c      spexec
c      numchr
c      rinfin
c      epsmac
c      sqreps
c      irefre
c      irefwr
c      irefer
c      iref1
c      iref2
c      iref3
c      iref17
c      iref10
c      iref73
c      iref74
c      iref75
c      indication of formatted or unformatted mesh output file
c      indication of formatted or unformatted sepcomp.out file
c      spappn
c      namef4
c      namef3
c      namems
c      namere
c      namewr
c      name10
c      name73
c      name74
c      name75
c      namef2
c      lenwor
c      nrec2
c      nrec3
c      icarcn
c      10 dummies
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c     Initialize commons
c     Check if there is a local environment file
c     Open environment file
c     Read and check all quantities if necessary
c ======================================================================
c
c     --- Initialize commons

      do 100 i = 1, 96
         rmach(i) = 0d0
100   continue
      do 110 i = 1, 91
         imach(i) = 0
110   continue
      do 120 i = 1, 91
         irefs(i) = 0
120   continue
      do 130 i = 1, 25
         dumtxt(i) = ' '
130   continue
      do i = 1, 24
         dumtx1(i) = ' '
      end do

c     --- Set some constants
c         The text that is read from the environment file will not be printed

      lenspr = index ( sprenv, ' ' ) - 1
      lenhom = index ( sphome, ' ' ) - 1
      ioutpt  = -1
      namems = ' '
      iref17 = 0
      ispecl = 6

c     --- Several characters, used to control terminals with

      null = char(0)
      escape = char(27)
      formfd = char(12)
      mvt100 = char(24)

c     --- Check for the presence of a local environment file

      inquire ( file = sprenv(1:lenspr), exist = check )

      if ( .not. check ) then

c     --- Local environment file does not exist, use global file
c         At this moment sphome does contain the name of the home directory
c         of the global environment file, including a possible / or \

         namefl(1:lenhom) = sphome(1:lenhom)
         namefl(lenhom+1:lenspr+lenhom) = sprenv(1:lenspr)
         lenspr = lenspr + lenhom
         sprenv(1:lenspr) = namefl(1:lenspr)

      end if

c     --- Open the environment file

      open ( unit = irefenv, file = sprenv(1:lenspr), status='old',
     +       iostat = ios )
      if ( ios.ne.0 ) then

c     --- ios # 0, environment file does not exist, or is incorrect

         write(*,200) sprenv(1:lenspr), ios
200      format(//' The ISNaS environment file ',a,' can not be',
     +            ' opened correctly'/
     +            ' Either the file does not exist or is incorrect'//
     +            ' Execution will be terminated' ///
     +            ' Extra information: iostatus is',i4 )
         stop

      end if

c     --- Read ISNaS home directory
c         The preceding file is overwritten
c         Letters are not translated into capitals
c         Next read operating system, machine name, version and name executable
c         Plus type of computer

      irefre = irefenv
      irefwr = 6
      call isreadm0 ( macnam )
      sphome = sphome(1:lenhom-8)
      call isreadm4 ( ivers, stoper )
      call isreadm0 ( stoper )
      call isreadm0 ( macnam )
      call isreadm1 ( machin )
      call isreadm0 ( spvers )
      call isreadm0 ( spexec )
      if ( machin.eq.2 .or. machin .eq. 8 ) then

c      Cyber or Cray version  (single precision)

         intlen = 1

      else

c      Other computers  (double precision)

         intlen = 2

      end if

c     --- Read numchr, rinfin, epsmac, sqreps

      call isreadm1 ( numchr )
      call isreadm2 ( rinfin )
      call isreadm2 ( epsmac )
      call isreadm2 ( sqreps )

c     --- Read irefre, irefwr, irefer, iref2, iref3, irefpl, iref10,
c              iref73, iref74, iref75
c              irefre is temporarily stored in ireft

      call isreadm1 ( irefrt )
      call isreadm1 ( irefwr )
      call isreadm1 ( irefer )
      call isreadm1 ( iref2  )
      call isreadm1 ( iref3  )
      call isreadm1 ( iref17 )
      call isreadm1 ( iref10 )
      call isreadm1 ( iref73 )
      call isreadm1 ( iref74 )
      call isreadm1 ( iref75 )

c     --- Read indication of formatted or unformatted mesh output file,
c              indication of formatted or unformatted sepcomp.out file

      call isreadm3 ( iref10 )
      call isreadm3 ( iref74 )

c     --- Read append mode output file

      call isreadm0 ( spappn )

c     --- Read names of files: namef4, namef3, namems, namere,
c                            namewr, name10, name73, name74, namef2

      call isreadm0 ( namef4 )
      call isreadm0 ( namems )
      call isreadm0 ( namef3 )
      call isreadm0 ( namere )
      call isreadm0 ( namewr )
      call isreadm0 ( name10 )
      call isreadm0 ( name73 )
      call isreadm0 ( name74 )
      call isreadm0 ( name75 )
      call isreadm0 ( namef2 )

c     --- Read extra information about files: lenwor, nrec1, nrec2, nrec3

      call isreadm1 ( lenwor )
      call isreadm1 ( nrec2  )
      call isreadm1 ( nrec3  )

c     --- Read carriage control parameter

      call isreadm0 ( namefl )
      if ( namefl(1:3).eq.'yes' ) then
         icarcn = 1
      else
         icarcn = 0
      end if

c     --- Read last 10 dummies

      do 300 i = 1, 10
         call isreadm1 ( idummy  )
300   continue

c     --- Fill irefre

      close ( irefenv )
      irefre = irefrt
      ioutpt = 0
      ispecl = 0
      end
