      subroutine dinvl3(x,b,matrix,n,ndimso,nconct,
     1                  prec,nprec,infmat)
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.3      date 04-08-1992  Zdenek: ndimso instead of ndim
c     version 1.2      date 01-06-1992
c     developed at     Convex, HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by L, the preconditioning matrix given in
c     prec.
c
c     In this case we obtain:
c                 -1
c            b = L  x.
c
c     The array prec should be filled by dmlu3.f. This subroutine
c     contains compiler directives to run in vector speed on the
c     Convex.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilu
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c
c     b       o  the result vector, which contains:
c                         -1
c                    b = L  x.
c
c     infmat  i  if infmat(1) is 1 we use the momentum equations,
c                whereas if infmat(1) is larger than or equal to 4
c                we use equations, with a structure similar to the
c                pressure equations.
c
c     matrix  i  the coefficient matrix for the momentum or
c                an equation similar to the pressure equation.
c
c     n       i  number of unknowns in the solution vector.
c
c     nconct  i  number of connections in one row of the matrix
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. In this subroutine nprec = nconct.
c
c     prec    i  the preconditioning matrix.
c
c     x       i  the original vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter.
c
c     ii      loop counter.
c
c     j       loop counter.
c
c     jjmax   loop bound.
c
c     jjmin   loop bound.
c
c     nx      number of points in the x-direction.
c
c     ny      number of points in the y-direction.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of x with L preconditioning matrix given
c     in matrix and prec. In this case we obtain:
c                 -1
c            b = L  x.
c
c     The arrays prec and matrix should be filled by subroutine dmlu3.f
c
c     **************************************************************
c
c     Change1 15-02-1991 C. Vuik.
c
c     The reason of this change is to solve the lower triangular system
c     in vector speed.
c
c     Literature : High performance preconditioning.
c                  H. A. van der Vorst.
c                  SIAM. J. Sci. Stat. Comput. 10 pp.1174-1185 (1989).
c
c     Implementation : Compare subroutine precd in the program ictest
c                      written by H. A. van der Vorst.
c
c     Remark     : In this version the virtual points along the bound-
c                  ary are made zero. Thereafter the loop only runs
c                  over the non-boundary points.
c
c     **************************************************************
c
c     Change2 01-06-1992 C. Vuik
c
c     The virtual points are removed.
c
c     **************************************************************
c
c     Remark
c
c     - at this moment only infmat(1).ge.4 is allowed.
c
c     **************************************************************
c
c ==================================================================
c
c     Parameters
c
      integer          infmat(*),n,ndimso,nconct,nprec
      double precision matrix(1:n,1:ndimso,1:nconct),
     1                 prec(1:n,1:ndimso,1:nprec),
     2                 x(n,ndimso),b(n,ndimso)
c
c     local parameters
c
      integer i,j,ii,nx,ny,jjmin,jjmax
      nx = infmat(2)+1
      ny = infmat(3)+1
c                             -1
c     The multiplication b = L  x
c
c     First we fill the points on the lower horizontal boundary.
c
      b(1,1) = x(1,1)
      do i = 2, nx
         b(i,1) = x(i,1)-prec(i,1,5)*b(i-1,1)
      enddo
      i = nx+1
      b(i,1) = x(i,1)-prec(i,1,4)*b(2,1)
     1               -prec(i,1,3)*b(1,1)
      i = nx+2
      b(i,1) = x(i,1)-prec(i,1,5)*b(i-1,1)
     1               -prec(i,1,4)*b(3,1)
     2               -prec(i,1,3)*b(2,1)
     3               -prec(i,1,2)*b(1,1)
c
c     change1
c
      do ii = 3, ny+(nx-1)/2
	 jjmin = max(2,ii-(nx-1)/2)
	 jjmax = min(ii,ny)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+1-2*j+(j-1)*nx
            b(i,1) = x(i,1)-prec(i,1,5)*b(i-1   ,1)
     1                  -prec(i,1,4)*b(i-nx+1,1)
     2                  -prec(i,1,3)*b(i-nx  ,1)
     3                  -prec(i,1,2)*b(i-nx-1,1)
         enddo
	 jjmin = max(2,ii+1-nx/2)
	 jjmax = min(ii,ny)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+2-2*j+(j-1)*nx
            b(i,1) = x(i,1)-prec(i,1,5)*b(i-1   ,1)
     1                  -prec(i,1,4)*b(i-nx+1,1)
     2                  -prec(i,1,3)*b(i-nx  ,1)
     3                  -prec(i,1,2)*b(i-nx-1,1)
         enddo
      enddo
      end
