      subroutine gmritz(n,amat,rhsd,usol,solutv,itmax,work,icontr,
     +                  infmat,prec,nprec,ndim,nconct,rho,dt,iinmtvc)
c ======================================================================
c
c     programmer       Kees Vuik
c
c     version 1.6      date 19-08-1998 Kees K. iinmtvc added,corrected
c     version 1.5      date 21-11-1997 Kees K. iinmtvc added
c     version 1.4      date 31-10-1997 Kees V. changes for uncoupled
c     version 1.3      date 02-04-1997 Kees K. literal constants in
c                                      double precision for f90
c     version 1.2      date 03-10-1995 Kees K. solutv added
c     version 1.1      date 04-04-1995 Guus replace mtvc1 by ismatvec
c     version 1.0      date 02-07-1992
c     developed at     Convex,HP700
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The subroutine GMRITZ calculates the first itmax Ritz values
c     for the (preconditioned) matrix.
c **********************************************************************
c
c                       KEYWORDS
c
c     eigenvalue
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c     amat    i   Matrix for which the eigenvalues are approximted by
c                 Ritz values.
c     dt          Timestep.
c     icontr i-o  Integer array in which information about the solution
c                 process must be given by the user. On output information
c                 about the solution process is provided. ICONTR must be
c                 filled as follows:
c               1: Parameter controlling the preconditioning. Possible
c                  values are:
c                  0: No preconditioning.
c                  1: Diagonal preconditioner.
c                 -1: Diagonal postconditioner.
c                  2: ILUD preconditioner, with Eisenstat implementa-
c                     tion.
c                 -2: ILUD postconditioner.
c                  3: ILU preconditioner.
c                 -3: ILU postconditioner.
c               2: Control parameter indicating the amount of output
c                  required. Possible values:
c                 <0: No output.
c                  1: Only fatal errors will be printed.
c                  2: Additional information about the iteration is printed.
c                  3: Gives a maximal amount of output concerning the
c                     iteration process.
c               3: At output an error indication is stored. Possible values:
c                  0: No fatal errors.
c               4: Contains at output the actual number of iterations
c                  performed.
c               5: Parameter indicating the file number to which output
c                  should be written.
c     infmat  i   Integer array with information of the matrix structure,`
c                 to be used in matrix-vector multiplication subroutine
c     itmax   i   The maximum number of iterations to be performed.
c     n       i   The number of rows in the matrix A.
c     nconct  i   Maximal number of connections in one row of the matrix.
c     ndim    i   Integer indicating the amount of unknowns in every grid-
c                 point. In the momentum equations ndim = 2 or 3, whereas
c                 in the pressure and transport equations ndim = 1.
c     nprec   i   Number of diagonals which are used in the precond-
c                 itioning.
c     prec    i   Array which contains part of the preconditioning matrix
c     rho     i   Specific mass.
c     rhsd    i   Vector containing the right-hand side vector of the
c                 system of equations.
c     usol   i-o  Solution vector of length n. On input the array should
c                 contain a starting vector. At output the array contains
c                 the last iterate, which is an approximation to the
c                 solution of the system.
c     work    o   Work array of size n * (itmax +1). On output the first
c                 n elements contain the residual vector.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c     c        Array determining the plane rotations which transform the
c              upper Hessenberg-matrix into an upper triangular matrix.
c     dirich   Variable which is used to check for dirichlet boundary
c              conditions.
c     eigv     Contains the Ritz vectors for the structure of this
c              array see EISPACK guide p. 391 variables mm and z.
c     eps      Is used to prevent a division by zero
c     epsmac   The machine precision.
c     heig     Contains the original Hessenberg matrix H.
c     hh       Array containing an upper Hessenberg-matrix of dimension
c              (i+1).
c     i        Loop variable for the inner iteration.
c     ierr     Variable used in subroutine hqr.
c     igh      Variable used in subroutine hqr.
c     imax     This is the minimum of 150 and itmax.
c     iter     The current iteration number.
c     j        Counting variable.
c     k        Counting variable.
c     l        Counting variable.
c     lmin     Variable used to sort the Ritzvalues.
c     low      Variable used in subroutine hqr.
c     m        Variable used in subroutine invit.
c     ne       Number of Ritz values to be calculated.
c     nm       Leading dimension of array heig.
c     residu   This array contains the residuals of the Ritzvalues.
c     resmax   Resmax is the value of the maximal residual.
c     rm1      Variable used in subroutine invit.
c     rnorm    Norm of the residual vector.
c     rs       Array containing the norms of the residuals from the inner
c              iterations.
c     rv1      Variable used in subroutine invit.
c     rv2      Variable used in subroutine invit.
c     s        Array determining the plane rotations which transform the
c              upper Hessenberg-matrix into an upper triangular matrix.
c     select   Logical array which denotes the wanted Ritz vectors.
c     start    Logical value indicating whether a restart is made.
c     t        Auxiliary variable.
c     wi       Contains the imaginary parts of the Ritz values.
c     wmin     Variable used to sort the Ritzvalues.
c     wr       Contains the real parts of the Ritz values.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     daxpy   Computes a vector + a constant * other vector.
c     dcopy   Copy a vector to another vector.
c     ddot    Computes an inner product.
c     dinvl   Computes the multiplication of the inverse of a lower
c             triangular matrix, stored in arrays prec and amat,
c             and a vector.
c     dinvu   Computes the multiplication of the inverse of an upper
c             triangular matrix, stored in arrays prec and amat,
c             and a vector.
c     dnrm2   Calculates the 2-norm of a given vector.
c     hqr     eispack subroutine to calculate the ritz values.
c     invit   eispack subroutine to calculate the ritz vectors.
c     ismatvec   Computes the product of the sparse matrix, stored in AMAT,
c             and a vector.
c     prevc   Multiplies a vector with a preconditioner
c     prires  Provides information about the solution process; the amount
c             of output is determined by the value of ICONTR(5).
c **********************************************************************
c
c                       I/O
c
c     There is I/O done. These statements are grouped together at the
c     end of the subroutine.
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine GMRITZ calculates the first itmax Ritz values
c     for the (preconditioned) matrix. Furthermore it calculates the
c     residual of these Ritz values. In this subroutine itmax should
c     be less than 150.
c
c **change3**
c
c reason    :  To implement the Eisenstat trick
c
c literature:  H. A. van der Vorst
c              Journal of Computational Physics 44 pp. 1-19 (1981)
c
c              S. C. Eisenstat
c              Research Report No. 185, Yale University, 1980
c
c **change3** end
c
c     We start with initial vector equal to zero, so the first
c     direction vector is equal to rhsd. In the case of a Dir-
c     ichlet boundary condition, we change the first direction
c     vector such that the corresponding eigenvalues are not
c     taken into account, because they are not important (for
c     convergence of the iterative method and the time integra-
c     tion).
c
c     The Ritz values are calculated using the EISPACK
c     subroutine hqr. See EISPACK guide (second edition)
c     p. 330-333 for further details.
c
c     The Ritz vectors are calculated using the EISPACK
c     subroutine invit. See EISPACK guide (second edition)
c     p. 390-394 for further details.
c
c     We calculate an approximation v of an eigenvector, which is used
c     to calculate the relative residual
c
c     | A*v - sigma*v | / |sigma|*|v| .
c
c     This vector v is obtained from v = V*y. Furthermore the matrix
c     A is the original matrix, combined with a pre- or postcondition-
c     ing if icontr(1).ne.0.
c
c     If infmat(1).eq.1 and icontr(1).eq.0 we also give the
c     approximations for the eigenvalues of the momentum
c     equations, translated and scaled as follows:
c
c          1/(dt)-sigma/rho
c
c     For details see forthcoming report.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Parameters:

      integer          n,itmax,icontr(6), infmat(*),
     +                 nprec,ndim,nconct, iinmtvc(*)
      double precision amat(n,*),rhsd(n),usol(n),solutv(*),
     +                 work(n,*),prec(n,*),rho,dt

c     --- Local parameters:

      logical select(150),start
      integer i,imax,iter,j,k,nm,ne,low,igh,ierr,m,l,
     +        lmin
      double precision rnorm,t,epsmac,eps,hh(151,150),c(150),
     +     s(150),rs(151),ddot,dnrm2,heig(151,150),wr(150),
     +     wi(150),wmin,eigv(151,150),rm1(151,150),rv1(150),
     +     rv2(150), residu(150),sqrt,resmax,dirich
      data epsmac/1.0d-16/

c     --- Determination of the maximum number of inner iterations.

      do j = 1, 150
         do k = 1, 151
            heig(k,j) = 0d0
         end do
      end do
      start = .true.
      icontr(4) = 0
      imax = min(150,itmax)

c     --- Main iteration loop.

      do 400 iter=1,imax

c     --- Compute residual vector and its norm in case of restart.

         if (start) then
           i=0

c          --- We start with initial vector equal to zero, so the first
c              direction vector is equal to rhsd. In the case of a Dir-
c              ichlet boundary condition, we change the first direction
c              vector such that the corresponding eigenvalues are not
c              taken into account, because they are not important (for
c              convergence of the iterative method and the time integra-
c              tion). At this moment we use the test:
c              |amat(j,1)-1| < 100*epsmac,
c              |amat(j,k)|   < 100*epsmac, k=2,3,...,nconct,
c              to discriminate for a Dirichlet boundary condition.

            do j=1,n
               work(j,2)=rhsd(j)
            end do
            do j=1,n
              dirich = abs(amat(j,1)-1.0)
              do k = 2, nconct
                 dirich = max(dirich,abs(amat(j,k)))
              end do
              if(dirich.lt.100.0*epsmac)
     +        work(j,2) = 0d0
            end do
            if(icontr(1).eq.2) then
             call dinvl(work(1,2),work,amat,n/ndim,ndim,
     +                  nconct,prec,nprec,infmat)
            else
             if(icontr(1).gt.0) then
              call prevc(n,work(1,2),work,amat,ndim,nconct,
     +                   prec,nprec,infmat,icontr(1))
             else
              call dcopy(n,work(1,2),1,work,1)
             end if
            end if
           rnorm = dnrm2(n,work,1)
           rs(1)=rnorm
           eps = epsmac*rnorm*1000.0d0

c          --- Check for convergence.

           if (rnorm.le.eps) then
              icontr(3)=0
              goto 500
           end if

           start = .false.
           do j = 1, n
              work(j,1) = work(j,1)/rnorm
           end do
         end if

c        --- Print intermediate results and set iteration level.

         call prires(' GMRITZ ',rnorm,icontr,.false.)
         icontr(4) = iter

c        --- Inner iteration step, apply modified Gram-Schmid to form a
c                 new column of hh.

         i=i+1
         if(icontr(1).eq.2) then

c        --- change3**

            call dinvu(work(1,i),work(1,i+1),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            do j = 1, n
               work(j,i+2) = (amat(j,1)-2d0)*work(j,i+1)
     +                       +work(j,i)
            end do
            call dinvl(work(1,i+2),work(1,i+3),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            call daxpy(n,1d0,work(1,i+3),1,work(1,i+1),1)

c           --- change3** end

         else
          if(icontr(1).ge.0) then



            call ismatvec(amat,work(1,i),solutv,work(1,i+2), infmat,
     +                    icontr, iinmtvc )

            call prevc(n,work(1,i+2),work(1,i+1),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))
          else
            call prevc(n,work(1,i),work(1,i+2),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))



            call ismatvec(amat,work(1,i+2),solutv,work(1,i+1), infmat,
     +                    icontr, iinmtvc )

          end if
         end if
         do j=1,i
            t = ddot(n,work(1,j),1,work(1,i+1),1)
            hh(j,i)=t
            heig(j,i)=t
            call daxpy(n,-t,work(1,j),1,work(1,i+1),1)
         end do
         hh(i+1,i)=dnrm2(n,work(1,i+1),1)
         heig(i+1,i)=hh(i+1,i)

c        --- Perform plane rotations on new column.

         if (i.gt.1) then
            do k=2,i
               t=hh(k-1,i)
               hh(k-1,i) =  c(k-1)*t + s(k-1)*hh(k,i)
               hh(k,i)   = -s(k-1)*t + c(k-1)*hh(k,i)
            end do
         end if

c        --- Calculate new plane rotation.

         t = sqrt(hh(i,i)**2 + hh(i+1,i)**2)
         if (t.eq.0.0d0) t=epsmac
         c(i) = hh(i,i)/t
         s(i) = hh(i+1,i)/t
         rs(i+1) = -s(i)*rs(i)
         rs(i) = c(i)*rs(i)
         hh(i,i) = c(i)*hh(i,i) + s(i)*hh(i+1,i)

c        --- Determine residual norm.

         rnorm = abs(rs(i+1))

c        --- Check for termination of inner iterations.

         if(rnorm.le.eps) then
            i = i-1
            goto 500
         else
            do j = 1, n
               work(j,i+1) = work(j,i+1)/hh(i+1,i)
            end do
         end if
 400  continue
500   continue
      icontr(3) = 0
      call prires('GMRITZ ',rnorm,icontr,.true.)

c     --- The Ritz values are calculated using the EISPACK
c         subroutine hqr. See EISPACK guide (second edition)
c         p. 330-333 for further details.

      nm  = 151
      ne  = i
      low = 1
      igh = ne
      do j = 1, 150
         do k = 1, 151
            hh(k,j) = heig(k,j)
         end do
      end do
      call hqr(nm,ne,low,igh,hh,wr,wi,ierr)

c     --- The Ritz vectors are calculated using the EISPACK
c         subroutine invit. See EISPACK guide (second edition)
c         p. 390-394 for further details.

      do j = 1, 150
         do k = 1, 151
            hh(k,j) = heig(k,j)
         end do
      end do
      do j = 1, ne
         select(j) = .true.
      end do
      call invit(nm,ne,hh,wr,wi,select,2*ne,m,
     +           eigv,ierr,rm1,rv1,rv2)

c     --- We calculate the residual of the Ritz value, which is given
c         by |H*y-sigma*y| / |sigma|*|y|

      i = 1
600   continue
      if(i.le.ne) then
         do k = 1, ne
            work(k,ne+1) = 0d0
            work(k,ne+2) = 0d0
         end do
         if(wi(i).eq.0) then
           do j = 1, ne
             do k = 1, ne
                work(k,ne+1) = work(k,ne+1)+heig(k,j)*eigv(j,i)
             end do
           end do
            do k = 1, ne
               work(k,ne+1) = work(k,ne+1)-wr(i)*eigv(k,i)
            end do
            residu(i) = dnrm2(ne,work(1,ne+1),1)/
     +                  (abs(wr(i))*dnrm2(ne,eigv(1,i),1))
            i = i+1
         else

c        --- If w(i).ne.0 then wr(i) is the real part and wi(i) is the
c            imaginary part of a pair of complex conjugate eigenvalues.
c            The real part of the Ritz vector is stored in eigv(.,i),
c            whereas the imaginary part of the Ritz vector is stored
c            in eigv(.,i+1).
c            See EISPACK guide (second edition)
c            p. 390-394 for further details.

            do j = 1, ne
               do k = 1, ne
                  work(k,ne+1) = work(k,ne+1)+heig(k,j)*eigv(j,i)
                  work(k,ne+2) = work(k,ne+2)+heig(k,j)*eigv(j,i+1)
               end do
            end do
            do k = 1, ne
               work(k,ne+1) = work(k,ne+1)-wr(i)*eigv(k,i)
     +                              +wi(i)*eigv(k,i+1)
               work(k,ne+2) = work(k,ne+2)-wr(i)*eigv(k,i+1)
     +                              -wi(i)*eigv(k,i)
            end do
            residu(i) = sqrt(dnrm2(ne,work(1,ne+1),1)**2
     +      +dnrm2(ne,work(1,ne+2),1)**2)/(sqrt(wr(i)**2
     +      +wi(i)**2)*sqrt(dnrm2(ne,eigv(1,i),1)**2
     +      +dnrm2(ne,eigv(1,i+1),1)**2))
            residu(i+1) = residu(i)
            i = i+2
         end if
         goto 600
      end if
      resmax = 0d0
      do k = 1, ne
         resmax = max(resmax,residu(k))
      end do

c     --- We calculate an approximation v of an eigenvector, which is used
c         to calculate the relative residual
c         | A*v - sigma*v | / |sigma|*|v| .
c         This vector v is obtained from v = V*y. Furthermore the matrix
c         A is the original matrix, combined with a pre- or postcondition-
c         ing if icontr(1).ne.0.

      i = 1
700   continue
      if(i.le.ne) then
         if(wi(i).eq.0) then
           do k = 1, n
              work(k,ne+1) = 0d0
           end do
           do j = 1, ne
              call daxpy(n,eigv(j,i),work(1,j),1,work(1,ne+1),1)
           end do
           k = ne+1
           if(icontr(1).eq.2) then

c          --- change3**

            call dinvu(work(1,k),work(1,k+1),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            do j = 1, n
               work(j,k+2) = (amat(j,1)-2d0)*work(j,k+1)
     +                       +work(j,k)
            end do
            call dinvl(work(1,k+2),work(1,k+3),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            call daxpy(n,1d0,work(1,k+3),1,work(1,k+1),1)

c           --- change3** end

         else
          if(icontr(1).ge.0) then



            call ismatvec(amat,work(1,k),solutv,work(1,k+2), infmat,
     +                    icontr , iinmtvc )

            call prevc(n,work(1,k+2),work(1,k+1),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))
          else
            call prevc(n,work(1,k),work(1,k+2),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))



            call ismatvec(amat,work(1,k+2),solutv,work(1,k+1), infmat,
     +                    icontr, iinmtvc )

          end if
         end if
            do k = 1, n
               work(k,ne+2) = work(k,ne+2)-wr(i)*work(k,ne+1)
            end do

c           --- residu(i) = dnrm2(n,work(1,ne+2),1)
c               write(6,*) residu(i),heig(ne+1,ne)*eigv(ne,i)

            residu(i) = dnrm2(n,work(1,ne+2),1)/
     +                  (abs(wr(i))*dnrm2(n,work(1,ne+1),1))
            i = i+1
         else
         do l = 1, 2
           do k = 1, n
              work(k,ne+l) = 0d0
           end do
           do j = 1, ne
              call daxpy(n,eigv(j,i+l-1),work(1,j),1,work(1,ne+l),1)
           end do
           k = ne+l
           if(icontr(1).eq.2) then

c          --- change3**

            call dinvu(work(1,k),work(1,k+1),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            do j = 1, n
               work(j,k+2) = (amat(j,1)-2d0)*work(j,k+1)
     +                       +work(j,k)
            end do
            call dinvl(work(1,k+2),work(1,k+3),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            call daxpy(n,1d0,work(1,k+3),1,work(1,k+1),1)

c           --- change3** end

         else
          if(icontr(1).ge.0) then



            call ismatvec(amat,work(1,k),solutv,work(1,k+2), infmat,
     +                    icontr, iinmtvc )

            call prevc(n,work(1,k+2),work(1,k+1),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))
          else
            call prevc(n,work(1,k),work(1,k+2),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))



            call ismatvec(amat,work(1,k+2),solutv,work(1,k+1), infmat,
     +                    icontr, iinmtvc )
          end if
         end if
         if(l.eq.1) then
            call dcopy(n,work(1,ne+2),1,work(1,ne+6),1)
         end if
         end do
         call dcopy(n,work(1,ne+3),1,work(1,ne+4),1)
         call dcopy(n,work(1,ne+6),1,work(1,ne+3),1)
            do k = 1, n
               work(k,ne+3) = work(k,ne+3)-wr(i)*work(k,ne+1)
     +                                    +wi(i)*work(k,ne+2)
               work(k,ne+4) = work(k,ne+4)-wr(i)*work(k,ne+2)
     +                                    -wi(i)*work(k,ne+1)
            end do

c           --- residu(i) = sqrt(dnrm2(n,work(1,ne+3),1)**2
c               1                  +dnrm2(n,work(1,ne+4),1)**2)
c               residu(i+1) = sqrt(eigv(ne,i)**2+eigv(ne,i+1)**2)*
c               1                    abs(heig(ne+1,ne))

            residu(i) = sqrt(dnrm2(n,work(1,ne+3),1)**2
     +                  +dnrm2(n,work(1,ne+4),1)**2)/
     +      (sqrt(wr(i)**2+wi(i)**2)*
     +       sqrt(dnrm2(n,work(1,ne+1),1)**2+
     +            dnrm2(n,work(1,ne+2),1)**2))
             residu(i+1) = residu(i)
            i = i+2
         end if
         goto 700
      end if


c     --- Find the eigenvalue with the smallest real part.
c         ----------------------------------------------------------------

      do l = 1, ne
         wmin = wr(l)
         lmin = l
         do k = l,ne
            if(wmin.gt.wr(k)) then
               wmin = wr(k)
               lmin = k
            end if
         end do


c        --- Change the real and imaginary parts.
c            ----------------------------------------------------------------

         wr(lmin) = wr(l)
         wr(l)    = wmin
         wmin     = wi(lmin)
         wi(lmin) = wi(l)
         wi(l)    = wmin
         wmin     = residu(lmin)
         residu(lmin) = residu(l)
         residu(l)    = wmin
      end do

c     --- End of the Ritz value calculation.
c         -----------------------------------------------------------------

      write(6,1000)
      if(infmat(1).eq.1) then
         write(6,1010)
      else
         if(infmat(1).eq.4) then
            write(6,1020)
         else
            write(6,1030) infmat(1)-4
         end if
      end if
      if(icontr(1).ne.0) write(6,1040) icontr(1)
      if(i.lt.imax) then
         write(6,1050) i
      else
         if(imax.lt.itmax) then
            write(6,1060) imax
         else
            write(6,1070) imax
         end if
      end if
      write(6,1075) resmax
      write(6,1080)
      do j = 1, ne
         write(6,1090) wr(j),wi(j),residu(j)
      end do

c     --- If infmat(1).eq.1 and icontr(1).eq.0 we also give the
c         approximations for the eigenvalues of the momentum
c         equations, translated and scaled as follows:
c              1/(dt)-sigma/rho
c         For details see forthcoming report.

      if(infmat(1).eq.1.and.icontr(1).eq.0) then
         do j = 1, ne
            wr(j) = 1.0/dt-(wr(j)/rho)
            wi(j) = wi(j)/rho
         end do
         write(6,1100)
         do j = 1, ne
            write(6,1090) wr(j),wi(j),residu(j)
         end do
      end if
1000  format(/,/,' Ritz values are approximations of eigenvalues.',
     +       /,' In the table below, some Ritz values are'
     +        ,' calculated. ',/,/)
1010  format(' The momentum equations',
     +     /,' ----------------------')
1020  format(' The pressure equation',
     +     /,' ---------------------')
1030  format(' The transport equation with number',i4,
     +     /,' ----------------------------------')
1040  format(/,' We use the preconditioning',i4)
1050  format(/,' The current righthand-side vector has only',
     +     i4,/,' nonzero components in eigenvectordirections',
     +     /,' corresponding to different eigenvalues')
1060  format(' There are only',i4,' Ritz values calculated')
1070  format(' There are',i4,' Ritz values calculated')
1075  format(' The Ritz values are calculated with a relative',
     +     /,' accuracy equal to ',e15.5)
1080  format(/,/,' The calculated Ritz values are :',/,/,
     +    '    Real part      Imaginary part     Residual'
     + ,/,'    --------- --------------     --------',/)
1090  format(e15.5,e15.5,'     ',e15.5)
1100  format(/,/,' The momentum equations can be written as',
     +       /,/,'        du/dt = M(u)u.',/,/,
     +          ' The eigenvalues of the operator M(u) are',/,
     +          ' approximated by:',/,
     +          ' (warning: this is only correct if rho is',
     +          ' constant)',/,/,
     +    '    Real part      Imaginary part     Residual'
     + ,/,'    --------- --------------     --------',/)
      return
      end
