      subroutine isdinvl ( bresul, amat, nusol, ndimso, nconct,
     +                     initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.2    date 06-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.1    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.0    date 28-10-1994  extracted from old ISNaS
c                       subroutine dinvl.
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of b by L, the preconditioning matrix given
c     in amat.
c     In this case we obtain:
c                 -1
c            b = L  * b.
c     The array amat should be filled by isdmlu.f This subroutine
c     contains compiler directives to run in vector speed on the
c     Convex.
c     Warning
c     -------
c     In this subroutine it is possible that shifts correctly point
c     into another degree of freedom, when the momentum equations
c     are solved. For instance the loop before the comment
c     --- Below the remaining part is solved for the momentum equations.
c     refers to bresul(nusol+1,1) which is correct and should be
c     treated as bresul(1,2).
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*),nusol,ndimso,nconct
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 bresul(nusol,ndimso)

c     amat      i    The coefficients matrix used in the linear system
c     bresul   i/o   The result vector
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, ii, nx, ny, jjmin, jjmax
      logical coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     ii        Loop variable
c     j         General loop variable
c     jjmax     Loop bound
c     jjmin     Loop bound
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of b by L preconditioning matrix given in amat.
c     In this case we obtain:
c                 -1
c            b = L  b.
c
c     The array amat should be filled by subroutine isdmlu.f. This
c     subroutine is closely linked to isdinvu.f, where the upper
c     triangular system is solved.
c
c     In this subroutine the lower triangular system is solved in such
c     a way that it runs in vector speed on the Convex C3840 vector-
c     computer. The literature where this approach is described is:
c
c     Literature
c
c     C.C. Ashcraft and R.G. Grimes,
c     On vectorizing incomplete factorization and SSOR preconditioners,
c     SIAM. J. Sci. Stat. Comput. 9 pp. 122--151 (1988).
c
c     H. A. van der Vorst,
c     High performance preconditioning,
c     SIAM. J. Sci. Stat. Comput. 10 pp.1174-1185 (1989).
c
c     C. Vuik,
c     Fast iterative solvers for the discretized incompressible Navier-
c     Stokes equations,
c     TUD Report 93-98,
c
c
c     Below we give the vectorized code for the main do-loops. We start
c     with the 9-point stencil used in the pressure and transport equation,
c     and the first part of the momentum equations.
c
c     We note that due to the 9-point stencil the vectorization
c     is more difficult than for a 5-point stencil. Consider the
c     following two-dimensional grid:
c
c
c                o  o  o  o  o
c
c                o  o  o  o  o
c
c                +  *  *  o  o
c         ^
c         |      +  *  *  *  o
c
c        i2      +  +  +  +  +
c
c                   i1 ->
c
c     * 9-point stencil, o value of b unknown, + value of b known.
c
c     We describe the position by two counters i1 along the x1-axis,
c     and i2 along the x2-axis. It follows that the points on the
c     diagonals  i1 + 2*i2 = constant, can be calculated independently
c     (so in vector speed). We distinguish odd and even diagonals. The
c     ii-th odd diagonal is a line through the points (1,ii) and
c     (2*ii-1,1) and is considered in the first loop (constant = 2*ii+1),
c     the ii-th even diagonal is a line through the points (2,ii) and
c     (2*ii,1) and is considered in the second loop (constant = 2*ii+2).
c     The number of odd (even) diagonals is bounded by ii <= ny+(nx-1)/2.
c     The other bounds, jjmin and jjmax, come from the fact that the
c     diagonals are only considered in the rectangular block:
c     1 <= i1 <= nx, and 2  <= i2 <= ny (note that i2 = 1 is considered
c     separately). Finally we explain the formula
c
c                 i = 2*ii+1-2*j+(j-1)*nx                            (1)
c
c     in the first loop (the explanation for the second loop is
c     comparable). First we have the formula
c
c                 i1 + 2*i2 = 2*ii + 1
c
c     so i1 = 2*ii + 1 - 2*i2. Secondly the 2-dimensional vector is
c     stored as a one dimensional array, where
c
c                 i = i1 + (i2-1)*nx                                 (2)
c
c     Combination of (1) and (2) leads to
c
c                 i = 2*ii+1-2*i2+(i2-1)*nx,
c
c     substitute j = i2 and one gets the correct formula.
c
c     If the momentum equations are solved we have to solve the second
c     part too. The do loops corresponding to the coupling between both
c     parts are trivially vectorized. Finally the remaining equations can
c     be vectorized in exactly the same way as the first part.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nx = initer(12)+1
      ny = initer(13)+1
      coupled  = initer(29).eq.0

c     --- First we fill the points on the lower horizontal boundary.

      do i = 2, nx
         bresul(i,1) = bresul(i,1)
     +                 -amat(i,1,5)*bresul(i-1,1)
      end do

c     --- Thereafter the first two points are filled on the second horizontal
c         line.

      i = nx+1
      bresul(i,1) = bresul(i,1)
     +              -amat(i,1,5)*bresul(i-1,1)
     +              -amat(i,1,4)*bresul(2,1)
     +              -amat(i,1,3)*bresul(1,1)
      i = nx+2
      bresul(i,1) = bresul(i,1)
     +              -amat(i,1,5)*bresul(i-1,1)
     +              -amat(i,1,4)*bresul(3,1)
     +              -amat(i,1,3)*bresul(2,1)
     +              -amat(i,1,2)*bresul(1,1)

      do ii = 3, ny+(nx-1)/2
         jjmin = max(2,ii-(nx-1)/2)
         jjmax = min(ii,ny)

c$dir no_recurrence

         do j = jjmin, jjmax

c        --- the odd diagonals

            i = 2*ii+1-2*j+(j-1)*nx
            bresul(i,1) = bresul(i,1)
     +                    -amat(i,1,5)*bresul(i-1   ,1)
     +                    -amat(i,1,4)*bresul(i-nx+1,1)
     +                    -amat(i,1,3)*bresul(i-nx  ,1)
     +                    -amat(i,1,2)*bresul(i-nx-1,1)
         end do
         jjmin = max(2,ii+1-nx/2)
         jjmax = min(ii,ny)

c$dir no_recurrence

         do j = jjmin, jjmax

c        --- the even diagonals

            i = 2*ii+2-2*j+(j-1)*nx
            bresul(i,1) = bresul(i,1)
     +                    -amat(i,1,5)*bresul(i-1   ,1)
     +                    -amat(i,1,4)*bresul(i-nx+1,1)
     +                    -amat(i,1,3)*bresul(i-nx  ,1)
     +                    -amat(i,1,2)*bresul(i-nx-1,1)
         end do
      end do

c     --- If initer(11).eq.1 .and. coupled we solve the coupled momentum
c         equations. In this case b(i,2) should be calculated too.

      if ( initer(11).eq.1 .and. coupled ) then

c     --- First the coupling between the velocities.
c         First we fill the points on the lower horizontal boundary.

c$dir no_recurrence

         do i = 1, nx-1
            bresul(i,2) = bresul(i,2)
     +                    -amat(i,2,13)*bresul(i+1   ,1)
     +                    -amat(i,2,12)*bresul(i     ,1)
         end do

         i = nx
         bresul(i,2) = bresul(i,2)
     +                 -amat(i,2,13)*bresul(i+1   ,1)
     +                 -amat(i,2,12)*bresul(i     ,1)
     +                 -amat(i,2,11)*bresul(i-nx+1,1)

c$dir no_recurrence

         do i = nx+1, nusol
            bresul(i,2) = bresul(i,2)
     +                    -amat(i,2,13)*bresul(i+1   ,1)
     +                    -amat(i,2,12)*bresul(i     ,1)
     +                    -amat(i,2,11)*bresul(i-nx+1,1)
     +                    -amat(i,2,10)*bresul(i-nx  ,1)
         end do

c        --- Below the remaining part is solved for the coupled momentum
c            equations.
c            First we fill the points on the lower horizontal boundary.

         do i = 2, nx
            bresul(i,2) = bresul(i,2)
     +                    -amat(i,2,5)*bresul(i-1,2)
         end do

c        --- Thereafter the first two points are filled on the second horizontal
c            line.

         i = nx+1
         bresul(i,2) = bresul(i,2)
     +                 -amat(i,2,5)*bresul(i-1,2)
     +                 -amat(i,2,4)*bresul(2,2)
     +                 -amat(i,2,3)*bresul(1,2)
         i = nx+2
         bresul(i,2) = bresul(i,2)
     +                 -amat(i,2,5)*bresul(i-1,2)
     +                 -amat(i,2,4)*bresul(3,2)
     +                 -amat(i,2,3)*bresul(2,2)
     +                 -amat(i,2,2)*bresul(1,2)

         do ii = 3, ny+(nx-1)/2
            jjmin = max(2,ii-(nx-1)/2)
            jjmax = min(ii,ny)

c$dir no_recurrence

            do j = jjmin, jjmax

c           --- the odd diagonals

               i = 2*ii+1-2*j+(j-1)*nx
               bresul(i,2) = bresul(i,2)
     +                       -amat(i,2,5)*bresul(i-1   ,2)
     +                       -amat(i,2,4)*bresul(i-nx+1,2)
     +                       -amat(i,2,3)*bresul(i-nx  ,2)
     +                       -amat(i,2,2)*bresul(i-nx-1,2)
            end do
            jjmin = max(2,ii+1-nx/2)
            jjmax = min(ii,ny)

c$dir no_recurrence

            do j = jjmin, jjmax

c           --- the even diagonals

               i = 2*ii+2-2*j+(j-1)*nx
               bresul(i,2) = bresul(i,2)
     +                       -amat(i,2,5)*bresul(i-1   ,2)
     +                       -amat(i,2,4)*bresul(i-nx+1,2)
     +                       -amat(i,2,3)*bresul(i-nx  ,2)
     +                       -amat(i,2,2)*bresul(i-nx-1,2)
            end do

         end do
      end if
      end
