      subroutine isdinvl4(x,b,n,ndimso,
     1                  prec,nprec,initer)
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.3      date 04-08-1992  Zdenek: ndimso instead of ndim
c     version 1.2      date 01-06-1992
c     developed at     Convex, HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by L, the preconditioning matrix given in
c     prec.
c
c     In this case we obtain:
c                 -1
c            b = L  x.
c
c     The array prec should be filled by dmlu3.f. This subroutine
c     contains compiler directives to run in vector speed on the
c     Convex.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilu
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c
c     b       o  the result vector, which contains:
c                         -1
c                    b = L  x.
c
c     infmat  i  if infmat(1) is 1 we use the momentum equations,
c                whereas if infmat(1) is larger than or equal to 4
c                we use equations, with a structure similar to the
c                pressure equations.
c
c     n       i  number of unknowns in the solution vector.
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. 
c
c     prec    i  the preconditioning matrix.
c
c     x       i  the original vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter.
c
c     ii      loop counter.
c
c     j       loop counter.
c
c     jjmax   loop bound.
c
c     jjmin   loop bound.
c
c     nx      number of points in the x-direction.
c
c     ny      number of points in the y-direction.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of x with L preconditioning matrix given
c     in matrix and prec. In this case we obtain:
c                 -1
c            b = L  x.
c
c     The arrays prec and matrix should be filled by subroutine dmlu3.f
c
c     **************************************************************
c
c     Change1 15-02-1991 C. Vuik.
c
c     The reason of this change is to solve the lower triangular system
c     in vector speed.
c
c     Literature : High performance preconditioning.
c                  H. A. van der Vorst.
c                  SIAM. J. Sci. Stat. Comput. 10 pp.1174-1185 (1989).
c
c     Implementation : Compare subroutine precd in the program ictest
c                      written by H. A. van der Vorst.
c
c     Remark     : In this version the virtual points along the bound-
c                  ary are made zero. Thereafter the loop only runs
c                  over the non-boundary points.
c
c     **************************************************************
c
c     Change2 01-06-1992 C. Vuik
c
c     The virtual points are removed.
c
c     **************************************************************
c
c     Remark
c
c     - at this moment only infmat(1).ge.4 is allowed.
c
c     **************************************************************
c
c ==================================================================
c
c     Parameters
c
      implicit none
      integer          initer(*),n,ndimso,nprec, p
      double precision prec(1:n,1:nprec),
     2                 x(n),b(n)
c
c     local parameters
c
      integer i,n1,nfill,nfilon,nfilbo

c     statement hierna moet weg bij opschonen

      i = ndimso

c     statement hierboven moet weg bij opschonen

      n1 = initer(12)+1
c                             -1
c     The multiplication b = L  x
c
      nfill = (nprec-9)/2
      nfilon = nfill/2
      nfilbo = nfill-nfilon
      do i = 1, n1+1
         b(i) = x(i)
         do p = 1, nfilon+3
            if((i-n1-2+p).ge.1) then
               b(i) = b(i)-prec(i,p+1)*b(i-n1-2+p)
            endif
         enddo
         do p = 1, nfilbo+1
            if((i-nfilbo-2+p).ge.1) then
               b(i) = b(i)-prec(i,p+nfilon+4)*b(i-nfilbo-2+p)
            endif
         enddo
      enddo
      if(nfill.eq.n1-3) then
         do i = n1+2, n
            b(i) = x(i)
            do p = 2, n1+2
               if(i-n1-3+p.ge.1) then
                  b(i) = b(i)-prec(i,p)*b(i-n1-3+p)
               endif
            enddo
         enddo
      elseif(nfill.eq.0) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,5)*b(i-1)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.1) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,6)*b(i-1)
     .                    -prec(i,5)*b(i-2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.2) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,7)*b(i-1)
     .                    -prec(i,6)*b(i-2)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.3) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,8)*b(i-1)
     .                    -prec(i,7)*b(i-2)
     .                    -prec(i,6)*b(i-3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.4) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,9)*b(i-1)
     .                    -prec(i,8)*b(i-2)
     .                    -prec(i,7)*b(i-3)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.5) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,10)*b(i-1)
     .                    -prec(i,9)*b(i-2)
     .                    -prec(i,8)*b(i-3)
     .                    -prec(i,7)*b(i-4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.6) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,11)*b(i-1)
     .                    -prec(i,10)*b(i-2)
     .                    -prec(i,9)*b(i-3)
     .                    -prec(i,8)*b(i-4)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.7) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,12)*b(i-1)
     .                    -prec(i,11)*b(i-2)
     .                    -prec(i,10)*b(i-3)
     .                    -prec(i,9)*b(i-4)
     .                    -prec(i,8)*b(i-5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.8) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,13)*b(i-1)
     .                    -prec(i,12)*b(i-2)
     .                    -prec(i,11)*b(i-3)
     .                    -prec(i,10)*b(i-4)
     .                    -prec(i,9)*b(i-5)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.9) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,14)*b(i-1)
     .                    -prec(i,13)*b(i-2)
     .                    -prec(i,12)*b(i-3)
     .                    -prec(i,11)*b(i-4)
     .                    -prec(i,10)*b(i-5)
     .                    -prec(i,9)*b(i-6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.10) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,15)*b(i-1)
     .                    -prec(i,14)*b(i-2)
     .                    -prec(i,13)*b(i-3)
     .                    -prec(i,12)*b(i-4)
     .                    -prec(i,11)*b(i-5)
     .                    -prec(i,10)*b(i-6)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.11) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,16)*b(i-1)
     .                    -prec(i,15)*b(i-2)
     .                    -prec(i,14)*b(i-3)
     .                    -prec(i,13)*b(i-4)
     .                    -prec(i,12)*b(i-5)
     .                    -prec(i,11)*b(i-6)
     .                    -prec(i,10)*b(i-7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.12) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,17)*b(i-1)
     .                    -prec(i,16)*b(i-2)
     .                    -prec(i,15)*b(i-3)
     .                    -prec(i,14)*b(i-4)
     .                    -prec(i,13)*b(i-5)
     .                    -prec(i,12)*b(i-6)
     .                    -prec(i,11)*b(i-7)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.13) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,18)*b(i-1)
     .                    -prec(i,17)*b(i-2)
     .                    -prec(i,16)*b(i-3)
     .                    -prec(i,15)*b(i-4)
     .                    -prec(i,14)*b(i-5)
     .                    -prec(i,13)*b(i-6)
     .                    -prec(i,12)*b(i-7)
     .                    -prec(i,11)*b(i-8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.14) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,19)*b(i-1)
     .                    -prec(i,18)*b(i-2)
     .                    -prec(i,17)*b(i-3)
     .                    -prec(i,16)*b(i-4)
     .                    -prec(i,15)*b(i-5)
     .                    -prec(i,14)*b(i-6)
     .                    -prec(i,13)*b(i-7)
     .                    -prec(i,12)*b(i-8)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.15) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,20)*b(i-1)
     .                    -prec(i,19)*b(i-2)
     .                    -prec(i,18)*b(i-3)
     .                    -prec(i,17)*b(i-4)
     .                    -prec(i,16)*b(i-5)
     .                    -prec(i,15)*b(i-6)
     .                    -prec(i,14)*b(i-7)
     .                    -prec(i,13)*b(i-8)
     .                    -prec(i,12)*b(i-9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.16) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,21)*b(i-1)
     .                    -prec(i,20)*b(i-2)
     .                    -prec(i,19)*b(i-3)
     .                    -prec(i,18)*b(i-4)
     .                    -prec(i,17)*b(i-5)
     .                    -prec(i,16)*b(i-6)
     .                    -prec(i,15)*b(i-7)
     .                    -prec(i,14)*b(i-8)
     .                    -prec(i,13)*b(i-9)
     .                    -prec(i,12)*b(i-n1+9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.17) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,22)*b(i-1) -prec(i,21)*b(i-2)
     .                    -prec(i,20)*b(i-3) -prec(i,19)*b(i-4)
     .                    -prec(i,18)*b(i-5) -prec(i,17)*b(i-6)
     .                    -prec(i,16)*b(i-7)
     .                    -prec(i,15)*b(i-8)
     .                    -prec(i,14)*b(i-9)
     .                    -prec(i,13)*b(i-10)
     .                    -prec(i,12)*b(i-n1+9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.18) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,23)*b(i-1) -prec(i,22)*b(i-2)
     .                    -prec(i,21)*b(i-3) -prec(i,20)*b(i-4)
     .                    -prec(i,19)*b(i-5) -prec(i,18)*b(i-6)
     .                    -prec(i,17)*b(i-7)
     .                    -prec(i,16)*b(i-8)
     .                    -prec(i,15)*b(i-9)
     .                    -prec(i,14)*b(i-10)
     .                    -prec(i,13)*b(i-n1+10)
     .                    -prec(i,12)*b(i-n1+9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.19) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,24)*b(i-1) -prec(i,23)*b(i-2)
     .                    -prec(i,22)*b(i-3) -prec(i,21)*b(i-4)
     .                    -prec(i,20)*b(i-5) -prec(i,19)*b(i-6)
     .                    -prec(i,18)*b(i-7)
     .                    -prec(i,17)*b(i-8)
     .                    -prec(i,16)*b(i-9)
     .                    -prec(i,15)*b(i-10)
     .                    -prec(i,14)*b(i-11)
     .                    -prec(i,13)*b(i-n1+10)
     .                    -prec(i,12)*b(i-n1+9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      elseif(nfill.eq.20) then
         do i = n1+2, n
               b(i) = x(i)-prec(i,25)*b(i-1) -prec(i,24)*b(i-2)
     .                    -prec(i,23)*b(i-3) -prec(i,22)*b(i-4)
     .                    -prec(i,21)*b(i-5) -prec(i,20)*b(i-6)
     .                    -prec(i,19)*b(i-7) -prec(i,18)*b(i-8)
     .                    -prec(i,17)*b(i-9)
     .                    -prec(i,16)*b(i-10)
     .                    -prec(i,15)*b(i-11)
     .                    -prec(i,14)*b(i-n1+11)
     .                    -prec(i,13)*b(i-n1+10)
     .                    -prec(i,12)*b(i-n1+9)
     .                    -prec(i,11)*b(i-n1+8)
     .                    -prec(i,10)*b(i-n1+7)
     .                    -prec(i,9)*b(i-n1+6)
     .                    -prec(i,8)*b(i-n1+5)
     .                    -prec(i,7)*b(i-n1+4)
     .                    -prec(i,6)*b(i-n1+3)
     .                    -prec(i,5)*b(i-n1+2)
     .                    -prec(i,4)*b(i-n1+1)
     .                    -prec(i,3)*b(i-n1  )
     .                    -prec(i,2)*b(i-n1-1)
         enddo
      else
         do i = n1+2, n
            b(i) = x(i)
            do p = 1, nfilon+3
               if((i-n1-2+p).ge.1) then
                  b(i) = b(i)-prec(i,p+1)*b(i-n1-2+p)
               endif
            enddo
            do p = 1, nfilbo+1
               if((i-nfilbo-2+p).ge.1) then
                  b(i) = b(i)-prec(i,p+nfilon+4)*b(i-nfilbo-2+p)
               endif
            enddo
         enddo
      endif
c      write(6,1000) nfill
c1000  format('de waarde van nfill',i4)
      end
