      subroutine isextim2 ( solut, rhside, rho, dt, ni, nj, nvirtual,
     +                      ndegfd, mcom, heunstep )
c ======================================================================
c
c        programmer     Kees Vuik
c
c        version 1.1    date 29-10-1998   Duncan: adapted for compressible
c        version 1.0    date 30-11-1995
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Subroutine isextim2 calculates the solution at the new
c    time level
c **********************************************************************
c
c                       KEYWORDS
c
c     time integration
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ndegfd, mcom, heunstep
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual, ndegfd, *),
     +                 rho  (1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 rhside(ni+1,nj+1,2), dt

c     dt        i    Time step
c     heunstep  i    Stepnumber in Heun's method
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rho       i    Estimate of the asymptotic convergence rate
c     rhside    i    reserved
c     solut     o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i, j, k
      double precision rhoint

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     rhoint    Value of rho in integration point
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The right hand side vector is multiplied by dt and divided by
c    rho (density) and added to the solution of the preceding
c    time level. An average of rho is done to calculate it in
c    the velocity points. The 1 and 2 direction are updated
c    seperately
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen( 'isextim2')

      if( mcom.eq.1) then

c     --- Update in 1 direction

c         --- changed for shv rk method

         k = 1
         do j = 1, nj
            do i = 1, ni+1
               rhoint = 0.5d0*(rho(i-1,j)+rho(i,j))
               solut(i,j,k,2) = solut(i,j,k,1)*rhoint+
     +         solut(i,j,k,2)*dble(heunstep-1)+rhside(i,j,k)*dt
            end do
         end do

c        --- Update in 2 direction

         k = 2
         do j = 1, nj+1
            do i = 1, ni
               rhoint = 0.5d0*(rho(i,j-1)+rho(i,j))
               solut(i,j,k,2) = solut(i,j,k,1)*rhoint+
     +         solut(i,j,k,2)*dble(heunstep-1)+rhside(i,j,k)*dt
            end do
         end do

      else if( mcom.eq.0) then

c     --- Update in 1 direction

         k = 1
         do j = 1, nj
            do i = 1, ni+1
               rhoint = 0.5d0*(rho(i-1,j)+rho(i,j))
               solut(i,j,k,2) = solut(i,j,k,2)+rhside(i,j,k)*dt/rhoint
            end do
         end do

c        --- Update in 2 direction

         k = 2
         do j = 1, nj+1
            do i = 1, ni
               rhoint = 0.5d0*(rho(i,j-1)+rho(i,j))
               solut(i,j,k,2) = solut(i,j,k,2)+rhside(i,j,k)*dt/rhoint
            end do
         end do

      end if

      call iserclos('isextim2')

      end
