      subroutine isextr ( iequa, solut, infmat, work, nusol, nsolut,
     +                    ndegfd, ipstrt, ndim,
     +                    matrix, rhside, nconct ,n, coupled)
c ======================================================================
c
c        programmer :    Kees Kassels
c
c     version      3.3  07-07-1998  Kees V: nusol, nsolut cleaned
c     version      3.2  13-10-1997  Charles: uncoupled allowed
c     version      3.1  28-07-1994  Zijlema: correction in view of 3D
c     version      3.0  24-01-1994  Kees K. change for new structure
c     version      2.1  26-04-1993  C. Vuik: The startvector satisfies the
c                                   Dirichlet boundary conditions.
c     version      2.0  14-07-1992  Zdenek: extension for 3D,
c                                   new parameter list
c   copyright (c) 1994-1998 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine extracts the first estimate of the solution vector
c     from the array SOLUT and puts the result into array WORK.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  infmat(*), nusol, nsolut, ndegfd, ipstrt, ndim,
     +         nconct, n, iequa
      logical coupled
      double precision solut(*), work(*), matrix(n,nconct), rhside(n)

c     coupled   i    Logical which is true if the momentum equations
c                    are coupled
c     iequa     i    General equation sequence number
c     infmat    i    Integer information for the matrix
c     ipstrt    i    Choice parameter for the startvector
c                      IPSTRT = iinsol(4) ;
c                      IPSTRT = 0 : Startvector = 0
c                      IPSTRT = 2 : Startvector = taken from the previous
c                                                 multi-block iteration
c                                                 or time step.
c     matrix    i    reserved
c     n         i    Number of different array descriptions in infmat
c     nconct    i    Number of connections in a row of the matrix
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     nsolut    i    Number of unknowns for all degrees of freedom in one block
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     rhside    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c      -
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idegfd, istartv, j, ntimlv
      double precision rdummy

c     i         General loop variable
c     idegfd    Loop variable with respect to a given unknown
c     istartv   First free position in a vector
c     j         General loop variable
c     ntimlv    Number of time levels
c     rdummy    Real dummy
c **********************************************************************
c
c                       I/O
c
c      -
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1000    Actual degree of freedom to small or too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     -
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isextr')

c     --- Put present solution from level time into work

      if ( coupled ) then
         if( iequa.eq.1 ) idegfd =1
         if( iequa.gt.1 ) idegfd = iequa + ndim - 1
      else
         idegfd = iequa
      end if

      if ( idegfd.le.0 .or. idegfd.gt.ndegfd ) then
         call iserrsub ( 1000, 0, 0, 0 )
         stop
      end if

      ntimlv = nsolut/nusol/ndegfd
      istartv = (idegfd-1)*nusol + (ntimlv-1)*nusol*ndegfd

      if ( infmat(8).eq.1  ) then

c     --- Coupled velocity unknown

         if ( ipstrt.eq.2 ) then

            do i = 1, nusol*ndim
               work(i) = solut(istartv+i)
            end do

         else

            do i = 1, nusol*ndim
               work(i) = 0d0
            end do

         end if

      else if ( infmat(8) .eq.3 ) then

c     --- Pressure equation

         if ( ipstrt.eq.2 ) then
            do i = 1, nusol
               work(i) = solut(istartv+i)
            end do
         else
            do i = 1, nusol
               work(i) =  0d0
            end do
         end if

      else

c     --- Other type of equations

         if ( ipstrt.eq.2 ) then
            do i = 1, nusol
               work(i) = solut(istartv+i)
            end do
         else
            do i = 1, nusol
               work(i) = 0d0
            end do
         end if

      end if

c     --- **change**  C. Vuik

      do i = 1, n
         if ( matrix(i,1).eq.1 ) then
            rdummy = 0d0
            do j = 2, nconct
               rdummy = rdummy + abs(matrix(i,j))
            end do
            if ( rdummy.eq.0 ) then
               work(i) = rhside(i)
            end if
         end if
      end do

c     --- end of **change**  C. Vuik

      call iserclos('isextr')
      end
