      subroutine islsol ( iequa, iinput, rinput, infmm,
     +                    infmt, matrix, rhside, solut, solutv,
     +                    prec, work ,lwork, iinmtvc  )
c ======================================================================
c
c     programmer   Kees Kassels
c     version 4.5  date  07-07-1998 Kees V: nusol, nsolut cleaned
c     version 4.4  date  20-11-1997 Kees K. iinmtvc added
c     version 4.3  date  19-11-1997 Guus: correction computation ipiinsol
c     version 4.2  date  06-11-1997 Guus: remove printing
c     version 4.1  date  19-09-1997 Kees V. changes for uncoupled
c     version 4.0  date  06-08-1996 Guus: Remove infmg
c     version 3.8  date  23-04-1996 Guus: New timing subroutines
c     version 3.7  date  18-03-1996 Kees V. adding ILU in 3D
c     version 3.6  date  03-10-1995 Kees K. added array solutv
c     version 3.5  date  15-09-1995 Kees V. added ILU_Fill
c     version 3.4  date  04-05-1995 Guus: correction start address print
c                                   pressure
c     version 3.3  date  21-03-1995 Guus: correction print pressure
c     version 3.2  date  27-02-1995 Guus: print solution
c     version 3.1  date  14-02-1995 Kees V. clean up and timing routines
c     version 3.0  date  20-01-1994 change for new data structure
c     version 2.3  date  11-12-1992 Kees K.: changed call of lmgslv
c     version 2.2  date  21-09-1992 Kees V.: correct allocation of
c                                   workspace for the CG-
c                                   type solver.
c     version 2.1  date  09-09-1992 Guus: extension for multigrid
c     version 2.0  date  29-07-1992 Zdenek: extension for 3D
c
c
c   copyright (c) 1991-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine to solve the system of equations
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer iequa, iinput(*), infmm(*), iinmtvc(*),
     +        infmt(*), lwork
      double precision rinput(*), matrix(*), rhside(*), solut(*),
     +                 prec(*), work(*), solutv(*)

c     iequa     i    General equation sequence number
c     iinmtvc   i    reserved
c     iinput    i    reserved
c     infmm     i    Integer information for Momentum matrix
c     infmt     i    Integer information for Pressure and Transport matrix
c     lwork     i    Length of array work
c     matrix    i    reserved
c     prec      i    The preconditioning matrix
c     rhside    i    reserved
c     rinput    i    reserved
c     solut    i/o   reserved
c     solutv   i/o   Work array for solut with virtual points
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinsol, iprinsol, leniinsol, lenrinsol,
     +        ndegfd, ndim, ni, nj, nk, nprec, nsolut, ntrnsp, nturb,
     +        nusol
      logical coupled
      character*10 matname
      character*39 text

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     iprinsol  Starting address of array RINSOL in array RINPUT
c     leniinsol Length of first component of array iinsol
c     lenrinsol Length of first component of array rinsol
c     matname   Name of matrix to be build or solved
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nprec     Maximum number of diagonals in preconditioning matrix
c     nsolut    Number of unknowns for all degrees of freedom in one block
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     text      String in which a text is stored
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKCLK  Print CPU time used
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISSETCLK  Set time in common cistime equal to actual CPU time
c     ISSOLV    Subroutine to solve the linear system
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   fills all the information used in the issolv
c   subroutine and the timing of the linear solver.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('islsol')

      ni       = iinput( 51)
      nj       = iinput( 52)
      nk       = iinput( 53)
      ndegfd   = iinput( 55)
      ndim     = iinput( 57)
      ntrnsp   = iinput( 63)
      nturb    = iinput( 70)
      leniinsol = iinput(105)
      lenrinsol = iinput(130)
      coupled  = iinput(76).eq.0

      nusol    = (ni+1)*(nj+1)*(nk+1)
      nsolut   = (ni+1)*(nj+1)*(nk+1)
     +          *(ndim+1+ntrnsp+nturb)*2
      ipiinsol = iinput(5)+(iequa-1)*leniinsol
      iprinsol = iinput(30)+(iequa-1)*lenrinsol

      nprec = 0
      iinput(ipiinsol-1+22) = min ( iinput(ipiinsol-1+22), ni-2 )
      if( abs(iinput(ipiinsol-1+2)).eq.1) nprec = 1
      if( abs(iinput(ipiinsol-1+2)).eq.2) nprec = 2
      if( abs(iinput(ipiinsol-1+2)).eq.3) then
         if ( ndim.eq.2 ) nprec = 9
         if ( ndim.eq.3 ) nprec = 19
      end if
      if ( abs(iinput(ipiinsol-1+2)).eq.4 )
     +    nprec = 2*iinput(ipiinsol-1+22)+9

      if ( itime.eq.1 ) then

c     --- Start timer for linear system solver

         call issetclk(1)

      end if

      if ( nerror.ne.0 ) go to 1000
      if ( iequa.eq.1 .and. coupled ) then

         call issolv( iequa, iinput(ipiinsol), rinput(iprinsol), matrix,
     +                rhside, solut, solutv, nsolut, nusol, ndegfd,
     +                infmm, work  , lwork, prec, nprec, iinmtvc )
         if ( nerror.ne.0 ) go to 1000

      else

         if ( coupled ) then
            if ( iequa.eq.2 ) then
               infmt(8) = 3
            else
               infmt(8) = 4
            end if
         else
            if ( iequa.le.ndim) then
               infmt(8) = 1
            else if ( iequa.eq.ndim+1 ) then
               infmt(8) = 3
            else
               infmt(8) = 4

            end if
         end if

         call issolv( iequa, iinput(ipiinsol), rinput(iprinsol), matrix,
     +                rhside, solut, solutv, nsolut, nusol, ndegfd,
     +                infmt, work  , lwork, prec, nprec, iinmtvc )

      end if
      if ( nerror.ne.0 ) go to 1000

      if ( itime.eq.1 ) then

c     --- report timing results for linear system solver

         if ( coupled ) then
            if ( iequa.eq.1 ) then
               matname = 'Momentum'
            else if ( iequa.eq.2 ) then
               matname='Pressure'
            else
               matname='Transport'
            end if
         else
            if ( iequa.le.ndim ) then
               matname = 'Momentum'
            else if ( iequa.eq.ndim+1 ) then
               matname='Pressure'
            else
               matname='Transport'
            end if
         end if
         text = 'for solving the '
         write(text(17:26),200) matname
200      format(a)
         text(27:39) = ' equation is '
         call isaskclk ( 1, text )

      end if

1000  call iserclos('islsol')

      end
