      subroutine islsolpr ( iequa, iinput, solut )
c ======================================================================
c
c     programmer   Guus Segal
c     version 1.3  date  07-07-1998 Kees V: nusol, nsolut cleaned
c     version 1.2  date  19-11-1997 Guus: correction computation ipiinsol
c     version 1.1  date  17-11-1997 Correction for virtual points
c     version 1.0  date  05-11-1997
c
c
c   copyright (c) 1997-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine to print the solution just after the computation by
c     the linear solver
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer iequa, iinput(*)
      double precision solut(*)

c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     solut    i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinsol, ipiintim, ipsolut, itime1,
     +        itime2, itimeact, itimest, leniinsol, iprinsl,
     +        ndim, ni, nj, nk, nsolut, ntrnsp, nturb, nvirtual,
     +        nusol
      logical coupled
      character*39 text

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     iprinsl   If 1 the solution must be printed
c     ipsolut   Starting address of array solut in buffer
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     leniinsol Length of first component of array iinsol
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nsolut    Number of unknowns for all degrees of freedom in one block
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     nvirtual  Number of rows of virtual cells
c     text      String in which a text is stored
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISPRAR01  Print general field array defined in R2
c     ISPRAR02  Print general field array defined in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   fills all the information used in the issolv
c   subroutine and the timing of the linear solver.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      ni       = iinput( 51)
      nj       = iinput( 52)
      nk       = iinput( 53)
      nvirtual = iinput( 54)
      ndim     = iinput( 57)
      ntrnsp   = iinput( 63)
      nturb    = iinput( 70)
      leniinsol = iinput(105)
      ipiintim = iinput(4)
      coupled  = iinput(76).eq.0
      ipiinsol = iinput(5)+(iequa-1)*leniinsol
      iprinsl  = iinput(ipiinsol+16)
      itime1   = iinput(ipiinsol+17)
      itime2   = iinput(ipiinsol+18)
      itimest  = iinput(ipiinsol+19)
      itimeact = iinput(ipiintim+3)

      nusol    = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if ( ndim.eq.3 ) nusol = nusol*(nk+1+2*nvirtual)
      nsolut   = nusol*(ndim+1+ntrnsp+nturb)*2

      if ( iequa.eq.1 .and. coupled ) then

         if ( iprinsl.eq.1 ) then

c        --- Print velocity

            if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +           mod(itimeact-itime1,itimest).eq.0 ) then

               ipsolut = nsolut/2+1
               if ( ndim.eq.2 ) then
                  write(irefwr,200) 'U*', iblockrf
                  call isprar01 ( solut(ipsolut), nvirtual, ni, nj,
     +                            2 )
               else
                  write(irefwr,200) 'U*', iblockrf
                  call isprar02 ( solut(ipsolut), nvirtual, ni, nj, nk,
     +                            3 )
               end if
            end if

         end if

      else

         if ( iprinsl.eq.1 ) then

c        --- Print uncoupled momentum, pressure, or transport quantity

            if ( coupled ) then
               if ( iequa.eq.2 ) then
                  text = 'pressure correction'
               else if ( iequa.le.2+ntrnsp ) then
                  write(text,100) iequa-2
100               format( 'transport quantity', i3 )
               else
                  write(text,110) iequa-ntrnsp-2
110               format( 'turbulent quantity', i3 )
               end if
               ipsolut = nsolut/2+1+(ndim+iequa-2)*nusol
            else
               if ( iequa.le.ndim) then
                  text = 'velocity'
               else if ( iequa.eq.ndim+1 ) then
                  text = 'pressure correction'
               else if ( iequa.le.ndim+1+ntrnsp ) then
                  write(text,100) iequa-ndim-1
               else
                  write(text,110) iequa-ntrnsp-ndim-1
               end if
               ipsolut = nsolut/2+1+(iequa-1)*nusol

            end if
            if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +           mod(itimeact-itime1,itimest).eq.0 ) then

               if ( ndim.eq.2 ) then
                  write(irefwr,200) text, iblockrf
200               format( a / ' Block number ', i4 )
                  call isprar01 ( solut(ipsolut), nvirtual, ni, nj,
     +                            1 )
               else
                  write(irefwr,200) text, iblockrf
                  call isprar02 ( solut(ipsolut), nvirtual, ni, nj, nk,
     +                            1 )
               end if
            end if

         end if

      end if

      end
