      subroutine ismtvc01 ( matrix, solut, rhside, nusol, nconct,
     +                      ndegfd, infmat )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.0    date 04-04-1995 Renamed subroutine ismvc01
c
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector
c     multiplication for the momentum equations
c       A x = b, where b and x have the same structure
c       Warning:  the matrix A can be extended with zero elements referring
c                 to vector elements outside the region of interest
c                 Therefore it is necessary that positions in the memory
c                 corresponding to solut -ni to solut+ni contain real
c                 values  (in 2D case and ??? in 3D case )
c     Warning. The shifts can correctly point into an other degree of freedom
c              and then  ( i+ishift,         j   ) should be treated as
c                        ( i+ishift-k*nusol, j+k )
c              where  k  satisfies the inequality
c                     1 <= i+ishift-k*nusol <= nusol,
c              so     k = (i+ishift-1) modulo nusol
c                         where modulo is as in the pure mathematics
c                              ( periodic w.r.t the left operand ).
c              If the resulting  j+k  is in  1..ndegfd  then the term is needed,
c              otherwise the matrix must yield 0.0 and the term must be skipped.
c              To avoid the slow arithmetics modulo k, we ensure only
c              the condition for  j+k  which can be expressed as
c                     1 <= i+ishift + (j-1)*nusol <= nusol*ndegfd
c              and this means that we do not leave the array solut(nusol,ndegfd)
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nusol, nconct, ndegfd, infmat(nconct,ndegfd)
      double precision matrix(nusol, ndegfd, nconct),
     +                 rhside(nusol, ndegfd),  solut(nusol, ndegfd)

c     infmat    i    Integer information for the matrix
c     matrix    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndegfd    i    Number of degrees of freedom
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     rhside    o    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, ishift

c     i         General loop variable
c     ishift    Shift in i-direction
c     j         General loop variable
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c
c     In order to get maximum vector length in the multiplication, we
c     multiply over the diagonals, rather than in the row direction
c
c     So:
c
c            For j = 1 (1) ndegfd
c
c                (j)    (j)   (j)
c               f    = d     u          (d is diagonal)
c                i      ii    i
c
c
c            For j = 1 (1) ndegfd
c
c               For k = 2 (1) nconct
c
c                  (j)     (j)    (j)   (j)
c                 f    =  f    + a     u
c                  i       i      i l   l
c                                    k   k
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Multiply by diagonal

      do 110 j = 1, ndegfd
         do 100 i = 1, nusol
            rhside(i,j) = matrix(i,j,1) * solut(i,j)
100      continue
110   continue

c     --- Multiply rest of matrix
c         Loop over the degrees of freedom

      do 220 j = 1, ndegfd

c     --- Loop over the connections

         do 210 k = 2, nconct

            ishift = infmat(k,j)

c           --- From the full range i=1,nusol we should deal only the cases when
c               1 <= i+ishift + (j-1)*nusol <= nusol*ndegfd
c               Loop over the unknowns

            do 200 i = max (     1,     1-ishift-     (j-1)*nusol ),
     +                 min ( nusol, nusol-ishift+(ndegfd-j)*nusol )

c           --- i+ishift  can be fare out of the range  1..nusol
c               and  matrix(i,j,k)  nonzero,
c               to access the other columns of the solut(*,*) array

               rhside(i,j) = rhside(i,j) + matrix(i,j,k) *
     +                       solut(i+ishift,j)

200         continue
210      continue
220   continue

      end
