      subroutine ismtvc03 ( matrix, solut, rhside, nusol, nconct,
     +                      ndegfd, infmat )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.0    date 04-04-1995 Renamed subroutine ismvc03
c
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the matrix vector multiplication for the
c     pressure equations first part i.e. G p = u
c     The vector p is smaller than u so the matrix G is non-square
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nusol, nconct, ndegfd, infmat(1:nconct,1:ndegfd)
      double precision matrix(nusol, ndegfd, nconct),
     +                 rhside(nusol, ndegfd),  solut(nusol)

c     infmat    i    Integer information for the matrix
c     matrix    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndegfd    i    Number of degrees of freedom
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     rhside    o    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, ishift

c     i         General loop variable
c     ishift    Shift in i-direction
c     j         General loop variable
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c
c     In order to get maximum vector length in the multiplication, we
c     multiply over the diagonals, rather than in the row direction
c
c     So:
c
c            For j = 1 (1) ndegfd
c
c                (j)
c               f    = 0
c                i
c
c
c            For j = 1 (1) ndegfd
c
c               For k = 2 (1) nconct
c
c                  (j)     (j)    (j)   (j)
c                 f    =  f    + a     u
c                  i       i      il    l
c                                   k    k
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Multiply by diagonal
c         Loop over the degrees of freedom
c         Clear right-hand side

      do 110 j = 1, ndegfd
         do 100 i = 1, nusol
            rhside(i,j) = 0d0
100      continue
110   continue

      do 220 j = 1, ndegfd

c     --- Loop over the connections

         do 210 k = 1, nconct

            ishift = infmat(k,j)

c           --- Loop over the unknowns

            do 200 i = max (     1,     1-ishift ),
     +                 min ( nusol, nusol-ishift )
               rhside(i,j) = rhside(i,j) + matrix(i,j,k) *
     +                       solut(i+ishift)
200         continue
210      continue
220   continue

      end
