      subroutine isprevc ( nusol2, xresul, bresul, amat, prec,
     +                     infmat, initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 2.6    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 2.5    date 25-08-1997 Kees V. Multi-grid prec 3D added
c        version 2.4    date 17-04-1997 Kees V. Multi-grid prec added
c        version 2.3    date 18-03-1996 Kees V. adding ILU in 3D
c        version 2.2    date 18-09-1995 Kees V. adding ILU_Fill
c        version 2.1    date 15-05-1995 Kees V. satisfying the standards
c        version 2.0    date 24-01-1994 Kees K. change for new structure
c        version 1.1    date 19-05-1992
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Isprevc multiplies the vector x with a preconditioner.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol2,infmat(*),initer(*)
      double precision xresul(*),bresul(*),amat(*), prec(*)

c     amat      i    The coefficients matrix used in the linear system
c     bresul    o    The result vector
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     prec      i    The preconditioning matrix
c     xresul    i    The original vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          mkind,nconct,ndim,ndimso,nprec,nusol, i,
     +                 nsmooth

c     i         General loop variable
c     mkind     Control parameter indicating the preconditioner.
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c     nprec     Maximum number of diagonals in preconditioning matrix
c     nsmooth   Number of times smoother is applied (all levels)
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISDIAGMU  Computes the multiplication of a diagonal preconditioner
c               stored in array prec and a vector.
c     ISDINVL   Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu.f.
c     ISDINVL2  Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu2.f.
c     ISDINVL3  Computes the multiplication of the inverse of a lower
c               triangular matrix stored in array prec and a vector.
c               Prec should be filled by isdmlu3.f.
c     ISDINVL4  Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec and a vector.
c               Prec should be filled by isdmlu4.f.
c     ISDINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu.f.
c     ISDINVU2  Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu2.f.
c     ISDINVU3  Computes the multiplication of the inverse of an upper
c               triangular matrix stored in array prec and a vector.
c               Prec should be filled by isdmlu3.f.
c     ISDINVU4  Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec and a vector.
c               Prec should be filled by isdmlu4.f.
c     ISTINVL   Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu.f.
c               This subroutine works for ndim = 3.
c     ISTINVL2  Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu2.f.
c               This subroutine works for ndim = 3.
c     ISTINVL3  Computes the multiplication of the inverse of a lower
c               triangular matrix stored in array prec and a vector.
c               Prec should be filled by istmlu3.f.
c               This subroutine works for ndim = 3.
c     ISTINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu.f.
c               This subroutine works for ndim = 3.
c     ISTINVU2  Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu2.f.
c               This subroutine works for ndim = 3.
c     ISTINVU3  Computes the multiplication of the inverse of an upper
c               triangular matrix stored in array prec and a vector.
c               Prec should be filled by istmlu3.f.
c               This subroutine works for ndim = 3.
c     V1STEP    Preconditioner based on one MG V cycle (MG)
c     V1STEP3   Preconditioner based on one MG3 V cycle (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine isprevc is used to multiply x with a preconditioner
c     and store the result in b. The choice of the preconditioner is
c     given by mkind.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      mkind   = initer(1)
      ndim    = initer(15)
      ndimso  = initer(16)
      nprec   = initer(17)
      nconct  = initer(18)
      nsmooth = initer(28)
      nusol  = nusol2/ndimso

      if ( mkind.eq.0 ) then

c     --- no preconditioner is used

         call dcopy ( nusol2, xresul, 1, bresul, 1 )
      end if

      if ( abs(mkind).eq.1 ) then

c     --- use a diagonal scaling preconditioning matrix

         call dcopy ( nusol2, xresul, 1, bresul, 1 )
         call isdiagmu ( nusol2, bresul, prec )
      end if

      if ( mkind.eq.2 ) then

c     --- use a (M)ILU preconditioner, with D = I.

         call dcopy ( nusol2, xresul, 1, bresul, 1 )

         if ( ndim.eq.2 ) then
            call isdinvl ( bresul, amat, nusol, ndimso,
     +                     nconct, initer)
            call isdinvu ( bresul, amat, nusol, ndimso,
     +                     nconct, initer )
         else
            call istinvl ( bresul, amat, nusol, ndimso,
     +                     nconct, infmat, initer )
            call istinvu ( bresul, amat, nusol, ndimso,
     +                     nconct, infmat, initer )
         end if
      end if

      if ( mkind.eq.-2 ) then

c     --- use a (M)ILU preconditioner, with D is not equal to I.

         call dcopy ( nusol2, xresul, 1, bresul, 1 )

         if ( ndim.eq.2 ) then
            call isdinvl2 ( bresul, amat, nusol, ndimso,
     +                      nconct, prec, nprec, initer )
         else
            call istinvl2 ( bresul, amat, nusol, ndimso,
     +                      nconct, prec, nprec, infmat, initer )
         end if

c        --- Multiplication with D

         call isdiagmu ( nusol2, bresul, prec )

         if ( ndim.eq.2 ) then
            call isdinvu2 ( bresul, amat, nusol, ndimso,
     +                      nconct, prec, nprec, initer )
         else
            call istinvu2 ( bresul, amat, nusol, ndimso,
     +                      nconct, prec, nprec, infmat, initer )
         end if
      end if

      if ( abs(mkind).eq.3 ) then

c     --- use an incomplete LU decomposition of A such that
c         the sparseness pattern of L and U is the same as A.

         call dcopy ( nusol2, xresul, 1, bresul, 1 )

         if ( ndim.eq.2 ) then
            call isdinvl3 ( bresul, nusol, ndimso,
     +                      prec, nprec, initer )
         else
            call istinvl3 ( bresul, nusol, ndimso,
     +                      prec, nprec, initer )
         end if

         if ( ndim.eq.2 ) then
            call isdinvu3 ( bresul, nusol, ndimso,
     +                      prec, nprec, initer )
         else
            call istinvu3 ( bresul, nusol, ndimso,
     +                      prec, nprec, initer )
         end if
      end if

      if ( abs(mkind).eq.4 ) then

c     --- use an incomplete LU decomposition of A where fill-in
c         is allowed.

         call dcopy ( nusol2, xresul, 1, bresul, 1 )
         call isdinvl4 ( bresul, bresul, nusol, ndimso,
     +                   prec, nprec, initer )
         call isdinvu4 ( bresul, bresul, nusol, ndimso,
     +                   prec, nprec, initer )
      end if

      if ( abs(mkind).eq.5 ) then

c     --- use a multigrid preconditioner.

         do i = 1, nusol2
            bresul(i) = 0d0
         end do

         if ( ndim.eq.2 ) then
            call v1step ( amat, xresul, nsmooth, bresul, prec,
     +                    initer(initer(27)) )
         else
            call v1step3 ( amat, xresul, nsmooth, bresul, prec,
     +                    initer(initer(27)) )
         end if

      end if

      end
