      subroutine istinvu2 ( bresul, amat, nusol, ndimso, nconct,
     +                      prec, nprec, infmat, initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.2    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.1    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.0    date 11-05-1995 extracted from old ISNaS
c                       subroutine tinvu2.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by U preconditioning matrix given
c     in amat and prec.
c     In this case we obtain:
c                 -1
c            b = U  x.
c     The array amat and prec should be filled by subroutine istmlu2.f
c     This subroutine contains compiler directives to run in vector
c     speed on the Convex.
c     Warning
c     -------
c     In this subroutine it is possible that shifts correctly point
c     into another degree of freedom, when the momentum equations
c     are solved.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          infmat(*),initer(*),nusol,ndimso,nconct,nprec
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 prec(1:nusol,1:ndimso,1:nprec),
     +                 bresul(nusol, ndimso)

c     amat      i    The coefficients matrix used in the linear system
c     bresul   i/o   The result vector
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, icshift, icstart, j, k, m, ncube, nx, ny, npxy, nz
      logical coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     icshift   Shift to compute address of column in the matrix
c     icstart   Starting address in array infmat for a specific matrix
c     j         General loop variable
c     k         General loop variable
c     m         General loop variable
c     ncube     Product of nx, ny, and nz
c     npxy      Product of nx and ny
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c     nz        Number of points in the z-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      nx = initer(12)+1
      ny = initer(13)+1
      nz = initer(14)+1
      npxy = nx*ny
      coupled  = initer(29).eq.0

c     --- The multiplication b = U^(-1)  x

      bresul(nusol,ndimso) = bresul(nusol,ndimso)
     +                        *prec(nusol,ndimso,2)

      do i = nusol-1, nusol-nx+2, -1
         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       )*prec(i,ndimso,2)
      end do

      i = nusol-nx+1

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    )*prec(i,ndimso,2)

      i = nusol-nx

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    )*prec(i,ndimso,2)

      do i = nusol-nx-1, nusol-npxy+nx+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       )*prec(i,ndimso,2)
      end do

      do i = nusol-npxy+nx, nusol-npxy+2, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       )*prec(i,ndimso,2)
      end do

      i = nusol-npxy+1

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                    -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                    -amat(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                    )*prec(i,ndimso,2)

      i = nusol-npxy

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                    -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                    -amat(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                    -amat(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                    )*prec(i,ndimso,2)

      do i = nusol-npxy-1, nusol-npxy-nx+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       -amat(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                       -amat(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                       -amat(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                       )*prec(i,ndimso,2)
      end do

      do i = nusol-npxy-nx, nusol-2*npxy+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -amat(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       -amat(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                       -amat(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                       -amat(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                       -amat(i,ndimso,19)*bresul(i+npxy+nx,ndimso)
     +                       )*prec(i,ndimso,2)
      end do

      do k = 3, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (nz-k)*npxy+j

            bresul(i,ndimso) = bresul(i,ndimso)
     +                      -amat(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                      -amat(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                      -amat(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                      -amat(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                      -amat(i,ndimso,19)*bresul(i+npxy+nx,ndimso)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,ndimso) = bresul(i,ndimso)
     +                         -amat(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                         -amat(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                         -amat(i,ndimso,9)*bresul(i+nx+1,ndimso)
            end do

            do m = nx, 1, -1

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,ndimso) = (bresul(i,ndimso)
     +                         -amat(i,ndimso,6)*bresul(i+1,ndimso)
     +                         )*prec(i,ndimso,2)
            end do
         end do
      end do

      if ( initer(11).eq.1 .and. coupled ) then

c     --- If initer(11).eq.1 we solve the coupled momentum equations.
c         The upper triangular system is divided in the following blocks:
c         | U   : U   : U   | | b  |   | x  |
c         |  11 :  12 :  13 | |  1 |   |   1|
c         | ................| |....|   |....|
c         | O   : U   : U   | | b  |   | x  |
c         |     :  22 :  23 | |  2 | = |  2 |
c         | ................| |....|   |....|
c         | O   : O   : U   | | b  |   | x  |
c         |     :     :  33 | |  3 |   |  3 |
c                                  -1
c         The multiplication b  = U   x  is already done in the part given
c                         3    33  3
c         above.
c         The computation b  = x  - L   b
c                          2    2    23  3
c         We divide the computation into two parts
c         firstly the parts where the shift may be larger than n,
c         secondly the part where the shift is always less than n.
c         The first part

      ncube  = npxy*nz
      icstart = infmat(13)+50

      do j = 20, 35

         icshift = infmat(icstart+j)-ncube

         do i = nusol-2*npxy+1, min(nusol-icshift,nusol)
            bresul(i,2) = bresul(i,2)-amat(i,2,j)*bresul(icshift+i,3)
         end do
      end do

c     --- The second part

      do i = 1, nusol-2*npxy
         bresul(i,2) = bresul(i,2)-amat(i,2,20)*bresul(i-npxy-nx  ,3)
     +                  -amat(i,2,21)*bresul(i-npxy     ,3)
     +                  -amat(i,2,22)*bresul(i-nx-1     ,3)
     +                  -amat(i,2,23)*bresul(i-nx       ,3)
     +                  -amat(i,2,24)*bresul(i-nx+1     ,3)
     +                  -amat(i,2,25)*bresul(i-1        ,3)
     +                  -amat(i,2,26)*bresul(i          ,3)
     +                  -amat(i,2,27)*bresul(i+1        ,3)
     +                  -amat(i,2,28)*bresul(i+npxy-nx-1,3)
     +                  -amat(i,2,29)*bresul(i+npxy-nx  ,3)
     +                  -amat(i,2,30)*bresul(i+npxy-nx+1,3)
     +                  -amat(i,2,31)*bresul(i+npxy-1   ,3)
     +                  -amat(i,2,32)*bresul(i+npxy     ,3)
     +                  -amat(i,2,33)*bresul(i+npxy+1   ,3)
     +                  -amat(i,2,34)*bresul(i+2*npxy-nx,3)
     +                  -amat(i,2,35)*bresul(i+2*npxy   ,3)
      end do

c     --- The computation b  = U  ^(-1) (x  - U   b )
c                          2    22        2    23  3

      do k = 1, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (nz-k)*npxy+j
            bresul(i,2) = bresul(i,2)
     +                    -amat(i,2,15)*bresul(i+npxy-nx,2)
     +                    -amat(i,2,16)*bresul(i+npxy-1 ,2)
     +                    -amat(i,2,17)*bresul(i+npxy   ,2)
     +                    -amat(i,2,18)*bresul(i+npxy+1 ,2)
     +                    -amat(i,2,19)*bresul(i+npxy+nx,2)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,2) = bresul(i,2)
     +                       -amat(i,2,7) *bresul(i+nx-1,2)
     +                       -amat(i,2,8) *bresul(i+nx  ,2)
     +                       -amat(i,2,9) *bresul(i+nx+1,2)
            end do

            do m = nx, 1, -1

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,2) = (bresul(i,2)
     +                        -amat(i,2,6)*bresul(i+1,2))*prec(i,2,2)
        end do
       end do
      end do

c     --- The computation b  = x  - U   b
c                          1    1    13  3
c         We divide the computation into two parts
c         firstly the parts where the shift may be larger than n,
c         secondly the part where the shift is always less than n.
c         The first part

      icstart = icstart-51

      do j = 36, 51

         icshift = infmat(icstart+j)-2*ncube
         do i = nusol-2*npxy+1, min(nusol,nusol-icshift)
            bresul(i,1) = bresul(i,1)-amat(i,1,j)*bresul(icshift+i,3)
         end do
      end do

c     --- The second part

      do i = 1, nusol-2*npxy
         bresul(i,1) = bresul(i,1)-amat(i,1,36)*bresul(i-npxy-1   ,3)
     +                 -amat(i,1,37)*bresul(i-npxy     ,3)
     +                 -amat(i,1,38)*bresul(i-nx-1     ,3)
     +                 -amat(i,1,39)*bresul(i-nx       ,3)
     +                 -amat(i,1,40)*bresul(i-1        ,3)
     +                 -amat(i,1,41)*bresul(i          ,3)
     +                 -amat(i,1,42)*bresul(i+nx-1     ,3)
     +                 -amat(i,1,43)*bresul(i+nx       ,3)
     +                 -amat(i,1,44)*bresul(i+npxy-nx-1,3)
     +                 -amat(i,1,45)*bresul(i+npxy-nx  ,3)
     +                 -amat(i,1,46)*bresul(i+npxy-1   ,3)
     +                 -amat(i,1,47)*bresul(i+npxy     ,3)
     +                 -amat(i,1,48)*bresul(i+npxy+nx-1,3)
     +                 -amat(i,1,49)*bresul(i+npxy+nx  ,3)
     +                 -amat(i,1,50)*bresul(i+2*npxy-1 ,3)
     +                 -amat(i,1,51)*bresul(i+2*npxy   ,3)
      end do

c     --- The computation b  = x  - U   b  - U   b
c                          1    1    13  3    12  2

      do i = 1, nusol
         bresul(i,1) = bresul(i,1)-amat(i,1,20)*bresul(i-npxy-1   ,2)
     +                 -amat(i,1,21)*bresul(i-npxy     ,2)
     +                 -amat(i,1,22)*bresul(i-npxy+nx-1,2)
     +                 -amat(i,1,23)*bresul(i-npxy+nx  ,2)
     +                 -amat(i,1,24)*bresul(i-nx-1     ,2)
     +                 -amat(i,1,25)*bresul(i-nx       ,2)
     +                 -amat(i,1,26)*bresul(i-1        ,2)
     +                 -amat(i,1,27)*bresul(i          ,2)
     +                 -amat(i,1,28)*bresul(i+nx-1     ,2)
     +                 -amat(i,1,29)*bresul(i+nx       ,2)
     +                 -amat(i,1,30)*bresul(i+2*nx-1   ,2)
     +                 -amat(i,1,31)*bresul(i+2*nx     ,2)
     +                 -amat(i,1,32)*bresul(i+npxy-1   ,2)
     +                 -amat(i,1,33)*bresul(i+npxy     ,2)
     +                 -amat(i,1,34)*bresul(i+npxy+nx-1,2)
     +                 -amat(i,1,35)*bresul(i+npxy+nx  ,2)
      end do

c     --- The computation b  = U  ^(-1) (x  - U   b  - U   b )
c                          1    11        1    13  3    12  2

      do k = 1, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (nz-k)*npxy+j
            bresul(i,1) = bresul(i,1)
     +                    -amat(i,1,15)*bresul(i+npxy-nx,1)
     +                    -amat(i,1,16)*bresul(i+npxy-1 ,1)
     +                    -amat(i,1,17)*bresul(i+npxy   ,1)
     +                    -amat(i,1,18)*bresul(i+npxy+1 ,1)
     +                    -amat(i,1,19)*bresul(i+npxy+nx,1)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,1) = bresul(i,1)
     +                       -amat(i,1,7) *bresul(i+nx-1,1)
     +                       -amat(i,1,8) *bresul(i+nx  ,1)
     +                       -amat(i,1,9) *bresul(i+nx+1,1)
            end do

            do m = nx, 1, -1

               i = (nz-k)*npxy+(ny-j)*nx+m
               bresul(i,1) = (bresul(i,1)
     +                        -amat(i,1,6)*bresul(i+1,1))*prec(i,1,2)
            end do
         end do
      end do
      end if
      end
