      subroutine istmlu2 ( amat, nusol, ndimso, nconct, prec, nprec,
     +                     initer, reiter)
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.5    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.4    date 22-01-1998 Kees V..eq..false. is replaced .not.
c        version 1.3    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.2    date 02-04-1997 Kees K. literal constants in
c                                     double precision for f90
c        version 1.1    date 23-02-1996 Kees V. default for IBLU changed
c        version 1.0    date 11-08-1995 extracted from old ISNaS
c                       subroutine tmlu2.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Calculates a diagonal matrix D to obtain an incomplete decomposi-
c     tion for the matrix A.
c **********************************************************************
c
c                       KEYWORDS
c
c      linear_solver
c      preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*),nusol,ndimso,nconct,nprec
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 prec(1:nusol,1:ndimso,1:nprec), reiter(*)

c     amat      i    The coefficients matrix used in the linear system
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c     reiter    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i, j, nx, ny, npxy
      double precision amod
      logical          coupled

c     amod      Parameter used in the average of ILU and MILU
c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     j         General loop variable
c     npxy      Product of nx and ny
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     This subroutine calculates a diagonal matrix D to obtain an in-
c     complete decomposition, for the matrix A using the following
c     rules defined in:
c
c     H.A. van der Vorst
c     Preconditioning by Incomplete Decompositions (p. 65)
c     Thesis, Utrecht State University (1982)
c
c                   -1
c            A = L D  U - R,
c
c     (a) diag(L) = diag(U) = D,
c
c     (b) The off-diagonal parts of L and U are equal to the cor-
c         responding parts of A,
c
c                 -1
c     (c) diag(L D  U) = diag(A).
c
c     This subroutine is a combination of the above mentioned procedure
c     and the MILU decomposition as described in:
c
c     I. Gustafsson
c     A Class of First Order Factorization Methods
c     BIT 18 (1978) 142-156
c
c     The contributions on the diagonal are weighted by the scalar
c     amod, amod = 0 corresponds with ILU, whereas amod = 1 corresponds
c     with MILU.
c
c     The matrix inverse(D) is stored in prec(1:n,1:ndimso,2)
c
c     If this is used a preconditioner the matrix is scaled.
c     Scale the system Ax = b with D. This leads to
c
c                 -1      -1
c                D  Ax = D  b,
c
c     and
c                 -1      -1  -1     -1
c                D  A  = D  LD  U - D  R.
c
c     From (a), (b), and (c) it follows that:
c
c                -1           -1
c     (a') diag(D  L) = diag(D  U) = I,
c
c                                     -1       -1
c     (b') The off-diagonal parts of D  L and D  U are equal to the cor-
c                               -1
c          responding parts of D  A,
c
c                -1   -1           -1                              -1
c     (c') diag(D  L D  U) = diag(D  A), because r   = 0, implies D  r = 0.
c                                                 ii                  ii
c
c     With respect of the MILU-preconditioning we note that
c             -1             -1  -1
c     rowsum(D  A) = rowsum(D  LD  U), because rowsum(R) = 0 implies
c             -1
c     rowsum(D  R) = 0.
c
c     The actual scaling of the matrix is done in the subroutine
c     issolv.f by a call to isscalma.f
c
c
c     Remark
c
c     - It is known that in the virtual cells the corresponding
c       row and column in the matrix consists of zero elements. So it is
c       possible that d(i) = 0. To circumvent this possibility we imple-
c       ment the following check:
c          if(a(i,1).eq.0d0) then prec(i,1) = 1 else ...
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Filling of the parameter amod

      coupled  = initer(29).eq.0

      amod    = 0d0
      if ( reiter(7).eq.-1d0 ) then
         if ( initer(11).eq.3 ) then
            if ( initer(21).eq.0 ) then
               if ( initer(1).eq.2 ) then
                  amod = 0.99d0
               else
                  amod = 1d0
               end if
            else
               amod = 0.975d0
            end if
         end if
         if ( initer(9).eq.6 )  amod = 0.9d0
         reiter(7) = amod
      else
         amod = reiter(7)
      end if

      nx   = initer(12)+1
      ny   = initer(13)+1
      npxy = nx*ny

c     --- For the matrices with the same structure as the pressure
c         matrix the incomplete LU decomposition is built.

      if ( initer(11).ge.3 .or.
     +     initer(11).eq.1 .and. .not. coupled ) then
         if ( amod.ne.0 ) then

c        --- compute the row sum of U

            do i = 1, nusol
               prec(i,1,2) = amat(i,1,6)+amat(i,1,7)
     +                       +amat(i,1,8)+amat(i,1,9)
     +                       +amat(i,1,15)+amat(i,1,16)
     +                       +amat(i,1,17)+amat(i,1,18)
     +                       +amat(i,1,19)
            end do
         else
            do i = 1, nusol
               prec(i,1,2) = 0d0
            end do
         end if

c        --- Compute the incomplete decomposition
c            i = 1

         if ( amat(1,1,1).eq.0d0 ) then
            prec(1,1,1) = 1d0
         else
            prec(1,1,1) = amat(1,1,1)
         end if

c        --- i = 2, nx-1

         do i = 2, nx-1
            if(amat(i,1,1).eq.0d0) then
               prec(i,1,1) = 1d0
            else
               prec(i,1,1) = amat(i,1,1)
     +                       -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                       +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
            end if
         end do

c        --- i = nx

         i = nx
         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
         end if

c        --- i = nx+1

         i = nx+1
         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                    +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
         end if

c        --- i = nx+2, npxy-nx

         do i = nx+2, npxy-nx
            if ( amat(i,1,1).eq.0d0 ) then
               prec(i,1,1) = 1d0
            else
               prec(i,1,1) = amat(i,1,1)
     +                       -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                       +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                       -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                       +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                       -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                       +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                       -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                       +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
            end if
         end do

c        --- i = npxy-nx+1, npxy-1

         do i = npxy-nx+1, npxy-1
            if ( amat(i,1,1).eq.0d0 ) then
               prec(i,1,1) = 1d0
            else
               prec(i,1,1) = amat(i,1,1)
     +                       -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                       +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                       -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                       +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                       -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                       +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                       -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                       +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
     +                       -amat(i,1,14)*((1-amod)
     +                       *amat(i-npxy+nx,1,15)
     +                       +amod*prec(i-npxy+nx,1,2))
     +                       /prec(i-npxy+nx,1,1)
            end if
         end do

c        --- i = npxy

         i = npxy
         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                    -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                    +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
     +                    -amat(i,1,13)*((1-amod)*amat(i-npxy+1 ,1,16)
     +                    +amod*prec(i-npxy+1 ,1,2))/prec(i-npxy+1 ,1,1)
     +                    -amat(i,1,14)*((1-amod)*amat(i-npxy+nx,1,15)
     +                    +amod*prec(i-npxy+nx,1,2))/prec(i-npxy+nx,1,1)
         end if

c        --- i = npxy+1

         i = npxy+1
         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                    -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                    +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
     +                    -amat(i,1,12)*((1-amod)*amat(i-npxy   ,1,17)
     +                    +amod*prec(i-npxy   ,1,2))/prec(i-npxy   ,1,1)
     +                    -amat(i,1,13)*((1-amod)*amat(i-npxy+1 ,1,16)
     +                    +amod*prec(i-npxy+1 ,1,2))/prec(i-npxy+1 ,1,1)
     +                    -amat(i,1,14)*((1-amod)*amat(i-npxy+nx,1,15)
     +                    +amod*prec(i-npxy+nx,1,2))/prec(i-npxy+nx,1,1)
         end if

c        --- i = npxy+2, npxy+nx

         do i = npxy+2, npxy+nx
            if ( amat(i,1,1).eq.0d0 ) then
               prec(i,1,1) = 1d0
            else
               prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                    -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                    +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
     +                    -amat(i,1,11)*((1-amod)*amat(i-npxy-1,1,18)
     +                    +amod*prec(i-npxy-1,1,2))/prec(i-npxy-1,1,1)
     +                    -amat(i,1,12)*((1-amod)*amat(i-npxy,1,17)
     +                    +amod*prec(i-npxy,1,2))/prec(i-npxy,1,1)
     +                    -amat(i,1,13)*((1-amod)*amat(i-npxy+1,1,16)
     +                    +amod*prec(i-npxy+1 ,1,2))/prec(i-npxy+1 ,1,1)
     +                    -amat(i,1,14)*((1-amod)*amat(i-npxy+nx,1,15)
     +                    +amod*prec(i-npxy+nx,1,2))/prec(i-npxy+nx,1,1)
            end if
         end do

c        --- i = npxy+nx+1, nusol

         do i = npxy+nx+1, nusol
            if ( amat(i,1,1).eq.0d0 ) then
               prec(i,1,1) = 1d0
            else
               prec(i,1,1) = amat(i,1,1)
     +                    -amat(i,1,2)*((1-amod)*amat(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,2))/prec(i-nx-1,1,1)
     +                    -amat(i,1,3)*((1-amod)*amat(i-nx  ,1,8)
     +                    +amod*prec(i-nx  ,1,2))/prec(i-nx  ,1,1)
     +                    -amat(i,1,4)*((1-amod)*amat(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,2))/prec(i-nx+1,1,1)
     +                    -amat(i,1,5)*((1-amod)*amat(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,2))/prec(i-1   ,1,1)
     +                    -amat(i,1,10)*((1-amod)*amat(i-npxy-nx,1,19)
     +                    +amod*prec(i-npxy-nx,1,2))/prec(i-npxy-nx,1,1)
     +                    -amat(i,1,11)*((1-amod)*amat(i-npxy-1 ,1,18)
     +                    +amod*prec(i-npxy-1 ,1,2))/prec(i-npxy-1 ,1,1)
     +                    -amat(i,1,12)*((1-amod)*amat(i-npxy   ,1,17)
     +                    +amod*prec(i-npxy   ,1,2))/prec(i-npxy   ,1,1)
     +                    -amat(i,1,13)*((1-amod)*amat(i-npxy+1 ,1,16)
     +                    +amod*prec(i-npxy+1 ,1,2))/prec(i-npxy+1 ,1,1)
     +                    -amat(i,1,14)*((1-amod)*amat(i-npxy+nx,1,15)
     +                    +amod*prec(i-npxy+nx,1,2))/prec(i-npxy+nx,1,1)
            end if
         end do
      end if

c     --- For the coupled momentum equations only the SSOR preconditioner
c         is implemented

      if ( initer(11).eq.1 .and. coupled ) then
         do j = 1, ndimso
            do i = 1, nusol
               if ( amat(i,j,1).eq.0d0 ) then
                  prec(i,j,1) = 1d0
               else
                  prec(i,j,1) = amat(i,j,1)
               end if
            end do
         end do
      end if

c     --- The inverse is stored in prec(i,j,2)

      do j = 1, ndimso
         do i = 1, nusol
            prec(i,j,2) = 1d0/prec(i,j,1)
         end do
      end do

      end
