      subroutine tinvu(x,b,amat,n,ndimso,nconct,
     1                  prec,nprec,infmat)
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.2      date 15-03-1993 C. Vuik Vectorized on the
c                                      Convex
c     version 1.1      date 22-02-1993 C. Vuik Preconditioning of the
c                                      momentum equations.
c     version 1.0      date 20-11-1992
c     developed at     HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by U preconditioning matrix given
c     in amat and prec.
c
c     In this case we obtain:
c                 -1
c            b = U  x.
c
c     The arrays prec and amat should be filled by subroutine tmlu.f
c
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilud
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c     amat    i  the coefficient matrix for the momentum or
c                an equation similar to the pressure equation.
c
c     b       o  the result vector, which contains:
c                     -1
c                b = U  x.
c
c     infmat  i  if infmat(8) is 1 we use the momentum equations,
c                whereas if infmat(8) is larger than or equal to 3
c                we use equations, with a structure similar to the
c                pressure equations.
c
c     n       i  number of unknowns in the solution vector.
c
c     nconct  i  number of connections in one row of the matrix.
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. In this subroutine nprec = 1.
c
c     prec    i  the preconditioning matrix.
c
c     x       i  the original vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter.
c
c     ishift  shift to compute address of column in the matrix.
c
c     istart  starting address in array infmat for a specific matrix.
c
c     j       loop counter.
c
c     k       auxiliary variable, which contains ndimso.
c
c     kz      loop counter.
c
c     m       loop counter.
c
c     ncube   product of nx, ny, and nz.
c
c     npxy    product of nx and ny.
c
c     nx      number of points in the x-direction.
c
c     ny      number of points in the y-direction.
c
c     nz      number of points in the z-direction.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of x by U preconditioning matrix given
c     in amat and prec.
c
c     In this case we obtain:
c                 -1
c            b = U  x.
c
c     The arrays prec and amat should be filled by subroutine tmlu.f
c
c ==================================================================
c
c     Parameters
c
      integer          infmat(*),n,ndimso,nconct,nprec
      double precision amat(1:n,1:ndimso,1:nconct),
     1                 prec(1:n,1:ndimso,1:nprec),
     2                 x(n,ndimso),b(n,ndimso)
c
c     local parameters
c
      integer i,ishift,istart,j,k,kz,m,ncube,nx,ny,npxy,nz
      nx = infmat(2)+1
      ny = infmat(3)+1
      nz = infmat(4)+1
      npxy = nx*ny
c                             -1
c     The multiplication b = U  x
c
      k = ndimso
      b(n,k) = x(n,k)
      do i = n-1, n-nx+2, -1
         b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
      enddo
      i = n-nx+1
      b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                -amat(i,k,7)*b(i+nx-1,k)
      i = n-nx
      b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                -amat(i,k,7)*b(i+nx-1,k)
     .                -amat(i,k,8)*b(i+nx  ,k)
      do i = n-nx-1, n-npxy+nx+1, -1
         b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                   -amat(i,k,7)*b(i+nx-1,k)
     .                   -amat(i,k,8)*b(i+nx  ,k)
     .                   -amat(i,k,9)*b(i+nx+1,k)
      enddo
      do i = n-npxy+nx, n-npxy+2, -1
         b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                   -amat(i,k,7)*b(i+nx-1,k)
     .                   -amat(i,k,8)*b(i+nx  ,k)
     .                   -amat(i,k,9)*b(i+nx+1,k)
     .                   -amat(i,k,15)*b(i+npxy-nx,k)
      enddo
      i = n-npxy+1
      b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                -amat(i,k,7)*b(i+nx-1,k)
     .                -amat(i,k,8)*b(i+nx  ,k)
     .                -amat(i,k,9)*b(i+nx+1,k)
     .                -amat(i,k,15)*b(i+npxy-nx,k)
     .                -amat(i,k,16)*b(i+npxy-1 ,k)
      i = n-npxy
      b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                -amat(i,k,7)*b(i+nx-1,k)
     .                -amat(i,k,8)*b(i+nx  ,k)
     .                -amat(i,k,9)*b(i+nx+1,k)
     .                -amat(i,k,15)*b(i+npxy-nx,k)
     .                -amat(i,k,16)*b(i+npxy-1 ,k)
     .                -amat(i,k,17)*b(i+npxy   ,k)
      do i = n-npxy-1, n-npxy-nx+1, -1
         b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                   -amat(i,k,7)*b(i+nx-1,k)
     .                   -amat(i,k,8)*b(i+nx  ,k)
     .                   -amat(i,k,9)*b(i+nx+1,k)
     .                   -amat(i,k,15)*b(i+npxy-nx,k)
     .                   -amat(i,k,16)*b(i+npxy-1 ,k)
     .                   -amat(i,k,17)*b(i+npxy   ,k)
     .                   -amat(i,k,18)*b(i+npxy+1 ,k)
      enddo
      do i = n-npxy-nx, n-2*npxy+1, -1
         b(i,k) = x(i,k)-amat(i,k,6)*b(i+1   ,k)
     .                   -amat(i,k,7)*b(i+nx-1,k)
     .                   -amat(i,k,8)*b(i+nx  ,k)
     .                   -amat(i,k,9)*b(i+nx+1,k)
     .                   -amat(i,k,15)*b(i+npxy-nx,k)
     .                   -amat(i,k,16)*b(i+npxy-1 ,k)
     .                   -amat(i,k,17)*b(i+npxy   ,k)
     .                   -amat(i,k,18)*b(i+npxy+1 ,k)
     .                   -amat(i,k,19)*b(i+npxy+nx,k)
      enddo
      do kz = 3, nz
c$dir no_recurrence
       do j = 1, npxy
	 i = (nz-kz)*npxy+j
         b(i,k) = x(i,k)
     .                   -amat(i,k,15)*b(i+npxy-nx,k)
     .                   -amat(i,k,16)*b(i+npxy-1 ,k)
     .                   -amat(i,k,17)*b(i+npxy   ,k)
     .                   -amat(i,k,18)*b(i+npxy+1 ,k)
     .                   -amat(i,k,19)*b(i+npxy+nx,k)
       enddo
       do j = 1, ny
c$dir no_recurrence
        do m = 1, nx
	 i = (nz-kz)*npxy+(ny-j)*nx+m
         b(i,k) = b(i,k)
     .                   -amat(i,k,7)*b(i+nx-1,k)
     .                   -amat(i,k,8)*b(i+nx  ,k)
     .                   -amat(i,k,9)*b(i+nx+1,k)
        enddo
        do m = nx, 1, -1
	 i = (nz-kz)*npxy+(ny-j)*nx+m
	 b(i,k) = b(i,k)-amat(i,k,6)*b(i+1,k)
        enddo
       enddo
      enddo
      if(infmat(8).eq.1) then
c
c     If infmat(8).eq.1 we solve the momentum equations. The lower
c     triangular system is divided in the following blocks:
c
c        | U   : U   : U   | | b  |   | x  |
c        |  11 :  12 :  13 | |  1 |   |   1|
c        | ................| |....|   |....|
c        | O   : U   : U   | | b  |   | x  |
c        |     :  22 :  23 | |  2 | = |  2 |
c        | ................| |....|   |....|
c        | O   : O   : U   | | b  |   | x  |
c        |     :     :  33 | |  3 |   |  3 |
c
c
c                              -1
c     The multiplication b  = U   x  is already done in the part given
c                         3    33  3
c     above.
c
c     The computation b  = x  - L   b
c                      2    2    23  3
c
c     We divide the computation into two parts
c
c     firstly the parts where the shift may be larger than n,
c
c     secondly the part where the shift is always less than n.
c
c
c     The first part
c
      ncube  = npxy*nz
      istart = infmat(13)+50
      do i = n-2*npxy+1, n
         b(i,2) = x(i,2)
      enddo
      do j = 20, 35
         ishift = infmat(istart+j)-ncube
         do i = n-2*npxy+1, min(n-ishift,n)
            b(i,2) = b(i,2)-amat(i,2,j)*b(ishift+i,3)
         enddo
      enddo
c
c     The second part
c
      do i = 1, n-2*npxy
         b(i,2) = x(i,2)-amat(i,2,20)*b(i-npxy-nx  ,3)
     .                  -amat(i,2,21)*b(i-npxy     ,3)
     .                  -amat(i,2,22)*b(i-nx-1     ,3)
     .                  -amat(i,2,23)*b(i-nx       ,3)
     .                  -amat(i,2,24)*b(i-nx+1     ,3)
     .                  -amat(i,2,25)*b(i-1        ,3)
     .                  -amat(i,2,26)*b(i          ,3)
     .                  -amat(i,2,27)*b(i+1        ,3)
     .                  -amat(i,2,28)*b(i+npxy-nx-1,3)
     .                  -amat(i,2,29)*b(i+npxy-nx  ,3)
     .                  -amat(i,2,30)*b(i+npxy-nx+1,3)
     .                  -amat(i,2,31)*b(i+npxy-1   ,3)
     .                  -amat(i,2,32)*b(i+npxy     ,3)
     .                  -amat(i,2,33)*b(i+npxy+1   ,3)
     .                  -amat(i,2,34)*b(i+2*npxy-nx,3)
     .                  -amat(i,2,35)*b(i+2*npxy   ,3)
      enddo
c                           -1
c     The computation b  = U  (x  - U   b )
c                      2    22  2    23  3
c
      do kz = 1, nz
c$dir no_recurrence
       do j = 1, npxy
	 i = (nz-kz)*npxy+j
         b(i,2) = b(i,2)
     .                   -amat(i,2,15)*b(i+npxy-nx,2)
     .                   -amat(i,2,16)*b(i+npxy-1 ,2)
     .                   -amat(i,2,17)*b(i+npxy   ,2)
     .                   -amat(i,2,18)*b(i+npxy+1 ,2)
     .                   -amat(i,2,19)*b(i+npxy+nx,2)
       enddo
       do j = 1, ny
c$dir no_recurrence
        do m = 1, nx
	 i = (nz-kz)*npxy+(ny-j)*nx+m
         b(i,2) = b(i,2)
     .                   -amat(i,2,7) *b(i+nx-1,2)
     .                   -amat(i,2,8) *b(i+nx  ,2)
     .                   -amat(i,2,9) *b(i+nx+1,2)
        enddo
        do m = nx, 1, -1
	 i = (nz-kz)*npxy+(ny-j)*nx+m
         b(i,2) = b(i,2)-amat(i,2,6) *b(i+1   ,2)
        enddo
       enddo
      enddo
c
c     The computation b  = x  - U   b
c                      1    1    13  3
c
c     We divide the computation into two parts
c
c     firstly the parts where the shift may be larger than n,
c
c     secondly the part where the shift is always less than n.
c
c
c     The first part
c
      istart = istart-51
      do i = n-2*npxy+1, n
         b(i,1) = x(i,1)
      enddo
      do j = 36, 51
         ishift = infmat(istart+j)-2*ncube
         do i = n-2*npxy+1, min(n,n-ishift)
            b(i,1) = b(i,1)-amat(i,1,j)*b(ishift+i,3)
         enddo
      enddo
c
c     The second part
c
      do i = 1, n-2*npxy
         b(i,1) = x(i,1)-amat(i,1,36)*b(i-npxy-1   ,3)
     .                  -amat(i,1,37)*b(i-npxy     ,3)
     .                  -amat(i,1,38)*b(i-nx-1     ,3)
     .                  -amat(i,1,39)*b(i-nx       ,3)
     .                  -amat(i,1,40)*b(i-1        ,3)
     .                  -amat(i,1,41)*b(i          ,3)
     .                  -amat(i,1,42)*b(i+nx-1     ,3)
     .                  -amat(i,1,43)*b(i+nx       ,3)
     .                  -amat(i,1,44)*b(i+npxy-nx-1,3)
     .                  -amat(i,1,45)*b(i+npxy-nx  ,3)
     .                  -amat(i,1,46)*b(i+npxy-1   ,3)
     .                  -amat(i,1,47)*b(i+npxy     ,3)
     .                  -amat(i,1,48)*b(i+npxy+nx-1,3)
     .                  -amat(i,1,49)*b(i+npxy+nx  ,3)
     .                  -amat(i,1,50)*b(i+2*npxy-1 ,3)
     .                  -amat(i,1,51)*b(i+2*npxy   ,3)
      enddo
c
c     The computation b  = x  - U   b  - U   b
c                      1    1    13  3    12  2
c
      do i = 1, n
         b(i,1) = b(i,1)-amat(i,1,20)*b(i-npxy-1   ,2)
     .                  -amat(i,1,21)*b(i-npxy     ,2)
     .                  -amat(i,1,22)*b(i-npxy+nx-1,2)
     .                  -amat(i,1,23)*b(i-npxy+nx  ,2)
     .                  -amat(i,1,24)*b(i-nx-1     ,2)
     .                  -amat(i,1,25)*b(i-nx       ,2)
     .                  -amat(i,1,26)*b(i-1        ,2)
     .                  -amat(i,1,27)*b(i          ,2)
     .                  -amat(i,1,28)*b(i+nx-1     ,2)
     .                  -amat(i,1,29)*b(i+nx       ,2)
     .                  -amat(i,1,30)*b(i+2*nx-1   ,2)
     .                  -amat(i,1,31)*b(i+2*nx     ,2)
     .                  -amat(i,1,32)*b(i+npxy-1   ,2)
     .                  -amat(i,1,33)*b(i+npxy     ,2)
     .                  -amat(i,1,34)*b(i+npxy+nx-1,2)
     .                  -amat(i,1,35)*b(i+npxy+nx  ,2)
      enddo
c                           -1
c     The computation b  = U  (x  - U   b  - U   b )
c                      1    11  1    13  3    12  2
c
      do kz = 1, nz
c$dir no_recurrence
       do j = 1, npxy
	 i = (nz-kz)*npxy+j
         b(i,1) = b(i,1)
     .                   -amat(i,1,15)*b(i+npxy-nx,1)
     .                   -amat(i,1,16)*b(i+npxy-1 ,1)
     .                   -amat(i,1,17)*b(i+npxy   ,1)
     .                   -amat(i,1,18)*b(i+npxy+1 ,1)
     .                   -amat(i,1,19)*b(i+npxy+nx,1)
       enddo
       do j = 1, ny
c$dir no_recurrence
        do m = 1, nx
	 i = (nz-kz)*npxy+(ny-j)*nx+m
         b(i,1) = b(i,1)
     .                   -amat(i,1,7) *b(i+nx-1,1)
     .                   -amat(i,1,8) *b(i+nx  ,1)
     .                   -amat(i,1,9) *b(i+nx+1,1)
	enddo
        do m = nx, 1, -1
	 i = (nz-kz)*npxy+(ny-j)*nx+m
         b(i,1) = b(i,1)-amat(i,1,6) *b(i+1   ,1)
        enddo
       enddo
      enddo
      endif
      end
