      subroutine uscomprho ( eddy, solut, ni, nj, nk, nvirtual, ndim,
     +                       ndegfd)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Subroutine to compute the density rho of liquid hydrazine
c   The density rho is calculated from a Clausius modified equation
c   The density rho is put into eddy(.,2). This subroutine is used during
c   the initialization.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ndim, nvirtual, ndegfd
      double precision solut(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual,1:ndegfd),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 a, b, c, rk2, R, rn

c     eddy           o    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c                         beyond found upploop blank
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision rho, delta, alp1, alp2, dalp1, dalp2,
     +                 steq, dsteq, v
      integer i, j


c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c     usccoefhydr
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c

      call uscoefhydr(a,b,c,rk2,R,rn)

c     --- Calculation of the density from the sate equation using the
c         Newton-Raphson method.
c         eddy(i,j,2) is the density
c         eddy(i,j,7) is the temperature
c         solut(i,j,3) is the pressure

      do i=1-nvirtual,1+ni+nvirtual
         do j=1-nvirtual,1+nj+nvirtual
            rho = 1.d3
            delta = 1.d0
 100        alp1 = 1. - a*rho
            alp2 = (eddy(i,j,7)**rn)*((1d0 + b*rho)**2d0)
            dalp1 = - a
            dalp2 = 2d0*b*(eddy(i,j,7)**rn)*(1d0 + b*rho)
            steq = solut(i,j,3)*alp1*alp2 - R*rho*eddy(i,j,7)*alp2
     +             + c*alp1*(rho**2d0)
            dsteq = solut(i,j,3)*(alp1*dalp2 + dalp1*alp2)
     +              - R*eddy(i,j,7)*(alp2 + dalp2*rho)
     +              + c*(dalp1*(rho**2d0) + 2d0*alp1*rho)
            delta = - steq/dsteq
            rho = rho + delta
            if (delta.gt.(1.d-3)) goto 100
            eddy(i,j,2) = rho
         end do
      end do
      end
