      subroutine uscompvisc(visc,eddy,ni,nj,nk,nvirtual,ndim)
c ======================================================================
c
c        programmer     Philippe Reynier
c        version 1.0    date 12-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Sample user subroutine to calculate the dynamic viscosity for
c    liquid hydrazine (valid from 273K to 333K)
c    Here visc = exp(a + b/T + c*T +d*T*T)
c    The dynamic viscosity is put into visc
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
      implicit none
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni,nj,nk,ndim,nvirtual
      double precision visc(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7)

c     visc                array for the dynamic viscosity
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision a, b, c, d
      integer i, j

c     a              Constants of the model
c     b              Constants of the model
c     c              Constants of the model
c     d              Constants of the model
c     i              General loop variable
c     j              General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      double precision g
c ======================================================================
c
c     --- Constants of the model for hydrazine

      a = -25.959229d0
      b = 3047.770857d0
      c = 3.8603691d-2
      d = -3.1714523d-5

c     --- calculation of the dynamic viscosity
c         eddy(i,j,7) is the temperature

      do i=1-nvirtual,ni+nvirtual
         do j=1-nvirtual,nj+nvirtual
            visc(i,j) = exp(a + b/eddy(i,j,7) + c*eddy(i,j,7)
     +                 +d*(eddy(i,j,7)**2))
         end do
      end do
      i = ni+nvirtual+1
      do j=1-nvirtual,nj+nvirtual
         visc(i,j) = visc(i-1,j)
      end do
      j = nj+nvirtual+1
      visc(i,j) = visc(i,j-1)
      do i=1-nvirtual,ni+nvirtual
         visc(i,j) = visc(i,j-1)
      end do
      end
