      subroutine issource ( isource, icontext )
c ======================================================================
c
c        Programmer     Jason Frank
c
c     copyright (c) 1994-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Determines source of next message in given context
c     This implementation is completely based on MPI
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     receive
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isource, icontext

c     icontext i    the communication context, (message tag)
c     isource  o    source of the next message in the given context
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
cpar      include 'mpif.h'
cpar      integer ierr, istatus(MPI_STATUS_SIZE)

c     ierr      error status of an MPI call ( .eq.MPI_SUCCESS if ok)
c     istatus   MPI status array
c **********************************************************************
c
c                       I/O
c
c     Information is received on host or node 
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error subroutine
c     MPI_PROBE MPI function to probe information about an incoming
c               message without actually receiving it
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
cpar      call iseropen ( 'issource' )

c     ---  Probe message
cpar      call MPI_PROBE ( MPI_ANY_SOURCE, icontext, MPI_COMM_WORLD, 
cpar     +   istatus, ierr )

cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

cpar      isource = istatus (MPI_SOURCE)

cpar      call iserclos ( 'issource' )
      end
