      subroutine ispack ( inbuf, incount, intype, ipackbuf, ibufsize,
     +                    ibufpos, icomm, ierr )
c ======================================================================
c
c        Programmer     Jason Frank
c        version 1.0    date 08-09-1998
c
c
c     copyright (c) 1994-1998  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Wrapper for MPI_PACK which makes sure buffer is not overfilled
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer inbuf, incount, intype, ipackbuf, ibufsize
      integer ibufpos, icomm, ierr

c     ibufpos  i    current number of bytes in ipackbuf
c     ibufpos  i    total space in bytes allocated to ipackbuf
c     icomm    i    MPI communicator
c     ierr     i    MPI error status
c     inbuf    i    buffer to be packed from
c     incount  i    number of items to pack
c     intype   i    MPI datatype to of inbuf
c     ipackbuf i    buffer to be packed into
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      include 'mpif.h'
      integer ipacksz

c     ipacksz  i    number of bytes to be packed into ipackbuf
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRSPR       Special error message subroutine in case of parallel computing
c     MPI_PACK       Pack data into isndbuf
c     MPI_PACK_SIZE  Upper bound on packed size of new data
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'ispack' )

c     --- determine the size of packed data

      call MPI_PACK_SIZE (incount, intype, icomm, ipacksz, ierr)

      if (ierr.ne.MPI_SUCCESS) then

c     --- call to MPI_PACK_SIZE failed

         call iserrspr (1750, 0, 0, 0)

      end if

      if ((ibufpos + ipacksz).ge.mxmsglen) then

c     --- pack will fail, bail out

         call iserrspr (1750, 0, 0, 0)

      else

c     --- pack it

         call MPI_PACK (inbuf, incount, intype, ipackbuf,
     +      ibufsize, ibufpos, icomm, ierr )

      end if

      call iserclos ( 'ispack' )
      end
